var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
define("Character", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Character {
    }
    Character.WOODY = "WOODY";
    Character.BUZZ = "BUZZ";
    Character.REX = "REX";
    Character.HAMM = "HAMM";
    Character.JESSIE = "JESSIE";
    Character.SARGE = "SARGE";
    Character.TRIXIE = "TRIXIE";
    Character.PETE = "PETE";
    Character.PETE_FOUND = "PETE-FOUND";
    exports.Character = Character;
});
define("switchback-common/util/TextUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TextUtil {
        /**
         * Calculates the font size modifier used when getting the fontsize based on
         * the devices aspect ratio. This is used in getFontSize(). For wider devices the
         * fontSizeModifier is 600 and for narrow devices is uses 400. These values were
         * choosen because they make the text look nice.
         * @param gameWidth Canvas width
         * @param gameHeight Canvas height
         */
        static calculateFontSizeModifier(gameWidth, gameHeight) {
            TextUtil.fontSizeModifier = 400;
            let aspectRatio = (gameHeight / gameWidth);
            // Finding the aspect ratio of the biggest edge to the smallest edge. Using
            // height / width as the base check because most games should be run in portrait
            if (aspectRatio < 1) {
                // Invert aspect ratio if less than 1.
                aspectRatio = 1 / aspectRatio;
            }
            if (aspectRatio <= 1.5) {
                TextUtil.fontSizeModifier = 600;
            }
        }
        static leftPad(text, padCharacter, width) {
            return width - text.length > 0 ? padCharacter.repeat(width - text.length) + text : text;
        }
        static rightPad(text, padCharacter, width) {
            return width - text.length > 0 ? text + padCharacter.repeat(width - text.length) : text;
        }
        // Returns a font size value that is usable by Phaser at the scale of the desktop font.
        static getFontSize(gameWidth, fontPtSize) {
            return (fontPtSize / TextUtil.fontSizeModifier) * gameWidth;
        }
        static formattedTime(seconds) {
            const min = Math.max(0, Math.floor(seconds / 60)).toString();
            const sec = (seconds % 60).toFixed(0);
            return TextUtil.leftPad(min, "0", 2) + ":" + TextUtil.leftPad(sec, "0", 2);
        }
        static getOrdinalFormattedString(n) {
            const s = ["th", "st", "nd", "rd"];
            const v = n % 100;
            return n + (s[(v - 20) % 10] || s[v] || s[0]);
        }
        /**
         * Scales the contents of a textfield down if it would overflow the width.
         * This is used to ensure that single line textfields don't overflow.
         *
         * Will not work if word-wrap is turned on (as word wrap ensures it never overflows width)
         */
        static scaleDownSingleLineTextToWidth(textField, width) {
            const condition = (textField, sizeRestriction) => {
                return textField.width > sizeRestriction && textField.fontSize > 1;
            };
            TextUtil.scaleText(textField, width, condition);
        }
        /**
         * Scales the contents of a textfield down if it would overflow the height.
         */
        static scaleDownTextToHeight(textField, height) {
            const condition = (textField, sizeRestriction) => {
                return textField.height > sizeRestriction && textField.fontSize > 1;
            };
            TextUtil.scaleText(textField, height, condition);
        }
        static scaleUpTextToHeight(textField, height) {
            const condition = (textField, sizeRestriction) => {
                return textField.height < sizeRestriction;
            };
            TextUtil.scaleText(textField, height, condition, true);
        }
        /**
         * Passing in an array of @interface IFontSize @param textObjects to this function
         * will attempt to equalize their font sizes to either the largest,
         * or smallest, depending on @param equalizeOnSmallest.
         */
        static equalizeFontSizes(textObjects, equalizeOnSmallest) {
            let size = equalizeOnSmallest ? Infinity : 0;
            textObjects.forEach((obj) => {
                size = equalizeOnSmallest ? Math.min(size, obj.fontSize) : Math.max(size, obj.fontSize);
            });
            textObjects.forEach((obj) => {
                obj.fontSize = size;
            });
        }
        static scaleText(textField, sizeRestriction, condition, increment = false) {
            const alignment = textField.align;
            let fontSize = textField.fontSize;
            while (condition(textField, sizeRestriction)) {
                textField.fontSize = increment ? fontSize++ : fontSize--;
            }
            // this is required as the text doesn't repaint automatically (phaser quirk)
            textField.align = alignment;
        }
    }
    TextUtil.fontSizeModifier = 400;
    exports.TextUtil = TextUtil;
});
/**
 * @author nick@cerebralfix.com
 */
define("switchback-common/component/Signal", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Signal {
        constructor() {
            this.active = true;
            this.bindings = [];
        }
        add(listener, context) {
            if (this.bindings) {
                const binding = new SignalBinding(listener, context, this);
                this.bindings.push(binding);
                return binding;
            }
            return null;
        }
        addOnce(listener, context) {
            if (this.bindings) {
                const binding = new SignalBinding(listener, context, this, true);
                this.bindings.push(binding);
                return binding;
            }
            return null;
        }
        dispatch(...params) {
            if (this.active) {
                for (let i = 0; this.active && i < this.bindings.length; i++) {
                    const binding = this.bindings[i];
                    if (binding.addOnce) {
                        this.bindings.splice(i, 1);
                        i--;
                    }
                    if (binding.context) {
                        binding.listener.call(binding.context, ...params);
                    }
                    else {
                        binding.listener(...params);
                    }
                }
            }
        }
        removeBinding(binding) {
            if (this.active) {
                const index = this.bindings.indexOf(binding);
                if (index >= 0) {
                    this.bindings.splice(index, 1);
                }
            }
        }
        dispose() {
            this.active = false;
            delete this.bindings;
        }
    }
    exports.Signal = Signal;
    // tslint:disable-next-line:max-classes-per-file
    class SignalBinding {
        constructor(listener, context, signal, addOnce = false) {
            this.listener = listener;
            this.context = context;
            this.signal = signal;
            this.addOnce = addOnce;
        }
        remove() {
            this.signal.removeBinding(this);
        }
    }
    exports.SignalBinding = SignalBinding;
});
define("switchback-common/util/PromiseUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Class to provide helpful Promise wrapper functions
     * @author joel@cerebralfix.com
     * @author steven@cerebralfix.com
     */
    class PromiseUtil {
        /**
         * Phaser time
         * Smallest time you can wait is the length of one frame (16ms @ 60fps)
         *
         * @param pTime Time in miliseconds
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static time(pGame, pTime, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let timer;
                    const cancelCallback = () => {
                        if (timer) {
                            // stop timer as we've cancelled it
                            pGame.time.events.remove(timer);
                        }
                        reject();
                    };
                    const resolveCallback = () => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve();
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    timer = pGame.time.events.add(pTime, resolveCallback);
                });
            });
        }
        /**
         * Real time
         * Can update in smaller increments, but does not pause when the game loses focus
         * Use with caution
         *
         * @param pTime Time in miliseconds
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static realTime(pTime, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let timeoutId = -1;
                    const cancelCallback = () => {
                        if (timeoutId >= 0) {
                            // stop timeout as we've cancelled it
                            clearTimeout(timeoutId);
                        }
                        reject();
                    };
                    const resolveCallback = () => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve();
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    timeoutId = setTimeout(resolveCallback, pTime);
                });
            });
        }
        /*
         * Awaits a tween to complete
         *
         * @param pTween The tween to listen for completion on.
         * @param cancelSignal An optional signal that when triggered will cancel the tween and release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static tween2(pTween, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let resolveCallback;
                    const cancelCallback = () => {
                        if (pTween) {
                            // stop listening to the tween and stop it as we've cancelled it
                            pTween.onComplete.remove(resolveCallback);
                            pTween.stop();
                        }
                        reject();
                    };
                    resolveCallback = () => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve();
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    pTween.onComplete.addOnce(resolveCallback);
                });
            });
        }
        static tween(pGame, pTween, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield PromiseUtil.tween2(pTween, cancelSignal);
            });
        }
        static animation(pAnimation, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let resolveCallback;
                    const cancelCallback = () => {
                        if (pAnimation) {
                            // stop listening to callback as we've cancelled it
                            pAnimation.onComplete.remove(resolveCallback);
                        }
                        reject();
                    };
                    resolveCallback = () => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve();
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    pAnimation.onComplete.addOnce(resolveCallback);
                });
            });
        }
        /*
         * Awaits a signal to trigger
         *
         * @param pSignal The signal to listen for completion on.
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static signal(pSignal, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let resolveCallback;
                    let cancelCallback = () => {
                        if (pSignal) {
                            // stop listening to callback as we've cancelled it
                            pSignal.remove(resolveCallback);
                        }
                        reject();
                    };
                    resolveCallback = (args) => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.remove(cancelCallback);
                        }
                        resolve(args);
                    };
                    if (cancelSignal) {
                        cancelSignal.addOnce(cancelCallback);
                    }
                    pSignal.addOnce(resolveCallback);
                });
            });
        }
        /*
         * Awaits a common signal to trigger
         *
         * @param pSignal The signal to listen for completion on.
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static commonSignal(pSignal, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    let resolveCallback;
                    let resolveCallbackBinding;
                    let cancelCallbackBinding;
                    let cancelCallback = () => {
                        if (pSignal) {
                            // stop listening to callback as we've cancelled it
                            pSignal.removeBinding(resolveCallbackBinding);
                        }
                        reject();
                    };
                    resolveCallback = (args) => {
                        if (cancelSignal) {
                            // this resolved without needing to cancel
                            cancelSignal.removeBinding(cancelCallbackBinding);
                        }
                        resolve(args);
                    };
                    if (cancelSignal) {
                        cancelCallbackBinding = cancelSignal.addOnce(cancelCallback);
                    }
                    resolveCallbackBinding = pSignal.addOnce(resolveCallback);
                });
            });
        }
        static load(pLoader) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    pLoader.onLoadComplete.addOnce(resolve);
                    pLoader.start();
                });
            });
        }
        /**
         * Loops through all markers in @param audioSprite. If any are
         * playing, this function will block until they're finished.
         */
        static audioSpriteMarkers(audioSprite) {
            return __awaiter(this, void 0, void 0, function* () {
                const stopPromises = Object.keys(audioSprite.sounds).map((key) => {
                    const sfx = audioSprite.sounds[key];
                    if (sfx.isPlaying) {
                        return PromiseUtil.signal(sfx.onStop);
                    }
                });
                yield Promise.all(stopPromises);
            });
        }
        /**
         * Wait for a sound that is already playing to finish playing.
         *
         * @param sound The sound that you wish to wait to end
         * @param offsetDuration Handles the situation where you want to wait for a remainder of the time,
         * useful if this sound has already played for a while before being awaited.
         * @param cancelSignal An optional signal that when triggered will release the await.
         * If this is used caller must .catch(() => {}) the rejected promise.
         */
        static sound(sound, offsetDurationMs = 0, cancelSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                if (sound && sound.game && sound.isPlaying) {
                    // it has been possible for sounds to break and never stop,
                    // hense the additional time safety check
                    yield Promise.race([
                        PromiseUtil.signal(sound.onStop, cancelSignal),
                        PromiseUtil.time(sound.game, sound.durationMS - offsetDurationMs, cancelSignal),
                    ]);
                }
            });
        }
    }
    exports.PromiseUtil = PromiseUtil;
});
define("switchback-common/component/ThreePieceImage", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var JoinMethod;
    (function (JoinMethod) {
        JoinMethod[JoinMethod["STRETCHED_VERTICALLY"] = 0] = "STRETCHED_VERTICALLY";
        JoinMethod[JoinMethod["STRETCHED_HORIZONTALLY"] = 1] = "STRETCHED_HORIZONTALLY";
        JoinMethod[JoinMethod["TILED_VERTICALLY"] = 2] = "TILED_VERTICALLY";
        JoinMethod[JoinMethod["TILED_HORIZONTALLY"] = 3] = "TILED_HORIZONTALLY";
    })(JoinMethod = exports.JoinMethod || (exports.JoinMethod = {}));
    /**
     * This class creates a resizable three piece tiled image that can be used as a backplate.
     *
     * HOW TO USE:
     * this.backplate = new ThreePieceImage(game, ['topscore-leftplate', 'topscore-midplate',
     * 'topscore-rightplate'], JoinMethod.TILED_HORIZONTALLY)
     * this.backplate.height = this.game.height * 0.1; ~~~~~~~~~~~~~~~~~~~~> if it is joined
     * HORIZONTALLY, scale the backplate's height, and vice versa.
     * this.backplate.scale.x = this.backplate.scale.y;
     *
     * 	-- After this.content is created and filled --
     * this.backplate.resize(this.content.width);
     * this.content.centerX = this.backplate.centerX;
     * this.content.centerY = this.backplate.centerY;
     *
     * An example that uses this can be found in the MinigameTitleDialog class
     *
     * @author qi@cerebralfix.com
     */
    class ThreePieceImage extends Phaser.Group {
        constructor(game, assetKeys, joinMethod) {
            super(game);
            this.joinMethod = joinMethod;
            this.firstPiece = new Phaser.Image(game, 0, 0, assetKeys[0]);
            this.add(this.firstPiece);
            // we need to create this as a Phaser Image first to get the Original Width and Height of
            // the asset so we can use it later
            this.middlePiece = new Phaser.Image(game, 0, 0, assetKeys[1]);
            this.lastPiece = new Phaser.Image(game, 0, 0, assetKeys[2]);
            // if join method is Stretch, create the middle piece as a Phaser Image
            // if join method is Tile, create the middle piece as a Phaser TileSprite
            if (joinMethod === JoinMethod.TILED_HORIZONTALLY || joinMethod === JoinMethod.TILED_VERTICALLY) {
                this.middlePiece = new Phaser.TileSprite(game, 0, 0, this.middlePiece.width, this.middlePiece.height, assetKeys[1]);
            }
            // if method is Vertical (tile / stretch) place the pieces below each other
            if (joinMethod === JoinMethod.STRETCHED_VERTICALLY || joinMethod === JoinMethod.TILED_VERTICALLY) {
                this.middlePieceOrigWidthOrHeight = this.middlePiece.height; // save this off for resizing later
                this.middlePiece.top = this.firstPiece.bottom;
                this.add(this.middlePiece);
                this.lastPiece.top = this.middlePiece.bottom;
                this.add(this.lastPiece);
            }
            else if (joinMethod === JoinMethod.STRETCHED_HORIZONTALLY || joinMethod === JoinMethod.TILED_HORIZONTALLY) {
                this.middlePieceOrigWidthOrHeight = this.middlePiece.width; // save this off for resizing later
                this.middlePiece.left = this.firstPiece.right;
                this.add(this.middlePiece);
                this.lastPiece.left = this.middlePiece.right;
                this.add(this.lastPiece);
            }
        }
        /**
         * Resizes the image to be the @param widthOrHeight that is passed in by adding middle tiles
         *
         * @param widthOrHeight width or height that is needed (if the join method is horizontal
         * this will be the width needed, and vice versa)
         * @param padding (optional) extra padding on both sides (multiplied by 2)
         * @param joinMethod (optional) allows the altering of the join method
         */
        resize(widthOrHeight, padding = 0) {
            // removes all extra pieces and resets the size of the group
            // based on the old join method
            this.removeBetween(3, undefined, true); // removes everything except the first 3 pieces added
            switch (this.joinMethod) {
                case JoinMethod.TILED_VERTICALLY: {
                    this.tileVertically(widthOrHeight, padding);
                    break;
                }
                case JoinMethod.TILED_HORIZONTALLY: {
                    this.tileHorizontally(widthOrHeight, padding);
                    break;
                }
                case JoinMethod.STRETCHED_VERTICALLY: {
                    this.stretchVertically(widthOrHeight, padding);
                    break;
                }
                case JoinMethod.STRETCHED_HORIZONTALLY: {
                    this.stretchHorizontally(widthOrHeight, padding);
                    break;
                }
                default: {
                    break;
                }
            }
        }
        /////////////////////////////////////////////////////////////////////////////
        // TILE:
        // duplicates and tiles the middle piece to fill the missing piece or height
        /////////////////////////////////////////////////////////////////////////////
        tileHorizontally(width, padding) {
            // calculates the number of extra middle pieces needed
            const widthOfFirstAndLastPiece = (this.firstPiece.width + this.lastPiece.width) * this.scale.x;
            const missingWidth = Math.max(0, (width - widthOfFirstAndLastPiece) + (padding * 2));
            const numberOfExtraMidPiecesNeeded = Math.ceil(missingWidth / (this.middlePieceOrigWidthOrHeight * this.scale.x));
            this.middlePiece.width = numberOfExtraMidPiecesNeeded * this.middlePieceOrigWidthOrHeight;
            this.lastPiece.left = this.middlePiece.right;
        }
        tileVertically(height, padding) {
            // calculates the number of extra middle pieces needed
            const heightOfFirsttToLastPiece = (this.firstPiece.height + this.lastPiece.height) * this.scale.y;
            const missingHeight = Math.max(0, (height - heightOfFirsttToLastPiece) + (padding * 2));
            const numberOfExtraMidPiecesNeeded = Math.ceil(missingHeight / (this.middlePieceOrigWidthOrHeight * this.scale.y));
            this.middlePiece.height = numberOfExtraMidPiecesNeeded * this.middlePieceOrigWidthOrHeight;
            this.lastPiece.top = this.middlePiece.bottom;
        }
        ////////////////////////////////////////////////////////////////////////
        // STRETCH:
        // stretches the middle piece to fill the missing width or height
        ////////////////////////////////////////////////////////////////////////
        stretchHorizontally(width, padding) {
            // calculates the amount of missing width needed
            const widthOfFirstAndLastPiece = (this.firstPiece.width + this.lastPiece.width) * this.scale.x;
            const missingWidth = Math.max(0, (width - widthOfFirstAndLastPiece) + (padding * 2));
            this.middlePiece.width = missingWidth / this.scale.x;
            this.lastPiece.left = this.middlePiece.right;
        }
        stretchVertically(height, padding) {
            // calculates the amount of missing height needed
            const heightOfFirstAndLastPiece = (this.firstPiece.height + this.lastPiece.height) * this.scale.y;
            const missingHeight = Math.max(0, (height - heightOfFirstAndLastPiece) + (padding * 2));
            this.middlePiece.height = missingHeight / this.scale.y;
            this.lastPiece.top = this.middlePiece.bottom;
        }
    }
    exports.ThreePieceImage = ThreePieceImage;
});
define("component/dialog/BasicCTADialog", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil", "Game", "switchback-common/component/ThreePieceImage"], function (require, exports, PromiseUtil_1, TextUtil_1, Game_1, ThreePieceImage_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * A simple cta dialog with no prompt.
     * Fits to content.
     * Able to show text content, or a custom content.
     * @author Joel@CerebralFix.com
     * @author Jeronimo@CerebralFix.com
     */
    class BasicCTADialog extends Phaser.Group {
        constructor(game) {
            super(game);
            this.content = null;
            this.currentScale = 1;
            this.textField = null;
            this.visible = false;
            this.content = null;
            this.textField = null;
            this.backplate = new ThreePieceImage_1.ThreePieceImage(game, ['instruction-dialog-backplate-top',
                'instruction-dialog-backplate-mid',
                'instruction-dialog-backplate-btm-noprompt'], ThreePieceImage_1.JoinMethod.STRETCHED_VERTICALLY);
            this.backplate.width = game.width * 0.95;
            this.backplate.scale.y = this.backplate.scale.x;
            this.add(this.backplate);
        }
        // sets the correct size for the backplate as well as the correct position of the content
        refit() {
            const padY = this.game.height * 0.1;
            // stretches the 3 piece image based on the content's height
            this.backplate.resize(this.content.height, padY);
            // animations require the dialog to have a center anchor
            this.backplate.centerX = 0;
            this.backplate.centerY = 0;
            // positions the context (text stuff) based on the backplate
            this.content.centerX = this.backplate.centerX;
            this.content.top = this.backplate.top + (padY * 0.4);
        }
        /* Sets the content to a text content
        @param title : the title shown in different style
        @param body : the text on the body with its own style.
        @return Phaser.Text[] : returns a list of the titleField and textField so they
        can be animated
        */
        setTextContent(title, body) {
            const content = new Phaser.Group(this.game);
            const titleField = new Phaser.Text(this.game, 0, 0, title, {
                font: Game_1.Game.DEFAULT_FONT,
                fontSize: TextUtil_1.TextUtil.getFontSize(this.game.width, 40),
                fill: '#ffac00',
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.backplate.width * 0.55
            });
            titleField.setShadow(this.game.width * 0.005, this.game.width * 0.005, Game_1.Game.TEXT_SHADOW_DIAG, 0, true, true);
            titleField.lineSpacing = titleField.fontSize * -0.3;
            TextUtil_1.TextUtil.scaleDownTextToHeight(titleField, this.backplate.height * 0.22);
            this.textField = new Phaser.Text(this.game, 0, 0, body, {
                font: Game_1.Game.DEFAULT_FONT,
                fontSize: TextUtil_1.TextUtil.getFontSize(this.game.width, 30),
                fill: 'white',
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.backplate.width * 0.8
            });
            this.textField.setShadow(this.game.width * 0.005, this.game.width * 0.005, Game_1.Game.TEXT_SHADOW_DIAG, 0, true, true);
            this.textField.top = titleField.bottom;
            this.textField.centerY = Math.max(this.backplate.height * 0.25, this.textField.centerY);
            this.textField.centerX = titleField.centerX;
            content.add(titleField);
            content.add(this.textField);
            this.setCustomContent(content);
        }
        // Set the content to a custom one, replacing the old one if any.
        // This is usable on the instructionDialogs
        setCustomContent(newContent) {
            if (this.content) {
                this.content.destroy();
            }
            this.content = newContent;
            this.add(this.content);
            this.refit();
        }
        // Shows the dialog sliding from left
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.right = 0;
                this.visible = true;
                // slides right to the center of screen
                yield PromiseUtil_1.PromiseUtil.tween(this.game, this.game.add.tween(this)
                    .to({ centerX: this.game.width * 0.5 }, 500, Phaser.Easing.Sinusoidal.In, true));
            });
        }
        // Hides the dialog sliding to the right
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                // slides right till it is out of the screen
                yield PromiseUtil_1.PromiseUtil.tween(this.game, this.game.add.tween(this)
                    .to({ left: this.game.width }, 500, Phaser.Easing.Sinusoidal.In, true));
                this.visible = false;
            });
        }
        // Shows the dialog scaling from 0
        showScaling() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.scale.x > 0) {
                    this.currentScale = this.scale.x;
                }
                this.scale.set(0, 0);
                this.alpha = 0;
                this.visible = true;
                // fade text box in
                this.game.add.tween(this).to({ alpha: 1 }, 300, Phaser.Easing.Linear.None, true);
                // scale text box up
                yield PromiseUtil_1.PromiseUtil.tween(this.game, this.game.add.tween(this.scale).to({ x: this.currentScale, y: this.currentScale }, 250, Phaser.Easing.Linear.None, true));
            });
        }
        // Hides the dialog scaling to 0
        hideScaling() {
            return __awaiter(this, void 0, void 0, function* () {
                // makes the dialog invisible
                this.game.add.tween(this).to({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true);
                // scale the box down
                yield PromiseUtil_1.PromiseUtil.tween(this.game, this.game.add.tween(this.scale).to({ x: 0, y: 0 }, 250, Phaser.Easing.Linear.None, true));
                this.visible = false;
            });
        }
        waitForTap() {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_1.PromiseUtil.signal(this.onChildInputDown);
            });
        }
    }
    exports.BasicCTADialog = BasicCTADialog;
});
define("component/dialog/ConfirmationDialog", ["require", "exports", "switchback-common/util/TextUtil", "switchback-common/util/PromiseUtil", "Game", "component/dialog/BasicCTADialog"], function (require, exports, TextUtil_2, PromiseUtil_2, Game_2, BasicCTADialog_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ConfirmationDialog extends Phaser.Group {
        constructor(game, titleText, confirmationQnText, leftBtnString, rightButtonString, isNeutral, shadeOpacity = 0.75) {
            super(game);
            this.onButtonClickedSignal = new Phaser.Signal();
            this.shade = new Phaser.Graphics(this.game, 0, 0);
            this.shade.beginFill(0x000000, shadeOpacity);
            this.shade.drawRect(0, 0, this.game.width, this.game.height);
            this.shade.endFill();
            this.shade.alpha = 0;
            this.add(this.shade);
            /**
             * Top banner that descends down
             */
            this.topBanner = new Phaser.Group(game);
            const bannerHeader = new Phaser.Image(game, 0, 0, 'banner-header');
            bannerHeader.width = game.width;
            bannerHeader.scale.y = bannerHeader.scale.x;
            this.topBanner.add(bannerHeader);
            this.add(this.topBanner);
            this.topBanner.bottom = 0;
            this.topBanner.visible = false;
            /**
             * Bottom banner that rises up
             * Includes: banner footer, cancel and exit big buttons
             */
            this.bottomBanner = new Phaser.Group(game);
            const bannerFooter = new Phaser.Image(game, 0, 0, 'banner-footer');
            bannerFooter.width = game.width * 1.22;
            bannerFooter.scale.y = bannerFooter.scale.x;
            this.bannerFooterHeight = bannerFooter.height;
            this.leftBtn = new Phaser.Image(game, 0, 0, (isNeutral ? 'big-btn-left-neutral' : 'big-btn-left-clean'));
            this.leftBtn.width = bannerFooter.width * 0.48;
            this.leftBtn.scale.y = this.leftBtn.scale.x;
            this.leftBtn.top = bannerFooter.top - (this.leftBtn.height * 0.1);
            this.leftBtn.right = bannerFooter.width * 0.49;
            this.leftBtn.inputEnabled = true;
            this.leftBtn.events.onInputDown.add(() => {
                this.game.sound.play('snd_back');
                this.onButtonClickedSignal.dispatch(false);
            }, this);
            const bigBtnTextStyle = {
                font: Game_2.Game.CARNIVAL_FONT,
                fontSize: TextUtil_2.TextUtil.getFontSize(game.width, 45),
                fill: 'white',
                stroke: 'black',
                strokeThickness: this.leftBtn.width * 0.025
            };
            const leftBtnText = new Phaser.Text(game, 0, 0, leftBtnString, bigBtnTextStyle);
            leftBtnText.anchor.set(0.5);
            let paddingY = this.leftBtn.height * 0.02;
            leftBtnText.centerX = this.leftBtn.centerX;
            leftBtnText.centerY = this.leftBtn.centerY - paddingY;
            leftBtnText.angle = -4;
            const leftBtnLights = new Phaser.Sprite(game, 0, 0, 'big-btn-lights');
            leftBtnLights.width = this.leftBtn.width;
            leftBtnLights.scale.y = leftBtnLights.scale.x;
            leftBtnLights.centerX = this.leftBtn.centerX;
            leftBtnLights.centerY = this.leftBtn.centerY;
            leftBtnLights.animations.add('anticlockwise', [5, 4, 3, 2, 1, 0], 15, true);
            leftBtnLights.animations.play('anticlockwise');
            this.rightBtn = new Phaser.Image(game, 0, 0, (isNeutral ? 'big-btn-right-neutral' : 'big-btn-right-clean'));
            this.rightBtn.width = this.leftBtn.width;
            this.rightBtn.scale.y = this.rightBtn.scale.x;
            this.rightBtn.top = this.leftBtn.top;
            this.rightBtn.left = bannerFooter.width * 0.51;
            this.rightBtn.inputEnabled = true;
            this.rightBtn.events.onInputDown.add(() => {
                this.game.sound.play('snd_ui_click');
                this.onButtonClickedSignal.dispatch(true);
            }, this);
            const rightBtnText = new Phaser.Text(game, 0, 0, rightButtonString, bigBtnTextStyle);
            rightBtnText.anchor.set(0.5);
            rightBtnText.centerX = this.rightBtn.centerX;
            rightBtnText.centerY = this.rightBtn.centerY - paddingY;
            rightBtnText.angle = 5;
            const rightBtnLights = new Phaser.Sprite(game, 0, 0, 'big-btn-lights');
            rightBtnLights.width = this.rightBtn.width;
            rightBtnLights.scale.y = rightBtnLights.scale.x;
            rightBtnLights.scale.x = -rightBtnLights.scale.x;
            rightBtnLights.centerX = this.rightBtn.centerX;
            rightBtnLights.centerY = this.rightBtn.centerY;
            rightBtnLights.animations.add('anticlockwise', [5, 4, 3, 2, 1, 0], 15, true);
            rightBtnLights.animations.play('anticlockwise');
            this.bottomBanner.add(bannerFooter);
            this.bottomBanner.add(leftBtnLights);
            this.bottomBanner.add(this.leftBtn);
            this.bottomBanner.add(leftBtnText);
            this.bottomBanner.add(rightBtnLights);
            this.bottomBanner.add(this.rightBtn);
            this.bottomBanner.add(rightBtnText);
            this.add(this.bottomBanner);
            this.bottomBanner.top = game.world.height;
            this.bottomBanner.centerX = game.world.centerX;
            this.bottomBanner.visible = false;
            /**
             * Text box that fades and scales in
             */
            this.textBox = new BasicCTADialog_1.BasicCTADialog(this.game);
            this.textBox.setTextContent(titleText, confirmationQnText);
            this.add(this.textBox);
            this.textBox.centerX = game.world.centerX;
            this.textBox.centerY = game.world.height * 0.45;
            this.textBox.visible = false;
            // makes the entire dialog invisible until it is shown
            this.visible = false;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.visible = true;
                this.shade.inputEnabled = true;
                this.game.add.tween(this.shade).to({ alpha: 1 }, 500, Phaser.Easing.Linear.None, true);
                yield this.showBanners();
                yield this.textBox.showScaling();
                this.leftBtn.inputEnabled = true;
                this.rightBtn.inputEnabled = true;
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                this.leftBtn.inputEnabled = false;
                this.rightBtn.inputEnabled = false;
                this.game.add.tween(this.shade).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true);
                yield this.textBox.hideScaling();
                yield this.hideBanners();
                this.shade.inputEnabled = false;
                this.visible = false;
            });
        }
        showBanners() {
            return __awaiter(this, void 0, void 0, function* () {
                this.bottomBanner.visible = true;
                this.topBanner.visible = true;
                // move top banner down
                this.game.add.tween(this.topBanner).to({ y: 0 }, 300, Phaser.Easing.Linear.None, true);
                // move bottom banner up
                yield PromiseUtil_2.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomBanner).to({ top: this.game.height - this.bannerFooterHeight }, 300, Phaser.Easing.Linear.None, true));
            });
        }
        hideBanners() {
            return __awaiter(this, void 0, void 0, function* () {
                // move top banner up
                this.game.add.tween(this.topBanner).to({ bottom: 0 }, 150, Phaser.Easing.Linear.None, true);
                // move bottom banner down
                yield PromiseUtil_2.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomBanner).to({ top: this.game.height }, 150, Phaser.Easing.Linear.None, true));
                this.topBanner.visible = false;
                this.bottomBanner.visible = false;
            });
        }
        destroy() {
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.ConfirmationDialog = ConfirmationDialog;
});
/**
 * This class exists to support on-desktop testing of data saving and loading.
 * It should only be used by the MockPlayAPI.
 * @author joel@cerebralfix.com
 * @author nick@cerebralfix.com
 */
define("switchback-common/util/CookieUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CookieUtil {
        static get(name) {
            const nameEQ = name + "=";
            const ca = document.cookie.split(";");
            for (let i = 0; i < ca.length; i++) {
                let c = ca[i];
                while (c.charAt(0) === " ") {
                    c = c.substring(1, c.length);
                }
                if (c.indexOf(nameEQ) === 0) {
                    return decodeURIComponent(c.substring(nameEQ.length, c.length));
                }
            }
            return null;
        }
        static set(name, value, daysTilExpiry = 1) {
            const expiry = new Date();
            expiry.setTime(expiry.getTime() + daysTilExpiry * 24 * 60 * 60 * 1000);
            document.cookie = name + "=" + encodeURIComponent(value) + "; expires=" + expiry.toUTCString() + "; path = /";
        }
        static erase(name) {
            CookieUtil.set(name, "", -1);
        }
    }
    exports.CookieUtil = CookieUtil;
});
define("switchback-common/play/api/MockPlayAPI", ["require", "exports", "switchback-common/util/CookieUtil", "switchback-common/util/PromiseUtil", "switchback-common/component/Signal"], function (require, exports, CookieUtil_1, PromiseUtil_3, Signal_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * This class exists to support on-desktop testing of the PlayAPI.
     * It returns a bunch of fake data, and can save and load data via browser storage.
     * @author joel@cerebralfix.com
     */
    class MockPlayAPI {
        static init() {
            // Overwrite the PlayAPI functions
            PlayAPI.gameInit = MockPlayAPI.gameInit;
            PlayAPI.locationRead = MockPlayAPI.locationRead;
            PlayAPI.locationWaitTimeRead = MockPlayAPI.locationWaitTimeRead;
            PlayAPI.gameStateRead = MockPlayAPI.gameStateRead;
            PlayAPI.gameStateSave = MockPlayAPI.gameStateSave;
            PlayAPI.gameStatsSave = MockPlayAPI.gameStatsSave;
            PlayAPI.gameStatsRead = MockPlayAPI.gameStatsRead;
            PlayAPI.openPlayerManagement = MockPlayAPI.openPlayerManagement;
            PlayAPI.captureImage = MockPlayAPI.captureImage;
            PlayAPI.saveToPhotos = MockPlayAPI.saveToPhotos;
            PlayAPI.shareImage = MockPlayAPI.shareImage;
            PlayAPI.startCamera = MockPlayAPI.startCamera;
            PlayAPI.cameraAccessStatus = MockPlayAPI.cameraAccessStatus;
            PlayAPI.endCamera = MockPlayAPI.endCamera;
            PlayAPI.closeGame = MockPlayAPI.closeGame;
            PlayAPI.grantAchievementToPlayers = MockPlayAPI.grantAchievementToPlayers;
            PlayAPI.analyticsStateLog = MockPlayAPI.analyticsStateLog;
            PlayAPI.analyticsActionLog = MockPlayAPI.analyticsActionLog;
            PlayAPI.analyticsActionObjectLog = MockPlayAPI.analyticsActionObjectLog;
            PlayAPI.analyticsTimedActionStart = MockPlayAPI.analyticsTimedActionStart;
            PlayAPI.analyticsTimedActionEnd = MockPlayAPI.analyticsTimedActionEnd;
            PlayAPI.vibrateDevice = MockPlayAPI.vibrateDevice;
            PlayAPI.subscribeToEvent = MockPlayAPI.subscribeToEvent;
            PlayAPI.unsubscribeToEvent = MockPlayAPI.unsubscribeToEvent;
            PlayAPI.DBeacon.sessionInit = MockPlayAPI.dbSessionInit;
            PlayAPI.DBeacon.sessionEnd = MockPlayAPI.dbSessionEnd;
            PlayAPI.DBeacon.triggerEffect = MockPlayAPI.triggerEffect;
            PlayAPI.AugmentedReality.sessionStart = MockPlayAPI.arSessionStart;
            PlayAPI.AugmentedReality.sessionEnd = MockPlayAPI.arSessionEnd;
            PlayAPI.AugmentedReality.registerImage = MockPlayAPI.arRegisterImage;
            PlayAPI.AugmentedReality.isAvailable = MockPlayAPI.arIsAvailable;
            PlayAPI.LocalStorage.list = MockPlayAPI.localStorageList;
            PlayAPI.LocalStorage.read = MockPlayAPI.localStorageRead;
            PlayAPI.LocalStorage.write = MockPlayAPI.localStorageWrite;
            PlayAPI.LocalStorage.unlink = MockPlayAPI.localStorageUnlink;
            PlayAPI.cancelSpeech = MockPlayAPI.cancelSpeech;
            PlayAPI.speak = MockPlayAPI.speak;
            PlayAPI.listAvailableVoices = MockPlayAPI.listAvailableVoices;
            // TODO: Figure out why this causes TSM to not load
            // (window as any).webkit = {messageHandlers:{NativeInterop:{postMessage: function (msg) {}}}};
        }
        static subscribeToEvent(event, handler) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] subscribeToEvent()");
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return {};
            });
        }
        static unsubscribeToEvent(event, handler) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] unsubscribeToEvent()");
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return {};
            });
        }
        static vibrateDevice() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] vibrateDevice()");
                return {};
            });
        }
        static startCamera(cameraType) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] startCamera()");
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    if (MockPlayAPI.TEST_REJECTED_CAMERA_PERMISSIONS) {
                        reject();
                    }
                    else {
                        resolve({});
                    }
                }));
            });
        }
        static cameraAccessStatus() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] cameraAccessStatus()");
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return {
                    status: (MockPlayAPI.TEST_REJECTED_CAMERA_PERMISSIONS ? CameraAccessStatus.DENIED : CameraAccessStatus.AUTHORIZED)
                };
            });
        }
        static endCamera() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] endCamera()");
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsActionLog(action, category) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsActionLog(${action}, ${category})`);
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsActionObjectLog(action, data) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsActionObjectLog(${action}, ${JSON.stringify(data)})`);
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsStateLog(state, data) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsStateLog(${state}, ${JSON.stringify(data)})`);
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsTimedActionStart(state, data) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsTimedActionStart(${state}, ${JSON.stringify(data)})`);
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return {};
            });
        }
        static analyticsTimedActionEnd(state) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] analyticsTimedActionEnd(${state}})`);
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return {};
            });
        }
        static grantAchievementToPlayers(achievementId, playersIds, showNotification) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[MockPlayAPI] grantAchievementToPlayers()");
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return {};
            });
        }
        static gameInit() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameInit()`);
                yield PromiseUtil_3.PromiseUtil.realTime(500);
                return MockPlayAPI.getFakeGameInitResult();
            });
        }
        static locationRead() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] locationRead()`);
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return MockPlayAPI.getFakeLocation();
            });
        }
        static locationWaitTimeRead(location) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] locationWaitTimeRead()`);
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return MockPlayAPI.getFakeWaitTime();
            });
        }
        static gameStateRead() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameStateRead()`);
                return MockPlayAPI.doGameStateRead();
            });
        }
        static gameStateSave(status, state) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameStateSave()`);
                // CookieUtil.set(MockPlayAPI.GAME_STATE_TAG, JSON.stringify(state));
                return {};
            });
        }
        static dbSessionInit() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] dbSessionInit()`);
                return {};
            });
        }
        static triggerEffect(effectName, data, timeout) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] triggerEffect()`);
                return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    yield PromiseUtil_3.PromiseUtil.realTime(timeout * 1000);
                    reject(); // The MockAPI should never succeed here
                }));
            });
        }
        static dbSessionEnd() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] dbSessionEnd()`);
                return {};
            });
        }
        static arSessionStart(canvasElement) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] arSessionStart()`);
                PlayAPI.AugmentedReality.renderer = new THREE.WebGLRenderer({ alpha: true, canvas: canvasElement });
                PlayAPI.AugmentedReality.scene = new THREE.Scene();
                PlayAPI.AugmentedReality.renderer.setSize(window.innerWidth, window.innerHeight);
                PlayAPI.AugmentedReality.renderer.setPixelRatio(window.devicePixelRatio);
                PlayAPI.AugmentedReality.renderer.shadowMap.enabled = false;
                return {};
            });
        }
        static arSessionEnd() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] arSessionEnd()`);
                return {};
            });
        }
        static arRegisterImage(imageName, pathToImageInGameBundle, imageWidthInMeters) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] arRegisterImage(${imageName}, ${pathToImageInGameBundle}, ${imageWidthInMeters})`);
                return {};
            });
        }
        static arIsAvailable() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] arIsAvailable()`);
                return true;
            });
        }
        static localStorageList() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] localStorageList()`);
                return {
                    contents: []
                };
            });
        }
        static localStorageRead(fileName) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] localStorageRead()`);
                return {};
            });
        }
        static localStorageWrite(fileName, data) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] localStorageWrite()`);
                return {};
            });
        }
        static localStorageUnlink(fileName) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] localStorageUnlink()`);
                return {};
            });
        }
        static gameStatsRead() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameStatsRead()`);
                return MockPlayAPI.doGameStatsRead();
            });
        }
        static gameStatsSave(gameStats) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] gameStatsSave()`);
                CookieUtil_1.CookieUtil.set(MockPlayAPI.GAME_STATS_TAG, JSON.stringify(gameStats));
                return {};
            });
        }
        static openPlayerManagement() {
            return __awaiter(this, void 0, void 0, function* () {
                return MockPlayAPI.getFakePlayerManagementData();
            });
        }
        static captureImage() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] captureImage()`);
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return null;
            });
        }
        static saveToPhotos() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] saveToPhotos()`);
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return null;
            });
        }
        static shareImage() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[MockPlayAPI] shareImage()`);
                yield PromiseUtil_3.PromiseUtil.realTime(300);
                return true;
            });
        }
        static closeGame() {
            return __awaiter(this, void 0, void 0, function* () {
                return MockPlayAPI.fakeCloseGame();
            });
        }
        static getFakeLocation() {
            const response = {
                attractionName: "Little Foot's Wild Ride",
                landName: "Land Before Time",
                latitude: 66.33,
                locationID: "beacon_" + Math.floor(Math.random() * 20),
                longitude: 100.22,
                parkName: "Animal Kingdom",
            };
            return response;
        }
        static doGameStateRead() {
            console.log(`[MockPlayAPI] doGameStateRead()`);
            let gameState;
            const cookieData = JSON.parse(CookieUtil_1.CookieUtil.get(MockPlayAPI.GAME_STATE_TAG));
            if (cookieData == null) {
                gameState = MockPlayAPI.getFakeGameState();
            }
            else {
                gameState = {
                    created: cookieData.created || "",
                    state: cookieData.state || {},
                    status: cookieData.status || "1",
                    updated: cookieData.updated || "",
                };
            }
            return gameState;
        }
        static doGameStatsRead() {
            console.log(`[MockPlayAPI] doGameStatsRead()`);
            let gameStats;
            const cookieData = JSON.parse(CookieUtil_1.CookieUtil.get(MockPlayAPI.GAME_STATS_TAG));
            if (cookieData == null) {
                gameStats = MockPlayAPI.getFakeGameStats();
            }
            else {
                gameStats = {
                    score: cookieData.score || 0,
                };
            }
            return gameStats;
        }
        static listAvailableVoices() {
            return __awaiter(this, void 0, void 0, function* () {
                if (window.speechSynthesis) {
                    let voicesList = window.speechSynthesis.getVoices();
                    if (voicesList.length === 0) {
                        const voicesLoadedSignal = new Signal_1.Signal();
                        window.speechSynthesis.onvoiceschanged = () => {
                            voicesLoadedSignal.dispatch();
                        };
                        yield PromiseUtil_3.PromiseUtil.commonSignal(voicesLoadedSignal);
                        voicesList = window.speechSynthesis.getVoices();
                    }
                    voicesList.forEach(element => {
                        element["identifier"] = element.voiceURI;
                    });
                    return { "voices": voicesList };
                }
                else {
                    return { "voices": [] };
                }
            });
        }
        static speak(text, voiceId) {
            return __awaiter(this, void 0, void 0, function* () {
                if (window.speechSynthesis) {
                    const textToRead = new SpeechSynthesisUtterance(text);
                    const voice = window.speechSynthesis.getVoices().find((singleVoice) => {
                        return singleVoice.voiceURI === voiceId;
                    });
                    textToRead.voice = voice;
                    window.speechSynthesis.speak(textToRead);
                }
                return {};
            });
        }
        static cancelSpeech() {
            return __awaiter(this, void 0, void 0, function* () {
                if (window.speechSynthesis) {
                    window.speechSynthesis.cancel();
                }
                return {};
            });
        }
        /////////////////////////////////
        // FAKE DATA GENERATION
        /////////////////////////////////
        static getFakeGameInitResult() {
            const response = {
                gameDefinitionRead: MockPlayAPI.getFakeGameDefinition(),
                gameStateRead: MockPlayAPI.doGameStateRead(),
                playerInfoRead: MockPlayAPI.getFakePlayerInformation(),
                isGuestOnSite: true,
            };
            return response;
        }
        static getFakeWaitTime() {
            const response = {
                waitTime: Math.floor(Math.random() * 150),
            };
            return response;
        }
        static getFakePlayerInformation() {
            const response = {
                avatar: "",
                email: "example@gmail.com",
                name: "Joe A"
            };
            return response;
        }
        static getFakeGameDefinition() {
            const response = {
                stateSchema: {},
                statsSchema: {},
                title: "game_definition_title",
            };
            return response;
        }
        static getFakeGameState() {
            const response = {
                created: "game_state_created",
                state: "game_state_state",
                status: "game_state_status",
                updated: "game_state_updated",
            };
            return response;
        }
        static getFakeGameStats() {
            const response = {
                score: 0,
            };
            return response;
        }
        static getFakePlayerManagementData(limit = null) {
            const players = [{ name: "Cinderella", type: "TYPE_PRIMARY", identifier: "" },
                { name: "Elsa", type: "TYPE_SECONDARY", identifier: "" },
                { name: "Snow White", type: "TYPE_SECONDARY", identifier: "" },
                { name: "Leia", type: "TYPE_SECONDARY", identifier: "" },
                { name: "Tinker Bell", type: "TYPE_SECONDARY", identifier: "" },
            ];
            if (limit != null) {
                players.splice(limit);
            }
            return { players };
        }
        static fakeCloseGame() {
            console.log("The game got closed using mockplayApi!");
            return {};
        }
    }
    MockPlayAPI.TEST_REJECTED_CAMERA_PERMISSIONS = false;
    MockPlayAPI.GAME_STATE_TAG = "game-state";
    MockPlayAPI.GAME_STATS_TAG = "game-stats";
    exports.MockPlayAPI = MockPlayAPI;
});
define("switchback-common/enum/Park", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Listing of Park 'types'
     *
     * I know it's not an enum, but this is to prevent
     * people with older TS versions having issues.
     */
    class Park {
    }
    Park.DLR = "disneylandResort";
    Park.WDW = "waltDisneyWorld";
    exports.Park = Park;
});
define("StateNames", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class StateNames {
    }
    StateNames.LOAD = "Load";
    StateNames.INTRO = "Intro";
    StateNames.HUB = "Hub";
    StateNames.OUTRO = "Outro";
    StateNames.MINIGAME_TITLE = "MinigameTitle";
    StateNames.GAME_FLYING_TOSSERS = "Game-flying-tossers";
    StateNames.GAME_DRESS_UP = "Game-dress-up";
    StateNames.GAME_SARGE = "Game-sarge";
    StateNames.GAME_HAMM = "Game-hamm";
    StateNames.GAME_WOODY_BALANCE = "Game-woody-balance";
    StateNames.GAME_WOODY_LASSO = "Game-woody-lasso";
    StateNames.GAME_WOODY_THROW = "Game-woody-throw";
    StateNames.GAME_TRIXIE = "Game-trixie";
    StateNames.GAME_DRAWING = "Game-drawing";
    StateNames.RESULTS = "Game-results";
    StateNames.CUT_SCENE = "Cutscene";
    StateNames.POSTER = "Poster";
    StateNames.POSTER_INSTALLATION = "PosterInstallation";
    exports.StateNames = StateNames;
});
define("switchback-common/util/CollectionUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CollectionUtil {
        /**
         * Swaps two values of @param array by value (in place)
         */
        static swapByValue(array, x, y) {
            const xIndex = array.indexOf(x);
            const yIndex = array.indexOf(y);
            CollectionUtil.swapByIndex(array, xIndex, yIndex);
        }
        /**
         * Swaps two values of @param array by index (in place)
         */
        static swapByIndex(array, xIndex, yIndex) {
            if (xIndex >= 0 && yIndex >= 0) {
                let a = array[xIndex];
                array[xIndex] = array[yIndex];
                array[yIndex] = a;
            }
            else {
                console.warn(`[CollectionUtil] swap(): could not swap elements - one or more elements not present in array`);
            }
        }
        /**
         * Recursively merges 2 objects. The spread (...) operator doesn't act on
         * nested properties - this does.
         * https://stackoverflow.com/a/40276429/1343176
         * { "test": { "val1": 1 } }
         * +
         * { "test": { "val2": 2 } }
         * =
         * { "test": { "val1": 1, "val2": 2 } }
         */
        static mergeObjects(obj1, obj2) {
            for (var key in obj2) {
                if (typeof (obj1[key]) === 'object') {
                    CollectionUtil.mergeObjects(obj1[key], obj2[key]);
                }
                else {
                    if (obj1[key] || typeof (obj2[key]) === 'boolean') {
                        obj1[key] = obj2[key];
                    }
                }
            }
            return obj1;
        }
        /**
         * Returns an array pre-filled with sequential values from 0...(length - 1)
         * i.e. @param length = 5 will return [0, 1, 2, 3, 4]
         */
        static generateArrayWithSequentialNumbers(length) {
            return Array.from(Array(length).keys());
        }
        /**
         * Shuffles an array and then checks to see how 'shuffled' it is.
         * If more than half of the original members are in the same index
         * as the shuffled members, it shuffles again. A safeguard
         * counter is in place to prevent it infinitely looping.
         */
        static guaranteeShuffle(array) {
            let shuffledCopy = shuffle(Object.assign([], array));
            let safeguard = 0;
            while (CollectionUtil.checkArrayShuffleCoefficient(array, shuffledCopy) > 0.5 && safeguard < 1000) {
                shuffledCopy = shuffle(Object.assign([], array));
                safeguard++;
            }
            ;
            return shuffledCopy;
        }
        static checkArrayShuffleCoefficient(array, copy) {
            let numSame = 0;
            array.forEach((val, index) => {
                if (val === copy[index]) {
                    numSame++;
                }
            });
            return numSame / array.length;
        }
    }
    exports.CollectionUtil = CollectionUtil;
    function shuffle(array) {
        const arrayCopy = array;
        for (let i = array.length; i; i--) {
            const j = Math.floor(Math.random() * i);
            [arrayCopy[i - 1], arrayCopy[j]] = [array[j], array[i - 1]];
        }
        return arrayCopy;
    }
    exports.shuffle = shuffle;
    // puts the last index in first place before shuffling the rest
    function shuffleNonRepeatingArray(originalArray) {
        if (originalArray.length <= 1) {
            return originalArray;
        }
        // removes the last item in the array
        const lastItem = originalArray.pop();
        // shuffle the array without the last item in it
        originalArray = shuffle(originalArray);
        /*
        * inserts the last item back into the middle of the array
        * so that the it will not end up as lastItem forever
        */
        originalArray.splice(Math.ceil(originalArray.length * 0.5), 0, lastItem);
        return originalArray;
    }
    exports.shuffleNonRepeatingArray = shuffleNonRepeatingArray;
    /**
     * Finds the 2D index of an item in a 2D array
     * @param arr Array to search
     * @param k Item to look for
     */
    function getIndexIn2DArray(arr, k) {
        for (let i = 0; i < arr.length; i++) {
            const index = arr[i].indexOf(k);
            if (index > -1) {
                return { row: i, col: index };
            }
        }
    }
    exports.getIndexIn2DArray = getIndexIn2DArray;
    /**
     * Returns an array of numbers from 0 to length exclusive eg. getNumberRange(3) returns [0, 1, 2]
     * @param length number, the end of the range
     */
    function getNumberRange(length) {
        return Array.apply(null, { length: length }).map(Number.call, Number);
    }
    exports.getNumberRange = getNumberRange;
    /**
     * Returns an array of numbers from start (inclusive) to start + length (exclusive) eg. getNumberRange(2, 3) returns [2, 3, 4]
     * @param start number, the first number in the range
     * @param length number, the end of the range
     */
    function getNumberRangeFrom(start, length) {
        let range = [];
        getNumberRange(length).forEach((n) => {
            range.push(n + start);
        });
        return range;
    }
    exports.getNumberRangeFrom = getNumberRangeFrom;
    function cloneArray(arr) {
        let i, copy;
        if (Array.isArray(arr)) {
            copy = arr.slice(0);
            for (i = 0; i < copy.length; i++) {
                copy[i] = cloneArray(copy[i]);
            }
            return copy;
        }
        else if (typeof arr === 'object') {
            throw 'Cannot clone array containing an object!';
        }
        else {
            return arr;
        }
    }
    exports.cloneArray = cloneArray;
    function exclusiveLeftJoinArrays(leftList, rightList) {
        const exclusiveLeftJoin = leftList.filter(leftItem => {
            return rightList.indexOf(leftItem) === -1;
        });
        return exclusiveLeftJoin;
    }
    exports.exclusiveLeftJoinArrays = exclusiveLeftJoinArrays;
    /**
     * Return an array containing the matching elements from two arrays
     */
    function innerJoinArrays(leftList, rightList) {
        const innerJoin = leftList.filter(leftItem => {
            return rightList.indexOf(leftItem) >= 0;
        });
        return innerJoin;
    }
    exports.innerJoinArrays = innerJoinArrays;
    function getRandomItem(list) {
        const randomIndex = Math.floor(Math.random() * Math.floor(list.length));
        return list.length === 0 ? null : list[randomIndex];
    }
    exports.getRandomItem = getRandomItem;
});
define("logic/PlayerManager", ["require", "exports", "switchback-common/util/CollectionUtil"], function (require, exports, CollectionUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Tracks current player,
     * Handles changing and randomizing of players,
     * Ensures player will never have two turns in a row,
     * Handles killing of current player.
     *
     * How to use:
     * PlayerController.Instance.changePlayer()
     * PlayerController.Instance.currentPlayer.name
     *
     * If the minigame requires killing of any players, use:
     * removeCurrentPlayer()
     * resetPlayers() [in minigame's shutdown]
     * playerManager.outOfPlayersSignal.addOnce(this.gameOver, this);
     */
    class PlayerManager {
        constructor() {
            this.outOfPlayersSignal = new Phaser.Signal();
        }
        /*
        Player grouping
        Returns a session, which is a list of groups. A group is a list of players that plays in a round.
        The first player in a group is important because is the captain.
        Everybody will play at least twice.
        A, B, C, D, etc are randomized, but not the rest of the pattern.
        That means that the initial player list is randomized but not the session pattern, which makes it
        more fair, predictable and AC compliant (cooler?)
        @return list of groups. group=list of players : [ [A, B], [B, C], [C, A]]
        @param GroupSize number = 1 : how many players per group. =1 individual game, =2 pair game, =more for bigger groups
            If groupSize is bigger than the number of available players this will return []
        @param rounds number = null : hom many rounds are needed, =null as many as possible without repeating,
            if there aren't enough players it will start repeating
        @param fixedPairs boolean = false : If true, for pair games, avoids making the players switch sides
            after sequential rounds.
        */
        getPlayerSession(groupSize = 1, rounds = null, fixedPairs = false) {
            let result = [];
            // This is important, ensures that the list is fully free/fresh, that way a reshuffle will not happen in the middle of the query
            this.refreshPlayersArray();
            // if no rounds is specified, use as many rounds as necesary to make everybody play AS CAPTAIN (if applicable, or once)
            if (rounds == null) {
                rounds = this.playersInGame.length;
            }
            if (groupSize > this.playersInGame.length) {
                console.log("[GameController] Error, groupsize is bigger than playes."); // Not removing in case this return introduces bugs when not taken into account
                return [];
            }
            // set the pool of players
            let pool = [];
            while (pool.length < this.playersInGame.length) {
                pool.push(this.currentPlayer);
                this.changePlayer();
            }
            for (let i = 0; i < rounds; i++) {
                let group = pool.slice(0, groupSize);
                // In case the game is in pairs and we want static positions, revert every other group 
                if (fixedPairs && (groupSize === 2) && (i % 2 === 0)) {
                    group.reverse();
                }
                pool.push(pool.shift());
                result.push(group);
            }
            console.log("[GameController] Players session is ", result);
            return result;
            /* TODO after implementing other games,
            consider using an enum insteaf of groupSize & fixed pairs, so call is more clear
                enum GameType {
                    INDIVIDUAL,
                    PAIR_ALTERNATING,
                    PAIR_ROTATING,
                    GROUP
                }
            */
        }
        /* Changes the player index to the next one, upon reaching the end of the list,
        It will re-shuffle them ensuring the last player is not the first again */
        increaseNextPlayerIndex() {
            this.nextPlayerIndex++;
            if (this.nextPlayerIndex >= this.playersInGame.length) {
                // balancedly shuffle the array
                this.playersInGame = CollectionUtil_1.shuffleNonRepeatingArray(this.playersInGame);
                this.nextPlayerIndex = 0;
            }
        }
        /**
         * Calling this gives a fresh array with the players remaining
         * appended to the front of the array
         */
        getRefreshedPlayersArray() {
            this.refreshPlayersArray();
            return this.playersInGame;
        }
        /**
         * It is currently called AFTER a changePlayer() call
         * Calling this refreshes the players array with the players remaining
         * appended to the front of the array
         *
         * eg: if currentPlayer was 'Cinderella',
         * ['Cinderella' || 'Snow White', 'Leia', 'Elsa'] => ['Snow White', 'Leia', 'Elsa', 'Cinderella']
         * ['Leia', 'Cinderella' || 'Snow White', 'Elsa'] => ['Snow White', 'Elsa', 'Cinderella', 'Leia']
         * ['Snow White', 'Leia', 'Elsa', 'Cinderella'] => fully shuffled new array
         */
        refreshPlayersArray() {
            const newArray = this.playersInGame.slice();
            const indexOfCurrentPlayer = this.playersInGame.indexOf(this._currentPlayer);
            const numberOfPlayersRemaining = newArray.length - indexOfCurrentPlayer;
            // remove players that are still remaining in the array
            let remainingPlayers = [];
            remainingPlayers = newArray.splice(indexOfCurrentPlayer, numberOfPlayersRemaining);
            // shuffle the rest of the players
            Phaser.ArrayUtils.shuffle(newArray);
            // put remianing players back to the front of the array 
            if (remainingPlayers.length > 0) {
                for (let i = 0; i < remainingPlayers.length; i++) {
                    newArray.splice(i, 0, remainingPlayers[i]);
                }
            }
            this.playersInGame = newArray;
            this.nextPlayerIndex = 0;
        }
        // this is called every time the player management screen is opened
        setupPlayers(players) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[PlayerManager] setupPlayers()`);
                this.allPlayers = players.slice();
                Phaser.ArrayUtils.shuffle(this.allPlayers);
                this.playersInGame = this.allPlayers.slice();
                this.nextPlayerIndex = 0;
                // **Do not change this line.
                // If changed could cause a bug in changePlayer()
                this._currentPlayer = this.changePlayer();
            });
        }
        // call this to get the current player name
        get currentPlayer() {
            return this._currentPlayer;
        }
        /* Call this _AFTER_ the current player is done. To change to the next one
         Read note in _currentPlayer definition */
        changePlayer() {
            console.log(`[PlayerManager] changePlayer()`);
            let nextPlayer = this.playersInGame[this.nextPlayerIndex];
            this.increaseNextPlayerIndex();
            // check to ensure players will never get two turns in a row,
            // this catches any bugs that may arise any edge cases
            // So far this happens due to the way its initialized, and the fact that changePlayers is called AFTER
            // the player finished playing (which its how it was designed to be used)
            // shuffleNonRepeatingArray seems to be working fine, so this is not caused by that
            if (nextPlayer === this._currentPlayer) {
                console.log('[PlayerManager] Edge case found, player got 2 turns in a row. Player index moved again');
                nextPlayer = this.playersInGame[this.nextPlayerIndex];
                this.increaseNextPlayerIndex();
            }
            this._currentPlayer = nextPlayer;
            return this._currentPlayer;
        }
        // removes the current player from the minigame
        // and returns the player removed
        removeCurrentPlayer() {
            console.log(`[PlayerManager] removeCurrentPlayer()`);
            const playerRemoved = this._currentPlayer;
            const indexOfKilledPlayer = this.playersInGame.indexOf(this._currentPlayer);
            this.playersInGame.splice(indexOfKilledPlayer, 1);
            this.nextPlayerIndex--;
            // covers the edge case where if the index was at 0 and was reduced to become -1 which would crash the game
            if (this.nextPlayerIndex < 0) {
                this.nextPlayerIndex = this.playersInGame.length - 1;
            }
            // if there are still more than 1 player left, change the player
            if (this.playersInGame.length > 1) {
                this.changePlayer();
            }
            else {
                this._currentPlayer = this.playersInGame[0];
                // dispatch signal to say that it ran out of players!
                this.outOfPlayersSignal.dispatch();
            }
            return playerRemoved;
        }
        get numPlayersLeftInGame() {
            return this.playersInGame.length;
        }
        // if there had previously been players being killed off from the game,
        // recreate the players array for the new minigame
        resetPlayers() {
            console.log(`[PlayerManager] resetPlayers()`);
            if (this.numPlayersLeftInGame < this.allPlayers.length) {
                this.playersInGame = this.allPlayers.slice();
                this.nextPlayerIndex = 0;
            }
        }
    }
    exports.PlayerManager = PlayerManager;
});
define("model/GameInfo", ["require", "exports", "logic/GameController"], function (require, exports, GameController_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class NodeInfo {
        parse(data) {
            this.position = data.position || { x: 0, y: 0 };
            this.characterTag = data.characterTag || "";
            this.nodeTags = data.nodeTags || [];
            this.boxTint = Number(data.boxTint) || 0xFFFFFF;
            this.upgradesSelf = data.upgradesSelf !== undefined ? data.upgradesSelf : true;
            this.visibleWhenLocked = data.visibleWhenLocked !== undefined ? data.visibleWhenLocked : true;
            this.upgradeText = data.upgradeText || {};
            this.titleText = data.titleText || [];
            return this;
        }
        get maxLevel() {
            return this.nodeTags.length;
        }
    }
    exports.NodeInfo = NodeInfo;
    class GameInfo {
        parse(data) {
            this.stateName = data.stateName;
            this.introCutscenes = data.introCutscenes || [];
            this.resultsCutscenes = data.resultsCutscenes || [];
            this.titleCharacter = data.titleCharacter || "";
            this.resultsCharacter = data.resultsCharacter || "";
            this.titleTextKey = data.titleTextKey || "";
            this.titleCharacterKey = data.titleCharacterKey || "";
            this.scoreIconTag = data.scoreIconTag || "";
            this.scoreboardLabelKey = data.scoreboardLabelKey || "";
            this.quotaPerPerson = data.quotaPerPerson || 0;
            this.minigameSettings = data.minigameSettings || {};
            this.difficultyLevels = data.difficultyLevels || {};
            if (data.node) {
                this.node = new NodeInfo().parse(data.node);
            }
            return this;
        }
        get introCutsceneName() {
            const numPlays = GameController_1.GameController.Instance.state.getMinigameSaveData(this.stateName).numPlays;
            const numCutscenes = GameController_1.GameController.Instance.gameInfo.get(this.stateName).introCutscenes.length;
            return (numCutscenes === 0 ? "" : GameController_1.GameController.Instance.gameInfo.get(this.stateName).introCutscenes[numPlays % numCutscenes]);
        }
        get resultsCutsceneName() {
            const numPlays = GameController_1.GameController.Instance.state.getMinigameSaveData(this.stateName).numPlays;
            const numCutscenes = GameController_1.GameController.Instance.gameInfo.get(this.stateName).resultsCutscenes.length;
            return (numCutscenes === 0 ? "" : GameController_1.GameController.Instance.gameInfo.get(this.stateName).resultsCutscenes[numPlays % numCutscenes]);
        }
    }
    exports.GameInfo = GameInfo;
});
define("model/ToyStoryConfig", ["require", "exports", "switchback-common/enum/Park"], function (require, exports, Park_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Config singleton. Parsed from json.
     * @author joel@cerebralfix.com
     */
    class ToyStoryConfig {
        static get Instance() {
            return this.s_instance || (this.s_instance = new this());
        }
        parse(data) {
            if (typeof data === 'string') {
                try {
                    data = JSON.parse(data);
                }
                catch (e) {
                    console.log('Unable to parse config.');
                }
            }
            this.enableConsoleLogs = data.enableConsoleLogs === undefined ? false : data.enableConsoleLogs;
            this.enableFPS = data.enableFPS === undefined ? false : data.enableFPS;
            this.usePlayAPI = data.usePlayAPI === undefined ? true : data.usePlayAPI;
            this.debug = data.debug === undefined ? false : data.debug;
            this.quickTest = data.quickTest === undefined ? false : data.quickTest;
            this.unlockAllHubNodes = data.unlockAllHubNodes === undefined ? false : data.unlockAllHubNodes;
            this.skipCutscenes = data.skipCutscenes === undefined ? false : data.skipCutscenes;
            this.showDebugMenu = data.showDebugMenu === undefined ? false : data.showDebugMenu;
            this.testAllBuzzLevels = data.testAllBuzzLevels === undefined ? false : data.testAllBuzzLevels;
            this.drawDebugOverlays = data.drawDebugOverlays === undefined ? false : data.drawDebugOverlays;
            this.skipGames = data.skipGames === undefined ? false : data.skipGames;
            this.useWDWExitBeacon = data.useWDWExitBeacon === undefined ? false : data.useWDWExitBeacon;
            this.useWDWExitTimer = data.useWDWExitTimer === undefined ? false : data.useWDWExitTimer;
            this.useWDWInstallation = data.useWDWInstallation === undefined ? false : data.useWDWInstallation;
            this.showEffectNameDLR = data.showEffectNameDLR === undefined ? "dlr-dca-tsmm-puffer" : data.showEffectNameDLR;
            this.showEffectNameWDW = data.showEffectNameWDW === undefined ? "wdw-dhs-tsm-stinky" : data.showEffectNameWDW;
            // Defaulting this to DLR now that we cannot guarantee the gameInit response will give us something we expect
            this.park = data.park || Park_1.Park.DLR;
        }
    }
    exports.ToyStoryConfig = ToyStoryConfig;
});
define("switchback-common/play/AchievementManager", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class AchievementManager {
        initialise(players, achievementPrefix) {
            this.playerIds = players.map((player) => player.identifier);
            this.achievementPrefix = achievementPrefix;
            console.log(`[AchievementManager] initialise(${this.playerIds}, ${achievementPrefix}`);
        }
        grantAchievement(achievementId) {
            return __awaiter(this, void 0, void 0, function* () {
                const achievementString = `${this.achievementPrefix}-${achievementId}`;
                yield PlayAPI.grantAchievementToPlayers(achievementString, this.playerIds, true).then(() => {
                    console.log(`achievement ${achievementString} grant success`);
                }, (reason) => {
                    console.log(`achievement ${achievementString} grant fail`);
                    console.error(JSON.stringify(reason));
                });
            });
        }
    }
    exports.AchievementManager = AchievementManager;
});
define("switchback-common/localization/LocalizationManager", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Localization singleton. Parsed from json.
     * @author joel@cerebralfix.com
     */
    class LocalizationManager {
        static get Instance() {
            return this.instance || (this.instance = new this());
        }
        init(pLocale, data) {
            if (data === null) {
                throw new Error("Localization data not found");
            }
            this.polyglot = new Polyglot({
                locale: pLocale,
                phrases: data,
            });
            this.isReversed = false;
        }
        text(phrase) {
            if (!phrase) {
                return null;
            }
            if (this.isReversed) {
                return this.reverseString(this.polyglot.t(phrase));
            }
            return this.polyglot.t(phrase);
        }
        textWithCount(phrase, smartCount) {
            if (!phrase) {
                return null;
            }
            if (this.isReversed) {
                return this.reverseString(this.polyglot.t(phrase, smartCount));
            }
            return this.polyglot.t(phrase, smartCount);
        }
        textWithReplacements(phrase, interpolationOptions) {
            if (!phrase) {
                return null;
            }
            if (this.isReversed) {
                return this.reverseString(this.polyglot.t(phrase, interpolationOptions));
            }
            return this.polyglot.t(phrase, interpolationOptions);
        }
        setReversed(reversed) {
            this.isReversed = reversed;
        }
        /**
         * Please note that this method of reversal should only be used for
         * debugging purposes, and won't properly handle non-ASCII chars
         */
        reverseString(source) {
            return source.split("").reverse().join("");
        }
    }
    exports.LocalizationManager = LocalizationManager;
});
define("component/hub/HubNode", ["require", "exports", "logic/GameController", "switchback-common/localization/LocalizationManager"], function (require, exports, GameController_2, LocalizationManager_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HubNode extends Phaser.Group {
        constructor(game, stateName, maxLevel = HubNode.MAX_NODE_LEVEL) {
            super(game);
            this.playGameSignal = new Phaser.Signal();
            this.onChildInputDown.add(this.onNodeClicked, this);
            this.stateName = stateName;
            this.maxLevel = maxLevel;
        }
        onNodeClicked() {
            this.playGameSignal.dispatch(this.stateName, false);
        }
        get shouldUpgrade() {
            return this.targetLevel > this.visualLevel && this.visualLevel < this.maxLevel;
        }
        get isViewUnlocked() {
            return this.visualLevel > 0;
        }
        get shouldUnlock() {
            return !this.isViewUnlocked && this.targetLevel > 0;
        }
        get visualLevel() {
            return GameController_2.GameController.Instance.state.getGameVisualLevel(this.stateName);
        }
        get cappedVisualLevel() {
            return Math.max(0, Math.min(this.visualLevel, this.maxLevel));
        }
        get targetLevel() {
            return GameController_2.GameController.Instance.state.getGameTargetLevel(this.stateName);
        }
        get speakingCharacter() {
            return GameController_2.GameController.Instance.gameInfo.get(this.stateName).node.upgradeText[this.cappedVisualLevel].character;
        }
        get characterText() {
            const key = GameController_2.GameController.Instance.gameInfo.get(this.stateName).node.upgradeText[this.cappedVisualLevel].stringKey;
            return LocalizationManager_1.LocalizationManager.Instance.text(key);
        }
        unlock() {
            return __awaiter(this, void 0, void 0, function* () {
                GameController_2.GameController.Instance.state.incrementGameVisualLevel(this.stateName);
            });
        }
        upgradeOne() {
            return __awaiter(this, void 0, void 0, function* () {
                GameController_2.GameController.Instance.state.incrementGameVisualLevel(this.stateName);
            });
        }
        destroy() {
            this.playGameSignal.dispose();
            super.destroy();
        }
    }
    HubNode.MAX_NODE_LEVEL = 3;
    exports.HubNode = HubNode;
});
define("logic/TSMAchievementManager", ["require", "exports", "switchback-common/play/AchievementManager", "logic/GameController", "StateNames", "component/hub/HubNode"], function (require, exports, AchievementManager_1, GameController_3, StateNames_1, HubNode_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TSMAchievement {
    }
    TSMAchievement.UNBOX_ALL = "toyStoryUnboxed";
    TSMAchievement.BUZZ_EXTRA = "toyStorySpaceRanger";
    TSMAchievement.HAMM_EXTRA = "toyStorySnoutedOut";
    TSMAchievement.WOODY_EXTRA = "toyStorySteadyAsARock";
    TSMAchievement.TRIXIE_EXTRA = "toyStoryOverstuffed";
    TSMAchievement.EXTRAVAGANZA = "toyStoryExtravaganza";
    TSMAchievement.SARGE_EXTRA = "toyStoryInCharge";
    TSMAchievement.COMPLETE_WOODY = "toyStoryYeeHaw";
    TSMAchievement.COMPLETE_BUZZ = "toyStoryToInfinity";
    TSMAchievement.COMPLETE_SARGE = "toyStoryShippingOut";
    TSMAchievement.COMPLETE_TRIXIE = "toyStoryJurassicDart";
    TSMAchievement.COMPLETE_HAMM = "toyStoryGruntWork";
    TSMAchievement.COMPLETE_REX = "toyStoryDressToImpress";
    TSMAchievement.JESSIE_EXTRA = "toyStoryQuickDraw";
    class TSMAchievementManager extends AchievementManager_1.AchievementManager {
        constructor() {
            super(...arguments);
            this.hammScoreMultiplier = 5;
            this.trixieBalloonMultiplier = 15;
            this.minWoodyTargetsDropped = 0;
            this.sargePlatesMultiplier = 8;
            this.rexGamePlays = 5;
            this.drawingDuration = 5000; // seconds
        }
        onGameUnlocked() {
            return __awaiter(this, void 0, void 0, function* () {
                // Need to check number of plays of each core game
                // Will need to check based on name, as we can't assume the order the games will play in
                if (GameController_3.GameController.Instance.getAreAllGamesUnlocked()) {
                    yield this.grantAchievement(TSMAchievement.UNBOX_ALL);
                }
            });
        }
        onVisualLevelUpgrade() {
            return __awaiter(this, void 0, void 0, function* () {
                let needsUpgrade = false;
                GameController_3.GameController.CORE_GAMES.forEach((gameName) => {
                    const visualLevel = GameController_3.GameController.Instance.state.getGameVisualLevel(gameName);
                    // There are issues with accessing the max levels for each node without being on the hub state
                    // Each of the core games is currently defaulting to the same max level
                    if (visualLevel < HubNode_1.HubNode.MAX_NODE_LEVEL) {
                        needsUpgrade = true;
                    }
                });
                if (!needsUpgrade) {
                    yield this.grantAchievement(TSMAchievement.EXTRAVAGANZA);
                }
            });
        }
        onQuotaReached(game) {
            return __awaiter(this, void 0, void 0, function* () {
                let achievement;
                switch (game) {
                    case StateNames_1.StateNames.GAME_TRIXIE: {
                        achievement = TSMAchievement.COMPLETE_TRIXIE;
                        break;
                    }
                    case StateNames_1.StateNames.GAME_WOODY_LASSO: {
                        achievement = TSMAchievement.COMPLETE_WOODY;
                        break;
                    }
                    case StateNames_1.StateNames.GAME_FLYING_TOSSERS: {
                        achievement = TSMAchievement.COMPLETE_BUZZ;
                        break;
                    }
                    case StateNames_1.StateNames.GAME_SARGE: {
                        achievement = TSMAchievement.COMPLETE_SARGE;
                        break;
                    }
                    case StateNames_1.StateNames.GAME_HAMM: {
                        achievement = TSMAchievement.COMPLETE_HAMM;
                        break;
                    }
                }
                yield this.grantAchievement(achievement);
            });
        }
        onRexEndGame() {
            return __awaiter(this, void 0, void 0, function* () {
                const numPlays = GameController_3.GameController.Instance.state.getMinigameSaveData(StateNames_1.StateNames.GAME_DRESS_UP).numPlays;
                if (numPlays >= this.rexGamePlays) {
                    yield this.grantAchievement(TSMAchievement.COMPLETE_REX);
                }
            });
        }
        onBuzzExtraAchieved() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.grantAchievement(TSMAchievement.BUZZ_EXTRA);
            });
        }
        onHammEndGame(guessedCount) {
            return __awaiter(this, void 0, void 0, function* () {
                // Target score is currently 5 per player
                const targetScore = this.playerIds.length * this.hammScoreMultiplier;
                if (targetScore > 0 && guessedCount >= targetScore) {
                    yield this.grantAchievement(TSMAchievement.HAMM_EXTRA);
                }
            });
        }
        onWoodyBalancingEndGame(targetsDropped) {
            return __awaiter(this, void 0, void 0, function* () {
                if (targetsDropped <= this.minWoodyTargetsDropped) {
                    yield this.grantAchievement(TSMAchievement.WOODY_EXTRA);
                }
            });
        }
        onTrixieEndGame(numBalloons) {
            return __awaiter(this, void 0, void 0, function* () {
                // Target score is currently 15 per player
                const targetScore = this.playerIds.length * this.trixieBalloonMultiplier;
                if (targetScore > 0 && numBalloons >= targetScore) {
                    yield this.grantAchievement(TSMAchievement.TRIXIE_EXTRA);
                }
            });
        }
        onSargeEndGame(numCrates) {
            return __awaiter(this, void 0, void 0, function* () {
                const targetCratesCaught = this.playerIds.length * this.sargePlatesMultiplier;
                if (targetCratesCaught > 0 && numCrates >= targetCratesCaught) {
                    yield this.grantAchievement(TSMAchievement.SARGE_EXTRA);
                }
            });
        }
        onDrawingEndTurn(duration) {
            return __awaiter(this, void 0, void 0, function* () {
                if (duration <= this.drawingDuration) {
                    yield this.grantAchievement(TSMAchievement.JESSIE_EXTRA);
                }
            });
        }
    }
    exports.TSMAchievementManager = TSMAchievementManager;
});
define("switchback-common/play/AnalyticsManager", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class AnalyticsManager {
        constructor() {
            // Needs to be overwritten on the extending class
            this.gaNameMap = new Map();
            this.timings = new Map();
            this.gaCommonNameMap = new Map([
                ["park.name", "dimension1"],
                ["line.type", "dimension2"],
                ["attraction.estimatedWaitTime", "dimension3"],
                // hotfix, removing the function /renaming the dimensions have implications that might be wanted to be discussed
                // ['attraction.actualWaitTime', 'dimension4'],
                ["party.size", "dimension5"],
                ["party.visits", "dimension6"],
            ]);
            this.ga = null;
        }
        /**
         * "A" = Standard
         * "B" = Other
         * "N/A" = No queue distinctions
         */
        set lineType(type) {
            this.baseData["line.type"] = type;
        }
        set estimatedWaitTime(time) {
            this.baseData["attraction.estimatedWaitTime"] = time;
        }
        set numPartyVisits(numVisits) {
            this.baseData["party.visits"] = numVisits;
        }
        set actualWaitTime(time) {
            // Disabled because we were advised that this parameter should not be in adobe metrics
            // hotfix, removing the function /renaming the dimensions have implications that might be wanted to be discussed
            // after that this code can be cleaned up
            // this.baseData['attraction.actualWaitTime'] = time;
        }
        /**
         * Starts a timing event.
         * For Adobe, this is automatic.
         * For Google analytics, we record the time ourselves.
         *
         * @param adobeTimingKey The specific action being timed e.g. 'Trigger'
         */
        startTiming(adobeTimingKey, data = {}) {
            this.timings.set(adobeTimingKey, new Date().getTime());
            PlayAPI.analyticsTimedActionStart(adobeTimingKey, Object.assign({}, this.baseData, data));
        }
        /**
         * Ends and sends a timing event.
         *
         * @param adobeTimingKey The specific action being timed e.g. 'Trigger'
         * @param gaCategory The category of this event e.g. 'Installation'
         * @param gaLabel Additional information about the action e.g. 'Poster'
         * @param data Any associated data object. This will get sent through to supported trackers.
         */
        endTiming(adobeTimingKey, gaTimingKey, gaCategory, gaLabel, data = {}) {
            return __awaiter(this, void 0, void 0, function* () {
                const prevTime = this.timings.get(adobeTimingKey);
                if (prevTime) {
                    const totalTime = new Date().getTime() - prevTime;
                    this.ga("send", "timing", Object.assign({ timingCategory: gaCategory, timingLabel: gaLabel, timingValue: totalTime, timingVar: gaTimingKey }, this.transformFieldsObject(Object.assign({}, this.baseData, data))));
                }
                yield PlayAPI.analyticsTimedActionEnd(adobeTimingKey).then((result) => {
                    console.log(result);
                }).catch((error) => {
                    console.error(error);
                });
            });
        }
        /**
         * Tracks an Adobe 'Action' or a Google 'event'.
         *
         * @param gaCategory The category of action e.g. 'Game'
         * @param gaAction The specific action they took e.g. 'Play'
         * @param gaLabel Further information about that action e.g. 'Woody'
         * @param data Any associated data object. This will get sent through to supported trackers.
         */
        trackAction(adobeAction, gaCategory, gaAction, gaLabel, data = {}) {
            PlayAPI.analyticsActionObjectLog(adobeAction, Object.assign({}, this.baseData, data));
            // might be pending integration into PlayApp
            this.ga("send", Object.assign({ eventAction: gaAction, eventCategory: gaCategory, eventLabel: gaLabel, hitType: "event" }, this.transformFieldsObject(Object.assign({}, this.baseData, data))));
        }
        /**
         * Tracks an Adobe 'state' or a Google 'pageview'
         *
         * @param adobeStatePath e.g. 'content/game/inqueue/resume'
         * @param gaStateId e.g. 'Resume'
         * @param data Any associated data object. This will get sent through to supported trackers.
         */
        trackState(adobeStatePath, gaStateId, data = {}) {
            PlayAPI.analyticsStateLog(adobeStatePath, Object.assign({}, this.baseData, data));
            // might be pending integration into PlayApp
            this.ga("send", Object.assign({ hitType: "pageview", page: gaStateId }, this.transformFieldsObject(Object.assign({}, this.baseData, data))));
        }
        initialise(category, landName, attractionName, gameId, gameName, parkName, lineType, hasPrimaryPlayer, partySize, partyVisits, useGA = false) {
            this.baseData = {
                "attraction.name": attractionName,
                "game.id": gameId,
                "game.name": gameName,
                "land.name": landName,
                "line.type": lineType,
                "link.category": category,
                "park.name": parkName,
                "party.size": partySize,
                "party.visits": partyVisits,
                "primary.player": hasPrimaryPlayer ? 1 : 0,
            };
            // Combine both our name maps
            this.gaNameMap = new Map([
                ...Array.from(this.gaNameMap.entries()),
                ...Array.from(this.gaCommonNameMap.entries()),
            ]);
            if (useGA) {
                this.ga = window.ga;
            }
            else {
                this.ga = (...params) => { console.log(...params); };
            }
        }
        /**
         * Extract custom data into a format Google Analytics will accept
         */
        transformFieldsObject(data) {
            const transformedData = {};
            Object.keys(data).forEach((key) => {
                const mapping = this.gaNameMap.get(key);
                if (mapping && data[key] !== undefined && data[key] !== null) {
                    if (mapping.startsWith("dimension")) {
                        data[key] = data[key].toString(); // GA requires custom dimensions pass through as strings
                    }
                    transformedData[mapping] = data[key];
                }
            });
            return transformedData;
        }
    }
    AnalyticsManager.PARK_WDW = "WDW";
    AnalyticsManager.PARK_DLR = "DLR";
    AnalyticsManager.LAND_HS = "HS";
    AnalyticsManager.LAND_MK = "MK";
    AnalyticsManager.LAND_DCA = "DCA";
    AnalyticsManager.LAND_DL = "DL";
    AnalyticsManager.LAND_AK = "AK";
    AnalyticsManager.QUEUE_A = "A";
    AnalyticsManager.QUEUE_B = "B";
    AnalyticsManager.QUEUE_NONE = "N/A";
    exports.AnalyticsManager = AnalyticsManager;
});
define("switchback-common/play/ShowEffectController", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var ShowEffectResult;
    (function (ShowEffectResult) {
        ShowEffectResult[ShowEffectResult["FAILURE"] = 0] = "FAILURE";
        ShowEffectResult[ShowEffectResult["SUCCESS"] = 1] = "SUCCESS";
        ShowEffectResult[ShowEffectResult["IN_USE"] = 2] = "IN_USE";
    })(ShowEffectResult = exports.ShowEffectResult || (exports.ShowEffectResult = {}));
    class ShowEffectController {
        constructor() {
            this.showControlResult = new Phaser.Signal();
        }
        init() {
            console.log('[ShowEffectController] init()');
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_FAILURE, (response) => {
                console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_FAILURE received:\n" + JSON.stringify(response));
                this.showControlResult.dispatch(ShowEffectResult.FAILURE, response);
            }).then((result) => {
                console.log("PlayAPI.subscribeToEvent(SHOW_CONTROL_EFFECT_TRIGGER_FAILURE) succeeded:\n" + JSON.stringify(result));
            }).catch((error) => {
                console.error("PlayAPI.subscribeToEvent(SHOW_CONTROL_EFFECT_TRIGGER_FAILURE) failed:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS, (response) => {
                console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS received:\n" + JSON.stringify(response));
                this.showControlResult.dispatch(ShowEffectResult.SUCCESS, response);
            }).then((result) => {
                console.log("PlayAPI.subscribeToEvent(SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS) succeeded:\n" + JSON.stringify(result));
            }).catch((error) => {
                console.error("PlayAPI.subscribeToEvent(SHOW_CONTROL_EFFECT_TRIGGER_SUCCESS) failed:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RUNNING, (response) => {
                console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RUNNING received:\n" + JSON.stringify(response));
                this.showControlResult.dispatch(ShowEffectResult.IN_USE, response);
            }).then((result) => {
                console.log("PlayAPI.subscribeToEvent(SHOW_CONTROL_SHOW_RUNNING) succeeded:\n" + JSON.stringify(result));
            }).catch((error) => {
                console.error("PlayAPI.subscribeToEvent(SHOW_CONTROL_SHOW_RUNNING) failed:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RESETTING, (response) => {
                console.log("PlayAPI.DBeacon.events.SHOW_CONTROL_SHOW_RESETTING received:\n" + JSON.stringify(response));
                this.showControlResult.dispatch(ShowEffectResult.IN_USE, response);
            }).then((result) => {
                console.log("PlayAPI.subscribeToEvent(SHOW_CONTROL_SHOW_RESETTING) succeeded:\n" + JSON.stringify(result));
            }).catch((error) => {
                console.error("PlayAPI.subscribeToEvent(SHOW_CONTROL_SHOW_RESETTING) failed:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            PlayAPI.DBeacon.sessionInit().then((result) => {
                console.log("PlayAPI.DBeacon.sessionInit successfully called:\n" + JSON.stringify(result));
            }, (error) => {
                console.error("PlayAPI.DBeacon.sessionInit error:\n" + JSON.stringify(error));
                this.isInitializeFailed = true;
            });
            // N.B. We have to subscribe to this in order for the success events to work!
            // Do not remove!!
            PlayAPI.subscribeToEvent(PlayAPI.DBeacon.events.SHOW_CONTROL_EFFECT_IN_RANGE, (response) => { return; });
            console.log('[ShowEffectController] init() complete');
        }
        shutdown() {
            PlayAPI.DBeacon.sessionEnd().then((result) => {
                console.log("PlayAPI.DBeacon.sessionEnd successfully called:\n" + JSON.stringify(result));
            }, (error) => {
                console.error("PlayAPI.DBeacon.sessionEnd error:\n" + JSON.stringify(error));
            });
        }
        get defaultShowEffectName() {
            return this._defaultShowEffectName;
        }
        set defaultShowEffectName(effectName) {
            this._defaultShowEffectName = effectName;
        }
        triggerDefaultEffect(timeoutS = 5) {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this._defaultShowEffectName) {
                    return ShowEffectResult.FAILURE;
                }
                return yield this.triggerEffect(this._defaultShowEffectName, timeoutS);
            });
        }
        /**
         * Triggers an effect via the Play API.
         * with forced local timeout
         */
        triggerEffect(effectName, timeoutS = 5) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[ShowEffectController] triggerEffect() " + effectName);
                return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                    if (this.isInitializeFailed) {
                        resolve(ShowEffectResult.FAILURE);
                        return;
                    }
                    let secondaryTimeoutId;
                    this.showControlResult.add((result, response) => {
                        console.log("[ShowEffectController] triggerEffect() this.showControlResult.add(): " + result + " " + response.experienceName);
                        // ensure we heard the show control message for the effect we are waiting for
                        if (response.experienceName === effectName) {
                            if (secondaryTimeoutId !== undefined) {
                                clearTimeout(secondaryTimeoutId);
                            }
                            this.showControlResult.removeAll();
                            resolve(result);
                        }
                    });
                    PlayAPI.DBeacon.triggerEffect(effectName, {}, timeoutS).then((result) => {
                        console.log("PlayAPI.DBeacon.triggerEffect successfully called:\n" + JSON.stringify(result));
                        // additional safety check incase any of the show controller events fail to respond
                        // this is double the time that the PlayAPI says that requires to complete its calls
                        // this is only required as in somecases the PlayAPI has not been returning anything
                        secondaryTimeoutId = setTimeout(() => {
                            console.warn("PlayAPI.ShowControl.triggerEffect forced local timeout");
                            this.showControlResult.removeAll();
                            resolve(ShowEffectResult.FAILURE);
                        }, timeoutS * 2000);
                    }, (error) => {
                        console.error("PlayAPI.ShowControl.triggerEffect error:\n" + error);
                        if (secondaryTimeoutId !== undefined) {
                            clearTimeout(secondaryTimeoutId);
                        }
                        this.showControlResult.removeAll();
                        resolve(ShowEffectResult.FAILURE);
                    });
                }));
            });
        }
    }
    exports.ShowEffectController = ShowEffectController;
});
define("logic/TSMAnalyticsManager", ["require", "exports", "switchback-common/play/AnalyticsManager", "switchback-common/enum/Park", "logic/GameController", "StateNames"], function (require, exports, AnalyticsManager_1, Park_2, GameController_4, StateNames_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /*
    @author Joel@CerebralFix.com
    @author Jeronimo@CerebralFix.com
    */
    class TSMAnalyticsManager extends AnalyticsManager_1.AnalyticsManager {
        constructor() {
            super(...arguments);
            this.gaNameMap = new Map([
                [TSMAnalyticsManager.GAME_NAME, 'dimension7'],
                [TSMAnalyticsManager.GAME_NAME_MICRO, 'dimension8'],
                [TSMAnalyticsManager.GAME_SCORE, 'dimension9'],
                [TSMAnalyticsManager.GAME_SCORE_TOTAL, 'dimension10'],
                [TSMAnalyticsManager.GAME_PLAYS, 'dimension11'],
                [TSMAnalyticsManager.GAME_ALL_PLAYS, 'dimension12'],
                [TSMAnalyticsManager.GAME_PROGRESS, 'dimension13'],
                [TSMAnalyticsManager.CUTSCENE, 'dimension14'],
                [TSMAnalyticsManager.INSTALLATION_TRIGGER, 'dimension15'],
                [TSMAnalyticsManager.INSTALLATION_NAME, 'dimension16']
            ]);
            /// ~Posters
            /////// ~ Custom
        }
        static get Instance() {
            return this.s_instance || (this.s_instance = new this());
        }
        init(park, parkString, hasPrimaryPlayer, partySize, partyVisit) {
            let landString = park === Park_2.Park.WDW ? AnalyticsManager_1.AnalyticsManager.LAND_HS : AnalyticsManager_1.AnalyticsManager.LAND_DCA;
            super.initialise("InQueue", landString, "Toy Story", "toyStoryMania", "Party Playset", parkString, AnalyticsManager_1.AnalyticsManager.QUEUE_NONE, hasPrimaryPlayer, partySize, partyVisit);
            // TODO consider passing gaNameMap as parameter
            GameController_4.GameController.Instance.getEstimatedWaitTime().then((waitTime) => {
                // Using then instead of await to avoid locking the load time
                this.estimatedWaitTime = waitTime;
            });
        }
        //////////////////
        //// Custom calls
        // returns data pre-filled with `game.allPlays` and `game.progress`.
        getCommons() {
            const gc = GameController_4.GameController.Instance;
            const gameAllPlays = gc.state.allPlays; // The number of times any game has been played (inclusive of current). This is the total across all game sessions (not this game session).
            const gameProgress = gc.state.getGameProgress(); // [number of playsets built]_17
            const data = {
                [TSMAnalyticsManager.GAME_ALL_PLAYS]: gameAllPlays,
                [TSMAnalyticsManager.GAME_PROGRESS]: gameProgress + "_17"
            };
            return data;
        }
        // returns data pre-filled with `game.name`, `game.score.total` and `game.Plays`
        // also contains game.AllPlays and gameProgress from getCommons
        getGameAndCommons(state) {
            let data = this.getCommons();
            const game = TSMAnalyticsManager.GAME_NAMES_FOR_STATES.get(state);
            if (!game) {
                return null;
            } // erroneous state has been passed
            data[TSMAnalyticsManager.GAME_NAME] = game;
            // The number of times that this game has been played (inclusive).
            // is inclusive so i have to add +1
            data[TSMAnalyticsManager.GAME_PLAYS] = GameController_4.GameController.Instance.state.getTrackedMinigamePlay(state);
            const totalScore = GameController_4.GameController.Instance.getQuotaProgressForGame(state);
            data[TSMAnalyticsManager.GAME_SCORE_TOTAL] = Math.round(totalScore);
            return data;
        }
        ///////
        /// Hub
        // When ResumeDialog is being shown
        onGameResumeState(newGroup) {
            const aState = newGroup ? 'content/game/inqueue/resume/newgroup' : 'content/game/inqueue/resume';
            const gState = newGroup ? 'Resume' : 'ResumeNewGroup';
            this.trackState(aState, gState, this.getCommons());
        }
        // When one of the options in resumeDialog gets chosen
        onGameResumed(resume, newGroup) {
            const adobeAction = resume ? "Continue" : "Exit";
            const gaAction = resume ? "ContinueGame" : "NewGame";
            const gaLabel = newGroup ? "NewGroup" : "SameGroup";
            this.trackAction(adobeAction, "Resume", gaAction, gaLabel, this.getCommons());
        }
        // When the user confirms or cancel the exit prompt
        onGameResumeChoice(confirmed, newGroup) {
            const action = confirmed ? "Exit_Confirm" : "Exit_Cancel";
            const gaLabel = newGroup ? "NewGroup" : "SameGroup";
            this.trackAction(action, "Resume", action, gaLabel, this.getCommons());
        }
        onGameExit(exitStage) {
            const data = this.getCommons();
            const aAction = ["Exit", "Exit_Cancel", "Exit_Confirm"][exitStage];
            const gAction = ["Exit", "ExitCancel", "ExitConfirm"][exitStage];
            TSMAnalyticsManager.Instance.trackAction(aAction, "Hub", gAction, "", data);
        }
        onOutroPrompt() {
            let data = this.getCommons();
            this.trackState("content/game/inqueue/declare", "OutroAreYouHere", data);
        }
        onQueueEndConfirmation(confirmed) {
            let data = this.getCommons();
            let aAction, gaAction;
            if (confirmed) {
                aAction = "Declare_Yes";
                gaAction = "Yes";
            }
            else {
                aAction = "Declare_No";
                gaAction = "No";
            }
            this.trackAction(aAction, "Location", gaAction, "Outro", data);
        }
        // Each time the hub gets shown
        onGameHub() {
            this.trackState("content/game/inqueue/hub", "Hub", this.getCommons());
        }
        onGameStart(name) {
            const games = [StateNames_2.StateNames.GAME_WOODY_LASSO, StateNames_2.StateNames.GAME_FLYING_TOSSERS, StateNames_2.StateNames.GAME_TRIXIE, StateNames_2.StateNames.GAME_SARGE, StateNames_2.StateNames.GAME_HAMM];
            const state = games.indexOf(name) < 0 ? "PlayMiniGame" : "StartGame";
            const data = this.getGameAndCommons(name);
            this.trackState(state, state, data);
        }
        onIntroCutscene(page, totalPages) {
            const indexText = page.toString() + "_" + totalPages.toString();
            const adobeState = 'content/game/inqueue/cutscene/' + indexText;
            const gaState = "IntroCutscene" + indexText;
            let data = this.getCommons();
            data[TSMAnalyticsManager.CUTSCENE] = "Intro";
            this.trackState(adobeState, gaState, data);
        }
        // Tracks the time for the whole experience
        onFullGameTime(start) {
            return __awaiter(this, void 0, void 0, function* () {
                const data = this.getCommons();
                if (start) {
                    this.startTiming(TSMAnalyticsManager.T_PARTY_PLAYSET, data);
                }
                else {
                    yield this.endTiming(TSMAnalyticsManager.T_PARTY_PLAYSET, TSMAnalyticsManager.T_PARTY_PLAYSET, '', '', data);
                    this.trackAction("Exit", "Outro", "Exit", "", data);
                }
            });
        }
        /// ~ Hub
        /// Titles
        // Opens the title screen for a minigame
        onGameTitle(state) {
            const data = this.getGameAndCommons(state);
            if (!data) {
                return;
            }
            const gaState = data[TSMAnalyticsManager.GAME_NAME] + "Title";
            this.trackState("content/game/inqueue/minigame/title", gaState, data);
        }
        // Hits the back button on a title screen for a minigame
        onGameBack(state) {
            const data = this.getGameAndCommons(state);
            if (!data) {
                return;
            }
            const game = data[TSMAnalyticsManager.GAME_NAME];
            this.trackAction("Back", "Title", "Back", game, data);
        }
        // action == GAMETITLE_PLAY : Hits the play button on a title screen for a minigame
        // action == GAMETITLE_CONTINUE : Hits the continue button on the result screen for a minigame
        // action == GAMETITLE_REPLAY : Hits the replay button on the result screen for a minigame
        onGameTitleAction(name, action = TSMAnalyticsManager.GAMETITLE_PLAY, score = null, totalScore = null) {
            // TODO evaluate the instances where totalScore is being passed, if its gotten (or can be gotten) from minigameInfo.totalScore,
            // use that instead and finally remove this param. getGameAndCommons already takes that value
            const data = TSMAnalyticsManager.Instance.getGameAndCommons(name);
            const game = data[TSMAnalyticsManager.GAME_NAME];
            const adobeAction = ["PlayMiniGame", "ContinueGame", "PlayAgain"][action];
            const gaAction = ["Play", "Continue", "Replay"][action];
            if (score !== null) {
                data[TSMAnalyticsManager.GAME_SCORE] = score.toString();
            }
            if (totalScore !== null) {
                data[TSMAnalyticsManager.GAME_SCORE_TOTAL] = Math.round(totalScore);
            }
            this.trackAction(adobeAction, "Title", gaAction, game, data);
        }
        // tracks the time for a minigame
        // Doesn't include cutscene, pass & play & how to play. Does include in-game tutorial
        onGameTime(name, score = null, finish = false) {
            const data = this.getGameAndCommons(name);
            if (!data) {
                return;
            }
            if (score) {
                data[TSMAnalyticsManager.GAME_SCORE] = score;
            }
            const game = data[TSMAnalyticsManager.GAME_NAME];
            const aKey = "PartyPlayset:Mini:" + game;
            if (finish) {
                this.endTiming(aKey, game, "Game", "", data);
            }
            else {
                this.startTiming(aKey, data);
            }
        }
        /// ~Titles
        /// Minigame
        // Timing a micro game
        onMicroTime(name, micro, start = true, score = null) {
            const data = this.getGameAndCommons(name);
            if (!data) {
                return;
            }
            data[TSMAnalyticsManager.GAME_NAME_MICRO] = micro;
            if (score) {
                data[TSMAnalyticsManager.GAME_SCORE] = score;
            }
            const game = data[TSMAnalyticsManager.GAME_NAME] + micro;
            const aKey = "PartyPlayset:Micro:" + game;
            if (start) {
                this.startTiming(aKey, data);
            }
            else {
                this.endTiming(aKey, game, "Game", "", data);
            }
        }
        // Hits the back button on a title screen for a minigame
        onGameButtonClick(state, buttonName) {
            const data = this.getGameAndCommons(state);
            if (!data) {
                return;
            }
            const game = data[TSMAnalyticsManager.GAME_NAME];
            this.trackAction(buttonName, "Game", buttonName, game, data);
        }
        onMiniGameCutscene(sceneName, page, totalPages, score = null) {
            const sceneData = TSMAnalyticsManager.CUTSCENE_DATA.get(sceneName);
            if (!sceneData) {
                return;
            }
            const stateName = sceneData[0];
            const robbing = sceneData[1];
            const micro = sceneData[2];
            let data = this.getGameAndCommons(stateName);
            if (!data) {
                return;
            }
            if (score !== null) {
                data[TSMAnalyticsManager.GAME_SCORE] = score;
            }
            // create the proper names
            const index = page + "_" + totalPages;
            const adobeState = "content/game/inqueue/minigame/cutscene/" + index;
            let cutscene = data[TSMAnalyticsManager.GAME_NAME];
            if (robbing) {
                cutscene = "PeteRobs" + cutscene;
            }
            if (micro) {
                cutscene += micro;
                // If it's a poster, "Complete" and "Intro" shouldn't be posted in the game.micro variable
                if (stateName !== StateNames_2.StateNames.POSTER) {
                    data[TSMAnalyticsManager.GAME_NAME_MICRO] = micro;
                }
            }
            data[TSMAnalyticsManager.CUTSCENE] = cutscene;
            const pageName = cutscene + "Cutscene" + index;
            this.trackState(adobeState, pageName, data);
        }
        // Cutscene for catching pete
        onPeteCaughtCutscene(page, totalPages) {
            let data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            if (!data) {
                return;
            }
            const cutscene = "PeteCaught";
            data[TSMAnalyticsManager.CUTSCENE] = cutscene;
            data[TSMAnalyticsManager.INSTALLATION_NAME] = "Poster";
            const index = page + "_" + totalPages;
            const pageName = cutscene + "Cutscene" + index;
            const aState = "content/game/inqueue/installation/cutscene/" + index;
            this.trackState(aState, pageName, data);
        }
        // Outro cutscene
        onOutroCutscene(page, totalPages) {
            let data = this.getCommons();
            const cutscene = "Outro";
            data[TSMAnalyticsManager.CUTSCENE] = cutscene;
            const index = page + "_" + totalPages;
            const pageName = cutscene + "Cutscene" + index;
            const aState = "content/game/inqueue/outro/cutscene/" + index;
            this.trackState(aState, pageName, data);
        }
        // The messages on the loadInstructions page
        onLoadInstructionCutscene(page, totalPages) {
            let data = this.getCommons();
            const index = page + "_" + totalPages;
            const pageName = "LoadInstructions" + index;
            const aState = "content/game/inqueue/outro/loadinstructions/" + index;
            this.trackState(aState, pageName, data);
        }
        // When the installation poster gets triggered
        onInstallationStarts() {
            const data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            this.trackAction("PlayMicroGame", "Installation", "Play", "Poster", data); // TODO verify commons
        }
        // When a micro-game starts
        onMicroGameStart(name, micro) {
            let data = this.getGameAndCommons(name);
            if (!data) {
                return;
            }
            const state = "PlayMicroGame";
            const gaLabel = data[TSMAnalyticsManager.GAME_NAME] + micro;
            data[TSMAnalyticsManager.GAME_NAME_MICRO] = micro;
            this.trackAction(state, "Game", "Play", gaLabel, data);
        }
        // When the game "starts"
        onGameState(state, totalScore = null, micro = null) {
            let data = this.getGameAndCommons(state);
            if (!data) {
                return;
            }
            if (totalScore !== null) {
                data[TSMAnalyticsManager.GAME_SCORE_TOTAL] = Math.round(totalScore);
            }
            let adobeState = "content/game/inqueue/minigame";
            let gaState = data[TSMAnalyticsManager.GAME_NAME] + "Game";
            if (micro !== null) {
                data[TSMAnalyticsManager.GAME_NAME_MICRO] = micro;
                adobeState = "content/game/inqueue/minigame/microgame";
                gaState = data[TSMAnalyticsManager.GAME_NAME] + micro + "Game";
            }
            this.trackState(adobeState, gaState, data);
        }
        onGameInstructions(state, micro = null) {
            let data = this.getGameAndCommons(state);
            if (!data) {
                return;
            }
            let adobeState = "content/game/inqueue/minigame/instructions";
            let gaState = data[TSMAnalyticsManager.GAME_NAME] + "Instructions";
            if (micro !== null) {
                data[TSMAnalyticsManager.GAME_NAME_MICRO] = micro;
                adobeState = "content/game/inqueue/minigame/microgame/instructions";
                gaState = data[TSMAnalyticsManager.GAME_NAME] + micro + "Instructions";
            }
            this.trackState(adobeState, gaState, data);
        }
        onGamePassAndPlay(state, micro = null) {
            let data = this.getGameAndCommons(state);
            if (!data) {
                return;
            }
            let adobeState = "content/game/inqueue/minigame/passandplay";
            let gaState = data[TSMAnalyticsManager.GAME_NAME] + "PassAndPlay";
            if (micro !== null) {
                data[TSMAnalyticsManager.GAME_NAME_MICRO] = micro;
                adobeState = "content/game/inqueue/minigame/microgame/passandplay";
                gaState = data[TSMAnalyticsManager.GAME_NAME] + micro + "PassAndPlay";
            }
            this.trackState(adobeState, gaState, data);
        }
        // when a game shows the resultScreen
        onGameResult(state, score) {
            let data = this.getGameAndCommons(state);
            if (!data) {
                return;
            }
            const adobeState = "content/game/inqueue/minigame/result";
            const gaState = data[TSMAnalyticsManager.GAME_NAME] + "Result";
            data[TSMAnalyticsManager.GAME_SCORE] = score;
            this.trackState(adobeState, gaState, data);
        }
        // when a game / micro-game shows a "score" screen (inside the game, previous to the result screen)
        onGameScore(state, score, micro = null) {
            let data = this.getGameAndCommons(state);
            if (!data) {
                return;
            }
            data[TSMAnalyticsManager.GAME_SCORE] = score;
            let adobeState = "content/game/inqueue/minigame/score";
            let gaState = data[TSMAnalyticsManager.GAME_NAME] + "Score";
            if (micro !== null) {
                data[TSMAnalyticsManager.GAME_NAME_MICRO] = micro;
                adobeState = "content/game/inqueue/minigame/microgame/score";
                gaState = data[TSMAnalyticsManager.GAME_NAME] + micro + "Score";
            }
            this.trackState(adobeState, gaState, data);
        }
        /// ~Minigame
        /// Posters
        onPosterTitleState(micro, index, total) {
            // similar to onGameTitle but the index and micro differs, if that changes, those could be merged
            const aState = "content/game/inqueue/minigame/title/" + index + "_" + total;
            const data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            data[TSMAnalyticsManager.GAME_NAME_MICRO] = micro;
            this.trackState(aState, "PosterTitle", data);
        }
        // Happens it's presented the wayfind dialog for the installation poster
        onInstallationWayfindState() {
            const adobeState = 'content/game/inqueue/declare';
            const gaState = 'InstallationPosterAreYouHere';
            let data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            data[TSMAnalyticsManager.INSTALLATION_NAME] = "Poster";
            this.trackState(adobeState, gaState, data);
        }
        // Happens when the user taps one of the buttons on the wayfind dialog for the installation poster
        onInstallationWayfind(confirmed) {
            let data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            data[TSMAnalyticsManager.INSTALLATION_NAME] = "Poster";
            if (confirmed) {
                this.trackAction("Declare_Yes", "Location", "Yes", "InstallationPoster", data);
            }
            else {
                this.trackAction("Declare_No", "Location", "No", "InstallationPoster", data);
            }
        }
        // Happens it's presented the wayfind dialog for the poster room
        onPosterRoomWayfindState() {
            const adobeState = 'content/game/inqueue/declare';
            const gaState = 'PosterAreYouHere';
            let data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            this.trackState(adobeState, gaState, data);
        }
        // Happens when the user taps one of the buttons on the wayfind dialog for the poster room
        onPosterRoomWayfind(confirmed) {
            let data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            if (confirmed) {
                this.trackAction("Declare_Yes", "Location", "Confirm", "Poster", data);
            }
            else {
                this.trackAction("Declare_No", "Location", "Cancel", "Poster", data);
            }
        }
        onPosterTrigger(effectResult) {
            const adobeState = 'content/game/inqueue/installation/trigger';
            const gaState = 'InstallationPosterTrigger';
            let data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            data[TSMAnalyticsManager.INSTALLATION_NAME] = "Poster";
            this.trackState(adobeState, gaState, data);
            this.trackAction("Installation_Trigger", "Installation", "Trigger", "Poster", data);
            data[TSMAnalyticsManager.INSTALLATION_TRIGGER] = effectResult; // 0 = B-show, 1 = A-show, 2 = in-use
            this.trackAction("Installation_Result", "Installation", "Result", "Poster", data);
        }
        onPosterWatch() {
            let data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            data[TSMAnalyticsManager.INSTALLATION_NAME] = "Poster";
            this.trackState("content/game/inqueue/installation/watch", "InstallationPosterWatch", data);
        }
        onPosterInUse() {
            let data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            data[TSMAnalyticsManager.INSTALLATION_NAME] = "Poster";
            this.trackState("content/game/inqueue/installation/inuse", "InstallationPosterInUse", data);
        }
        onPosterAnimation() {
            let data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            data[TSMAnalyticsManager.INSTALLATION_NAME] = "Poster";
            this.trackState("content/game/inqueue/installation/animation", "InstallationPosterAnimation", data);
        }
        onPosterInstallationTime(start, trigger) {
            const data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            if (!data) {
                return;
            }
            data[TSMAnalyticsManager.INSTALLATION_NAME] = "Poster";
            let aKey, gaTimingKey, gaCategory, gaLabel;
            if (trigger) {
                aKey = "PartyPlayset:Installation:PosterTrigger";
                gaTimingKey = "Trigger";
                gaCategory = "Installation";
                gaLabel = "Poster";
            }
            else {
                aKey = "PartyPlayset:Installation:Poster";
                gaTimingKey = "Poster";
                gaCategory = "Installation";
                gaLabel = null;
            }
            if (start) {
                this.startTiming(aKey, data);
            }
            else {
                this.endTiming(aKey, gaTimingKey, gaCategory, gaLabel, data);
                this.trackAction("Installation_End", "Installation", "End", "Poster", data);
            }
        }
        onPosterHubExit() {
            const data = this.getGameAndCommons(StateNames_2.StateNames.POSTER);
            data[TSMAnalyticsManager.GAME_NAME] = "Poster";
            this.trackAction("GameResult_Home", "PosterHub", "Back", "", data);
        }
    }
    TSMAnalyticsManager.GAME_NAME = "game.name.mini";
    TSMAnalyticsManager.GAME_NAME_MICRO = "game.name.micro";
    TSMAnalyticsManager.GAME_SCORE = "game.score";
    TSMAnalyticsManager.GAME_SCORE_TOTAL = "game.score.total";
    TSMAnalyticsManager.GAME_PLAYS = "game.plays";
    TSMAnalyticsManager.GAME_ALL_PLAYS = "game.allplays";
    TSMAnalyticsManager.GAME_PROGRESS = "game.progress";
    TSMAnalyticsManager.CUTSCENE = "cutscene";
    TSMAnalyticsManager.INSTALLATION_TRIGGER = "installation.trigger";
    TSMAnalyticsManager.INSTALLATION_NAME = "installation.name";
    TSMAnalyticsManager.T_PARTY_PLAYSET = "PartyPlayset";
    TSMAnalyticsManager.GAME_NAMES_FOR_STATES = new Map([
        [StateNames_2.StateNames.GAME_WOODY_LASSO, "Woody"],
        [StateNames_2.StateNames.GAME_WOODY_BALANCE, "Woody"],
        [StateNames_2.StateNames.GAME_WOODY_THROW, "Woody"],
        [StateNames_2.StateNames.GAME_FLYING_TOSSERS, "Buzz"],
        [StateNames_2.StateNames.GAME_TRIXIE, "Trixie"],
        [StateNames_2.StateNames.GAME_SARGE, "Sarge"],
        [StateNames_2.StateNames.GAME_HAMM, "Hamm"],
        [StateNames_2.StateNames.GAME_DRAWING, "Jessie"],
        [StateNames_2.StateNames.GAME_DRESS_UP, "Rex"],
        [StateNames_2.StateNames.POSTER, "Poster"]
        // TODO add Jail when it exists [StateNames., "Jail"]
    ]);
    // When the user is prompted to exit, tracks the button actions
    // corresponds to Lines 26-28 of spreadsheet
    TSMAnalyticsManager.EXITING_SHOW = 0;
    TSMAnalyticsManager.EXITING_CANCEL = 1;
    TSMAnalyticsManager.EXITING_CONFIRM = 2;
    TSMAnalyticsManager.GAMETITLE_PLAY = 0;
    TSMAnalyticsManager.GAMETITLE_CONTINUE = 1;
    TSMAnalyticsManager.GAMETITLE_REPLAY = 2;
    // On a cutscene for a minigame or poster
    TSMAnalyticsManager.CUTSCENE_DATA = new Map([
        // Woody
        ['game_woody_intro', [StateNames_2.StateNames.GAME_WOODY_LASSO, false, "Lasso"]],
        ['game_woody_balance', [StateNames_2.StateNames.GAME_WOODY_BALANCE, false, "Balance"]],
        ['game_woody_toss', [StateNames_2.StateNames.GAME_WOODY_THROW, false, "Toss"]],
        ['pete_steal_woody', [StateNames_2.StateNames.GAME_WOODY_LASSO, true]],
        // Hamm
        ['game_hamm_intro', [StateNames_2.StateNames.GAME_HAMM, false]],
        ['pete_steal_hamm', [StateNames_2.StateNames.GAME_HAMM, true]],
        // Trixie
        ['game_rex_dino_darts_intro', [StateNames_2.StateNames.GAME_TRIXIE, false]],
        ['pete_steal_trixie', [StateNames_2.StateNames.GAME_TRIXIE, true]],
        // Sarge
        ['game_sarge_intro', [StateNames_2.StateNames.GAME_SARGE, false]],
        ['pete_steal_sarge', [StateNames_2.StateNames.GAME_SARGE, true]],
        // Buzz
        ['game_buzz_intro', [StateNames_2.StateNames.GAME_FLYING_TOSSERS, false]],
        ['pete_steal_buzz', [StateNames_2.StateNames.GAME_FLYING_TOSSERS, true]],
        // Jessie
        ['game_jessie_intro', [StateNames_2.StateNames.GAME_DRAWING, false]],
        // Rex
        ['game_rex_intro_woody', [StateNames_2.StateNames.GAME_DRESS_UP, false]],
        ['game_rex_intro_trixie', [StateNames_2.StateNames.GAME_DRESS_UP, false]],
        ['game_rex_intro_hamm', [StateNames_2.StateNames.GAME_DRESS_UP, false]],
        ['game_rex_intro_sarge', [StateNames_2.StateNames.GAME_DRESS_UP, false]],
        ['game_rex_intro_buzz', [StateNames_2.StateNames.GAME_DRESS_UP, false]],
        // Poster
        ['posters_intro', [StateNames_2.StateNames.POSTER, false, "Intro"]],
        ['posters_complete', [StateNames_2.StateNames.POSTER, false, "Complete"]]
    ]);
    exports.TSMAnalyticsManager = TSMAnalyticsManager;
});
define("switchback-common/save/SaveState", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SaveState {
        constructor() {
            this._isSaveEnabled = true;
        }
        save() {
            if (this._isSaveEnabled) {
                this.forceSave();
            }
        }
        clear() {
            console.log("[SaveState] clear");
            this.parse([]);
            this.forceSave();
        }
        /**
         * This disables all saves from this point. Clear will still function as expected.
         */
        disableSave() {
            this._isSaveEnabled = false;
        }
        forceSave() {
            PlayAPI.gameStateSave(GameStatus.GAME_STATUS_IN_PROGRESS, this).then(SaveState.onSaveSucceed, SaveState.onSaveFailed).catch(SaveState.onSaveExcept);
        }
        static onSaveSucceed() {
            console.log("[SaveState] save() success");
        }
        static onSaveFailed(err) {
            console.error("[SaveState] save() failed " + JSON.stringify(err));
        }
        static onSaveExcept(err) {
            console.error("[SaveState] save() inner exception" + JSON.stringify(err));
        }
    }
    exports.SaveState = SaveState;
});
define("logic/TSMSaveState", ["require", "exports", "StateNames", "logic/GameController", "switchback-common/save/SaveState"], function (require, exports, StateNames_3, GameController_5, SaveState_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MinigameSaveData {
        constructor(gameName) {
            this.gameName = gameName;
            this.highscore = 0;
            this.totalScore = 0;
            this.level = 0;
            this.visualLevel = 0;
            this.numPlays = 0;
            this.trackedPlays = 0;
            this.scorePendingResultsScreen = 0;
        }
    }
    exports.MinigameSaveData = MinigameSaveData;
    class TSMSaveState extends SaveState_1.SaveState {
        constructor() {
            super(...arguments);
            this._lastPosterShown = 0;
            // used for tracking the time a mini-game gets played
            // needs to be tracked separately because it needs to be persisted across sessions
            this._allPlays = 0;
        }
        get lastPosterShown() {
            return this._lastPosterShown;
        }
        set lastPosterShown(value) {
            this._lastPosterShown = value;
        }
        get sessionStarted() {
            return this._sessionStarted;
        }
        set sessionStarted(started) {
            this._sessionStarted = started;
            this.save();
        }
        get hasEnteredPosterRoom() {
            return this._hasEnteredPosterRoom;
        }
        set hasEnteredPosterRoom(val) {
            this._hasEnteredPosterRoom = val;
            this.save();
        }
        get hasCompletedPosterInstallation() {
            return this._hasCompletedPosterInstallation;
        }
        set hasCompletedPosterInstallation(val) {
            this._hasCompletedPosterInstallation = val;
            this.save();
        }
        get hasCompletedJailInstallation() {
            return this._hasCompletedJailInstallation;
        }
        set hasCompletedJailInstallation(val) {
            this._hasCompletedJailInstallation = val;
            this.save();
        }
        get playerNames() {
            return this._playerNames;
        }
        // No save here because it gave timing problems
        // also it gave the fake sense of security, because we need to reset and change it later,
        // so anyway, we needed to call .save after pushing all the elements
        set playerNames(playerNames) {
            this._playerNames = playerNames;
        }
        ///////////////////////////////////
        // Game Info
        ///////////////////////////////////
        // Minigame game info - if we can't find any info about the
        // queried game, add a new one and return a fresh game info
        getMinigameSaveData(gameName) {
            let gameInfo = this._minigameSaveData.find(obj => obj.gameName === gameName);
            if (!gameInfo) {
                gameInfo = new MinigameSaveData(gameName);
                this._minigameSaveData.push(gameInfo);
            }
            return gameInfo;
        }
        setMinigameScore(gameName, score) {
            let minigameSaveData = this.getMinigameSaveData(gameName);
            // Update game info with progress
            minigameSaveData.numPlays++;
            minigameSaveData.scorePendingResultsScreen += score;
            minigameSaveData.highscore = Math.max(minigameSaveData.highscore, score);
            this.save();
            // Check UNBOX_ALL achievement
            if (minigameSaveData.numPlays === 1) {
                GameController_5.GameController.Instance.achievementManager.onGameUnlocked();
            }
        }
        stealMinigameScore(gameName, score) {
            this.getMinigameSaveData(gameName).scorePendingResultsScreen -= score;
            this.save();
        }
        ///////////////////////////////////
        // Actual levels
        ///////////////////////////////////
        incrementGameTargetLevel(gameName) {
            this.setGameTargetLevel(gameName, this.getGameTargetLevel(gameName) + 1);
        }
        setGameTargetLevel(gameName, level) {
            this.getMinigameSaveData(gameName).level = level;
            this.save();
        }
        getGameTargetLevel(gameName) {
            const gameInfo = this.getMinigameSaveData(gameName);
            if (gameInfo) {
                return gameInfo.level;
            }
            return 0;
        }
        ///////////////////////////////////
        // Current view levels
        ///////////////////////////////////
        getGameVisualLevel(gameName) {
            const gameInfo = this.getMinigameSaveData(gameName);
            if (gameInfo) {
                return gameInfo.visualLevel;
            }
            return 0;
        }
        incrementGameVisualLevel(gameName) {
            this.setGameVisualLevel(gameName, this.getGameVisualLevel(gameName) + 1);
            GameController_5.GameController.Instance.achievementManager.onVisualLevelUpgrade();
        }
        setGameVisualLevel(gameName, visualLevel) {
            this.getMinigameSaveData(gameName).visualLevel = visualLevel;
            this.save();
        }
        ///////////////////////////////////
        // FTUE
        ///////////////////////////////////
        get ftueCompleted() {
            return this._ftueCompleted;
        }
        set ftueCompleted(ftueCompleted) {
            this._ftueCompleted = ftueCompleted;
            this.save();
        }
        ///////////////////////////////////
        // POSTERS
        ///////////////////////////////////
        getHasCompletedPoster(posterId) {
            return this._completedPosterIds.indexOf(posterId) !== -1;
        }
        addCompletedPoster(posterId) {
            if (!this.getHasCompletedPoster(posterId)) {
                this._completedPosterIds.push(posterId);
            }
            this.save();
        }
        ///////////////////////////////////
        // Exit Triggered
        ///////////////////////////////////
        get exitTriggered() {
            return this._exitTriggered;
        }
        set exitTriggered(exitTriggered) {
            this._exitTriggered = exitTriggered;
            this.save();
        }
        ///////////////////////////////////
        // FORCED PROGRESSION
        ///////////////////////////////////
        get forcedProgressionRound() {
            return this._forcedProgressionRound;
        }
        set forcedProgressionRound(round) {
            this._forcedProgressionRound = round;
            this.save();
        }
        get forcedProgressionStep() {
            return this._forcedProgressionStep;
        }
        set forcedProgressionStep(step) {
            this._forcedProgressionStep = step;
            this.save();
        }
        ///////////////////////////////////
        // Player ordering
        ///////////////////////////////////
        get playersThatHaveBeenFirst() {
            return this._playersThatHaveBeenFirst;
        }
        setPlayersThatHaveBeenFirst(playersThatHaveBeenFirst) {
            this._playersThatHaveBeenFirst = playersThatHaveBeenFirst;
            this.save();
        }
        ///////////////////////////////////
        // Total number of minigames played
        ///////////////////////////////////
        get totalNumPlays() {
            return this._minigameSaveData.reduce((a, b) => (a + b.numPlays), 0);
        }
        ///////////////////////////////////
        // Tracking
        ///////////////////////////////////
        // returns the sum of the levels of all the minigames
        getGameProgress() {
            let sum = 0;
            for (let gameInfo of this._minigameSaveData) {
                // we use this variable instead of .level so the metric only updates when the user sees the upgrade
                sum += gameInfo.visualLevel;
            }
            return sum;
        }
        getTrackedMinigamePlay(gameName) {
            // tracked plays for woody should be treated as one
            if (gameName === StateNames_3.StateNames.GAME_WOODY_THROW || gameName === StateNames_3.StateNames.GAME_WOODY_BALANCE) {
                gameName = StateNames_3.StateNames.GAME_WOODY_LASSO;
            }
            return this.getMinigameSaveData(gameName).trackedPlays;
        }
        trackMinigamePlay(gameName) {
            // tracked plays for woody should be treated as one
            if (gameName === StateNames_3.StateNames.GAME_WOODY_THROW || gameName === StateNames_3.StateNames.GAME_WOODY_BALANCE) {
                gameName = StateNames_3.StateNames.GAME_WOODY_LASSO;
            }
            let minigameSaveData = this.getMinigameSaveData(gameName);
            minigameSaveData.trackedPlays++;
            this._allPlays++;
        }
        get allPlays() {
            return this._allPlays;
        }
        set allPlays(value) {
            this._allPlays = value;
        }
        parse(data) {
            // Loading data out of the App
            if (!data) {
                data = [];
            }
            this._playersThatHaveBeenFirst = data._playersThatHaveBeenFirst || [];
            this._ftueCompleted = data._ftueCompleted || false;
            this._exitTriggered = data._exitTriggered || false;
            this._playerNames = data._playerNames || [];
            this._sessionStarted = data._sessionStarted || false;
            this._forcedProgressionRound = data._forcedProgressionRound || 0;
            this._forcedProgressionStep = data._forcedProgressionStep || 0;
            this._minigameSaveData = data._minigameSaveData || [];
            this._completedPosterIds = data._completedPosterIds || [];
            this._hasEnteredPosterRoom = data._hasEnteredPosterRoom || false;
            this._hasCompletedJailInstallation = data._hasCompletedJailInstallation || false;
            this._hasCompletedPosterInstallation = data._hasCompletedPosterInstallation || false;
            this._lastPosterShown = data._lastPosterShown || 0;
            this._allPlays = data._allPlays || 0;
            return this;
        }
    }
    exports.TSMSaveState = TSMSaveState;
});
define("switchback-common/util/MotionUtil", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MotionUtil {
        static checkGyroAvailability() {
            return __awaiter(this, void 0, void 0, function* () {
                MotionUtil.isAccelerometerAvailable = false;
                MotionUtil.isGyroAvailable = false;
                const motionCallback = (result) => {
                    const acceleration = result.userAcceleration;
                    if (acceleration.x !== 0 || acceleration.y !== 0 || acceleration.z !== 0) {
                        MotionUtil.isAccelerometerAvailable = true;
                    }
                    const roll = result.attitude.roll;
                    if (roll !== 0) {
                        MotionUtil.isGyroAvailable = true;
                    }
                };
                PlayAPI.subscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, motionCallback);
                yield PromiseUtil_4.PromiseUtil.realTime(1000);
                PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, motionCallback);
            });
        }
    }
    MotionUtil.isAccelerometerAvailable = false;
    MotionUtil.isGyroAvailable = false;
    exports.MotionUtil = MotionUtil;
});
define("logic/GameController", ["require", "exports", "switchback-common/play/api/MockPlayAPI", "switchback-common/enum/Park", "StateNames", "logic/PlayerManager", "model/GameInfo", "model/ToyStoryConfig", "logic/TSMAchievementManager", "logic/TSMAnalyticsManager", "switchback-common/util/TextUtil", "Game", "logic/TSMSaveState", "switchback-common/play/ShowEffectController", "switchback-common/util/MotionUtil"], function (require, exports, MockPlayAPI_1, Park_3, StateNames_4, PlayerManager_1, GameInfo_1, ToyStoryConfig_1, TSMAchievementManager_1, TSMAnalyticsManager_1, TextUtil_3, Game_3, TSMSaveState_1, ShowEffectController_1, MotionUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class TSMPlayer {
    }
    exports.TSMPlayer = TSMPlayer;
    class GameController {
        constructor() {
            this.players = [];
            this.gameInfo = new Map();
            this.exitReminderCount = 0;
            this.numPlaysOfLastExitReminder = 0;
            /**
             * Queue the title dialog of the global game and
             * open the title dialog after reaching the hub
             */
            this.queueTitleDialog = null;
            // PlayAPI.GameEvent signal responders
            this.onGameShouldExitTrigger = new Phaser.Signal();
            this.onLocationChanged = new Phaser.Signal();
            this.showEffectController = null;
            if (ToyStoryConfig_1.ToyStoryConfig.Instance.debug) {
                MockPlayAPI_1.MockPlayAPI.init();
            }
            this.playerManager = new PlayerManager_1.PlayerManager();
            this.achievementManager = new TSMAchievementManager_1.TSMAchievementManager();
            this.showEffectController = new ShowEffectController_1.ShowEffectController();
            // don't show the installation reminder when the game opens
            const now = new Date().getTime();
            this.timeOfLastExitReminder = this.timeOfLastInstallationReminder = now;
            this._onExitTriggeredSignal = new Phaser.Signal();
            MotionUtil_1.MotionUtil.checkGyroAvailability();
        }
        get PlayerInformation() {
            return this.playerInfo;
        }
        get GameState() {
            return this.gameState;
        }
        get GameStats() {
            return this.gameStats;
        }
        get GameDefinition() {
            return this.gameDefinition;
        }
        PartySize() {
            return this.partySize;
        }
        static get Instance() {
            return this.s_instance || (this.s_instance = new this());
        }
        get onExitTriggeredSignal() {
            return this._onExitTriggeredSignal;
        }
        openPlayerManagement() {
            return __awaiter(this, void 0, void 0, function* () {
                this.players = [];
                try {
                    let result = yield PlayAPI.openPlayerManagement();
                    // if result contains no players, populate the players array with dummy players
                    if (result && result.players && result.players.length > 0) {
                        this.players = result.players.map(player => {
                            return Object.assign({}, player, { instructionsViewed: new Set() });
                        });
                    }
                    else {
                        this.players = [
                            { name: "Player 1", type: "", identifier: "", instructionsViewed: new Set() },
                            { name: "Player 2", type: "", identifier: "", instructionsViewed: new Set() }
                        ];
                    }
                    // this relies on the savestate being loaded before calling this function
                    // and also relies on this function being always called upon init
                    // stores the previous player names for comparison
                    if (this.state.sessionStarted) {
                        this.previousSessionPlayerNames = this.state.playerNames.slice();
                    }
                    // saves the current players
                    this.state.playerNames = [];
                    for (let player of this.players) {
                        this.state.playerNames.push(player.name);
                    }
                    this.state.save();
                }
                catch (error) {
                    console.log("OpenPlayerManagement Error " + error);
                    // Initialize with fake data just so the app doesn't break
                    GameController.Instance.players = [
                        { name: "Player 1", type: "", identifier: "", instructionsViewed: new Set() },
                        { name: "Player 2", type: "", identifier: "", instructionsViewed: new Set() }
                    ];
                }
                // re-setup players every time the player management screen is opened
                this.playerManager.setupPlayers(this.players);
            });
        }
        init(gameInfoJSON) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[GameController] init()`);
                gameInfoJSON.games.forEach((gameJSON) => {
                    const gameInfo = new GameInfo_1.GameInfo().parse(gameJSON);
                    this.gameInfo.set(gameInfo.stateName, gameInfo);
                });
                this.gameOrder = gameInfoJSON.unlockOrder;
                console.log(this.gameInfo);
                this.state = new TSMSaveState_1.TSMSaveState();
                try {
                    const initResult = yield PlayAPI.gameInit();
                    // if there is a save state, load it			
                    if (initResult.gameStateRead && initResult.gameStateRead.state) {
                        this.state.parse(initResult.gameStateRead.state);
                    }
                    else {
                        // if there isn't a save state, create one with empty data
                        this.state.parse({});
                    }
                    // Figure out which park this app got launched from!
                    if (initResult.playerLocation === Park_3.Park.DLR) {
                        ToyStoryConfig_1.ToyStoryConfig.Instance.park = Park_3.Park.DLR;
                    }
                    if (initResult.playerLocation === Park_3.Park.WDW) {
                        ToyStoryConfig_1.ToyStoryConfig.Instance.park = Park_3.Park.WDW;
                    }
                }
                catch (error) {
                    console.log(error);
                }
                if (ToyStoryConfig_1.ToyStoryConfig.Instance.unlockAllHubNodes) {
                    this.unlockAllGames();
                }
                let showEffectName;
                if (ToyStoryConfig_1.ToyStoryConfig.Instance.park === Park_3.Park.DLR) {
                    this.parkPrefix = 'dlr';
                    showEffectName = ToyStoryConfig_1.ToyStoryConfig.Instance.showEffectNameDLR;
                }
                else if (ToyStoryConfig_1.ToyStoryConfig.Instance.park === Park_3.Park.WDW) {
                    this.parkPrefix = 'wdw';
                    showEffectName = ToyStoryConfig_1.ToyStoryConfig.Instance.showEffectNameWDW;
                }
                else {
                    showEffectName = "";
                }
                this.showEffectController.init();
                this.showEffectController.defaultShowEffectName = showEffectName;
                // TODO: The installation at WDW has been disabled until it is installed. Uses beacon for exit until then.
                if (ToyStoryConfig_1.ToyStoryConfig.Instance.park === Park_3.Park.WDW && ToyStoryConfig_1.ToyStoryConfig.Instance.useWDWExitBeacon) {
                    PlayAPI.subscribeToEvent(PlayAPI.gameEvents.GAME_EJECT, this.onExitBeaconTriggered);
                }
            });
        }
        // Fully closes the game immediately
        // Shuts down showEffectController and sends metrics
        closeGame() {
            return __awaiter(this, void 0, void 0, function* () {
                // if we don't await it, the closeGame() slams the door shut on the endTiming call that's still in progress
                yield TSMAnalyticsManager_1.TSMAnalyticsManager.Instance.onFullGameTime(false);
                GameController.Instance.showEffectController.shutdown();
                yield PlayAPI.closeGame();
            });
        }
        get didJustStartForcedProgressionRound() {
            return this._currentForcedProgressionRound !== this.state.forcedProgressionRound;
        }
        startForcedProgressionRound(roundNum) {
            this._currentForcedProgressionRound = roundNum;
        }
        // Returns true if the player names changed from previous session
        didPlayersChange() {
            const previous = this.previousSessionPlayerNames;
            const current = this.state.playerNames;
            if (previous && current && previous.length !== 0) {
                // Test for length
                if (previous.length !== current.length) {
                    return true;
                }
                // Test for the names
                for (let name of previous) {
                    if (current.indexOf(name) < 0) {
                        return true;
                    }
                }
            }
            return false;
        }
        getEstimatedWaitTime() {
            return __awaiter(this, void 0, void 0, function* () {
                const location = yield PlayAPI.locationRead();
                const wait = yield PlayAPI.locationWaitTimeRead(location);
                return wait.waitTime; // return value in minutes
            });
        }
        hasPrimaryPlayer() {
            for (let player of this.players) {
                if (player.type === "TYPE_PRIMARY") {
                    return true;
                }
            }
            return false;
        }
        /**
         * Controls whether or not we're allowed to remind the player
         * of the installation. This is mainly dictated by a timer,
         * and whether they've played any games at all.
         */
        shouldShowInstallationReminder() {
            const now = new Date().getTime();
            if (now > this.timeOfLastInstallationReminder + GameController.TIME_BETWEEN_INSTALLATION_REMINDERS_MS &&
                this.state.totalNumPlays >= 1) {
                this.timeOfLastInstallationReminder = now;
                return true;
            }
            return false;
        }
        /**
         * Controls whether or not we're allowed to remind the player
         * of the installation. It uses a timer that starts at 30 minutes and stage one complete,
         * then reduces to 5 and 2 game gap.
         */
        shouldShowExitReminder() {
            const now = new Date().getTime();
            const timeBetweenReminders = (this.exitReminderCount < 1 ? GameController.TIME_TO_FIRST_EXIT_REMINDER_MS : GameController.TIME_BETWEEN_EXIT_REMINDERS_MS);
            const currentPlays = this.state.totalNumPlays;
            if (now > this.timeOfLastExitReminder + timeBetweenReminders &&
                currentPlays >= 7 && currentPlays - this.numPlaysOfLastExitReminder >= 2) {
                this.exitReminderCount++;
                this.timeOfLastExitReminder = now;
                this.numPlaysOfLastExitReminder = this.state.totalNumPlays;
                return true;
            }
            return false;
        }
        unlockAllGames() {
            for (let i = 0; i < this.gameOrder.length; i++) {
                this.state.getMinigameSaveData(this.gameOrder[i]).level = 1;
                this.state.getMinigameSaveData(this.gameOrder[i]).visualLevel = 1;
            }
        }
        // Returns the maximum possible total score of the game (the target)
        getQuotaTargetForGame(game) {
            return this.gameInfo.get(game).quotaPerPerson * this.players.length;
        }
        // Returns the current score for the game
        getQuotaProgressForGame(game) {
            return this.state.getMinigameSaveData(game).totalScore;
        }
        recalculateQuotas() {
            let prevPlayerCount = 0;
            if (this.previousSessionPlayerNames) {
                prevPlayerCount = this.previousSessionPlayerNames.length;
            }
            const currentPlayerCount = this.players.length;
            if (prevPlayerCount > 0 && prevPlayerCount !== currentPlayerCount) {
                for (let game of GameController.CORE_GAMES) {
                    // getQuotaTargetForGame() uses GameController.Instance.players.length
                    // so the value returned is actually the new value
                    const newGameQuotaTarget = this.getQuotaTargetForGame(game);
                    const perPlayerQuota = newGameQuotaTarget / currentPlayerCount;
                    const oldQuotaProgress = this.getQuotaProgressForGame(game);
                    const oldGameQuotaTarget = perPlayerQuota * prevPlayerCount;
                    // Get proportional progress from previous game
                    const proportionalProgress = oldQuotaProgress / oldGameQuotaTarget;
                    const newQuotaProgress = newGameQuotaTarget * proportionalProgress;
                    // Assign new score
                    this.state.getMinigameSaveData(game).totalScore = Math.ceil(newQuotaProgress);
                }
            }
        }
        getQuotaToRedeemForGameResults(game) {
            return this.state.getMinigameSaveData(game).scorePendingResultsScreen;
        }
        redeemPendingScoreForGameResults(game) {
            this.state.getMinigameSaveData(game).totalScore += this.state.getMinigameSaveData(game).scorePendingResultsScreen;
            this.state.getMinigameSaveData(game).scorePendingResultsScreen = 0;
        }
        getGameToPlayNext() {
            return this.gameOrder[this.state.forcedProgressionStep % this.gameOrder.length];
        }
        getAreAllQuotasComplete() {
            let isComplete = true;
            GameController.CORE_GAMES.forEach((gameName) => {
                if (this.getQuotaTargetForGame(gameName) > this.getQuotaProgressForGame(gameName)) {
                    isComplete = false;
                }
            });
            return isComplete;
        }
        getAreAllCoreGamesUnlocked() {
            let gameLocked = false;
            GameController.CORE_GAMES.forEach((gameName) => {
                const level = GameController.Instance.state.getGameTargetLevel(gameName);
                if (level < 1) {
                    gameLocked = true;
                }
            });
            return !gameLocked;
        }
        getAreAllGamesUnlocked() {
            let gameLocked = false;
            GameController.CORE_GAMES.forEach((gameName) => {
                const level = GameController.Instance.state.getGameTargetLevel(gameName);
                if (level < 1) {
                    gameLocked = true;
                }
            });
            return !gameLocked;
        }
        postScore(gameName, score) {
            this.currentGameScore = score;
            this.state.setMinigameScore(gameName, score);
            // Figure out if that game score will put them into a new forced progression round...
            this.state.forcedProgressionRound = this.calculateForcedProgressionRound();
        }
        calculateForcedProgressionRound() {
            const fpSteps = this.state.forcedProgressionStep;
            return Math.floor(fpSteps / this.gameOrder.length);
        }
        /* Creates a NEW state to be parsed.
            In order to clear the state, while preserving specific values.
            Called when the ride ends.
            It has the values that must not be cleared, hence some values persists acros rides.
        */
        recycleState() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[GameController] Recycling state");
                // Is much better to create a new object rather than stripping the old one (has implications over reference sharing)
                let newData = {};
                // Reset the current players, but keep their names
                newData._playerNames = this.state.playerNames.slice();
                // Setting to false will avoid triggering the confirmation dialog by mistake
                newData._sessionStarted = false;
                // Must be persisted across sessions
                newData._allPlays = this.state.allPlays;
                this.state.parse(newData);
                this.state.save();
            });
        }
        calculateDefaultTextSizes(gameWidth, gameHeight) {
            TextUtil_3.TextUtil.calculateFontSizeModifier(gameWidth, gameHeight);
            Game_3.Game.FONT_SIZE_EXTRA_LARGE = TextUtil_3.TextUtil.getFontSize(gameWidth, 60);
            Game_3.Game.FONT_SIZE_LARGE = TextUtil_3.TextUtil.getFontSize(gameWidth, 40);
            Game_3.Game.FONT_SIZE_MEDIUM = TextUtil_3.TextUtil.getFontSize(gameWidth, 20);
            Game_3.Game.FONT_SIZE_SMALL = TextUtil_3.TextUtil.getFontSize(gameWidth, 10);
        }
        onExitBeaconTriggered() {
            // If we haven't already set the exit flag, set it
            if (!GameController.Instance.state.exitTriggered) {
                GameController.Instance.state.exitTriggered = true;
                this._onExitTriggeredSignal.dispatch();
            }
        }
    }
    GameController.MAX_NODE_UPGRADE_LEVEL = 4;
    GameController.NUM_SCORABLE_GAMES = 7;
    GameController.CORE_GAMES = [
        StateNames_4.StateNames.GAME_TRIXIE,
        StateNames_4.StateNames.GAME_FLYING_TOSSERS,
        StateNames_4.StateNames.GAME_SARGE,
        StateNames_4.StateNames.GAME_HAMM,
        StateNames_4.StateNames.GAME_WOODY_LASSO,
        StateNames_4.StateNames.GAME_DRAWING,
        StateNames_4.StateNames.GAME_DRESS_UP
    ];
    GameController.TIME_BETWEEN_INSTALLATION_REMINDERS_MS = 300000;
    GameController.TIME_TO_FIRST_EXIT_REMINDER_MS = 30 * 60 * 1000; // 30 minutes
    GameController.TIME_BETWEEN_EXIT_REMINDERS_MS = 8 * 60 * 1000; // 8 minutes
    exports.GameController = GameController;
});
define("switchback-common/util/ScaleUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function scaleToBounds(object, widthBound, heightBound, enlargeToFit = true) {
        if (!enlargeToFit && object.width <= widthBound && object.height <= heightBound) {
            return;
        }
        const xRatio = object.width / widthBound;
        const yRatio = object.height / heightBound;
        if (xRatio > yRatio) {
            object.width = widthBound;
            object.scale.y = object.scale.x;
        }
        else {
            object.height = heightBound;
            object.scale.x = object.scale.y;
        }
    }
    exports.scaleToBounds = scaleToBounds;
    // Usage: scaleToSingleBound(object, this.game.width * 0.3); width will be set to bound, and height in proportion
    // Usage: scaleToSingleBound(object, this.game.width * 0.3, false); height will be set to bound, and width in proportion
    function scaleToSingleBound(object, bound, isWidth = true) {
        if (isWidth) {
            object.width = bound;
            object.scale.y = object.scale.x;
        }
        else {
            object.height = bound;
            object.scale.x = object.scale.y;
        }
    }
    exports.scaleToSingleBound = scaleToSingleBound;
    // Scales the object such that its smallest dimension covers the specified bounds. Its
    // largest dimension will likely exceed the other bound.
    // Useful for ensuring that a background covers the screen without stretching
    function scaleToCover(object, widthBound, heightBound) {
        const xRatio = object.width / widthBound;
        const yRatio = object.height / heightBound;
        if (xRatio < yRatio) {
            object.width = widthBound;
            object.scale.y = object.scale.x;
        }
        else {
            object.height = heightBound;
            object.scale.x = object.scale.y;
        }
    }
    exports.scaleToCover = scaleToCover;
    class ScaleUtil {
        static getVerticalProportionalPosition(pt1, pt2, proportion = 0.5) {
            return pt1 + ((pt2 - pt1) * proportion);
        }
    }
    exports.ScaleUtil = ScaleUtil;
});
define("switchback-common/state/BaseState", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * BaseState is our extension of Phaser.State that all states within games should extend.
     * This is to allow us to handle things like state transitions and display object hierarchies in a common way.
     * @author joel@cerebralfix.com
     */
    class BaseState extends Phaser.State {
        constructor() {
            super(...arguments);
            // Duration of the fade graphic alpha in and out, in milliseconds
            this.FADE_DURATION_MS = 300;
            this.stateName = "BaseState";
            this.firstPointerDownTime = 0;
            this.tapCount = 0;
        }
        init(...args) {
            super.init(...args);
            console.log(`[${this.stateName}] init(${JSON.stringify(args)})`);
            this.shutdownSignal = new Phaser.Signal();
            this.isShutdown = false;
            // Hijack the state's add function to ensure all game objects are
            // added to the correct group (e.g. under the fade overlay)
            this.add.existing = this.addToGameLayer.bind(this);
            // Create our game group here
            this.gameGroup = new Phaser.Group(this.game);
            // Create our dev group here, goes on top of the game group
            this.createDevGroup();
            // Then immediately add our fade overlay on top. This ensures the
            // overlay is present for the duration of the loading
            this.fadeImage = this.add.graphics(0, 0);
            this.fadeImage.beginFill(BaseState.OVERLAY_COLOR);
            this.fadeImage.drawRect(0, 0, this.game.width, this.game.height);
            this.fadeImage.endFill();
            this.fadeImage.fixedToCamera = true;
            this.fadeImage.inputEnabled = true;
            this.game.input.onDown.add(this.onPointerDown, this);
        }
        create() {
            console.log(`[${this.stateName}] create()`);
            super.create();
        }
        removeChildFromGameGroup(child) {
            this.gameGroup.remove(child);
        }
        createDevGroup() {
            this.devGroup = new Phaser.Group(this.game);
            this.devGroup.visible = false;
            const data = this.game.cache.getJSON("build-info");
            if (data) {
                let versionString = `${data.hash}`;
                if (data.version) {
                    versionString = "v" + `${data.version}` + "." + `${data.build}` + "-" + `${data.agent}` + " (" + `${data.hash}` + ")";
                }
                const text = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height, versionString, {
                    fontSize: 50,
                    fill: "white",
                    stroke: "black",
                    strokeThickness: 5,
                    align: "center"
                });
                text.anchor.set(0.5, 1);
                this.game.input.onDown.add(this.onInputDown, this);
                text.inputEnabled = true;
                this.devGroup.add(text);
            }
        }
        onInputDown() {
            this.devGroup.visible = false;
        }
        onPointerDown(pointer) {
            // Was it in the right place?
            if (pointer.positionDown.x < this.game.width * 0.1 &&
                pointer.positionDown.y > this.game.height * 0.9) {
                // If it's been too long since the last tap in this area, start the timer and counter again
                if (this.game.time.now > this.firstPointerDownTime + 1000) {
                    this.firstPointerDownTime = this.game.time.now;
                    this.tapCount = 0;
                }
                this.tapCount++;
                if (this.tapCount === 5) {
                    this.devGroup.visible = true;
                    this.tapCount = 0;
                }
            }
            else {
                // If it wasn't in the right place, reset the counter.
                this.tapCount = 0;
            }
        }
        preload() {
            super.preload();
            console.log(`[${this.stateName}] preload()`);
            // Add a listener so we can see which files are loading
            this.load.onFileComplete.add((progress, key, success, loaded, total) => {
                console.log(`[${this.stateName}] preload() [${progress}% | ${loaded}/${total}] loading: ${key}`);
            });
            // Load the game's common assets into this state
            if (BaseState.loadCommonAssets) {
                BaseState.loadCommonAssets();
            }
        }
        ensurePointers(num) {
            // check if we can even do that
            if (num > Phaser.Input.MAX_POINTERS) {
                console.log(`[${this.stateName}] Can't add pointers. `, num, "/", Phaser.Input.MAX_POINTERS);
                return false;
            }
            for (let i = 1; i < num + 1; i++) {
                // if pointer doesn't exists, add it
                if (!this.game.input["pointer" + i]) {
                    this.game.input.addPointer();
                }
                // if it hasnt been added, explode
                if (!this.game.input["pointer" + i]) {
                    return false;
                }
            }
            // fix reduced inputs
            this.game.input.maxPointers = num;
            return true;
        }
        fadeToState(state, ...args) {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.showOverlay();
                this.game.state.start(state, true, false, ...args);
            });
        }
        /*
         * Shows the input-blocking overlay on top of the game, then tweens it to full alpha.
         * This function should be awaited before transitioning away to a loading screen.
         */
        showOverlay(alpha = 1, fadeDurationMS = this.FADE_DURATION_MS, newColor = undefined) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] showOverlay()`);
                this.game.tweens.removeFrom(this.fadeImage);
                this.fadeImage.alpha = 0;
                this.fadeImage.visible = true;
                if (newColor) {
                    BaseState.OVERLAY_COLOR = newColor;
                    this.fadeImage.clear();
                    this.fadeImage.beginFill(BaseState.OVERLAY_COLOR);
                    this.fadeImage.drawRect(0, 0, this.game.width, this.game.height);
                    this.fadeImage.endFill();
                }
                if (fadeDurationMS === 0) {
                    this.fadeImage.alpha = alpha;
                }
                else {
                    this.fadeImage.alpha = 0;
                    const fadeInTween = this.game.add.tween(this.fadeImage).to({ alpha }, fadeDurationMS, Phaser.Easing.Linear.None, true);
                    yield PromiseUtil_5.PromiseUtil.tween2(fadeInTween, this.shutdownSignal).catch(() => { });
                }
            });
        }
        /*
         * Tweens the overlay out to alpha 0 and then sets it to invisible (to remove input-blocking).
         * This function should be called when the state's create() function is complete
         * and everything has been setup on-screen.
         */
        dismissOverlay(alpha = 0, fadeDurationMS = this.FADE_DURATION_MS) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] dismissOverlay()`);
                this.game.tweens.removeFrom(this.fadeImage);
                this.fadeImage.alpha = 1;
                this.fadeImage.visible = true;
                if (fadeDurationMS === 0) {
                    this.fadeImage.alpha = alpha;
                    this.fadeImage.visible = false;
                }
                else {
                    this.fadeImage.alpha = 1;
                    const fadeOutTween = this.game.add.tween(this.fadeImage).to({ alpha }, fadeDurationMS, Phaser.Easing.Linear.None, true);
                    fadeOutTween.onComplete.addOnce(() => {
                        this.fadeImage.visible = false;
                        this.overlayDismissed();
                    });
                    yield PromiseUtil_5.PromiseUtil.tween2(fadeOutTween, this.shutdownSignal).catch(() => { });
                }
            });
        }
        shutdown() {
            console.log(`[${this.stateName}] shutdown()`);
            this.isShutdown = true;
            if (this.shutdownSignal) {
                this.shutdownSignal.dispatch();
                this.shutdownSignal.dispose();
            }
            this.game.input.onDown.remove(this.onPointerDown, this);
            // Remove our file load listener
            this.load.onFileComplete.removeAll();
            this.fadeImage.destroy();
            this.devGroup.destroy();
            this.gameGroup.destroy();
            super.shutdown();
            this.shutdownSignal = null;
            this.fadeImage = null;
            this.gameGroup = null;
            this.devGroup = null;
        }
        /*
         * Overridden by subclasses to determine when overlay has finished
         * dismissing
         */
        overlayDismissed() {
        }
        /*
         * This function is a replacement for this.add.existing and routes the
         * object added to the gameGroup instead of the top of the state.
         */
        addToGameLayer(object) {
            this.gameGroup.add(object);
            return object;
        }
    }
    BaseState.OVERLAY_COLOR = 0x000000;
    exports.BaseState = BaseState;
});
define("state/ToyStoryBaseState", ["require", "exports", "Game", "logic/GameController", "StateNames", "switchback-common/state/BaseState", "model/ToyStoryConfig", "switchback-common/enum/Park"], function (require, exports, Game_4, GameController_6, StateNames_5, BaseState_1, ToyStoryConfig_2, Park_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author qi@cerebralfix.com
     */
    class ToyStoryBaseState extends BaseState_1.BaseState {
        init(...params) {
            super.init(params);
            // refresh the player array on loading each state
            GameController_6.GameController.Instance.playerManager.refreshPlayersArray();
        }
        preload() {
            super.preload();
            this.load.image('plus', 'assets/common/plus.png');
            this.load.spritesheet('button-basic', 'assets/common/button-basic.png', 100, 60);
        }
        create() {
            super.create();
            if (ToyStoryConfig_2.ToyStoryConfig.Instance.enableFPS) {
                this.fpsText = this.game.add.text(this.game.width, 0, "", { fill: "white", align: 'right', fontSize: 50 });
                this.fpsText.anchor.set(1, 0);
                this.fpsText.fixedToCamera = true;
            }
            if (ToyStoryConfig_2.ToyStoryConfig.Instance.showDebugMenu) {
                this.debugButtonLayer = new Phaser.Group(this.game);
                // Make a toggle button that toggles the interrupt buttons
                const toggleButton = new Phaser.Button(this.game, 0, 0, 'plus', () => {
                    for (let i = 1; i < this.debugButtonLayer.children.length; i++) {
                        this.debugButtonLayer.children[i].visible = !this.debugButtonLayer.children[i].visible;
                    }
                }, this);
                toggleButton.width = toggleButton.height = this.game.width * 0.1;
                this.debugButtonLayer.add(toggleButton);
                this.debugButtonLayer.top = 0;
                // Force outro button
                if (ToyStoryConfig_2.ToyStoryConfig.Instance.park === Park_4.Park.WDW && !ToyStoryConfig_2.ToyStoryConfig.Instance.useWDWInstallation) {
                    this.debugButtonLayer.add(this.createDebugButton('X', 1, () => {
                        // If we haven't already set the exit flag, set it
                        if (!GameController_6.GameController.Instance.state.exitTriggered) {
                            GameController_6.GameController.Instance.onExitBeaconTriggered();
                        }
                    }));
                }
                // Clear save state button
                this.debugButtonLayer.add(this.createDebugButton('🗑', 2, () => __awaiter(this, void 0, void 0, function* () {
                    GameController_6.GameController.Instance.state.clear();
                    yield GameController_6.GameController.Instance.closeGame();
                })));
                // Clear save state button
                this.debugButtonLayer.add(this.createDebugButton('P', 3, () => {
                    this.game.state.start(StateNames_5.StateNames.POSTER);
                }));
            }
            GameController_6.GameController.Instance.onExitTriggeredSignal.addOnce(this.onExitTriggered, this);
        }
        /**
         * Handles exit trigger events. By default, does nothing,
         * and reports that the exit trigger was not followed up on.
         */
        onExitTriggered() {
            return __awaiter(this, void 0, void 0, function* () { return false; });
        }
        createDebugButton(label, num, callbackFn) {
            const debugButton = new Phaser.Button(this.game, 0, 0, 'button-basic', callbackFn, this, 0, 0, 1, 0);
            debugButton.width = debugButton.height = this.world.width * 0.1;
            debugButton.anchor.set(0.5);
            debugButton.top = Math.floor(num / 7.0) * (this.game.width * 0.12);
            debugButton.left = (num % 7) * (this.game.width * 0.12);
            const debugButtonText = new Phaser.Text(this.game, 0, 0, label, {
                fill: 'black',
                font: Game_4.Game.DEFAULT_FONT,
                fontSize: Game_4.Game.FONT_SIZE_LARGE,
                align: 'center'
            });
            debugButtonText.setScaleMinMax(1);
            debugButtonText.anchor.set(0.5);
            debugButton.addChild(debugButtonText);
            debugButton.visible = false;
            return debugButton;
        }
        update() {
            super.update();
            if (ToyStoryConfig_2.ToyStoryConfig.Instance.enableFPS && this.fpsText) {
                this.fpsText.text = this.game.time.fps.toString();
            }
        }
        shutdown() {
            this.debugButtonLayer = null;
            this.fpsText = null;
            GameController_6.GameController.Instance.onExitTriggeredSignal.remove(this.onExitTriggered, this);
            super.shutdown();
        }
    }
    exports.ToyStoryBaseState = ToyStoryBaseState;
});
define("component/ArrowPrompt", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ArrowPrompt extends Phaser.Sprite {
        constructor(game, x = 0, y = 0) {
            super(game, x, y, 'prompt-arrow');
            this.animations.add("idle", null, 20, true, true);
            this.play('idle');
            this.anchor.set(0.5);
        }
    }
    exports.ArrowPrompt = ArrowPrompt;
});
define("switchback-common/component/text/TypewriterText", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TypewriterText extends Phaser.Text {
        constructor() {
            super(...arguments);
            this.isSkipping = false;
            this.softSkip = false;
        }
        typewriter(letterDelay, soundTag = null, softSkipParent = null, shutdownSignal = null) {
            return __awaiter(this, void 0, void 0, function* () {
                if (softSkipParent && !this.isSkipping) {
                    softSkipParent.events.onInputDown.addOnce(this.onSoftSkip, this);
                }
                let sndTypewriterLoop;
                let soundVolumeTween;
                if (soundTag && !this.isSkipping) {
                    sndTypewriterLoop = this.game.sound.play(soundTag, 0.0, true);
                    sndTypewriterLoop.play();
                    sndTypewriterLoop.volume = 0;
                    soundVolumeTween = this.game.add.tween(sndTypewriterLoop).to({ volume: 0.5 }, 500, Phaser.Easing.Linear.None, true);
                }
                const numChars = this.text.length;
                this.softSkip = false;
                let isShutdown = false;
                for (let i = 0; !this.isSkipping && !this.softSkip && i < numChars; i++) {
                    this.clearColors();
                    this.addColor("rgba(0,0,0,0)", i);
                    yield PromiseUtil_6.PromiseUtil.realTime(letterDelay, shutdownSignal).catch(() => { isShutdown = true; });
                    if (isShutdown) {
                        if (sndTypewriterLoop) {
                            soundVolumeTween.stop();
                            sndTypewriterLoop.stop();
                        }
                        return;
                    }
                }
                if (softSkipParent) {
                    softSkipParent.events.onInputDown.remove(this.onSoftSkip, this);
                }
                this.clearColors();
                // Adding colors offsets the text kerning differently, so this has to be here for consistency even though it doesn't color anything
                this.addColor("rgba(0,0,0,0)", numChars);
                this.softSkip = false;
                if (sndTypewriterLoop) {
                    soundVolumeTween.stop();
                    sndTypewriterLoop.stop();
                }
            });
        }
        clearText() {
            this.addColor("rgba(0,0,0,0)", 0);
        }
        onSoftSkip() {
            this.softSkip = true;
        }
    }
    exports.TypewriterText = TypewriterText;
});
define("state/CutSceneState", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "model/ToyStoryConfig", "Game", "StateNames", "state/ToyStoryBaseState", "component/ArrowPrompt", "logic/TSMAnalyticsManager", "logic/GameController", "switchback-common/component/text/TypewriterText", "switchback-common/enum/Park"], function (require, exports, PromiseUtil_7, ScaleUtil_1, TextUtil_4, LocalizationManager_2, ToyStoryConfig_3, Game_5, StateNames_6, ToyStoryBaseState_1, ArrowPrompt_1, TSMAnalyticsManager_2, GameController_7, TypewriterText_1, Park_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     * @author jeronimo@cerebralfix.com
     */
    var CharacterName;
    (function (CharacterName) {
        CharacterName[CharacterName["WOODY"] = 0] = "WOODY";
        CharacterName[CharacterName["REX"] = 1] = "REX";
        CharacterName[CharacterName["HAMM"] = 2] = "HAMM";
        CharacterName[CharacterName["SARGE"] = 3] = "SARGE";
        CharacterName[CharacterName["BUZZ"] = 4] = "BUZZ";
        CharacterName[CharacterName["JESSIE"] = 5] = "JESSIE";
        CharacterName[CharacterName["TRIXIE"] = 6] = "TRIXIE";
        CharacterName[CharacterName["PETE"] = 7] = "PETE";
        CharacterName[CharacterName["PETE_FOUND"] = 8] = "PETE_FOUND";
    })(CharacterName || (CharacterName = {}));
    class CutSceneState extends ToyStoryBaseState_1.ToyStoryBaseState {
        static showCutScene(game, cutSceneToShow, stateToTransistionToAfter, score = null) {
            CutSceneState.nextState = stateToTransistionToAfter;
            CutSceneState.cutSceneToShow = cutSceneToShow;
            CutSceneState.lastScore = score;
            game.state.start(StateNames_6.StateNames.CUT_SCENE);
        }
        init() {
            this.stateName = StateNames_6.StateNames.CUT_SCENE;
            super.init();
            this.characters = new Map();
        }
        preload() {
            super.preload();
            this.game.load.image('cut-scene-bg', 'assets/cutscene/background.jpg');
            this.game.load.image('cut-scene-lights-back', 'assets/cutscene/lights_back.png');
            this.game.load.image('cut-scene-lights-front', 'assets/cutscene/lights_front.png');
            this.game.load.image('cut-scene-lights-strip', 'assets/cutscene/lights_strip.png');
            this.game.load.image('cut-scene-text-box-top', 'assets/cutscene/textbox_top.png');
            this.game.load.image('cut-scene-text-box-middle', 'assets/cutscene/textbox_middle.png');
            this.game.load.image('skip', 'assets/common/skip.png');
            this.game.load.image('portrait-woody-idle', 'assets/character-portraits/woody/idle.png');
            this.game.load.image('portrait-rex-idle', 'assets/character-portraits/rex/idle.png');
            this.game.load.image('portrait-trixie-idle', 'assets/character-portraits/trixie/idle.png');
            this.game.load.image('portrait-buzz-idle', 'assets/character-portraits/buzz/idle.png');
            this.game.load.image('portrait-hamm-idle', 'assets/character-portraits/hamm/idle.png');
            this.game.load.image('portrait-jessie-idle', 'assets/character-portraits/jessie/idle.png');
            this.game.load.image('portrait-sarge-idle', 'assets/character-portraits/sarge/idle.png');
            if (ToyStoryConfig_3.ToyStoryConfig.Instance.park === Park_5.Park.WDW) {
                this.game.load.image('portrait-pete-idle', 'assets/character-portraits/pete/idle-wdw.png');
                this.game.load.image('portrait-pete-found-idle', 'assets/character-portraits/pete-found/idle-wdw.png');
                this.game.load.image('overlay', 'assets/jail-installation/jail-overlay.png');
            }
            else {
                this.game.load.image('portrait-pete-idle', 'assets/character-portraits/pete/idle-dlr.png');
                this.game.load.image('portrait-pete-found-idle', 'assets/character-portraits/pete-found/idle-dlr.png');
            }
            this.game.load.json('cut-scenes', 'assets/json/cut-scenes.json');
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                const json = this.game.cache.getJSON('cut-scenes');
                this.cutScene = json[CutSceneState.cutSceneToShow];
                this.background = new Phaser.Image(this.game, 0, 0, 'cut-scene-bg');
                ScaleUtil_1.scaleToCover(this.background, this.game.width, this.game.height);
                this.background.centerX = this.game.width * 0.5;
                this.background.centerY = this.game.height * 0.5;
                this.background.inputEnabled = true;
                this.add.existing(this.background);
                this.loadCharacter(CharacterName.WOODY, 'common.character_woody', 'portrait-woody-idle');
                this.loadCharacter(CharacterName.REX, 'common.character_rex', 'portrait-rex-idle');
                this.loadCharacter(CharacterName.BUZZ, 'common.character_buzz', 'portrait-buzz-idle');
                this.loadCharacter(CharacterName.HAMM, 'common.character_hamm', 'portrait-hamm-idle');
                this.loadCharacter(CharacterName.JESSIE, 'common.character_jessie', 'portrait-jessie-idle');
                this.loadCharacter(CharacterName.SARGE, 'common.character_sarge', 'portrait-sarge-idle');
                this.loadCharacter(CharacterName.TRIXIE, 'common.character_trixie', 'portrait-trixie-idle');
                this.loadCharacter(CharacterName.PETE, 'common.character_pete', 'portrait-pete-idle');
                this.loadCharacter(CharacterName.PETE_FOUND, 'common.character_pete_found', 'portrait-pete-found-idle');
                // This is specifically for Stinky Pete in jail
                if (ToyStoryConfig_3.ToyStoryConfig.Instance.park === Park_5.Park.WDW) {
                    this.overlayImage = new Phaser.Image(this.game, 0, 0, "overlay");
                    this.overlayImage.width = this.game.width;
                    this.overlayImage.height = this.game.height;
                    this.overlayImage.visible = false;
                    this.add.existing(this.overlayImage);
                }
                this.speechBox = new Phaser.Group(this.game);
                this.speechBox.visible = false;
                this.add.existing(this.speechBox);
                const textBoxTop = new Phaser.Image(this.game, 0, 0, 'cut-scene-text-box-top');
                textBoxTop.width = this.game.width;
                textBoxTop.scale.y = textBoxTop.scale.x;
                this.speechBox.add(textBoxTop);
                const textBoxMiddle = new Phaser.Image(this.game, 0, 0, 'cut-scene-text-box-middle');
                textBoxMiddle.width = this.game.width;
                textBoxMiddle.height = this.game.height * 0.37;
                textBoxMiddle.top = textBoxTop.bottom;
                this.speechBox.add(textBoxMiddle);
                this.text = new TypewriterText_1.TypewriterText(this.game, 0, 0, '', {
                    font: Game_5.Game.DEFAULT_FONT,
                    fontSize: TextUtil_4.TextUtil.getFontSize(this.game.width, 35),
                    fill: 'white',
                    wordWrap: true,
                    wordWrapWidth: this.game.width * 0.94
                });
                this.text.setShadow(Game_5.Game.TEXT_SHADOW_OFFSET_MED_X, Game_5.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_5.Game.TEXT_SHADOW_DIAG, 0, true, true);
                this.text.left = this.speechBox.width * 0.03;
                this.text.top = textBoxMiddle.top;
                this.speechBox.add(this.text);
                this.characterName = new Phaser.Text(this.game, 0, 0, '', {
                    font: Game_5.Game.CARNIVAL_FONT,
                    fontSize: TextUtil_4.TextUtil.getFontSize(this.game.width, 36),
                    fill: Game_5.Game.TEXT_COLOR_CHAR_FILL,
                    stroke: Game_5.Game.TEXT_COLOR_CHAR_STROKE,
                    strokeThickness: this.speechBox.width * 0.01
                });
                this.speechBox.add(this.characterName);
                this.speechBox.bottom = this.game.height;
                const lightsBack = new Phaser.Image(this.game, 0, 0, 'cut-scene-lights-back');
                lightsBack.width = this.game.width;
                lightsBack.scale.y = lightsBack.scale.x;
                lightsBack.bottom = this.game.height;
                this.add.existing(lightsBack);
                this.lights = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height * 0.04, 'cut-scene-lights-strip');
                this.lights.tileScale.x = this.lights.width / this.lights.texture.width;
                this.lights.bottom = lightsBack.bottom;
                this.lights.autoScroll(this.game.width * 0.5, 0);
                this.lights.alpha = 0;
                this.add.existing(this.lights);
                const lightsFront = new Phaser.Image(this.game, 0, 0, 'cut-scene-lights-front');
                lightsFront.width = lightsBack.width;
                lightsFront.scale.y = lightsBack.scale.y;
                lightsFront.bottom = lightsBack.bottom;
                this.add.existing(lightsFront);
                this.prompt = new ArrowPrompt_1.ArrowPrompt(this.game, 0, 0);
                this.prompt.height = lightsBack.height * 0.7;
                this.prompt.scale.x = this.prompt.scale.y;
                this.prompt.right = this.game.width - (this.game.width * 0.005);
                this.prompt.bottom = this.game.height - (this.game.width * 0.01);
                this.prompt.visible = false;
                this.add.existing(this.prompt);
                const padding = this.game.width * 0.005;
                this.skipButton = new Phaser.Image(this.game, 0, 0, 'skip');
                this.skipButton.width = this.skipButton.height = this.game.width * 0.12;
                this.skipButton.top = padding;
                this.skipButton.right = this.game.width - padding;
                this.skipButton.inputEnabled = true;
                this.skipButton.events.onInputDown.addOnce(() => {
                    this.game.sound.play('snd_ui_click');
                    this.skip();
                });
                if (ToyStoryConfig_3.ToyStoryConfig.Instance.debug) {
                    this.add.existing(this.skipButton);
                }
                if (ToyStoryConfig_3.ToyStoryConfig.Instance.skipCutscenes) {
                    this.goToNextState();
                }
                yield this.dismissOverlay();
                if (this.isShutdown) {
                    return;
                }
                if (this.cutScene) {
                    // set the values to track
                    this._trackedTexts = 0;
                    this._trackedTextsCount = 0;
                    for (const scene of this.cutScene) {
                        this._trackedTextsCount += scene.text.length;
                    }
                    for (let i = 0; !this.text.isSkipping && i < this.cutScene.length; i++) {
                        yield this.showDialogue(this.cutScene[i].character, this.cutScene[i].overlay, this.cutScene[i].text);
                    }
                }
                yield this.showOverlay();
                this.goToNextState();
            });
        }
        goToNextState() {
            if (CutSceneState.cutSceneToShow === "pete_caught") {
                // AT DLR and WDW this cutscene indicates the installation has been triggered and the game is ready to exit
                GameController_7.GameController.Instance.state.exitTriggered = true;
            }
            // Tidy up stored transition data, and move to the next state
            const nextState = CutSceneState.nextState;
            CutSceneState.nextState = null;
            this.game.state.start(nextState);
        }
        loadCharacter(name, nameTag, portraitIdleTag) {
            const portraitIdle = new Phaser.Image(this.game, 0, 0, portraitIdleTag);
            portraitIdle.height = this.game.height * 0.68;
            portraitIdle.scale.x = portraitIdle.scale.y;
            portraitIdle.centerX = this.game.width * 0.5;
            portraitIdle.top = this.game.height * 0.05;
            portraitIdle.visible = false;
            this.add.existing(portraitIdle);
            this.characters.set(name, {
                name: LocalizationManager_2.LocalizationManager.Instance.text(nameTag),
                portraitIdle: portraitIdle
            });
        }
        showDialogue(character, overlay, text) {
            return __awaiter(this, void 0, void 0, function* () {
                const characterName = this.getCharacterFromName(character);
                if (characterName === null) {
                    this._trackedTexts += text.length;
                    return;
                }
                const characterData = this.characters.get(characterName);
                // Set character name
                this.characterName.text = characterData.name;
                this.characterName.fontSize = TextUtil_4.TextUtil.getFontSize(this.game.width, 36);
                TextUtil_4.TextUtil.scaleDownSingleLineTextToWidth(this.characterName, this.speechBox.width * 0.28);
                this.characterName.centerX = this.speechBox.width * 0.18;
                // note: can't set sized based on speechbox height as it varies with device
                this.characterName.centerY = this.speechBox.width * 0.14;
                // Clear text
                this.text.text = '';
                // Animate in
                this.speechBox.top = this.game.height;
                this.speechBox.visible = true;
                yield PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(this.speechBox).to({
                    bottom: this.game.height
                }, 350, Phaser.Easing.Cubic.Out).start());
                if (this.overlayImage) {
                    this.overlayImage.visible = (overlay != null);
                }
                const portrait = characterData.portraitIdle;
                portrait.right = 0;
                portrait.visible = true;
                yield PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(portrait).to({
                    centerX: this.game.width * 0.5
                }, 350, Phaser.Easing.Cubic.Out).start());
                // Write out each page of dialog
                for (let i = 0; !this.text.isSkipping && i < text.length; i++) {
                    this.trackCutscene();
                    this.text.text = LocalizationManager_2.LocalizationManager.Instance.text(text[i]);
                    this.text.fontSize = TextUtil_4.TextUtil.getFontSize(this.game.width, 35);
                    TextUtil_4.TextUtil.scaleDownTextToHeight(this.text, this.speechBox.height * 0.635);
                    yield this.text.typewriter(15, 'typewriter_loop', this.background);
                    if (!this.text.isSkipping) {
                        yield this.showPrompt();
                    }
                }
                // Animate out
                yield PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(portrait).to({
                    right: 0
                }, 350, Phaser.Easing.Cubic.In).start());
                portrait.visible = false;
                yield PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(this.speechBox).to({
                    top: this.game.height
                }, 350, Phaser.Easing.Cubic.In).start());
                this.speechBox.visible = false;
            });
        }
        getCharacterFromName(name) {
            let character = null;
            switch (name) {
                case "woody":
                    character = CharacterName.WOODY;
                    break;
                case "rex":
                    character = CharacterName.REX;
                    break;
                case "hamm":
                    character = CharacterName.HAMM;
                    break;
                case "sarge":
                    character = CharacterName.SARGE;
                    break;
                case "buzz":
                    character = CharacterName.BUZZ;
                    break;
                case "jessie":
                    character = CharacterName.JESSIE;
                    break;
                case "trixie":
                    character = CharacterName.TRIXIE;
                    break;
                case "pete":
                    character = CharacterName.PETE;
                    break;
                case "pete-found":
                    character = CharacterName.PETE_FOUND;
                    break;
                default:
                    console.log("Character not found: " + name);
                    break;
            }
            return character;
        }
        showPrompt() {
            return __awaiter(this, void 0, void 0, function* () {
                this.prompt.visible = true;
                this.prompt.alpha = 0;
                this.lights.visible = true;
                this.lights.alpha = 0;
                const promptFadeTime = 500;
                this.game.add.tween(this.prompt).to({ alpha: 1 }, promptFadeTime).start();
                yield PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(this.lights).to({ alpha: 1 }, promptFadeTime).start());
                return new Promise((resolve, reject) => {
                    this.skipPrompt = () => __awaiter(this, void 0, void 0, function* () {
                        this.game.add.tween(this.prompt).to({ alpha: 0 }, promptFadeTime).start();
                        PromiseUtil_7.PromiseUtil.tween(this.game, this.game.add.tween(this.lights).to({ alpha: 0 }, promptFadeTime).start()).then(() => {
                            this.prompt.visible = false;
                            this.lights.visible = false;
                        });
                        this.background.events.onInputDown.removeAll();
                        this.skipPrompt = null;
                        resolve();
                    });
                    this.background.events.onInputDown.addOnce(this.skipPrompt);
                });
            });
        }
        skip() {
            this.text.isSkipping = true;
            this.skipButton.visible = false;
            if (this.skipPrompt) {
                this.skipPrompt();
            }
        }
        trackCutscene() {
            this._trackedTexts++;
            const cutscene = CutSceneState.cutSceneToShow;
            switch (cutscene) {
                case "intro":
                    TSMAnalyticsManager_2.TSMAnalyticsManager.Instance.onIntroCutscene(this._trackedTexts, this._trackedTextsCount);
                    break;
                case "pete_caught":
                    TSMAnalyticsManager_2.TSMAnalyticsManager.Instance.onPeteCaughtCutscene(this._trackedTexts, this._trackedTextsCount);
                    break;
                case "outro":
                case "outro_with_pete":
                    TSMAnalyticsManager_2.TSMAnalyticsManager.Instance.onOutroCutscene(this._trackedTexts, this._trackedTextsCount);
                    break;
                default:
                    TSMAnalyticsManager_2.TSMAnalyticsManager.Instance.onMiniGameCutscene(cutscene, this._trackedTexts, this._trackedTextsCount, CutSceneState.lastScore);
                    break;
            }
        }
        shutdown() {
            this.characters = null;
            this.cutScene = null;
            this.background = null;
            this.speechBox = null;
            this.text = null;
            this.characterName = null;
            this.prompt = null;
            this.lights = null;
            this.skipButton = null;
            this.skipPrompt = null;
            super.shutdown();
        }
    }
    exports.CutSceneState = CutSceneState;
});
define("component/dialog/ResumeDialog", ["require", "exports", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "Game", "logic/GameController", "component/dialog/BasicCTADialog", "logic/TSMAnalyticsManager", "switchback-common/util/TextUtil"], function (require, exports, LocalizationManager_3, PromiseUtil_8, Game_6, GameController_8, BasicCTADialog_2, TSMAnalyticsManager_3, TextUtil_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ResumeDialog extends Phaser.Group {
        constructor(game) {
            super(game);
            this.onButtonClickedSignal = new Phaser.Signal();
            this.shade = new Phaser.Graphics(this.game, 0, 0);
            this.shade.beginFill(0x000000, 0.75);
            this.shade.drawRect(0, 0, this.game.width, this.game.height);
            this.shade.endFill();
            this.shade.alpha = 0;
            this.add(this.shade);
            /**
             * Top banner that descends down
             */
            this.topBanner = new Phaser.Group(game);
            const bannerHeader = new Phaser.Image(game, 0, 0, 'banner-header');
            bannerHeader.width = game.width;
            bannerHeader.scale.y = bannerHeader.scale.x;
            this.topBanner.add(bannerHeader);
            this.add(this.topBanner);
            this.topBanner.bottom = 0;
            this.topBanner.visible = false;
            /**
             * Bottom banner that rises up
             * Includes: banner footer, cancel and exit big buttons
             */
            this.bottomBanner = new Phaser.Group(game);
            const bannerFooter = new Phaser.Image(game, 0, 0, 'banner-footer');
            bannerFooter.width = game.width * 1.22;
            bannerFooter.scale.y = bannerFooter.scale.x;
            this.bannerFooterHeight = bannerFooter.height;
            this.restartBtn = new Phaser.Image(game, 0, 0, 'big-btn-left-clean');
            this.restartBtn.width = bannerFooter.width * 0.48;
            this.restartBtn.scale.y = this.restartBtn.scale.x;
            this.restartBtn.top = bannerFooter.top - (this.restartBtn.height * 0.1);
            this.restartBtn.right = bannerFooter.width * 0.49;
            this.restartBtn.inputEnabled = true;
            this.restartBtn.events.onInputDown.add(() => {
                this.game.sound.play('snd_ui_click');
                this.onBtnClick(false);
            }, this);
            const bigBtnTextStyle = {
                font: Game_6.Game.CARNIVAL_FONT,
                fontSize: TextUtil_5.TextUtil.getFontSize(game.width, 30),
                fill: 'white',
                stroke: 'black',
                strokeThickness: Game_6.Game.TEXT_STROKE_THICKNESS_HUMONGOUS
            };
            const restartBtnText = new Phaser.Text(game, 0, 0, LocalizationManager_3.LocalizationManager.Instance.text("resume.no").toUpperCase(), bigBtnTextStyle);
            restartBtnText.anchor.set(0.5);
            let paddingY = this.restartBtn.height * 0.02;
            restartBtnText.centerX = this.restartBtn.centerX;
            restartBtnText.centerY = this.restartBtn.centerY - paddingY;
            restartBtnText.angle = -4;
            const restartBtnLights = new Phaser.Sprite(game, 0, 0, 'big-btn-lights');
            restartBtnLights.width = this.restartBtn.width;
            restartBtnLights.scale.y = restartBtnLights.scale.x;
            restartBtnLights.centerX = this.restartBtn.centerX;
            restartBtnLights.centerY = this.restartBtn.centerY;
            restartBtnLights.animations.add('anticlockwise', [5, 4, 3, 2, 1, 0], 15, true);
            restartBtnLights.animations.play('anticlockwise');
            this.continueBtn = new Phaser.Image(game, 0, 0, 'big-btn-right-clean');
            this.continueBtn.width = this.restartBtn.width;
            this.continueBtn.scale.y = this.continueBtn.scale.x;
            this.continueBtn.top = this.restartBtn.top;
            this.continueBtn.left = bannerFooter.width * 0.51;
            this.continueBtn.inputEnabled = true;
            this.continueBtn.events.onInputDown.add(() => {
                this.game.sound.play('snd_ui_click');
                this.onBtnClick(true);
            }, this);
            const continueBtnText = new Phaser.Text(game, 0, 0, LocalizationManager_3.LocalizationManager.Instance.text("resume.yes").toUpperCase(), bigBtnTextStyle);
            continueBtnText.anchor.set(0.5);
            continueBtnText.centerX = this.continueBtn.centerX;
            continueBtnText.centerY = this.continueBtn.centerY - paddingY;
            continueBtnText.angle = 5;
            const continueBtnLights = new Phaser.Sprite(game, 0, 0, 'big-btn-lights');
            continueBtnLights.width = this.continueBtn.width;
            continueBtnLights.scale.y = continueBtnLights.scale.x;
            continueBtnLights.scale.x = -continueBtnLights.scale.x;
            continueBtnLights.centerX = this.continueBtn.centerX;
            continueBtnLights.centerY = this.continueBtn.centerY;
            continueBtnLights.animations.add('anticlockwise', [5, 4, 3, 2, 1, 0], 15, true);
            continueBtnLights.animations.play('anticlockwise');
            this.bottomBanner.add(bannerFooter);
            this.bottomBanner.add(restartBtnLights);
            this.bottomBanner.add(this.restartBtn);
            this.bottomBanner.add(restartBtnText);
            this.bottomBanner.add(continueBtnLights);
            this.bottomBanner.add(this.continueBtn);
            this.bottomBanner.add(continueBtnText);
            this.add(this.bottomBanner);
            this.bottomBanner.top = game.world.height;
            this.bottomBanner.centerX = game.world.centerX;
            this.bottomBanner.visible = false;
            /**
             * Text box that fades and scales in
             */
            const dialogContent = new Phaser.Group(game);
            this.add(dialogContent);
            const dialogTextStyle = {
                font: Game_6.Game.DEFAULT_FONT,
                fontSize: TextUtil_5.TextUtil.getFontSize(this.game.width, 27),
                fill: 'white',
                align: 'center',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.75
            };
            let warningString = LocalizationManager_3.LocalizationManager.Instance.text("confirm_dialog.warning_resume");
            const warningText = new Phaser.Text(game, 0, 0, warningString, Object.assign({}, dialogTextStyle, { fontSize: TextUtil_5.TextUtil.getFontSize(this.game.width, 38), fill: Game_6.Game.TEXT_COLOR_YELLOW }));
            warningText.setShadow(Game_6.Game.TEXT_SHADOW_OFFSET_MED_X, Game_6.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_6.Game.TEXT_SHADOW_DIAG, 0, true, true);
            dialogContent.add(warningText);
            let playerNamesText = null;
            this.playerChanged = GameController_8.GameController.Instance.didPlayersChange();
            // Only show the names if they've changed 
            if (this.playerChanged) {
                let title = LocalizationManager_3.LocalizationManager.Instance.text('resume.title');
                const titleText = new Phaser.Text(game, 0, 0, title, dialogTextStyle);
                titleText.setShadow(Game_6.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_6.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_6.Game.TEXT_SHADOW_DIAG, 0, true, true);
                titleText.top = warningText.bottom;
                titleText.centerX = warningText.centerX;
                dialogContent.add(titleText);
                let playerNamesString = GameController_8.GameController.Instance.previousSessionPlayerNames.join("\n");
                playerNamesText = new Phaser.Text(game, 0, 0, playerNamesString, Object.assign({}, dialogTextStyle, { fill: Game_6.Game.TEXT_COLOR_YELLOW }));
                playerNamesText.setShadow(Game_6.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_6.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_6.Game.TEXT_SHADOW_DIAG, 0, true, true);
                playerNamesText.width = Math.min(playerNamesText.width, this.game.width * 0.855);
                playerNamesText.scale.y = playerNamesText.scale.x;
                playerNamesText.top = titleText.bottom;
                playerNamesText.centerX = titleText.centerX;
                dialogContent.add(playerNamesText);
            }
            let yPosition = playerNamesText === null ? warningText.bottom : playerNamesText.bottom;
            let confirmationQnText = LocalizationManager_3.LocalizationManager.Instance.text('resume.description');
            const confirmationQn = new Phaser.Text(game, 0, 0, confirmationQnText, dialogTextStyle);
            confirmationQn.setShadow(Game_6.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_6.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_6.Game.TEXT_SHADOW_DIAG, 0, true, true);
            confirmationQn.top = yPosition;
            confirmationQn.centerX = warningText.centerX;
            dialogContent.add(confirmationQn);
            this.textBox = new BasicCTADialog_2.BasicCTADialog(this.game);
            this.add(this.textBox);
            this.textBox.setCustomContent(dialogContent);
            this.textBox.centerX = game.world.centerX;
            this.textBox.centerY = game.world.height * 0.5;
            this.textBox.visible = false;
            // makes the entire dialog invisible until it is shown
            this.visible = false;
            TSMAnalyticsManager_3.TSMAnalyticsManager.Instance.onGameResumeState(this.playerChanged);
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.visible = true;
                this.shade.inputEnabled = true;
                this.game.add.tween(this.shade).to({ alpha: 1 }, 500, Phaser.Easing.Linear.None, true);
                yield this.showBanners();
                yield this.textBox.showScaling();
                this.restartBtn.inputEnabled = true;
                this.continueBtn.inputEnabled = true;
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                this.restartBtn.inputEnabled = false;
                this.continueBtn.inputEnabled = false;
                this.game.add.tween(this.shade).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true);
                yield this.textBox.hideScaling();
                yield this.hideBanners();
                this.shade.inputEnabled = false;
                this.visible = false;
            });
        }
        showBanners() {
            return __awaiter(this, void 0, void 0, function* () {
                this.bottomBanner.visible = true;
                this.topBanner.visible = true;
                // move top banner down
                this.game.add.tween(this.topBanner).to({ y: 0 }, 300, Phaser.Easing.Linear.None, true);
                // move bottom banner up
                yield PromiseUtil_8.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomBanner).to({ top: this.game.height - this.bannerFooterHeight }, 300, Phaser.Easing.Linear.None, true));
            });
        }
        hideBanners() {
            return __awaiter(this, void 0, void 0, function* () {
                // move top banner up
                this.game.add.tween(this.topBanner).to({ bottom: 0 }, 150, Phaser.Easing.Linear.None, true);
                // move bottom banner down
                yield PromiseUtil_8.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomBanner).to({ top: this.game.height }, 150, Phaser.Easing.Linear.None, true));
                this.topBanner.visible = false;
                this.bottomBanner.visible = false;
            });
        }
        onBtnClick(resume) {
            TSMAnalyticsManager_3.TSMAnalyticsManager.Instance.onGameResumed(resume, this.playerChanged);
            this.onButtonClickedSignal.dispatch(resume);
        }
        destroy() {
            this.onButtonClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.ResumeDialog = ResumeDialog;
});
define("switchback-common/util/SoundUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SoundUtil {
        /**
         * Add sounds loaded via loadSound or loadSoundAll will be prefixed by this path
         */
        static setRootSoundPath(value) {
            SoundUtil._rootSoundPath = value;
        }
        /**
         * @param tag Phaser SoundManager tag for this sound
         * @param filename Doesn't include the filename
         */
        static loadSound(load, tag, filename) {
            load.audio(tag, [SoundUtil._rootSoundPath + filename + ".m4a",
                SoundUtil._rootSoundPath + filename + ".ogg"]);
        }
        /**
         * Loads a sequence of sounds starting from 0 running to count (exclusive)
         *
         * @param tag Phaser SoundManager base tag for this sound (will have the index appended)
         * @param filename Doesn't include the filename
         * @param count How many sounds to load
         */
        static loadSoundAll(load, tag, filename, count) {
            for (let i = 0; i < count; ++i) {
                load.audio(tag + i, [SoundUtil._rootSoundPath + filename + i + ".m4a",
                    SoundUtil._rootSoundPath + filename + i + ".ogg"]);
            }
        }
        /**
         * Unloads a sequence of sounds starting from 0 running to count (exclusive)
         *
         * @param tag Phaser SoundManager base tag for this sound
         * @param count How many sounds to unload
         */
        static unloadSoundAll(cache, tag, count) {
            for (let i = 0; i < count; ++i) {
                cache.removeSound(tag + i);
            }
        }
        /**
         * Play a sound at random from a set of sounds with the same base tag
         *
         * @param soundTag Phaser SoundManager base tag for this sound
         * @param count How many sounds in this set
         */
        static playRandom(game, soundTag, count) {
            return game.sound.play(soundTag + Math.floor(Math.random() * count));
        }
        /**
         * Play a sound on a fixed interval
         *
         * @param soundTag Phaser SoundManager base tag for this sound
         * @param interval How freqently to play the sound
         * @param volume Volume to play the sound
         * @returns An id that is used to stop this loop from playing
         */
        static playSoundOnInterval(game, soundTag, interval, volume) {
            return setInterval(() => {
                game.sound.play(soundTag, volume);
            }, interval);
        }
        /**
         *
         * @param intervalId This id generated by playSoundOnInterval used to stop the loop
         */
        static stopSoundOnInterval(intervalId) {
            clearInterval(intervalId);
        }
        /**
         * Allows to resume the audio context. Specially after a focus loss.
         * This gets called automatically by PauseUtil, which is the preferred method.
         * Accessible here for the special cases where you want this functionality and skipping PauseUtil.
         */
        static resumeAudio(game) {
            // you can't re-enable the sounds until the user has had an input and
            // the sounds aren't already running
            game.input.onDown.addOnce(() => {
                try {
                    if (game.sound.context.state !== "running") {
                        game.sound.context.resume();
                    }
                }
                catch (e) {
                    console.log("[SoundUtil.resumeAudio] Error attempting to resume sounds.");
                }
            }, Number.MAX_SAFE_INTEGER);
        }
    }
    SoundUtil._rootSoundPath = "";
    exports.SoundUtil = SoundUtil;
});
define("switchback-common/util/PauseUtil", ["require", "exports", "switchback-common/util/SoundUtil"], function (require, exports, SoundUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PauseUtil {
        /*
         * Use this function to listen for when the game has resumed changes visibility.
         * On iOS backgrounding the game can kill sound context and you'll
         * permanently lose audio. This function resumes it.
         * Function should be hooked once as the game loads.
         *
         * This needs to be called as the last part on initialization.
         * As in Android the player management card will trigger 2 onBlur.
         */
        static addHooks(game) {
            console.log("[PauseUtil] addHooks");
            PauseUtil.removeHooks(game);
            PauseUtil._isBlurred = false;
            PauseUtil.onBlur = new Phaser.Signal();
            PauseUtil.onFocus = new Phaser.Signal();
            PauseUtil._onBlurListener = PauseUtil.onBlurred.bind(null, game);
            PauseUtil._onFocusListener = PauseUtil.onFocused.bind(null, game);
            // These two are the only ones reliable on ios and android
            // onBlur and onFocus are not triggering on android for some reason
            // also document.addEventListener("visibilitychange") seems to be reliable too
            game.onPause.add(PauseUtil._onBlurListener);
            game.onResume.add(PauseUtil._onFocusListener);
            // Make sure the audio gets activated. Because the engine might have been stopped before this hook gets called.
            // Or in new updates the engine always starts stopped.
            SoundUtil_1.SoundUtil.resumeAudio(game);
        }
        static removeHooks(game) {
            if (PauseUtil.onBlur) {
                PauseUtil.onBlur.dispose();
                PauseUtil.onBlur = null;
            }
            if (PauseUtil.onFocus) {
                PauseUtil.onFocus.dispose();
                PauseUtil.onFocus = null;
            }
            if (PauseUtil._onBlurListener) {
                game.onBlur.remove(this._onBlurListener, null);
            }
            if (PauseUtil._onFocusListener) {
                game.onFocus.remove(PauseUtil._onFocusListener, null);
            }
        }
        static onBlurred(game) {
            console.log("[PauseUtil] onBlur. isBlurred=", PauseUtil._isBlurred);
            if (PauseUtil._isBlurred) {
                return;
            }
            PauseUtil._isBlurred = true;
            PauseUtil.onBlur.dispatch();
        }
        static onFocused(game) {
            console.log("[PauseUtil] onFocus. isBlurred=", PauseUtil._isBlurred);
            if (!PauseUtil._isBlurred) {
                return;
            }
            PauseUtil._isBlurred = false;
            SoundUtil_1.SoundUtil.resumeAudio(game);
            PauseUtil.onFocus.dispatch();
        }
    }
    exports.PauseUtil = PauseUtil;
});
define("state/LoadingState", ["require", "exports", "component/dialog/ConfirmationDialog", "logic/GameController", "StateNames", "state/CutSceneState", "switchback-common/state/BaseState", "switchback-common/util/PromiseUtil", "switchback-common/localization/LocalizationManager", "component/dialog/ResumeDialog", "model/ToyStoryConfig", "logic/TSMAnalyticsManager", "switchback-common/util/PauseUtil"], function (require, exports, ConfirmationDialog_1, GameController_9, StateNames_7, CutSceneState_1, BaseState_2, PromiseUtil_9, LocalizationManager_4, ResumeDialog_1, ToyStoryConfig_4, TSMAnalyticsManager_4, PauseUtil_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class LoadingState extends BaseState_2.BaseState {
        init() {
            this.stateName = StateNames_7.StateNames.LOAD;
            super.init();
            this.scale.scaleMode = Phaser.ScaleManager.RESIZE;
            this.game.time.advancedTiming = true;
        }
        preload() {
            super.preload();
            this.game.load.json('game-info', 'assets/json/game-info.json');
            this.game.load.json("config", "assets/json/config.json");
            this.game.load.json("i18n", "assets/lang/en.json");
            this.game.load.json('poster-data', 'assets/json/poster.json');
            this.game.load.json('build-info', 'assets/json/build-info.json');
            this.game.load.atlasXML("feedback-star", "assets/common/star.png", "assets/common/star.xml");
            this.game.load.spritesheet('sancreek-digits', 'assets/font/SancreekDigits_baked.png', 164, 231);
            this.game.load.image('sancreek-colon', 'assets/font/SancreekDigits_colon.png');
            this.game.load.image('instruction-dialog-backplate-top', 'assets/dialog/CTA_dialog_top.png');
            this.game.load.image('instruction-dialog-backplate-mid', 'assets/dialog/CTA_dialog_mid.png');
            this.game.load.image('instruction-dialog-backplate-btm-noprompt', 'assets/dialog/CTA_dialog_btm_noprompt.png');
            this.game.load.image('banner-footer', 'assets/common/midway-footer.png');
            this.game.load.image('banner-header', 'assets/common/midway-header.png');
            this.game.load.image('big-btn-left-clean', 'assets/dialog/BigBtn_left_clean.png');
            this.game.load.image('big-btn-right-clean', 'assets/dialog/BigBtn_right_clean.png');
            this.game.load.spritesheet('big-btn-lights', 'assets/dialog/bigbtn_lights.png', 635, 640);
            this.game.load.image('stage-bg', 'assets/common/stage-bg.png');
            this.game.load.spritesheet('prompt-arrow', 'assets/dialog/prompt-arrow.png', 180, 170);
            this.game.load.audio('snd_ui_click', ['assets/sound/ui/ui_click.m4a', 'assets/sound/ui_click.ogg']);
            this.game.load.audio('snd_back', ['assets/sound/ui/back.m4a', 'assets/sound/back.ogg']);
            this.game.load.audio('typewriter_loop', ['assets/sound/ui/typewriter_loop.m4a', 'assets/sound/ui/typewriter_loop.ogg']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                // As soon as we possibly can, we should load up the config file
                ToyStoryConfig_4.ToyStoryConfig.Instance.parse(this.game.cache.getJSON("config"));
                if (ToyStoryConfig_4.ToyStoryConfig.Instance.enableFPS) {
                    this.game.time.advancedTiming = true;
                }
                if (!ToyStoryConfig_4.ToyStoryConfig.Instance.enableConsoleLogs) {
                    console.log = () => { };
                }
                // And the localization strings
                LocalizationManager_4.LocalizationManager.Instance.init("en", this.game.cache.getJSON("i18n"));
                /**
                 * Stage background that slide up after banners
                 */
                this.background = new Phaser.Image(this.game, 0, 0, 'stage-bg');
                this.background.width = this.game.width;
                this.background.scale.y = this.background.scale.x;
                this.background.top = this.game.height;
                this.add.existing(this.background);
                this.dismissOverlay();
                const gameController = GameController_9.GameController.Instance;
                // Initialise the game controller, then proceed to the menu
                yield gameController.init(this.game.cache.getJSON("game-info"));
                yield gameController.openPlayerManagement();
                gameController.achievementManager.initialise(gameController.players, `toyStoryMania${gameController.parkPrefix.toUpperCase()}`);
                // This fixes sound being lost on iOS
                PauseUtil_1.PauseUtil.addHooks(this.game);
                GameController_9.GameController.Instance.calculateDefaultTextSizes(this.game.width, this.game.height);
                // Initialise our analytics manager with all the specific info we have
                TSMAnalyticsManager_4.TSMAnalyticsManager.Instance.init(ToyStoryConfig_4.ToyStoryConfig.Instance.park, gameController.parkPrefix.toUpperCase(), gameController.hasPrimaryPlayer(), gameController.players.length, 0);
                this.stage.disableVisibilityChange = ToyStoryConfig_4.ToyStoryConfig.Instance.quickTest;
                if (!this.shouldShowResumePrompt()) {
                    // N.B. We end this timer before we start it. This is because a
                    // new game can be started here, which should end the
                    // timer on the previous game before starting a new one.
                    yield TSMAnalyticsManager_4.TSMAnalyticsManager.Instance.onFullGameTime(false);
                    TSMAnalyticsManager_4.TSMAnalyticsManager.Instance.onFullGameTime(true);
                    yield this.showOverlay();
                    this.chooseStateToStart();
                    return;
                }
                const confirmDialog = new ConfirmationDialog_1.ConfirmationDialog(this.game, LocalizationManager_4.LocalizationManager.Instance.text("resume.confirm_dialog.title").toUpperCase(), LocalizationManager_4.LocalizationManager.Instance.text("resume.confirm_dialog.text"), LocalizationManager_4.LocalizationManager.Instance.text("resume.confirm_dialog.no").toUpperCase(), LocalizationManager_4.LocalizationManager.Instance.text("resume.confirm_dialog.yes").toUpperCase(), false);
                const resumeDialog = new ResumeDialog_1.ResumeDialog(this.game);
                /**
                 * Top banner that descends down, needs to be on top of all the dialogs
                 */
                this.topBanner = new Phaser.Image(this.game, 0, 0, 'banner-header');
                this.topBanner.width = this.game.width;
                this.topBanner.scale.y = this.topBanner.scale.x;
                this.add.existing(this.topBanner);
                this.topBanner.bottom = 0;
                this.topBanner.visible = false;
                // Dialogs to be in front of the curtain
                this.add.existing(resumeDialog);
                this.add.existing(confirmDialog);
                resumeDialog.onButtonClickedSignal.add((resume) => __awaiter(this, void 0, void 0, function* () {
                    if (resume) {
                        resumeDialog.onButtonClickedSignal.removeAll();
                        confirmDialog.onButtonClickedSignal.removeAll();
                        yield this.showOverlay();
                        GameController_9.GameController.Instance.recalculateQuotas();
                        this.chooseStateToStart();
                    }
                    else {
                        yield resumeDialog.hide();
                        yield confirmDialog.show();
                    }
                }), this);
                confirmDialog.onButtonClickedSignal.add((confirmed) => __awaiter(this, void 0, void 0, function* () {
                    TSMAnalyticsManager_4.TSMAnalyticsManager.Instance.onGameResumeChoice(confirmed, GameController_9.GameController.Instance.didPlayersChange());
                    // GameController.Instance.state.numVisits++; // Track new start of the attraction // TODO verify
                    // TODO notice this happens after TSMAnalyticsManager.init, but then the trackAction above needs that
                    // the same happens on peter pan
                    if (confirmed) {
                        // N.B. We end this timer before we start it. This is because a
                        // new game can be started here, which should end the
                        // timer on the previous game before starting a new one.
                        yield TSMAnalyticsManager_4.TSMAnalyticsManager.Instance.onFullGameTime(false);
                        TSMAnalyticsManager_4.TSMAnalyticsManager.Instance.onFullGameTime(true);
                        resumeDialog.onButtonClickedSignal.removeAll();
                        confirmDialog.onButtonClickedSignal.removeAll();
                        // if they confirm to start a new game,
                        // clear the state and start the intro cutscene
                        yield Promise.all([
                            this.showOverlay(),
                            GameController_9.GameController.Instance.recycleState()
                        ]);
                        this.chooseStateToStart();
                    }
                    else {
                        yield confirmDialog.hide();
                        yield resumeDialog.show();
                    }
                }), this);
                yield this.showBackground();
                yield this.showTopBanner();
                yield resumeDialog.show();
            });
        }
        chooseStateToStart() {
            GameController_9.GameController.Instance.state.sessionStarted = true;
            // if they have already seen the intro, skip directly to hub
            if (GameController_9.GameController.Instance.state.ftueCompleted) {
                this.game.state.start(StateNames_7.StateNames.INTRO);
            }
            else {
                CutSceneState_1.CutSceneState.showCutScene(this.game, "intro", StateNames_7.StateNames.INTRO);
            }
        }
        /**
         * Returns true if they have a sessionStarted
         */
        shouldShowResumePrompt() {
            // If the players have a current session going, then we should show the dialog
            return GameController_9.GameController.Instance.state.sessionStarted;
        }
        // Background
        showBackground() {
            return __awaiter(this, void 0, void 0, function* () {
                this.background.visible = true;
                // move background up
                yield PromiseUtil_9.PromiseUtil.tween(this.game, this.game.add.tween(this.background).to({ bottom: this.game.height }, 400, Phaser.Easing.Linear.None, true));
            });
        }
        showTopBanner() {
            return __awaiter(this, void 0, void 0, function* () {
                this.topBanner.visible = true;
                // move top banner down
                this.game.add.tween(this.topBanner).to({ y: 0 }, 300, Phaser.Easing.Linear.None, true);
            });
        }
        shutdown() {
            this.background = null;
            this.topBanner = null;
            super.shutdown();
        }
    }
    exports.LoadingState = LoadingState;
});
define("state/OutroState", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "Game", "logic/GameController", "StateNames", "state/ToyStoryBaseState", "component/ArrowPrompt", "logic/TSMAnalyticsManager", "switchback-common/component/text/TypewriterText"], function (require, exports, PromiseUtil_10, TextUtil_6, ScaleUtil_2, LocalizationManager_5, Game_7, GameController_10, StateNames_8, ToyStoryBaseState_2, ArrowPrompt_2, TSMAnalyticsManager_5, TypewriterText_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     * @author nick@cerebralfix.com
     * @author jeronimo@cerebralfix.com (though mostly stolen from the intro)
     */
    class OutroState extends ToyStoryBaseState_2.ToyStoryBaseState {
        init() {
            this.stateName = StateNames_8.StateNames.OUTRO;
            super.init();
        }
        preload() {
            super.preload();
            this.game.load.image('outro-ride', 'assets/outro/ride_vehicle.png');
            this.game.load.image('outro-light', 'assets/outro/ride_light_effect.png');
            this.game.load.image('outro-bg', 'assets/outro/bg.jpg');
            this.game.load.image('close-game-btn', 'assets/common/exit-button.png');
            this.game.load.image('banner-header', 'assets/common/midway-header.png');
            this.game.load.image('title-tsm', 'assets/common/logo.png');
            this.game.load.image('cut-scene-bg', 'assets/cutscene/background.jpg');
            this.game.load.image('cut-scene-lights-back', 'assets/cutscene/lights_back.png');
            this.game.load.image('cut-scene-lights-front', 'assets/cutscene/lights_front.png');
            this.game.load.image('cut-scene-lights-strip', 'assets/cutscene/lights_strip.png');
            this.game.load.image('cut-scene-text-box-top', 'assets/cutscene/textbox_top.png');
            this.game.load.image('cut-scene-text-box-middle', 'assets/cutscene/textbox_middle.png');
            this.game.load.image('skip', 'assets/common/skip.png');
            this.game.load.image('character-dialog-woody', 'assets/dialog/character_dialog_woody.png');
        }
        create() {
            super.create();
            this.bg = new Phaser.Image(this.game, 0, 0, 'outro-bg');
            this.bg.width = this.game.width;
            this.bg.height = this.game.height;
            this.bg.inputEnabled = true;
            this.add.existing(this.bg);
            const rideLight = new Phaser.Image(this.game, 0, 0, 'outro-light');
            rideLight.width = this.game.width;
            rideLight.scale.y = rideLight.scale.x;
            this.add.existing(rideLight);
            const rideImage = new Phaser.Image(this.game, 0, 0, 'outro-ride');
            this.add.existing(rideImage);
            rideImage.width = this.game.width * 0.75;
            rideImage.scale.y = rideImage.scale.x;
            rideImage.centerX = this.game.width * 0.5;
            rideImage.y = this.game.height * 0.24;
            rideLight.centerX = rideImage.centerX;
            rideLight.top = this.game.height * -0.02;
            this.game.add.tween(rideImage).to({ top: rideImage.top * 0.75 }, 1500, Phaser.Easing.Quadratic.InOut, true, 0, Infinity, true);
            /**
             * It would be nice to have this as a component class
             * Top banner that descends down
             * Includes: banner, game title, and secret button
             */
            this.topBanner = new Phaser.Group(this.game);
            const bannerHeader = new Phaser.Image(this.game, 0, 0, 'banner-header');
            bannerHeader.width = this.game.width;
            bannerHeader.scale.y = bannerHeader.scale.x;
            let paddingY = this.game.world.height * 0.01;
            this.gameTitle = new Phaser.Image(this.game, 0, 0, 'title-tsm');
            this.gameTitle.width = bannerHeader.width * 0.5;
            this.gameTitle.scale.y = this.gameTitle.scale.x;
            this.gameTitle.centerX = bannerHeader.centerX;
            this.gameTitle.top = paddingY;
            this.topBanner.add(bannerHeader);
            this.topBanner.add(this.gameTitle);
            this.add.existing(this.topBanner);
            this.topBanner.top = 0;
            ///
            /// stuff at the bottom
            this.createBottomDialog();
            this.closeGameBtn = new Phaser.Image(this.game, 0, 0, 'close-game-btn');
            ScaleUtil_2.scaleToBounds(this.closeGameBtn, this.game.width * 0.15, this.game.height * 0.1);
            this.closeGameBtn.top = 0;
            this.closeGameBtn.right = this.game.width;
            this.closeGameBtn.inputEnabled = true;
            this.closeGameBtn.events.onInputDown.add(this.closeGame, this);
            this.closeGameBtn.visible = false;
            this.add.existing(this.closeGameBtn);
            /**
             * Disney asked us to remove the Load Instructions. I've left this here incase they change their mind back again.
             */
            // this.showDialogue(["outro.messages.1", "outro.messages.2", "outro.messages.3", "outro.messages.4", "outro.messages.5"]);
            this.showDialogue(["outro.messages.1"]);
            this.dismissOverlay();
        }
        closeGame() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play('snd_ui_click');
                yield GameController_10.GameController.Instance.closeGame();
            });
        }
        // Thebottom dialog is so complex 
        createBottomDialog() {
            this.portrait = new Phaser.Image(this.game, 0, 0, 'character-dialog-woody');
            this.portrait.height = this.game.height * 0.47;
            this.portrait.scale.x = -this.portrait.scale.y;
            this.portrait.left = this.game.width * 1.13;
            this.portrait.y = this.game.height * 0.3;
            this.add.existing(this.portrait);
            this.speechBox = new Phaser.Group(this.game);
            this.speechBox.visible = false;
            this.add.existing(this.speechBox);
            const textBoxTop = new Phaser.Image(this.game, 0, 0, 'cut-scene-text-box-top');
            textBoxTop.width = this.game.width;
            textBoxTop.scale.y = textBoxTop.scale.x;
            this.speechBox.add(textBoxTop);
            const textBoxMiddle = new Phaser.Image(this.game, 0, 0, 'cut-scene-text-box-middle');
            textBoxMiddle.width = this.game.width;
            textBoxMiddle.height = this.game.height * 0.37;
            textBoxMiddle.top = textBoxTop.bottom;
            this.speechBox.add(textBoxMiddle);
            this.text = new TypewriterText_2.TypewriterText(this.game, 0, 0, '', {
                font: Game_7.Game.DEFAULT_FONT,
                fontSize: TextUtil_6.TextUtil.getFontSize(this.game.width, 35),
                fill: 'white',
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.94
            });
            this.text.setShadow(Game_7.Game.TEXT_SHADOW_OFFSET_MED_X, Game_7.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_7.Game.TEXT_SHADOW_DIAG, 0, true, true);
            this.text.left = this.speechBox.width * 0.03;
            this.text.top = textBoxMiddle.top;
            this.speechBox.add(this.text);
            this.characterName = new Phaser.Text(this.game, 0, 0, '', {
                font: Game_7.Game.CARNIVAL_FONT,
                fontSize: TextUtil_6.TextUtil.getFontSize(this.game.width, 36),
                fill: Game_7.Game.TEXT_COLOR_CHAR_FILL,
                stroke: Game_7.Game.TEXT_COLOR_CHAR_STROKE,
                strokeThickness: this.speechBox.width * 0.01
            });
            this.speechBox.add(this.characterName);
            this.speechBox.bottom = this.game.height;
            const lightsBack = new Phaser.Image(this.game, 0, 0, 'cut-scene-lights-back');
            lightsBack.width = this.game.width;
            lightsBack.scale.y = lightsBack.scale.x;
            lightsBack.bottom = this.game.height;
            this.add.existing(lightsBack);
            this.lights = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height * 0.04, 'cut-scene-lights-strip');
            this.lights.tileScale.x = this.lights.width / this.lights.texture.width;
            this.lights.bottom = lightsBack.bottom;
            this.lights.autoScroll(this.game.width * 0.5, 0);
            this.lights.alpha = 0;
            this.add.existing(this.lights);
            const lightsFront = new Phaser.Image(this.game, 0, 0, 'cut-scene-lights-front');
            lightsFront.width = lightsBack.width;
            lightsFront.scale.y = lightsBack.scale.y;
            lightsFront.bottom = lightsBack.bottom;
            this.add.existing(lightsFront);
            this.prompt = new ArrowPrompt_2.ArrowPrompt(this.game);
            this.prompt.height = lightsBack.height * 0.7;
            this.prompt.scale.x = this.prompt.scale.y;
            this.prompt.right = this.game.width - (this.game.width * 0.005);
            this.prompt.bottom = this.game.height - (this.game.width * 0.01);
            this.prompt.visible = false;
            this.add.existing(this.prompt);
            const padding = this.game.width * 0.005;
            this.skipButton = new Phaser.Image(this.game, 0, 0, 'skip');
            this.skipButton.width = this.skipButton.height = this.game.width * 0.12;
            this.skipButton.top = padding;
            this.skipButton.right = this.game.width - padding;
            this.skipButton.inputEnabled = true;
            this.skipButton.events.onInputDown.addOnce(this.skip, this);
        }
        skip() {
            this.game.sound.play('snd_ui_click');
            this.text.isSkipping = true;
            this.skipButton.visible = false;
            if (this.skipPrompt) {
                this.skipPrompt();
            }
        }
        showDialogue(text) {
            return __awaiter(this, void 0, void 0, function* () {
                // Set character name
                this.characterName.text = 'Woody';
                this.characterName.fontSize = TextUtil_6.TextUtil.getFontSize(this.game.width, 36);
                TextUtil_6.TextUtil.scaleDownSingleLineTextToWidth(this.characterName, this.speechBox.width * 0.28);
                this.characterName.centerX = this.speechBox.width * 0.18;
                // note: can't set sized based on speechbox height as it varies with device
                this.characterName.centerY = this.speechBox.width * 0.14;
                // Clear text
                this.text.text = '';
                // Animate in
                this.speechBox.top = this.game.height;
                this.speechBox.visible = true;
                yield PromiseUtil_10.PromiseUtil.tween(this.game, this.game.add.tween(this.speechBox).to({
                    top: this.game.height * 0.565
                }, 600, Phaser.Easing.Cubic.Out).start());
                // Write out each page of dialog
                for (let i = 0; !this.text.isSkipping && i < text.length; i++) {
                    TSMAnalyticsManager_5.TSMAnalyticsManager.Instance.onLoadInstructionCutscene(i + 1, text.length);
                    this.text.text = LocalizationManager_5.LocalizationManager.Instance.text(text[i]);
                    this.text.fontSize = TextUtil_6.TextUtil.getFontSize(this.game.width, 35);
                    TextUtil_6.TextUtil.scaleDownTextToHeight(this.text, this.speechBox.height * 0.45);
                    yield this.text.typewriter(15, 'typewriter_loop', this.bg);
                    if (!this.text.isSkipping && i < text.length - 1) {
                        yield this.showPrompt();
                    }
                }
                this.closeGameBtn.visible = true;
                this.closeGameBtn.alpha = 0;
                this.game.add.tween(this.closeGameBtn).to({ alpha: 1 }, 750, Phaser.Easing.Linear.None, true);
            });
        }
        showPrompt() {
            return __awaiter(this, void 0, void 0, function* () {
                this.prompt.visible = true;
                this.prompt.alpha = 0;
                this.lights.visible = true;
                this.lights.alpha = 0;
                const promptFadeTime = 500;
                this.game.add.tween(this.prompt).to({ alpha: 1 }, promptFadeTime).start();
                yield PromiseUtil_10.PromiseUtil.tween(this.game, this.game.add.tween(this.lights).to({ alpha: 1 }, promptFadeTime).start());
                return new Promise((resolve, reject) => {
                    this.skipPrompt = () => __awaiter(this, void 0, void 0, function* () {
                        this.game.add.tween(this.prompt).to({ alpha: 0 }, promptFadeTime).start();
                        PromiseUtil_10.PromiseUtil.tween(this.game, this.game.add.tween(this.lights).to({ alpha: 0 }, promptFadeTime).start()).then(() => {
                            this.prompt.visible = false;
                            this.lights.visible = false;
                        });
                        this.bg.events.onInputDown.removeAll();
                        this.skipPrompt = null;
                        resolve();
                    });
                    this.bg.events.onInputDown.addOnce(this.skipPrompt);
                });
            });
        }
        shutdown() {
            this.topBanner = null;
            this.gameTitle = null;
            this.speechBox = null;
            this.text = null;
            this.characterName = null;
            this.prompt = null;
            this.lights = null;
            this.portrait = null;
            this.closeGameBtn = null;
            this.bg = null;
            this.skipButton = null;
            this.game.cache.removeImage('outro-ride');
            this.game.cache.removeImage('outro-light');
            this.game.cache.removeImage('outro-bg');
            this.game.cache.removeImage('close-game-btn');
            this.game.cache.removeImage('banner-header');
            this.game.cache.removeImage('title-tsm');
            this.game.cache.removeImage('cut-scene-bg');
            this.game.cache.removeImage('cut-scene-lights-back');
            this.game.cache.removeImage('cut-scene-lights-front');
            this.game.cache.removeImage('cut-scene-lights-strip');
            this.game.cache.removeImage('cut-scene-text-box-top');
            this.game.cache.removeImage('cut-scene-text-box-middle');
            this.game.cache.removeImage('skip');
            this.game.cache.removeImage('character-dialog-woody');
            super.shutdown();
        }
    }
    exports.OutroState = OutroState;
});
/**
 * @author ewan@cerebralfix.com
 */
define("minigame/flying-tosser/Hatch", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Hatch extends Phaser.Group {
        constructor(game, x, y, active) {
            super(game);
            this.occupied = false;
            this.hole = new Phaser.Sprite(this.game, x, y, 'hole', 0);
            this.add(this.hole);
        }
        activateHatch() {
            this.lights = new Phaser.Image(this.game, this.hole.centerX, this.hole.bottom, 'hatch-glowing');
            this.lights.animations.add('loop', [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13], 8, true).play();
            this.lights.width = this.hole.width * 2.5;
            this.lights.scale.y = this.lights.scale.x;
            this.lights.anchor.set(0.49, 0.89);
            this.add(this.lights);
            // enable the body for this hatch
            this.game.physics.enable(this.hole);
            this.hole.body.setSize(this.hole.width * 0.4, this.hole.height * 0.4, this.hole.width * 0.3, this.hole.height * 0.3);
            this.hole.body.immovable = true;
            this.hole.frame = 1;
        }
        turnOffLights() {
            this.game.add.tween(this.lights).to({ alpha: 0 }, 250, Phaser.Easing.Linear.None, true);
        }
        /**
         * Change the texture of the hatch to reflect it's correct
         * state (inactive).
         */
        deactivateHatch() {
            this.hole.frame = 0;
            if (this.lights) {
                this.lights.destroy();
            }
        }
        get hatchCenter() {
            return new Phaser.Point(this.hole.worldPosition.x + (this.hole.width * 0.5), this.hole.worldPosition.y + (this.hole.height * 0.5));
        }
        set isOccupied(b) {
            this.occupied = b;
        }
        get isOccupied() {
            return this.occupied;
        }
    }
    exports.Hatch = Hatch;
});
define("minigame/flying-tosser/FlyingTossersGameController", ["require", "exports", "logic/GameController", "switchback-common/util/CollectionUtil", "model/ToyStoryConfig", "StateNames"], function (require, exports, GameController_11, CollectionUtil_2, ToyStoryConfig_5, StateNames_9) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@cerebralfix.com
     */
    var GameState;
    (function (GameState) {
        GameState[GameState["LOADING"] = 0] = "LOADING";
        GameState[GameState["SWITCHING"] = 1] = "SWITCHING";
        GameState[GameState["ENTRANCE"] = 2] = "ENTRANCE";
        GameState[GameState["PLAYING"] = 3] = "PLAYING";
        GameState[GameState["OVER"] = 4] = "OVER";
    })(GameState = exports.GameState || (exports.GameState = {}));
    /**
     * This class is used to construct objects in the game
     */
    class LevelData {
    }
    exports.LevelData = LevelData;
    class FlyingTossersGameController {
        constructor(json) {
            this.currentRoundIndex = 0; // index of the current round
            this.roundsCompleted = 0;
            this.roundTimeTakenMs = 0;
            this._gameState = GameState.LOADING;
            const regularLevels = json.levels;
            const tutorialLevel = json.tutorial;
            // Get if they've played it before
            this._isTutorial = GameController_11.GameController.Instance.state.getMinigameSaveData(StateNames_9.StateNames.GAME_FLYING_TOSSERS).numPlays === 0;
            this._wasTutorialPlay = this._isTutorial;
            this.playerSession = GameController_11.GameController.Instance.playerManager.getPlayerSession(2, GameController_11.GameController.Instance.players.length, true);
            this.levels = regularLevels;
            this.timeRemaining = 15000 * GameController_11.GameController.Instance.players.length;
            // Shuffle the levels unless we're trying to test them in order
            if (!ToyStoryConfig_5.ToyStoryConfig.Instance.testAllBuzzLevels) {
                Phaser.ArrayUtils.shuffle(this.levels);
            }
            if (ToyStoryConfig_5.ToyStoryConfig.Instance.testAllBuzzLevels) {
                this.timeRemaining = 3600000;
            }
            if (this._isTutorial) {
                // Add the tutorial
                this.levels.unshift(tutorialLevel);
                // The first players go twice (tutorial, then real level)
                this.playerSession.unshift(this.playerSession[0]);
            }
        }
        /*
            Returns the list of the current players
        */
        getCurrentPlayers() {
            return this.playerSession[this.currentRoundIndex % GameController_11.GameController.Instance.players.length];
        }
        getRoundsCompleted() {
            return this.roundsCompleted;
        }
        get hasEveryonePlayed() {
            // when the tutorial is played the first two guests play two levels, a tutorial one and a regular one
            let roundsToPlay = (this._wasTutorialPlay ? GameController_11.GameController.Instance.players.length + 1 : GameController_11.GameController.Instance.players.length);
            return this.roundsCompleted >= roundsToPlay;
        }
        get isTutorial() {
            return this._isTutorial;
        }
        get wasTutorialEnabled() {
            return this._wasTutorialPlay;
        }
        getCurrentRound() {
            return this.currentRoundIndex;
        }
        getCurrentLevel() {
            return this.levels[this.currentRoundIndex];
        }
        nextLevel() {
            this.currentRoundIndex++;
            this.roundsCompleted++;
            // If they've completed a level, they must have at least completed the tutorial.
            if (this.currentRoundIndex > 0) {
                this._isTutorial = false;
            }
            if (this.currentRoundIndex > this.levels.length) {
                this.levels = CollectionUtil_2.shuffleNonRepeatingArray(this.levels);
                this.currentRoundIndex = 0;
            }
        }
        set gameState(state) {
            if (state === GameState.LOADING) {
                return;
            }
            this._gameState = state;
        }
        get gameState() {
            return this._gameState;
        }
        // Returns true if there is still enough time remaining
        enoughTimeForNextLevel() {
            return this.timeRemaining > FlyingTossersGameController.MIN_TIME_TO_START_LEVEL_MS;
        }
    }
    FlyingTossersGameController.MIN_TIME_TO_START_LEVEL_MS = 4000;
    exports.FlyingTossersGameController = FlyingTossersGameController;
});
define("minigame/flying-tosser/Magnet", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_11) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@cerebralfix.com
     */
    class Magnet extends Phaser.Group {
        constructor(game, ID, magnetButton) {
            super(game);
            this.SHAKE_OFFSET_X = 5;
            this.SHAKE_OFFSET_Y = 5;
            this.pressedSignal = new Phaser.Signal();
            this.canShake = false;
            this._ID = ID;
            this._isDown = false;
            this.button = magnetButton;
            this.button.animations.add('down', [0, 1, 2], 18);
            this.button.animations.add('up', [2, 1, 0], 18);
            this.button.inputEnabled = true;
            this.magnet = new Phaser.Image(game, 0, 0, "magnet-head", 0);
            this.magnet.anchor.set(0, 1);
            this.magnetOutPosition = new Phaser.Point(this.game.width * 0.075, -this.game.height * 0.075);
            this.ray = new Phaser.Image(game, 0, 0, "magnet-ray");
            this.ray.anchor.set(0, 1);
            this.ray.width = this.game.width * 0.75;
            this.ray.scale.y = this.ray.scale.x;
            this.ray.alpha = 0;
            this.ray.visible = false;
            this.add(this.ray);
            this.add(this.magnet);
            // The magnet rays are always tweening, we just toggle their visibility later
            const durationMs = 500;
            const rayScaleTween = this.game.add.tween(this.ray.scale).to({ x: 0, y: 0 }, durationMs, Phaser.Easing.Linear.None, true, 0, Infinity);
            rayScaleTween.pause();
            const rayAlphaTween = this.game.add.tween(this.ray).to({ alpha: 1 }, durationMs, Phaser.Easing.Exponential.Out, true, 0, Infinity);
            rayAlphaTween.pause();
            this.rayTweens = [rayScaleTween, rayAlphaTween];
            this.button.events.onInputDown.add(this.onButtonPressed, this);
            this.button.events.onInputUp.add(this.onButtonReleased, this);
        }
        onButtonPressed() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play('magnet-click', 0.5);
                this.pressedSignal.dispatch(true);
                this.button.animations.stop();
                this.button.animations.play('down');
                this._isDown = true;
                this.activateMagnet();
            });
        }
        onButtonReleased() {
            return __awaiter(this, void 0, void 0, function* () {
                this._isDown = false;
                this.pressedSignal.dispatch(false);
                this.button.animations.stop();
                this.button.animations.play('up');
                this.deactivateMagnet();
            });
        }
        /**
         * Tweens the magnet out and shakes the magnet
         * while the the button is held down
         */
        activateMagnet() {
            return __awaiter(this, void 0, void 0, function* () {
                this.magnet.frame = 1;
                this.ray.visible = true;
                this.rayTweens.forEach(tween => tween.resume());
                this.game.tweens.removeFrom(this.magnet);
                yield PromiseUtil_11.PromiseUtil.tween(this.game, this.game.add.tween(this.magnet)
                    .to({ x: this.magnetOutPosition.x, y: this.magnetOutPosition.y }, 150, Phaser.Easing.Linear.None, true));
                this.canShake = true;
            });
        }
        update() {
            super.update();
            if (this.canShake) {
                const randX = this.game.rnd.integerInRange(this.magnetOutPosition.x + this.SHAKE_OFFSET_X, this.magnetOutPosition.x - this.SHAKE_OFFSET_X);
                const randY = this.game.rnd.integerInRange(this.magnetOutPosition.y + this.SHAKE_OFFSET_Y, this.magnetOutPosition.y - this.SHAKE_OFFSET_Y);
                this.magnet.position.set(randX, randY);
            }
        }
        /**
         * Retracts the magnet to its original
         * position once the button has been released
         */
        deactivateMagnet() {
            return __awaiter(this, void 0, void 0, function* () {
                this.magnet.frame = 0;
                this.ray.visible = false;
                this.rayTweens.forEach(tween => tween.pause());
                this.canShake = false;
                this.game.tweens.removeFrom(this.magnet);
                yield PromiseUtil_11.PromiseUtil.tween(this.game, this.game.add.tween(this.magnet)
                    .to({ x: 0, y: 0 }, 150, Phaser.Easing.Linear.None, true));
            });
        }
        get isDown() {
            return this._isDown;
        }
        get ID() {
            return this._ID;
        }
        disable() {
            this.button.inputEnabled = false;
            this._isDown = false;
            this.deactivateMagnet();
            this.button.animations.play('down').onComplete.addOnce(() => {
                this.button.tint = 0x333333;
            });
        }
        enable() {
            this.button.animations.play('up');
            this.button.inputEnabled = true;
            this.button.tint = 0xFFFFFF;
        }
        destroy() {
            this.rayTweens.forEach(tween => tween.stop());
            this.rayTweens = null;
            this.pressedSignal.removeAll();
            if (this.button) {
                this.button.events.onInputDown.remove(this.onButtonPressed, this);
                this.button.events.onInputUp.remove(this.onButtonReleased, this);
                this.button = null;
            }
            super.destroy();
        }
    }
    exports.Magnet = Magnet;
});
/**
 * @author ewan@cerebralfix.com
 */
define("minigame/flying-tosser/Alien", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Alien extends Phaser.Sprite {
        constructor(game, x, y) {
            super(game, x, y);
            /* Constants for spinning animation */
            this.MIN_SPIN_DELAY = 60;
            this.MAX_SPIN_DELAY = 100;
            this.RANGE_SPIN_DELAY = this.MIN_SPIN_DELAY + this.MAX_SPIN_DELAY;
            this.IDLE_SPIN_DELAY = 150;
            // create the alien
            this._alien = new Phaser.Sprite(game, 0, 0, "aliens", 0);
            this._alien.width = this.game.width * 0.125;
            this._alien.scale.y = this._alien.scale.x;
            // create the platform
            this.platform = new Phaser.Sprite(game, 0, 0, "platform");
            this.platform.width = this._alien.width * 1.5;
            this.platform.scale.y = this.platform.scale.x;
            this.platform.centerX = this._alien.centerX;
            // create the beam
            this.beam = new Phaser.Image(game, 0, 0, "platform-glow", 0);
            this.beam.width = this.platform.width * 1.9;
            this.beam.scale.y = this.beam.scale.x;
            this.beam.centerX = this.platform.centerX;
            this.beam.alpha = 0;
            this.addChild(this.beam);
            this.addChild(this.platform);
            this.addChild(this._alien);
            this.platform.centerY = this._alien.bottom + (this.platform.height * 0.05);
            this.beam.bottom = this.platform.bottom + this.platform.height * 0.15;
            // setup the spinning animation
            this.spinAnimation = this._alien.animations.add('spin');
            this._weight = game.rnd.realInRange(0.4, 0.9);
            game.physics.enable(this);
            this.body.mass = 1;
            this.body.drag = new Phaser.Point(100 * this.body.mass, 100 * this.body.mass);
            this.body.collideWorldBounds = true;
            this.body.bounce.set(0.2); // set horizontal and vertical vectors to 1. Value of 1 means 100% energy return
            this.body.setSize(this._alien.width * 0.6, this.platform.height * 0.5, this._alien.width * 0.2, this._alien.height * 0.55);
            this._hatchCollision = new Phaser.Sprite(game, 0, 0, '');
            this._hatchCollision.width = this.body.width * 0.15;
            this._hatchCollision.height = this.body.height * 0.15;
            this._hatchCollision.centerX = this.platform.centerX;
            this._hatchCollision.centerY = this.platform.centerY;
            game.physics.enable(this._hatchCollision);
            this.addChild(this._hatchCollision);
        }
        update() {
            super.update();
            let positiveVelocity = new Phaser.Point(this.body.velocity.x, this.body.velocity.y).getMagnitude();
            positiveVelocity = Phaser.Math.clamp(positiveVelocity, this.MIN_SPIN_DELAY, this.MAX_SPIN_DELAY);
            // flip the velocity values (f(x) = this.RANGE_SPIN_DELAY - x) so that the higher the velocity, the lower the delay
            // this is because Phaser.Animation.delay works in ms and higher velocity means slower frame rate
            const frameDelay = this.RANGE_SPIN_DELAY - positiveVelocity;
            // if the sprite is in motion, play the spinning animation
            if ((this.body.velocity.y !== 0) || (this.body.velocity.x !== 0)) {
                this.spinAnimation.delay = frameDelay;
                if (!this.spinAnimation.isPlaying) {
                    this.spinAnimation.play(frameDelay, true);
                }
            }
            else { // if the alien is idle, pause the animation
                this.spinAnimation.delay = this.IDLE_SPIN_DELAY;
            }
        }
        activate() {
            this.game.add.tween(this.beam).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true);
        }
        /**
         * Stops the spinning animation from playing
         */
        deactivate() {
            this.spinAnimation.stop();
            this.game.add.tween(this.beam).to({ alpha: 0 }, 250, Phaser.Easing.Linear.None, true);
        }
        /**
         * This function updates the platform's texture
         * from default to magnetized and vice versa
         *
         * @param shouldMagnetize - whether we should load the magnetize texture
         */
        magnetize(shouldMagnetize) {
            this.platform.frame = shouldMagnetize ? 1 : 0;
            this.beam.frame = shouldMagnetize ? 1 : 0;
        }
        get weight() {
            return this._weight;
        }
        get alienBase() {
            return new Phaser.Point(this._alien.worldPosition.x + (this._alien.width * 0.5), this._alien.worldPosition.y + (this._alien.height * 0.8));
        }
        get alien() {
            return this._alien;
        }
        get hatchCollision() {
            return this._hatchCollision;
        }
        get alienCenter() {
            return new Phaser.Point(this.x + (this._alien.width * 0.5), this.y + (this._alien.height * 0.5));
        }
    }
    exports.Alien = Alien;
});
define("minigame/flying-tosser/VerticalWall", ["require", "exports", "minigame/flying-tosser/FlyingTossersState"], function (require, exports, FlyingTossersState_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@cerebralfix.com
     */
    var ObstacleType;
    (function (ObstacleType) {
        ObstacleType[ObstacleType["VERTICAL"] = 0] = "VERTICAL";
        ObstacleType[ObstacleType["SINGLE"] = 1] = "SINGLE";
    })(ObstacleType || (ObstacleType = {}));
    class VerticalWall extends Phaser.Sprite {
        constructor(game, x, y, length) {
            super(game, x, y);
            this.xOriginal = y * game.width * FlyingTossersState_1.FlyingTossersState.TILE_WIDTH;
            this.yOriginal = game.height * FlyingTossersState_1.FlyingTossersState.GRID_STARTING_Y + (x * game.height * FlyingTossersState_1.FlyingTossersState.TILE_HEIGHT);
            const mod = length % 2;
            let totalWidth = game.width * FlyingTossersState_1.FlyingTossersState.TILE_WIDTH;
            let totalHeight = game.height * FlyingTossersState_1.FlyingTossersState.TILE_HEIGHT * length;
            let i = 0;
            while (i < length - mod) {
                this.createWall(i, ObstacleType.VERTICAL);
                i += 2;
            }
            // create the remainder of the aliens (1 or 2) for vertical walls
            if (mod > 0) {
                for (let j = 0; j < mod; j++) {
                    this.createWall(i, ObstacleType.SINGLE);
                    i++;
                }
            }
            game.physics.enable(this);
            this.body.immovable = true;
            this.body.setSize(totalWidth, totalHeight, 0, this.game.height * FlyingTossersState_1.FlyingTossersState.TILE_HEIGHT);
            this.x = this.xOriginal;
            this.y = this.yOriginal;
        }
        /**
         * Creates and adds a child wall to this sprite
         */
        createWall(i, obstacleType) {
            const randomFrame = this.game.rnd.integerInRange(0, 5);
            let wall;
            switch (obstacleType) {
                case ObstacleType.VERTICAL: {
                    wall = new Phaser.Sprite(this.game, 0, this.game.height * FlyingTossersState_1.FlyingTossersState.TILE_HEIGHT * (i + 0.3), "obstacleVertical", randomFrame);
                    wall.height = this.game.height * FlyingTossersState_1.FlyingTossersState.TILE_HEIGHT * VerticalWall.VERTICAL_HEIGHT_SCALE;
                    break;
                }
                case ObstacleType.SINGLE:
                default: {
                    wall = new Phaser.Sprite(this.game, 0, this.game.height * FlyingTossersState_1.FlyingTossersState.TILE_HEIGHT * (i + 0.3), "singleObstacle", randomFrame);
                    wall.height = this.game.height * FlyingTossersState_1.FlyingTossersState.TILE_HEIGHT * VerticalWall.SINGLE_HEIGHT_SCALE;
                    break;
                }
            }
            wall.animations.add('idle', [0, 1, 2, 3, 4, 5], this.game.rnd.integerInRange(8, 12), true).play();
            wall.scale.x = wall.scale.y;
            wall.centerX -= (wall.width - this.game.width * FlyingTossersState_1.FlyingTossersState.TILE_WIDTH) * 0.5;
            this.addChild(wall);
        }
    }
    VerticalWall.VERTICAL_HEIGHT_SCALE = 2.7;
    VerticalWall.SINGLE_HEIGHT_SCALE = 1.7;
    exports.VerticalWall = VerticalWall;
});
define("minigame/flying-tosser/HorizontalWall", ["require", "exports", "minigame/flying-tosser/FlyingTossersState"], function (require, exports, FlyingTossersState_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@cerebralfix.com
     */
    var ObstacleType;
    (function (ObstacleType) {
        ObstacleType[ObstacleType["HORIZONTAL"] = 0] = "HORIZONTAL";
        ObstacleType[ObstacleType["SINGLE"] = 1] = "SINGLE";
    })(ObstacleType || (ObstacleType = {}));
    class HorizontalWall extends Phaser.Sprite {
        constructor(game, x, y, length) {
            super(game, x, y);
            this.xOriginal = y * game.width * FlyingTossersState_2.FlyingTossersState.TILE_WIDTH; // this will be the x offset for the body
            this.yOriginal = game.height * FlyingTossersState_2.FlyingTossersState.GRID_STARTING_Y + (x * game.height * FlyingTossersState_2.FlyingTossersState.TILE_HEIGHT); // this value is used for creating the body (offsetY)
            const totalWidth = game.width * FlyingTossersState_2.FlyingTossersState.TILE_WIDTH * length;
            const totalHeight = game.height * FlyingTossersState_2.FlyingTossersState.TILE_HEIGHT;
            const mod = length % 2;
            let i = 0;
            while (i < length - mod) {
                this.createWall(i, ObstacleType.HORIZONTAL);
                i += 2;
            }
            if (mod > 0) {
                this.createWall(i, ObstacleType.SINGLE);
            }
            game.physics.enable(this);
            this.body.immovable = true;
            this.body.setSize(totalWidth, totalHeight, 0, this.game.height * FlyingTossersState_2.FlyingTossersState.TILE_HEIGHT);
            this.x = this.xOriginal;
            this.y = this.yOriginal;
        }
        /**
         * Creates and adds a child wall to this sprite
         */
        createWall(i, obstacleType) {
            const randomFrame = this.game.rnd.integerInRange(0, 5);
            let obstacleWidth;
            let wall;
            switch (obstacleType) {
                case ObstacleType.HORIZONTAL: {
                    wall = new Phaser.Sprite(this.game, this.game.width * FlyingTossersState_2.FlyingTossersState.TILE_WIDTH * i, this.game.height * FlyingTossersState_2.FlyingTossersState.TILE_HEIGHT * 0.3, "obstacleHorizontal", randomFrame);
                    wall.height = this.game.height * FlyingTossersState_2.FlyingTossersState.TILE_HEIGHT * HorizontalWall.HEIGHT_SCALE;
                    obstacleWidth = 2;
                    break;
                }
                case ObstacleType.SINGLE:
                default: {
                    wall = new Phaser.Sprite(this.game, this.game.width * FlyingTossersState_2.FlyingTossersState.TILE_WIDTH * i, this.game.height * FlyingTossersState_2.FlyingTossersState.TILE_HEIGHT * 0.3, "singleObstacle", randomFrame);
                    wall.height = this.game.height * FlyingTossersState_2.FlyingTossersState.TILE_HEIGHT * HorizontalWall.HEIGHT_SCALE;
                    obstacleWidth = 1;
                    break;
                }
            }
            wall.animations.add('idle', [0, 1, 2, 3, 4, 5], this.game.rnd.integerInRange(8, 12), true).play();
            wall.scale.x = wall.scale.y;
            wall.centerX -= (wall.width - this.game.width * FlyingTossersState_2.FlyingTossersState.TILE_WIDTH * obstacleWidth) * 0.5;
            this.addChild(wall);
        }
    }
    HorizontalWall.HEIGHT_SCALE = 1.7;
    exports.HorizontalWall = HorizontalWall;
});
define("minigame/flying-tosser/ObstacleFactory", ["require", "exports", "minigame/flying-tosser/VerticalWall", "minigame/flying-tosser/HorizontalWall"], function (require, exports, VerticalWall_1, HorizontalWall_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@cerebralfix.com
     */
    var ObstacleType;
    (function (ObstacleType) {
        ObstacleType[ObstacleType["VERTICAL_WALL"] = 0] = "VERTICAL_WALL";
        ObstacleType[ObstacleType["HORIZONTAL_WALL"] = 1] = "HORIZONTAL_WALL";
    })(ObstacleType = exports.ObstacleType || (exports.ObstacleType = {}));
    class ObstacleFactory {
        static makeObstacle(game, type, length, row, col) {
            switch (type) {
                case ObstacleType.VERTICAL_WALL: {
                    return new VerticalWall_1.VerticalWall(game, row, col, length);
                }
                case ObstacleType.HORIZONTAL_WALL: {
                    return new HorizontalWall_1.HorizontalWall(game, row, col, length);
                }
                default: {
                    break;
                }
            }
        }
    }
    exports.ObstacleFactory = ObstacleFactory;
});
define("component/hub/HubFooter", ["require", "exports", "Game"], function (require, exports, Game_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HubFooter extends Phaser.Group {
        constructor(game, imageTag, textString) {
            super(game);
            const bgPanel = new Phaser.Image(this.game, 0, 0, 'hub-footer-panel');
            bgPanel.width = this.game.width;
            bgPanel.scale.y = bgPanel.scale.x;
            bgPanel.inputEnabled = true;
            bgPanel.input.pixelPerfectClick = true;
            this.add(bgPanel);
            const posterIcon = new Phaser.Image(this.game, 0, bgPanel.height, imageTag);
            posterIcon.anchor.set(0, 1);
            posterIcon.scale.set(bgPanel.scale.x);
            posterIcon.inputEnabled = true;
            posterIcon.input.pixelPerfectClick = true;
            this.add(posterIcon);
            const text = new Phaser.Text(this.game, bgPanel.width * 0.64, bgPanel.height * 0.6, textString, {
                font: Game_8.Game.DEFAULT_FONT,
                fill: Game_8.Game.TEXT_COLOR_WHITE,
                fontSize: Game_8.Game.FONT_SIZE_MEDIUM,
                wordWrap: true,
                wordWrapWidth: bgPanel.width * 0.65
            });
            text.setShadow(Game_8.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_8.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_8.Game.TEXT_SHADOW_HARD, 0, true, true);
            text.lineSpacing = text.fontSize * -0.2;
            text.anchor.set(0.5);
            this.add(text);
        }
    }
    exports.HubFooter = HubFooter;
});
define("component/button/BigButton", ["require", "exports", "Game", "switchback-common/util/TextUtil"], function (require, exports, Game_9, TextUtil_7) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class BigButton extends Phaser.Group {
        constructor(game, labelString, buttonKey, disabledKey) {
            super(game);
            this.inputEnableChildren = true;
            this.enabledKey = buttonKey;
            this.disabledKey = disabledKey || buttonKey;
            this.button = new Phaser.Image(this.game, 0, 0, this.enabledKey);
            this.frame = new Phaser.Image(this.game, 0, 0, 'button-big-frame');
            this.backplate = new Phaser.Image(this.game, 0, 0, 'button-big-backplate');
            this.label = new Phaser.Text(this.game, this.frame.centerX, this.frame.height * 0.8, labelString, {
                font: Game_9.Game.DEFAULT_FONT,
                fill: Game_9.Game.TEXT_COLOR_WHITE,
                stroke: Game_9.Game.TEXT_COLOR_CHAR_STROKE,
                strokeThickness: Game_9.Game.TEXT_STROKE_THICKNESS_LARGE,
                fontSize: Game_9.Game.FONT_SIZE_LARGE,
                align: 'center'
            });
            TextUtil_7.TextUtil.scaleDownSingleLineTextToWidth(this.label, this.frame.width * 0.51);
            this.label.anchor.set(0.5);
            this.add(this.backplate);
            this.add(this.button);
            this.add(this.frame);
            this.add(this.label);
            this.onChildInputDown.add(this.press, this);
        }
        disable() {
            this.ignoreChildInput = true;
            this.stopPulse();
            this.button.loadTexture(this.disabledKey);
            this.label.fill = Game_9.Game.TEXT_COLOR_DISABLED;
            this.label.stroke = Game_9.Game.TEXT_COLOR_STROKE_DISABLED;
        }
        enable() {
            this.button.loadTexture(this.enabledKey);
            this.startPulse();
            this.ignoreChildInput = false;
            this.label.fill = Game_9.Game.TEXT_COLOR_WHITE;
            this.label.stroke = Game_9.Game.TEXT_COLOR_CHAR_STROKE;
        }
        stopPulse() {
            this.game.tweens.removeFrom(this.button);
            this.button.y = 0;
        }
        startPulse() {
            this.stopPulse(); // ensure the animation doesnt play twice no matter how this is called
            this.game.add.tween(this.button)
                .to({ y: this.button.y + this.frame.height * 0.05 }, 200, Phaser.Easing.Sinusoidal.InOut, true, 1600, 0, true)
                .onComplete.addOnce(this.startPulse, this);
        }
        set text(text) {
            this.label.text = text;
            this.label.fontSize = Game_9.Game.FONT_SIZE_LARGE;
            TextUtil_7.TextUtil.scaleDownSingleLineTextToWidth(this.label, this.frame.width * 0.51);
        }
        press() {
            this.game.tweens.removeFrom(this.button);
            this.button.y = 0;
            this.game.add.tween(this.button)
                .to({ y: this.button.y + this.frame.height * 0.1 }, 400, Phaser.Easing.Back.Out, true, 0, 0, true);
        }
    }
    exports.BigButton = BigButton;
});
define("component/dialog/WayfindingDialog", ["require", "exports", "component/button/BigButton", "switchback-common/localization/LocalizationManager", "Game", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "component/dialog/ConfirmationDialog"], function (require, exports, BigButton_1, LocalizationManager_6, Game_10, ScaleUtil_3, PromiseUtil_12, ConfirmationDialog_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class WayfindingDialog extends Phaser.Group {
        constructor(game, imageTag, text, isInstallation) {
            super(game);
            this.responseSignal = new Phaser.Signal();
            this.isInstallation = isInstallation;
            this.overlay = new Phaser.Graphics(this.game);
            this.overlay.beginFill(0x000000, 0.75);
            this.overlay.drawRect(0, 0, this.game.width, this.game.height);
            this.overlay.endFill();
            this.overlay.visible = false;
            this.overlay.inputEnabled = true;
            this.wayfindingImage = new Phaser.Image(this.game, this.game.width * 0.5, 0, imageTag);
            this.wayfindingImage.anchor.set(0.5, 0);
            ////////////////////////////////////
            // GROUPS
            ///////////////////////////////////
            this.topUIGroup = new Phaser.Group(this.game);
            this.bottomUIGroup = new Phaser.Group(this.game);
            ////////////////////////////////////
            // HEADER
            ///////////////////////////////////
            this.headerBack = new Phaser.Image(this.game, 0, 0, 'ui-top-panel');
            this.headerBack.width = this.game.width;
            this.headerBack.height = this.game.height * 0.225;
            const headerText = new Phaser.Text(this.game, this.game.width * 0.5, this.headerBack.height * 0.55, text, {
                font: Game_10.Game.DEFAULT_FONT,
                fill: Game_10.Game.TEXT_COLOR_WHITE,
                fontSize: Game_10.Game.FONT_SIZE_MEDIUM,
                wordWrap: true,
                wordWrapWidth: this.headerBack.width * 0.6,
                align: 'center'
            });
            headerText.lineSpacing = headerText.fontSize * -0.15;
            headerText.anchor.set(0.5);
            headerText.setShadow(Game_10.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_10.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_10.Game.TEXT_SHADOW_DIAG, 0, true, true);
            ////////////////////////////////////
            // FOOTER
            ///////////////////////////////////
            this.footerBack = new Phaser.Image(this.game, 0, this.game.height, 'ui-bottom-panel');
            this.footerBack.anchor.set(0, 1);
            this.footerBack.width = this.game.width;
            this.footerBack.height = this.game.height * 0.2;
            ////////////////////////////////////
            // YES BUTTON
            ///////////////////////////////////
            this.yesButton = new BigButton_1.BigButton(this.game, LocalizationManager_6.LocalizationManager.Instance.text("common.yes"), 'button-green-next');
            this.yesButton.width = this.game.width * 0.45;
            this.yesButton.scale.y = this.yesButton.scale.x;
            this.yesButton.left = this.game.width * 0.5;
            this.yesButton.bottom = this.game.height * 0.99;
            ////////////////////////////////////
            // NO BUTTON
            ///////////////////////////////////
            this.noButton = new BigButton_1.BigButton(this.game, LocalizationManager_6.LocalizationManager.Instance.text("common.no"), 'button-orange-replay');
            this.noButton.width = this.game.width * 0.4;
            this.noButton.scale.y = this.noButton.scale.x;
            this.noButton.right = this.game.width * 0.475;
            this.noButton.bottom = this.game.height * 0.99;
            ////////////////////////////////////
            // ORDERING
            ///////////////////////////////////
            this.add(this.overlay);
            this.add(this.wayfindingImage);
            this.add(this.topUIGroup);
            this.add(this.bottomUIGroup);
            this.topUIGroup.add(this.headerBack);
            this.topUIGroup.add(headerText);
            this.bottomUIGroup.add(this.footerBack);
            this.bottomUIGroup.add(this.yesButton);
            this.bottomUIGroup.add(this.noButton);
            ////////////////////////////////////
            // ADDITIONAL POSITIONING
            ///////////////////////////////////
            this.topUIGroup.bottom = 0;
            this.bottomUIGroup.top = this.game.height;
            ScaleUtil_3.scaleToCover(this.wayfindingImage, this.game.width, this.game.height - this.footerBack.height - (this.headerBack.height * 0.5));
            this.wayfindingImage.top = this.game.height;
            ////////////////////////////////////
            // CONFIGURE RESPONSE DISPATCHERS
            ///////////////////////////////////
            this.yesButton.onChildInputDown.add(() => {
                this.game.sound.play('snd_ui_click');
                if (this.isInstallation) {
                    // when showing installation wayfinder show additional confirmation that this will end their game
                    const confirmDialog = new ConfirmationDialog_2.ConfirmationDialog(this.game, LocalizationManager_6.LocalizationManager.Instance.text("installation_confirm_dialog.title"), LocalizationManager_6.LocalizationManager.Instance.text("installation_confirm_dialog.warning"), LocalizationManager_6.LocalizationManager.Instance.text("installation_confirm_dialog.no"), LocalizationManager_6.LocalizationManager.Instance.text("installation_confirm_dialog.yes"), true);
                    confirmDialog.onButtonClickedSignal.add((confirmed) => __awaiter(this, void 0, void 0, function* () {
                        yield confirmDialog.hide();
                        this.responseSignal.dispatch(confirmed);
                    }), this);
                    confirmDialog.show();
                }
                else {
                    this.responseSignal.dispatch(true);
                }
            }, this);
            this.noButton.onChildInputDown.add(() => {
                this.game.sound.play('snd_back');
                this.responseSignal.dispatch(false);
            }, this);
        }
        reset() {
            this.topUIGroup.bottom = 0;
            this.bottomUIGroup.top = this.game.height;
            this.wayfindingImage.top = this.game.height;
            this.overlay.alpha = 0;
            this.overlay.visible = false;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.reset();
                this.overlay.visible = true;
                yield Promise.all([
                    PromiseUtil_12.PromiseUtil.tween(this.game, this.game.add.tween(this.overlay)
                        .to({ alpha: 1 }, 500, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_12.PromiseUtil.tween(this.game, this.game.add.tween(this.topUIGroup)
                        .to({ top: 0 }, 350, Phaser.Easing.Linear.None, true, 200)),
                    PromiseUtil_12.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomUIGroup)
                        .to({ bottom: this.game.height }, 350, Phaser.Easing.Linear.None, true, 200)),
                    PromiseUtil_12.PromiseUtil.tween(this.game, this.game.add.tween(this.wayfindingImage)
                        .to({ bottom: this.game.height * 0.8 }, 350, Phaser.Easing.Linear.None, true, 200)),
                ]);
                const response = yield PromiseUtil_12.PromiseUtil.signal(this.responseSignal);
                yield PromiseUtil_12.PromiseUtil.time(this.game, 200);
                return response;
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                yield Promise.all([
                    PromiseUtil_12.PromiseUtil.tween(this.game, this.game.add.tween(this.topUIGroup)
                        .to({ bottom: 0 }, 350, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_12.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomUIGroup)
                        .to({ top: this.game.height }, 350, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_12.PromiseUtil.tween(this.game, this.game.add.tween(this.wayfindingImage)
                        .to({ top: this.game.height }, 350, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_12.PromiseUtil.tween(this.game, this.game.add.tween(this.overlay)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                ]);
                this.overlay.visible = false;
            });
        }
        destroy() {
            this.responseSignal.removeAll();
            super.destroy();
        }
    }
    exports.WayfindingDialog = WayfindingDialog;
});
define("component/dialog/PosterReminderDialog", ["require", "exports", "component/dialog/BasicCTADialog", "component/button/BigButton", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "model/ToyStoryConfig", "switchback-common/enum/Park"], function (require, exports, BasicCTADialog_3, BigButton_2, LocalizationManager_7, ScaleUtil_4, ToyStoryConfig_6, Park_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PosterReminderDialog extends BasicCTADialog_3.BasicCTADialog {
        constructor(game) {
            super(game);
            this.overlay = new Phaser.Graphics(this.game, 0, 0);
            this.overlay.beginFill(0x000000, 0.5);
            this.overlay.drawRect(0, 0, this.game.width, this.game.height);
            this.overlay.inputEnabled = true;
            this.overlay.visible = true;
            this.dialogClosedSignal = new Phaser.Signal();
            this.largeArrow = new Phaser.Image(game, 0, 0, "arrow_large");
            ScaleUtil_4.scaleToBounds(this.largeArrow, game.width * 0.35, game.height * 0.25);
            this.largeButton = new BigButton_2.BigButton(game, LocalizationManager_7.LocalizationManager.Instance.text('poster_reminder.btn_continue'), 'button-green-next');
            ScaleUtil_4.scaleToBounds(this.largeButton, game.width * 0.5, game.height * 0.125);
            this.largeButton.centerX = this.backplate.centerX;
            this.largeButton.bottom = this.backplate.bottom;
            this.add(this.largeButton);
            this.largeButton.onChildInputDown.add(() => {
                this.game.sound.play('snd_ui_click');
                this.hide();
            }, this);
            let textBody = "";
            if (ToyStoryConfig_6.ToyStoryConfig.Instance.park === Park_6.Park.DLR) {
                textBody = LocalizationManager_7.LocalizationManager.Instance.text('poster_reminder.dialog_body_posters');
            }
            else if (ToyStoryConfig_6.ToyStoryConfig.Instance.park === Park_6.Park.WDW) {
                textBody = LocalizationManager_7.LocalizationManager.Instance.text('poster_reminder.dialog_body_jail');
            }
            else { // Fall back to DLR if park is undefined.
                textBody = LocalizationManager_7.LocalizationManager.Instance.text('poster_reminder.dialog_body_posters');
            }
            this.setTextContent(LocalizationManager_7.LocalizationManager.Instance.text('poster_reminder.dialog_title'), textBody);
        }
        refit() {
            super.refit();
            this.largeButton.centerX = this.backplate.centerX;
            this.largeButton.bottom = this.backplate.bottom;
        }
        get DialogClosedSignal() {
            return this.dialogClosedSignal;
        }
        show() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                yield _super("show").call(this);
                this.overlay.x = -this.x; // Move overlay to the top left
                this.overlay.y = -this.y;
                this.largeArrow.left = this.game.width * 0.25 - this.x;
                this.largeArrow.bottom = this.game.height * 0.9 - this.game.width * 0.2 - this.y;
                this.addAt(this.largeArrow, 0);
                this.addAt(this.overlay, 0);
                this.animationTween = this.game.add.tween(this.largeArrow);
                this.animationTween.to({ x: this.largeArrow.x - this.width * 0.1,
                    y: this.largeArrow.y + this.width * 0.1 }, 300, Phaser.Easing.Linear.None, true, 0, -1, true);
            });
        }
        hide() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                if (this.animationTween != null) {
                    this.animationTween.stop();
                    this.animationTween = null;
                }
                this.remove(this.largeArrow);
                this.remove(this.overlay);
                yield _super("hide").call(this);
                this.dialogClosedSignal.dispatch();
            });
        }
        destroy() {
            this.largeButton.onChildInputDown.removeAll();
            this.dialogClosedSignal.removeAll();
            super.destroy();
        }
    }
    exports.PosterReminderDialog = PosterReminderDialog;
});
define("component/Curtain", ["require", "exports", "switchback-common/util/PromiseUtil", "component/hub/HubFooter", "model/ToyStoryConfig", "switchback-common/enum/Park", "component/dialog/WayfindingDialog", "switchback-common/localization/LocalizationManager", "state/CutSceneState", "StateNames", "logic/GameController", "component/dialog/PosterReminderDialog", "component/dialog/ConfirmationDialog", "logic/TSMAnalyticsManager"], function (require, exports, PromiseUtil_13, HubFooter_1, ToyStoryConfig_7, Park_7, WayfindingDialog_1, LocalizationManager_8, CutSceneState_2, StateNames_10, GameController_12, PosterReminderDialog_1, ConfirmationDialog_3, TSMAnalyticsManager_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Curtain extends Phaser.Group {
        constructor(game, footerStartsShowing = false) {
            super(game);
            this.canClickInstallationButton = true;
            // TODO: The installation at WDW has been disabled until it is installed
            if (ToyStoryConfig_7.ToyStoryConfig.Instance.park === Park_7.Park.WDW && !ToyStoryConfig_7.ToyStoryConfig.Instance.useWDWInstallation) {
                footerStartsShowing = false;
            }
            this.curtainRight = new Phaser.Image(this.game, 0, 0, 'cta-curtain');
            this.curtainRight.height = this.game.height;
            this.curtainRight.width = this.game.width * 0.5;
            this.curtainRight.top = 0;
            this.curtainRight.left = this.game.width * 0.5;
            this.add(this.curtainRight);
            this.curtainLeft = new Phaser.Image(this.game, 0, 0, 'cta-curtain');
            this.curtainLeft.height = this.game.height;
            this.curtainLeft.width = this.game.width * 0.5;
            this.curtainLeft.scale.x *= -1;
            this.curtainLeft.top = 0;
            this.curtainLeft.left = this.game.width * 0.5;
            this.add(this.curtainLeft);
            let footerStringKey = 'poster.hub_description';
            let footerImageTag = 'poster-icon';
            if (ToyStoryConfig_7.ToyStoryConfig.Instance.park === Park_7.Park.WDW) {
                footerStringKey = "jail.hub_description";
                footerImageTag = "jail-icon";
            }
            this.footer = new HubFooter_1.HubFooter(this.game, footerImageTag, LocalizationManager_8.LocalizationManager.Instance.text(footerStringKey));
            if (footerStartsShowing) {
                this.footer.bottom = this.game.height;
            }
            else {
                this.footer.top = this.game.height;
            }
            this.add(this.footer);
            this.footer.onChildInputDown.add(this.showWayfindDialog, this);
            this.curtainsShowing = true;
            this.clickBlocker = new Phaser.Graphics(this.game, 0, 0);
            this.clickBlocker.beginFill(0x330000, 0);
            this.clickBlocker.drawRect(0, 0, this.game.width, this.game.height);
            this.clickBlocker.inputEnabled = false;
            this.clickBlocker.visible = false;
        }
        get isShowing() {
            return this.curtainsShowing;
        }
        showWayfindDialog() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.canClickInstallationButton) {
                    return;
                }
                this.canClickInstallationButton = false;
                let hasReachedInstallation = GameController_12.GameController.Instance.state.hasEnteredPosterRoom;
                if (hasReachedInstallation) {
                    // warn the guest they are leaving the mini-game
                    const gameInfo = GameController_12.GameController.Instance.gameInfo.get(GameController_12.GameController.Instance.currentGameState);
                    const gameName = LocalizationManager_8.LocalizationManager.Instance.text(gameInfo.titleCharacterKey) + "\n" +
                        LocalizationManager_8.LocalizationManager.Instance.text(gameInfo.titleTextKey);
                    // have increased the opasity of the shade so that you can't see the animated ready button behind
                    const confirmDialog = new ConfirmationDialog_3.ConfirmationDialog(this.game, LocalizationManager_8.LocalizationManager.Instance.text("minigame_exit_confirm_dialog.title"), LocalizationManager_8.LocalizationManager.Instance.textWithReplacements("minigame_exit_confirm_dialog.warning", { game_name: gameName }), LocalizationManager_8.LocalizationManager.Instance.text("minigame_exit_confirm_dialog.no"), LocalizationManager_8.LocalizationManager.Instance.text("minigame_exit_confirm_dialog.yes"), false, 1);
                    confirmDialog.onButtonClickedSignal.add((confirmed) => __awaiter(this, void 0, void 0, function* () {
                        if (confirmed) {
                            this.game.sound.play('snd_ui_click');
                            confirmDialog.onButtonClickedSignal.dispose();
                            this.transitionToInstallation();
                        }
                        else {
                            this.game.sound.play('snd_back');
                            confirmDialog.hide();
                        }
                    }), this);
                    this.parent.add(confirmDialog);
                    yield confirmDialog.show();
                    this.canClickInstallationButton = true;
                    return;
                }
                // ask if they've reached the installation room
                let wayfindStringKey = 'poster.have_you_entered';
                let wayfindImageTag = 'poster-room-wayfind';
                let isInstallation = false;
                if (ToyStoryConfig_7.ToyStoryConfig.Instance.park === Park_7.Park.WDW) {
                    wayfindStringKey = "jail.can_you_see";
                    wayfindImageTag = "jail-wayfind";
                    isInstallation = true;
                }
                if (!this.installationWayfindingDialog) {
                    this.installationWayfindingDialog = new WayfindingDialog_1.WayfindingDialog(this.game, wayfindImageTag, LocalizationManager_8.LocalizationManager.Instance.text(wayfindStringKey), isInstallation);
                    this.parent.add(this.installationWayfindingDialog);
                }
                this.parent.bringToTop(this.installationWayfindingDialog);
                hasReachedInstallation = yield this.installationWayfindingDialog.show();
                GameController_12.GameController.Instance.state.hasEnteredPosterRoom = hasReachedInstallation;
                TSMAnalyticsManager_6.TSMAnalyticsManager.Instance.onPosterRoomWayfind(hasReachedInstallation);
                if (hasReachedInstallation) {
                    this.transitionToInstallation();
                }
                else {
                    yield this.installationWayfindingDialog.hide();
                    yield this.showPosterReminder();
                }
                this.canClickInstallationButton = true;
            });
        }
        transitionToInstallation() {
            if (ToyStoryConfig_7.ToyStoryConfig.Instance.park === Park_7.Park.WDW) {
                CutSceneState_2.CutSceneState.showCutScene(this.game, 'pete_caught', StateNames_10.StateNames.HUB);
            }
            else if (ToyStoryConfig_7.ToyStoryConfig.Instance.park === Park_7.Park.DLR) {
                // walk back a step as this game wasn't completed, it has been interrupted by the poster game
                GameController_12.GameController.Instance.state.forcedProgressionStep--;
                CutSceneState_2.CutSceneState.showCutScene(this.game, "posters_intro", StateNames_10.StateNames.POSTER);
            }
        }
        showPosterReminder() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.isShowing) {
                    if (!this.posterReminderDialog) {
                        this.posterReminderDialog = new PosterReminderDialog_1.PosterReminderDialog(this.game);
                        this.posterReminderDialog.centerX = this.game.width * 0.5;
                        this.posterReminderDialog.centerY = this.game.height * 0.6;
                    }
                    if (this.parent) {
                        this.parent.addChild(this.clickBlocker);
                        this.parent.addChild(this.posterReminderDialog);
                    }
                    this.clickBlocker.inputEnabled = true;
                    this.clickBlocker.visible = true;
                    yield this.posterReminderDialog.show();
                    // Need to move the footer to the front layer so the overlay doesn't cover it.
                    if (this.parent) {
                        this.remove(this.footer);
                        this.parent.addChild(this.footer);
                    }
                    yield PromiseUtil_13.PromiseUtil.signal(this.posterReminderDialog.DialogClosedSignal);
                    if (this.parent) {
                        this.parent.removeChild(this.posterReminderDialog);
                        this.parent.removeChild(this.footer);
                        this.addChild(this.footer);
                        this.parent.removeChild(this.clickBlocker);
                    }
                    this.clickBlocker.inputEnabled = false;
                    this.clickBlocker.visible = false;
                }
            });
        }
        show(showFooter = false) {
            return __awaiter(this, void 0, void 0, function* () {
                // TODO: Temporary change. Stinky Pete has been removed at WDW until the installation is installed.
                if (ToyStoryConfig_7.ToyStoryConfig.Instance.park === Park_7.Park.WDW && !ToyStoryConfig_7.ToyStoryConfig.Instance.useWDWInstallation) {
                    showFooter = false;
                }
                this.game.add.tween(this.curtainLeft).to({ left: this.game.width * 0.5 }, 500, Phaser.Easing.Sinusoidal.In, true);
                yield PromiseUtil_13.PromiseUtil.tween(this.game, this.game.add.tween(this.curtainRight)
                    .to({ left: this.game.width * 0.5 }, 500, Phaser.Easing.Sinusoidal.In, true));
                if (showFooter) {
                    yield PromiseUtil_13.PromiseUtil.tween(this.game, this.game.add.tween(this.footer)
                        .to({ bottom: this.game.height }, 350, Phaser.Easing.Sinusoidal.In, true));
                    this.canClickInstallationButton = true;
                }
                this.curtainsShowing = true;
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                this.canClickInstallationButton = false;
                this.game.add.tween(this.curtainLeft).to({ left: 0 }, 500, Phaser.Easing.Sinusoidal.Out, true);
                yield Promise.all([
                    PromiseUtil_13.PromiseUtil.tween(this.game, this.game.add.tween(this.curtainRight)
                        .to({ left: this.game.width }, 500, Phaser.Easing.Sinusoidal.Out, true)),
                    PromiseUtil_13.PromiseUtil.tween(this.game, this.game.add.tween(this.footer)
                        .to({ top: this.game.height }, 350, Phaser.Easing.Sinusoidal.Out, true))
                ]);
                this.curtainsShowing = false;
            });
        }
        destroy() {
            if (this.clickBlocker) {
                this.clickBlocker.destroy(); // May not be on screen at the time so wont be picked up in super.destroy();
                this.clickBlocker = null;
            }
            if (this.posterReminderDialog) {
                this.posterReminderDialog.destroy();
                this.posterReminderDialog = null;
            }
            if (this.footer) {
                this.footer.onChildInputDown.removeAll();
                this.footer.destroy();
                this.footer = null;
            }
            super.destroy();
        }
    }
    exports.Curtain = Curtain;
});
define("component/MessageBar", ["require", "exports", "Game", "switchback-common/util/TextUtil"], function (require, exports, Game_11, TextUtil_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MessageBar extends Phaser.Group {
        constructor(game, text = "", fontSize = 40) {
            super(game);
            fontSize = TextUtil_8.TextUtil.getFontSize(game.width, fontSize);
            const height = this.game.height * 0.15;
            this.tilingBackground = new Phaser.TileSprite(this.game, 0, 0, this.game.width, height, 'tiling-bar');
            const tileScale = height / this.tilingBackground.texture.height;
            this.tilingBackground.tileScale.set(tileScale, tileScale);
            this.add(this.tilingBackground);
            this.instructionText = new Phaser.Text(this.game, this.game.width * 0.5, this.tilingBackground.height * 0.48, text, {
                font: Game_11.Game.DEFAULT_FONT,
                fontSize: fontSize,
                fill: Game_11.Game.TEXT_COLOR_WHITE,
                stroke: Game_11.Game.TEXT_COLOR_BLUE,
                strokeThickness: Game_11.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.96,
                align: 'center'
            });
            this.instructionText.anchor.set(0.5);
            this.instructionText.lineSpacing = this.instructionText.fontSize * -0.3;
            this.add(this.instructionText);
            TextUtil_8.TextUtil.scaleDownTextToHeight(this.instructionText, this.tilingBackground.height * 0.88);
            this.tilingBackground.autoScroll(-this.game.width * 0.25, 0);
        }
        set text(text) {
            this.instructionText.text = text;
            TextUtil_8.TextUtil.scaleDownTextToHeight(this.instructionText, this.tilingBackground.height * 0.88);
        }
        set fontSize(size) {
            this.instructionText.fontSize = size;
            TextUtil_8.TextUtil.scaleDownTextToHeight(this.instructionText, this.tilingBackground.height * 0.88);
        }
    }
    exports.MessageBar = MessageBar;
    class MessageBarWithContent extends Phaser.Group {
        constructor(game, content) {
            super(game);
            const height = content.height * 1.3;
            const tilingBackground = new Phaser.TileSprite(this.game, 0, 0, this.game.width, height, 'tiling-bar');
            const tileScale = height / tilingBackground.texture.height;
            tilingBackground.tileScale.set(tileScale, tileScale);
            this.add(tilingBackground);
            tilingBackground.autoScroll(-this.game.width * 0.25, 0);
            this.add(content);
            content.position.set(this.game.width * 0.5, height * 0.5);
        }
    }
    exports.MessageBarWithContent = MessageBarWithContent;
    class MessageBarWithHeader extends Phaser.Group {
        constructor(game, headerString, subtextString) {
            super(game);
            const contentGroup = new Phaser.Group(this.game);
            const headerText = new Phaser.Text(this.game, 0, 0, headerString, {
                font: Game_11.Game.DEFAULT_FONT,
                fontSize: Game_11.Game.FONT_SIZE_EXTRA_LARGE,
                fill: Game_11.Game.TEXT_COLOR_YELLOW,
                stroke: Game_11.Game.TEXT_COLOR_BLUE,
                strokeThickness: Game_11.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                align: 'center'
            });
            headerText.anchor.set(0.5, 0.9);
            TextUtil_8.TextUtil.scaleDownSingleLineTextToWidth(headerText, this.game.width * 0.95);
            contentGroup.add(headerText);
            const subText = new Phaser.Text(this.game, 0, 0, subtextString, {
                font: Game_11.Game.DEFAULT_FONT,
                fontSize: Game_11.Game.FONT_SIZE_LARGE,
                fill: Game_11.Game.TEXT_COLOR_WHITE,
                stroke: Game_11.Game.TEXT_COLOR_BLUE,
                strokeThickness: Game_11.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                align: 'center'
            });
            subText.anchor.set(0.5, 0.1);
            TextUtil_8.TextUtil.scaleDownSingleLineTextToWidth(subText, this.game.width * 0.95);
            contentGroup.add(subText);
            const containingBar = new MessageBarWithContent(this.game, contentGroup);
            this.add(containingBar);
        }
    }
    exports.MessageBarWithHeader = MessageBarWithHeader;
    class LandscapeMessageBar extends Phaser.Group {
        constructor(game, text = "", fontSize = 50) {
            super(game);
            this.originalFontSize = fontSize = TextUtil_8.TextUtil.getFontSize(game.width, fontSize);
            const height = this.game.width * 0.2;
            this.tilingBackground = new Phaser.TileSprite(this.game, 0, 0, this.game.height, height, 'tiling-bar');
            const tileScale = height / this.tilingBackground.texture.height;
            this.tilingBackground.tileScale.set(tileScale, tileScale);
            this.add(this.tilingBackground);
            this.instructionText = new Phaser.Text(this.game, this.game.height * 0.5, this.tilingBackground.height * 0.48, text, {
                font: Game_11.Game.DEFAULT_FONT,
                fontSize: fontSize,
                fill: Game_11.Game.TEXT_COLOR_WHITE,
                stroke: Game_11.Game.TEXT_COLOR_BLUE,
                strokeThickness: Game_11.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                align: 'center'
            });
            this.instructionText.anchor.set(0.5);
            this.add(this.instructionText);
            TextUtil_8.TextUtil.scaleDownSingleLineTextToWidth(this.instructionText, this.game.height * 0.95);
            this.tilingBackground.autoScroll(-this.game.width * 0.25, 0);
            this.angle = -90;
        }
        set text(text) {
            this.instructionText.text = text;
        }
        setTextWidth(width) {
            this.instructionText.fontSize = this.originalFontSize;
            TextUtil_8.TextUtil.scaleDownSingleLineTextToWidth(this.instructionText, width);
        }
    }
    exports.LandscapeMessageBar = LandscapeMessageBar;
});
define("component/button/TextArrowButton", ["require", "exports", "Game", "switchback-common/util/TextUtil", "switchback-common/util/ScaleUtil"], function (require, exports, Game_12, TextUtil_9, ScaleUtil_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TextArrowButton extends Phaser.Group {
        constructor(game, text, fontSize = 35) {
            super(game);
            const background = new Phaser.Image(this.game, 0, 0, 'next-button');
            ScaleUtil_5.scaleToBounds(background, game.width * 0.6, game.height * 0.15);
            background.anchor.set(0.5);
            this.add(background);
            background.inputEnabled = true;
            this.textField = new Phaser.Text(this.game, -background.width * 0.125, -background.height * 0.05, text, {
                font: Game_12.Game.DEFAULT_FONT,
                fontSize: TextUtil_9.TextUtil.getFontSize(game.width, fontSize),
                wordWrap: true,
                wordWrapWidth: background.width * 0.75,
                fill: Game_12.Game.TEXT_COLOR_WHITE
            });
            this.textField.anchor.set(0.5);
            this.add(this.textField);
        }
        set text(text) {
            this.textField.text = text;
        }
        startPulse() {
            this.origScale = this.scale.x;
            this.game.add.tween(this.scale).to({ x: this.origScale * 1.2, y: this.origScale * 1.2 }, 650, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
        }
    }
    exports.TextArrowButton = TextArrowButton;
});
define("minigame/flying-tosser/Claw", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Claw extends Phaser.Group {
        constructor(game) {
            super(game);
            this.claw = new Phaser.Sprite(this.game, 0, 0, 'claw');
            this.claw.anchor.set(0.25, 0.5);
            this.claw.width = this.game.width * 0.2;
            this.claw.scale.y = this.claw.scale.x;
            this.add(this.claw);
            const cordWidth = this.claw.width * 0.03;
            const clawCord = new Phaser.TileSprite(this.game, this.claw.left + this.claw.width * 0.49, this.claw.top, cordWidth, this.game.height, 'claw-cord');
            const tileScale = cordWidth / clawCord.texture.height;
            clawCord.tileScale.set(tileScale, tileScale);
            clawCord.bottom = this.claw.top;
            this.add(clawCord);
        }
        open() {
            this.claw.frame = 1;
        }
        close() {
            this.claw.frame = 0;
        }
    }
    exports.Claw = Claw;
});
define("minigame/flying-tosser/ButtonPressTutorialPrompt", ["require", "exports", "Game", "switchback-common/util/PromiseUtil", "switchback-common/localization/LocalizationManager"], function (require, exports, Game_13, PromiseUtil_14, LocalizationManager_9) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ButtonPressTutorialPrompt extends Phaser.Group {
        constructor(game) {
            super(game);
            const window = new Phaser.Image(this.game, 0, 0, 'prompt-window');
            window.anchor.set(0.5, 0.9);
            this.add(window);
            this.holdAnim = new Phaser.Sprite(this.game, 0, -window.height * 0.1, 'hold-prompt');
            this.holdAnim.animations.add('loop', [0, 1, 2, 3, 4, 4, 4, 3, 2, 1, 0], 18, true);
            this.holdAnim.anchor.set(0.5, 1);
            this.holdAnim.height = window.height * 0.8;
            this.holdAnim.scale.x = this.holdAnim.scale.y;
            this.holdAnim.visible = false;
            this.add(this.holdAnim);
            this.releaseAnim = new Phaser.Sprite(this.game, 0, -window.height * 0.1, 'release-prompt');
            this.releaseAnim.animations.add('loop', [0, 1, 2, 3, 4, 5, 5, 5], 18, true);
            this.releaseAnim.anchor.set(0.5, 1);
            this.releaseAnim.height = window.height * 0.8;
            this.releaseAnim.scale.x = this.releaseAnim.scale.y;
            this.releaseAnim.visible = true;
            this.add(this.releaseAnim);
            this.instructionText = new Phaser.Text(this.game, 0, -window.height * 0.75, '', {
                font: Game_13.Game.DEFAULT_FONT,
                fontSize: 75,
                fill: Game_13.Game.TEXT_COLOR_WHITE,
                stroke: "#000000",
                strokeThickness: Game_13.Game.TEXT_STROKE_THICKNESS_EXTRA_LARGE,
                align: 'center'
            });
            this.instructionText.anchor.set(0.5);
            this.add(this.instructionText);
            this.isShowing = false;
            this.alpha = 0;
        }
        showHoldAnimation(textKey) {
            return __awaiter(this, void 0, void 0, function* () {
                this.releaseAnim.visible = false;
                this.releaseAnim.animations.stop();
                this.holdAnim.visible = true;
                this.holdAnim.animations.play('loop');
                this.instructionText.text = LocalizationManager_9.LocalizationManager.Instance.text(textKey);
                yield this.show();
            });
        }
        showReleaseAnimation() {
            return __awaiter(this, void 0, void 0, function* () {
                this.holdAnim.visible = false;
                this.holdAnim.animations.stop();
                this.releaseAnim.visible = true;
                this.releaseAnim.animations.play('loop');
                this.instructionText.text = LocalizationManager_9.LocalizationManager.Instance.text('buzz.release');
                yield this.show();
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.isShowing) {
                    this.isShowing = false;
                    this.game.tweens.removeFrom(this);
                    yield PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true));
                }
            });
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.isShowing) {
                    this.isShowing = true;
                    this.game.tweens.removeFrom(this);
                    yield PromiseUtil_14.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true));
                }
            });
        }
    }
    exports.ButtonPressTutorialPrompt = ButtonPressTutorialPrompt;
});
define("state/GameBaseState", ["require", "exports", "state/ToyStoryBaseState", "logic/GameController", "logic/TSMAnalyticsManager"], function (require, exports, ToyStoryBaseState_3, GameController_13, TSMAnalyticsManager_7) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author steven@cerebralfix.com
     */
    class GameBaseState extends ToyStoryBaseState_3.ToyStoryBaseState {
        constructor() {
            super(...arguments);
            this._trackFirstRound = true;
            this._trackMicroName = "";
            this._trackMicroFirstRound = true;
            this.MAX_TIMEOUT_STUCK_MS = Infinity;
        }
        init(...params) {
            super.init(params);
            this._trackFirstRound = true;
        }
        loadConfig(type) {
            const gameData = GameController_13.GameController.Instance.gameInfo.get(this.stateName);
            if (!gameData) {
                return;
            }
            const minigameSaveData = gameData.minigameSettings;
            if (!minigameSaveData) {
                return;
            }
            const config = new type();
            config.parse(minigameSaveData);
            return config;
        }
        loadDifficultyConfig(type) {
            const gameData = GameController_13.GameController.Instance.gameInfo.get(this.stateName);
            if (!gameData) {
                return;
            }
            const difficultyLevelsData = gameData.difficultyLevels;
            if (!difficultyLevelsData) {
                return;
            }
            // parse all of the difficulty levels for this game
            const difficultyLevels = this.parseDifficultySettings(difficultyLevelsData, type);
            // get the one difficulty settings we need to play this game right now
            const stageNum = GameController_13.GameController.Instance.state.forcedProgressionRound;
            return this.getDifficulty(difficultyLevels, stageNum);
        }
        parseDifficultySettings(gameData, type) {
            let difficultyLevels = new Array();
            let difficultyLevel;
            if (!gameData) {
                // default
                difficultyLevel = new type();
                difficultyLevel.parse({});
                difficultyLevels.push(difficultyLevel);
            }
            else {
                gameData.forEach((difficultyData) => {
                    difficultyLevel = new type();
                    difficultyLevel.parse(difficultyData);
                    difficultyLevels.push(difficultyLevel);
                });
            }
            return difficultyLevels;
        }
        /**
         * The difficulty increases by 1 with each round to a max difficulty
         *
         * @param roundIndex The round the guest is up to
         */
        getDifficulty(difficultyLevels, roundIndex) {
            const difficultyLevel = (roundIndex < difficultyLevels.length ? roundIndex : difficultyLevels.length - 1);
            return difficultyLevels[difficultyLevel];
        }
        transitionTo(state) {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.showOverlay();
                this.game.state.start(state);
            });
        }
        ///// Analytics
        /// Micro
        trackMicroStart() {
            // Game time doesn't include instruction and pass&play time
            TSMAnalyticsManager_7.TSMAnalyticsManager.Instance.onGameTime(this.stateName);
            // ideally this one doesn't either
            TSMAnalyticsManager_7.TSMAnalyticsManager.Instance.onMicroGameStart(this.stateName, this._trackMicroName);
            if (this._trackMicroFirstRound) {
                this._trackMicroFirstRound = false;
                TSMAnalyticsManager_7.TSMAnalyticsManager.Instance.onMicroTime(this.stateName, this._trackMicroName);
            }
        }
        trackMicroEnd(score) {
            TSMAnalyticsManager_7.TSMAnalyticsManager.Instance.onGameScore(this.stateName, score, this._trackMicroName);
            TSMAnalyticsManager_7.TSMAnalyticsManager.Instance.onMicroTime(this.stateName, this._trackMicroName, false, score);
        }
        /// ~ Micro
        trackGameStartTime() {
            if (!this._trackFirstRound) {
                return;
            }
            this._trackFirstRound = false;
            TSMAnalyticsManager_7.TSMAnalyticsManager.Instance.onGameTime(this.stateName);
        }
        trackGameState(totalScore = null, micro = null) {
            TSMAnalyticsManager_7.TSMAnalyticsManager.Instance.onGameState(this.stateName, totalScore, micro);
        }
        trackGameInstructions(microGame = null) {
            TSMAnalyticsManager_7.TSMAnalyticsManager.Instance.onGameInstructions(this.stateName, microGame);
        }
        trackGamePassAndPlay(microGame = null) {
            TSMAnalyticsManager_7.TSMAnalyticsManager.Instance.onGamePassAndPlay(this.stateName, microGame);
        }
    }
    exports.GameBaseState = GameBaseState;
});
define("minigame/flying-tosser/FlyingTossersState", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/util/TextUtil", "switchback-common/util/PromiseUtil", "switchback-common/localization/LocalizationManager", "StateNames", "logic/GameController", "Game", "minigame/flying-tosser/Hatch", "minigame/flying-tosser/FlyingTossersGameController", "minigame/flying-tosser/Magnet", "minigame/flying-tosser/Alien", "minigame/flying-tosser/FlyingTossersGameController", "minigame/flying-tosser/ObstacleFactory", "model/ToyStoryConfig", "component/Curtain", "component/MessageBar", "component/button/TextArrowButton", "minigame/flying-tosser/Claw", "minigame/flying-tosser/ButtonPressTutorialPrompt", "state/GameBaseState"], function (require, exports, ScaleUtil_6, TextUtil_10, PromiseUtil_15, LocalizationManager_10, StateNames_11, GameController_14, Game_14, Hatch_1, FlyingTossersGameController_1, Magnet_1, Alien_1, FlyingTossersGameController_2, ObstacleFactory_1, ToyStoryConfig_8, Curtain_1, MessageBar_1, TextArrowButton_1, Claw_1, ButtonPressTutorialPrompt_1, GameBaseState_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@cerebralfix.com
     */
    class FlyingTossersState extends GameBaseState_1.GameBaseState {
        init(...params) {
            this.stateName = StateNames_11.StateNames.GAME_FLYING_TOSSERS;
            super.init(params);
            this.hatch = null;
            this.levelComplete = false;
            this.tutorialPrompts = [];
            this.alienHasCollided = false;
            this.MAX_TIMEOUT_STUCK_MS = 120000;
        }
        preload() {
            super.preload();
            this.game.load.image('aliens-background', 'assets/minigame/planetary-putt/bg.jpg');
            this.game.load.image('split-dialog', 'assets/dialog/split-dialog.png');
            // load the magnetic button
            this.game.load.atlas('magnet-head', 'assets/minigame/planetary-putt/button/buzz-magnet.png', 'assets/minigame/planetary-putt/button/buzz-magnet.json');
            this.game.load.image('magnet-ray', 'assets/minigame/planetary-putt/button/Magnet_ray.png');
            this.game.load.atlas('magnet-button', 'assets/minigame/planetary-putt/button/buzz-big-red-button.png', 'assets/minigame/planetary-putt/button/buzz-big-red-button.json');
            this.game.load.image('buzz-player-panel', 'assets/minigame/planetary-putt/buzz-player-panel.png');
            // load the obstacles
            this.game.load.atlas("obstacleHorizontal", "assets/minigame/planetary-putt/obstacles/horizontal_obstacle.png", "assets/minigame/planetary-putt/obstacles/horizontal_obstacle.json", Phaser.Loader.TEXTURE_ATLAS_JSON_HASH);
            this.game.load.atlas("obstacleVertical", "assets/minigame/planetary-putt/obstacles/vertical_obstacle.png", "assets/minigame/planetary-putt/obstacles/vertical_obstacle.json", Phaser.Loader.TEXTURE_ATLAS_JSON_HASH);
            this.game.load.atlas("singleObstacle", "assets/minigame/planetary-putt/obstacles/single_obstacle.png", "assets/minigame/planetary-putt/obstacles/single_obstacle.json", Phaser.Loader.TEXTURE_ATLAS_JSON_HASH);
            // load the levels json file
            this.game.load.json("buzz-levels", "assets/minigame/planetary-putt/buzz-levels.json");
            // load the alien spritesheet
            this.game.load.spritesheet("aliens", "assets/minigame/planetary-putt/alien/greenman.png", 170, 189);
            this.game.load.atlas("platform-glow", "assets/minigame/planetary-putt/alien/platform-glow.png", "assets/minigame/planetary-putt/alien/platform-glow.json");
            this.game.load.atlas('instruction_graphic_buzz', 'assets/dialog/instruction_graphic_buzz.png', 'assets/dialog/instruction_graphic_buzz.json');
            this.game.load.atlas('platform', 'assets/minigame/planetary-putt/alien/platform.png', 'assets/minigame/planetary-putt/alien/platform.json');
            this.game.load.atlas('claw', 'assets/minigame/planetary-putt/claw.png', 'assets/minigame/planetary-putt/claw.json');
            this.game.load.image("claw-cord", "assets/minigame/planetary-putt/claw_cord_tile.png");
            this.game.load.image("player-shadow", "assets/minigame/planetary-putt/alien/player_shadow.png");
            this.game.load.image("alien-glow", "assets/minigame/planetary-putt/alien_glow.png");
            this.game.load.spritesheet("hatch-glowing", "assets/minigame/planetary-putt/hole-glowing.png", 302, 302);
            this.game.load.spritesheet("hole", "assets/minigame/planetary-putt/hole.png", 235, 235);
            this.game.load.image("hatch-overlay", "assets/minigame/planetary-putt/hole_overlay.png");
            this.game.load.image("rainbow-arrow", "assets/minigame/planetary-putt/rainbow-arrow.png");
            this.game.load.image('tiling-bar', 'assets/common/CTAbar_tilingbg.png');
            this.game.load.image('cta-curtain', 'assets/common/cta-curtain.png');
            this.game.load.image('next-button', 'assets/common/next-button.png');
            this.game.load.spritesheet("hold-prompt", "assets/minigame/planetary-putt/hold-prompt.png", 150, 304);
            this.game.load.spritesheet("release-prompt", "assets/minigame/planetary-putt/release-prompt.png", 151, 304);
            this.game.load.image("prompt-window", "assets/minigame/planetary-putt/prompt-window.png");
            // load the sounds
            this.game.load.audio('magnet-click', ['assets/sound/buzz/click_magnet.m4a', 'assets/sound/buzz/click_magnet.ogg']);
            this.game.load.audio('magnet-loop', ['assets/sound/buzz/magnet_loop.m4a', 'assets/sound/buzz/magnet_loop.ogg']);
            this.game.load.audio('alien-in-hole', ['assets/sound/buzz/alien_hole.m4a', 'assets/sound/buzz/alien_hole.ogg']);
            this.game.load.audio('hit-wall-1', ['assets/sound/buzz/hit_wall_1.m4a', 'assets/sound/buzz/hit_wall_1.ogg']);
            this.game.load.audio('hit-wall-2', ['assets/sound/buzz/hit_wall_2.m4a', 'assets/sound/buzz/hit_wall_2.ogg']);
            this.game.load.audio('hit-wall-3', ['assets/sound/buzz/hit_wall_3.m4a', 'assets/sound/buzz/hit_wall_3.ogg']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[FlyingTossersState] create()");
                _super("create").call(this);
                this.trackGameState();
                this.controller = new FlyingTossersGameController_1.FlyingTossersGameController(this.game.cache.getJSON("buzz-levels"));
                this.background = new Phaser.Image(this.game, 0, 0, 'aliens-background');
                this.background.width = this.game.width;
                this.background.height = this.game.height;
                this.add.existing(this.background);
                // Rainbow arrow that draws between alien and hatch
                const rainbowArrowHeight = this.game.height * 0.05;
                this.rainbowArrow = new Phaser.TileSprite(this.game, 0, 0, this.game.width * 0.5, rainbowArrowHeight, 'rainbow-arrow');
                const tileScale = rainbowArrowHeight / this.rainbowArrow.texture.height;
                this.rainbowArrow.tileScale.set(tileScale, tileScale);
                this.rainbowArrow.pivot.set(this.rainbowArrow.width * 0.5, this.rainbowArrow.height * 0.5);
                this.rainbowArrow.autoScroll(this.game.width * 0.4, 0);
                this.rainbowArrow.alpha = 0;
                this.add.existing(this.rainbowArrow);
                this.hatches = [];
                this.displayHatches();
                // create the boundary
                this.boundary = new Phaser.Group(this.game);
                this.add.existing(this.boundary);
                this.setupBoundary();
                // create the obstacles group
                this.aliens = new Phaser.Group(this.game);
                this.add.existing(this.aliens);
                this.aliens.physicsBodyType = Phaser.Physics.ARCADE;
                this.claw = new Claw_1.Claw(this.game);
                this.add.existing(this.claw);
                this.game.physics.startSystem(Phaser.Physics.ARCADE); // start the physics
                // create the magnetic buttons
                this.ensurePointers(4);
                this.createMagneticButtons();
                this.visited = [];
                // create the hole overlay but set it's visibility to false until we need it 
                this.holeOverlay = new Phaser.Image(this.game, 0, 0, 'hatch-overlay');
                this.holeOverlay.visible = false;
                this.add.existing(this.holeOverlay);
                this.alienGlowGroup = new Phaser.Group(this.game);
                const alienGlow1 = new Phaser.Image(this.game, 0, 0, 'alien-glow');
                alienGlow1.anchor.set(0.5);
                alienGlow1.visible = false;
                alienGlow1.width = this.game.width * 0.1;
                alienGlow1.scale.y = alienGlow1.scale.x;
                this.alienGlowGroup.add(alienGlow1);
                const alienGlow2 = new Phaser.Image(this.game, 0, 0, 'alien-glow');
                alienGlow2.anchor.set(0.5);
                alienGlow2.visible = false;
                alienGlow2.width = alienGlow1.width;
                alienGlow2.height = alienGlow1.height;
                this.alienGlowGroup.add(alienGlow2);
                this.add.existing(this.alienGlowGroup);
                this.alienGlowShowing = false;
                for (let i = 0; i < 4; i++) {
                    const magnet = this.magnets[i];
                    const tutorialPrompt = new ButtonPressTutorialPrompt_1.ButtonPressTutorialPrompt(this.game);
                    tutorialPrompt.position.set(magnet.button.centerX, magnet.button.top);
                    tutorialPrompt.width = this.game.width * 0.25;
                    tutorialPrompt.scale.y = tutorialPrompt.scale.x;
                    if (i < 2) {
                        tutorialPrompt.angle = 180;
                    }
                    this.add.existing(tutorialPrompt);
                    this.tutorialPrompts.push(tutorialPrompt);
                }
                //////////////////////////
                // TIMER TEXTS
                //////////////////////////
                this.topTimerText = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height * 0.03, "0:00", {
                    font: Game_14.Game.ARCADE_FONT,
                    fontSize: Game_14.Game.FONT_SIZE_EXTRA_LARGE,
                    fill: "#55ff00",
                    stroke: "#002312",
                    strokeThickness: this.game.width * 0.02,
                    align: 'center'
                });
                this.topTimerText.anchor.set(0.5, 0.5);
                this.topTimerText.angle = 180;
                this.add.existing(this.topTimerText);
                this.bottomTimerText = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height * 0.97, "0:00", {
                    font: Game_14.Game.ARCADE_FONT,
                    fontSize: Game_14.Game.FONT_SIZE_EXTRA_LARGE,
                    fill: "#55ff00",
                    stroke: "#002312",
                    strokeThickness: this.game.width * 0.02,
                    align: 'center'
                });
                this.bottomTimerText.anchor.set(0.5, 0.5);
                this.add.existing(this.bottomTimerText);
                this.timerRefreshRef = setInterval(this.refreshTimerText.bind(this), 1000);
                this.refreshTimerText();
                this.curtain = new Curtain_1.Curtain(this.game, true);
                this.add.existing(this.curtain);
                this.topReadyButton = new TextArrowButton_1.TextArrowButton(this.game, LocalizationManager_10.LocalizationManager.Instance.text("common.ready"));
                this.topReadyButton.angle = -180;
                this.topReadyButton.centerX = this.game.width * 0.5;
                this.topReadyButton.centerY = this.game.height * FlyingTossersState.TOP_BUTTON_POSITION;
                this.topReadyButton.alpha = 0;
                this.topReadyButton.visible = false;
                ScaleUtil_6.scaleToBounds(this.topReadyButton, this.game.width * 0.5, this.game.height * 0.15);
                this.topReadyButton.startPulse();
                this.bottomReadyButton = new TextArrowButton_1.TextArrowButton(this.game, LocalizationManager_10.LocalizationManager.Instance.text("common.ready"));
                this.bottomReadyButton.centerX = this.game.width * 0.5;
                this.bottomReadyButton.centerY = this.game.height * FlyingTossersState.BOTTOM_BUTTON_POSITION;
                this.bottomReadyButton.alpha = 0;
                this.bottomReadyButton.visible = false;
                ScaleUtil_6.scaleToBounds(this.bottomReadyButton, this.game.width * 0.5, this.game.height * 0.15);
                this.bottomReadyButton.startPulse();
                this.topMessageBar = new MessageBar_1.MessageBar(this.game, '', 25);
                this.topMessageBar.angle = -180;
                this.topMessageBar.centerX = this.game.width * 0.5;
                this.topMessageBar.bottom = 0;
                this.bottomMessageBar = new MessageBar_1.MessageBar(this.game, '', 25);
                this.bottomMessageBar.centerX = this.game.width * 0.5;
                this.bottomMessageBar.top = this.game.height;
                this.infographic = new Phaser.Image(this.game, 0, 0, 'instruction_graphic_buzz');
                ScaleUtil_6.scaleToBounds(this.infographic, this.game.width * 0.8, this.game.height * 0.35);
                this.infographic.centerX = this.game.width * 0.5;
                this.infographic.centerY = this.game.height * FlyingTossersState.GRAPHIC_POSITION;
                this.infographic.alpha = 0;
                this.infographic.animations.add('loop', [0, 0, 1, 1], 3, true).play();
                this.add.existing(this.infographic);
                this.add.existing(this.bottomMessageBar);
                this.add.existing(this.topMessageBar);
                this.add.existing(this.topReadyButton);
                this.add.existing(this.bottomReadyButton);
                // load the magnet looping sound
                this.magnetLoopSound = this.game.sound.add('magnet-loop', 0, true);
                yield this.dismissOverlay();
                if (ToyStoryConfig_8.ToyStoryConfig.Instance.skipGames) {
                    this.onGameOver(10);
                }
                else {
                    this.startLevel();
                }
                if (ToyStoryConfig_8.ToyStoryConfig.Instance.drawDebugOverlays) {
                    let graphics = new Phaser.Graphics(this.game);
                    graphics.lineStyle(2, 0xff0000);
                    this.add.existing(graphics);
                    let xPos = 0;
                    let xIncrement = this.game.width * FlyingTossersState.TILE_WIDTH;
                    for (let i = 0; i < 10; i++) {
                        graphics.moveTo(xPos, this.game.height * 0.15);
                        graphics.lineTo(xPos, this.game.height);
                        xPos += xIncrement;
                    }
                    let yPos = this.game.height * 0.15;
                    let yIncrement = this.game.height * FlyingTossersState.TILE_HEIGHT;
                    for (let i = 0; i < 15; i++) {
                        graphics.moveTo(0, yPos);
                        graphics.lineTo(this.game.width, yPos);
                        yPos += yIncrement;
                    }
                }
            });
        }
        update() {
            super.update();
            if (this.controller.gameState === FlyingTossersGameController_2.GameState.PLAYING) {
                this.updateGame();
            }
            this.aliens.sort('y', Phaser.Group.SORT_ASCENDING);
        }
        updateGame() {
            // In all cases, prevent the players from getting stuck on a round. Even the tutorial.
            this.controller.roundTimeTakenMs += this.game.time.elapsedMS;
            if (this.controller.roundTimeTakenMs > this.MAX_TIMEOUT_STUCK_MS) {
                this.onTimeUp();
                return;
            }
            if (this.controller.isTutorial) {
                this.showTutorialPrompts();
            }
            else {
                // Update the timers (but not in the tutorial)
                this.controller.timeRemaining -= this.game.time.elapsedMS;
                if (this.controller.timeRemaining < 0 && this.controller.hasEveryonePlayed) {
                    this.onTimeUp();
                    return;
                }
            }
            // Show glow on alien if there has been no interaction for 1 second
            if (!this.alienGlowShowing && this.game.time.now - this.lastClickTimer > FlyingTossersState.ALIEN_INACTIVITY_DURATION) {
                this.showAlienGlow();
            }
            // Ensure pulsing glow follows the alien
            this.alienGlowGroup.centerX = this.alien.alienCenter.x;
            this.alienGlowGroup.centerY = this.alien.alienCenter.y;
            this.isAlienColliding = this.game.physics.arcade.collide(this.aliens);
            // if the alien was not colliding last frame, but now is, play the hit sound
            // this prevents the sound from being spammed as the alien is dragged along a wall. The sound will only play once every time it hits a wall.
            if ((!this.isAlienCollidingLastFrame) && (this.isAlienColliding)) {
                this.game.sound.play(`hit-wall-${this.game.rnd.integerInRange(1, 3)}`);
                // Once we've hit the wall once, we can't get the 'Space Ranger' achievement
                this.alienHasCollided = true;
            }
            this.isAlienCollidingLastFrame = this.isAlienColliding; // store the alien's collision state
            this.game.physics.arcade.collide(this.aliens, this.boundary);
            if (!this.levelComplete) {
                if (this.game.physics.arcade.overlap(this.alien.hatchCollision, this.hatch)) {
                    this.levelComplete = true;
                    this.controller.gameState = FlyingTossersGameController_2.GameState.SWITCHING; // set the game state to switching while we tween alien into the hatch and switch levels
                    // stop the magnet looping sound from playing while switching levels because the magnet button signal isn't processed during level switching
                    // which means the sound will loop forever (or until you press and release the magnet again in the next level)
                    if (this.magnetLoopSound.isPlaying) {
                        this.magnetLoopSound.stop();
                    }
                    this.hideAlienGlow();
                    this.tweenAlienIntoHatch().then(() => {
                        if (!this.alienHasCollided && !this.controller.isTutorial) {
                            GameController_14.GameController.Instance.achievementManager.onBuzzExtraAchieved();
                        }
                        this.controller.nextLevel();
                        this.startLevel();
                    });
                }
            }
            for (let i = 0; i < this.magnets.length; i++) {
                const button = this.magnets[i];
                // if the button is held, move the aliens towards the button
                if (button.isDown) {
                    if (button.ID === 0) { // top left
                        this.alien.body.velocity.x -= FlyingTossersState.PULL_FORCE * this.alien.weight;
                        this.alien.body.velocity.y -= FlyingTossersState.PULL_FORCE * this.alien.weight;
                    }
                    else if (button.ID === 1) { // top right
                        this.alien.body.velocity.x += FlyingTossersState.PULL_FORCE * this.alien.weight;
                        this.alien.body.velocity.y -= FlyingTossersState.PULL_FORCE * this.alien.weight;
                    }
                    else if (button.ID === 2) { // bottom right
                        this.alien.body.velocity.x += FlyingTossersState.PULL_FORCE * this.alien.weight;
                        this.alien.body.velocity.y += FlyingTossersState.PULL_FORCE * this.alien.weight;
                    }
                    else { // bottom left
                        this.alien.body.velocity.x -= FlyingTossersState.PULL_FORCE * this.alien.weight;
                        this.alien.body.velocity.y += FlyingTossersState.PULL_FORCE * this.alien.weight;
                    }
                }
            }
            if (this.controller.isTutorial) {
                const alienBase = this.alien.alienBase;
                const holeCenter = this.hatch.hatchCenter;
                const line = new Phaser.Line(alienBase.x, alienBase.y, holeCenter.x, holeCenter.y);
                this.rainbowArrow.position = line.midPoint();
                this.rainbowArrow.width = line.length;
                this.rainbowArrow.rotation = line.angle;
                this.rainbowArrow.pivot.set(this.rainbowArrow.width * 0.5, this.rainbowArrow.height * 0.5);
            }
        }
        /**
         * We can direct the alien in 8 cardinal directions.
         *
         * If the alien is to the left and below the hole, we should
         * direct the players to hold the top-right button, and
         * release any other buttons they may be holding
         */
        showTutorialPrompts() {
            return __awaiter(this, void 0, void 0, function* () {
                let chosenButton = -1;
                if (this.alien.x < this.hatch.hole.centerX) {
                    if (this.alien.y > this.hatch.hole.centerY) {
                        // The alien is SW
                        chosenButton = 1;
                    }
                    else if (this.alien.y < this.hatch.hole.centerY) {
                        // The alien is NW
                        chosenButton = 2;
                    }
                }
                else if (this.alien.x > this.hatch.hole.centerX) {
                    if (this.alien.y > this.hatch.hole.centerY) {
                        // The alien is SE
                        chosenButton = 0;
                    }
                    else if (this.alien.y < this.hatch.hole.centerY) {
                        // The alien is NE
                        chosenButton = 3;
                    }
                }
                for (let i = 0; i < 4; i++) {
                    if (i === chosenButton) {
                        // If this is the button they're supposed to push
                        // If they're pushing it - great, we can hide the thing that tells them to do anything
                        if (this.magnets[i].isDown) {
                            this.tutorialPrompts[i].hide();
                        }
                        else {
                            // If they're not, they should be - show the push prompt
                            this.tutorialPrompts[i].showHoldAnimation('buzz.hold');
                        }
                    }
                    else {
                        // They shouldn't be pushing this button.
                        // If they are, tell them to stop
                        if (this.magnets[i].isDown) {
                            this.tutorialPrompts[i].showReleaseAnimation();
                        }
                        else {
                            // If they're not, great, we can hide the thing that tells them to do anything.
                            this.tutorialPrompts[i].hide();
                        }
                    }
                }
            });
        }
        showAlienGlow() {
            return __awaiter(this, void 0, void 0, function* () {
                const pulseDuration = 2000;
                this.alienGlowShowing = true;
                for (let glow of this.alienGlowGroup.children) {
                    // Reset glow to original size
                    glow.width = this.game.width * 0.1;
                    glow.scale.y = glow.scale.x;
                    // Reset alpha
                    glow.alpha = 1;
                    glow.visible = true;
                    this.game.add.tween(glow).to({ width: this.game.width * 0.5, height: this.game.width * 0.5 }, pulseDuration, Phaser.Easing.Cubic.Out, true, 0, Infinity, false);
                    this.game.add.tween(glow).to({ alpha: 0 }, pulseDuration, Phaser.Easing.Linear.None, true, 0, Infinity, false)
                        .onComplete.add(() => {
                        glow.alpha = 1;
                    });
                    // Offset each glow by half the tween duration
                    yield PromiseUtil_15.PromiseUtil.time(this.game, pulseDuration * 0.5);
                }
            });
        }
        hideAlienGlow() {
            for (let glow of this.alienGlowGroup.children) {
                this.game.tweens.removeFrom(glow);
                glow.visible = false;
            }
            this.alienGlowShowing = false;
        }
        refreshTimerText() {
            // when everyone hasn't had a turn at a level the time will go negative, but will still play
            // in this case we show the timer counting up (overtime)
            let timeRemaining = Math.abs(this.controller.timeRemaining);
            // Calculate the time difference in minutes and seconds, ensuring they don't go below 0
            let secsTaken = Math.max(0, Math.round(timeRemaining / 1000));
            let minsTaken = Math.max(0, Math.floor(secsTaken / 60));
            secsTaken -= (minsTaken * 60);
            // Pad the text with 0's
            this.topTimerText.setText(`${minsTaken}:${TextUtil_10.TextUtil.leftPad(secsTaken.toString(), "0", 2)}`);
            this.bottomTimerText.setText(`${minsTaken}:${TextUtil_10.TextUtil.leftPad(secsTaken.toString(), "0", 2)}`);
        }
        /**
         * This function randomly chooses and displays a level from
         * the list of currently available levels. The chosen level
         * is removed from the pool once it is loaded.
         */
        startLevel() {
            return __awaiter(this, void 0, void 0, function* () {
                // (Theoretically it's not possible to get here in this state, but just in-case)
                if (this.controller.gameState === FlyingTossersGameController_2.GameState.OVER) {
                    return;
                }
                this.controller.roundTimeTakenMs = 0;
                this.isAlienCollidingLastFrame = true;
                // if we have no time left, end the game
                if (!this.controller.enoughTimeForNextLevel() && this.controller.hasEveryonePlayed) {
                    this.onTimeUp();
                    return;
                }
                const skipCurtains = (this.controller.getCurrentRound() > 0 && GameController_14.GameController.Instance.players.length === 2) ||
                    (this.controller.wasTutorialEnabled && this.controller.getCurrentRound() === 1);
                // load the next level and display all the objects
                const chosenLevel = this.controller.getCurrentLevel();
                // Show the curtains
                if (!this.curtain.isShowing) {
                    yield this.curtain.show(!skipCurtains);
                }
                this.disableButtons();
                // clear all the objects in the game
                if (this.aliens) {
                    this.aliens.removeAll(true);
                }
                if (this.alien) {
                    this.alien.destroy();
                }
                if (this.hatch) {
                    this.holeOverlay.visible = false;
                    this.hatch.deactivateHatch();
                }
                this.parseLevel(chosenLevel);
                /**
                 * Show the pass and play dialogs only in these scenarios:
                 * 1. There are more than 2 players
                 * 2. There are 2 players, and this is their first round (N.B this value starts at 1...)
                 * 3. There are more than 2 players and it isn't immediately following the tutorial as the same guests will play twice in a row
                 */
                if (skipCurtains) {
                    // Show nothing, immediately resolve the curtains
                }
                else {
                    // Show dialogs
                    yield Promise.all([
                        this.doTopInterstitial(),
                        this.doBottomInterstitial()
                    ]);
                    // Hide dialogs
                    yield this.dismissBothInterstitials();
                }
                this.trackGameStartTime();
                // Grab the alien's destination position and quickly move him off the screen...
                const alienDestination = this.alien.position.clone();
                this.alien.position.set(0, 0);
                // Ensure pulsing glow starts off the screen
                this.alienGlowGroup.centerX = 0;
                this.alienGlowGroup.centerY = 0;
                yield this.curtain.hide();
                this.levelComplete = false;
                this.controller.gameState = FlyingTossersGameController_2.GameState.ENTRANCE;
                yield this.doAlienEntrance(alienDestination);
                this.enableButtons();
                this.controller.gameState = FlyingTossersGameController_2.GameState.PLAYING; // set the game state back to playing
                if (this.controller.isTutorial) {
                    this.game.add.tween(this.rainbowArrow).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
                }
            });
        }
        disableButtons() {
            this.magnets.forEach((magnet) => {
                magnet.disable();
            });
        }
        enableButtons() {
            this.magnets.forEach((magnet) => {
                magnet.enable();
            });
        }
        doAlienEntrance(destination) {
            return __awaiter(this, void 0, void 0, function* () {
                // Move the claw into position
                this.claw.x = destination.x;
                this.claw.bottom = 0;
                // Add the alien to the claw
                this.claw.addAt(this.alien, 0);
                const destToDrop = destination.y - this.game.height * 0.05;
                const duration = (destination.y / this.game.height) * 1500;
                // Move the claw down
                yield PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.claw).to({ y: destToDrop }, duration, Phaser.Easing.Sinusoidal.Out, true));
                this.claw.open();
                this.alien.position.set(destination.x, destToDrop);
                this.aliens.add(this.alien);
                yield PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.alien).to({ y: destination.y }, 500, Phaser.Easing.Bounce.Out, true));
                this.alien.activate();
                this.game.add.tween(this.claw).to({ bottom: 0 }, duration, Phaser.Easing.Sinusoidal.Out, true).onComplete.addOnce(() => {
                    this.claw.close();
                });
                // Ensure pulsing glow starts with the alien
                this.alienGlowGroup.centerX = this.alien.alienCenter.x;
                this.alienGlowGroup.centerY = this.alien.alienCenter.y;
                this.showAlienGlow();
            });
        }
        doTopInterstitial() {
            return __awaiter(this, void 0, void 0, function* () {
                this.trackGamePassAndPlay();
                this.topMessageBar.fontSize = 90;
                this.topMessageBar.text = LocalizationManager_10.LocalizationManager.Instance.textWithReplacements("buzz.instructions_pass", {
                    player_name: this.controller.getCurrentPlayers()[1].name
                }).toUpperCase();
                this.topReadyButton.text = LocalizationManager_10.LocalizationManager.Instance.text("common.ok");
                yield PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                    .to({ centerY: this.game.height * FlyingTossersState.TOP_MESSAGE_BAR_POSITION }, 350, Phaser.Easing.Sinusoidal.InOut, true));
                yield PromiseUtil_15.PromiseUtil.time(this.game, 1500);
                this.topReadyButton.visible = true;
                yield PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.topReadyButton)
                    .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true));
                yield PromiseUtil_15.PromiseUtil.signal(this.topReadyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                // Instructions
                this.trackGameInstructions();
                yield Promise.all([
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                        .to({ bottom: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.topReadyButton)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true))
                ]);
                if (this.controller.getCurrentRound() < 1) {
                    // If the infographic is hidden, show it
                    if (this.infographic.alpha === 0) {
                        this.game.add.tween(this.infographic).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
                    }
                    this.topMessageBar.fontSize = 75;
                    this.topMessageBar.text = LocalizationManager_10.LocalizationManager.Instance.text("buzz.instructions_how_to_play").toUpperCase();
                    this.topReadyButton.text = LocalizationManager_10.LocalizationManager.Instance.text("common.ready").toUpperCase();
                    yield Promise.all([
                        PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                            .to({ centerY: this.game.height * FlyingTossersState.TOP_MESSAGE_BAR_POSITION }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                        PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.topReadyButton)
                            .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true, 2000))
                    ]);
                    yield PromiseUtil_15.PromiseUtil.signal(this.topReadyButton.onChildInputDown);
                    this.game.sound.play('snd_ui_click');
                    this.topMessageBar.fontSize = 90;
                    this.topMessageBar.text = LocalizationManager_10.LocalizationManager.Instance.textWithReplacements("buzz.instructions_waiting_for_player", {
                        player_name: this.controller.getCurrentPlayers()[0].name
                    }).toUpperCase();
                    this.game.add.tween(this.topReadyButton)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true).onComplete.addOnce(() => {
                        this.topReadyButton.visible = false;
                    });
                }
            });
        }
        doBottomInterstitial() {
            return __awaiter(this, void 0, void 0, function* () {
                this.bottomMessageBar.fontSize = 90;
                this.bottomMessageBar.text = LocalizationManager_10.LocalizationManager.Instance.textWithReplacements("buzz.instructions_pass", {
                    player_name: this.controller.getCurrentPlayers()[0].name
                }).toUpperCase();
                this.bottomReadyButton.text = LocalizationManager_10.LocalizationManager.Instance.text("common.ok");
                yield PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                    .to({ centerY: this.game.height * FlyingTossersState.BOTTOM_MESSAGE_BAR_POSITION }, 350, Phaser.Easing.Sinusoidal.InOut, true));
                yield PromiseUtil_15.PromiseUtil.time(this.game, 1500);
                this.bottomReadyButton.visible = true;
                yield PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomReadyButton)
                    .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true));
                yield PromiseUtil_15.PromiseUtil.signal(this.bottomReadyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                yield Promise.all([
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                        .to({ top: this.game.height }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomReadyButton)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true))
                ]);
                if (this.controller.getCurrentRound() < 1) {
                    // If the infographic is hidden, show it
                    if (this.infographic.alpha === 0) {
                        this.game.add.tween(this.infographic).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
                    }
                    this.bottomMessageBar.fontSize = 75;
                    this.bottomMessageBar.text = LocalizationManager_10.LocalizationManager.Instance.text("buzz.instructions_how_to_play").toUpperCase();
                    this.bottomReadyButton.text = LocalizationManager_10.LocalizationManager.Instance.text("common.ready").toUpperCase();
                    yield Promise.all([
                        PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                            .to({ centerY: this.game.height * FlyingTossersState.BOTTOM_MESSAGE_BAR_POSITION }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                        PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomReadyButton)
                            .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true, 2000))
                    ]);
                    yield PromiseUtil_15.PromiseUtil.signal(this.bottomReadyButton.onChildInputDown);
                    this.game.sound.play('snd_ui_click');
                    this.bottomMessageBar.fontSize = 90;
                    this.bottomMessageBar.text = LocalizationManager_10.LocalizationManager.Instance.textWithReplacements("buzz.instructions_waiting_for_player", {
                        player_name: this.controller.getCurrentPlayers()[1].name
                    }).toUpperCase();
                    this.game.add.tween(this.bottomReadyButton)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true).onComplete.addOnce(() => {
                        this.bottomReadyButton.visible = false;
                    });
                }
            });
        }
        dismissBothInterstitials() {
            return __awaiter(this, void 0, void 0, function* () {
                yield Promise.all([
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                        .to({ top: this.game.height }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                        .to({ bottom: 0 }, 350, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.infographic)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                ]);
            });
        }
        /**
         * Render the alien on screen
         *
         * @param spawnLocation The alien's location
         */
        displayAlien(row, col) {
            const xPos = col * this.game.width * FlyingTossersState.TILE_WIDTH;
            const yPos = this.game.height * FlyingTossersState.GRID_STARTING_Y + (row * this.game.height * FlyingTossersState.TILE_HEIGHT);
            this.alien = new Alien_1.Alien(this.game, xPos, yPos);
            this.aliens.add(this.alien);
        }
        /**
         * Render the hatch on screen
         *
         * @param hatch The hatch to render
         */
        displayHatch(hatchID) {
            this.hatch = this.hatches[hatchID - 1];
            this.hatch.activateHatch(); // activate the hatch so we can allow collision detection
            this.add.existing(this.hatch);
        }
        createMagneticButtons() {
            const playerPanelBottom = new Phaser.Image(this.game, 0, this.game.height, 'buzz-player-panel');
            playerPanelBottom.anchor.set(0, 1);
            playerPanelBottom.width = this.game.width;
            playerPanelBottom.scale.y = playerPanelBottom.scale.x;
            const playerPanelTop = new Phaser.Image(this.game, 0, 0, 'buzz-player-panel');
            playerPanelTop.width = this.game.width;
            playerPanelTop.scale.y = -playerPanelTop.scale.x;
            playerPanelTop.left = 0;
            playerPanelTop.bottom = 0;
            const buttons = [];
            this.magnets = [];
            for (let i = 0; i < 4; i++) {
                const button = new Phaser.Image(this.game, 0, 0, "magnet-button", 0);
                button.width = this.game.width * 0.265;
                button.scale.y = button.scale.x;
                buttons.push(button);
                const magnet = new Magnet_1.Magnet(this.game, i, button);
                this.magnets.push(magnet);
                this.add.existing(magnet);
                switch (i) {
                    case 0: { // top left
                        button.scale.y *= -1;
                        button.bottom = 0;
                        button.left = 0;
                        magnet.scale.y *= -1;
                        magnet.top = 0;
                        magnet.left = 0;
                        break;
                    }
                    case 1: { // top right
                        button.scale.y *= -1;
                        button.scale.x *= -1;
                        button.bottom = 0;
                        button.left = this.game.width;
                        magnet.scale.y *= -1;
                        magnet.scale.x *= -1;
                        magnet.top = 0;
                        magnet.right = this.game.width;
                        break;
                    }
                    case 2: { // bottom right
                        button.scale.x *= -1;
                        button.bottom = this.game.height;
                        button.left = this.game.width;
                        magnet.scale.x *= -1;
                        magnet.bottom = this.game.height;
                        magnet.right = this.game.width;
                        break;
                    }
                    case 3: // bottom left
                    default: {
                        button.bottom = this.game.height;
                        button.left = 0;
                        magnet.bottom = this.game.height;
                        magnet.left = 0;
                        break;
                    }
                }
                magnet.pressedSignal.add((shouldMagnetize) => { this.magnetizeAlien(shouldMagnetize); }, this);
            }
            // Add panels
            this.add.existing(playerPanelBottom);
            this.add.existing(playerPanelTop);
            // Add big red buttons
            buttons.forEach((button) => {
                this.add.existing(button);
            });
        }
        onTimeUp() {
            return __awaiter(this, void 0, void 0, function* () {
                this.controller.gameState = FlyingTossersGameController_2.GameState.OVER;
                this.disableButtons();
                this.hideAlienGlow();
                this.topMessageBar.text = this.bottomMessageBar.text = LocalizationManager_10.LocalizationManager.Instance.text("buzz.time_up");
                yield this.curtain.show(true);
                yield Promise.all([
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                        .to({ centerY: this.game.height * 0.25 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                        .to({ centerY: this.game.height * 0.75 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_15.PromiseUtil.time(this.game, 3000)
                ]);
                this.onGameOver(this.controller.getRoundsCompleted());
            });
        }
        /**
         * Determines whether we should magnetize (change texture) the alien or not.
         *
         * @param shouldMagnetize - whether we should magnetize the alien
         */
        magnetizeAlien(shouldMagnetize) {
            if (this.controller.gameState !== FlyingTossersGameController_2.GameState.PLAYING) {
                return;
            }
            // if one magnet triggers off but there is another that is active, don't update alien's platform texture
            if (!shouldMagnetize) {
                for (let i = 0; i < this.magnets.length; i++) {
                    const button = this.magnets[i];
                    if (button.isDown) {
                        return;
                    }
                }
            }
            this.lastClickTimer = this.game.time.now;
            this.hideAlienGlow();
            this.playLoopingMagnetSound(shouldMagnetize);
            this.alien.magnetize(shouldMagnetize);
        }
        playLoopingMagnetSound(playSound) {
            if (playSound) {
                // play the magnet loop sound and tween the volume to 1 over 250ms
                this.magnetLoopSound.play();
                this.game.add.tween(this.magnetLoopSound).to({ volume: 1 }, 250, Phaser.Easing.Linear.None, true);
            }
            else {
                // stop the magnet looping music with a fade (250ms)
                this.game.add.tween(this.magnetLoopSound).to({ volume: 0 }, 250, Phaser.Easing.Linear.None, true);
                this.magnetLoopSound.stop();
            }
        }
        tweenAlienIntoHatch() {
            return __awaiter(this, void 0, void 0, function* () {
                this.alien.body.velocity.x = this.alien.body.velocity.y = 0;
                this.hideAlienGlow();
                this.game.sound.play('alien-in-hole');
                this.alien.deactivate();
                this.disableButtons();
                // Remove the rainbow arrow if we ever had it - it's only ever shown once at most
                if (this.rainbowArrow) {
                    this.rainbowArrow.destroy();
                }
                // Hide all tutorial prompts
                this.tutorialPrompts.forEach(prompt => prompt.hide());
                yield PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.alien)
                    .to({
                    centerX: this.hatch.hole.centerX - (this.hatch.hole.width * 0.25),
                    y: this.hatch.hole.centerY - (this.hatch.hole.height * 0.66)
                }, 250, Phaser.Easing.Linear.None, true, 0));
                this.hatch.turnOffLights();
                // add the overlay over the hole and tween the alien down into it
                this.holeOverlay.centerX = this.hatch.hole.centerX;
                this.holeOverlay.bottom = this.hatch.hole.bottom;
                this.holeOverlay.visible = true;
                yield PromiseUtil_15.PromiseUtil.tween(this.game, this.game.add.tween(this.alien)
                    .to({ y: this.hatch.hole.centerY - (this.hatch.hole.height * 0.2) }, 250, Phaser.Easing.Linear.None, true, 0));
            });
        }
        /**
         * Creates two rectangles at the top and bottom of the cage which
         * serve as the game's bounds.
         *
         * Using `game.physics.arcade.setBounds(x, y, width, height)`
         * causes problems when the window loses focus (objects no longer collide with the
         * bounds), so this is just a workaround.
         */
        setupBoundary() {
            const topBoundary = new Phaser.Sprite(this.game, 0, 0, '');
            this.game.physics.enable(topBoundary);
            topBoundary.body.immovable = true;
            topBoundary.body.setSize(this.game.width, this.game.height * FlyingTossersState.TOP_BOUNDARY_Y, 0, 0);
            this.boundary.add(topBoundary);
            const bottomBoundary = new Phaser.Sprite(this.game, 0, 0, '');
            this.game.physics.enable(bottomBoundary);
            bottomBoundary.body.immovable = true;
            bottomBoundary.body.setSize(this.game.width, this.game.height * 0.11, 0, this.game.height * FlyingTossersState.BOTTOM_BOUNDARY_Y_OFFSET);
            this.boundary.add(bottomBoundary);
        }
        /**
         * Parses a 2d array and adds all the parsed objects into
         * the obstacles group.
         */
        parseLevel(chosenLevel) {
            this.displayHatch(chosenLevel.hatchID);
            this.clearVisitedArray(chosenLevel.obstacles);
            for (let row = 0; row < chosenLevel.obstacles.length; row++) {
                for (let col = 0; col < chosenLevel.obstacles[row].length; col++) {
                    // if we've already visited this object, don't bother checking again
                    if (this.visited[row][col]) {
                        continue;
                    }
                    switch (chosenLevel.obstacles[row][col]) {
                        case "|": {
                            this.parseVerticalWall(chosenLevel.obstacles, row, col);
                            break;
                        }
                        case "-": {
                            this.parseHorizontalWall(chosenLevel.obstacles, row, col);
                            break;
                        }
                        case "S": {
                            this.displayAlien(row, col);
                            break;
                        }
                        default: {
                            break;
                        }
                    }
                }
            }
        }
        /**
         * Traverse down the column looking for how big the vertical wall is
         * @param chosenLevel - the level being parsed
         * @param row - the row the vertical wall is on
         * @param col - the column to go down
         */
        parseVerticalWall(chosenLevel, row, col) {
            let length = 0;
            for (let i = row; i < chosenLevel.length; i++) {
                if (chosenLevel[i][col] === "|") {
                    this.visited[i][col] = true; // mark this wall as visited so we don't look at it again
                    length++;
                }
                else {
                    break;
                }
            }
            // make a new vertical wall with length
            const wall = ObstacleFactory_1.ObstacleFactory.makeObstacle(this.game, ObstacleFactory_1.ObstacleType.VERTICAL_WALL, length, row, col);
            this.aliens.add(wall);
        }
        /**
         * Traverse across the row looking for how big the horizontal wall is
         * @param chosenLevel - the level being parsed
         * @param row - the row to go across
         * @param col - the column the row starts at
         */
        parseHorizontalWall(chosenLevel, row, col) {
            let length = 0;
            for (let i = col; i < chosenLevel[row].length; i++) {
                if (chosenLevel[row][i] === "-") {
                    this.visited[row][i] = true; // mark this wall as visited so we don't look at it again
                    length++;
                }
                else {
                    break;
                }
            }
            // make a new horizontal wall with length
            const wall = ObstacleFactory_1.ObstacleFactory.makeObstacle(this.game, ObstacleFactory_1.ObstacleType.HORIZONTAL_WALL, length, row, col);
            this.aliens.add(wall);
        }
        /**
         * Helper function to set every cell in the visited array
         * to false. This is used during level parsing so we know which
         * objects have been visited and so we can skip over them when parsing.
         */
        clearVisitedArray(chosenLevel) {
            for (let row = 0; row < chosenLevel.length; row++) {
                this.visited[row] = [];
                for (let col = 0; col < chosenLevel[row].length; col++) {
                    this.visited[row][col] = false;
                }
            }
        }
        /**
         * Creates and displays the eight hatches
         */
        displayHatches() {
            FlyingTossersState.HATCH_POSITIONS.forEach((pos) => {
                const hatch = new Hatch_1.Hatch(this.game, this.game.width * pos[0], this.game.height * pos[1], false);
                this.add.existing(hatch);
                this.hatches.push(hatch);
            });
        }
        /**
         * This function is called when time runs out.
         * It creates an exit button so players can return
         * back to the hub.
         */
        onGameOver(score) {
            return __awaiter(this, void 0, void 0, function* () {
                // Stop updating the timers!
                clearInterval(this.timerRefreshRef);
                this.controller.gameState = FlyingTossersGameController_2.GameState.OVER;
                GameController_14.GameController.Instance.postScore(this.stateName, score);
                this.transitionTo();
            });
        }
        transitionTo() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                this.controller.gameState = FlyingTossersGameController_2.GameState.OVER;
                yield _super("transitionTo").call(this, StateNames_11.StateNames.RESULTS);
            });
        }
        shutdown() {
            // Stop updating the timers!
            clearInterval(this.timerRefreshRef);
            this.background = null;
            this.controller = null;
            this.magnets = null;
            this.boundary = null;
            this.aliens = null;
            this.rainbowArrow = null;
            this.hatch = null;
            this.alien = null;
            this.topTimerText = null;
            this.bottomTimerText = null;
            this.magnetLoopSound = null;
            this.visited = null;
            this.curtain = null;
            this.topMessageBar = null;
            this.bottomMessageBar = null;
            this.topReadyButton = null;
            this.bottomReadyButton = null;
            this.infographic = null;
            this.claw = null;
            this.alienGlowGroup = null;
            this.tutorialPrompts = null;
            this.hatches = null;
            this.holeOverlay = null;
            this.game.cache.removeImage('aliens-background');
            // unload the magnetic button
            this.game.cache.removeImage('magnet-head');
            this.game.cache.removeImage('magnet-ray');
            this.game.cache.removeImage('magnet-button');
            this.game.cache.removeImage('buzz-player-panel');
            // unload the obstacles
            this.game.cache.removeImage("obstacleHorizontal");
            this.game.cache.removeImage("obstacleVertical");
            this.game.cache.removeImage("singleObstacle");
            // unload the levels json file
            this.game.cache.removeJSON("buzz-levels");
            // unload the alien spritesheet
            this.game.cache.removeImage("aliens");
            this.game.cache.removeImage("platform-glow");
            this.game.cache.removeImage('instruction_graphic_buzz');
            this.game.cache.removeImage('platform');
            this.game.cache.removeImage('claw');
            this.game.cache.removeImage("claw-cord");
            this.game.cache.removeImage("player-shadow");
            this.game.cache.removeImage("alien-glow");
            this.game.cache.removeImage("hatch-glowing");
            this.game.cache.removeImage("hole");
            this.game.cache.removeImage("hatch-overlay");
            this.game.cache.removeImage("rainbow-arrow");
            this.game.cache.removeImage("hold-prompt");
            this.game.cache.removeImage("release-prompt");
            this.game.cache.removeImage("prompt-window");
            // unload the sounds
            this.game.cache.removeSound('magnet-click');
            this.game.cache.removeSound('magnet-loop');
            this.game.cache.removeSound('alien-in-hole');
            this.game.cache.removeSound('hit-wall-1');
            this.game.cache.removeSound('hit-wall-2');
            this.game.cache.removeSound('hit-wall-3');
            super.shutdown();
        }
        render() {
            super.render();
            if (ToyStoryConfig_8.ToyStoryConfig.Instance.drawDebugOverlays) {
                this.aliens.forEach((o) => {
                    this.game.debug.body(o);
                }, this);
            }
            if (ToyStoryConfig_8.ToyStoryConfig.Instance.testAllBuzzLevels) {
                this.game.debug.text(this.controller.getCurrentRound().toString(), this.game.width * 0.9, 20, 'red');
            }
        }
    }
    // width and height of each tile in the grid
    FlyingTossersState.TILE_WIDTH = 0.1;
    FlyingTossersState.TILE_HEIGHT = 0.05;
    FlyingTossersState.GRID_STARTING_Y = 0.1;
    FlyingTossersState.TOP_BOUNDARY_Y = 0.15;
    FlyingTossersState.BOTTOM_BOUNDARY_Y_OFFSET = 0.89;
    FlyingTossersState.ALIEN_INACTIVITY_DURATION = 1000;
    FlyingTossersState.PULL_FORCE = 15; // the force that the magnet applies on the alien
    FlyingTossersState.GRAPHIC_POSITION = 0.45;
    FlyingTossersState.TOP_MESSAGE_BAR_POSITION = 0.2;
    FlyingTossersState.BOTTOM_MESSAGE_BAR_POSITION = 0.7;
    FlyingTossersState.TOP_BUTTON_POSITION = 0.075;
    FlyingTossersState.BOTTOM_BUTTON_POSITION = 0.825;
    // hatch positions as percentage of screen width and height
    FlyingTossersState.HATCH_POSITIONS = [
        [0.22, 0.23],
        [0.55, 0.23],
        [0.39, 0.37],
        [0.14, 0.48],
        [0.63, 0.48],
        [0.39, 0.56],
        [0.2, 0.67],
        [0.57, 0.67]
    ];
    exports.FlyingTossersState = FlyingTossersState;
});
define("minigame/hamm/Animal", ["require", "exports", "switchback-common/util/ScaleUtil"], function (require, exports, ScaleUtil_7) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@cerebralfix.com
     */
    class Animal extends Phaser.Image {
        constructor(game, x, y, asset, isBarnyard) {
            super(game, x, y, asset);
            this.isBarnyard = false;
            ScaleUtil_7.scaleToBounds(this, game.width * 0.6, game.height * 0.6);
            this.anchor.set(0.5, 1);
            this.angle = -90;
        }
        wobble() {
            const randX = this.game.rnd.integerInRange(this.x + Animal.WOBBLE_OFFSET_X, this.x - Animal.WOBBLE_OFFSET_X);
            const randY = this.game.rnd.integerInRange(this.y + Animal.WOBBLE_OFFSET_Y, this.y - Animal.WOBBLE_OFFSET_Y);
            this.game.add.tween(this)
                .to({ x: randX, y: randY }, 140, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, true).onComplete.addOnce(this.wobble, this);
        }
    }
    exports.Animal = Animal;
});
define("component/LandscapeCurtain", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_16) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class LandscapeCurtain extends Phaser.Group {
        constructor(game) {
            super(game);
            this.curtainTop = new Phaser.Image(this.game, 0, 0, 'cta-curtain-landscape');
            this.curtainTop.anchor.set(0, 0.5);
            this.curtainTop.height = this.game.width;
            this.curtainTop.width = this.game.height * 0.5;
            this.curtainTop.angle = -90;
            this.add(this.curtainTop);
            this.curtainBottom = new Phaser.Image(this.game, 0, 0, 'cta-curtain-landscape');
            this.curtainBottom.anchor.set(0, 0.5);
            this.curtainBottom.height = this.game.width;
            this.curtainBottom.width = this.game.height * 0.5;
            this.curtainBottom.scale.x *= -1;
            this.curtainBottom.angle = -90;
            this.add(this.curtainBottom);
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this.curtainBottom).to({ y: 0 }, 500, Phaser.Easing.Sinusoidal.In, true);
                yield PromiseUtil_16.PromiseUtil.tween(this.game, this.game.add.tween(this.curtainTop).to({ y: 0 }, 500, Phaser.Easing.Sinusoidal.In, true));
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this.curtainBottom).to({ y: this.game.height * 0.5 }, 500, Phaser.Easing.Sinusoidal.Out, true);
                yield PromiseUtil_16.PromiseUtil.tween(this.game, this.game.add.tween(this.curtainTop).to({ y: -this.game.height * 0.5 }, 500, Phaser.Easing.Sinusoidal.Out, true));
            });
        }
    }
    exports.LandscapeCurtain = LandscapeCurtain;
});
define("minigame/hamm/HammInstructionBar", ["require", "exports", "component/MessageBar", "switchback-common/localization/LocalizationManager", "switchback-common/util/MotionUtil"], function (require, exports, MessageBar_2, LocalizationManager_11, MotionUtil_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HammInstructionBar extends Phaser.Group {
        constructor(game) {
            super(game);
            const instructionTag = (MotionUtil_2.MotionUtil.isGyroAvailable ? 'hamm.instructions_charades' : 'hamm.instructions_charades_b');
            const messageBar = new MessageBar_2.LandscapeMessageBar(this.game, LocalizationManager_11.LocalizationManager.Instance.text(instructionTag), 40);
            messageBar.right = 0;
            messageBar.setTextWidth(this.game.height * 0.62);
            const leftChar = new Phaser.Sprite(this.game, 0, messageBar.bottom, 'in-game-character-left');
            leftChar.animations.add('loop', [0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1], 2, true).play();
            leftChar.anchor.set(0, 0.875);
            leftChar.angle = -90;
            leftChar.height = this.game.width * 0.35;
            leftChar.scale.x = leftChar.scale.y;
            const rightChar = new Phaser.Sprite(this.game, 0, messageBar.top, 'in-game-character-right');
            rightChar.animations.add('loop', [1, 1, 1, 0, 1, 0, 1, 0], 5, true).play();
            rightChar.anchor.set(1, 0.875);
            rightChar.angle = -90;
            rightChar.scale = leftChar.scale;
            this.add(messageBar);
            this.add(leftChar);
            this.add(rightChar);
        }
    }
    exports.HammInstructionBar = HammInstructionBar;
});
define("minigame/hamm/HammConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HammConfig {
        parse(data) {
            this.gameDuration = data.gameDuration || 30000;
            this.downThreshold = data.downThreshold || -0.75;
            this.upThreshold = data.upThreshold || 0.75;
            this.resetThreshold = data.resetThreshold || 0.4; // this is close to 22 degrees
        }
    }
    exports.HammConfig = HammConfig;
});
define("minigame/hamm/HammDifficultyConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HammDifficultyConfig {
        parse(data) {
            this.animals = data.animals || ['bird'];
            return this;
        }
    }
    exports.HammDifficultyConfig = HammDifficultyConfig;
});
define("minigame/hamm/CircleButton", ["require", "exports", "Game", "switchback-common/util/TextUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil"], function (require, exports, Game_15, TextUtil_11, LocalizationManager_12, PromiseUtil_17) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CircleButton extends Phaser.Group {
        constructor(game, buttonTag, glowTag, textTag) {
            super(game);
            this.onClickedSignal = new Phaser.Signal();
            const buttonGlow = new Phaser.Image(this.game, 0, 0, glowTag);
            const button = new Phaser.Button(this.game, 0, 0, buttonTag);
            button.onInputDown.add(() => __awaiter(this, void 0, void 0, function* () {
                button.inputEnabled = false;
                this.onClickedSignal.dispatch();
                yield PromiseUtil_17.PromiseUtil.tween2(this.game.add.tween(buttonGlow).to({ alpha: 1 }, 400, Phaser.Easing.Linear.None, true, 0, 1, true));
                button.inputEnabled = true;
            }), this);
            button.anchor.set(0.5);
            buttonGlow.alpha = 0;
            buttonGlow.centerX = button.centerX;
            buttonGlow.centerY = button.centerY;
            const buttonLabel = new Phaser.Text(this.game, 0, 0, LocalizationManager_12.LocalizationManager.Instance.text(textTag), {
                font: Game_15.Game.DEFAULT_FONT,
                fill: Game_15.Game.TEXT_COLOR_WHITE,
                fontSize: 50,
                stroke: Game_15.Game.TEXT_COLOR_BLUE,
                strokeThickness: Game_15.Game.TEXT_STROKE_THICKNESS_MED,
                align: 'center'
            });
            TextUtil_11.TextUtil.scaleDownSingleLineTextToWidth(buttonLabel, button.width * 0.75);
            buttonLabel.anchor.set(0.5);
            buttonLabel.centerX = button.centerX;
            buttonLabel.y = button.y - button.height * 0.08;
            this.add(button);
            this.add(buttonGlow);
            this.add(buttonLabel);
        }
        destroy() {
            this.onClickedSignal.dispose();
            super.destroy();
        }
    }
    exports.CircleButton = CircleButton;
});
define("minigame/hamm/HammGameState", ["require", "exports", "switchback-common/util/CollectionUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "StateNames", "logic/GameController", "minigame/hamm/Animal", "Game", "component/LandscapeCurtain", "component/MessageBar", "minigame/hamm/HammInstructionBar", "model/ToyStoryConfig", "minigame/hamm/HammConfig", "minigame/hamm/HammDifficultyConfig", "state/GameBaseState", "component/button/TextArrowButton", "minigame/hamm/CircleButton", "switchback-common/util/TextUtil", "switchback-common/util/MotionUtil"], function (require, exports, CollectionUtil_3, LocalizationManager_13, PromiseUtil_18, StateNames_12, GameController_15, Animal_1, Game_16, LandscapeCurtain_1, MessageBar_3, HammInstructionBar_1, ToyStoryConfig_9, HammConfig_1, HammDifficultyConfig_1, GameBaseState_2, TextArrowButton_2, CircleButton_1, TextUtil_12, MotionUtil_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var GameState;
    (function (GameState) {
        GameState[GameState["NOT_STARTED"] = 0] = "NOT_STARTED";
        GameState[GameState["LOCKED"] = 1] = "LOCKED";
        GameState[GameState["AWAITING_INPUT"] = 2] = "AWAITING_INPUT";
        GameState[GameState["AWAITING_RESET"] = 3] = "AWAITING_RESET";
    })(GameState || (GameState = {}));
    var PlayerInput;
    (function (PlayerInput) {
        PlayerInput[PlayerInput["NONE"] = 0] = "NONE";
        PlayerInput[PlayerInput["PASS"] = 1] = "PASS";
        PlayerInput[PlayerInput["GUESSED"] = 2] = "GUESSED";
    })(PlayerInput || (PlayerInput = {}));
    class HammGameState extends GameBaseState_2.GameBaseState {
        init() {
            this.stateName = StateNames_12.StateNames.GAME_HAMM;
            super.init();
            this.config = this.loadConfig(HammConfig_1.HammConfig);
            this.difficultyConfig = this.loadDifficultyConfig(HammDifficultyConfig_1.HammDifficultyConfig);
            Animal_1.Animal.WOBBLE_OFFSET_X = this.game.width * 0.025;
            Animal_1.Animal.WOBBLE_OFFSET_Y = this.game.height * 0.02;
            this.animalAnchorRight = this.game.width * 0.85;
            this.currentPlayerIndex = 0;
            this.animalIndex = 0;
            this.players = GameController_15.GameController.Instance.playerManager.getPlayerSession();
            this.playerResponseSignal = new Phaser.Signal();
            this.reorientationSignal = new Phaser.Signal();
            this.stars = [];
            this.animalList = Phaser.ArrayUtils.shuffle(this.difficultyConfig.animals.slice());
            this.gameState = GameState.NOT_STARTED;
            this.timeRemainingMs = this.config.gameDuration;
            this.activeGuessPromiseResolution = null;
            this.curtainsAlreadyShowing = true;
            this.numGuessed = 0;
            this.hasCompletedFirstAction = false;
            this.hasCompletedSecondAction = false;
        }
        preload() {
            super.preload();
            this.game.load.image('hamm-background', 'assets/minigame/hamm/background.jpg');
            this.game.load.image('barn-01', 'assets/minigame/hamm/barn_01.png');
            this.game.load.image('barn-02', 'assets/minigame/hamm/barn_02.png');
            this.game.load.image('hamm-overlay', 'assets/minigame/hamm/overlay.png');
            this.game.load.image('cta-curtain-landscape', 'assets/common/cta-curtain-landscape.jpg');
            this.game.load.image('tiling-bar', 'assets/common/CTAbar_tilingbg.png');
            this.game.load.image('hold-on-forehead', 'assets/dialog/hold-on-forehead.png');
            this.game.load.image('player-watching', 'assets/dialog/player_2.png');
            this.game.load.image('bshow-tutorial', 'assets/minigame/hamm/bshow-tutorial.png');
            this.game.load.image('arrow', 'assets/dialog/arrow.png');
            this.game.load.image('timer-icon', 'assets/common/timer-icon.png');
            this.game.load.atlas('in-game-character-left', 'assets/minigame/hamm/in-game-character-left.png', 'assets/minigame/hamm/in-game-character-left.json');
            this.game.load.atlas('in-game-character-right', 'assets/minigame/hamm/in-game-character-right.png', 'assets/minigame/hamm/in-game-character-right.json');
            this.game.load.image('pass-radio-btn', 'assets/minigame/sarge/radio_btn_blank.png');
            this.game.load.image('pass-radio-btn-glow', 'assets/minigame/sarge/radio_btn_glow.png');
            this.game.load.image('guessed-radio-btn', 'assets/minigame/hamm/radio_green_btn_blank.png');
            this.game.load.image('guessed-radio-btn-glow', 'assets/minigame/hamm/radio_green_btn_glow.png');
            // Success animation stolen from woody
            this.game.load.image('throw-success-burst', 'assets/minigame/woody/snatch_radialfx.png');
            this.game.load.image('lasso-snatch-star-green', 'assets/minigame/woody/star_green.png');
            this.game.load.image('lasso-snatch-star-pink', 'assets/minigame/woody/star_pink.png');
            this.game.load.image('lasso-snatch-star-blue', 'assets/minigame/woody/star_blue.png');
            this.animalList.forEach(animalTag => {
                this.game.load.image(animalTag, `assets/minigame/hamm/animal_${animalTag}.png`);
            });
            this.game.load.audio('hamm-correct', ['assets/sound/hamm/correct.m4a', 'assets/sound/hamm/correct.ogg']);
            this.game.load.audio('hamm-incorrect', ['assets/sound/hamm/incorrect.m4a', 'assets/sound/hamm/incorrect.ogg']);
            this.game.load.audio('hamm-new-animal-1', ['assets/sound/hamm/new_animal_1.m4a', 'assets/sound/hamm/new_animal_1.ogg']);
            this.game.load.audio('hamm-new-animal-2', ['assets/sound/hamm/new_animal_2.m4a', 'assets/sound/hamm/new_animal_2.ogg']);
            this.game.load.audio('hamm-new-animal-3', ['assets/sound/hamm/new_animal_3.m4a', 'assets/sound/hamm/new_animal_3.ogg']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.trackGameState();
                // create background
                const background = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.5, 'hamm-background');
                background.anchor.set(0.5);
                background.angle = -90;
                background.width = this.game.height;
                background.height = this.game.width;
                this.instructionBar = new HammInstructionBar_1.HammInstructionBar(this.game);
                this.instructionBar.left = this.game.width;
                this.instructionBar.centerY = this.game.height * 0.5;
                const timerIcon = new Phaser.Image(this.game, this.game.width * 0.5, 0, 'timer-icon');
                timerIcon.width = this.game.height * 0.1;
                timerIcon.scale.y = timerIcon.scale.x;
                timerIcon.anchor.set(1, 0.5);
                timerIcon.angle = -90;
                const barnBack = new Phaser.Image(this.game, this.game.width, 192, 'barn-02');
                barnBack.anchor.set(1, 1);
                barnBack.angle = -90;
                const barnFore = new Phaser.Image(this.game, this.game.width, 0, 'barn-01');
                barnFore.anchor.set(1, 1);
                barnFore.angle = -90;
                this.overlay = new Phaser.Image(this.game, 0, 0, 'hamm-overlay');
                this.overlay.width = this.game.width;
                this.overlay.height = this.game.height;
                this.overlay.alpha = 0;
                this.animal = new Animal_1.Animal(this.game, 0, 0, this.animalList[0]);
                this.animal.height = this.game.width * 0.75;
                this.animal.scale.x = this.animal.scale.y;
                this.animal.x = this.animalAnchorRight;
                this.animal.top = this.game.height;
                this.originalAnimalScale = this.animal.scale.x;
                this.timeText = new Phaser.Text(this.game, this.game.width * 0.5, timerIcon.height, '', {
                    font: Game_16.Game.DEFAULT_FONT,
                    fill: Game_16.Game.TEXT_COLOR_YELLOW,
                    fontSize: 200,
                    stroke: Game_16.Game.TEXT_COLOR_BLUE,
                    strokeThickness: Game_16.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                    align: 'right'
                });
                this.timeText.anchor.set(1, 0.6);
                this.timeText.angle = -90;
                this.instructionCountdownText = new Phaser.Text(this.game, this.game.width * 0.48, this.game.height * 0.5, '', {
                    font: Game_16.Game.DEFAULT_FONT,
                    fill: Game_16.Game.TEXT_COLOR_WHITE,
                    fontSize: 200,
                    stroke: Game_16.Game.TEXT_COLOR_BLUE,
                    strokeThickness: Game_16.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                    align: 'center',
                    wordWrap: true,
                    wordWrapWidth: this.game.height * 0.35
                });
                this.instructionCountdownText.anchor.set(0.5);
                this.instructionCountdownText.angle = -90;
                this.instructionCountdownText.alpha = 0;
                this.playerResponseText = new Phaser.Text(this.game, this.game.width * 0.4, this.game.height * 0.5, '', {
                    font: Game_16.Game.DEFAULT_FONT,
                    fill: Game_16.Game.TEXT_COLOR_WHITE,
                    fontSize: 200,
                    stroke: Game_16.Game.TEXT_COLOR_BLUE,
                    strokeThickness: Game_16.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                    align: 'center'
                });
                this.playerResponseText.anchor.set(0.5);
                this.playerResponseText.angle = -90;
                this.playerResponseText.alpha = 0;
                this.curtain = new LandscapeCurtain_1.LandscapeCurtain(this.game);
                this.curtain.x = this.game.width * 0.5;
                this.curtain.y = this.game.height * 0.5;
                this.topMessageBar = new MessageBar_3.LandscapeMessageBar(this.game, '');
                this.topMessageBar.right = 0;
                this.topMessageBar.centerY = this.game.height * 0.5;
                const instructionTag = (MotionUtil_3.MotionUtil.isGyroAvailable ? 'hamm.instructions_hold' : 'hamm.instructions_bshow');
                this.bottomMessageBar = new MessageBar_3.LandscapeMessageBar(this.game, LocalizationManager_13.LocalizationManager.Instance.text(instructionTag));
                this.bottomMessageBar.left = this.game.width;
                this.bottomMessageBar.centerY = this.game.height * 0.5;
                this.topInGameMessageBar = new MessageBar_3.LandscapeMessageBar(this.game, LocalizationManager_13.LocalizationManager.Instance.text("hamm.instructions_tilt_up"), 40);
                this.topInGameMessageBar.right = 0;
                this.topInGameMessageBar.centerY = this.game.height * 0.5;
                if (!MotionUtil_3.MotionUtil.isGyroAvailable) {
                    this.topInGameMessageBar.visible = false;
                }
                const player1Tag = (MotionUtil_3.MotionUtil.isGyroAvailable ? 'hold-on-forehead' : 'bshow-tutorial');
                this.holdOnForeheadImage = new Phaser.Image(this.game, 0, this.game.height * 0.85, player1Tag);
                this.holdOnForeheadImage.height = this.game.width * 0.7;
                this.holdOnForeheadImage.scale.x = this.holdOnForeheadImage.scale.y;
                this.holdOnForeheadImage.anchor.set(0.5, 1);
                this.holdOnForeheadImage.angle = -90;
                this.holdOnForeheadImage.alpha = 0;
                this.holdOnForeheadImage.x = this.game.width * 0.9;
                this.playerWatchingImage = new Phaser.Image(this.game, 0, this.game.height * 0.15, 'player-watching');
                this.playerWatchingImage.scale.set(this.holdOnForeheadImage.scale.x);
                this.playerWatchingImage.anchor.set(0.5, 1);
                this.playerWatchingImage.angle = -90;
                this.playerWatchingImage.alpha = 0;
                this.playerWatchingImage.x = this.game.width * 0.9;
                this.arrow = new Phaser.Image(this.game, 0, 0, 'arrow');
                this.arrow.scale.set(this.holdOnForeheadImage.scale.x);
                this.arrow.anchor.set(1);
                this.arrow.angle = -90;
                this.arrow.alpha = 0;
                this.arrow.x = this.playerWatchingImage.x - this.playerWatchingImage.height * 0.65;
                this.arrow.y = this.playerWatchingImage.y + this.playerWatchingImage.width * 0.39;
                if (!MotionUtil_3.MotionUtil.isGyroAvailable) {
                    this.arrow.visible = false;
                }
                if (!MotionUtil_3.MotionUtil.isGyroAvailable) {
                    this.readyButton = new TextArrowButton_2.TextArrowButton(this.game, LocalizationManager_13.LocalizationManager.Instance.text('common.ready').toUpperCase());
                    this.readyButton.angle = -90;
                    this.readyButton.centerX = this.game.width * 0.7;
                    this.readyButton.centerY = this.game.height * 0.5;
                    this.readyButton.alpha = 0;
                    this.readyButton.startPulse();
                    this.guessedButton = new CircleButton_1.CircleButton(this.game, 'guessed-radio-btn', 'guessed-radio-btn-glow', 'hamm.guessed');
                    this.guessedButton.angle = -90;
                    this.guessedButton.centerX = this.game.width * 0.2;
                    this.guessedButton.centerY = this.game.height * 0.2;
                    this.guessedButton.inputEnableChildren = false;
                    this.passButton = new CircleButton_1.CircleButton(this.game, 'pass-radio-btn', 'pass-radio-btn-glow', 'hamm.pass');
                    this.passButton.angle = -90;
                    this.passButton.centerX = this.game.width * 0.2;
                    this.passButton.centerY = this.game.height * 0.8;
                    this.passButton.inputEnableChildren = false;
                }
                // The Burst
                this.successBurst = new Phaser.Image(this.game, 0, 0, 'throw-success-burst');
                this.successBurst.width = this.game.width;
                this.successBurst.scale.y = this.successBurst.scale.x;
                this.successBurst.anchor.set(0.5);
                this.successBurst.alpha = 0;
                this.successBurst.angle = -90;
                // The Stars
                const colors = ['pink', 'blue', 'green'];
                for (let i = 0; i < 4; i++) {
                    let star = new Phaser.Image(this.game, 0, 0, 'lasso-snatch-star-' + colors[i % colors.length]);
                    star.alpha = 0;
                    star.width = star.height = this.game.width * 0.2;
                    star.anchor.set(0.5);
                    this.stars.push(star);
                }
                this.add.existing(background);
                this.add.existing(barnBack);
                this.add.existing(barnFore);
                this.add.existing(this.overlay);
                this.add.existing(this.successBurst);
                this.stars.forEach(star => this.add.existing(star));
                this.add.existing(this.animal);
                this.add.existing(this.playerResponseText);
                this.add.existing(this.instructionBar);
                this.add.existing(this.topInGameMessageBar);
                this.add.existing(timerIcon);
                this.add.existing(this.timeText);
                if (this.guessedButton) {
                    this.add.existing(this.guessedButton);
                    this.add.existing(this.passButton);
                }
                this.add.existing(this.curtain);
                this.add.existing(this.holdOnForeheadImage);
                this.add.existing(this.playerWatchingImage);
                this.add.existing(this.arrow);
                this.add.existing(this.instructionCountdownText);
                this.add.existing(this.topMessageBar);
                this.add.existing(this.bottomMessageBar);
                if (this.readyButton) {
                    this.add.existing(this.readyButton);
                }
                if (MotionUtil_3.MotionUtil.isGyroAvailable) {
                    this.gyroCallBack = (result) => {
                        this.onGyro(result);
                    };
                }
                yield this.dismissOverlay();
                if (ToyStoryConfig_9.ToyStoryConfig.Instance.skipGames) {
                    this.endGame(10);
                }
                else {
                    this.play();
                }
            });
        }
        onGuessedClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                this.playerResponseSignal.dispatch(PlayerInput.GUESSED);
                yield PromiseUtil_18.PromiseUtil.time(this.game, 100);
                this.reorientationSignal.dispatch();
            });
        }
        onPassClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                this.playerResponseSignal.dispatch(PlayerInput.PASS);
                yield PromiseUtil_18.PromiseUtil.time(this.game, 100);
                this.reorientationSignal.dispatch();
            });
        }
        play() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[HammGameState] play()`);
                PlayAPI.subscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.gyroCallBack);
                while (this.currentPlayerIndex < this.players.length) {
                    yield this.showPassAndPlay();
                    this.trackGameStartTime();
                    yield this.doTurn();
                }
                this.endGame(this.numGuessed);
            });
        }
        showPassAndPlay() {
            return __awaiter(this, void 0, void 0, function* () {
                this.topMessageBar.text = LocalizationManager_13.LocalizationManager.Instance.textWithReplacements("hamm.instructions_pass", {
                    player_name: this.players[this.currentPlayerIndex][0].name
                }).toUpperCase();
                const durationMs = 500;
                // Wait for the curtains, if needed
                if (!this.curtainsAlreadyShowing) {
                    yield this.curtain.show();
                }
                // Reset the timer here (it's visible, and should be done when the curtains are drawn)
                this.timeRemainingMs = this.config.gameDuration;
                this.timeText.text = (this.timeRemainingMs / 1000).toFixed(0);
                // Bring the messages and infographic in
                yield Promise.all([
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.holdOnForeheadImage).to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.playerWatchingImage).to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.arrow).to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar).to({ left: 0 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar).to({ right: this.game.width }, durationMs, Phaser.Easing.Sinusoidal.InOut, true))
                ]);
                if (this.readyButton) {
                    this.game.add.tween(this.readyButton).to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true);
                    yield PromiseUtil_18.PromiseUtil.signal(this.readyButton.onChildInputDown);
                }
                else {
                    yield PromiseUtil_18.PromiseUtil.time(this.game, 3000);
                    // Wait for them to orient
                    this.gameState = GameState.AWAITING_RESET;
                    yield PromiseUtil_18.PromiseUtil.signal(this.reorientationSignal);
                }
                this.gameState = GameState.NOT_STARTED;
                // Show the countdown text (& move the infographic across)
                this.instructionCountdownText.text = LocalizationManager_13.LocalizationManager.Instance.text("hamm.countdown_ready");
                this.instructionCountdownText.fontSize = 200;
                this.game.add.tween(this.instructionCountdownText).to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.arrow).to({ alpha: 0 }, durationMs, Phaser.Easing.Linear.None, true);
                if (this.readyButton) {
                    this.game.add.tween(this.readyButton).to({ alpha: 0 }, durationMs, Phaser.Easing.Linear.None, true);
                }
                // Start the countdown!
                const countdownStrings = ['3', '2', '1', LocalizationManager_13.LocalizationManager.Instance.text("hamm.countdown_go")];
                for (let i = 0; i < countdownStrings.length; i++) {
                    yield PromiseUtil_18.PromiseUtil.time(this.game, 1000);
                    this.instructionCountdownText.fontSize = 300;
                    this.instructionCountdownText.text = countdownStrings[i];
                    this.game.add.tween(this.instructionCountdownText.scale).to({ x: 1.5, y: 1.5 }, durationMs * 0.5, Phaser.Easing.Cubic.InOut, true, 0, 0, true);
                }
                // We're done! exit everything
                yield Promise.all([
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionCountdownText).to({ alpha: 0 }, durationMs, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.holdOnForeheadImage).to({ alpha: 0 }, durationMs, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.playerWatchingImage).to({ alpha: 0 }, durationMs, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar).to({ right: 0 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar).to({ left: this.game.width }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    yield this.curtain.hide()
                ]);
                this.curtainsAlreadyShowing = false;
            });
        }
        doTurn() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[HammGameState] doTurn()`);
                // Reset for a new turn
                this.gameState = GameState.LOCKED;
                this.timeRemainingMs = this.config.gameDuration;
                this.topInGameMessageBar.text = LocalizationManager_13.LocalizationManager.Instance.text('hamm.instructions_tilt_down');
                if (this.guessedButton) {
                    this.guessedButton.onClickedSignal.add(this.onGuessedClicked, this);
                    this.passButton.onClickedSignal.add(this.onPassClicked, this);
                }
                // Show the in game instructions for the audience
                this.game.add.tween(this.instructionBar)
                    .to({ x: this.game.width }, 350, Phaser.Easing.Sinusoidal.InOut, true);
                // Show the in game instructions for the player after a short delay
                this.game.add.tween(this.topInGameMessageBar)
                    .to({ left: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true, 2000);
                // While this player still has time left
                while (this.timeRemainingMs > 0) {
                    const response = yield new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                        // Save off a promise that we can resolve early if we run out of time
                        this.activeGuessPromiseResolution = resolve;
                        // Show the animal
                        yield this.showAnimal(this.getNextAnimal());
                        // Wait for input (up or down)
                        this.gameState = GameState.AWAITING_INPUT;
                        const playerResponse = yield PromiseUtil_18.PromiseUtil.signal(this.playerResponseSignal);
                        if (!this.activeGuessPromiseResolution) {
                            return;
                        }
                        // They've responded! Tint and show the overlay to indicate that we've heard their response.
                        this.overlay.tint = playerResponse === PlayerInput.GUESSED ? HammGameState.GREEN_OVERLAY_TINT : HammGameState.RED_OVERLAY_TINT;
                        this.game.add.tween(this.overlay)
                            .to({ alpha: 0.5 }, 350, Phaser.Easing.Linear.None, true);
                        // Wait for an orientation reset
                        this.gameState = GameState.AWAITING_RESET;
                        yield PromiseUtil_18.PromiseUtil.signal(this.reorientationSignal);
                        if (!this.activeGuessPromiseResolution) {
                            return;
                        }
                        // Resolve this promise!
                        return this.activeGuessPromiseResolution(playerResponse);
                    }));
                    this.activeGuessPromiseResolution = null;
                    // Stop them from doing anything now, they've 
                    // made their choice (or they've run out of time!)
                    this.gameState = GameState.LOCKED;
                    // Get rid of the animal, making room for the next one
                    yield this.dismissAnimal(response);
                    if (response === PlayerInput.GUESSED) {
                        this.numGuessed++;
                        // If they hadn't completed the first tutorial step - they have now!
                        if (!this.hasCompletedFirstAction) {
                            this.game.add.tween(this.topInGameMessageBar)
                                .to({ right: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true).onComplete.addOnce(() => {
                                this.topInGameMessageBar.text = LocalizationManager_13.LocalizationManager.Instance.text('hamm.instructions_tilt_up');
                                this.hasCompletedFirstAction = true;
                                this.game.add.tween(this.topInGameMessageBar)
                                    .to({ left: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true);
                            });
                        }
                    }
                    else if (response === PlayerInput.PASS) {
                        // Only allow the second action to register after the first
                        if (this.hasCompletedFirstAction && !this.hasCompletedSecondAction) {
                            this.hasCompletedSecondAction = true;
                            // This is our cue to stop hand holding the player and slide out the in-game bar
                            this.game.add.tween(this.topInGameMessageBar)
                                .to({ right: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true);
                        }
                    }
                }
                if (this.guessedButton) {
                    this.guessedButton.onClickedSignal.remove(this.onGuessedClicked, this);
                    this.passButton.onClickedSignal.remove(this.onPassClicked, this);
                }
                // Time for the next player! Clean up this one.
                this.gameState = GameState.NOT_STARTED;
                this.currentPlayerIndex++;
                this.hasCompletedFirstAction = false;
                this.hasCompletedSecondAction = false;
                // Slide out any in-game bars
                this.game.add.tween(this.instructionBar)
                    .to({ left: this.game.width }, 350, Phaser.Easing.Sinusoidal.InOut, true);
                this.game.add.tween(this.topInGameMessageBar)
                    .to({ right: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true);
                // Let them know their time is up
                this.topMessageBar.text = LocalizationManager_13.LocalizationManager.Instance.text("hamm.time_up");
                yield Promise.all([
                    this.game.add.tween(this.topMessageBar)
                        .to({ centerX: this.game.width * 0.5 }, 350, Phaser.Easing.Sinusoidal.InOut, true),
                    PromiseUtil_18.PromiseUtil.time(this.game, 3000),
                    this.curtain.show()
                ]);
            });
        }
        getNextAnimal() {
            this.animalIndex++;
            if (this.animalIndex >= this.animalList.length) {
                this.animalList = CollectionUtil_3.shuffleNonRepeatingArray(this.animalList);
                this.animalIndex = 0;
            }
            return this.animalList[this.animalIndex];
        }
        showAnimal(animalTag) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[HammGameState] showAnimal(${animalTag})`);
                this.animal.loadTexture(animalTag);
                PromiseUtil_18.PromiseUtil.time(this.game, 250).then(() => {
                    this.game.sound.play(`hamm-new-animal-${this.game.rnd.integerInRange(1, 3)}`);
                });
                yield PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.animal)
                    .to({ y: this.game.height * 0.6 }, HammGameState.ANIMAL_TRANSITION_IN_DURATION_MS, Phaser.Easing.Sinusoidal.InOut, true));
                this.animal.wobble();
            });
        }
        dismissAnimal(playerInput) {
            return __awaiter(this, void 0, void 0, function* () {
                // Set up the overlay tint and response text according to their input
                switch (playerInput) {
                    case PlayerInput.GUESSED: {
                        this.overlay.tint = HammGameState.GREEN_OVERLAY_TINT;
                        this.playerResponseText.text = LocalizationManager_13.LocalizationManager.Instance.text("hamm.correct");
                        TextUtil_12.TextUtil.scaleDownSingleLineTextToWidth(this.playerResponseText, this.game.width * 0.65);
                        break;
                    }
                    case PlayerInput.PASS: {
                        this.overlay.tint = HammGameState.RED_OVERLAY_TINT;
                        this.playerResponseText.text = LocalizationManager_13.LocalizationManager.Instance.text("hamm.pass");
                        break;
                    }
                    default:
                    case PlayerInput.NONE: {
                        this.overlay.tint = HammGameState.RED_OVERLAY_TINT;
                        this.playerResponseText.text = '';
                        break;
                    }
                }
                this.game.sound.play(playerInput === PlayerInput.GUESSED ? 'hamm-correct' : 'hamm-incorrect');
                // Show the response text
                const origTextScale = this.playerResponseText.scale.x;
                this.playerResponseText.scale.set(0);
                const promises = [
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.playerResponseText)
                        .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.playerResponseText.scale)
                        .to({ x: origTextScale, y: origTextScale }, 350, Phaser.Easing.Back.Out, true))
                ];
                // If we aren't already showing the overlay, fade it in here
                // (it can be shown when an action is registered, but you can 
                // get here without an action being registered)
                if (this.overlay.alpha === 0) {
                    promises.push(PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.overlay)
                        .to({ alpha: 0.5 }, 350, Phaser.Easing.Linear.None, true)));
                }
                if (playerInput === PlayerInput.GUESSED) {
                    promises.push(this.playSuccess());
                }
                yield Promise.all(promises);
                // Stop the animal wobbling and move it out
                this.game.tweens.removeFrom(this.animal);
                if (playerInput === PlayerInput.GUESSED) {
                    yield this.dismissAnimalGuessed();
                }
                else {
                    yield this.dismissAnimalPassed();
                }
                // Fade out the response text & overlay
                yield Promise.all([
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.overlay)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.playerResponseText)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true))
                ]);
                // Reset the animal back for the next one
                this.animal.scale.set(this.originalAnimalScale);
                this.animal.top = this.game.height;
                this.animal.x = this.animalAnchorRight;
            });
        }
        dismissAnimalGuessed() {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.animal)
                    .to({ y: -(this.animal.width * 0.5), x: this.animalAnchorRight }, HammGameState.ANIMAL_TRANSITION_OUT_DURATION_MS, Phaser.Easing.Sinusoidal.InOut, true));
            });
        }
        dismissAnimalPassed() {
            return __awaiter(this, void 0, void 0, function* () {
                const durationMs = HammGameState.ANIMAL_TRANSITION_PASS_DURATION_MS;
                yield Promise.all([
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.animal.scale)
                        .to({ x: this.originalAnimalScale * 1.25, y: this.originalAnimalScale * 1.25 }, durationMs * 0.5, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.animal.scale)
                        .to({ y: this.originalAnimalScale * 0.25 }, durationMs * 0.5, Phaser.Easing.Sinusoidal.InOut, true, durationMs * 0.5)),
                    PromiseUtil_18.PromiseUtil.tween(this.game, this.game.add.tween(this.animal)
                        .to({ x: this.game.width + this.animal.height }, durationMs * 0.5, Phaser.Easing.Sinusoidal.InOut, true, durationMs * 0.5))
                ]);
            });
        }
        // Does the animation for when an item is taken
        playSuccess() {
            return __awaiter(this, void 0, void 0, function* () {
                // Starting positions
                const animationPromises = [];
                const originPoint = new Phaser.Point((this.animal.centerY - this.animal.height * 0.2), (this.animal.centerX + this.animal.width * 0.1));
                // Ray
                this.successBurst.position.set(originPoint.x, originPoint.y);
                this.successBurst.alpha = 0;
                // Ray's tweens
                const burstScaleTween = this.game.add.tween(this.successBurst.scale)
                    .from({ x: this.successBurst.scale.x * 0.5, y: this.successBurst.scale.y * 0.5 }, HammGameState.SUCCESS_ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.successBurst)
                    .to({ alpha: 1 }, HammGameState.SUCCESS_ANIMATION_DURATION_MS * 0.5, Phaser.Easing.Linear.None, true, 0, 0, true);
                animationPromises.push(PromiseUtil_18.PromiseUtil.tween(this.game, burstScaleTween));
                // Stars
                for (let i = 0; i < this.stars.length; i++) {
                    const star = this.stars[i];
                    star.position.set(originPoint.x, originPoint.y);
                    star.alpha = 0;
                    star.angle = 0;
                    star.visible = true;
                    this.game.tweens.removeFrom(star);
                    // Alpha
                    const starAlphaTween = this.game.add.tween(star)
                        .to({ alpha: 1 }, HammGameState.SUCCESS_ANIMATION_DURATION_MS * 0.5, Phaser.Easing.Linear.None, true, 0, 0, true);
                    starAlphaTween.onComplete.addOnce(() => {
                        star.visible = false;
                    });
                    // X movement
                    const starYMovementTween = this.game.add.tween(star)
                        .to({ y: star.y + (this.game.height * HammGameState.STAR_Y_DEST[i]) }, HammGameState.SUCCESS_ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true);
                    // Y movement (first bit)
                    const starXMovementTweenUp = this.game.add.tween(star)
                        .to({ x: star.x + (this.game.width * HammGameState.STAR_X_DEST_1[i]) }, HammGameState.SUCCESS_ANIMATION_DURATION_MS * 0.5, Phaser.Easing.Quadratic.Out, true);
                    // Y movement (second bit)
                    const starXMovementTweenDown = this.game.add.tween(star)
                        .to({ x: star.x + (this.game.width * HammGameState.STAR_X_DEST_2[i]) }, HammGameState.SUCCESS_ANIMATION_DURATION_MS * 0.5, Phaser.Easing.Quadratic.In, false);
                    starXMovementTweenUp.chain(starXMovementTweenDown);
                    // Rotation
                    this.game.add.tween(star)
                        .to({ angle: star.angle + HammGameState.STAR_ANGLE_DEST[i] }, HammGameState.SUCCESS_ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true);
                    animationPromises.push(PromiseUtil_18.PromiseUtil.tween(this.game, starYMovementTween));
                }
                yield Promise.all(animationPromises);
            });
        }
        endGame(score) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[HammGameState] endGame()`);
                yield GameController_15.GameController.Instance.achievementManager.onHammEndGame(score);
                GameController_15.GameController.Instance.postScore(this.stateName, score);
                yield this.curtain.show();
                this.transitionTo(StateNames_12.StateNames.RESULTS);
            });
        }
        onGyro(result) {
            const roll = result.attitude.roll - (Math.PI * 0.5);
            if (this.gameState === GameState.AWAITING_INPUT) {
                if (roll < this.config.downThreshold) {
                    this.playerResponseSignal.dispatch(PlayerInput.PASS);
                }
                else if (roll > this.config.upThreshold) {
                    this.playerResponseSignal.dispatch(PlayerInput.GUESSED);
                }
            }
            else if (this.gameState === GameState.AWAITING_RESET) {
                if (Math.abs(roll) < this.config.resetThreshold) {
                    this.reorientationSignal.dispatch();
                }
            }
        }
        update() {
            super.update();
            if (this.gameState !== GameState.NOT_STARTED) {
                this.timeRemainingMs -= this.game.time.elapsedMS;
                this.timeText.text = Math.max(0, Math.floor((this.timeRemainingMs / 1000))).toFixed(0);
                if (this.timeRemainingMs <= 0 && this.activeGuessPromiseResolution) {
                    this.activeGuessPromiseResolution(PlayerInput.NONE);
                }
            }
        }
        shutdown() {
            if (this.playerResponseSignal) {
                this.playerResponseSignal.dispose();
                this.playerResponseSignal = null;
            }
            if (this.reorientationSignal) {
                this.reorientationSignal.dispose();
                this.reorientationSignal = null;
            }
            PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.gyroCallBack);
            this.gyroCallBack = null;
            this.activeGuessPromiseResolution = null;
            this.timeText = null;
            this.playerResponseText = null;
            this.instructionCountdownText = null;
            this.players = null;
            this.animal = null;
            this.overlay = null;
            this.successBurst = null;
            this.stars = null;
            this.curtain = null;
            this.topMessageBar = null;
            this.bottomMessageBar = null;
            this.topInGameMessageBar = null;
            this.holdOnForeheadImage = null;
            this.playerWatchingImage = null;
            this.arrow = null;
            this.instructionBar = null;
            this.gameState = null;
            this.game.cache.removeImage('arrow');
            this.game.cache.removeImage('hamm-background');
            this.game.cache.removeImage('barn-01');
            this.game.cache.removeImage('barn-02');
            this.game.cache.removeImage('hamm-overlay');
            this.game.cache.removeImage('in-game-character-left');
            this.game.cache.removeImage('in-game-character-right');
            this.game.cache.removeImage('pass-radio-btn');
            this.game.cache.removeImage('pass-radio-btn-glow');
            this.game.cache.removeImage('guessed-radio-btn');
            this.game.cache.removeImage('guessed-radio-btn-glow');
            // Success animation stolen from woody
            this.game.cache.removeImage('throw-success-burst');
            this.game.cache.removeImage('lasso-snatch-star-green');
            this.game.cache.removeImage('lasso-snatch-star-pink');
            this.game.cache.removeImage('lasso-snatch-star-blue');
            // clean up the animal assets
            this.animalList.forEach(animalTag => {
                this.game.cache.removeImage(animalTag);
            });
            this.game.cache.removeSound('hamm-correct');
            this.game.cache.removeSound('hamm-incorrect');
            this.game.cache.removeSound('hamm-new-animal-1');
            this.game.cache.removeSound('hamm-new-animal-2');
            this.game.cache.removeSound('hamm-new-animal-3');
            super.shutdown();
        }
    }
    HammGameState.GREEN_OVERLAY_TINT = 0x56B887;
    HammGameState.RED_OVERLAY_TINT = 0xC25C67;
    HammGameState.ANIMAL_TRANSITION_IN_DURATION_MS = 250;
    HammGameState.ANIMAL_TRANSITION_OUT_DURATION_MS = 250;
    HammGameState.ANIMAL_TRANSITION_PASS_DURATION_MS = 250;
    HammGameState.SUCCESS_ANIMATION_DURATION_MS = 500;
    HammGameState.STAR_X_DEST_1 = [-.15, -.25, -.25, -.15];
    HammGameState.STAR_X_DEST_2 = [.1, -.1, -.1, .1];
    HammGameState.STAR_Y_DEST = [.35, 0.2, -0.2, -0.35];
    HammGameState.STAR_ANGLE_DEST = [-90, -90, 90, 90];
    exports.HammGameState = HammGameState;
});
define("switchback-common/extension/Easing", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function shake(aPeriod1, aPeriod2) {
        return (k) => {
            const current1 = k * Math.PI * 2 * aPeriod1;
            const current2 = k * (Math.PI * 2 * aPeriod2 + Math.PI / 2);
            return Math.sin(current1) * Math.cos(current2);
        };
    }
    exports.shake = shake;
});
define("minigame/woody/balance/Carriage", ["require", "exports", "switchback-common/extension/Easing"], function (require, exports, Easing_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * This class exists to contain the carriage image. This is a group so that its x/y can remain
     * constant while the image within the group can shake up and down.
     * @author joel@cerebralfix.com
     */
    class Carriage extends Phaser.Group {
        constructor(game) {
            super(game);
            // Random carriage color
            const carriageNum = game.rnd.integerInRange(0, 2);
            const carriage = new Phaser.Image(game, 0, 0, `carriage-${carriageNum}`);
            carriage.anchor.set(0.5);
            this.add(carriage);
            // Random shaking tween
            this.game.add.tween(carriage).to({ y: carriage.y - game.rnd.integerInRange(this.game.height * 0.01, this.game.height * 0.02) }, this.game.rnd.integerInRange(100, 200), Easing_1.shake(0.1, 0.1), true, game.rnd.integerInRange(0, 75), Infinity);
        }
    }
    exports.Carriage = Carriage;
});
define("minigame/woody/balance/Train", ["require", "exports", "minigame/woody/balance/Carriage"], function (require, exports, Carriage_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * This class does the slightly complex depth graphical trickery
     * to give the impression that the player is moving along the train.
     *
     * It contains an array of Carriages which can all move forward to the next
     * 'spot' in line through a combination of y-movement, skewing and scaling
     * to give the impression that the camera is moving forwards.
     *
     * Only 3 cars are visible at a time, but up to 5 players can play the game.
     * Therefore up to 2 extra carriages need to be spawned at the end of the train
     * after they have all moved forward a place.
     * @author joel@cerebralfix.com
     */
    class Train extends Phaser.Group {
        constructor(game, numPlayers) {
            super(game);
            /**
             * Arrays of magic numbers. These denote screen width/height
             * proportions for each carriage in the train. e.g. the closest
             * carriage, number 0, uses the values at index 0. The furthest
             * carriage visible uses the end values.
             */
            this.widthScales = [1.75, 1, 0.55, 0.3];
            this.yHeights = [1.3, 1, 0.84, 0.75];
            this.skews = [0, 0, 0.15, 0.3];
            this.centerXs = [0.5, 0.5, 0.54, 0.59];
            // Collection of train cars.
            this.cars = [];
            this.numCarsPassed = 0;
            this.numCarsTotal = numPlayers + 1;
            const numCars = Math.min(this.numCarsTotal, Train.NUM_CARS_MAX);
            // Create all the train cars
            for (let i = 0; i < numCars; i++) {
                this.cars.push(this.createCar(i));
            }
            // Add them to the scene in the reverse order
            for (let i = this.cars.length - 1; i >= 0; i--) {
                this.add(this.cars[i]);
            }
        }
        createCar(i) {
            const car = new Carriage_1.Carriage(this.game);
            car.width = this.game.width * this.widthScales[i];
            car.scale.y = car.scale.x;
            car.centerX = this.game.width * this.centerXs[i];
            car.skew = new Phaser.Point(this.skews[i], 0);
            car.y = this.game.height * this.yHeights[i];
            return car;
        }
        moveCarsForward(timeMs) {
            return __awaiter(this, void 0, void 0, function* () {
                for (let i = 0; i < this.cars.length; i++) {
                    const car = this.cars[i];
                    // Resizing, moving & skewing
                    let widthDest;
                    let heightDest;
                    let yDest;
                    let centerXDest;
                    let skewXDest;
                    if (i === 0) {
                        centerXDest = this.game.width * 0.5;
                        widthDest = this.game.width * 1.75;
                        heightDest = car.height * (widthDest / car.width);
                        yDest = this.game.height + (heightDest * 1.75);
                        skewXDest = 0;
                    }
                    else {
                        yDest = this.cars[i - 1].y;
                        centerXDest = this.cars[i - 1].centerX;
                        widthDest = this.cars[i - 1].width;
                        heightDest = car.height * (widthDest / car.width);
                        skewXDest = this.skews[i - 1];
                    }
                    // Skew tween
                    this.game.add.tween(car.skew).to({ x: skewXDest }, timeMs, Phaser.Easing.Sinusoidal.InOut, true, 0);
                    // y-movement, x-movement, scaling tween
                    const movementTween = this.game.add.tween(car).to({ y: yDest, centerX: centerXDest, width: widthDest, height: heightDest }, timeMs, Phaser.Easing.Sinusoidal.InOut, true, 0);
                    // If this car was the first one, it's just gone offscreen (behind).
                    if (i === 0) {
                        // When it finishes its movement, destroy it.
                        movementTween.onComplete.addOnce(() => {
                            const carToDestroy = this.cars.shift();
                            carToDestroy.destroy();
                            this.numCarsPassed++;
                            // If we need to make more cars, add one here.
                            if (this.cars.length + this.numCarsPassed < this.numCarsTotal) {
                                const car = this.createCar(Train.NUM_CARS_MAX - 1);
                                // Add and layer new car correctly.
                                this.cars.push(car);
                                this.add(car);
                                this.sendToBack(car);
                            }
                        }, this);
                    }
                }
            });
        }
    }
    Train.NUM_CARS_MAX = 4;
    exports.Train = Train;
});
define("model/CharacterInfo", ["require", "exports", "switchback-common/localization/LocalizationManager", "Character"], function (require, exports, LocalizationManager_14, Character_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CharacterInfo {
        static getInfo(characterName) {
            let _charName;
            let _dialogKey;
            let _portraitKey;
            // Get character specific data
            switch (characterName) {
                case Character_1.Character.WOODY:
                    _charName = LocalizationManager_14.LocalizationManager.Instance.text('common.character_woody');
                    _portraitKey = 'portrait-woody-idle';
                    _dialogKey = 'character-dialog-woody';
                    break;
                case Character_1.Character.BUZZ:
                    _charName = LocalizationManager_14.LocalizationManager.Instance.text('common.character_buzz');
                    _portraitKey = 'portrait-buzz-idle';
                    _dialogKey = 'character-dialog-buzz';
                    break;
                case Character_1.Character.JESSIE:
                    _charName = LocalizationManager_14.LocalizationManager.Instance.text('common.character_jessie');
                    _portraitKey = 'portrait-jessie-idle';
                    _dialogKey = 'character-dialog-jessie';
                    break;
                case Character_1.Character.REX:
                    _charName = LocalizationManager_14.LocalizationManager.Instance.text('common.character_rex');
                    _portraitKey = 'portrait-rex-idle';
                    _dialogKey = 'character-dialog-rex';
                    break;
                case Character_1.Character.SARGE:
                    _charName = LocalizationManager_14.LocalizationManager.Instance.text('common.character_sarge');
                    _portraitKey = 'portrait-sarge-idle';
                    _dialogKey = 'character-dialog-sarge';
                    break;
                case Character_1.Character.HAMM:
                    _charName = LocalizationManager_14.LocalizationManager.Instance.text('common.character_hamm');
                    _portraitKey = 'portrait-hamm-idle';
                    _dialogKey = 'character-dialog-hamm';
                    break;
                case Character_1.Character.TRIXIE:
                    _charName = LocalizationManager_14.LocalizationManager.Instance.text('common.character_trixie');
                    _portraitKey = 'portrait-trixie-idle';
                    _dialogKey = 'character-dialog-trixie';
                    break;
                case Character_1.Character.PETE:
                    _charName = LocalizationManager_14.LocalizationManager.Instance.text('common.character_pete');
                    _portraitKey = 'portrait-pete-idle';
                    _dialogKey = 'character-dialog-pete';
                    break;
                case Character_1.Character.PETE_FOUND:
                    _charName = LocalizationManager_14.LocalizationManager.Instance.text('common.character_pete_found');
                    _portraitKey = 'portrait-pete-found-idle';
                    _dialogKey = 'character-dialog-pete-found';
                    break;
                default:
                    console.log('Character not found: ' + characterName);
                    break;
            }
            return {
                characterName: _charName,
                portraitKey: _portraitKey,
                dialogKey: _dialogKey
            };
        }
    }
    exports.CharacterInfo = CharacterInfo;
});
define("component/dialog/CharacterDialog", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil", "Game", "model/CharacterInfo", "component/ArrowPrompt", "switchback-common/component/text/TypewriterText"], function (require, exports, PromiseUtil_19, TextUtil_13, Game_17, CharacterInfo_1, ArrowPrompt_3, TypewriterText_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class CharacterDialog extends Phaser.Group {
        constructor(game, character) {
            super(game);
            this.visible = false;
            this.inputEnableChildren = true;
            const characterInfo = CharacterInfo_1.CharacterInfo.getInfo(character);
            this.portrait = new Phaser.Image(game, 0, 0, characterInfo.dialogKey);
            this.portrait.width = game.width * 0.52;
            this.portrait.scale.y = this.portrait.scale.x;
            this.portrait.centerX = game.width * 0.14;
            this.add(this.portrait);
            this.portrait.input.pixelPerfectClick = true;
            this.backplate = new Phaser.Image(game, 0, 0, 'character-dialog-backplate');
            this.backplate.width = game.width * 0.79;
            this.backplate.scale.y = this.backplate.scale.x;
            this.backplate.centerX = game.width * 0.62;
            this.backplate.centerY = this.portrait.height * 0.53;
            this.add(this.backplate);
            this.backplate.input.pixelPerfectClick = true;
            this.characterName = new Phaser.Text(game, 0, 0, characterInfo.characterName, {
                font: Game_17.Game.CARNIVAL_FONT,
                fill: '#e7e0ce',
                stroke: '#811e23',
                strokeThickness: game.width * 0.015,
                fontSize: TextUtil_13.TextUtil.getFontSize(this.game.width, 23)
            });
            TextUtil_13.TextUtil.scaleDownSingleLineTextToWidth(this.characterName, this.backplate.width * 0.20);
            this.characterName.centerX = this.backplate.centerX;
            this.characterName.centerY = this.backplate.centerY - (this.backplate.height * 0.365);
            this.add(this.characterName);
            this.textField = new TypewriterText_3.TypewriterText(game, 0, 0, '', {
                font: Game_17.Game.DEFAULT_FONT,
                fill: 'white',
                fontSize: TextUtil_13.TextUtil.getFontSize(this.game.width, 26),
                wordWrap: true,
                wordWrapWidth: this.backplate.width * 0.75
            });
            this.textField.setShadow(this.game.width * 0.005, this.game.width * 0.005, 'rgba(17, 91, 166, 1)', 0, true, true);
            this.textField.x = this.backplate.centerX - (this.backplate.width * 0.375);
            this.add(this.textField);
            this.prompt = new ArrowPrompt_3.ArrowPrompt(game);
            this.prompt.width = this.backplate.width * 0.14;
            this.prompt.scale.y = this.prompt.scale.x;
            this.prompt.x = this.backplate.centerX - (this.backplate.width * 0.012);
            this.prompt.y = this.backplate.centerY + (this.backplate.height * 0.299);
            this.prompt.visible = false;
            this.add(this.prompt);
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.visible = true;
                this.right = 0;
                yield PromiseUtil_19.PromiseUtil.tween(this.game, this.game.add.tween(this)
                    .to({ x: 0 }, 500, Phaser.Easing.Sinusoidal.Out, true));
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_19.PromiseUtil.tween(this.game, this.game.add.tween(this)
                    .to({ left: this.game.width }, 500, Phaser.Easing.Sinusoidal.In, true));
                this.visible = false;
                this.textField.text = '';
            });
        }
        writeMessage(message) {
            return __awaiter(this, void 0, void 0, function* () {
                this.textField.text = message;
                this.textField.fontSize = TextUtil_13.TextUtil.getFontSize(this.game.width, 26);
                TextUtil_13.TextUtil.scaleDownTextToHeight(this.textField, this.backplate.height * 0.45);
                this.textField.centerY = this.backplate.centerY - (this.backplate.height * 0.065);
                yield this.textField.typewriter(CharacterDialog.TYPEWRITER_DELAY, 'typewriter_loop');
            });
        }
        waitForTap() {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    this.onChildInputDown.addOnce(() => {
                        this.prompt.visible = false;
                        resolve();
                    });
                    this.prompt.visible = true;
                });
            });
        }
    }
    CharacterDialog.TYPEWRITER_DELAY = 5;
    exports.CharacterDialog = CharacterDialog;
});
define("component/PassAndPlayShowable", ["require", "exports", "component/MessageBar", "switchback-common/util/PromiseUtil"], function (require, exports, MessageBar_4, PromiseUtil_20) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PassAndPlayShowable extends Phaser.Group {
        constructor(game, text, fontSize) {
            super(game);
            this.messageBar = new MessageBar_4.MessageBar(this.game, text, fontSize);
            this.messageBar.centerX = 0;
            this.rightFigure = new Phaser.Image(this.game, 0, 0, 'right-figure');
            this.rightFigure.height = this.game.height * 0.3;
            this.rightFigure.scale.x = this.rightFigure.scale.y;
            this.rightFigure.left = this.game.width * 0.5;
            this.rightFigure.bottom = this.messageBar.top + this.messageBar.height * 0.3;
            this.leftFigure = new Phaser.Image(this.game, 0, 0, 'left-figure');
            this.leftFigure.height = this.game.height * 0.2;
            this.leftFigure.scale = this.rightFigure.scale;
            this.leftFigure.bottom = this.rightFigure.bottom;
            this.deviceImage = new Phaser.Image(this.game, 0, 0, 'phone');
            this.deviceImage.scale = this.rightFigure.scale;
            this.deviceImage.bottom = this.leftFigure.bottom;
            this.deviceImage.right = this.game.width * -0.5;
            this.leftFigure.right = this.deviceImage.left;
            this.messageBar.top = this.game.height;
            this.add(this.deviceImage);
            this.add(this.leftFigure);
            this.add(this.rightFigure);
            this.add(this.messageBar);
        }
        set text(text) {
            this.messageBar.text = text;
        }
        set fontSize(fontSize) {
            this.messageBar.fontSize = fontSize;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    // Total duration of animation
                    const durationMs = 1000;
                    // Message bar slides in from the bottom of the screen
                    this.messageBar.worldPosition.y = this.game.height;
                    this.game.add.tween(this.messageBar).to({ y: 0 }, 300, Phaser.Easing.Sinusoidal.InOut, true);
                    // Figure slides in from left
                    this.game.add.tween(this.leftFigure).to({ right: 0 }, durationMs / 2, Phaser.Easing.Sinusoidal.InOut, true).onComplete.add(() => {
                        // Figure returns to left after 1s delay
                        this.game.add.tween(this.leftFigure).to({ right: this.game.width * -0.5 - this.deviceImage.width }, durationMs / 2, Phaser.Easing.Sinusoidal.InOut, true, durationMs);
                    });
                    const jumpSize = this.rightFigure.height * 0.15;
                    // Device slides in with left figure
                    this.game.add.tween(this.deviceImage).to({ centerX: 0 }, durationMs / 2, Phaser.Easing.Sinusoidal.InOut, true);
                    // Figure slides in from right, after 200m delay
                    this.game.add.tween(this.rightFigure).to({ left: 0 }, durationMs / 2, Phaser.Easing.Sinusoidal.InOut, true, 200).onComplete.add(() => {
                        // Figure jumps up and down
                        // Resolve promise when we're finished jumping
                        this.game.add.tween(this.rightFigure)
                            .to({ bottom: this.rightFigure.bottom - jumpSize }, 200, Phaser.Easing.Sinusoidal.InOut, true, durationMs / 2, 3, true)
                            .onComplete.addOnce(resolve);
                        // Device jumps up and down with right figure
                        this.game.add.tween(this.deviceImage).to({ bottom: this.deviceImage.bottom - jumpSize }, 200, Phaser.Easing.Sinusoidal.InOut, true, durationMs / 2, 3, true);
                    });
                });
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                // Slide right figure off screen
                this.game.add.tween(this.rightFigure).to({ left: this.game.width * 0.5 }, 200, Phaser.Easing.Sinusoidal.InOut, true);
                this.game.add.tween(this.deviceImage).to({ left: this.game.width * 0.5 }, 200, Phaser.Easing.Sinusoidal.InOut, true)
                    // Return back to initial position
                    .onComplete.addOnce(() => { this.deviceImage.right = this.game.width * -0.5; });
                // Slide message bar off bottom of screen
                yield PromiseUtil_20.PromiseUtil.tween(this.game, this.game.add.tween(this.messageBar).to({ top: this.game.height }, 500, Phaser.Easing.Sinusoidal.InOut, true, 300));
            });
        }
    }
    exports.PassAndPlayShowable = PassAndPlayShowable;
});
define("component/HeaderBar", ["require", "exports", "Game"], function (require, exports, Game_18) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HeaderBar extends Phaser.Group {
        constructor(game, text, fontSize = 60) {
            super(game);
            const background = new Phaser.Image(this.game, 0, 0, 'cta-header');
            background.width = this.game.width;
            background.scale.y = background.scale.x;
            this.add(background);
            background.inputEnabled = true;
            this.textField = new Phaser.Text(this.game, background.width * 0.5, background.height * 0.45, text, {
                font: Game_18.Game.CARNIVAL_FONT,
                fontSize: fontSize,
                wordWrap: true,
                wordWrapWidth: background.width * 0.45,
                fill: Game_18.Game.TEXT_COLOR_CREAM,
                stroke: Game_18.Game.TEXT_COLOR_PINK,
                strokeThickness: Game_18.Game.TEXT_STROKE_THICKNESS_LARGE,
                align: 'center'
            });
            this.textField.setScaleMinMax(1);
            this.textField.anchor.set(0.5);
            this.add(this.textField);
        }
        set text(text) {
            this.textField.text = text;
        }
    }
    exports.HeaderBar = HeaderBar;
});
define("minigame/woody/throw/Location", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Location {
        constructor(lane, horizontalPos) {
            this.lane = lane;
            this.horizontalPos = horizontalPos;
        }
        getValidPositionsInLane(laneCount) {
            const distanceFromCamera = laneCount - this.lane - 1;
            let positions;
            switch (distanceFromCamera) {
                // Foreground lane, closest to UI, bottom of screen
                case 0: {
                    positions = 2;
                    break;
                }
                // Midground lane, middle of screen
                case 1: {
                    positions = 3;
                    break;
                }
                // Background lane, closest to sky
                default: {
                    positions = 4;
                    break;
                }
            }
            return positions;
        }
    }
    exports.Location = Location;
});
define("minigame/woody/throw/Bullseye", ["require", "exports", "switchback-common/util/PromiseUtil", "minigame/woody/throw/Location"], function (require, exports, PromiseUtil_21, Location_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Bullseye extends Phaser.Group {
        constructor(game, laneCount) {
            super(game);
            this.laneCount = laneCount;
            // Start Bullseye off in the middle of the screen
            this.location = Bullseye.CENTER_LOCATION;
            this.bullseyeSprite = new Phaser.Sprite(game, 0, 0, "bullseye");
            this.bullseyeSprite.anchor.set(0.5);
            this.bullseyeSprite.animations.add('run', null, Bullseye.BASE_FRAME_RATE, true).play();
            this.runLoop = this.game.add.sound('woody-bullseye-run', 1.0, true);
            this.add(this.bullseyeSprite);
            this.width = this.game.width * 0.9;
            this.scale.y = this.scale.x;
            this.origScale = this.scale.x;
        }
        play() {
            this.bullseyeSprite.animations.play('run');
            this.runLoop.play(null, 0, 1.0, true, true);
            this.game.add.tween(this.runLoop).from({ volume: 0 }, 500, Phaser.Easing.Linear.None, true);
        }
        /**
         * Scales based on the lane Bullseye is in
         * (0: background, 1: midground, 2: foreground)
         */
        getScaleForLane() {
            let range = this.location.lane / (this.laneCount - 1);
            return this.origScale * (0.75 + 0.5 * range);
        }
        moveToPoint(point) {
            return __awaiter(this, void 0, void 0, function* () {
                const scale = this.getScaleForLane();
                // Scale duration by distance
                const speedPxPerS = 700;
                const distance = point.distance(new Phaser.Point(this.bullseyeSprite.worldPosition.x, this.bullseyeSprite.worldPosition.y));
                const durationMs = (1000 / speedPxPerS) * distance;
                yield Promise.all([
                    PromiseUtil_21.PromiseUtil.tween(this.game, this.game.add.tween(this.bullseyeSprite.animations.getAnimation('run')).to({ speed: Bullseye.ACCELERATED_FRAME_RATE }, durationMs * 0.5, Phaser.Easing.Linear.None, true, 0, 0, true)),
                    PromiseUtil_21.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ centerX: point.x, y: point.y }, durationMs, Phaser.Easing.Sinusoidal.In, true, 0)),
                    PromiseUtil_21.PromiseUtil.tween(this.game, this.game.add.tween(this.bullseyeSprite.scale).to({ x: scale, y: scale }, durationMs, Phaser.Easing.Sinusoidal.In, true, 0))
                ]);
            });
        }
        getBasketWorldPosition() {
            return new Phaser.Point((this.bullseyeSprite.worldPosition.x + this.bullseyeSprite.width * 0.04), (this.bullseyeSprite.worldPosition.y - this.bullseyeSprite.height * 0.37));
        }
        destroy() {
            this.runLoop.destroy();
            super.destroy();
        }
    }
    Bullseye.CENTER_LOCATION = new Location_1.Location(1, 1);
    Bullseye.BASE_FRAME_RATE = 17;
    Bullseye.ACCELERATED_FRAME_RATE = 34;
    exports.Bullseye = Bullseye;
});
define("minigame/woody/throw/Cactus", ["require", "exports", "switchback-common/util/ScaleUtil"], function (require, exports, ScaleUtil_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Cactus extends Phaser.Sprite {
        constructor(game, frequency, height, y) {
            super(game, 0, 0, "cactus");
            ScaleUtil_8.scaleToBounds(this, height, height);
            this.frequency = frequency;
            this.y = y;
            // Gets called at the end of each tween
            this.moveAcrossScreen();
        }
        moveAcrossScreen() {
            // Position off screen to start
            this.right = 0;
            // Generate random delay to stagger cacti placement
            let delay = Math.random() * 2000;
            this.game.add.tween(this)
                .to({ left: this.game.width }, this.frequency, Phaser.Easing.Linear.None, true, delay)
                .onComplete.addOnce(this.moveAcrossScreen, this);
        }
    }
    exports.Cactus = Cactus;
});
define("component/fx/Snatch", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_22) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Snatch extends Phaser.Group {
        show(posX, posY, scaleFactor = 1.125) {
            return __awaiter(this, void 0, void 0, function* () {
                let animDurationMS = 500;
                // Starting positions
                let pos = [posX, posY];
                let promises = [];
                // Spark
                // Like a spark in the dark
                let spark = new Phaser.Image(this.game, 0, 0, 'snatch-radialfx');
                // The base scale
                let finalScale = (this.game.width / spark.width) * scaleFactor;
                spark.scale.x = spark.scale.y = finalScale * 0.2;
                spark.anchor.set(0.5);
                spark.pivot.set(0.5, 0.5);
                spark.angle = Math.random() * 360;
                spark.position.set(pos[0], pos[1]);
                // for a group add is preferred over addChild - one belongs to Phaser and ones belongs to PIXI -
                // the one that belongs to PIXI (addChild) doesn't go through the phaser 'route' of adding things to groups. so it doesn't get affected by things like inputEnableChildren, etc
                this.add(spark);
                // Tweenified
                // angle
                let tween = this.game.add.tween(spark).to({ angle: spark.angle + 45 }, animDurationMS, Phaser.Easing.Linear.None, true);
                // scale
                this.game.add.tween(spark.scale).to({ x: finalScale, y: finalScale }, animDurationMS, Phaser.Easing.Linear.None, true);
                // alpha (notice it starts at half duration)
                this.game.add.tween(spark).to({ alpha: 0 }, animDurationMS / 2.0, Phaser.Easing.Linear.None, true, animDurationMS / 2.0);
                promises.push(PromiseUtil_22.PromiseUtil.tween(this.game, tween));
                // stars
                let colors = ['pink', 'blue', 'green'];
                let count = 4; // how many stars to show
                let stars = []; // stored so we can destroy them later
                // how much each star moves
                let distsX = [.3, 0.2, -0.2, -0.3]; // on X
                // Y movement is done in 2 tweens (up and down)
                let distsYA = [-.125, -.25, -.25, -.125];
                let distsYB = [-.0, -.125, -.125, -.0];
                let angleDifs = [45, 45, -45, -45];
                for (let i = 0; i < count; i++) {
                    let star = new Phaser.Image(this.game, 0, 0, 'snatch-star-' + colors[i % colors.length]);
                    this.add(star);
                    star.scale.x = star.scale.y = finalScale * 0.25;
                    star.anchor.set(0.5);
                    star.pivot.set(0.5, 0.5);
                    star.position.set(pos[0], pos[1]);
                    // X movement
                    this.game.add.tween(star).to({ x: star.x + (this.game.width * distsX[i]) }, animDurationMS, Phaser.Easing.Linear.None, true);
                    // Y movement
                    tween = this.game.add.tween(star).to({ y: star.y + (this.game.width * distsYA[i]) }, animDurationMS / 2.0, Phaser.Easing.Quadratic.Out, true);
                    tween.chain(this.game.add.tween(star).to({ y: star.y + (this.game.width * distsYB[i]) }, animDurationMS / 2.0, Phaser.Easing.Quadratic.In, false));
                    // Rotation
                    tween = this.game.add.tween(star).to({ angle: star.angle + angleDifs[i] }, animDurationMS, Phaser.Easing.Linear.None, true);
                    promises.push(PromiseUtil_22.PromiseUtil.tween(this.game, tween));
                    // Alpha
                    const alphaTween = this.game.add.tween(star).to({ alpha: 0 }, animDurationMS / 2.0, Phaser.Easing.Linear.None, true, animDurationMS / 2.0);
                    promises.push(PromiseUtil_22.PromiseUtil.tween(this.game, alphaTween));
                    stars.push(star);
                }
                let allPromises = Promise.all(promises);
                // destroy everything when it ends
                allPromises.then(() => {
                    // need to check if these assets still exist before destroying them as they may have 
                    // been cleaned up if the state it was attached to was shutdown
                    if (spark.exists) {
                        spark.destroy();
                    }
                    for (let star of stars) {
                        if (star.exists) {
                            star.destroy();
                        }
                    }
                });
                yield allPromises;
            });
        }
    }
    exports.Snatch = Snatch;
});
define("minigame/woody/throw/ThrowConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ThrowConfig {
        parse(data) {
            this.throwsPerPlayer = data.throwsPerPlayer || 6;
        }
    }
    exports.ThrowConfig = ThrowConfig;
});
define("minigame/woody/throw/ThrowDifficultyConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ThrowDifficultyConfig {
        parse(data) {
            this.numberOfLanes = data.numberOfLanes || 5;
        }
    }
    exports.ThrowDifficultyConfig = ThrowDifficultyConfig;
});
define("component/dialog/BasicDialog", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil", "Game", "model/CharacterInfo", "component/ArrowPrompt", "switchback-common/component/text/TypewriterText"], function (require, exports, PromiseUtil_23, TextUtil_14, Game_19, CharacterInfo_2, ArrowPrompt_4, TypewriterText_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class BasicDialog extends Phaser.Group {
        constructor(game, character, blockInput = true) {
            super(game);
            this.visible = false;
            if (blockInput) {
                this.inputEnableChildren = true;
            }
            const characterInfo = CharacterInfo_2.CharacterInfo.getInfo(character);
            this.backplate = new Phaser.Image(game, 0, 0, 'character-dialog-backplate');
            this.backplate.width = game.width;
            this.backplate.scale.y = this.backplate.scale.x;
            this.add(this.backplate);
            this.characterName = new Phaser.Text(game, 0, 0, characterInfo.characterName, {
                font: Game_19.Game.CARNIVAL_FONT,
                fill: '#e7e0ce',
                stroke: '#811e23',
                strokeThickness: game.width * 0.015,
                fontSize: TextUtil_14.TextUtil.getFontSize(this.game.width, 23)
            });
            TextUtil_14.TextUtil.scaleDownSingleLineTextToWidth(this.characterName, this.backplate.width * 0.20);
            this.characterName.centerX = this.backplate.centerX;
            this.characterName.centerY = this.backplate.centerY - (this.backplate.height * 0.365);
            this.add(this.characterName);
            this.textField = new TypewriterText_4.TypewriterText(game, 0, 0, '', {
                font: Game_19.Game.DEFAULT_FONT,
                fill: 'white',
                fontSize: TextUtil_14.TextUtil.getFontSize(this.game.width, 26),
                wordWrap: true,
                wordWrapWidth: this.backplate.width * 0.75
            });
            this.textField.setShadow(this.game.width * 0.005, this.game.width * 0.005, 'rgba(17, 91, 166, 1)', 0, true, true);
            this.textField.x = this.backplate.centerX - (this.backplate.width * 0.375);
            this.add(this.textField);
            this.prompt = new ArrowPrompt_4.ArrowPrompt(this.game, 0, 0);
            this.prompt.width = this.backplate.width * 0.14;
            this.prompt.scale.y = this.prompt.scale.x;
            this.prompt.x = this.backplate.centerX - (this.backplate.width * 0.012);
            this.prompt.y = this.backplate.centerY + (this.backplate.height * 0.299);
            this.prompt.visible = false;
            this.add(this.prompt);
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this.right = 0;
                this.visible = true;
                // slides right to the center of screen
                yield PromiseUtil_23.PromiseUtil.tween(this.game, this.game.add.tween(this)
                    .to({ centerX: this.game.width * 0.5 }, 500, Phaser.Easing.Sinusoidal.In, true));
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                // slides right till it is out of the screen
                yield PromiseUtil_23.PromiseUtil.tween(this.game, this.game.add.tween(this)
                    .to({ left: this.game.width }, 500, Phaser.Easing.Sinusoidal.In, true));
                this.visible = false;
                this.textField.text = '';
            });
        }
        writeMessage(message) {
            return __awaiter(this, void 0, void 0, function* () {
                this.textField.text = message;
                this.textField.fontSize = TextUtil_14.TextUtil.getFontSize(this.game.width, 26);
                TextUtil_14.TextUtil.scaleDownTextToHeight(this.textField, this.backplate.height * 0.45);
                this.textField.centerY = this.backplate.centerY - (this.backplate.height * 0.065);
                yield this.textField.typewriter(BasicDialog.TYPEWRITER_DELAY, 'typewriter_loop');
            });
        }
        changeCharName(newCharName) {
            this.characterName.text = newCharName;
        }
        waitForTap() {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    this.onChildInputDown.addOnce(() => {
                        this.prompt.visible = false;
                        resolve();
                    });
                    this.prompt.visible = true;
                });
            });
        }
    }
    BasicDialog.TYPEWRITER_DELAY = 5;
    exports.BasicDialog = BasicDialog;
});
define("minigame/woody/throw/ThrowState", ["require", "exports", "switchback-common/util/TextUtil", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "switchback-common/extension/Easing", "model/ToyStoryConfig", "component/dialog/CharacterDialog", "component/Curtain", "component/PassAndPlayShowable", "component/button/TextArrowButton", "component/MessageBar", "component/HeaderBar", "Game", "StateNames", "state/CutSceneState", "minigame/woody/throw/Location", "minigame/woody/throw/Bullseye", "minigame/woody/throw/Cactus", "logic/GameController", "Character", "component/fx/Snatch", "minigame/woody/throw/ThrowConfig", "state/GameBaseState", "minigame/woody/throw/ThrowDifficultyConfig", "component/dialog/BasicDialog"], function (require, exports, TextUtil_15, PromiseUtil_24, ScaleUtil_9, LocalizationManager_15, Easing_2, ToyStoryConfig_10, CharacterDialog_1, Curtain_2, PassAndPlayShowable_1, TextArrowButton_3, MessageBar_5, HeaderBar_1, Game_20, StateNames_13, CutSceneState_3, Location_2, Bullseye_1, Cactus_1, GameController_16, Character_2, Snatch_1, ThrowConfig_1, GameBaseState_3, ThrowDifficultyConfig_1, BasicDialog_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ThrowState extends GameBaseState_3.GameBaseState {
        constructor() {
            super(...arguments);
            this.isBullseyeMoving = false; // keeps track of when bullseye is moving his position
        }
        init(...params) {
            this.stateName = StateNames_13.StateNames.GAME_WOODY_THROW;
            this._trackMicroName = "Toss";
            super.init(params);
            this.MAX_TIMEOUT_STUCK_MS = 120000;
            this.roundTimeMs = 0;
            this.isPlaying = false;
            this.config = this.loadConfig(ThrowConfig_1.ThrowConfig);
            this.difficultyConfig = this.loadDifficultyConfig(ThrowDifficultyConfig_1.ThrowDifficultyConfig);
        }
        preload() {
            super.preload();
            this.game.load.image('arrow', 'assets/minigame/woody/arrow.png');
            this.game.load.image('basket-symbol', 'assets/minigame/woody/basket_symbol.png');
            this.game.load.image('cactus', 'assets/minigame/woody/cactus.png');
            this.game.load.image('target', 'assets/minigame/woody/target.png');
            this.game.load.atlas('throw-target-shard', 'assets/minigame/woody/target-tumbling.png', 'assets/minigame/woody/target-tumbling.json');
            this.game.load.atlas('bullseye', 'assets/minigame/woody/bullseye-run-sheet.png', 'assets/minigame/woody/bullseye-run-sheet.json');
            this.game.load.spritesheet('target-anim', 'assets/minigame/woody/target_anim.png', 512, 256);
            this.game.load.image('target-symbol', 'assets/minigame/woody/target_symbol.png');
            this.game.load.image('target-shadow', 'assets/minigame/woody/target_shadow.png');
            this.game.load.image('carriage-roof', 'assets/minigame/woody/carriage_roof.png');
            this.game.load.image('clouds', 'assets/minigame/woody/clouds.png');
            this.game.load.image('bg-detail', 'assets/minigame/woody/bg_detail.png');
            this.game.load.image('throw-tiling-bg', 'assets/minigame/woody/throw-tiling-bg.png');
            this.game.load.image('lasso-floor-tiling', 'assets/minigame/woody/lasso-floor-tiling.png');
            this.game.load.image('throw-success-ray', 'assets/minigame/woody/success_ray.png');
            this.game.load.image('snatch-radialfx', 'assets/minigame/woody/snatch_radialfx.png');
            this.game.load.image('snatch-star-green', 'assets/minigame/woody/star_green.png');
            this.game.load.image('snatch-star-pink', 'assets/minigame/woody/star_pink.png');
            this.game.load.image('snatch-star-blue', 'assets/minigame/woody/star_blue.png');
            // Instructions
            this.game.load.image('instruction-graphic-woody-throw', 'assets/dialog/instruction_graphic_woody3.png');
            this.game.load.image('character-dialog-woody', 'assets/dialog/character_dialog_woody.png');
            this.game.load.audio('woody-target-basket', ['assets/sound/woody/target_basket.m4a', 'assets/sound/woody/target_basket.ogg']);
            this.game.load.audio('woody-toss-target-1', ['assets/sound/woody/toss_target_1.m4a', 'assets/sound/woody/toss_target_1.ogg']);
            this.game.load.audio('woody-toss-target-2', ['assets/sound/woody/toss_target_2.m4a', 'assets/sound/woody/toss_target_2.ogg']);
            this.game.load.audio('woody-toss-target-3', ['assets/sound/woody/toss_target_3.m4a', 'assets/sound/woody/toss_target_3.ogg']);
            this.game.load.audio('woody-target-break-1', ['assets/sound/woody/target_break_1.m4a', 'assets/sound/woody/target_break_1.ogg']);
            this.game.load.audio('woody-target-break-2', ['assets/sound/woody/target_break_2.m4a', 'assets/sound/woody/target_break_2.ogg']);
            this.game.load.audio('woody-target-break-3', ['assets/sound/woody/target_break_3.m4a', 'assets/sound/woody/target_break_3.ogg']);
            this.game.load.audio('woody-train-loop', ['assets/sound/woody/train_loop.m4a', 'assets/sound/woody/train_loop.ogg']);
            this.game.load.audio('woody-bullseye-run', ['assets/sound/woody/bullseye_run.m4a', 'assets/sound/woody/bullseye_run.ogg']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.trackGameState(null, ThrowState.TRACKING_MICRO_NAME);
                this.playerIndex = 0;
                this.targetsCaught = 0;
                this.throwsRemaining = this.config.throwsPerPlayer;
                this.horizonLineHeight = this.game.height * 0.42;
                this.uiAreaHeight = this.game.height * 0.12;
                // Active game area starts below horizon (this.horizonLineHeight)
                // and ends at UI area (this.uiAreaHeight)
                this.horizontalPadding = this.game.width * 0.1;
                this.gameAreaHeight = this.game.height - this.horizonLineHeight - this.uiAreaHeight;
                this.gameAreaWidth = this.game.width - (this.horizontalPadding * 2);
                this.laneHeight = this.gameAreaHeight / this.difficultyConfig.numberOfLanes;
                // Tiling room background
                const overPad = this.game.width * 0.05;
                this.background = new Phaser.TileSprite(this.game, -overPad, -overPad, this.game.width + (overPad * 2), this.game.height + (overPad * 2), "throw-tiling-bg");
                const bgTextureScale = this.background.height / this.background.texture.height;
                this.background.tileScale.y = bgTextureScale;
                this.background.inputEnabled = true;
                const scrollSpeed = this.game.width * 0.1;
                // Clouds
                const tilingClouds = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.horizonLineHeight, "clouds");
                const cloudTextureScale = tilingClouds.height / tilingClouds.texture.height;
                tilingClouds.tileScale.set(cloudTextureScale, cloudTextureScale);
                tilingClouds.bottom = this.game.height * 0.175;
                tilingClouds.autoScroll(scrollSpeed * 0.5, 0);
                // Background detail
                const bgDetail = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height * 0.075, "bg-detail");
                const bgDetailScale = bgDetail.height / bgDetail.texture.height;
                bgDetail.tileScale.set(bgDetailScale, bgDetailScale);
                bgDetail.bottom = this.horizonLineHeight;
                bgDetail.autoScroll(scrollSpeed, 0);
                const floorboards = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height - this.horizonLineHeight, "lasso-floor-tiling");
                const floorboardScale = floorboards.height / floorboards.texture.height;
                floorboards.tileScale.set(floorboardScale, floorboardScale);
                floorboards.top = this.horizonLineHeight;
                floorboards.autoScroll(scrollSpeed * 1.5, 0);
                // Carriage roof
                const carriageRoof = new Phaser.Image(this.game, 0, 0, "carriage-roof");
                carriageRoof.width = this.game.width;
                carriageRoof.height = this.uiAreaHeight;
                carriageRoof.bottom = this.game.height + this.game.height * 0.01;
                // Add carriage roof shake animation
                this.game.add.tween(carriageRoof).to({ y: carriageRoof.y - this.game.height * 0.01 }, 200, Easing_2.shake(0.1, 0.1), true, 0, Infinity);
                // Bottom UI
                // Background cacti
                const cactus1 = new Cactus_1.Cactus(this.game, 2500, this.game.height * 0.05, this.horizonLineHeight - this.game.height * 0.025);
                const cactus2 = new Cactus_1.Cactus(this.game, 2200, this.game.height * 0.08, this.horizonLineHeight);
                const cactus3 = new Cactus_1.Cactus(this.game, 1800, this.game.height * 0.1, this.horizonLineHeight + this.game.height * 0.05);
                // Foreground cacti
                const cactus4 = new Cactus_1.Cactus(this.game, 800, this.game.height * 0.3, 0);
                cactus4.top = this.game.height * 0.8;
                // Arrow overlay (visible for first throw only)
                this.arrow = new Phaser.Image(this.game, 0, 0, "arrow");
                ScaleUtil_9.scaleToBounds(this.arrow, this.game.width * 0.4, this.game.height * 0.65);
                this.arrow.bottom = carriageRoof.top;
                this.arrow.centerX = this.game.width * 0.5;
                this.arrow.alpha = 0;
                // Targets left to throw
                this.throwsRemainingIcon = new Phaser.Image(this.game, 0, 0, "target-symbol");
                this.throwsRemainingIcon.height = this.game.height * 0.15;
                this.throwsRemainingIcon.width = this.throwsRemainingIcon.height;
                this.throwsRemainingIcon.left = 0;
                this.throwsRemainingIcon.bottom = this.game.height;
                this.throwsRemainingText = new Phaser.Text(this.game, 0, 0, "0", {
                    font: Game_20.Game.DEFAULT_FONT,
                    fontSize: TextUtil_15.TextUtil.getFontSize(this.game.width, 30),
                    fill: "white",
                    align: 'center',
                    stroke: '#1a4ea2',
                    strokeThickness: 8
                });
                ScaleUtil_9.scaleToBounds(this.throwsRemainingText, this.throwsRemainingIcon.height * 0.6, this.throwsRemainingIcon.height * 0.6);
                this.throwsRemainingText.bottom = this.throwsRemainingIcon.bottom;
                this.throwsRemainingText.left = this.throwsRemainingIcon.width * 0.6;
                // Targets caught
                this.targetsCaughtIcon = new Phaser.Image(this.game, 0, 0, "basket-symbol");
                this.targetsCaughtIcon.height = this.game.height * 0.15;
                this.targetsCaughtIcon.width = this.targetsCaughtIcon.height;
                this.targetsCaughtIcon.right = this.game.width;
                this.targetsCaughtIcon.bottom = this.game.height;
                this.targetsCaughtText = new Phaser.Text(this.game, 0, 0, "0", {
                    font: Game_20.Game.DEFAULT_FONT,
                    fontSize: TextUtil_15.TextUtil.getFontSize(this.game.width, 30),
                    fill: "white",
                    align: 'center',
                    stroke: '#1a4ea2',
                    strokeThickness: 8
                });
                ScaleUtil_9.scaleToBounds(this.targetsCaughtText, this.targetsCaughtIcon.height * 0.6, this.targetsCaughtIcon.height * 0.6);
                this.targetsCaughtText.bottom = this.targetsCaughtIcon.bottom;
                this.targetsCaughtText.right = this.game.width - (this.targetsCaughtIcon.width * 0.6);
                // Bullseye
                this.bullseye = new Bullseye_1.Bullseye(this.game, this.difficultyConfig.numberOfLanes);
                this.bullseye.left = this.game.width;
                this.bullseye.centerY = this.game.height * 0.5;
                this.targetShard1 = new Phaser.Sprite(this.game, 0, 0, 'throw-target-shard');
                this.targetShard1.width = this.game.width * 0.5;
                this.targetShard1.anchor.set(0.5);
                this.targetShard1.scale.y = this.targetShard1.scale.x;
                this.targetShard1.alpha = 0;
                this.targetShard1.animations.add('loop', null, 30, true).play();
                this.targetShard2 = new Phaser.Sprite(this.game, 0, 0, 'throw-target-shard');
                this.targetShard2.scale.set(this.targetShard1.scale.x);
                this.targetShard2.anchor.set(0.5);
                this.targetShard2.alpha = 0;
                this.targetShard2.animations.add('loop', null, 24, true).play();
                this.targetShardOrigScale = this.targetShard1.scale.x;
                this.snatchFx = new Snatch_1.Snatch(this.game);
                this.dialog = new CharacterDialog_1.CharacterDialog(this.game, Character_2.Character.WOODY);
                this.dialog.centerY = this.game.height * 0.15;
                // Add background
                this.add.existing(this.background);
                this.add.existing(tilingClouds);
                this.add.existing(bgDetail);
                this.add.existing(floorboards);
                // Add background cacti
                this.add.existing(cactus1);
                this.add.existing(cactus2);
                this.add.existing(cactus3);
                // Add bullseye
                this.add.existing(this.bullseye);
                this.add.existing(this.targetShard1);
                this.add.existing(this.targetShard2);
                this.add.existing(this.arrow);
                this.add.existing(this.snatchFx);
                // Add foreground cacti
                this.add.existing(cactus4);
                // Add bottom toolbar/UI
                this.add.existing(carriageRoof);
                this.add.existing(this.throwsRemainingIcon);
                this.add.existing(this.targetsCaughtIcon);
                this.add.existing(this.throwsRemainingText);
                this.add.existing(this.targetsCaughtText);
                // Instructions and Pass & Play
                this.curtain = new Curtain_2.Curtain(this.game, true);
                this.add.existing(this.curtain);
                this.passAndPlayAnimation = new PassAndPlayShowable_1.PassAndPlayShowable(this.game);
                this.passAndPlayAnimation.x = this.game.width * 0.5;
                this.passAndPlayAnimation.y = this.game.height * 0.5;
                this.add.existing(this.passAndPlayAnimation);
                this.readyButton = new TextArrowButton_3.TextArrowButton(this.game, LocalizationManager_15.LocalizationManager.Instance.text('common.ready').toUpperCase());
                this.readyButton.centerX = this.game.width * 0.5;
                this.readyButton.centerY = this.game.height * 0.785;
                this.readyButton.alpha = 0;
                this.readyButton.startPulse();
                this.add.existing(this.readyButton);
                this.tutorialTopMessageBar = new MessageBar_5.MessageBar(this.game, LocalizationManager_15.LocalizationManager.Instance.text("target_toss.instructions_intro_swipe"));
                this.tutorialTopMessageBar.centerX = this.game.width * 0.5;
                this.tutorialTopMessageBar.bottom = 0;
                this.add.existing(this.tutorialTopMessageBar);
                this.tutorialBottomMessageBar = new MessageBar_5.MessageBar(this.game, LocalizationManager_15.LocalizationManager.Instance.text("target_toss.instructions_intro_aim"));
                this.tutorialBottomMessageBar.centerX = this.game.width * 0.5;
                this.tutorialBottomMessageBar.top = this.game.height;
                this.add.existing(this.tutorialBottomMessageBar);
                this.instructionImage = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.425, 'instruction-graphic-woody-throw');
                this.instructionImage.alpha = 0;
                this.instructionImage.anchor.set(0.5);
                this.add.existing(this.instructionImage);
                this.headerBar = new HeaderBar_1.HeaderBar(this.game, LocalizationManager_15.LocalizationManager.Instance.text("common.how_to_play"));
                this.headerBar.centerX = this.game.width * 0.5;
                this.headerBar.bottom = 0;
                this.add.existing(this.headerBar);
                this.trainLoop = this.game.add.sound('woody-train-loop', 1.0, true);
                this.trainLoop.play();
                this.game.add.tween(this.trainLoop).from({ volume: 0 }, 500, Phaser.Easing.Linear.None, true);
                yield this.dismissOverlay();
                if (ToyStoryConfig_10.ToyStoryConfig.Instance.skipGames) {
                    this.onGameOver(20);
                }
                else {
                    this.startNewRound(true);
                }
            });
        }
        // Shows the guiding arrow as well as centering Bullseye
        doNewThrow() {
            return __awaiter(this, void 0, void 0, function* () {
                // Specifying the first location so that we can correctly place the tutorial arrow
                yield this.moveBullseye(Bullseye_1.Bullseye.CENTER_LOCATION);
                // Add new target to throw
                this.resetTargets();
                this.staticTarget.inputEnabled = this.isPlaying;
                this.game.add.tween(this.arrow).to({ alpha: 1 }, 400, Phaser.Easing.Sinusoidal.InOut, true);
                // Remove tutorial arrow on first throw
                this.staticTarget.events.onInputDown.addOnce((sprite, pointer) => {
                    // The arrow will always be shown at the same time at the top message bar
                    // Hide the message bar when the arrow is hidden
                    this.game.add.tween(this.tutorialTopMessageBar).to({ bottom: 0 }, 200, Phaser.Easing.Sinusoidal.Out, true);
                    this.game.tweens.removeFrom(this.arrow);
                    this.game.add.tween(this.arrow).to({ alpha: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true);
                }, this);
            });
        }
        // Starts a new round (shows the guide, centers bullseye and resets the targets)
        startNewRound(isFirstRound) {
            return __awaiter(this, void 0, void 0, function* () {
                this.roundTimeMs = 0;
                this.isPlaying = false;
                if (isFirstRound) {
                    // Display instructions
                    yield this.showThenHideInstructions();
                    this.updateTargetsCaught(0);
                }
                else {
                    // Set new player
                    this.playerIndex++;
                    if (this.playerIndex < ThrowState.players.length) {
                        // Display pass and play
                        yield this.passToNextPlayer();
                        // Display instructions
                        yield this.showThenHideInstructions();
                    }
                    else {
                        // Everyone has played; Launch exit sequence
                        this.onGameOver(this.targetsCaught);
                        return;
                    }
                }
                this.isPlaying = true;
                this.bullseye.play();
                this.trackMicroStart();
                yield this.doNewThrow();
                // Reset throws remaining
                this.updateThrowsRemaining(this.config.throwsPerPlayer);
            });
        }
        showThenHideInstructions() {
            return __awaiter(this, void 0, void 0, function* () {
                this.trackGameInstructions(ThrowState.TRACKING_MICRO_NAME);
                if (!this.curtain.isShowing) {
                    yield this.curtain.show(true);
                }
                const durationMs = 500;
                // Reposition and fade in all the instruction elements
                const yPadding = this.game.height * 0.015;
                yield Promise.all([
                    PromiseUtil_24.PromiseUtil.tween(this.game, this.game.add.tween(this.headerBar)
                        .to({ top: 0 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_24.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialTopMessageBar)
                        .to({ bottom: this.instructionImage.top - yPadding }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_24.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialBottomMessageBar)
                        .to({ top: this.instructionImage.bottom + yPadding }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_24.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionImage)
                        .to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_24.PromiseUtil.tween(this.game, this.game.add.tween(this.readyButton)
                        .to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true)),
                ]);
                yield PromiseUtil_24.PromiseUtil.signal(this.readyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                yield Promise.all([
                    PromiseUtil_24.PromiseUtil.tween(this.game, this.game.add.tween(this.headerBar)
                        .to({ bottom: 0 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    // Move top message bar to top of screen for first turn
                    PromiseUtil_24.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialTopMessageBar)
                        .to({ top: this.game.height * 0.015 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_24.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialBottomMessageBar)
                        .to({ top: this.game.height }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_24.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionImage)
                        .to({ alpha: 0 }, durationMs * 0.5, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_24.PromiseUtil.tween(this.game, this.game.add.tween(this.readyButton)
                        .to({ alpha: 0 }, durationMs * 0.5, Phaser.Easing.Linear.None, true)),
                ]);
                this.curtain.hide();
            });
        }
        passToNextPlayer(hideCurtain = false) {
            return __awaiter(this, void 0, void 0, function* () {
                // Hide target, otherwise it shows over the curtain
                this.staticTarget.visible = false;
                // Set pass and play text
                const passAndPlayText = LocalizationManager_15.LocalizationManager.Instance.textWithReplacements('target_toss.instructions_pass', { player_name: ThrowState.players[this.playerIndex].name }).toUpperCase();
                this.passAndPlayAnimation.text = passAndPlayText;
                // Show curtains, animation, button
                yield this.curtain.show(true);
                yield this.passAndPlayAnimation.show();
                yield this.game.add.tween(this.readyButton).to({ alpha: 1 }, 200, Phaser.Easing.Sinusoidal.In, true);
                // Wait for button input
                yield PromiseUtil_24.PromiseUtil.signal(this.readyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                // Hide button, animation, curtains
                yield this.game.add.tween(this.readyButton).to({ alpha: 0 }, 200, Phaser.Easing.Sinusoidal.Out, true);
                yield this.passAndPlayAnimation.hide();
                if (hideCurtain) {
                    yield this.curtain.hide();
                }
            });
        }
        /**
         * This returns the left and right bounds of the given location
         */
        getLocationBounds(location) {
            const laneWidth = this.gameAreaWidth / location.getValidPositionsInLane(this.difficultyConfig.numberOfLanes);
            const leftX = (laneWidth * location.horizontalPos) + (this.game.width * 0.1);
            const rightX = leftX + laneWidth;
            const y = (this.laneHeight * location.lane) + this.horizonLineHeight;
            return { left: new Phaser.Point(leftX, y), right: new Phaser.Point(rightX, y) };
        }
        resetTargets() {
            if (this.staticTarget) {
                this.staticTarget.destroy();
                this.spinningTarget.destroy();
                this.targetShadow.destroy();
            }
            // Target visible on top of stack
            this.staticTarget = new Phaser.Sprite(this.game, 0, 0, "target");
            this.staticTarget.inputEnabled = true;
            this.staticTarget.visible = true;
            // Animated target that moves towards Bullseye
            this.spinningTarget = new Phaser.Sprite(this.game, 0, 0, "target-anim");
            this.spinningTarget.animations.add("target-spin", [0, 1, 2, 3, 4], 30);
            this.spinningTarget.visible = false;
            // Shadow of animated target
            this.targetShadow = new Phaser.Sprite(this.game, 0, 0, "target-shadow");
            this.targetShadow.visible = true;
            // Scale/place target sprites in the same way
            [this.staticTarget, this.spinningTarget, this.targetShadow].forEach((sprite) => {
                ScaleUtil_9.scaleToBounds(sprite, this.game.width * 0.397, this.game.width * 0.397);
                sprite.bottom = this.game.height * 0.8 + sprite.height * 0.8;
                sprite.centerX = this.game.width * 0.5;
            });
            this.add.existing(this.targetShadow);
            this.add.existing(this.spinningTarget);
            this.add.existing(this.staticTarget);
            this.staticTarget.events.onInputDown.addOnce((sprite, pointer) => {
                this.dragStartPoint = new Phaser.Point(pointer.x, pointer.y);
            }, this);
            // When input up triggers on target, initiate throw
            this.staticTarget.events.onInputUp.add((sprite, pointer) => {
                // if bullseye is moving to a new location, don't allow input to throw
                if (this.isBullseyeMoving) {
                    return;
                }
                this.dragEndPoint = new Phaser.Point(pointer.x, pointer.y);
                const dragDistance = Phaser.Point.distance(this.dragStartPoint, this.dragEndPoint);
                if (dragDistance > this.game.height * 0.05) {
                    this.staticTarget.inputEnabled = false;
                    this.updateThrowsRemaining();
                    this.throwTarget();
                }
                else {
                    this.resetTargets();
                }
            }, this);
        }
        throwTarget() {
            return __awaiter(this, void 0, void 0, function* () {
                const basketLocation = this.bullseye.getBasketWorldPosition();
                const bullseyeBounds = this.getLocationBounds(this.bullseye.location);
                const throwAngle = Phaser.Math.radToDeg(Phaser.Point.angle(this.dragStartPoint, this.dragEndPoint));
                const destAngleLeft = Phaser.Math.radToDeg(Phaser.Point.angle(this.dragStartPoint, bullseyeBounds.left));
                const destAngleRight = Phaser.Math.radToDeg(Phaser.Point.angle(this.dragStartPoint, bullseyeBounds.right));
                const throwSuccessful = (destAngleLeft <= throwAngle && throwAngle <= destAngleRight);
                let throwTween;
                let scale;
                let destination;
                if (throwSuccessful) {
                    // Tween into basket
                    const distance = Phaser.Point.distance(this.dragStartPoint, basketLocation);
                    scale = 0.8 - (distance / this.game.height);
                    destination = basketLocation;
                }
                else {
                    // Otherwise tween towards where they threw the target
                    const distantPoint = Phaser.Point.interpolate(this.dragStartPoint, this.dragEndPoint, 1.3);
                    // Ensure target doesn't go beyond the horizon
                    distantPoint.y = Math.max(distantPoint.y, this.horizonLineHeight);
                    const distance = Phaser.Point.distance(this.dragStartPoint, distantPoint);
                    scale = 0.8 - (distance / this.game.height);
                    destination = distantPoint;
                }
                const speedPxPerS = 1200;
                const distance = this.game.height - destination.y;
                const durationMs = (1000 / speedPxPerS) * distance;
                const halfDurationMs = durationMs * 0.5;
                const arcHeight = (this.game.height - destination.y);
                // Two tweens arc the target up and down on y axis
                const arcUpTween = this.game.add.tween(this.spinningTarget).to({
                    centerY: this.targetShadow.centerY - arcHeight
                }, halfDurationMs, Phaser.Easing.Cubic.Out, false);
                const arcDownTween = this.game.add.tween(this.spinningTarget).to({
                    centerY: destination.y
                }, halfDurationMs, Phaser.Easing.Sinusoidal.In, false);
                arcUpTween.chain(arcDownTween);
                arcUpTween.start();
                // throwTween moves target linearly on x axis, and scales
                throwTween = this.game.add.tween(this.spinningTarget).to({
                    centerX: destination.x,
                    width: this.spinningTarget.width * scale,
                    height: this.spinningTarget.height * scale
                }, durationMs, Phaser.Easing.Linear.None, false, 0);
                // Shadow moves and scales linearly
                this.game.add.tween(this.targetShadow).to({
                    width: this.spinningTarget.width * scale,
                    height: this.spinningTarget.height * scale,
                    centerX: destination.x,
                    centerY: throwSuccessful ? this.bullseye.bottom : destination.y
                }, durationMs, Phaser.Easing.Linear.None, true, 0);
                this.targetShadow.visible = true;
                this.spinningTarget.visible = true;
                this.staticTarget.visible = false;
                // If the throw was successful, and in the foreground lane, 
                // we don't want to show the shadow, because it looks wrong
                if (throwSuccessful && this.bullseye.location.lane === 2) {
                    this.targetShadow.visible = false;
                }
                // Play spinning plate anim, tween to destination
                this.game.sound.play(`woody-toss-target-${this.game.rnd.integerInRange(1, 3)}`);
                this.spinningTarget.animations.play('target-spin', 20, true);
                yield PromiseUtil_24.PromiseUtil.tween(this.game, throwTween.start());
                this.targetShadow.visible = false;
                this.spinningTarget.visible = false;
                if (throwSuccessful) {
                    this.updateTargetsCaught();
                    this.game.sound.play('woody-target-basket');
                    const basketPos = this.bullseye.getBasketWorldPosition();
                    this.snatchFx.show(basketPos.x, basketPos.y);
                }
                else {
                    yield this.animateFailedThrow(destination, scale);
                }
                if (this.throwsRemaining === 0) {
                    // move bullseye offscreen once the player's turn is over
                    yield this.bullseye.moveToPoint(new Phaser.Point(this.game.width + this.bullseye.width, this.bullseye.y));
                    // Pass to next player, resets target too, moves bullseye
                    yield this.startNewRound(false);
                }
                else {
                    // Moves Bullseye only if it's a successful throw and not the start of a round
                    // (in which case it will got re-centered)
                    if (throwSuccessful) {
                        yield this.moveBullseye();
                    }
                    yield this.resetTargets();
                }
            });
        }
        // Show broken target falling to the floor
        // Target changes image, rotates, falls and fades out
        animateFailedThrow(point, scale) {
            return __awaiter(this, void 0, void 0, function* () {
                this.staticTarget.visible = false;
                const animationDurationMs = 550;
                this.targetShard1.position.set(point.x, point.y);
                this.targetShard2.position.set(point.x, point.y);
                this.targetShard1.scale.set(this.targetShardOrigScale * scale);
                this.targetShard2.scale.set(this.targetShardOrigScale * scale);
                this.targetShard1.alpha = 1;
                this.targetShard2.alpha = 1;
                // Alpha tweens to fade both shards out
                const alphaTween1 = this.game.add.tween(this.targetShard1)
                    .to({ alpha: 0 }, animationDurationMs * 0.5, Phaser.Easing.Linear.None, true, animationDurationMs * 0.5);
                this.game.add.tween(this.targetShard2)
                    .to({ alpha: 0 }, animationDurationMs * 0.5, Phaser.Easing.Linear.None, true, animationDurationMs * 0.5);
                // Move both shards along to the right
                this.game.add.tween(this.targetShard1)
                    .to({ x: this.targetShard1.x + this.game.width * scale }, animationDurationMs, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.targetShard2)
                    .to({ x: this.targetShard2.x + this.game.width * scale }, animationDurationMs, Phaser.Easing.Linear.None, true);
                // Move the first shard up, then down, following an arc (chained together)
                const movementYTweenUp1 = this.game.add.tween(this.targetShard1)
                    .to({ y: this.targetShard1.y - this.game.height * 0.35 * scale }, animationDurationMs * 0.5, Phaser.Easing.Sinusoidal.Out, true);
                const movementYTweenDown1 = this.game.add.tween(this.targetShard1)
                    .to({ y: this.targetShard1.y - this.game.height * 0.25 * scale }, animationDurationMs * 0.5, Phaser.Easing.Sinusoidal.In, false);
                movementYTweenUp1.chain(movementYTweenDown1);
                // Move the second shard up, then down, following a slightly smoother arc than the first (chained together)
                const movementYTweenUp2 = this.game.add.tween(this.targetShard2)
                    .to({ y: this.targetShard2.y - this.game.height * 0.15 * scale }, animationDurationMs * 0.5, Phaser.Easing.Sinusoidal.Out, true);
                const movementYTweenDown2 = this.game.add.tween(this.targetShard2)
                    .to({ y: this.targetShard2.y - this.game.height * 0.1 * scale }, animationDurationMs * 0.5, Phaser.Easing.Sinusoidal.In, false);
                movementYTweenUp2.chain(movementYTweenDown2);
                this.game.sound.play(`woody-target-break-${this.game.rnd.integerInRange(1, 3)}`);
                yield PromiseUtil_24.PromiseUtil.tween(this.game, alphaTween1);
                this.staticTarget.visible = true;
            });
        }
        updateThrowsRemaining(value) {
            if (value) {
                this.throwsRemaining = value;
            }
            else {
                this.throwsRemaining--;
            }
            this.throwsRemainingText.text = this.throwsRemaining.toString();
            this.game.add.tween(this.throwsRemainingIcon).to({
                width: this.throwsRemainingIcon.width * 0.6,
                height: this.throwsRemainingIcon.height * 0.6,
                centerX: this.throwsRemainingIcon.centerX,
                centerY: this.throwsRemainingIcon.centerY
            }, 100, Phaser.Easing.Sinusoidal.In, true, 0, 0, true);
        }
        updateTargetsCaught(value) {
            if (value !== undefined) {
                this.targetsCaught = value;
            }
            else {
                this.targetsCaught++;
            }
            this.targetsCaughtText.text = this.targetsCaught.toString();
            this.game.add.tween(this.targetsCaughtIcon).to({
                width: this.targetsCaughtIcon.width * 0.6,
                height: this.targetsCaughtIcon.height * 0.6,
                centerX: this.targetsCaughtIcon.centerX,
                centerY: this.targetsCaughtIcon.centerY
            }, 100, Phaser.Easing.Sinusoidal.In, true, 0, 0, true);
        }
        generateBullseyePos() {
            let location = new Location_2.Location();
            location.lane = Math.floor(Math.random() * this.difficultyConfig.numberOfLanes);
            location.horizontalPos = Math.floor(Math.random() * location.getValidPositionsInLane(this.difficultyConfig.numberOfLanes));
            return location;
        }
        moveBullseye(loc) {
            return __awaiter(this, void 0, void 0, function* () {
                this.isBullseyeMoving = true; // set bullseye moving to true
                // Randomly generate a position for Bullseye to go
                if (!loc) {
                    let location = this.generateBullseyePos();
                    while (this.bullseye.location
                        && location.lane === this.bullseye.location.lane
                        && location.horizontalPos === this.bullseye.location.horizontalPos) {
                        location = this.generateBullseyePos();
                    }
                    this.bullseye.location = location;
                }
                else {
                    this.bullseye.location = loc;
                }
                // Calculate position on screen
                // Get bounds of location (left and rightmost points of the location block)
                const bounds = this.getLocationBounds(this.bullseye.location);
                // Move Bullseye to the new location
                yield this.bullseye.moveToPoint(new Phaser.Point(Phaser.Point.centroid([bounds.left, bounds.right]).x, bounds.left.y));
                this.isBullseyeMoving = false; // set bullseye moving to false
            });
        }
        update() {
            super.update();
            if (this.isPlaying) {
                this.roundTimeMs += this.game.time.elapsedMS;
                if (this.roundTimeMs > this.MAX_TIMEOUT_STUCK_MS) {
                    this.onGameTimeout();
                }
            }
        }
        onGameTimeout() {
            return __awaiter(this, void 0, void 0, function* () {
                this.arrow.alpha = 0;
                this.isPlaying = false;
                this.staticTarget.inputEnabled = false;
                this.game.add.tween(this.tutorialTopMessageBar).to({ bottom: 0 }, 200, Phaser.Easing.Sinusoidal.Out, true);
                this.game.add.tween(this.tutorialBottomMessageBar).to({ top: this.game.height }, 200, Phaser.Easing.Sinusoidal.Out, true);
                const timeoutDialog = new BasicDialog_1.BasicDialog(this.game, Character_2.Character.WOODY, true);
                timeoutDialog.centerY = this.game.height * 0.5;
                this.add.existing(timeoutDialog);
                yield timeoutDialog.show();
                yield timeoutDialog.writeMessage(LocalizationManager_15.LocalizationManager.Instance.text("time_up.main"));
                yield Promise.race([
                    this.dialog.waitForTap(),
                    PromiseUtil_24.PromiseUtil.signal(this.game.input.onTap)
                ]);
                yield timeoutDialog.hide();
                timeoutDialog.destroy();
                this.onGameOver(this.targetsCaught);
            });
        }
        onGameOver(targetsCaught) {
            return __awaiter(this, void 0, void 0, function* () {
                this.isPlaying = false;
                // Disable target
                if (this.staticTarget) {
                    this.staticTarget.inputEnabled = false;
                    this.staticTarget.visible = false;
                }
                // Display results dialog
                const message = LocalizationManager_15.LocalizationManager.Instance.textWithReplacements("target_toss.results_message", { items_caught: targetsCaught });
                // Update total score
                // TODO this is a little bit unreliable, it might be better to just store the value in this.totalScore on init like BalanceState
                const totalScore = ThrowState.score + targetsCaught;
                this.trackMicroEnd(targetsCaught);
                GameController_16.GameController.Instance.postScore(StateNames_13.StateNames.GAME_WOODY_LASSO, totalScore);
                ThrowState.score = 0;
                yield this.dialog.show();
                yield this.dialog.writeMessage(message);
                // allows for tapping the whole screen
                yield Promise.race([
                    this.dialog.waitForTap(),
                    PromiseUtil_24.PromiseUtil.signal(this.game.input.onTap)
                ]);
                yield this.dialog.hide();
                this.transitionTo();
            });
        }
        transitionTo() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this.trainLoop).to({ volume: 0 }, 500, Phaser.Easing.Linear.None, true);
                yield this.showOverlay();
                this.game.state.start(StateNames_13.StateNames.RESULTS);
            });
        }
        static startWoodyThrow(game, players, score) {
            ThrowState.players = players;
            ThrowState.score = score;
            CutSceneState_3.CutSceneState.showCutScene(game, "game_woody_toss", StateNames_13.StateNames.GAME_WOODY_THROW);
        }
        shutdown() {
            this.dialog = null;
            this.bullseye = null;
            this.snatchFx = null;
            this.staticTarget = null;
            this.spinningTarget = null;
            this.targetShadow = null;
            this.background = null;
            this.arrow = null;
            this.targetShard1 = null;
            this.targetShard2 = null;
            this.targetsCaughtIcon = null;
            this.targetsCaughtText = null;
            this.throwsRemainingText = null;
            this.throwsRemainingIcon = null;
            this.curtain = null;
            this.passAndPlayAnimation = null;
            this.readyButton = null;
            this.tutorialTopMessageBar = null;
            this.tutorialBottomMessageBar = null;
            this.instructionImage = null;
            this.headerBar = null;
            this.trainLoop.destroy();
            this.trainLoop = null;
            this.game.cache.removeImage('arrow');
            this.game.cache.removeImage('basket-symbol');
            this.game.cache.removeImage('cactus');
            this.game.cache.removeImage('target');
            this.game.cache.removeImage('throw-target-shard');
            this.game.cache.removeImage('bullseye');
            this.game.cache.removeImage('target-anim');
            this.game.cache.removeImage('target-symbol');
            this.game.cache.removeImage('target-shadow');
            this.game.cache.removeImage('carriage-roof');
            this.game.cache.removeImage('clouds');
            this.game.cache.removeImage('bg-detail');
            this.game.cache.removeImage('throw-tiling-bg');
            this.game.cache.removeImage('lasso-floor-tiling');
            this.game.cache.removeImage('throw-success-ray');
            this.game.cache.removeImage('snatch-radialfx');
            this.game.cache.removeImage('snatch-star-green');
            this.game.cache.removeImage('snatch-star-pink');
            this.game.cache.removeImage('snatch-star-blue');
            // Instructions
            this.game.cache.removeImage('instruction-graphic-woody-throw');
            this.game.cache.removeImage('character-dialog-woody');
            this.game.cache.removeSound('woody-target-basket');
            this.game.cache.removeSound('woody-toss-target-1');
            this.game.cache.removeSound('woody-toss-target-2');
            this.game.cache.removeSound('woody-toss-target-3');
            this.game.cache.removeSound('woody-target-break-1');
            this.game.cache.removeSound('woody-target-break-2');
            this.game.cache.removeSound('woody-target-break-3');
            this.game.cache.removeSound('woody-train-loop');
            this.game.cache.removeSound('woody-bullseye-run');
            super.shutdown();
        }
    }
    ThrowState.TRACKING_MICRO_NAME = "Toss";
    exports.ThrowState = ThrowState;
});
define("minigame/woody/balance/BalanceConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class BalanceConfig {
        parse(data) {
            this.minItemsBetweenCrates = data.minItemsBetweenCrates || 3;
            this.baseAngleModifier = data.baseAngleModifier || 0.2;
            this.maxAngle = data.maxAngle || 25;
            this.timePerPlayerMs = data.timePerPlayerMs || 7500;
            this.passDurationMs = data.passDurationMs || 5000;
            this.gameplayHintBufferTime = data.gameplayHintBufferTime || 1800;
            this.passingHintBufferTime = data.passingHintBufferTime || 900;
            this.minHintDuration = data.minHintDuration || 1500;
        }
    }
    exports.BalanceConfig = BalanceConfig;
});
define("minigame/woody/balance/BalanceDifficultyConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class BalanceDifficultyConfig {
        parse(data) {
            this.minTimeBetweenItemsDroppingMs = data.minTimeBetweenItemsDroppingMs || 250;
            this.dropItemAngleThreshold = data.dropItemAngleThreshold || 20;
            this.naturalWobbleMax = data.naturalWobbleMax || 20;
        }
    }
    exports.BalanceDifficultyConfig = BalanceDifficultyConfig;
});
define("minigame/woody/balance/BalanceState", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "model/ToyStoryConfig", "Game", "StateNames", "minigame/woody/balance/Train", "component/dialog/CharacterDialog", "logic/GameController", "minigame/woody/throw/ThrowState", "state/CutSceneState", "Character", "component/Curtain", "component/PassAndPlayShowable", "component/button/TextArrowButton", "component/MessageBar", "component/HeaderBar", "minigame/woody/balance/BalanceConfig", "state/GameBaseState", "minigame/woody/balance/BalanceDifficultyConfig", "switchback-common/util/MotionUtil"], function (require, exports, PromiseUtil_25, ScaleUtil_10, LocalizationManager_16, ToyStoryConfig_11, Game_21, StateNames_14, Train_1, CharacterDialog_2, GameController_17, ThrowState_1, CutSceneState_4, Character_3, Curtain_3, PassAndPlayShowable_2, TextArrowButton_4, MessageBar_6, HeaderBar_2, BalanceConfig_1, GameBaseState_4, BalanceDifficultyConfig_1, MotionUtil_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    /**
     * The item class exists to hold on to the xOffset of itself.
     * It also does a bit of work with that value to figure out its true
     * pivot point so that it looks correct when the stack wobbles.
     */
    class Item extends Phaser.Image {
        constructor(game, stackOffsetX, type) {
            super(game, 0, 0, `target-${type}`);
            this.stackOffsetX = 0;
            this.stackOffsetX = stackOffsetX;
            this.anchor.set(0.5, 1);
            const xPivot = (this.width * 0.5 - stackOffsetX) / this.width;
            this.pivot.set(xPivot, 1);
        }
    }
    /**
     * Custom yellow and black warning bar
     * Used for pass and play instructions
     * Appears above pass animation
     */
    class WarningBar extends MessageBar_6.MessageBar {
        constructor(game) {
            super(game, LocalizationManager_16.LocalizationManager.Instance.text("stacker.careful"));
            this.tilingBackground.loadTexture('warning-tile');
            this.instructionText.fill = Game_21.Game.TEXT_COLOR_RED;
            this.instructionText.strokeThickness = Game_21.Game.TEXT_STROKE_THICKNESS_HUMONGOUS;
            this.instructionText.stroke = 'black';
        }
    }
    exports.WarningBar = WarningBar;
    class BalanceState extends GameBaseState_4.GameBaseState {
        constructor() {
            super(...arguments);
            // Current angle of the phone TODO: Remove (?)
            this.angle = 0;
            this.naturalWobble = 0;
        }
        init(...params) {
            this.stateName = StateNames_14.StateNames.GAME_WOODY_BALANCE;
            this._trackMicroName = "Balance";
            super.init(params);
            this.config = this.loadConfig(BalanceConfig_1.BalanceConfig);
            this.difficultyConfig = this.loadDifficultyConfig(BalanceDifficultyConfig_1.BalanceDifficultyConfig);
            this.isGameRunning = false;
            this.isTimerRunning = false;
            this.canDropItem = true;
            this.inPassAndPlay = false;
            this.timeRemaining = 0;
            this.itemsDropped = 0;
            this.currentPlayerIndex = 0;
            this.angle = 0;
            this.naturalWobble = 0;
            this.totalScore = BalanceState.score;
            this.angleInDegrees = 0;
        }
        preload() {
            super.preload();
            this.game.load.image('target-1', 'assets/minigame/woody/target_1.png');
            this.game.load.image('target-2', 'assets/minigame/woody/target_2.png');
            this.game.load.image('target-10', 'assets/minigame/woody/target_10.png');
            this.game.load.image('clouds', 'assets/minigame/woody/clouds.png');
            this.game.load.image('bg-detail', 'assets/minigame/woody/bg_detail.png');
            this.game.load.image('tiling-bg', 'assets/minigame/woody/tiling-bg.jpg');
            this.game.load.image('target-dropping-1', 'assets/minigame/woody/target_dropping_1.png');
            this.game.load.image('carriage-0', 'assets/minigame/woody/carriage_blue.png');
            this.game.load.image('carriage-1', 'assets/minigame/woody/carriage_green.png');
            this.game.load.image('carriage-2', 'assets/minigame/woody/carriage_yellow.png');
            this.game.load.image('hands-base', 'assets/minigame/woody/hands_base.png');
            this.game.load.atlas('track-sheet', 'assets/minigame/woody/track_sheet.png', 'assets/minigame/woody/track_sheet.json');
            this.game.load.image('character-dialog-woody', 'assets/dialog/character_dialog_woody.png');
            this.game.load.image('character-dialog-backplate', 'assets/dialog/character_dialog_backplate.png');
            this.game.load.image('instruction-dialog-backplate', 'assets/dialog/instruction_dialog_backplate.png');
            // Instructions
            this.game.load.image('instruction-graphic-woody-balance', 'assets/dialog/instruction_graphic_woody2.png');
            this.game.load.image('warning-tile', 'assets/minigame/woody/warningstrip_tile.png');
            this.game.load.image('warning-sign', 'assets/minigame/woody/warningstrip_tri.png');
            this.game.load.spritesheet('balance-prompt', 'assets/minigame/woody/balance_prompt.png', 375, 371);
            this.game.load.image('hand', 'assets/minigame/woody/hand.png');
            this.game.load.audio('woody-target-fall-1', ['assets/sound/woody/target_fall_1.m4a', 'assets/sound/woody/target_fall_1.ogg']);
            this.game.load.audio('woody-target-fall-2', ['assets/sound/woody/target_fall_2.m4a', 'assets/sound/woody/target_fall_2.ogg']);
            this.game.load.audio('woody-target-fall-3', ['assets/sound/woody/target_fall_3.m4a', 'assets/sound/woody/target_fall_3.ogg']);
            this.game.load.audio('woody-target-break-1', ['assets/sound/woody/target_break_1.m4a', 'assets/sound/woody/target_break_1.ogg']);
            this.game.load.audio('woody-target-break-2', ['assets/sound/woody/target_break_2.m4a', 'assets/sound/woody/target_break_2.ogg']);
            this.game.load.audio('woody-target-break-3', ['assets/sound/woody/target_break_3.m4a', 'assets/sound/woody/target_break_3.ogg']);
            this.game.load.audio('woody-train-loop', ['assets/sound/woody/train_loop.m4a', 'assets/sound/woody/train_loop.ogg']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.trackGameState(null, BalanceState.TRACKING_MICRO_NAME);
                this.game.stage.backgroundColor = 0x5F7CA6;
                const bgFloor = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height, "tiling-bg");
                const bgTextureScale = bgFloor.height / bgFloor.texture.height;
                bgFloor.tileScale.y = bgTextureScale;
                this.add.existing(bgFloor);
                const scrollSpeed = -this.game.width * 0.05;
                const tilingClouds = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height * 0.5, "clouds");
                const cloudTextureScale = tilingClouds.height / tilingClouds.texture.height;
                tilingClouds.tileScale.set(cloudTextureScale, cloudTextureScale);
                tilingClouds.autoScroll(scrollSpeed * 0.5, 0);
                this.add.existing(tilingClouds);
                const bgDetail = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height * 0.075, "bg-detail");
                const bgDetailScale = bgDetail.height / bgDetail.texture.height;
                bgDetail.tileScale.set(bgDetailScale, bgDetailScale);
                bgDetail.bottom = this.game.height * 0.715;
                bgDetail.autoScroll(scrollSpeed, 0);
                this.add.existing(bgDetail);
                const tracks = new Phaser.Sprite(this.game, this.game.width, this.game.height, "track-sheet");
                tracks.anchor.set(1, 0.95);
                tracks.height = this.game.height * 0.34;
                tracks.scale.x = tracks.scale.y;
                tracks.animations.add("loop", [0, 1, 2, 3, 4]);
                tracks.animations.play("loop", 30, true);
                this.add.existing(tracks);
                this.train = new Train_1.Train(this.game, BalanceState.players.length);
                this.add.existing(this.train);
                this.stackBase = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height, 'hands-base');
                this.stackBase.anchor.set(0.5, 0.8);
                this.stackBase.width = this.game.width;
                this.stackBase.scale.y = this.stackBase.scale.x;
                this.add.existing(this.stackBase);
                this.stack = new Phaser.Group(this.game);
                this.stack.x = this.game.width * 0.5;
                this.stack.y = this.stackBase.top + (this.stackBase.height * 0.5);
                this.add.existing(this.stack);
                this.addItems(BalanceState.STARTING_ITEM_NUM);
                this.dialog = new CharacterDialog_2.CharacterDialog(this.game, Character_3.Character.WOODY);
                this.dialog.centerY = this.game.height * 0.15;
                this.add.existing(this.dialog);
                this.trainLoop = this.game.add.sound('woody-train-loop', 1.0, true);
                this.trainLoop.play();
                this.game.add.tween(this.trainLoop).from({ volume: 0 }, 500, Phaser.Easing.Linear.None, true);
                this.hand = new Phaser.Image(this.game, 0, this.game.height * 0.8, 'hand');
                this.hand.alpha = 0;
                this.hand.anchor.set(0.5, 0.2);
                this.add.existing(this.hand);
                this.curtain = new Curtain_3.Curtain(this.game, true);
                this.add.existing(this.curtain);
                // Pass and play
                this.passAndPlayAnimation = new PassAndPlayShowable_2.PassAndPlayShowable(this.game, '', 35);
                this.passAndPlayAnimation.x = this.game.width * 0.5;
                this.passAndPlayAnimation.y = this.game.height * 0.5;
                this.add.existing(this.passAndPlayAnimation);
                this.warningBar = new WarningBar(this.game);
                this.warningBar.centerX = this.game.width * 0.5;
                this.warningBar.bottom = 0;
                this.add.existing(this.warningBar);
                // Instructions
                this.readyButton = new TextArrowButton_4.TextArrowButton(this.game, LocalizationManager_16.LocalizationManager.Instance.text('common.ready').toUpperCase());
                this.readyButton.centerX = this.game.width * 0.5;
                this.readyButton.centerY = this.game.height * 0.785;
                this.readyButton.alpha = 0;
                this.readyButton.startPulse();
                this.add.existing(this.readyButton);
                const instructionTag = (MotionUtil_4.MotionUtil.isGyroAvailable ? "stacker.instructions_tilt" : "stacker.instructions_tap");
                this.tutorialTopMessageBar = new MessageBar_6.MessageBar(this.game, LocalizationManager_16.LocalizationManager.Instance.text(instructionTag));
                this.tutorialTopMessageBar.centerX = this.game.width * 0.5;
                this.tutorialTopMessageBar.bottom = 0;
                this.add.existing(this.tutorialTopMessageBar);
                this.tutorialBottomMessageBar = new MessageBar_6.MessageBar(this.game, LocalizationManager_16.LocalizationManager.Instance.text("stacker.instructions_careful"));
                this.tutorialBottomMessageBar.centerX = this.game.width * 0.5;
                this.tutorialBottomMessageBar.top = this.game.height;
                this.add.existing(this.tutorialBottomMessageBar);
                this.instructionImage = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.425, 'instruction-graphic-woody-balance');
                this.instructionImage.alpha = 0;
                this.instructionImage.anchor.set(0.5);
                this.add.existing(this.instructionImage);
                if (!MotionUtil_4.MotionUtil.isGyroAvailable) {
                    this.instructionImage.visible = false;
                }
                this.headerBar = new HeaderBar_2.HeaderBar(this.game, LocalizationManager_16.LocalizationManager.Instance.text("common.how_to_play"));
                this.headerBar.centerX = this.game.width * 0.5;
                this.headerBar.bottom = 0;
                this.add.existing(this.headerBar);
                // Hints
                this.timeElapsedOverThreshold = 0;
                this.timeElapsedShowingHint = 0;
                this.hintAnimation = new Phaser.Sprite(this.game, 0, 0, 'balance-prompt', 0);
                this.hintAnimation.width = this.game.width * 0.8;
                this.hintAnimation.scale.y = this.hintAnimation.scale.x;
                this.hintAnimation.centerX = this.game.width * 0.5;
                this.hintAnimation.centerY = this.game.height * 0.5;
                this.hintAnimation.alpha = 0;
                this.hintAnimation.animations.add('tilt', null, 10, true);
                this.add.existing(this.hintAnimation);
                yield this.dismissOverlay();
                if (ToyStoryConfig_11.ToyStoryConfig.Instance.skipGames) {
                    this.itemsDropped = 5;
                    this.transitionToNextGame(50);
                }
                else {
                    this.startGame();
                }
            });
        }
        startGame() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.showThenHideInstructions();
                // Change the message bar text so we can reuse it for hints later
                this.tutorialTopMessageBar.text = LocalizationManager_16.LocalizationManager.Instance.text("stacker.balance_the_stack");
                this.train.moveCarsForward(this.config.timePerPlayerMs);
                this.isGameRunning = true;
                // Start the game timer
                this.isTimerRunning = true;
                this.timeRemaining = ToyStoryConfig_11.ToyStoryConfig.Instance.quickTest ? 1000 : this.config.timePerPlayerMs;
                // Listen for gyro events. We save off this callback to ensure we have a reference to unsubscribe from it later
                // and we have to wrap this.onAttitudeChanged in a lambda to bind the context correctly.
                if (MotionUtil_4.MotionUtil.isGyroAvailable) {
                    this.gyroCallback = (result) => {
                        this.onMotionChanged(result);
                    };
                    PlayAPI.subscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.gyroCallback);
                }
                else {
                    this.game.input.addMoveCallback(this.onBShowTapMove, this);
                    this.game.input.onUp.add(this.onBShowTapRelease, this);
                }
                // This part kicks of a natural oscillation of the stack. It can be improved later, 
                // but for now it serves the purpose of making players adjust the device attitude rather than just holding it dead still
                const wobbleRightTween = this.game.add.tween(this).to({ naturalWobble: this.difficultyConfig.naturalWobbleMax }, 1500, Phaser.Easing.Quadratic.InOut, true, 0);
                yield PromiseUtil_25.PromiseUtil.tween(this.game, wobbleRightTween);
                this.game.add.tween(this).to({ naturalWobble: -this.difficultyConfig.naturalWobbleMax }, 3000, Phaser.Easing.Quadratic.InOut, true, 0, Infinity, true);
                yield PromiseUtil_25.PromiseUtil.time(this.game, 2000);
                // Hide woody's dialog
                this.dialog.hide();
                this.trackMicroStart();
            });
        }
        showThenHideInstructions() {
            return __awaiter(this, void 0, void 0, function* () {
                this.trackGameInstructions(BalanceState.TRACKING_MICRO_NAME);
                if (!this.curtain.isShowing) {
                    yield this.curtain.show(true);
                }
                const durationMs = 500;
                // Reposition and fade in all the instruction elements
                const yPadding = this.game.height * 0.015;
                yield Promise.all([
                    PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.headerBar)
                        .to({ top: 0 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialTopMessageBar)
                        .to({ bottom: this.instructionImage.top - yPadding }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialBottomMessageBar)
                        .to({ top: this.instructionImage.bottom + yPadding }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionImage)
                        .to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.readyButton)
                        .to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true)),
                ]);
                yield PromiseUtil_25.PromiseUtil.signal(this.readyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                yield Promise.all([
                    PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.headerBar)
                        .to({ bottom: 0 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialTopMessageBar)
                        .to({ bottom: 0 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialBottomMessageBar)
                        .to({ top: this.game.height }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionImage)
                        .to({ alpha: 0 }, durationMs * 0.5, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_25.PromiseUtil.tween(this.game, this.game.add.tween(this.readyButton)
                        .to({ alpha: 0 }, durationMs * 0.5, Phaser.Easing.Linear.None, true)),
                ]);
                this.curtain.hide();
            });
        }
        onMotionChanged(result) {
            this.angleInDegrees = result.attitude.roll * (180 / Math.PI);
        }
        onBShowTapMove(pointer, x, y) {
            if (this.hand.alpha <= 0) {
                this.hand.x = x;
                // show the hand move from the players finger (unless it is too low on the screen)
                this.hand.y = Math.min(y, this.game.height * 0.7);
            }
            this.angleInDegrees = this.config.maxAngle * 0.8 * ((x - this.game.width * 0.5) / (this.game.width * 0.5));
        }
        onBShowTapRelease() {
            if (this.hand.alpha <= 0) {
                this.hand.y = this.game.height * 0.7;
            }
        }
        transitionToNextPlayer() {
            return __awaiter(this, void 0, void 0, function* () {
                this.currentPlayerIndex++;
                this.inPassAndPlay = true;
                // Communicate the device handover
                // Set text on pass and play animation
                const passAndPlayText = LocalizationManager_16.LocalizationManager.Instance.textWithReplacements('stacker.instructions_pass', {
                    player_name: BalanceState.players[this.currentPlayerIndex].name
                }).toUpperCase();
                this.passAndPlayAnimation.text = passAndPlayText;
                // Show pass and play animation
                // Note, the ready button doesn't show on this P&P
                yield this.passAndPlayAnimation.show();
                // Wait for 5 seconds/passDurationMs
                yield PromiseUtil_25.PromiseUtil.time(this.game, this.config.passDurationMs);
                // Hide pass and play, warning bar (if visible)
                this.game.add.tween(this.warningBar).to({ bottom: 0 }, 500, Phaser.Easing.Sinusoidal.Out, true);
                yield this.passAndPlayAnimation.hide();
                // Kick off the timer to count down to the next changeover
                this.isTimerRunning = true;
                this.timeRemaining = this.config.timePerPlayerMs;
                this.train.moveCarsForward(this.timeRemaining);
                this.inPassAndPlay = false;
            });
        }
        addItems(numItems) {
            return __awaiter(this, void 0, void 0, function* () {
                // Items to distribute randomly in the stack
                const items = [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 10, 10];
                Phaser.ArrayUtils.shuffle(items);
                for (let i = 0; i < items.length; i++) {
                    // The first item is never offset on X
                    const stackOffsetX = i === 0 ? 0 : this.game.rnd.integerInRange(-this.game.width * 0.03, this.game.width * 0.03);
                    const item = new Item(this.game, stackOffsetX, items[i]);
                    item.width = this.game.width * 0.5;
                    item.scale.y = item.scale.x;
                    let bottom = 0;
                    if (i !== 0) {
                        bottom = this.stack.getAt(0).top;
                    }
                    const itemOverlap = this.game.height * 0.005;
                    item.position.set(item.stackOffsetX, bottom + itemOverlap);
                    this.stack.addAt(item, 0);
                }
            });
        }
        update() {
            super.update();
            if (!this.isGameRunning) {
                return;
            }
            if (this.isTimerRunning) {
                this.timeRemaining -= this.game.time.elapsedMS;
                if (this.timeRemaining <= 0) {
                    this.isTimerRunning = false;
                    // If there are more players, pass it to them. Else, game over.
                    if (this.currentPlayerIndex < BalanceState.players.length - 1) {
                        this.transitionToNextPlayer();
                    }
                    else {
                        this.onGameOver(this.totalScore);
                    }
                    return;
                }
            }
            // adjust the stack angle
            const adjustedAngle = this.angleInDegrees - this.naturalWobble;
            this.angle = Phaser.Math.clamp(adjustedAngle, -this.config.maxAngle, this.config.maxAngle);
            // TODO: Cap the angle?
            this.updateStack(this.angle);
            // If enough time has passed since we lost our last plate && the stack is over the angle threshold
            if (Math.abs(this.angle) > this.difficultyConfig.dropItemAngleThreshold) {
                // countdown timer for showing hint animation
                this.timeElapsedOverThreshold += this.game.time.elapsedMS;
                if (this.timeElapsedShowingHint === 0) {
                    if (this.inPassAndPlay && this.timeElapsedOverThreshold >= this.config.passingHintBufferTime) {
                        // Show caution message bar
                        this.showHint();
                    }
                    if (!this.inPassAndPlay && this.timeElapsedOverThreshold >= this.config.gameplayHintBufferTime) {
                        // Show gameplay hint
                        this.showHint();
                    }
                }
                else if (this.timeElapsedShowingHint > 0) {
                    // We are currently showing hint - increase time elapsed
                    this.timeElapsedShowingHint += this.game.time.elapsedMS;
                }
                if (this.canDropItem) {
                    // Lock off plate dropping
                    this.canDropItem = false;
                    PromiseUtil_25.PromiseUtil.time(this.game, this.difficultyConfig.minTimeBetweenItemsDroppingMs).then(() => {
                        this.canDropItem = true;
                    });
                    this.itemsDropped++;
                    this.totalScore = Math.max(0, this.totalScore - 1);
                    // Drop a plate
                    this.dropItem();
                }
            }
            else if (this.timeElapsedShowingHint > 0) {
                this.hideHint();
            }
        }
        dropItem() {
            return __awaiter(this, void 0, void 0, function* () {
                const xPos = (this.game.width * 0.5) + (Math.sign(this.angle) * this.game.width * 0.25);
                const itemToDrop = new Phaser.Image(this.game, xPos, 0, "target-dropping-1");
                itemToDrop.anchor.set(0.5, 1);
                ScaleUtil_10.scaleToBounds(itemToDrop, this.game.width * 0.5, this.game.height * 0.2);
                this.game.add.existing(itemToDrop);
                // Spin around
                this.game.add.tween(itemToDrop).to({
                    angle: Math.sign(this.angle) * 360
                }, 2000, Phaser.Easing.Linear.None, true, 0, Infinity);
                // Vertical drop, with destroy onComplete
                this.game.add.tween(itemToDrop).to({
                    y: this.game.height * 0.9
                }, 1000, Phaser.Easing.Sinusoidal.In, true, 0).onComplete.addOnce(() => {
                    itemToDrop.destroy();
                    this.game.sound.play(`woody-target-break-${this.game.rnd.integerInRange(1, 3)}`);
                }, this);
                this.game.sound.play(`woody-target-fall-${this.game.rnd.integerInRange(1, 3)}`);
            });
        }
        showHint() {
            if (this.inPassAndPlay) {
                // Show warning bar
                this.game.add.tween(this.warningBar).to({ top: this.game.height * 0.07 }, 200, Phaser.Easing.Sinusoidal.In, true);
            }
            else {
                if (MotionUtil_4.MotionUtil.isGyroAvailable) {
                    // Flip hint animation based on angle the stack is leaning
                    let scale = (this.angle < 0) ? Math.abs(this.hintAnimation.scale.x) * -1 : Math.abs(this.hintAnimation.scale.x);
                    this.hintAnimation.scale.x = scale;
                    // Recenter after scaling
                    this.hintAnimation.centerX = this.game.width * 0.5;
                    // Show hint animation
                    this.game.add.tween(this.tutorialTopMessageBar).to({ top: this.game.height * 0.07 }, 200, Phaser.Easing.Sinusoidal.In, true);
                    this.game.add.tween(this.hintAnimation).to({ alpha: 1 }, 200, Phaser.Easing.Sinusoidal.In, true)
                        .onComplete.add(() => { this.hintAnimation.play('tilt'); });
                }
                else {
                    this.hand.alpha = 1;
                    this.game.add.tween(this.hand).to({
                        x: this.game.width * 0.5 + (this.game.width * 0.4 * (this.angle < 0 ? 1 : -1))
                    }, 300, Phaser.Easing.Sinusoidal.In, true);
                }
            }
            this.timeElapsedShowingHint += this.game.time.elapsedMS;
        }
        hideHint() {
            // Calculate time to wait before hiding hints
            const timeRemaining = Math.max(this.config.minHintDuration - this.timeElapsedShowingHint, 0);
            // Reset timer
            // Doing this first prevents show/hide tweens from firing while we're waiting
            this.timeElapsedShowingHint = -1;
            // Wait until the hint has shown for the min. duration
            PromiseUtil_25.PromiseUtil.time(this.game, timeRemaining).then(() => {
                // Reset timer so that we're ready to show hints again, if necessary
                this.timeElapsedShowingHint = 0;
                // Hide warning bar, if visible
                this.game.add.tween(this.warningBar).to({ bottom: 0 }, 200, Phaser.Easing.Sinusoidal.Out, true);
                if (MotionUtil_4.MotionUtil.isGyroAvailable) {
                    // Hide top message bar, if visible
                    this.game.add.tween(this.tutorialTopMessageBar).to({ bottom: 0 }, 200, Phaser.Easing.Sinusoidal.Out, true);
                    // Fade out hint animation, if visible
                    this.game.add.tween(this.hintAnimation).to({ alpha: 0 }, 200, Phaser.Easing.Sinusoidal.In, true);
                }
                else {
                    this.hand.alpha = 0;
                }
            });
        }
        updateStack(angle) {
            for (let i = 0; i < this.stack.length; i++) {
                const item = this.stack.children[i];
                // Children of the stack group are ordered in reverse (for z-layering purposes) so flip the index when positioning
                const index = this.stack.length - 1 - i;
                // Rotates the plates. Plates higher up in the stack are more angled than plates below
                const angleDest = (angle / BalanceState.STACK_ANGLE_COEFFICIENT) * (Math.pow(index, 2) * BalanceState.STACK_ANGLE_HEIGHT_COEFFICIENT);
                item.angle = Phaser.Math.linearInterpolation([item.angle, angleDest], 0.05);
                // Moves the plates horizontally. Plates higher up will be more offset than plates below (see the i * i call), giving the stack a curve
                const xOffset = item.stackOffsetX + (angle / BalanceState.ITEM_HORIZONTAL_OFFSET_ANGLE_COEFFICIENT) * (Math.pow(index, 2) * BalanceState.ITEM_HORIZONTAL_OFFSET_HEIGHT_COEFFICIENT);
                item.x = Phaser.Math.linearInterpolation([item.x, xOffset], 0.05);
            }
            this.stackBase.angle = Phaser.Math.linearInterpolation([this.stackBase.angle, this.angle * this.config.baseAngleModifier], 0.05);
            this.stack.angle = Phaser.Math.linearInterpolation([this.stack.angle, this.angle * this.config.baseAngleModifier], 0.05);
        }
        onGameOver(score) {
            return __awaiter(this, void 0, void 0, function* () {
                this.isGameRunning = false;
                this.isTimerRunning = false;
                this.game.input.deleteMoveCallback(this.onBShowTapMove, this);
                this.game.input.onUp.remove(this.onBShowTapRelease, this);
                PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.gyroCallback);
                this.hideHint();
                yield GameController_17.GameController.Instance.achievementManager.onWoodyBalancingEndGame(this.itemsDropped);
                let resultsString;
                if (this.itemsDropped === 0) {
                    resultsString = LocalizationManager_16.LocalizationManager.Instance.text("stacker.result_message_alt");
                }
                else {
                    resultsString = LocalizationManager_16.LocalizationManager.Instance.textWithReplacements("stacker.results_message", { items_dropped: this.itemsDropped });
                }
                yield this.dialog.show();
                yield this.dialog.writeMessage(resultsString);
                // allows for tapping the whole screen
                yield Promise.race([
                    this.dialog.waitForTap(),
                    PromiseUtil_25.PromiseUtil.signal(this.game.input.onTap)
                ]);
                yield this.dialog.hide();
                this.transitionToNextGame(score);
            });
        }
        transitionToNextGame(score) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this.trainLoop).to({ volume: 0 }, 500, Phaser.Easing.Linear.None, true);
                yield this.showOverlay();
                this.trackMicroEnd(this.itemsDropped);
                ////
                ThrowState_1.ThrowState.startWoodyThrow(this.game, BalanceState.players.reverse(), score);
            });
        }
        shutdown() {
            this.game.stage.backgroundColor = null;
            BalanceState.players = [];
            BalanceState.score = 0;
            this.game.input.deleteMoveCallback(this.onBShowTapMove, this);
            this.game.input.onUp.remove(this.onBShowTapRelease, this);
            PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.gyroCallback);
            this.gyroCallback = null;
            this.dialog = null;
            this.stack = null;
            this.stackBase = null;
            this.train = null;
            this.passAndPlayAnimation = null;
            this.curtain = null;
            this.readyButton = null;
            this.tutorialTopMessageBar = null;
            this.tutorialBottomMessageBar = null;
            this.instructionImage = null;
            this.headerBar = null;
            this.warningBar = null;
            this.hintAnimation = null;
            this.trainLoop.destroy();
            this.trainLoop = null;
            this.game.cache.removeImage('target-1');
            this.game.cache.removeImage('target-2');
            this.game.cache.removeImage('target-10');
            this.game.cache.removeImage('clouds');
            this.game.cache.removeImage('bg-detail');
            this.game.cache.removeImage('tiling-bg');
            this.game.cache.removeImage('target-dropping-1');
            this.game.cache.removeImage('carriage-0');
            this.game.cache.removeImage('carriage-1');
            this.game.cache.removeImage('carriage-2');
            this.game.cache.removeImage('hands-base');
            this.game.cache.removeImage('track-sheet');
            this.game.cache.removeImage('character-dialog-woody');
            // Instructions
            this.game.cache.removeImage('instruction-graphic-woody-balance');
            this.game.cache.removeImage('warning-tile');
            this.game.cache.removeImage('warning-sign');
            this.game.cache.removeImage('balance-prompt');
            this.game.cache.removeSound('woody-target-fall-1');
            this.game.cache.removeSound('woody-target-fall-2');
            this.game.cache.removeSound('woody-target-fall-3');
            this.game.cache.removeSound('woody-target-break-1');
            this.game.cache.removeSound('woody-target-break-2');
            this.game.cache.removeSound('woody-target-break-3');
            super.shutdown();
        }
        static startWoodyBalance(game, players, score) {
            BalanceState.players = players;
            BalanceState.score = score;
            CutSceneState_4.CutSceneState.showCutScene(game, "game_woody_balance", StateNames_14.StateNames.GAME_WOODY_BALANCE);
        }
    }
    // Making this LARGER makes the plates angle LESS
    BalanceState.STACK_ANGLE_COEFFICIENT = 150;
    // Making this LARGER makes the plates angle MORE as they are higher up the stack
    BalanceState.STACK_ANGLE_HEIGHT_COEFFICIENT = 0.25;
    // Making this LARGER makes the plates move MORE (horizontally) in relation to the plate below
    BalanceState.ITEM_HORIZONTAL_OFFSET_HEIGHT_COEFFICIENT = 0.4;
    // Making this LARGER makes the plates move LESS (horizontally) as the angle increases
    BalanceState.ITEM_HORIZONTAL_OFFSET_ANGLE_COEFFICIENT = 15;
    BalanceState.STARTING_ITEM_NUM = 25;
    // tracking
    BalanceState.TRACKING_MICRO_NAME = "Balance";
    exports.BalanceState = BalanceState;
});
define("minigame/woody/component/Item", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * The item class exists to hold on to the xOffset and value.
     * It also does a bit of work with the xOffset value to figure out its true
     * pivot point so that it looks correct when the stack wobbles.
     * @author joel@cerebralfix.com
     */
    class Item extends Phaser.Image {
        constructor(game, stackOffsetX, value) {
            super(game, 0, 0, `target-${value}`);
            this.stackOffsetX = 0;
            this._highlighted = false;
            this.value = value;
            this.stackOffsetX = stackOffsetX;
            this.anchor.set(0.5, 1);
            const xPivot = (this.width * 0.5 - stackOffsetX) / this.width;
            this.pivot.set(xPivot, 1);
            // Highlighted overlay
            this.highlightGraphic = new Phaser.Graphics(game, this.left, this.top);
            this.highlightGraphic.beginFill(0xFFFFFF, 1);
            this.highlightGraphic.drawRect(0, 0, this.width, this.height);
            this.highlightGraphic.endFill();
            this.highlightGraphic.visible = false;
            this.addChild(this.highlightGraphic);
        }
        /**
         * Controls the highlighted state of the item.
         * When this gets turned on or off for the first time,
         * tweens the highlight overlay in or out quickly.
         */
        set highlighted(h) {
            // Don't do anything if we're already in this state
            if (this._highlighted === h) {
                return;
            }
            this._highlighted = h;
            this.game.tweens.removeFrom(this.highlightGraphic);
            if (this._highlighted) {
                this.highlightGraphic.visible = true;
                this.game.add.tween(this.highlightGraphic).to({ alpha: 0.8 }, 100, Phaser.Easing.Linear.None, true, 0);
            }
            else {
                this.game.add.tween(this.highlightGraphic).to({ alpha: 0 }, 100, Phaser.Easing.Linear.None, true, 0).onComplete.addOnce(() => {
                    this.highlightGraphic.visible = false;
                }, this);
            }
        }
        get highlighted() {
            return this._highlighted;
        }
    }
    exports.Item = Item;
});
define("minigame/woody/component/Cracker", ["require", "exports", "minigame/woody/component/Item"], function (require, exports, Item_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author jeronimo@cerebralfix.com
     */
    class Cracker extends Item_1.Item {
        constructor(game, stackOffsetX, value) {
            super(game, stackOffsetX, value);
            // Creates a spark
            this.spark = new Phaser.Image(game, 0, 0, 'lasso-fuse-spark');
            this.spark.anchor.set(0.5);
            this.spark.x = this.width * 0.5;
            this.spark.y = this.height * -0.63;
            this.addChildAt(this.spark, 0);
            // random starting angle
            this.spark.angle = Math.random() * 360;
            // animations
            this.game.add.tween(this.spark).to({ angle: this.spark.angle + 360 }, 750, Phaser.Easing.Linear.None, true, 0, Infinity);
            this.game.add.tween(this.spark).to({ alpha: 0.8 }, 150, Phaser.Easing.Linear.None, true, 0, Infinity, true);
            this.game.add.tween(this.spark.scale).to({ x: 0.8, y: 0.8 }, 150, Phaser.Easing.Linear.None, true, 0, Infinity, true);
        }
    }
    exports.Cracker = Cracker;
});
define("minigame/woody/lasso/LassoConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class LassoConfig {
        parse(data) {
            this.minItemsBetweenCrates = data.minItemsBetweenCrates || 3;
            this.minItemsBetweenCrackers = data.minItemsBetweenCrackers || 2;
            this.throwsPerPlayer = data.throwsPerPlayer || 4;
            this.throwThreshold = data.throwThreshold || 0.6;
            this.pullThreshold = data.pullThreshold || 0.3;
            this.crackerBlowDistance = data.crackerBlowDistance || 3;
            this.crackerMaxThrows = data.crackerMaxThrows || 2;
            this.timeBetweenThrowsToShowPromptMs = data.timeBetweenThrowsToShowPromptMs || 5000;
            this.stackDropTimeMs = data.stackDropTimeMs || 300;
            this.pullDurationMs = data.pullDurationMs || 700;
            this.timeDelayBeforePullMs = data.timeDelayBeforePullMs || 250;
        }
    }
    exports.LassoConfig = LassoConfig;
});
define("minigame/woody/lasso/LassoDifficultyConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class LassoDifficultyConfig {
        parse(data) {
            this.lassoDropDuration = data.lassoDropDuration || 2500;
            this.isCrackerEnabled = data.isCrackerEnabled || true;
        }
    }
    exports.LassoDifficultyConfig = LassoDifficultyConfig;
});
define("minigame/woody/lasso/LassoState", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "switchback-common/extension/Easing", "Game", "StateNames", "component/dialog/CharacterDialog", "logic/GameController", "minigame/woody/component/Item", "minigame/woody/component/Cracker", "Character", "component/MessageBar", "component/Curtain", "component/button/TextArrowButton", "component/HeaderBar", "component/PassAndPlayShowable", "component/fx/Snatch", "minigame/woody/lasso/LassoConfig", "minigame/woody/lasso/LassoDifficultyConfig", "state/GameBaseState", "minigame/woody/balance/BalanceState", "component/dialog/BasicDialog", "switchback-common/util/MotionUtil", "model/ToyStoryConfig"], function (require, exports, PromiseUtil_26, TextUtil_16, ScaleUtil_11, LocalizationManager_17, Easing_3, Game_22, StateNames_15, CharacterDialog_3, GameController_18, Item_2, Cracker_1, Character_4, MessageBar_7, Curtain_4, TextArrowButton_5, HeaderBar_3, PassAndPlayShowable_3, Snatch_2, LassoConfig_1, LassoDifficultyConfig_1, GameBaseState_5, BalanceState_1, BasicDialog_2, MotionUtil_5, ToyStoryConfig_12) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Woody's Game - Part 1 - Lassoing Targets
     *
     * This game revolves around using motion controls to grab targets from a stack.
     * The stack will replenish after a target is grabbed, and will decrement the count of
     * throws the current player has. Score is calculated based on the value of the item grabbed
     * If the player is out of throws, the next player steps up.
     *
     * Once all players have finished, the game is over and transitions to pt 2.
     *
     * The main complexity in this class comes from the lassoing animation,
     * which is split into 2 parts - throw and pull - detailed at their respective functions
     *
     * @author joel@cerebralfix.com
     */
    var PlayerInput;
    (function (PlayerInput) {
        PlayerInput[PlayerInput["LOCKED"] = 0] = "LOCKED";
        PlayerInput[PlayerInput["CAN_THROW"] = 1] = "CAN_THROW";
        PlayerInput[PlayerInput["CAN_PULL"] = 2] = "CAN_PULL";
    })(PlayerInput || (PlayerInput = {}));
    var TutorialState;
    (function (TutorialState) {
        TutorialState[TutorialState["FIRST_TUTORIAL"] = 0] = "FIRST_TUTORIAL";
        TutorialState[TutorialState["SOFT_TUTORIAL"] = 1] = "SOFT_TUTORIAL";
        TutorialState[TutorialState["NO_TUTORIAL"] = 2] = "NO_TUTORIAL"; // No tutorial is used on people's second+ turn
    })(TutorialState || (TutorialState = {}));
    class LassoState extends GameBaseState_5.GameBaseState {
        init(...params) {
            this.stateName = StateNames_15.StateNames.GAME_WOODY_LASSO;
            this._trackMicroName = "Lasso";
            super.init(params);
            this.MAX_TIMEOUT_STUCK_MS = 120000;
            this.roundTimeMs = 0;
            this.config = this.loadConfig(LassoConfig_1.LassoConfig);
            this.difficultyConfig = this.loadDifficultyConfig(LassoDifficultyConfig_1.LassoDifficultyConfig);
            this.inputState = PlayerInput.LOCKED;
            this.lassoYPos = 0;
            this.numThrowsLeft = this.config.throwsPerPlayer;
            this.throwsUntilExplosion = this.config.crackerMaxThrows;
            this._score = 0;
            this.playerIndex = 0;
            this.players = GameController_18.GameController.Instance.playerManager.getRefreshedPlayersArray();
            this.promptShowing = false;
            this.areInstructionsAlreadyShowing = true;
            this.tutorialState = MotionUtil_5.MotionUtil.isAccelerometerAvailable &&
                GameController_18.GameController.Instance.state.getMinigameSaveData(this.stateName).numPlays === 0 ?
                TutorialState.FIRST_TUTORIAL : TutorialState.NO_TUTORIAL;
            this.itemsGeneratedSinceLastCracker = 0;
            this.itemsGeneratedSinceLastCrate = 0;
        }
        preload() {
            super.preload();
            this.game.load.image('lasso-tiling-front', 'assets/minigame/woody/lasso_tiling_front.png');
            this.game.load.image('lasso-tiling-back', 'assets/minigame/woody/lasso_tiling_back.png');
            this.game.load.image('target-0', 'assets/minigame/woody/target_0.png');
            this.game.load.image('target-1', 'assets/minigame/woody/target_1.png');
            this.game.load.image('target-2', 'assets/minigame/woody/target_2.png');
            this.game.load.image('target-10', 'assets/minigame/woody/target_10.png');
            this.game.load.image('clouds', 'assets/minigame/woody/clouds.png');
            this.game.load.image('bg-detail', 'assets/minigame/woody/bg_detail.png');
            this.game.load.image('tiling-bg', 'assets/minigame/woody/tiling-bg.jpg');
            this.game.load.image('lasso-fuse-spark', 'assets/minigame/woody/fuse_spark.png');
            this.game.load.image('target-dropping-1', 'assets/minigame/woody/target_dropping_1.png');
            this.game.load.image('target-dropping-2', 'assets/minigame/woody/target_dropping_2.png');
            this.game.load.image('target-dropping-10', 'assets/minigame/woody/target_dropping_10.png');
            this.game.load.image('carriage-base', 'assets/minigame/woody/carriage_base.png');
            this.game.load.atlas('train-front-pete', 'assets/minigame/woody/train_front_pete.png', 'assets/minigame/woody/train_front_pete.json');
            this.game.load.image('target-whole-icon', 'assets/minigame/woody/target_whole_icon.png');
            this.game.load.image('hands-base', 'assets/minigame/woody/hands_base.png');
            this.game.load.image('lasso-icon', 'assets/minigame/woody/lasso_icon.png');
            this.game.load.image('screen-flash', 'assets/minigame/woody/screen_flash.png');
            // Snatch FX
            this.game.load.image('snatch-radialfx', 'assets/minigame/woody/snatch_radialfx.png');
            this.game.load.image('snatch-star-green', 'assets/minigame/woody/star_green.png');
            this.game.load.image('snatch-star-pink', 'assets/minigame/woody/star_pink.png');
            this.game.load.image('snatch-star-blue', 'assets/minigame/woody/star_blue.png');
            this.game.load.atlas('track-sheet', 'assets/minigame/woody/track_sheet.png', 'assets/minigame/woody/track_sheet.json');
            this.game.load.spritesheet('lasso-sheet', 'assets/minigame/woody/lasso_sheet.png', 540, 1080);
            this.game.load.spritesheet('lasso-explosion', 'assets/minigame/woody/cracker_explosion.png', 672, 507);
            // Instructions
            this.game.load.image('character-dialog-woody', 'assets/dialog/character_dialog_woody.png');
            this.game.load.image('character-dialog-backplate', 'assets/dialog/character_dialog_backplate.png');
            this.game.load.image('instruction-dialog-backplate', 'assets/dialog/instruction_dialog_backplate.png');
            this.game.load.image('instruction-graphic-woody-lasso', 'assets/dialog/instruction_graphic_woody1.png');
            this.game.load.atlas('lasso-prompt', 'assets/minigame/woody/lasso-prompt.png', 'assets/minigame/woody/lasso-prompt.json');
            this.game.load.atlas('tutorial-arrow', 'assets/minigame/woody/tutorial-arrow.png', 'assets/minigame/woody/tutorial-arrow.json');
            this.game.load.image('tiling-bar', 'assets/common/CTAbar_tilingbg.png');
            this.game.load.image('cta-curtain', 'assets/common/cta-curtain.png');
            this.game.load.image('cta-header', 'assets/dialog/cta_header.png');
            this.game.load.image('next-button', 'assets/common/next-button.png');
            this.game.load.image('right-figure', 'assets/common/right_figure.png');
            this.game.load.image('left-figure', 'assets/common/left_figure.png');
            this.game.load.image('phone', 'assets/common/phone.png');
            this.game.load.audio('woody-cracker-explode', ['assets/sound/woody/cracker_explode.m4a', 'assets/sound/woody/cracker_explode.ogg']);
            this.game.load.audio('woody-pull-crate', ['assets/sound/woody/pull_crate.m4a', 'assets/sound/woody/pull_crate.ogg']);
            this.game.load.audio('woody-pull-target', ['assets/sound/woody/pull_target.m4a', 'assets/sound/woody/pull_target.ogg']);
            this.game.load.audio('woody-pull-lasso-1', ['assets/sound/woody/pull_lasso_1.m4a', 'assets/sound/woody/pull_lasso_1.ogg']);
            this.game.load.audio('woody-pull-lasso-2', ['assets/sound/woody/pull_lasso_2.m4a', 'assets/sound/woody/pull_lasso_2.ogg']);
            this.game.load.audio('woody-pull-lasso-3', ['assets/sound/woody/pull_lasso_3.m4a', 'assets/sound/woody/pull_lasso_3.ogg']);
            this.game.load.audio('woody-throw-lasso-1', ['assets/sound/woody/throw_lasso_1.m4a', 'assets/sound/woody/throw_lasso_1.ogg']);
            this.game.load.audio('woody-throw-lasso-2', ['assets/sound/woody/throw_lasso_2.m4a', 'assets/sound/woody/throw_lasso_2.ogg']);
            this.game.load.audio('woody-throw-lasso-3', ['assets/sound/woody/throw_lasso_3.m4a', 'assets/sound/woody/throw_lasso_3.ogg']);
            this.game.load.audio('woody-train-loop', ['assets/sound/woody/train_loop.m4a', 'assets/sound/woody/train_loop.ogg']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.trackGameState(null, LassoState.TRACKING_MICRO_NAME);
                // Snatch group for fxs
                this.snatchGroup = new Snatch_2.Snatch(this.game);
                this.add.existing(this.snatchGroup);
                ///////////////////////////
                // Tiling room background
                ///////////////////////////
                const bgFloor = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height, "tiling-bg");
                const bgTextureScale = bgFloor.height / bgFloor.texture.height;
                bgFloor.tileScale.y = bgTextureScale;
                const scrollSpeed = -this.game.width * 0.05;
                ///////////////////////////
                // Clouds
                ///////////////////////////
                const tilingClouds = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height * 0.5, "clouds");
                const cloudTextureScale = tilingClouds.height / tilingClouds.texture.height;
                tilingClouds.tileScale.set(cloudTextureScale, cloudTextureScale);
                tilingClouds.autoScroll(scrollSpeed * 0.5, 0);
                ///////////////////////////
                // Background detail
                ///////////////////////////
                const bgDetail = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height * 0.075, "bg-detail");
                const bgDetailScale = bgDetail.height / bgDetail.texture.height;
                bgDetail.tileScale.set(bgDetailScale, bgDetailScale);
                bgDetail.bottom = this.game.height * 0.715;
                bgDetail.autoScroll(scrollSpeed, 0);
                ///////////////////////////
                // Animating Tracks
                ///////////////////////////
                const tracks = new Phaser.Sprite(this.game, this.game.width, this.game.height, "track-sheet");
                tracks.anchor.set(1, 0.95);
                tracks.height = this.game.height * 0.34;
                tracks.scale.x = tracks.scale.y;
                tracks.animations.add("loop", [0, 1, 2, 3, 4]);
                tracks.animations.play("loop", 30, true);
                ///////////////////////////
                // Carriage Image (carrying stack)
                ///////////////////////////
                const stackBase = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height, 'carriage-base');
                stackBase.anchor.set(0.5, 0.8);
                stackBase.width = this.game.width;
                stackBase.scale.y = stackBase.scale.x;
                this.game.add.tween(stackBase).to({ y: stackBase.y - this.game.height * 0.01 }, 200, Easing_3.shake(0.1, 0.1), true, 0, Infinity);
                ///////////////////////////
                // Train Image
                ///////////////////////////
                const trainFront = new Phaser.Image(this.game, this.game.width * 0.5, stackBase.top, 'train-front-pete');
                trainFront.anchor.set(0.15, 0.5);
                trainFront.width = this.game.width * 0.5;
                trainFront.scale.y = trainFront.scale.x;
                trainFront.animations.add("loop", null, 18, true).play();
                this.game.add.tween(trainFront).to({ y: trainFront.y - this.game.height * 0.0075 }, 150, Easing_3.shake(0.1, 0.1), true, 0, Infinity);
                ///////////////////////////
                // Stack
                ///////////////////////////
                this.stack = new Phaser.Group(this.game);
                this.stack.x = this.game.width * 0.5;
                this.stack.y = stackBase.top + stackBase.height * 0.15;
                this.addItem(this.stack.children.length, this.getRandomItemValue());
                this.game.add.tween(this.stack).to({ y: this.stack.y - this.game.height * 0.01 }, 200, Easing_3.shake(0.1, 0.1), true, 0, Infinity);
                ///////////////////////////
                // Dialog Prompt
                ///////////////////////////
                this.dialog = new CharacterDialog_3.CharacterDialog(this.game, Character_4.Character.WOODY);
                this.dialog.centerY = this.game.height * 0.15;
                ///////////////////////////
                // Score + icon
                ///////////////////////////
                this.itemIcon = new Phaser.Image(this.game, 0, this.game.height, 'target-whole-icon');
                ScaleUtil_11.scaleToBounds(this.itemIcon, this.game.width * 0.2, this.game.height * 0.2);
                this.itemIcon.anchor.set(0, 1);
                this.itemCounterText = new Phaser.Text(this.game, this.itemIcon.left + this.itemIcon.width * 0.7, this.itemIcon.top + this.itemIcon.height * 0.6, "x0", {
                    fontSize: TextUtil_16.TextUtil.getFontSize(this.game.width, 30),
                    font: Game_22.Game.DEFAULT_FONT,
                    fill: "#F8F1D5",
                    align: 'left',
                    stroke: "#194DA1",
                    strokeThickness: 4
                });
                this.itemCounterText.anchor.set(0, 0.5);
                ///////////////////////////
                // Animating Rope Pull
                ///////////////////////////
                this.pullRope = new Phaser.Sprite(this.game, 0, 0, 'lasso-sheet');
                this.pullRope.animations.add('loop', [0, 1, 2, 3, 4, 5, 6, 7], 30);
                this.pullRope.width = this.game.width;
                this.pullRope.scale.y = this.pullRope.scale.x;
                this.pullRope.visible = false;
                ///////////////////////////
                // Swirling Lasso Rope
                ///////////////////////////
                const scrollSpeedX = this.game.width * 2;
                this.backRope = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height * 0.2, 'lasso-tiling-back');
                this.backRope.height = this.backRope.texture.height;
                this.backRope.autoScroll(-scrollSpeedX, 0);
                this.backRope.anchor.y = 0.5;
                this.frontRope = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height * 0.2, 'lasso-tiling-front');
                this.frontRope.height = this.frontRope.texture.height;
                this.frontRope.autoScroll(scrollSpeedX, 0);
                this.frontRope.anchor.y = 0.5;
                this.resetLasso(true);
                ///////////////////////////
                // Lasso Prompt
                ///////////////////////////
                this.lassoPrompt = new Phaser.Image(this.game, 0, this.game.height, 'lasso-prompt');
                this.lassoPrompt.width = this.game.width;
                this.lassoPrompt.scale.y = this.lassoPrompt.scale.x;
                this.lassoPrompt.bottom = this.game.height;
                this.lassoPrompt.animations.add('pull', [0, 0, 0, 0, 0, 0, 0, 1, 1, 1], 12, true);
                this.lassoPrompt.animations.add('throw', [1, 1, 1, 1, 1, 1, 1, 0, 0, 0], 12, true);
                this.lassoPrompt.alpha = 0;
                ///////////////////////////
                // Tutorial Arrow
                ///////////////////////////
                this.tutorialArrow = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.35, 'tutorial-arrow');
                this.tutorialArrow.width = this.game.width * 0.75;
                this.tutorialArrow.scale.y = this.tutorialArrow.scale.x;
                this.tutorialArrow.anchor.set(0.5);
                this.tutorialArrow.alpha = 0;
                ///////////////////////////
                // Curtain
                ///////////////////////////
                this.curtain = new Curtain_4.Curtain(this.game, true);
                this.curtain.centerX = this.game.width * 0.5;
                ///////////////////////////
                // Instruction Image
                ///////////////////////////
                this.instructionImage = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.45, 'instruction-graphic-woody-lasso');
                ScaleUtil_11.scaleToBounds(this.instructionImage, this.game.width * 0.8, this.game.height * 0.225);
                this.instructionImage.alpha = 0;
                this.instructionImage.anchor.set(0.5);
                ///////////////////////////
                // Message Bars
                ///////////////////////////
                this.tutorialTopMessageBar = new MessageBar_7.MessageBar(this.game, LocalizationManager_17.LocalizationManager.Instance.text("lasso.tutorial_throw"));
                this.tutorialTopMessageBar.centerX = this.game.width * 0.5;
                this.tutorialTopMessageBar.bottom = 0;
                this.tutorialBottomMessageBar = new MessageBar_7.MessageBar(this.game, LocalizationManager_17.LocalizationManager.Instance.text("lasso.tutorial_pull"));
                this.tutorialBottomMessageBar.centerX = this.game.width * 0.5;
                this.tutorialBottomMessageBar.top = this.game.height;
                ///////////////////////////
                // Next Button
                ///////////////////////////
                this.instructionNextButton = new TextArrowButton_5.TextArrowButton(this.game, LocalizationManager_17.LocalizationManager.Instance.text("common.ready").toUpperCase());
                this.instructionNextButton.alpha = 0;
                this.instructionNextButton.centerX = this.game.width * 0.5;
                this.instructionNextButton.centerY = this.game.height * 0.785;
                this.instructionNextButton.startPulse();
                ///////////////////////////
                // Pass and Play
                ///////////////////////////
                this.passAndPlayShowable = new PassAndPlayShowable_3.PassAndPlayShowable(this.game, LocalizationManager_17.LocalizationManager.Instance.textWithReplacements("lasso.instructions_pass", {
                    player_name: this.players[this.playerIndex].name
                }).toUpperCase());
                this.passAndPlayShowable.x = this.game.width * 0.5;
                this.passAndPlayShowable.y = this.game.height * 0.5;
                ///////////////////////////
                // HeaderBar
                ///////////////////////////
                this.headerBar = new HeaderBar_3.HeaderBar(this.game, LocalizationManager_17.LocalizationManager.Instance.text("common.how_to_play"));
                this.headerBar.centerX = this.game.width * 0.5;
                this.headerBar.bottom = 0;
                ///////////////////////////
                // Motion Listener
                ///////////////////////////
                if (MotionUtil_5.MotionUtil.isAccelerometerAvailable) {
                    this.motionCallback = (result) => {
                        this.onMotionChanged(result);
                    };
                }
                else {
                    this.tutorialState = TutorialState.NO_TUTORIAL;
                    this.game.input.onDown.add(this.onBShowTap, this);
                }
                ///////////////////////////
                // 'Throws Left' Icons
                ///////////////////////////
                this.throwsLeftIcons = new Phaser.Group(this.game);
                let xPos = this.game.width;
                for (let i = 0; i < this.config.throwsPerPlayer; i++) {
                    const icon = new Phaser.Image(this.game, xPos, this.game.height, 'lasso-icon');
                    icon.anchor.set(1);
                    ScaleUtil_11.scaleToBounds(icon, this.game.width * 0.175, this.game.height * 0.1);
                    this.throwsLeftIcons.add(icon);
                    xPos -= icon.width * 0.8;
                }
                ///////////////////////////
                // Layering
                ///////////////////////////
                this.add.existing(bgFloor);
                this.add.existing(tilingClouds);
                this.add.existing(bgDetail);
                this.add.existing(tracks);
                this.add.existing(trainFront);
                this.add.existing(this.backRope);
                this.add.existing(stackBase);
                this.add.existing(this.stack);
                this.add.existing(this.frontRope);
                this.add.existing(this.pullRope);
                this.add.existing(this.lassoPrompt);
                this.add.existing(this.tutorialArrow);
                this.add.existing(this.throwsLeftIcons);
                this.add.existing(this.itemIcon);
                this.add.existing(this.itemCounterText);
                this.add.existing(this.dialog);
                this.add.existing(this.curtain);
                this.add.existing(this.instructionImage);
                this.add.existing(this.passAndPlayShowable);
                this.add.existing(this.tutorialTopMessageBar);
                this.add.existing(this.tutorialBottomMessageBar);
                this.add.existing(this.instructionNextButton);
                this.add.existing(this.headerBar);
                // Move interrupt buttons to the top of the screen for this game
                this.trainLoop = this.game.add.sound('woody-train-loop', 1.0, true);
                this.trainLoop.play();
                this.game.add.tween(this.trainLoop).from({ volume: 0 }, 500, Phaser.Easing.Linear.None, true);
                yield this.dismissOverlay();
                if (ToyStoryConfig_12.ToyStoryConfig.Instance.skipGames) {
                    this.transitionToNextGame(50);
                }
                else {
                    this.startGame();
                }
            });
        }
        startGame() {
            return __awaiter(this, void 0, void 0, function* () {
                this.roundTimeMs = 0;
                this.inputState = PlayerInput.LOCKED;
                PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.motionCallback);
                // Wait for the curtains to show, unless they're already there
                if (!this.areInstructionsAlreadyShowing) {
                    yield this.curtain.show(true);
                }
                // Show the correct player to pass to
                yield this.showThenHidePassAndPlay();
                // Show the how to play instructions
                if (MotionUtil_5.MotionUtil.isAccelerometerAvailable) {
                    yield this.showThenHideInstructions();
                }
                // Hide the curtain, showing the graphical prompt afterwards if this is their first time playing
                yield this.curtain.hide();
                this.showPrompt('throw');
                // Now that the dialog has been dismissed, actually listen for motion
                PlayAPI.subscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.motionCallback);
                this.areInstructionsAlreadyShowing = false;
                this.timeSinceThrowMs = 0;
                this.inputState = PlayerInput.CAN_THROW;
                this.throwsUntilExplosion = this.config.crackerMaxThrows;
                this.trackMicroStart();
            });
        }
        showThenHidePassAndPlay() {
            return __awaiter(this, void 0, void 0, function* () {
                this.trackGamePassAndPlay(LassoState.TRACKING_MICRO_NAME);
                this.passAndPlayShowable.text = LocalizationManager_17.LocalizationManager.Instance.textWithReplacements("lasso.instructions_pass", {
                    player_name: this.players[this.playerIndex].name
                }).toUpperCase();
                yield this.passAndPlayShowable.show();
                yield PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionNextButton)
                    .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true));
                yield PromiseUtil_26.PromiseUtil.signal(this.instructionNextButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                yield Promise.all([
                    PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionNextButton)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                    this.passAndPlayShowable.hide()
                ]);
            });
        }
        showThenHideInstructions() {
            return __awaiter(this, void 0, void 0, function* () {
                this.trackGameInstructions(LassoState.TRACKING_MICRO_NAME);
                // reset the bottom bar's text (it may have changed)
                this.tutorialBottomMessageBar.text = LocalizationManager_17.LocalizationManager.Instance.text('lasso.tutorial_pull');
                const durationMs = 500;
                // Reposition and fade in all the instruction elements
                const yPadding = this.game.height * 0.0125;
                yield Promise.all([
                    PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this.headerBar)
                        .to({ top: 0 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialTopMessageBar)
                        .to({ bottom: this.instructionImage.top - yPadding }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialBottomMessageBar)
                        .to({ top: this.instructionImage.bottom + yPadding }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionImage)
                        .to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionNextButton)
                        .to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true))
                ]);
                yield PromiseUtil_26.PromiseUtil.signal(this.instructionNextButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                // Get rid of all the tutorial elements
                // Speed up the fades so they look a bit better
                Promise.all([
                    this.showMessage(true),
                    this.hideMessage(false),
                    PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this.headerBar)
                        .to({ bottom: 0 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionImage)
                        .to({ alpha: 0 }, durationMs * 0.5, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionNextButton)
                        .to({ alpha: 0 }, durationMs * 0.5, Phaser.Easing.Linear.None, true))
                ]);
            });
        }
        /**
         * The Throw
         *
         * This is triggered after the player throws the phone forwards.
         *
         * This starts a vertically descending tween
         * of 2 TileSprites moving in opposite directions - these are the front and back loops of rope (layered infront
         * and behind the stack). As the rope descends through the stack, it highlights the currently selected item.
         * If the rope reaches the bottom of the stack without the player pulling it back in, it fades out and
         * is counted as a failed throw. The rope then resets back to the top to await the next throw.
         */
        throw() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.inputState !== PlayerInput.CAN_THROW) {
                    return;
                }
                // TUTORIAL Hide throw graphic
                this.hidePrompt();
                this.hideMessage(true);
                this.inputState = PlayerInput.LOCKED;
                this.currentlySelectedItem = null;
                // Show the ropes and begin moving the yPos down the stack 
                // (from the top of the screen to the base of the stack)
                this.frontRope.alpha = this.backRope.alpha = 1;
                // The lasso drops to the bottom of the stack
                let lassoDest = this.stack.bottom;
                // But we may be moving to an /offset/ from here e.g. (part ways up) if we're in a tutorial
                switch (this.tutorialState) {
                    case TutorialState.FIRST_TUTORIAL: {
                        // TUTORIAL If tutorial stage 1 throwDest is 3/4 of the way down the stack
                        // Find something that isn't a cracker to stop on!
                        let childIndex = Math.floor(this.stack.children.length * 0.75);
                        if (typeof this.stack.children[childIndex] === typeof Cracker_1.Cracker) {
                            childIndex++;
                        }
                        lassoDest += this.stack.children[childIndex].centerY;
                        break;
                    }
                    case TutorialState.SOFT_TUTORIAL: {
                        // TUTORIAL If tutorial stage 2 throwDest is second from bottom of the stack
                        lassoDest += this.stack.children[this.stack.children.length - 1].centerY;
                        break;
                    }
                }
                this.lassoYPosTween = this.game.add.tween(this).to({ lassoYPos: lassoDest }, this.difficultyConfig.lassoDropDuration, Phaser.Easing.Linear.None, true, 0);
                this.game.sound.play(`woody-throw-lasso-${this.game.rnd.integerInRange(1, 3)}`);
                if (this.tutorialState === TutorialState.NO_TUTORIAL) {
                    // TUTORIAL If no tutorial, listen for hitting the bottom - a failure!
                    this.lassoYPosTween.onComplete.addOnce(this.onFailedToSelect, this);
                }
                else {
                    // TUTORIAL If tutorial, listen for finishing - and show the pull graphic
                    this.lassoYPosTween.onComplete.addOnce(() => {
                        this.showMessage(false, LocalizationManager_17.LocalizationManager.Instance.text(this.tutorialState === TutorialState.FIRST_TUTORIAL ? 'lasso.tutorial_pull' : 'lasso.tutorial_bottom_of_stack'));
                        this.showPrompt('pull');
                    }, this);
                }
                // After a short delay, pulling is enabled
                PromiseUtil_26.PromiseUtil.time(this.game, this.config.timeDelayBeforePullMs).then(() => {
                    this.inputState = PlayerInput.CAN_PULL;
                });
            });
        }
        // returns a list of exploding items in the same order they're stored in the stack. Handles chaining of crackles
        getExplodingItems(cracker) {
            let items = [];
            let currentItemIndex = this.stack.children.indexOf(cracker);
            let top = Math.min(this.stack.length - 1, currentItemIndex + this.config.crackerBlowDistance);
            // now go forward (this way we get all the items in order which is important for animation)
            for (let i = currentItemIndex; i <= top; i++) {
                let item = this.stack.children[i];
                // if the item hasn't been visited, add it and process it
                if (items.indexOf(item) >= 0) {
                    continue;
                }
                // add it
                items.push(item);
                // if its a cracker process it (theoretically there should never be a cracker below this cracker)
                if (item instanceof Cracker_1.Cracker) {
                    top = Math.min(this.stack.length - 1, Math.max(top, i + this.config.crackerBlowDistance));
                }
            }
            return items;
        }
        // Shakes the camera. if somebody else needs this, then it should be moved to a library
        cameraShake(intensity, durationMS) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.camera.shake(intensity, durationMS);
                yield PromiseUtil_26.PromiseUtil.time(this.game, durationMS);
            });
        }
        // Animates an explosion
        animateExplosion(items) {
            return __awaiter(this, void 0, void 0, function* () {
                let toLeft = true;
                let promises = [];
                for (let item of items) {
                    // if it's a cracker do the explosion
                    if ((item instanceof Cracker_1.Cracker)) {
                        let explosion = new Phaser.Sprite(this.game, 0, 0, "lasso-explosion");
                        explosion.animations.add('loop', [0, 1, 2, 3, 4, 5], 10);
                        // last true will destroy the sprite when it ends (hopefully!)
                        explosion.animations.play('loop', 10, false, true);
                        this.add.existing(explosion);
                        explosion.anchor.set(0.5);
                        explosion.scale.x = explosion.scale.y = (this.game.width / explosion.width) * 2;
                        // important to get the coords from worldPosition (tried otherwise and didn't work)
                        explosion.x = item.worldPosition.x;
                        explosion.y = item.worldPosition.y - (item.height * 0.5);
                        // remove from parent (but don't destroy)
                        if (item.parent) {
                            item.parent.removeChild(item);
                        }
                    }
                    else {
                        // If its a normal item, fly outscreen
                        toLeft = !toLeft;
                        // convert the coords (this works here, but NOT with cracker (no idea why))
                        item.x += this.stack.x;
                        item.y += this.stack.y;
                        item.visible = true;
                        // adds to screen, (removing from stack)
                        this.add.existing(item);
                        // animate rotation and direction
                        let newAngle = item.angle + (toLeft ? -180 : 180);
                        let newX = item.x + (this.game.width * (toLeft ? -1 : 1));
                        this.game.add.tween(item).to({ x: newX }, 500, Phaser.Easing.Linear.None, true);
                        let tween = this.game.add.tween(item).to({ angle: newAngle }, 500, Phaser.Easing.Linear.None, true);
                        let promise = PromiseUtil_26.PromiseUtil.tween(this.game, tween);
                        promises.push(promise);
                    }
                }
                promises.push(this.cameraShake(0.02, 500));
                yield Promise.all(promises);
            });
        }
        // Explodes a certain item, it will not destroy it, but it will destroy the items exploded by this item
        explode(explodedItem) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play(`woody-cracker-explode`);
                // Reset the counter
                this.throwsUntilExplosion = this.config.crackerMaxThrows;
                // Get items
                let items = this.getExplodingItems(explodedItem);
                // Animate the explosions
                yield this.animateExplosion(items);
                // removes the explodedItem from the list (so it doesn't get destroyed)
                items.splice(items.indexOf(explodedItem), 1);
                // and randomizes it (so they are added in random order)
                Phaser.ArrayUtils.shuffle(items);
                // put the currentSelected first (it should be a cracker)
                // so it comply with AC to re insert a cracker as first element
                this.addItem(this.stack.children.length, this.getRandomItemValue());
                for (let item of items) {
                    item.destroy();
                }
                // Drop the stack! (*edm music plays*)
                yield this.dropStack();
            });
        }
        // search for the last cracker and tries to make it explode
        explodeLastCracker() {
            return __awaiter(this, void 0, void 0, function* () {
                // iterate in reverse
                for (let item of this.stack.children.slice().reverse()) {
                    if (item instanceof Cracker_1.Cracker) {
                        yield this.explode(item);
                        item.destroy();
                        break;
                    }
                }
            });
        }
        isLastCrackerOnScreen() {
            // iterate in reverse
            for (let item of this.stack.children.slice().reverse()) {
                if (item instanceof Cracker_1.Cracker) {
                    return item.worldPosition.y > 0;
                }
            }
            return false;
        }
        // Branches the animation of the stack dropping so it looks better
        dropStackBranched() {
            return __awaiter(this, void 0, void 0, function* () {
                let delayMS = this.config.pullDurationMs * 0.3;
                // Part way through the pull animation...
                PromiseUtil_26.PromiseUtil.time(this.game, delayMS).then(() => __awaiter(this, void 0, void 0, function* () {
                    // ...drop the stack to fill in the gap created by the pulled item
                    this.dropStack();
                }));
                // Part way through the pull animation...
                yield PromiseUtil_26.PromiseUtil.time(this.game, delayMS);
            });
        }
        // Puts the snatch group on top (again)
        putSnatchOnTop() {
            // "this" is not a "drawable" object, so i need to make some turnarounds
            let snatchParent = this.snatchGroup.parent;
            snatchParent.setChildIndex(this.snatchGroup, snatchParent.children.length - 1);
        }
        // Does the animation for when an item is taken
        animateSnatch(item) {
            return __awaiter(this, void 0, void 0, function* () {
                // Ensure we are on top
                this.putSnatchOnTop();
                yield this.snatchGroup.show(item.worldPosition.x, item.worldPosition.y - (item.height * 0.5), 1.125);
            });
        }
        /**
         * The Pull
         *
         * Triggered by the player pulling the phone towards them.
         *
         * This function does a lot, and calls out to a bunch of animation helper functions.
         * Essentially, it grabs the item that is highlighted, removes it, plays some
         * animations to emulate a lasso grab and adds score for the grabbed item.
         */
        pull() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.inputState !== PlayerInput.CAN_PULL) {
                    return;
                }
                // If there is no currently selected item in the stack, don't let them pull anything
                if (!this.currentlySelectedItem) {
                    return;
                }
                // TUTORIAL Hide the pull prompt
                this.hidePrompt();
                this.hideMessage(false);
                this.inputState = PlayerInput.LOCKED;
                // Add points according to item's value
                this.score += this.currentlySelectedItem.value;
                /// Mostly animation ahead
                // Reset the swirling lasso rope so it's offscreen
                this.resetLasso(true);
                // Show the lasso whipping animation, centered on the item's world center Y
                this.showPullAnimation(this.currentlySelectedItem.worldPosition.y + (this.currentlySelectedItem.height * 0.5));
                if (this.currentlySelectedItem.value === 10) {
                    this.game.sound.play(`woody-pull-crate`);
                }
                else {
                    this.game.sound.play(`woody-pull-target`);
                }
                // currentlySelectedItem shouldn't be destroyed yet, we use it in a few places below
                let isCracker = this.currentlySelectedItem.value === 0;
                if (isCracker) {
                    yield this.explode(this.currentlySelectedItem);
                    // notice that explode resets the throwsUntilExplosion and drops the stack (both are intentional)
                }
                else {
                    // Add another item to the stack
                    this.addItem(this.stack.children.length, this.getRandomItemValue());
                    // Pull selected item towards the screen
                    yield this.pullItemTowardsCamera(this.currentlySelectedItem);
                    // Only decrement our cracker counter if there's actually one on the screen.
                    if (this.isLastCrackerOnScreen()) {
                        // tries to destroy stuff (notice inside !isCracker)
                        this.throwsUntilExplosion--;
                        // If we should explode now, or are about to change players
                        // then force an explosion. 1st part complies with AC, 2nd part too, 
                        // by doing this, it explodes only one time before the next player, 
                        // wether the current player catched the cracker or not
                        if (this.throwsUntilExplosion <= 0 || this.numThrowsLeft <= 1) {
                            yield this.explodeLastCracker(); // explodes, resets, drops the stack
                        }
                    }
                }
                // We're done with this object now, destroy it
                this.currentlySelectedItem.destroy();
                this.pullComplete();
            });
        }
        /**
         * pullComplete() settles the state of things after
         * a throw - whether it was successful or not.
         *
         * It updates the number of throws left and checks
         * for whether the players should switch or if the
         * game should end.
         */
        pullComplete() {
            return __awaiter(this, void 0, void 0, function* () {
                this.numThrowsLeft--;
                this.timeSinceThrowMs = 0;
                if (this.tutorialState === TutorialState.FIRST_TUTORIAL) {
                    this.tutorialState = TutorialState.SOFT_TUTORIAL;
                }
                // Update the icons
                this.updateThrowsLeftIcons();
                // If this player is out of throws
                if (this.numThrowsLeft <= 0) {
                    this.inputState = PlayerInput.LOCKED;
                    // Check for end of game (have all players had a turn?)
                    if (this.playerIndex >= this.players.length - 1) {
                        // End game
                        this.onGameOver(this.score);
                    }
                    else {
                        // Change Player
                        this.playerIndex++;
                        yield this.startGame();
                        // Refresh throws
                        this.numThrowsLeft = this.config.throwsPerPlayer;
                        // Update the icons
                        this.updateThrowsLeftIcons();
                        // Allow a new throw
                        this.inputState = PlayerInput.CAN_THROW;
                    }
                }
                else {
                    // The player has more throws left
                    this.inputState = PlayerInput.CAN_THROW;
                }
            });
        }
        /**
         * Greys out the icons representing how many throws have been made
         */
        updateThrowsLeftIcons() {
            for (let i = 0; i < this.throwsLeftIcons.length; i++) {
                this.throwsLeftIcons.children[i].tint = i < this.numThrowsLeft ? 0xFFFFFF : 0x333333;
            }
        }
        set score(s) {
            // Don't do anything if this didn't actually 
            // change, otherwise the icon would animate
            if (s === this._score) {
                return;
            }
            this._score = s;
            // Update score text
            this.itemCounterText.text = `x${this._score}`;
            // Pulse the scale of the score icon
            this.game.add.tween(this.itemIcon.scale).to({ x: this.itemIcon.scale.x * 1.2, y: this.itemIcon.scale.y * 1.2 }, 75, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, true);
        }
        get score() {
            return this._score;
        }
        /**
         * Probably the most complex function in the class in terms of animations.
         *
         * Essentially, this creates a 'fake' item that flies towards the camera,
         * representing the item that the player just lasso'd. The animation does as follows:
         * 1. Create image to animate
         * 2. Begins scaling it up - this lasts for the entire duration of the pull
         * 2. Begins rotating the item CCW - this lasts for the entire duration of the pull
         * 3. Moves it down a bit
         * 4. Then up a bit
         * 5. Then down to the bottom of the screen
         * 6. As the item nears the bottom of the screen, it begins to fade out
         * 8. Destroys the fake item
         */
        pullItemTowardsCamera(selectedItem) {
            return __awaiter(this, void 0, void 0, function* () {
                // Remove the selected item from the stack and hide it
                this.stack.remove(selectedItem, false);
                selectedItem.visible = false;
                // drops the stack after a delay and animate it in the background
                yield this.dropStackBranched();
                // Grab the item's value for use later
                const itemValue = selectedItem.value;
                const itemToPull = new Phaser.Image(this.game, this.game.width * 0.5, selectedItem.worldPosition.y, `target-dropping-${itemValue}`);
                itemToPull.anchor.set(0.5);
                itemToPull.width = selectedItem.width;
                itemToPull.scale.y = itemToPull.scale.x;
                this.add.existing(itemToPull);
                // Stars shining out!
                this.animateSnatch(this.currentlySelectedItem);
                const scaleDest = this.game.width / itemToPull.width;
                this.game.add.tween(itemToPull.scale).to({ x: scaleDest, y: scaleDest }, this.config.pullDurationMs, Phaser.Easing.Linear.None, true, 0);
                this.game.add.tween(itemToPull).to({ angle: -200 }, this.config.pullDurationMs, Phaser.Easing.Linear.None, true, 0);
                const firstMoveDownTween = this.game.add.tween(itemToPull).to({ centerY: itemToPull.centerY + (this.game.height * 0.25) }, this.config.pullDurationMs * 0.1, Phaser.Easing.Sinusoidal.In, true, 0);
                yield PromiseUtil_26.PromiseUtil.tween(this.game, firstMoveDownTween);
                const moveUpTween = this.game.add.tween(itemToPull).to({ centerY: itemToPull.centerY - (this.game.height * 0.33) }, this.config.pullDurationMs * 0.25, Phaser.Easing.Sinusoidal.Out, true, 0);
                yield PromiseUtil_26.PromiseUtil.tween(this.game, moveUpTween);
                // N.B. Fade out tween starts towards the end of the final item drop, so is delayed
                this.game.add.tween(itemToPull).to({ alpha: 0 }, this.config.pullDurationMs * 0.25, Phaser.Easing.Linear.None, true, this.config.pullDurationMs * 0.4);
                // If it was a dud item, shake the camera
                if (itemValue === 0) {
                    PromiseUtil_26.PromiseUtil.time(this.game, this.config.pullDurationMs * 0.5).then(() => __awaiter(this, void 0, void 0, function* () {
                        this.game.camera.shake(0.01, 300);
                    }));
                }
                const secondMoveDownTween = this.game.add.tween(itemToPull).to({ centerY: this.game.height * 1.25 }, this.config.pullDurationMs * 0.65, Phaser.Easing.Sinusoidal.In, true, 0);
                yield PromiseUtil_26.PromiseUtil.tween(this.game, secondMoveDownTween);
                itemToPull.destroy();
            });
        }
        // Position, show, await then hide the pulling animation
        showPullAnimation(targetY) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play(`woody-pull-lasso-${this.game.rnd.integerInRange(1, 3)}`);
                this.pullRope.top = targetY - (this.pullRope.height * 0.2);
                this.pullRope.visible = true;
                yield PromiseUtil_26.PromiseUtil.animation(this.pullRope.animations.play('loop', 20));
                this.pullRope.visible = false;
            });
        }
        /**
         * The rope hit the bottom of the stack without being pulled!
         * That's a failure, and it needs to be reset.
         */
        onFailedToSelect() {
            return __awaiter(this, void 0, void 0, function* () {
                this.inputState = PlayerInput.LOCKED;
                yield this.resetLasso(false);
                this.pullComplete();
            });
        }
        resetLasso(immediate) {
            return __awaiter(this, void 0, void 0, function* () {
                // Stop the yPos tween
                if (this.lassoYPosTween) {
                    this.lassoYPosTween.stop();
                }
                /**
                 * And hide both parts (either fade or immediately)
                 * Immediately reset in the case of a pull
                 * Fade out briefly in the case of it hitting the bottom of the stack
                 */
                if (immediate) {
                    this.frontRope.alpha = this.backRope.alpha = 0;
                }
                else {
                    yield Promise.all([
                        PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this.frontRope).to({ alpha: 0 }, 150, Phaser.Easing.Linear.None, true, 0)),
                        PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(this.backRope).to({ alpha: 0 }, 150, Phaser.Easing.Linear.None, true, 0))
                    ]);
                }
                // Reset the lasso to off the top
                this.lassoYPos = -this.frontRope.height;
            });
        }
        /**
         * Iterates through the stack calculating new positions for items.
         * If items need to drop down, create tweens for them.
         */
        dropStack() {
            return __awaiter(this, void 0, void 0, function* () {
                const dropPromises = [];
                const dropTweens = [];
                const itemOverlap = this.game.height * 0.005;
                for (let i = this.stack.length - 1; i >= 0; i--) {
                    if (!(this.stack.children[i] instanceof Item_2.Item)) {
                        continue;
                    }
                    // Calculate where each item in the stack should be 
                    // dropping to (basically, the one below it)
                    const item = this.stack.children[i];
                    let newBottom = 0;
                    if (i !== this.stack.length - 1) {
                        const itemBelow = this.stack.getAt(i + 1);
                        newBottom = itemBelow.top;
                    }
                    const oldBottom = item.bottom;
                    item.bottom = newBottom + itemOverlap;
                    /**
                     * If there's movement to be done, make a tween and add it to our tween array.
                     * N.B. autoStart = false!
                     * This needs to be created but not started here as otherwise only the first item
                     * will drop down. The act of starting the tween for the first item
                     * throws off the subsequent item's positions.
                     */
                    if (newBottom !== oldBottom) {
                        const dropTween = this.game.add.tween(item).from({
                            bottom: oldBottom
                        }, this.config.stackDropTimeMs, Phaser.Easing.Sinusoidal.In, false, 0);
                        dropTweens.push(dropTween);
                    }
                }
                // Now that all of our tweens have been set up, we can start them all and await them
                dropTweens.forEach((tween) => {
                    dropPromises.push(PromiseUtil_26.PromiseUtil.tween(this.game, tween.start()));
                }, this);
                yield Promise.all(dropPromises);
            });
        }
        /**
         * If real mode:
         * 10% chance for cracker
         * 30% chance for 10-target crate
         * 30% chance for 2-target crate
         * 30% chance for 1-target crate
         *
         * If tutorial mode:
         * 40% chance for 10-target crate
         * 30% chance for 2-target crate
         * 30% chance for 1-target crate
         */
        getRandomItemValue() {
            let itemToGenerate;
            if (!this.difficultyConfig.isCrackerEnabled) {
                // Real mode. Crackers are enabled.
                // 25% chance for cracker, when they're allowed
                if (this.game.rnd.frac() > 0.75 && this.itemsGeneratedSinceLastCracker > this.config.minItemsBetweenCrackers) {
                    itemToGenerate = 0;
                    this.itemsGeneratedSinceLastCracker = 0;
                    // 75% chance for crate, when they're allowed
                }
                else if (this.game.rnd.frac() < 0.75 && this.itemsGeneratedSinceLastCrate > this.config.minItemsBetweenCrates) {
                    itemToGenerate = 10;
                    this.itemsGeneratedSinceLastCrate = 0;
                }
            }
            else {
                // Tutorial mode
                // 40% chance for crate, when they're allowed
                if (this.game.rnd.frac() > 0.6 && this.itemsGeneratedSinceLastCrate >= this.config.minItemsBetweenCrates) {
                    itemToGenerate = 10;
                    this.itemsGeneratedSinceLastCrate = 0;
                }
            }
            // If we didn't generate a crate or a cracker, it's a coin toss for 1 and 2.
            if (itemToGenerate === undefined) {
                itemToGenerate = this.game.rnd.pick([1, 2]);
                this.itemsGeneratedSinceLastCracker++;
                this.itemsGeneratedSinceLastCrate++;
            }
            return itemToGenerate;
        }
        /**
         * Adds a single item of a specific value.
         * Places it at the top of the stack.
         */
        addItem(i, value) {
            // The first item is never offset on X
            const stackOffsetX = i === 0 ? 0 : this.game.rnd.integerInRange(-this.game.width * 0.03, this.game.width * 0.03);
            let item;
            if (value === 0) {
                item = new Cracker_1.Cracker(this.game, stackOffsetX, value);
                item.width = this.game.width * 0.65;
            }
            else {
                item = new Item_2.Item(this.game, stackOffsetX, value);
                item.width = this.game.width * 0.5;
            }
            item.scale.y = item.scale.x;
            let bottom = 0;
            if (i !== 0) {
                bottom = this.stack.getAt(0).top;
            }
            const itemOverlap = this.game.height * 0.0025;
            item.position.set(item.stackOffsetX, bottom + itemOverlap);
            this.stack.addAt(item, 0);
            // Continue adding items if we're still short, until 
            // the stack goes about 30% off the top of the screen.
            if (bottom >= -(this.stack.y * 1.5)) {
                this.addItem(this.stack.children.length, this.getRandomItemValue());
            }
        }
        /**
         * The actual user input. Receives motion data from the Play API.
         */
        onMotionChanged(result) {
            return __awaiter(this, void 0, void 0, function* () {
                const acceleration = result.userAcceleration;
                if (this.inputState === PlayerInput.CAN_PULL &&
                    (acceleration.y > this.config.pullThreshold ||
                        acceleration.x > this.config.pullThreshold)) {
                    this.pull();
                }
                else if (this.inputState === PlayerInput.CAN_THROW &&
                    (acceleration.z < -this.config.throwThreshold ||
                        acceleration.x < -this.config.throwThreshold)) {
                    this.throw();
                }
            });
        }
        onBShowTap() {
            if (this.inputState === PlayerInput.CAN_THROW) {
                this.throw();
            }
            else if (this.inputState === PlayerInput.CAN_PULL) {
                this.pull();
            }
        }
        showMessage(top, message) {
            return __awaiter(this, void 0, void 0, function* () {
                const messageBarToUse = top ? this.tutorialTopMessageBar : this.tutorialBottomMessageBar;
                this.game.tweens.removeFrom(messageBarToUse);
                if (message) {
                    messageBarToUse.text = message;
                }
                if (top) {
                    yield PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(messageBarToUse)
                        .to({ top: this.game.height * 0.015 }, 250, Phaser.Easing.Sinusoidal.InOut, true));
                }
                else {
                    yield PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(messageBarToUse)
                        .to({ bottom: this.game.height * 0.985 }, 250, Phaser.Easing.Sinusoidal.InOut, true));
                }
            });
        }
        hideMessage(top) {
            return __awaiter(this, void 0, void 0, function* () {
                const messageBarToUse = top ? this.tutorialTopMessageBar : this.tutorialBottomMessageBar;
                this.game.tweens.removeFrom(messageBarToUse);
                if (top) {
                    yield PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(messageBarToUse)
                        .to({ bottom: 0 }, 250, Phaser.Easing.Sinusoidal.InOut, true));
                }
                else {
                    yield PromiseUtil_26.PromiseUtil.tween(this.game, this.game.add.tween(messageBarToUse)
                        .to({ top: this.game.height }, 250, Phaser.Easing.Sinusoidal.InOut, true));
                }
            });
        }
        showPrompt(animationName) {
            return __awaiter(this, void 0, void 0, function* () {
                if (!MotionUtil_5.MotionUtil.isAccelerometerAvailable || this.promptShowing) {
                    return;
                }
                this.game.tweens.removeFrom(this.lassoPrompt);
                this.lassoPrompt.alpha = 0;
                this.promptShowing = true;
                if (animationName) {
                    this.lassoPrompt.animations.play(animationName);
                    this.tutorialArrow.frame = (animationName === 'pull' ? 0 : 1);
                }
                this.game.add.tween(this.tutorialArrow).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true);
                const fadeTween = this.game.add.tween(this.lassoPrompt).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_26.PromiseUtil.tween(this.game, fadeTween);
            });
        }
        hidePrompt() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!MotionUtil_5.MotionUtil.isAccelerometerAvailable || !this.promptShowing) {
                    return;
                }
                this.lassoPrompt.animations.stop();
                this.game.tweens.removeFrom(this.lassoPrompt);
                this.lassoPrompt.alpha = 1;
                this.promptShowing = false;
                this.game.add.tween(this.tutorialArrow).to({ alpha: 0 }, 250, Phaser.Easing.Linear.None, true);
                const fadeTween = this.game.add.tween(this.lassoPrompt).to({ alpha: 0 }, 250, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_26.PromiseUtil.tween(this.game, fadeTween);
            });
        }
        update() {
            super.update();
            if (this.inputState !== PlayerInput.LOCKED) {
                this.roundTimeMs += this.game.time.elapsedMS;
                if (this.roundTimeMs > this.MAX_TIMEOUT_STUCK_MS) {
                    this.onGameTimeout();
                }
            }
            // Move the rope to where it should be (determined by the lassoYPosTween)
            this.frontRope.y = this.lassoYPos;
            this.backRope.y = this.lassoYPos;
            this.timeSinceThrowMs += this.game.time.elapsedMS;
            // If they haven't thrown in a while, prompt them
            if (!this.promptShowing &&
                this.inputState === PlayerInput.CAN_THROW &&
                this.timeSinceThrowMs > this.config.timeBetweenThrowsToShowPromptMs &&
                MotionUtil_5.MotionUtil.isAccelerometerAvailable) {
                this.showPrompt('throw');
                this.showMessage(true, LocalizationManager_17.LocalizationManager.Instance.text("lasso.tutorial_throw"));
            }
            if (this.inputState === PlayerInput.CAN_PULL) {
                this.updateSelectedItem();
            }
            else {
                if (this.currentlySelectedItem) {
                    this.currentlySelectedItem.highlighted = false;
                }
            }
        }
        onGameTimeout() {
            return __awaiter(this, void 0, void 0, function* () {
                this.inputState = PlayerInput.LOCKED;
                this.game.input.onDown.remove(this.onBShowTap, this);
                PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.motionCallback);
                this.game.add.tween(this.lassoPrompt).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.tutorialArrow).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
                this.hideMessage(true);
                const timeoutDialog = new BasicDialog_2.BasicDialog(this.game, Character_4.Character.WOODY, true);
                timeoutDialog.centerY = this.game.height * 0.5;
                this.add.existing(timeoutDialog);
                yield timeoutDialog.show();
                yield timeoutDialog.writeMessage(LocalizationManager_17.LocalizationManager.Instance.text("time_up.main"));
                yield Promise.race([
                    timeoutDialog.waitForTap(),
                    PromiseUtil_26.PromiseUtil.signal(this.game.input.onTap)
                ]);
                yield timeoutDialog.hide();
                timeoutDialog.destroy();
                this.onGameOver(this.score);
            });
        }
        /**
         * Adjusts the currently selected item (and highlights it)
         * according to the lassoYPos (which changes with a tween)
         */
        updateSelectedItem() {
            let itemToHighlight;
            for (let i = 0; i < this.stack.children.length; i++) {
                const item = this.stack.children[i];
                // Check the world bounds of the item - if they contain the lassoYPos, we've found our item!
                if (item.worldPosition.y - item.height <= this.lassoYPos && item.worldPosition.y >= this.lassoYPos) {
                    itemToHighlight = item;
                    break;
                }
            }
            // If we found anything, set its highlight (and disable the previous one, if there was one)
            if (itemToHighlight) {
                if (this.currentlySelectedItem && this.currentlySelectedItem !== itemToHighlight) {
                    this.currentlySelectedItem.highlighted = false;
                }
                this.currentlySelectedItem = itemToHighlight;
                this.currentlySelectedItem.highlighted = true;
            }
        }
        onGameOver(score) {
            return __awaiter(this, void 0, void 0, function* () {
                PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.motionCallback);
                const resultsString = LocalizationManager_17.LocalizationManager.Instance.textWithReplacements("lasso.results_message", { items_grabbed: this.score });
                yield this.dialog.show();
                yield this.dialog.writeMessage(resultsString);
                // allows for tapping the whole screen
                yield Promise.race([
                    this.dialog.waitForTap(),
                    PromiseUtil_26.PromiseUtil.signal(this.game.input.onTap)
                ]);
                yield this.dialog.hide();
                this.transitionToNextGame(score);
            });
        }
        transitionToNextGame(score) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this.trainLoop).to({ volume: 0 }, 500, Phaser.Easing.Linear.None, true);
                yield this.showOverlay();
                this.trackMicroEnd(score);
                BalanceState_1.BalanceState.startWoodyBalance(this.game, this.players.reverse(), score);
            });
        }
        shutdown() {
            this.game.input.onDown.remove(this.onBShowTap, this);
            PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.motionCallback);
            this.motionCallback = null;
            this.dialog = null;
            this.stack = null;
            this.itemCounterText = null;
            this.itemIcon = null;
            this.currentlySelectedItem = null;
            this.snatchGroup = null;
            this.tutorialState = null;
            this.lassoPrompt = null;
            this.tutorialArrow = null;
            this.tutorialTopMessageBar = null;
            this.tutorialBottomMessageBar = null;
            this.instructionImage = null;
            this.curtain = null;
            this.instructionNextButton = null;
            this.headerBar = null;
            this.passAndPlayShowable = null;
            this.inputState = null;
            this.lassoYPosTween = null;
            this.frontRope = null;
            this.backRope = null;
            this.pullRope = null;
            this.throwsLeftIcons = null;
            this.players = null;
            this.trainLoop.destroy();
            this.trainLoop = null;
            this.game.cache.removeImage('lasso-tiling-front');
            this.game.cache.removeImage('lasso-tiling-back');
            this.game.cache.removeImage('target-0');
            this.game.cache.removeImage('target-1');
            this.game.cache.removeImage('target-2');
            this.game.cache.removeImage('target-10');
            this.game.cache.removeImage('clouds');
            this.game.cache.removeImage('bg-detail');
            this.game.cache.removeImage('tiling-bg');
            this.game.cache.removeImage('lasso-fuse-spark');
            this.game.cache.removeImage('target-dropping-1');
            this.game.cache.removeImage('target-dropping-2');
            this.game.cache.removeImage('target-dropping-10');
            this.game.cache.removeImage('carriage-base');
            this.game.cache.removeImage('train-front-pete');
            this.game.cache.removeImage('target-whole-icon');
            this.game.cache.removeImage('hands-base');
            this.game.cache.removeImage('lasso-icon');
            this.game.cache.removeImage('screen-flash');
            // Snatch FX
            this.game.cache.removeImage('snatch-radialfx');
            this.game.cache.removeImage('snatch-star-green');
            this.game.cache.removeImage('snatch-star-pink');
            this.game.cache.removeImage('snatch-star-blue');
            this.game.cache.removeImage('track-sheet');
            this.game.cache.removeImage('lasso-sheet');
            this.game.cache.removeImage('lasso-explosion');
            // Instructions
            this.game.cache.removeImage('character-dialog-woody');
            this.game.cache.removeImage('instruction-graphic-woody-lasso');
            this.game.cache.removeImage('lasso-prompt');
            this.game.cache.removeImage('tutorial-arrow');
            this.game.cache.removeSound('woody-cracker-explode');
            this.game.cache.removeSound('woody-pull-crate');
            this.game.cache.removeSound('woody-pull-target');
            this.game.cache.removeSound('woody-pull-lasso-1');
            this.game.cache.removeSound('woody-pull-lasso-2');
            this.game.cache.removeSound('woody-pull-lasso-3');
            this.game.cache.removeSound('woody-throw-lasso-1');
            this.game.cache.removeSound('woody-throw-lasso-2');
            this.game.cache.removeSound('woody-throw-lasso-3');
            super.shutdown();
        }
    }
    LassoState.TRACKING_MICRO_NAME = "Lasso";
    exports.LassoState = LassoState;
});
define("minigame/dress-up/DressUpOptions", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var OutfitPart;
    (function (OutfitPart) {
        OutfitPart[OutfitPart["ARMS"] = 0] = "ARMS";
        OutfitPart[OutfitPart["FACE"] = 1] = "FACE";
        OutfitPart[OutfitPart["HAT"] = 2] = "HAT";
        OutfitPart[OutfitPart["BODY"] = 3] = "BODY";
        OutfitPart[OutfitPart["WAIST"] = 4] = "WAIST";
        OutfitPart[OutfitPart["FEET"] = 5] = "FEET";
        OutfitPart[OutfitPart["PROP"] = 6] = "PROP";
    })(OutfitPart = exports.OutfitPart || (exports.OutfitPart = {}));
    class DressUpOptions {
    }
    DressUpOptions.ARM_OPTIONS = [
        "arms_claw",
        "arms_dishwashing",
        "arms_mitts",
        "arms_waterwings"
    ];
    DressUpOptions.FACE_OPTIONS = [
        "face_blueshades",
        "face_beak",
        "face_scuba",
        "face_tsmm_glasses"
    ];
    DressUpOptions.HAT_OPTIONS = [
        "hat_space",
        "hat_army",
        "hat_hardhat",
        "hat_straw"
    ];
    DressUpOptions.BODY_OPTIONS = [
        "body_hivis",
        "body_armor",
        "body_lifejacket",
        "body_lights"
    ];
    DressUpOptions.WAIST_OPTIONS = [
        "waist_apron",
        "waist_grass",
        "waist_toolbelt",
        "waist_duckie",
    ];
    DressUpOptions.FEET_OPTIONS = [
        "feet_flippers",
        "feet_kneepads",
        "feet_gumboots",
        "feet_sandals"
    ];
    DressUpOptions.PROP_OPTIONS = [
        "prop_jetpack",
        "prop_campingbag",
        "prop_fairy",
        "prop_feathers"
    ];
    DressUpOptions.OPTIONS_MAP = [
        DressUpOptions.ARM_OPTIONS,
        DressUpOptions.FACE_OPTIONS,
        DressUpOptions.HAT_OPTIONS,
        DressUpOptions.BODY_OPTIONS,
        DressUpOptions.WAIST_OPTIONS,
        DressUpOptions.FEET_OPTIONS,
        DressUpOptions.PROP_OPTIONS
    ];
    exports.DressUpOptions = DressUpOptions;
});
define("minigame/dress-up/DressUpController", ["require", "exports", "minigame/dress-up/DressUpOptions", "logic/GameController"], function (require, exports, DressUpOptions_1, GameController_19) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DressUpController {
        constructor() {
            // Generate random outfit
            this.currentTargetOutfit = this.generateRandomOutfit();
            // Set current controller
            this.currentPlayer = GameController_19.GameController.Instance.playerManager.currentPlayer;
            // Change the player so a new player can play the next game
            GameController_19.GameController.Instance.playerManager.changePlayer();
        }
        generateRandomOutfit() {
            const randomOutfit = [];
            randomOutfit[DressUpOptions_1.OutfitPart.ARMS] = Math.floor(Math.random() * DressUpOptions_1.DressUpOptions.ARM_OPTIONS.length);
            randomOutfit[DressUpOptions_1.OutfitPart.FACE] = Math.floor(Math.random() * DressUpOptions_1.DressUpOptions.FACE_OPTIONS.length);
            randomOutfit[DressUpOptions_1.OutfitPart.HAT] = Math.floor(Math.random() * DressUpOptions_1.DressUpOptions.HAT_OPTIONS.length);
            randomOutfit[DressUpOptions_1.OutfitPart.BODY] = Math.floor(Math.random() * DressUpOptions_1.DressUpOptions.BODY_OPTIONS.length);
            randomOutfit[DressUpOptions_1.OutfitPart.WAIST] = Math.floor(Math.random() * DressUpOptions_1.DressUpOptions.WAIST_OPTIONS.length);
            randomOutfit[DressUpOptions_1.OutfitPart.FEET] = Math.floor(Math.random() * DressUpOptions_1.DressUpOptions.FEET_OPTIONS.length);
            randomOutfit[DressUpOptions_1.OutfitPart.PROP] = Math.floor(Math.random() * DressUpOptions_1.DressUpOptions.PROP_OPTIONS.length);
            return randomOutfit;
        }
        getCurrentTargetOutfit() {
            return this.currentTargetOutfit;
        }
    }
    exports.DressUpController = DressUpController;
});
define("minigame/dress-up/Rex", ["require", "exports", "minigame/dress-up/DressUpOptions"], function (require, exports, DressUpOptions_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Rex extends Phaser.Group {
        constructor(game, isDressed = true, outfit) {
            super(game);
            this.currentOutfit = Array(7).fill(-1);
            this.parts = [];
            if (outfit) {
                this.currentOutfit = outfit;
            }
            this.parts = [];
            this.parts[DressUpOptions_2.OutfitPart.PROP] = new Phaser.Image(this.game, 0, 0, DressUpOptions_2.DressUpOptions.PROP_OPTIONS[outfit ? outfit[DressUpOptions_2.OutfitPart.PROP] : 0]);
            this.parts[DressUpOptions_2.OutfitPart.PROP].anchor.set(0.5, 1);
            this.add(this.parts[DressUpOptions_2.OutfitPart.PROP]);
            // Body needs to be added between prop and feet
            this.add(this.body = new Phaser.Image(this.game, 0, 0, 'rex'));
            this.body.anchor.set(0.5, 1);
            // Blinking
            const blink = new Phaser.Sprite(this.game, 0, 0, 'rex-blink');
            blink.anchor.set(0.5, 1);
            this.add(blink);
            blink.animations.add('blink', [1, 2, 1].concat(Array(50).fill(0)), 20, true);
            blink.play('blink');
            this.parts[DressUpOptions_2.OutfitPart.FEET] = new Phaser.Image(this.game, 0, 0, DressUpOptions_2.DressUpOptions.FEET_OPTIONS[outfit ? outfit[DressUpOptions_2.OutfitPart.FEET] : 0]);
            this.parts[DressUpOptions_2.OutfitPart.FEET].anchor.set(0.5, 1);
            this.add(this.parts[DressUpOptions_2.OutfitPart.FEET]);
            this.parts[DressUpOptions_2.OutfitPart.WAIST] = new Phaser.Image(this.game, 0, 0, DressUpOptions_2.DressUpOptions.WAIST_OPTIONS[outfit ? outfit[DressUpOptions_2.OutfitPart.WAIST] : 0]);
            this.parts[DressUpOptions_2.OutfitPart.WAIST].anchor.set(0.5, 1);
            this.add(this.parts[DressUpOptions_2.OutfitPart.WAIST]);
            this.parts[DressUpOptions_2.OutfitPart.BODY] = new Phaser.Image(this.game, 0, 0, DressUpOptions_2.DressUpOptions.BODY_OPTIONS[outfit ? outfit[DressUpOptions_2.OutfitPart.BODY] : 0]);
            this.parts[DressUpOptions_2.OutfitPart.BODY].anchor.set(0.5, 1);
            this.add(this.parts[DressUpOptions_2.OutfitPart.BODY]);
            this.parts[DressUpOptions_2.OutfitPart.HAT] = new Phaser.Image(this.game, 0, 0, DressUpOptions_2.DressUpOptions.HAT_OPTIONS[outfit ? outfit[DressUpOptions_2.OutfitPart.HAT] : 0]);
            this.parts[DressUpOptions_2.OutfitPart.HAT].anchor.set(0.5, 1);
            this.add(this.parts[DressUpOptions_2.OutfitPart.HAT]);
            this.parts[DressUpOptions_2.OutfitPart.FACE] = new Phaser.Image(this.game, 0, 0, DressUpOptions_2.DressUpOptions.FACE_OPTIONS[outfit ? outfit[DressUpOptions_2.OutfitPart.FACE] : 0]);
            this.parts[DressUpOptions_2.OutfitPart.FACE].anchor.set(0.5, 1);
            this.add(this.parts[DressUpOptions_2.OutfitPart.FACE]);
            this.parts[DressUpOptions_2.OutfitPart.ARMS] = new Phaser.Image(this.game, 0, 0, DressUpOptions_2.DressUpOptions.ARM_OPTIONS[outfit ? outfit[DressUpOptions_2.OutfitPart.ARMS] : 0]);
            this.parts[DressUpOptions_2.OutfitPart.ARMS].anchor.set(0.5, 1);
            this.add(this.parts[DressUpOptions_2.OutfitPart.ARMS]);
            if (!isDressed) {
                this.parts.forEach((value) => {
                    value.visible = false;
                });
                this.parts[DressUpOptions_2.OutfitPart.ARMS].loadTexture('arms_base');
                this.parts[DressUpOptions_2.OutfitPart.ARMS].visible = true;
            }
        }
        startAnimation() {
            this.game.add.tween(this.scale).to({ y: this.scale.y * 0.99 }, 700, Phaser.Easing.Linear.None, true, 0, Infinity, true);
        }
        incrementItem(part) {
            let imageTag;
            // Remove any existing tweens, reset scale 
            // so the buttons can handle rapid tapping
            this.game.tweens.removeFrom(this.parts[part]);
            this.parts[part].alpha = 1;
            this.parts[part].scale.x = 1;
            this.parts[part].scale.y = 1;
            const options = DressUpOptions_2.DressUpOptions.OPTIONS_MAP[part];
            const nextItemIndex = (this.currentOutfit[part] + 1) % options.length;
            imageTag = options[nextItemIndex];
            this.parts[part].loadTexture(imageTag);
            this.game.add.tween(this.parts[part]).from({ alpha: 0.2 }, 80, Phaser.Easing.Exponential.In, true)
                .onStart.add(() => { this.parts[part].visible = true; });
            this.game.add.tween(this.parts[part].scale).from({ x: 0.4, y: 0.4 }, 80, Phaser.Easing.Linear.None, true);
            this.currentOutfit[part] = nextItemIndex;
        }
    }
    exports.Rex = Rex;
});
define("minigame/dress-up/UIElements", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class AssemblyButton extends Phaser.Group {
        constructor(game, imageTag, part, x, y, arrowRotation) {
            super(game);
            this.assemblySignal = new Phaser.Signal();
            this.toggleAnswerSignal = new Phaser.Signal();
            this.part = part;
            this.buttonImage = new Phaser.Image(game, 0, 0, imageTag);
            this.buttonImage.anchor.set(0.5);
            this.buttonImage.inputEnabled = true;
            this.arrow = new Phaser.Image(game, 0, 0, 'assembly-arrow');
            this.arrow.anchor.set(0, 0.5);
            this.arrow.left = this.buttonImage.centerX;
            this.arrow.centerY = this.buttonImage.centerY;
            this.arrow.angle = arrowRotation;
            this.add(this.arrow);
            this.add(this.buttonImage);
            this.buttonImage.events.onInputUp.add(this.onClick, this);
            this.centerX = x + this.width * 0.5;
            this.centerY = y + this.height * 0.5;
            this.origScale = this.scale.x;
            this.game.add.tween(this.scale).to({ x: this.origScale * 0.95, y: this.origScale * 0.95 }, 400, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
        }
        onClick() {
            this.game.add.tween(this.scale).to({ x: 0.6, y: 0.6 }, 150, Phaser.Easing.Sinusoidal.InOut, true, 0, 0, true);
            if (this.resultImage) {
                // We are in the result screen, show the answer UI
                this.toggleAnswerSignal.dispatch();
            }
            else {
                // Send signal to change clothing part
                this.assemblySignal.dispatch(this.part);
            }
        }
        showResult(isCorrect) {
            this.buttonImage.loadTexture('button-results');
            this.resultImage = new Phaser.Image(this.game, 0, 0, isCorrect ? 'correct-tick' : 'incorrect-cross');
            this.resultImage.anchor.set(0.5);
            this.resultImage.centerX = this.buttonImage.centerX;
            this.resultImage.centerY = this.buttonImage.centerY;
            this.resultImage.inputEnabled = false;
            this.buttonImage.inputEnabled = false;
            // Cancel the previous scaling tween
            this.game.tweens.removeFrom(this);
            this.scale.set(this.origScale);
            // And make a new one
            const scaleDest = isCorrect ? this.origScale * 0.95 : this.origScale * 1.2;
            this.game.add.tween(this.scale).to({ x: scaleDest, y: scaleDest }, 400, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
            this.add(this.resultImage);
        }
        enableTapForSolution() {
            this.buttonImage.inputEnabled = true;
        }
        destroy() {
            if (this.buttonImage) {
                this.buttonImage.events.onInputUp.removeAll();
            }
            if (this.resultImage) {
                this.resultImage.events.onInputUp.removeAll();
            }
            this.assemblySignal.removeAll();
            this.toggleAnswerSignal.removeAll();
            super.destroy();
        }
    }
    exports.AssemblyButton = AssemblyButton;
});
define("component/TimerBar", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TimerBar extends Phaser.Group {
        constructor(game, timeAllocatedMS) {
            super(game);
            this._isRunning = false;
            this.timeAllocatedMS = timeAllocatedMS;
            this.timeRemainingMS = timeAllocatedMS;
            const clockIcon = new Phaser.Sprite(game, 0, 0, 'timerbar-icon');
            clockIcon.animations.add("tick", [0, 1], 2, true);
            clockIcon.play("tick");
            clockIcon.height = game.height * 0.04;
            clockIcon.scale.y = clockIcon.scale.x;
            clockIcon.left = game.width * 0.005;
            const background = new Phaser.Image(game, 0, 0, 'timerbar-frame-tile');
            this.timeBarForeground = new Phaser.Image(game, 0, 0, 'timerbar-timer');
            background.x = this.timeBarForeground.x = clockIcon.right - clockIcon.width * 0.1;
            background.width = this.timeBarForeground.width = game.width - this.timeBarForeground.x;
            background.height = this.timeBarForeground.height = game.height * 0.037;
            clockIcon.centerY = background.centerY;
            this.timeUpSignal = new Phaser.Signal();
            this.add(background);
            this.add(this.timeBarForeground);
            this.add(clockIcon);
        }
        get isRunning() {
            return this._isRunning;
        }
        start() {
            // Starts the timer
            this._isRunning = true;
        }
        /**
         * You can pause the timer by calling .stop()
         * and unpause it again by calling .start()
         */
        stop() {
            // Stops the timer
            this._isRunning = false;
        }
        update() {
            super.update();
            if (this._isRunning) {
                this.timeRemainingMS -= this.game.time.elapsed; // logic
                this.timeBarForeground.width = (this.game.width - this.timeBarForeground.x) * (this.timeRemainingMS / this.timeAllocatedMS); // view
                if (this.timeRemainingMS <= 0) {
                    this.stop();
                    this.timeUpSignal.dispatch();
                }
            }
        }
    }
    exports.TimerBar = TimerBar;
});
define("minigame/dress-up/DressUpState", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "StateNames", "Character", "logic/GameController", "component/dialog/BasicDialog", "minigame/dress-up/DressUpController", "minigame/dress-up/DressUpOptions", "minigame/dress-up/Rex", "minigame/dress-up/UIElements", "component/TimerBar", "component/Curtain", "component/MessageBar", "component/button/TextArrowButton", "component/HeaderBar", "model/ToyStoryConfig", "state/GameBaseState"], function (require, exports, PromiseUtil_27, ScaleUtil_12, LocalizationManager_18, StateNames_16, Character_5, GameController_20, BasicDialog_3, DressUpController_1, DressUpOptions_3, Rex_1, UIElements_1, TimerBar_1, Curtain_5, MessageBar_8, TextArrowButton_6, HeaderBar_4, ToyStoryConfig_13, GameBaseState_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DressUpState extends GameBaseState_6.GameBaseState {
        init(...params) {
            this.stateName = StateNames_16.StateNames.GAME_DRESS_UP;
            super.init(params);
            this.hasChangedClothing = false;
            this.score = 0;
        }
        preload() {
            super.preload();
            this.game.load.image('cloud-backplate', 'assets/minigame/dress-up/Cloud_backplate.png');
            this.game.load.image('cloud-particle', 'assets/minigame/dress-up/Cloud_particle.png');
            this.game.load.image('character-dialog-backplate', 'assets/dialog/character_dialog_backplate.png');
            this.game.load.image('instruction-graphic-portrait', 'assets/dialog/instruction_graphic_portrait.png');
            this.game.load.image('instruction-dialog-backplate', 'assets/dialog/instruction_dialog_backplate.png');
            this.game.load.spritesheet('timerbar-icon', 'assets/common/timerbar-icon.png', 104, 114);
            this.game.load.image('timerbar-timer', 'assets/common/timerbar-timer.png');
            this.game.load.image('timerbar-frame-tile', 'assets/common/timerbar-frame-tile.png');
            this.game.load.image('rex', 'assets/minigame/dress-up/Rex_base.png');
            this.game.load.atlas('rex-blink', 'assets/minigame/dress-up/blink.png', 'assets/minigame/dress-up/blink.json');
            this.game.load.image('shadow', 'assets/minigame/dress-up/Shadow_Rex.png');
            this.game.load.image('room-bg', 'assets/cutscene/background.jpg');
            this.game.load.image('button-arrow', 'assets/common/next-button.png');
            this.game.load.image('assembly-arrow', 'assets/minigame/dress-up/ui/arrow.png');
            this.game.load.image('button-reveal', 'assets/minigame/dress-up/ui/Reveal_btn.png');
            this.game.load.image('correct-tick', 'assets/minigame/dress-up/ui/Results_correct.png');
            this.game.load.image('incorrect-cross', 'assets/minigame/dress-up/ui/Results_incorrect.png');
            this.game.load.image('button-results', 'assets/minigame/dress-up/ui/ToggleItemBtn_Results.png');
            this.game.load.image('button-arms', 'assets/minigame/dress-up/ui/ToggleItemBtn_Arms.png');
            this.game.load.image('button-body', 'assets/minigame/dress-up/ui/ToggleItemBtn_Body.png');
            this.game.load.image('button-face', 'assets/minigame/dress-up/ui/ToggleItemBtn_Face.png');
            this.game.load.image('button-feet', 'assets/minigame/dress-up/ui/ToggleItemBtn_Feet.png');
            this.game.load.image('button-hat', 'assets/minigame/dress-up/ui/ToggleItemBtn_Hat.png');
            this.game.load.image('button-prop', 'assets/minigame/dress-up/ui/ToggleItemBtn_Prop.png');
            this.game.load.image('button-waist', 'assets/minigame/dress-up/ui/ToggleItemBtn_Waist.png');
            this.game.load.image('arms_base', 'assets/minigame/dress-up/arms/arms_base.png');
            // How to play dialog
            this.game.load.image('cta-header', 'assets/dialog/cta_header.png');
            this.game.load.image('infographic', 'assets/dialog/instruction_graphic_rex.png');
            this.game.load.image('tiling-bar', 'assets/common/CTAbar_tilingbg.png');
            this.game.load.image('cta-curtain', 'assets/common/cta-curtain.png');
            this.game.load.image('next-button', 'assets/common/next-button.png');
            // Tutorial
            this.game.load.image('tutorial-diagram', 'assets/minigame/dress-up/ui/tutorial_diagram.png');
            for (let option of DressUpOptions_3.DressUpOptions.ARM_OPTIONS) {
                this.game.load.image(option, 'assets/minigame/dress-up/arms/' + option + '.png');
            }
            for (let option of DressUpOptions_3.DressUpOptions.BODY_OPTIONS) {
                this.game.load.image(option, 'assets/minigame/dress-up/body/' + option + '.png');
            }
            for (let option of DressUpOptions_3.DressUpOptions.FACE_OPTIONS) {
                this.game.load.image(option, 'assets/minigame/dress-up/face/' + option + '.png');
            }
            for (let option of DressUpOptions_3.DressUpOptions.FEET_OPTIONS) {
                this.game.load.image(option, 'assets/minigame/dress-up/feet/' + option + '.png');
            }
            for (let option of DressUpOptions_3.DressUpOptions.HAT_OPTIONS) {
                this.game.load.image(option, 'assets/minigame/dress-up/hat/' + option + '.png');
            }
            for (let option of DressUpOptions_3.DressUpOptions.PROP_OPTIONS) {
                this.game.load.image(option, 'assets/minigame/dress-up/prop/' + option + '.png');
            }
            for (let option of DressUpOptions_3.DressUpOptions.WAIST_OPTIONS) {
                this.game.load.image(option, 'assets/minigame/dress-up/waist/' + option + '.png');
            }
            // load the sounds
            this.game.load.audio('rex-complete', ['assets/sound/rex/complete.m4a', 'assets/sound/rex/complete.ogg']);
            this.game.load.audio('rex-countdown-loop', ['assets/sound/rex/countdown_loop.m4a', 'assets/sound/rex/countdown_loop.ogg']);
            this.game.load.audio('rex-toggle', ['assets/sound/rex/toggle.m4a', 'assets/sound/rex/toggle.ogg']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.trackGameStartTime();
                this.controller = new DressUpController_1.DressUpController();
                const bg = new Phaser.Image(this.game, 0, 0, 'room-bg');
                bg.width = this.game.width;
                bg.height = this.game.height;
                this.add.existing(bg);
                const shadow = new Phaser.Image(this.game, 0, 0, 'shadow');
                shadow.width = this.game.width;
                shadow.height = this.game.height;
                this.add.existing(shadow);
                this.assemblyRex = new Rex_1.Rex(this.game, false);
                this.assemblyRex.height = this.game.height * 0.75;
                this.assemblyRex.scale.x = this.assemblyRex.scale.y;
                this.assemblyRex.y = this.game.height;
                this.assemblyRex.centerX = this.game.width * 0.5;
                this.add.existing(this.assemblyRex);
                this.assemblyRex.startAnimation();
                this.animCloud = new Phaser.Image(this.game, 0, 0, 'cloud-backplate');
                ScaleUtil_12.scaleToBounds(this.animCloud, this.game.width * 0.25, this.game.height * 0.25);
                this.animCloud.anchor.set(0.5);
                this.animCloud.centerX = this.game.width * 0.5;
                // This approximates where the cloud needs to start at in order to hit the 
                // bottom of the screen once at full height
                this.animCloud.centerY = this.game.height * 0.57;
                this.animCloud.alpha = 0;
                this.add.existing(this.animCloud);
                this.createAssemblyUI();
                this.assemblyUI.visible = false;
                // Display Rex in target outfit
                this.createMemoriseUI();
                this.timerLoopSound = this.game.sound.add('rex-countdown-loop', 1, true);
                this.curtain = new Curtain_5.Curtain(this.game);
                this.headerBar = new HeaderBar_4.HeaderBar(this.game, LocalizationManager_18.LocalizationManager.Instance.text("common.how_to_play"));
                this.headerBar.centerX = this.game.width * 0.5;
                this.headerBar.top = 0;
                this.readyButton = new TextArrowButton_6.TextArrowButton(this.game, LocalizationManager_18.LocalizationManager.Instance.text("common.ready").toUpperCase());
                this.readyButton.centerX = this.game.width * 0.5;
                this.readyButton.centerY = this.game.height * 0.9;
                this.readyButton.alpha = 0;
                this.readyButton.startPulse();
                this.topMessageBar = new MessageBar_8.MessageBar(this.game, LocalizationManager_18.LocalizationManager.Instance.textWithReplacements("dress_up.instructions_controller", {
                    player_name: this.controller.currentPlayer.name
                }).toUpperCase(), 35);
                this.topMessageBar.centerX = this.game.width * 0.5;
                this.topMessageBar.centerY = this.game.height * 0.25;
                this.bottomMessageBar = new MessageBar_8.MessageBar(this.game, LocalizationManager_18.LocalizationManager.Instance.text("dress_up.instructions_everyone_else"), 35);
                this.bottomMessageBar.centerX = this.game.width * 0.5;
                this.bottomMessageBar.centerY = this.game.height * 0.75;
                this.infographic = new Phaser.Image(this.game, 0, 0, 'infographic');
                ScaleUtil_12.scaleToBounds(this.infographic, this.game.width * 0.8, this.game.height * 0.5);
                this.infographic.centerX = this.game.width * 0.5;
                this.infographic.centerY = this.game.height * 0.5;
                this.add.existing(this.curtain);
                this.add.existing(this.infographic);
                this.add.existing(this.topMessageBar);
                this.add.existing(this.bottomMessageBar);
                this.add.existing(this.headerBar);
                this.add.existing(this.readyButton);
                yield this.dismissOverlay();
                if (ToyStoryConfig_13.ToyStoryConfig.Instance.skipGames) {
                    this.score = 6;
                    this.onGameEnd();
                    this.transitionTo();
                }
                else {
                    this.play();
                }
            });
        }
        play() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.showFirstTutorial();
                yield this.displayMemoriseUI();
                yield PromiseUtil_27.PromiseUtil.signal(this.timerBar.timeUpSignal);
                yield this.hideMemoriseUI();
                yield this.showSecondTutorial();
                yield this.showAssemblyUI();
                yield PromiseUtil_27.PromiseUtil.signal(this.finishButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                yield this.finishAssembly();
                yield this.hideAssemblyUI();
                yield this.displayResults();
                yield PromiseUtil_27.PromiseUtil.signal(this.finishButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                this.onGameEnd();
                yield this.curtain.show();
                this.transitionTo();
            });
        }
        onGameEnd() {
            GameController_20.GameController.Instance.postScore(this.stateName, this.score);
        }
        showFirstTutorial() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("showFirstTutorial()");
                this.trackGameState(null, DressUpState.TRACKING_MICRO_NAME_REMEMBER);
                this.trackGameInstructions(DressUpState.TRACKING_MICRO_NAME_REMEMBER);
                yield PromiseUtil_27.PromiseUtil.time(this.game, 2000);
                yield PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.readyButton)
                    .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true));
                yield PromiseUtil_27.PromiseUtil.signal(this.readyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                this.topMessageBar.text = '';
                yield Promise.all([
                    this.curtain.hide(),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.headerBar)
                        .to({ bottom: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                        .to({ top: this.timerBar.bottom }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                        .to({ top: this.game.height }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.readyButton)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.infographic)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true))
                ]);
                this.readyButton.visible = false;
                this.topMessageBar.text = LocalizationManager_18.LocalizationManager.Instance.text("dress_up.remember_outfit");
            });
        }
        showSecondTutorial() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("showSecondTutorial()");
                this.trackGameState(null, DressUpState.TRACKING_MICRO_NAME_DRESS);
                this.trackGameInstructions(DressUpState.TRACKING_MICRO_NAME_DRESS);
                // Set up the text on the message bars
                this.topMessageBar.fontSize = 65;
                this.topMessageBar.text = LocalizationManager_18.LocalizationManager.Instance.textWithReplacements("dress_up.instructions_1", {
                    player_name: this.controller.currentPlayer.name
                }).toUpperCase();
                this.bottomMessageBar.text = LocalizationManager_18.LocalizationManager.Instance.text("dress_up.instructions_2");
                const instructionImage = new Phaser.Image(this.game, 0, 0, "tutorial-diagram");
                ScaleUtil_12.scaleToBounds(instructionImage, this.game.width * 0.95, this.game.height * 0.5);
                instructionImage.centerX = this.game.width * 0.5;
                instructionImage.centerY = this.game.height * 0.5;
                instructionImage.alpha = 0;
                this.add.existing(instructionImage);
                // Move everything to the right place
                yield Promise.all([
                    this.curtain.show(),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.headerBar)
                        .to({ top: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                        .to({ centerY: this.game.height * 0.27 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                        .to({ centerY: this.game.height * 0.72 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(instructionImage)
                        .to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true))
                ]);
                // Wait a bit, then show and listen for the ready button
                yield PromiseUtil_27.PromiseUtil.time(this.game, 2000);
                this.readyButton.centerY = this.game.height * 0.9;
                this.readyButton.visible = true;
                yield PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.readyButton)
                    .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true));
                yield PromiseUtil_27.PromiseUtil.signal(this.readyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                // Get rid of everything
                yield Promise.all([
                    this.curtain.hide(),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.headerBar)
                        .to({ bottom: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                        .to({ bottom: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                        .to({ top: this.game.height * 0.015 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.readyButton)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(instructionImage)
                        .to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true))
                ]);
                this.readyButton.visible = false;
            });
        }
        createMemoriseUI() {
            console.log("createMemoriseUI()");
            this.memoriseUI = new Phaser.Group(this.game);
            this.overlay = this.add.graphics(0, 0);
            this.overlay.beginFill(0x000000);
            this.overlay.drawRect(0, 0, this.game.width, this.game.height);
            this.overlay.endFill();
            this.overlay.alpha = 0;
            this.overlay.inputEnabled = true;
            this.memoriseUI.add(this.overlay);
            this.backplateCloud = new Phaser.Image(this.game, 0, 0, 'cloud-backplate');
            this.backplateCloud.anchor.set(0.5);
            this.backplateCloud.width = this.game.width;
            this.backplateCloud.height = this.game.height * 0.85;
            this.backplateCloud.centerX = this.game.width * 0.5;
            this.backplateCloud.centerY = this.game.height - this.backplateCloud.height * 0.5;
            this.memoriseUI.add(this.backplateCloud);
            this.timerBar = new TimerBar_1.TimerBar(this.game, DressUpState.MEMORISE_TIME_S * 1000);
            this.timerBar.top = this.game.height * 0.01;
            this.memoriseUI.add(this.timerBar);
            this.targetRex = new Rex_1.Rex(this.game, true, this.controller.getCurrentTargetOutfit());
            this.targetRex.centerY = this.backplateCloud.centerY;
            this.targetRex.centerX = this.game.width * 0.5;
            this.memoriseUI.add(this.targetRex);
            this.targetRex.startAnimation();
            this.memoriseUI.visible = false;
            this.add.existing(this.memoriseUI);
            this.answerVisible = false;
            // this button is not being added to the memoriseUI, but needs to appear over the top of it,
            // so it's being created after the memoriseUI
            this.revealButton = new Phaser.Image(this.game, 0, 0, 'button-reveal');
            this.revealButton.events.onInputUp.add(this.toggleAnswer, this);
            this.revealButton.inputEnabled = true;
            this.revealButton.left = this.game.width * 0.02;
            this.revealButton.bottom = 0;
            this.revealButton.visible = false;
            this.add.existing(this.revealButton);
        }
        displayMemoriseUI() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("displayMemoriseUI()");
                // Delay before displaying memorise UI, so that cloud animations look good
                yield PromiseUtil_27.PromiseUtil.time(this.game, 800);
                // Fade cloud in from background
                this.game.add.tween(this.animCloud).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.animCloud).to({
                    width: this.game.width,
                    height: this.game.height * 0.85,
                    bottom: this.game.height
                }, 350, Phaser.Easing.Linear.None, true));
                // Pop main cloud in and out
                const initialScale = this.backplateCloud.scale.x;
                this.game.add.tween(this.backplateCloud).to({}, 400, Phaser.Easing.Linear.None, true, 0, Infinity).onRepeat.add(() => {
                    const newScale = this.backplateCloud.scale.x === initialScale ? initialScale * 0.95 : initialScale;
                    this.backplateCloud.scale.x = newScale;
                    this.backplateCloud.scale.y = newScale;
                });
                // Display rex
                this.memoriseUI.visible = true;
                // Fade in the overlay too
                this.game.add.tween(this.overlay).to({ alpha: 0.5 }, 350, Phaser.Easing.Linear.None, true);
                // Animate Rex (movement similar to cloud)
                this.game.add.tween(this.targetRex).to({}, 400, Phaser.Easing.Linear.None, true, 0, Infinity).onRepeat.add(() => {
                    const rotation = this.targetRex.angle === 0 ? -1 : 0;
                    this.targetRex.angle = rotation;
                });
                this.timerBar.start();
                this.timerLoopSound.play(); // begin the looped timer sound during memorizing phase
                this.animCloud.visible = false;
            });
        }
        hideMemoriseUI() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("hideMemoriseUI()");
                this.timerLoopSound.destroy(); // stop the timer looping sound now that the memorizing phase is over
                this.timerLoopSound = null;
                this.animCloud.visible = true;
                // Show 4 cloud particles
                const cloudParticles = [
                    new Phaser.Image(this.game, this.game.width * 0.4, this.game.height * 0.4, 'cloud-particle'),
                    new Phaser.Image(this.game, this.game.width * 0.6, this.game.height * 0.45, 'cloud-particle'),
                    new Phaser.Image(this.game, this.game.width * 0.3, this.game.height * 0.7, 'cloud-particle'),
                    new Phaser.Image(this.game, this.game.width * 0.7, this.game.height * 0.75, 'cloud-particle')
                ];
                let delay = 0;
                const riseDelay = 300;
                // Add animations to cloud particles
                for (let cloud of cloudParticles) {
                    ScaleUtil_12.scaleToBounds(cloud, this.game.width * 0.05, this.game.height * 0.05);
                    cloud.anchor.set(0.5);
                    cloud.alpha = 0.9;
                    this.add.existing(cloud);
                    // Fade out cloud
                    this.game.add.tween(cloud).to({
                        width: this.game.width * 0.4,
                        height: this.game.width * 0.4,
                        alpha: 0
                    }, 650, Phaser.Easing.Linear.None, true, delay);
                    // Particle rises up after a delay
                    this.game.add.tween(cloud).to({ y: cloud.y - this.game.height * 0.5, angle: 90 }, 750, Phaser.Easing.Linear.None, true, riseDelay)
                        .onComplete.add(() => {
                        cloud.destroy();
                    });
                    // Increment delay to stagger appearance of particles
                    delay += 100;
                }
                yield PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.overlay).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true));
                this.memoriseUI.visible = false;
                this.game.tweens.removeFrom(this.backplateCloud);
                // Show animating cloud backdrop, increase size and fade out
                this.game.add.tween(this.animCloud).to({ width: this.game.width * 1.15, height: this.game.height * 0.95 }, 500, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.animCloud).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true));
            });
        }
        toggleAnswer() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play('snd_ui_click');
                if (this.answerVisible) {
                    // Go back to results
                    this.memoriseUI.visible = false;
                    this.overlay.alpha = 0;
                    this.answerVisible = false;
                }
                else {
                    // Show background mask and memorise UI (without timer)
                    this.overlay.alpha = 0.5;
                    this.timerBar.visible = false;
                    this.memoriseUI.visible = true;
                    this.answerVisible = true;
                }
            });
        }
        createAssemblyUI() {
            console.log("createAssemblyUI()");
            // Add all buttons, put in a group, hide
            this.assemblyUI = new Phaser.Group(this.game);
            const btnArms = new UIElements_1.AssemblyButton(this.game, 'button-arms', DressUpOptions_3.OutfitPart.ARMS, this.game.width * 0.7, this.game.height * 0.44, 140);
            this.assemblyUI.add(btnArms);
            const btnFace = new UIElements_1.AssemblyButton(this.game, 'button-face', DressUpOptions_3.OutfitPart.FACE, this.game.width * 0.68, this.game.height * 0.2, 130);
            this.assemblyUI.add(btnFace);
            const btnHat = new UIElements_1.AssemblyButton(this.game, 'button-hat', DressUpOptions_3.OutfitPart.HAT, this.game.width * 0.38, this.game.height * 0.15, 85);
            this.assemblyUI.add(btnHat);
            const btnProp = new UIElements_1.AssemblyButton(this.game, 'button-prop', DressUpOptions_3.OutfitPart.PROP, this.game.width * 0.07, this.game.height * 0.25, 45);
            this.assemblyUI.add(btnProp);
            const btnBody = new UIElements_1.AssemblyButton(this.game, 'button-body', DressUpOptions_3.OutfitPart.BODY, this.game.width * 0.02, this.game.height * 0.45, 20);
            this.assemblyUI.add(btnBody);
            const btnWaist = new UIElements_1.AssemblyButton(this.game, 'button-waist', DressUpOptions_3.OutfitPart.WAIST, this.game.width * 0.05, this.game.height * 0.72, 300);
            this.assemblyUI.add(btnWaist);
            const btnFeet = new UIElements_1.AssemblyButton(this.game, 'button-feet', DressUpOptions_3.OutfitPart.FEET, this.game.width * 0.35, this.game.height * 0.83, 320);
            this.assemblyUI.add(btnFeet);
            this.assemblyUI.children.forEach((value) => {
                value.assemblySignal.add(() => {
                    this.game.sound.play('rex-toggle');
                    this.assemblyRex.incrementItem(value.part);
                    if (!this.hasChangedClothing) {
                        this.hasChangedClothing = true;
                        this.finishButton.visible = true;
                        this.game.add.tween(this.bottomMessageBar).to({ bottom: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true);
                    }
                });
                value.toggleAnswerSignal.add(this.toggleAnswer, this);
                value.visible = false;
            });
            this.finishButton = new TextArrowButton_6.TextArrowButton(this.game, LocalizationManager_18.LocalizationManager.Instance.text("dress_up.finish"));
            this.finishButton.top = this.game.height * 0.015;
            this.finishButton.right = this.game.width;
            this.finishButton.visible = false;
            this.assemblyUI.add(this.finishButton);
            this.add.existing(this.assemblyUI);
        }
        showAssemblyUI(revealResults = false) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("showAssemblyUI()");
                const uiTweenPromises = [];
                let delay = 0;
                this.assemblyUI.children.forEach((button) => {
                    if (button.part !== undefined) {
                        let tween;
                        if (revealResults) {
                            // We want to show the tick/cross animating in
                            tween = this.game.add.tween(button.buttonImage).from({ height: 0, width: 0 }, 200, Phaser.Easing.Back.Out, true, delay)
                                .chain(this.game.add.tween(button.resultImage).from({ height: 0, width: 0 }, 300, Phaser.Easing.Back.Out, true, delay + 150));
                            delay += 500;
                        }
                        else {
                            // We just need to display the input button icon
                            tween = this.game.add.tween(button).from({ height: 0, width: 0 }, 200, Phaser.Easing.Back.Out, true, delay);
                            delay += 200;
                        }
                        tween.onStart.addOnce(() => {
                            button.visible = true;
                        }, this);
                        uiTweenPromises.push(PromiseUtil_27.PromiseUtil.tween(this.game, tween));
                    }
                });
                this.assemblyUI.visible = true;
                yield Promise.all(uiTweenPromises);
            });
        }
        hideAssemblyUI() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("hideAssemblyUI()");
                const uiTweenPromises = [];
                let delay = 0;
                this.assemblyUI.children.forEach((button) => {
                    if (button.part !== undefined) {
                        // These are clothing toggles
                        const initialSize = button.height;
                        let tween = this.game.add.tween(button).to({ height: 0, width: 0 }, 50, Phaser.Easing.Linear.None, true, delay);
                        tween.onComplete.addOnce(() => {
                            button.visible = false;
                            button.width = initialSize;
                            button.height = initialSize;
                        }, this);
                        delay += 80;
                        uiTweenPromises.push(PromiseUtil_27.PromiseUtil.tween(this.game, tween));
                    }
                });
                uiTweenPromises.push(PromiseUtil_27.PromiseUtil.tween(this.game, this.game.add.tween(this.finishButton).to({ alpha: 0 }, 150, Phaser.Easing.Linear.None, true, 0)));
                this.finishButton.visible = false;
                yield Promise.all(uiTweenPromises);
            });
        }
        finishAssembly() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("finishAssembly()");
                // Hide buttons, show dialog
                yield this.hideAssemblyUI();
                this.game.sound.play('rex-complete');
                const dialog = new BasicDialog_3.BasicDialog(this.game, Character_5.Character.REX);
                dialog.width = this.game.width;
                dialog.scale.y = dialog.scale.x;
                dialog.top = 0;
                yield dialog.show();
                yield dialog.writeMessage(LocalizationManager_18.LocalizationManager.Instance.text("dress_up.results"));
                yield Promise.race([
                    dialog.waitForTap(),
                    PromiseUtil_27.PromiseUtil.signal(this.game.input.onDown)
                ]);
                yield dialog.hide();
            });
        }
        displayResults() {
            return __awaiter(this, void 0, void 0, function* () {
                // create the buttons
                this.assemblyUI.children.forEach((button) => {
                    if (button.part !== undefined) {
                        let result = false;
                        if (this.targetRex.currentOutfit[button.part] === this.assemblyRex.currentOutfit[button.part]) {
                            result = true;
                            this.score++;
                        }
                        button.showResult(result);
                    }
                });
                // animate in the buttons
                yield this.showAssemblyUI(true);
                // input enable the buttons
                this.assemblyUI.children.forEach((button) => {
                    if (button.part !== undefined) {
                        button.enableTapForSolution();
                    }
                });
                // enable the rest of the ui
                this.revealButton.visible = true;
                this.game.add.tween(this.revealButton).to({ top: 0 }, 150, Phaser.Easing.Linear.None, true, 0);
                this.memoriseUI.onChildInputDown.add(this.toggleAnswer, this);
                this.finishButton.visible = true;
                this.game.add.tween(this.finishButton).to({ alpha: 1 }, 150, Phaser.Easing.Linear.None, true, 0);
            });
        }
        transitionTo() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                GameController_20.GameController.Instance.achievementManager.onRexEndGame();
                yield _super("transitionTo").call(this, StateNames_16.StateNames.RESULTS);
            });
        }
        shutdown() {
            this.timerLoopSound = null;
            this.assemblyRex = null;
            this.targetRex = null;
            this.assemblyUI = null;
            this.memoriseUI = null;
            this.timerBar = null;
            this.backplateCloud = null;
            this.animCloud = null;
            this.finishButton = null;
            this.overlay = null;
            this.revealButton = null;
            this.controller = null;
            this.headerBar = null;
            this.curtain = null;
            this.topMessageBar = null;
            this.bottomMessageBar = null;
            this.readyButton = null;
            this.infographic = null;
            this.game.cache.removeImage('instruction-graphic-portrait');
            this.game.cache.removeImage('infographic');
            this.game.cache.removeImage('cloud-backplate');
            this.game.cache.removeImage('cloud-particle');
            this.game.cache.removeImage('timerbar-icon');
            this.game.cache.removeImage('timerbar-timer');
            this.game.cache.removeImage('timerbar-frame-tile');
            this.game.cache.removeImage('rex');
            this.game.cache.removeImage('rex-blink');
            this.game.cache.removeImage('shadow');
            this.game.cache.removeImage('assembly-arrow');
            this.game.cache.removeImage('button-reveal');
            this.game.cache.removeImage('correct-tick');
            this.game.cache.removeImage('incorrect-cross');
            this.game.cache.removeImage('button-results');
            this.game.cache.removeImage('button-arms');
            this.game.cache.removeImage('button-body');
            this.game.cache.removeImage('button-face');
            this.game.cache.removeImage('button-feet');
            this.game.cache.removeImage('button-hat');
            this.game.cache.removeImage('button-prop');
            this.game.cache.removeImage('button-waist');
            this.game.cache.removeImage('arms_base');
            // Tutorial
            this.game.cache.removeImage('tutorial-diagram');
            for (let option of DressUpOptions_3.DressUpOptions.ARM_OPTIONS) {
                this.game.cache.removeImage(option);
            }
            for (let option of DressUpOptions_3.DressUpOptions.BODY_OPTIONS) {
                this.game.cache.removeImage(option);
            }
            for (let option of DressUpOptions_3.DressUpOptions.FACE_OPTIONS) {
                this.game.cache.removeImage(option);
            }
            for (let option of DressUpOptions_3.DressUpOptions.FEET_OPTIONS) {
                this.game.cache.removeImage(option);
            }
            for (let option of DressUpOptions_3.DressUpOptions.HAT_OPTIONS) {
                this.game.cache.removeImage(option);
            }
            for (let option of DressUpOptions_3.DressUpOptions.PROP_OPTIONS) {
                this.game.cache.removeImage(option);
            }
            for (let option of DressUpOptions_3.DressUpOptions.WAIST_OPTIONS) {
                this.game.cache.removeImage(option);
            }
            // unload the sounds
            this.game.cache.removeSound('rex-complete');
            this.game.cache.removeSound('rex-countdown-loop');
            this.game.cache.removeSound('rex-toggle');
            super.shutdown();
        }
    }
    DressUpState.MEMORISE_TIME_S = 10;
    DressUpState.TRACKING_MICRO_NAME_REMEMBER = "Remember";
    DressUpState.TRACKING_MICRO_NAME_DRESS = "Dress";
    exports.DressUpState = DressUpState;
});
define("minigame/Trixie/component/Puff", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_28) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Puff extends Phaser.Image {
        constructor(game) {
            super(game, 0, 0, 'puff');
            this.anchor.set(0.5);
            this.visible = false;
        }
        emit(delayMS) {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_28.PromiseUtil.time(this.game, delayMS);
                this.origScale = this.scale.x;
                this.alpha = 0;
                this.scale.set(0.3);
                this.visible = true;
                yield Promise.all([
                    // fades steam in
                    PromiseUtil_28.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true)).then(() => __awaiter(this, void 0, void 0, function* () {
                        // fades steam out
                        yield PromiseUtil_28.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ alpha: 0 }, 250, Phaser.Easing.Linear.None, true));
                    })),
                    // scale up
                    PromiseUtil_28.PromiseUtil.tween(this.game, this.game.add.tween(this.scale).to({ x: this.origScale, y: this.origScale }, 500, Phaser.Easing.Sinusoidal.Out, true))
                ]);
                this.destroy();
            });
        }
    }
    exports.Puff = Puff;
});
define("minigame/Trixie/TrixieConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    // Warning: Change this with caution and only if you know what you're doing!!
    // 
    // This correlates to the data that is input in the 
    // game-info.json file for trixie's minigame settings.
    // It is used as the index to obtain each value in the arrays.
    var BalloonType;
    (function (BalloonType) {
        BalloonType[BalloonType["ORANGE"] = 0] = "ORANGE";
        BalloonType[BalloonType["HAZARD"] = 1] = "HAZARD";
        BalloonType[BalloonType["RED"] = 2] = "RED";
    })(BalloonType = exports.BalloonType || (exports.BalloonType = {}));
    class TrixieConfig {
        parse(data) {
            this.balloonPoints = data.balloonPoints || [100, 0, 500];
            this.balloonAssetKeys = data.balloonAssetKeys ||
                ["balloon-orange", "balloon-hazard", "balloon-red"];
            // Balloon Size
            this.balloonMaxWidthGameScreenPercent =
                data.balloonMaxWidthGameScreenPercent || [0.23, 0.23, 0.3];
            this.balloonMaxHeightGameScreenPercent =
                data.balloonMaxHeightGameScreenPercent || [0.22, 0.22, 0.25];
            // Balloon Speed
            this.balloonBounce = data.balloonBounce || 0.75;
            // Balloons / time per person
            this.maxTimePerPlayerTurnMS = data.maxTimePerPlayerTurnMS || 10000;
            this.redSpawnChance = data.redSpawnChance || 0.3;
            this.hazardSpawnChances = data.hazardSpawnChances || 0.3;
        }
    }
    exports.TrixieConfig = TrixieConfig;
});
define("minigame/Trixie/component/Balloon", ["require", "exports", "minigame/Trixie/TrixieConfig", "switchback-common/util/PromiseUtil"], function (require, exports, TrixieConfig_1, PromiseUtil_29) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author ewan@cerebralfix.com
     * @author qi@cerebralfix.com
     */
    class Balloon extends Phaser.Sprite {
        constructor(game, x, y, asset, points, bounce, gravity, balloonType) {
            super(game, x, y, asset);
            this.anchor.set(0.5);
            this._pointsAwarded = points; // initialize the points awarded for inserting this ballloon
            this._balloonType = balloonType; // initialize the balloon type
            game.physics.enable(this);
            this.body.fixedRotation = true; // don't allow the balloon to rotate/spin
            this.body.collideWorldBounds = false;
            this.body.mass = 1;
            this.body.drag = new Phaser.Point(100 * this.body.mass, 100 * this.body.mass);
            this.body.bounce.set(bounce); // set horizontal and vertical vectors to 1 (100% energy) // TODO: Play with this value (lower it) - might prevent balloons from v tecing up 
            this.inputEnabled = true;
            this.input.enableDrag(true);
            this.isBeingDragged = false;
            this.events.onDragStart.add(this.startDrag, this);
            this.events.onDragStop.add(this.stopDrag, this);
            this.body.gravity.y = -gravity;
            this.body.velocity.y = -gravity;
        }
        update() {
            if (this.bottom < 0) {
                this.destroy();
            }
        }
        startDrag() {
            this.body.moves = false;
            this.isBeingDragged = true;
            this.game.sound.play(`balloon-grab-${this.game.rnd.integerInRange(1, 5)}`, 0.75);
        }
        stopDrag() {
            this.input.draggable = true;
            this.body.moves = true;
            this.isBeingDragged = false;
        }
        /**
         * Sets the balloons gravity to a given value
         */
        adjustGravity(gravity) {
            this.body.gravity.y = gravity;
        }
        get balloonType() {
            return this._balloonType;
        }
        get pointsAwarded() {
            return this._pointsAwarded;
        }
        tweenIntoVolcano(volcanoLocation) {
            return __awaiter(this, void 0, void 0, function* () {
                // need to set gravity, x and y velocity to 0
                // or there will be bugs with tweening the balloon
                this.body.gravity.y = 0;
                this.body.velocity.x = 0;
                this.body.velocity.y = 0;
                if (this._balloonType === TrixieConfig_1.BalloonType.HAZARD) {
                    // 'Pop' the balloon
                    this.game.add.tween(this).to({ width: this.width * 0.8, height: this.height * 0.7 }, 100, Phaser.Easing.Sinusoidal.Out, true);
                    yield PromiseUtil_29.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ alpha: 0 }, 100, Phaser.Easing.Sinusoidal.Out, true));
                    this.destroy();
                }
                else {
                    // move balloon to middle of volcano width
                    this.game.add.tween(this).to({ centerX: volcanoLocation }, 250, Phaser.Easing.Sinusoidal.Out, true);
                    this.game.add.tween(this).to({ width: this.game.width * 0.2 }, 250, Phaser.Easing.Sinusoidal.Out, true, 200);
                    // move balloon down the volcano
                    yield PromiseUtil_29.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ bottom: this.game.height }, 500, Phaser.Easing.Quadratic.In, true, 0));
                    this.destroy();
                }
            });
        }
        destroy() {
            this.events.onDragStart.removeAll(this);
            this.events.onDragStop.removeAll(this);
            this.events.onInputDown.removeAll(this);
            super.destroy();
        }
    }
    exports.Balloon = Balloon;
});
define("minigame/Trixie/component/TiledBg", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_30) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author qi@cerebralfix.com
     */
    class TiledBg extends Phaser.Group {
        constructor(game, tileKeys, width) {
            super(game);
            this.DELAY_BETWEEN_TILE_ANIMATION_MS = 100;
            this.currentFrame = 0;
            this.tileWidth = width;
            tileKeys.forEach((tileKey, index) => {
                const tile = new Phaser.Image(game, 0, 0, tileKey, this.currentFrame);
                tile.anchor.set(0.5);
                tile.width = this.tileWidth;
                tile.scale.y = tile.scale.x;
                const paddingY = tile.height * 0.025;
                tile.top = -paddingY;
                tile.left = 0;
                // positions the tiles below one another	
                if (index > 0) {
                    const previousTile = this.children[0];
                    tile.top = previousTile.bottom - paddingY;
                }
                // this is required to layer the bottom tiles above the others due to extra padding in the sprite
                this.addAt(tile, 0);
                // save this off to be used in tweens later
                this.origTileHeight = tile.height;
            });
        }
        changeState(frameNumber, delayMS = 0) {
            return __awaiter(this, void 0, void 0, function* () {
                if (frameNumber === this.currentFrame) {
                    // prevents this from animating if the same frame is given
                    return;
                }
                yield PromiseUtil_30.PromiseUtil.time(this.game, delayMS);
                this.currentFrame = frameNumber;
                for (let i = 0; i < this.children.length; i++) {
                    const tile = this.children[i];
                    // add a tween out with a slight delay based on their index
                    PromiseUtil_30.PromiseUtil.tween(this.game, this.game.add.tween(tile).to({ height: 0 }, 200, Phaser.Easing.Sinusoidal.In, true, i * this.DELAY_BETWEEN_TILE_ANIMATION_MS)).then(() => {
                        // when the tween is done, change the frame of the spritesheet, and tween it back up 
                        tile.frame = frameNumber;
                        tile.width = this.tileWidth;
                        this.game.add.tween(tile).to({ height: this.origTileHeight }, 200, Phaser.Easing.Sinusoidal.Out, true);
                    });
                }
            });
        }
    }
    exports.TiledBg = TiledBg;
});
define("minigame/Trixie/component/MiniDino", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_31) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author qi@cerebralfix.com
     */
    var ObjectState;
    (function (ObjectState) {
        ObjectState[ObjectState["SHOWING"] = 0] = "SHOWING";
        ObjectState[ObjectState["HIDING"] = 1] = "HIDING";
        ObjectState[ObjectState["IS_TWEENING"] = 2] = "IS_TWEENING"; // prevents this from showing/hiding while tweening
    })(ObjectState || (ObjectState = {}));
    class MiniDino extends Phaser.Image {
        constructor(game) {
            super(game, 0, 0, 'mini-dino');
            this.width = this.game.width * 0.12;
            this.scale.y = this.scale.x;
            // save this off for tweens later
            this.origScale = this.scale.x;
            this.origHeight = this.height;
            this.anchor.set(0.5, 1);
            const rndDurationMS = this.game.rnd.integerInRange(1000, 2000);
            const rndAngle = this.game.rnd.integerInRange(6, 10);
            game.add.tween(this).to({ angle: rndAngle }, rndDurationMS, Phaser.Easing.Linear.None, true, null, Infinity, true);
            this.objectState = ObjectState.SHOWING;
        }
        show(delayMS = 0) {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_31.PromiseUtil.time(this.game, delayMS);
                if (this.objectState !== ObjectState.HIDING) {
                    return;
                }
                this.scale.x = this.origScale * 0.9;
                this.scale.y = this.origScale * 1.1;
                this.objectState = ObjectState.IS_TWEENING;
                // move up, squeeze out width, squeeze in height
                this.game.add.tween(this).to({ y: this.y - this.origHeight }, 300, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_31.PromiseUtil.tween(this.game, this.game.add.tween(this.scale).to({ x: this.origScale, y: this.origScale }, 200, Phaser.Easing.Linear.None, true, 200));
                this.objectState = ObjectState.SHOWING;
            });
        }
        hide(delayMS = 0) {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_31.PromiseUtil.time(this.game, delayMS);
                if (this.objectState !== ObjectState.SHOWING) {
                    return;
                }
                this.objectState = ObjectState.IS_TWEENING;
                // squeeze in width, squeeze out height, move down
                this.game.add.tween(this.scale).to({ x: this.origScale * 0.9, y: this.origScale * 1.1 }, 200, Phaser.Easing.Linear.None, true);
                yield PromiseUtil_31.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ y: this.y + this.origHeight }, 300, Phaser.Easing.Linear.None, true, 100));
                this.scale.set(this.origScale);
                this.objectState = ObjectState.HIDING;
            });
        }
    }
    exports.MiniDino = MiniDino;
});
define("minigame/Trixie/component/Steam", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_32) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author qi@cerebralfix.com
     */
    class Steam extends Phaser.Image {
        constructor(game) {
            super(game, 0, 0, 'steam');
            this.anchor.set(0.5);
            this.visible = false;
        }
        emit() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.origPos) {
                    this.origPos = new Phaser.Point(this.x, this.y);
                    this.origScale = this.scale.x;
                }
                this.alpha = 0;
                this.visible = true;
                yield Promise.all([
                    // fades steam in
                    PromiseUtil_32.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true)).then(() => __awaiter(this, void 0, void 0, function* () {
                        // fades steam out
                        yield PromiseUtil_32.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ alpha: 0 }, 250, Phaser.Easing.Linear.None, true));
                    })),
                    // move steam up				
                    PromiseUtil_32.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ y: this.y - (this.height * 1.5) }, 500, Phaser.Easing.Linear.None, true)),
                    // scale up
                    PromiseUtil_32.PromiseUtil.tween(this.game, this.game.add.tween(this.scale).to({ x: this.origScale * 1.1, y: this.origScale * 1.1 }, 500, Phaser.Easing.Linear.None, true))
                ]);
                this.visible = false;
                // resets the alpha, x, and y position
                this.scale.set(this.origScale);
                this.x = this.origPos.x;
                this.y = this.origPos.y;
            });
        }
    }
    exports.Steam = Steam;
});
/**
 * @author qi@cerebralfix.com
 */
define("minigame/Trixie/component/IShakeableComponent", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("minigame/Trixie/component/LavaFlow", ["require", "exports", "switchback-common/util/PromiseUtil", "minigame/Trixie/component/Steam"], function (require, exports, PromiseUtil_33, Steam_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author qi@cerebralfix.com
     */
    class LavaFlow extends Phaser.Image {
        constructor(game, shakeOffset = 0) {
            super(game, 0, 0, 'lava-flow');
            this.anchor.set(0.5);
            this.steam = new Steam_1.Steam(game);
            this.steam.width = game.width * 0.9;
            this.steam.scale.y = this.steam.scale.x;
            this.steam.y = this.height * 0.7;
            this.addChild(this.steam);
            // if shakeOffset is not provided, this lavaflow will never shake
            this.shakeOffset = shakeOffset;
            this.isShaking = false;
        }
        emitSteam() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.steam.emit();
            });
        }
        startShake() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.isShaking) {
                    this.isShaking = true;
                    // start shaking, wait 0.2s, then emit steam once
                    this.updateShake(1);
                    yield PromiseUtil_33.PromiseUtil.time(this.game, 200);
                    this.emitSteam();
                }
            });
        }
        stopShake() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.isShaking) {
                    // stop shaking if it is shaking
                    this.updateShake(0);
                    this.isShaking = false;
                }
            });
        }
        updateShake(timeScale) {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.isShaking) {
                    // if this is currently not shaking, do not allow to update the shake.
                    return;
                }
                if (!this.shakeYTween) {
                    // if the shake tween doesnt yet exist, create one
                    this.shakeYTween = this.game.add.tween(this).to({ y: this.y + this.shakeOffset }, 300, Phaser.Easing.Linear.None, true, null, Infinity, true);
                }
                this.shakeYTween.timeScale = timeScale;
            });
        }
    }
    exports.LavaFlow = LavaFlow;
});
define("minigame/Trixie/component/TrixieEmote", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil"], function (require, exports, PromiseUtil_34, ScaleUtil_13) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author qi@cerebralfix.com
     */
    var EmoteState;
    (function (EmoteState) {
        EmoteState[EmoteState["NEUTRAL"] = 0] = "NEUTRAL";
        EmoteState[EmoteState["WORRIED"] = 1] = "WORRIED";
        EmoteState[EmoteState["ABSOLUTELY_TERRIFIED"] = 2] = "ABSOLUTELY_TERRIFIED";
    })(EmoteState = exports.EmoteState || (exports.EmoteState = {}));
    class TrixieEmote extends Phaser.Image {
        constructor(game) {
            super(game, 0, 0, 'trixie-emotes', 0);
            ScaleUtil_13.scaleToBounds(this, game.width * 0.7, game.height * 0.45);
            this.anchor.set(1, 1);
            this.angle = 90;
            this.isShowing = false;
        }
        show(emoteState) {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.isShowing) {
                    yield this.hide();
                }
                this.frame = emoteState;
                ScaleUtil_13.scaleToBounds(this, this.game.width * 0.65, this.game.height * 0.4);
                const tweenIn = this.game.add.tween(this).to({ angle: 0 }, 250, Phaser.Easing.Sinusoidal.In, true);
                yield PromiseUtil_34.PromiseUtil.tween(this.game, tweenIn);
                this.isShowing = true;
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                const tweenOut = this.game.add.tween(this).to({ angle: 90 }, 150, Phaser.Easing.Sinusoidal.Out, true);
                yield PromiseUtil_34.PromiseUtil.tween(this.game, tweenOut);
                this.isShowing = false;
            });
        }
        updateShake(timeScale) {
            return __awaiter(this, void 0, void 0, function* () {
                // if the shake tween doesnt yet exist, create one
                if (!this.shakeYTween) {
                    this.shakeYTween = this.game.add.tween(this).to({ y: this.y + this.game.height * 0.01 }, 150, Phaser.Easing.Linear.None, true, null, Infinity, true);
                }
                if (!this.shakeXTween) {
                    this.shakeXTween = this.game.add.tween(this).to({ x: this.x + this.game.width * 0.01 }, 123, Phaser.Easing.Linear.None, true, null, Infinity, true);
                }
                this.shakeYTween.timeScale = timeScale;
                this.shakeXTween.timeScale = timeScale;
            });
        }
    }
    exports.TrixieEmote = TrixieEmote;
});
define("minigame/Trixie/component/LavaBalloon", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil"], function (require, exports, ScaleUtil_14, PromiseUtil_35) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author qi@cerebralfix.com
     */
    class LavaBalloon extends Phaser.Image {
        constructor(game, maxWidth, maxHeight, angle) {
            super(game, 0, 0, 'lava-balloon');
            this.anchor.set(0.5, 0);
            ScaleUtil_14.scaleToBounds(this, maxWidth, maxHeight);
            this.angle = angle;
            // save these off for tweening later
            this.origScale = this.scale.x;
            this.origAngle = angle;
            // makes the balloon sway from side to side
            const rndDelayMS = this.game.rnd.integerInRange(0, 1000);
            this.game.add.tween(this).to({ angle: this.angle + 10 }, 1000, Phaser.Easing.Linear.None, true, rndDelayMS, Infinity, true);
            this.visible = false;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                // prevents the lavaballoon from animating in again if it is already showing
                if (this.isReleasing || this.visible) {
                    return;
                }
                this.scale.set(0);
                this.visible = true;
                // scale up
                yield PromiseUtil_35.PromiseUtil.tween(this.game, this.game.add.tween(this.scale).to({ x: this.origScale, y: this.origScale }, 500, Phaser.Easing.Linear.None, true));
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                // prevents the lavaballoon from animating out again if it is already hidden
                if (!this.visible) {
                    return;
                }
                // scale down
                yield PromiseUtil_35.PromiseUtil.tween(this.game, this.game.add.tween(this.scale).to({ x: 0, y: 0 }, 500, Phaser.Easing.Linear.None, true));
                this.visible = false;
            });
        }
        release() {
            return __awaiter(this, void 0, void 0, function* () {
                this.isReleasing = true;
                this.origPos = new Phaser.Point(this.x, this.y);
                const rndXPos = this.game.rnd.integerInRange(0, this.game.width);
                const rndAngle = this.game.rnd.integerInRange(-180, 180);
                const floatUpDurationMS = 1500;
                // float up and fade out
                yield Promise.all([
                    PromiseUtil_35.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true, floatUpDurationMS - 500)),
                    PromiseUtil_35.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ x: rndXPos - this.worldPosition.x, y: -this.height - this.worldPosition.y, angle: this.angle + rndAngle }, floatUpDurationMS, Phaser.Easing.Sinusoidal.Out, true))
                ]);
                this.visible = false;
                // resets the angle, alpha, x, and y position
                this.angle = this.origAngle;
                this.x = this.origPos.x;
                this.y = this.origPos.y;
                this.alpha = 1;
                this.isReleasing = false;
            });
        }
    }
    exports.LavaBalloon = LavaBalloon;
});
define("minigame/Trixie/component/Volcano", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "minigame/Trixie/component/LavaBalloon", "minigame/Trixie/component/Steam"], function (require, exports, PromiseUtil_36, ScaleUtil_15, LavaBalloon_1, Steam_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author qi@cerebralfix.com
     */
    class Volcano extends Phaser.Group {
        constructor(game) {
            super(game);
            // changeable values for lava balloons
            this.NUM_LAVA_BALLOONS = 3;
            this.BIGGEST_LAVA_BALLOON_ANGLE = 20;
            this.steam = new Steam_2.Steam(game);
            this.steam.width = game.width * 0.75;
            this.steam.scale.y = this.steam.scale.x;
            this.add(this.steam);
            this.volcanoLayer = new Phaser.Group(game);
            this.add(this.volcanoLayer);
            const volcanoBase = new Phaser.Image(this.game, 0, 0, 'volcano-base');
            volcanoBase.anchor.set(0.5);
            ScaleUtil_15.scaleToBounds(volcanoBase, this.game.width, this.game.height * 0.25);
            this.volcanoLayer.add(volcanoBase);
            // position the eruption steam in the center of the volcano
            this.steam.centerX = volcanoBase.centerX;
            this.steam.bottom = volcanoBase.bottom;
            this.lavaBalloons = [];
            const angleToIncrement = (this.BIGGEST_LAVA_BALLOON_ANGLE * 2) / (this.NUM_LAVA_BALLOONS - 1);
            const startXPos = volcanoBase.width * 0.45;
            const xInc = volcanoBase.width * 0.054;
            // create [this.NUM_LAVA_BALLOONS] number of lava balloons
            for (let i = 0; i < this.NUM_LAVA_BALLOONS; i++) {
                const angle = this.BIGGEST_LAVA_BALLOON_ANGLE - (i * angleToIncrement);
                const lavaBalloon = new LavaBalloon_1.LavaBalloon(this.game, volcanoBase.width * 0.065, volcanoBase.height * 0.93, angle);
                lavaBalloon.top = volcanoBase.top + volcanoBase.height * 0.18;
                lavaBalloon.x = volcanoBase.left + startXPos + (xInc * i);
                this.volcanoLayer.add(lavaBalloon);
                this.lavaBalloons.push(lavaBalloon);
            }
        }
        playEruptAnimation() {
            return __awaiter(this, void 0, void 0, function* () {
                const animationDurationMs = 500;
                this.steam.emit();
                this.origScale = this.volcanoLayer.scale.x;
                // Greatly reduce body height and increase body width	
                yield PromiseUtil_36.PromiseUtil.tween(this.game, this.game.add.tween(this.volcanoLayer.scale).to({ x: this.origScale * 0.5, y: this.origScale * 2.2 }, 600, Phaser.Easing.Back.In, true));
                PlayAPI.vibrateDevice();
                this.game.camera.shake(0.015, animationDurationMs * 1.2, true, Phaser.Camera.SHAKE_VERTICAL); // shakes the camera to simulate eruption
                this.releaseLavaBalloons();
                // Return the volcano back to its original size
                this.game.add.tween(this.volcanoLayer.scale).to({ x: this.origScale, y: this.origScale }, animationDurationMs, Phaser.Easing.Cubic.Out, true);
            });
        }
        setVisibleLavaBalloons(count) {
            return __awaiter(this, void 0, void 0, function* () {
                // show this number of lava balloons
                for (let i = 0; i < count; i++) {
                    this.lavaBalloons[i].show();
                }
                // hide the rest
                for (let i = count; i < this.lavaBalloons.length; i++) {
                    this.lavaBalloons[i].hide();
                }
            });
        }
        releaseLavaBalloons() {
            return __awaiter(this, void 0, void 0, function* () {
                this.lavaBalloons.forEach((lavaBalloon) => {
                    lavaBalloon.release();
                });
            });
        }
        startShake() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.isShaking) {
                    this.isShaking = true;
                    // start shaking
                    this.updateShake(1);
                }
            });
        }
        stopShake() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.isShaking) {
                    // stop shaking if it is shaking
                    this.updateShake(0);
                    this.isShaking = false;
                }
            });
        }
        updateShake(timeScale) {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.isShaking) {
                    // if this is currently not shaking, do not allow to update the shake.
                    return;
                }
                if (!this.shakeYTween) {
                    // if the shake tween doesnt yet exist, create one
                    this.shakeYTween = this.game.add.tween(this).to({ y: this.y + this.game.height * 0.008 }, 300, Phaser.Easing.Linear.None, true, null, Infinity, true);
                }
                this.shakeYTween.timeScale = timeScale;
            });
        }
    }
    exports.Volcano = Volcano;
});
define("minigame/Trixie/component/ScoreBar", ["require", "exports", "Game", "component/MessageBar", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil"], function (require, exports, Game_23, MessageBar_9, LocalizationManager_19, ScaleUtil_16) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Custom message bar
     * Used to display player scores on final screen
     */
    class ScoreBar extends MessageBar_9.MessageBar {
        constructor(game, name, score, isHighScore) {
            super(game, LocalizationManager_19.LocalizationManager.Instance.text("stacker.careful"));
            // Set the height of the bars to be shorter than normal
            // Screen height * 0.1 rather than * 0.15
            const tileScale = this.game.height * 0.1 / this.tilingBackground.texture.height;
            this.tilingBackground.tileScale.set(tileScale, tileScale);
            this.tilingBackground.height = this.game.height * 0.1;
            // Recenter the text, 0.48 accounts for text padding
            this.instructionText.centerY = this.tilingBackground.top + this.tilingBackground.height * 0.48;
            const imgHeight = this.tilingBackground.height * 0.85;
            const balloonIcon = new Phaser.Image(this.game, 0, 0, 'score-icon');
            ScaleUtil_16.scaleToBounds(balloonIcon, imgHeight, imgHeight);
            balloonIcon.right = this.tilingBackground.right;
            balloonIcon.centerY = this.instructionText.centerY;
            this.add(balloonIcon);
            const scoreText = new Phaser.Text(this.game, 0, 0, score.toString(), {
                font: Game_23.Game.DEFAULT_FONT,
                fontSize: 90,
                fill: Game_23.Game.TEXT_COLOR_WHITE,
                stroke: Game_23.Game.TEXT_COLOR_BLUE,
                strokeThickness: Game_23.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                wordWrap: true,
                wordWrapWidth: this.game.width * 0.95,
                align: 'center'
            });
            scoreText.right = balloonIcon.left - this.game.width * 0.005;
            scoreText.centerY = balloonIcon.centerY;
            this.add(scoreText);
            this.instructionText.text = name;
            this.instructionText.wordWrap = true;
            this.instructionText.wordWrapWidth = scoreText.left;
            this.instructionText.centerX = scoreText.left * 0.53;
            if (isHighScore) {
                // Add a star to the score bar
                const star = new Phaser.Image(this.game, 0, 0, 'rank-star');
                ScaleUtil_16.scaleToBounds(star, this.tilingBackground.height * 0.4, this.tilingBackground.height * 0.4);
                star.right = this.instructionText.left - star.width * 0.3;
                star.centerY = this.tilingBackground.height * 0.5;
                this.add(star);
            }
        }
    }
    exports.ScoreBar = ScoreBar;
});
define("minigame/Trixie/component/ResultAnimations", ["require", "exports", "switchback-common/util/PromiseUtil", "minigame/Trixie/TrixieConfig"], function (require, exports, PromiseUtil_37, TrixieConfig_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SuccessAnimation extends Phaser.Group {
        constructor(game) {
            super(game);
            this.successRay = new Phaser.Image(this.game, 0, 0, 'success-ray');
            this.successRay.width = this.game.width * 0.8;
            this.successRay.scale.y = this.successRay.scale.x;
            this.successRay.anchor.set(0.5, 1);
            this.successRay.alpha = 0;
            this.add(this.successRay);
            this.stars = [];
            const colors = ['pink', 'blue', 'green'];
            for (let i = 0; i < 4; i++) {
                let star = new Phaser.Image(this.game, 0, 0, 'star-' + colors[i % colors.length]);
                star.alpha = 0;
                star.width = star.height = this.game.width * 0.1;
                star.anchor.set(0.5);
                this.stars.push(star);
                this.add(star);
            }
        }
        show(originPoint) {
            return __awaiter(this, void 0, void 0, function* () {
                // Starting positions
                const animationPromises = [];
                // Ray
                this.successRay.position.set(originPoint.x, originPoint.y);
                this.successRay.alpha = 0;
                // Ray's tweens
                const rayScaleTween = this.game.add.tween(this.successRay.scale)
                    .from({ x: 0.5 }, SuccessAnimation.SUCCESS_ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.successRay)
                    .to({ alpha: 1 }, SuccessAnimation.SUCCESS_ANIMATION_DURATION_MS * 0.5, Phaser.Easing.Linear.None, true, 0, 0, true);
                animationPromises.push(PromiseUtil_37.PromiseUtil.tween(this.game, rayScaleTween));
                // Stars
                for (let i = 0; i < this.stars.length; i++) {
                    const star = this.stars[i];
                    star.position.set(originPoint.x, originPoint.y);
                    star.alpha = 0;
                    star.angle = 0;
                    star.visible = true;
                    this.game.tweens.removeFrom(star);
                    // Alpha
                    const starAlphaTween = this.game.add.tween(star)
                        .to({ alpha: 1 }, SuccessAnimation.SUCCESS_ANIMATION_DURATION_MS * 0.5, Phaser.Easing.Linear.None, true, 0, 0, true);
                    starAlphaTween.onComplete.addOnce(() => {
                        star.visible = false;
                    });
                    // X movement
                    const starXMovementTween = this.game.add.tween(star)
                        .to({ x: star.x + (this.game.width * SuccessAnimation.STAR_X_DEST[i]) }, SuccessAnimation.SUCCESS_ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true);
                    // Y movement (first bit)
                    const starYMovementTweenUp = this.game.add.tween(star)
                        .to({ y: star.y + (this.game.height * SuccessAnimation.STAR_Y_DEST_1[i]) }, SuccessAnimation.SUCCESS_ANIMATION_DURATION_MS * 0.5, Phaser.Easing.Quadratic.Out, true);
                    // Y movement (second bit)
                    const starYMovementTweenDown = this.game.add.tween(star)
                        .to({ y: star.y + (this.game.height * SuccessAnimation.STAR_Y_DEST_2[i]) }, SuccessAnimation.SUCCESS_ANIMATION_DURATION_MS * 0.5, Phaser.Easing.Quadratic.In, false);
                    starYMovementTweenUp.chain(starYMovementTweenDown);
                    // Rotation
                    this.game.add.tween(star)
                        .to({ angle: star.angle + SuccessAnimation.STAR_ANGLE_DEST[i] }, SuccessAnimation.SUCCESS_ANIMATION_DURATION_MS, Phaser.Easing.Linear.None, true);
                    animationPromises.push(PromiseUtil_37.PromiseUtil.tween(this.game, starXMovementTween));
                }
                yield Promise.all(animationPromises);
                this.destroy();
            });
        }
    }
    SuccessAnimation.SUCCESS_ANIMATION_DURATION_MS = 1000;
    SuccessAnimation.STAR_X_DEST = [.3, 0.1, -0.1, -0.3];
    SuccessAnimation.STAR_Y_DEST_1 = [-.1, -.14, -.14, -.1];
    SuccessAnimation.STAR_Y_DEST_2 = [.05, 0, 0, .05];
    SuccessAnimation.STAR_ANGLE_DEST = [90, 90, -90, -90];
    exports.SuccessAnimation = SuccessAnimation;
    class BurstAnimation extends Phaser.Group {
        constructor(game, type) {
            super(game);
            let spriteKey;
            switch (type) {
                case TrixieConfig_2.BalloonType.HAZARD:
                    spriteKey = "balloon-hazard-pop";
                    break;
                case TrixieConfig_2.BalloonType.RED:
                    spriteKey = "balloon-red-pop";
                    break;
                default:
                    spriteKey = "balloon-orange-pop";
                    break;
            }
            this.explosion = new Phaser.Sprite(this.game, 0, 0, spriteKey, 0);
            this.explosion.width = this.game.width * 0.6;
            this.explosion.scale.y = this.explosion.scale.x;
            this.explosion.anchor.set(0.5);
            this.explosion.alpha = 0;
            this.explosion.animations.add('pop', null, 24, false);
            this.add(this.explosion);
            this.popEffect = new Phaser.Image(this.game, 0, 0, "balloon-pop");
            this.popEffect.width = this.game.width * 0.8;
            this.popEffect.scale.y = this.popEffect.scale.x;
            this.popEffect.anchor.set(0.5);
            this.popEffect.alpha = 0;
            this.add(this.popEffect);
        }
        show(originPoint) {
            return __awaiter(this, void 0, void 0, function* () {
                this.explosion.alpha = 1;
                this.explosion.centerX = originPoint.x;
                this.explosion.centerY = originPoint.y;
                this.popEffect.alpha = 1;
                this.popEffect.centerX = originPoint.x;
                this.popEffect.centerY = originPoint.y;
                const animationPromises = [];
                animationPromises.push(PromiseUtil_37.PromiseUtil.animation(this.explosion.animations.play('pop')));
                animationPromises.push(PromiseUtil_37.PromiseUtil.tween(this.game, this.game.add.tween(this.explosion).to({ alpha: 0 }, 450, Phaser.Easing.Quintic.In, true)));
                animationPromises.push(PromiseUtil_37.PromiseUtil.tween(this.game, this.game.add.tween(this.popEffect.scale).from({ x: 0, y: 0 }, 800, Phaser.Easing.Sinusoidal.Out, true, 250)));
                animationPromises.push(PromiseUtil_37.PromiseUtil.tween(this.game, this.game.add.tween(this.popEffect).to({ alpha: 0 }, 800, Phaser.Easing.Linear.None, true, 250)));
                yield Promise.all(animationPromises);
                this.destroy();
            });
        }
    }
    exports.BurstAnimation = BurstAnimation;
});
define("minigame/Trixie/TrixieDifficultyConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TrixieDifficultyConfig {
        parse(data) {
            this.balloonGravity = data.balloonGravity || [500, 500, 500];
            this.spawnRate = data.spawnRate || 800;
            this.hasRed = data.hasRed || false;
        }
    }
    exports.TrixieDifficultyConfig = TrixieDifficultyConfig;
});
define("minigame/Trixie/TrixieState", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "minigame/Trixie/component/Puff", "Game", "StateNames", "logic/GameController", "minigame/Trixie/component/Balloon", "minigame/Trixie/component/TiledBg", "minigame/Trixie/component/MiniDino", "minigame/Trixie/component/LavaFlow", "minigame/Trixie/component/TrixieEmote", "minigame/Trixie/component/Volcano", "minigame/Trixie/component/ScoreBar", "minigame/Trixie/component/ResultAnimations", "minigame/Trixie/TrixieConfig", "model/ToyStoryConfig", "component/Curtain", "component/PassAndPlayShowable", "component/button/TextArrowButton", "component/MessageBar", "component/HeaderBar", "logic/TSMAnalyticsManager", "state/GameBaseState", "minigame/Trixie/TrixieDifficultyConfig"], function (require, exports, PromiseUtil_38, ScaleUtil_17, LocalizationManager_20, Puff_1, Game_24, StateNames_17, GameController_21, Balloon_1, TiledBg_1, MiniDino_1, LavaFlow_1, TrixieEmote_1, Volcano_1, ScoreBar_1, ResultAnimations_1, TrixieConfig_3, ToyStoryConfig_14, Curtain_6, PassAndPlayShowable_4, TextArrowButton_7, MessageBar_10, HeaderBar_5, TSMAnalyticsManager_8, GameBaseState_7, TrixieDifficultyConfig_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var GameState;
    (function (GameState) {
        GameState[GameState["LOADING"] = 0] = "LOADING";
        GameState[GameState["PLAYING"] = 1] = "PLAYING";
        GameState[GameState["TRANSITIONING"] = 2] = "TRANSITIONING";
    })(GameState = exports.GameState || (exports.GameState = {}));
    var BackgroundFrame;
    (function (BackgroundFrame) {
        BackgroundFrame[BackgroundFrame["NEUTRAL"] = 0] = "NEUTRAL";
        BackgroundFrame[BackgroundFrame["ERUPTING"] = 1] = "ERUPTING";
    })(BackgroundFrame = exports.BackgroundFrame || (exports.BackgroundFrame = {}));
    class TrixieState extends GameBaseState_7.GameBaseState {
        constructor() {
            super(...arguments);
            // Balloon timer
            this.balloonsSpawning = false;
        }
        init(...params) {
            this.stateName = StateNames_17.StateNames.GAME_TRIXIE;
            super.init(params);
            this.config = this.loadConfig(TrixieConfig_3.TrixieConfig);
            this.difficultyConfig = this.loadDifficultyConfig(TrixieDifficultyConfig_1.TrixieDifficultyConfig);
            this.players = GameController_21.GameController.Instance.playerManager.getPlayerSession();
            this.currentPlayerIndex = 0;
            this.timeLeftInTurn = 0;
            this.timeTilNextBalloonSpawn = 0;
            this.volatility = 0;
            this.displayedVolatility = 0;
            this.score = [this.players.length];
            this.score[this.currentPlayerIndex] = 0;
            this.numBalloonsSinceHazard = 0;
        }
        preload() {
            super.preload();
            this.game.load.image('balloon-orange', "assets/minigame/trixie/balloons/balloon_orange.png");
            this.game.load.spritesheet('balloon-orange-pop', "assets/minigame/trixie/balloons/balloon_orange_pop.png", 495, 539);
            this.game.load.image('balloon-red', "assets/minigame/trixie/balloons/balloon_red.png");
            this.game.load.spritesheet('balloon-red-pop', "assets/minigame/trixie/balloons/balloon_red_pop.png", 495, 539);
            this.game.load.image('balloon-hazard', "assets/minigame/trixie/balloons/balloon_hazard.png");
            this.game.load.spritesheet('balloon-hazard-pop', "assets/minigame/trixie/balloons/balloon_hazard_pop.png", 514, 539);
            // bg scenes
            for (let i = 1; i <= 5; i++) {
                this.game.load.spritesheet(`bg-scene-${i}`, `assets/minigame/trixie/bg/scene_${i}.jpg`, 756, 190);
            }
            // fg layers 
            this.game.load.image('mini-volcanoes', 'assets/minigame/trixie/fg-layers/background_layer_1.png');
            this.game.load.image('ground-1', 'assets/minigame/trixie/fg-layers/foreground_layer_1.png');
            this.game.load.image('ground-2', 'assets/minigame/trixie/fg-layers/foreground_layer_2.png');
            this.game.load.image('tree-tilt-right', 'assets/minigame/trixie/fg-layers/tree_2_merged.png');
            this.game.load.image('tree-tilt-left', 'assets/minigame/trixie/fg-layers/tree_1_merged.png');
            this.game.load.image('lava-flow', 'assets/minigame/trixie/fg-layers/lava_fore_base.png');
            this.game.load.image('mini-dino', 'assets/minigame/trixie/fg-layers/smalldino_1.png');
            // volcano
            this.game.load.image('volcano-base', 'assets/minigame/trixie/volcano_base.png');
            // trixie emote states
            this.game.load.spritesheet('trixie-emotes', 'assets/minigame/trixie/trixie-emote-states.png', 573, 600);
            // effects
            this.game.load.image('lava-balloon', 'assets/minigame/trixie/effects/balloon_squigglelava.png');
            this.game.load.image('steam', 'assets/minigame/trixie/effects/lavasteam.png');
            this.game.load.image('puff', 'assets/minigame/trixie/effects/puff.png');
            // Instructions
            this.game.load.image('character-dialog-backplate', 'assets/dialog/character_dialog_backplate.png');
            this.game.load.image('instruction-graphic-volcano', 'assets/dialog/instruction_graphic_volcano.png');
            this.game.load.image('tiling-bar', 'assets/common/CTAbar_tilingbg.png');
            this.game.load.image('cta-curtain', 'assets/common/cta-curtain.png');
            this.game.load.image('cta-header', 'assets/dialog/cta_header.png');
            this.game.load.image('next-button', 'assets/common/next-button.png');
            this.game.load.image('right-figure', 'assets/common/right_figure.png');
            this.game.load.image('left-figure', 'assets/common/left_figure.png');
            this.game.load.image('phone', 'assets/common/phone.png');
            // UI
            this.game.load.image('arrow-right', 'assets/minigame/trixie/ui/arrow_right.png');
            this.game.load.image('timer-icon', 'assets/common/timer-icon.png');
            this.game.load.image('score-icon', 'assets/minigame/trixie/ui/score_icon.png');
            this.game.load.image('rank-star', 'assets/minigame/trixie/ui/rankstar.png');
            // Success/failure UI
            this.game.load.image('success-ray', 'assets/minigame/woody/success_ray.png');
            this.game.load.image('star-green', 'assets/minigame/woody/star_green.png');
            this.game.load.image('star-pink', 'assets/minigame/woody/star_pink.png');
            this.game.load.image('star-blue', 'assets/minigame/woody/star_blue.png');
            this.game.load.image('balloon-pop', 'assets/minigame/trixie/effects/pop.png');
            // Audio
            this.game.load.audio('balloon-correct', ['assets/sound/trixie/balloon_correct.m4a', 'assets/sound/trixie/balloon_correct.ogg']);
            this.game.load.audio('balloon-grab-1', ['assets/sound/trixie/grab_balloon_1.m4a', 'assets/sound/trixie/grab_balloon_1.ogg']);
            this.game.load.audio('balloon-grab-2', ['assets/sound/trixie/grab_balloon_2.m4a', 'assets/sound/trixie/grab_balloon_2.ogg']);
            this.game.load.audio('balloon-grab-3', ['assets/sound/trixie/grab_balloon_3.m4a', 'assets/sound/trixie/grab_balloon_3.ogg']);
            this.game.load.audio('balloon-grab-4', ['assets/sound/trixie/grab_balloon_4.m4a', 'assets/sound/trixie/grab_balloon_4.ogg']);
            this.game.load.audio('balloon-grab-5', ['assets/sound/trixie/grab_balloon_5.m4a', 'assets/sound/trixie/grab_balloon_5.ogg']);
            this.game.load.audio('balloon-pop-1', ['assets/sound/trixie/balloon_pop_1.m4a', 'assets/sound/trixie/balloon_pop_1.ogg']);
            this.game.load.audio('balloon-pop-2', ['assets/sound/trixie/balloon_pop_2.m4a', 'assets/sound/trixie/balloon_pop_2.ogg']);
            this.game.load.audio('balloon-pop-3', ['assets/sound/trixie/balloon_pop_3.m4a', 'assets/sound/trixie/balloon_pop_3.ogg']);
            this.game.load.audio('countdown-beep', ['assets/sound/trixie/countdown_beep.m4a', 'assets/sound/trixie/countdown_beep.ogg']);
            this.game.load.audio('volcano-explode', ['assets/sound/trixie/volcano_explode.m4a', 'assets/sound/trixie/volcano_explode.ogg']);
        }
        create() {
            super.create();
            this.trackGameState();
            // start ARCADE physics engine
            this.game.physics.startSystem(Phaser.Physics.ARCADE);
            this.volcanoInsertionHitBox = this.game.add.sprite(0, 0);
            this.game.physics.enable(this.volcanoInsertionHitBox);
            this.volcanoInsertionHitBox.body.setSize(this.game.width * 0.04, this.game.height * 0.01);
            this.volcanoInsertionHitBox.x = this.game.width * 0.48;
            this.volcanoInsertionHitBox.y = this.game.height * 0.45;
            this.volcanoInsertionHitBox.body.allowGravity = false;
            this.volcanoInsertionHitBox.body.immovable = true;
            /////////////////////////////////////////////////////////
            // BACKGROUND LAYER
            // Includes: 5 piece image bg which animate and change on eruption
            /////////////////////////////////////////////////////////
            const bgKeys = [
                'bg-scene-1',
                'bg-scene-2',
                'bg-scene-3',
                'bg-scene-4',
                'bg-scene-5'
            ];
            // bg that change change states 1 tile at a time from bottom to top
            this.tiledBg = new TiledBg_1.TiledBg(this.game, bgKeys, this.game.width);
            this.add.existing(this.tiledBg);
            this.balloonInsertingLayer = new Phaser.Group(this.game);
            this.add.existing(this.balloonInsertingLayer);
            /////////////////////////////////////////////////////////
            // VOLCANO LAYER
            // Includes: Mini-volcanoes, Main Volcano which erupts, 
            // and lava balloons
            /////////////////////////////////////////////////////////
            const miniVolcanoBg = new Phaser.Image(this.game, 0, 0, 'mini-volcanoes');
            ScaleUtil_17.scaleToBounds(miniVolcanoBg, this.game.width, this.game.height * 0.15);
            miniVolcanoBg.centerX = this.game.world.centerX;
            miniVolcanoBg.top = this.game.height * 0.59;
            this.add.existing(miniVolcanoBg);
            this.mainVolcano = new Volcano_1.Volcano(this.game);
            this.mainVolcano.centerX = this.game.world.centerX;
            this.mainVolcano.top = this.game.height * 0.5;
            this.add.existing(this.mainVolcano);
            /////////////////////////////////////////////////////////
            // BACKDINO LAYER
            // Includes: Back Mini-dinosaurs 
            /////////////////////////////////////////////////////////
            this.miniDinos = [];
            let MiniDinoPos;
            (function (MiniDinoPos) {
                MiniDinoPos[MiniDinoPos["BACK"] = 0] = "BACK";
                MiniDinoPos[MiniDinoPos["FRONT"] = 1] = "FRONT";
            })(MiniDinoPos || (MiniDinoPos = {}));
            // add/change minidino positions here!
            const miniDinoPositions = [
                [
                    [this.game.width * 0.25, this.game.height * 0.76],
                    // back middle minidino
                    [this.game.width * 0.68, this.game.height * 0.76],
                ],
                [
                    [this.game.width * 0.08, this.game.height * 0.91]
                ]
            ];
            const backDinoLayer = new Phaser.Group(this.game);
            const backDinoPositions = miniDinoPositions[MiniDinoPos.BACK];
            for (let i = 0; i < backDinoPositions.length; i++) {
                const miniDino = new MiniDino_1.MiniDino(this.game);
                miniDino.x = backDinoPositions[i][0];
                miniDino.y = backDinoPositions[i][1];
                this.miniDinos.push(miniDino);
                backDinoLayer.add(miniDino);
            }
            this.add.existing(backDinoLayer);
            /////////////////////////////////////////////////////////
            // MIDGROUND LAYER
            // Includes: Midground and Trees
            /////////////////////////////////////////////////////////
            const midGrdLayer = new Phaser.Group(this.game);
            const midGrd = new Phaser.Image(this.game, 0, 0, 'ground-2');
            ScaleUtil_17.scaleToBounds(midGrd, this.game.width, this.game.height * 0.3);
            midGrd.centerX = this.game.world.centerX;
            midGrd.top = this.game.height * 0.61;
            midGrdLayer.add(midGrd);
            const leftTree = new Phaser.Image(this.game, 0, 0, 'tree-tilt-right');
            ScaleUtil_17.scaleToBounds(leftTree, this.game.width * 0.3, this.game.height * 0.28);
            leftTree.anchor.set(0.5, 0.92);
            leftTree.right = midGrd.left + midGrd.width * 0.2;
            leftTree.bottom = midGrd.bottom;
            this.game.add.tween(leftTree).to({ angle: 8 }, 5000, Phaser.Easing.Linear.None, true, null, Infinity, true);
            midGrdLayer.add(leftTree);
            const rightTree = new Phaser.Image(this.game, 0, 0, 'tree-tilt-left');
            ScaleUtil_17.scaleToBounds(rightTree, this.game.width * 0.3, this.game.height * 0.3);
            rightTree.anchor.set(0.87, 0.92);
            rightTree.left = midGrd.left + midGrd.width * 0.7;
            rightTree.bottom = midGrd.top + midGrd.height * 0.865;
            this.game.add.tween(rightTree).to({ angle: -8 }, 4500, Phaser.Easing.Linear.None, true, null, Infinity, true);
            midGrdLayer.add(rightTree);
            this.add.existing(midGrdLayer);
            /////////////////////////////////////////////////////////
            // 2 Lava flows which animate and change on eruption
            /////////////////////////////////////////////////////////
            this.lavaFlows = [];
            const lavaFlowSmall = new LavaFlow_1.LavaFlow(this.game, this.game.height * 0.02);
            ScaleUtil_17.scaleToBounds(lavaFlowSmall, this.game.width * 0.484, this.game.height * 0.085);
            lavaFlowSmall.centerX = this.game.world.centerX;
            lavaFlowSmall.top = this.game.height * 0.71;
            this.add.existing(lavaFlowSmall);
            this.lavaFlows.unshift(lavaFlowSmall); // pushes the item into the start of the array
            const lavaFlowMed = new LavaFlow_1.LavaFlow(this.game, this.game.height * 0.015);
            ScaleUtil_17.scaleToBounds(lavaFlowMed, this.game.width * 0.724, this.game.height * 0.12);
            lavaFlowMed.centerX = this.game.world.centerX;
            lavaFlowMed.top = this.game.height * 0.76;
            this.add.existing(lavaFlowMed);
            this.lavaFlows.unshift(lavaFlowMed); // pushes the item into the start of the array
            /////////////////////////////////////////////////////////
            // FRONTDINO LAYER
            // Includes: Front Mini-dinosaurs 
            /////////////////////////////////////////////////////////
            const frontDinoLayer = new Phaser.Group(this.game);
            const frontDinoPositions = miniDinoPositions[MiniDinoPos.FRONT];
            for (let i = 0; i < frontDinoPositions.length; i++) {
                const miniDino = new MiniDino_1.MiniDino(this.game);
                miniDino.x = frontDinoPositions[i][0];
                miniDino.y = frontDinoPositions[i][1];
                this.miniDinos.push(miniDino);
                frontDinoLayer.add(miniDino);
            }
            this.add.existing(frontDinoLayer);
            /////////////////////////////////////////////////////////
            // FOREGROUND
            // Includes: Ground
            /////////////////////////////////////////////////////////
            const fg = new Phaser.Image(this.game, 0, 0, 'ground-1');
            ScaleUtil_17.scaleToBounds(fg, this.game.width, this.game.height * 0.165);
            fg.centerX = this.game.world.centerX;
            fg.top = this.game.height * 0.874;
            this.add.existing(fg);
            /////////////////////////////////////////////////////////
            // Largest lava flow which animate
            /////////////////////////////////////////////////////////
            const lavaFlowLarge = new LavaFlow_1.LavaFlow(this.game, this.game.height * 0.02);
            ScaleUtil_17.scaleToBounds(lavaFlowLarge, this.game.width, this.game.height * 0.165);
            lavaFlowLarge.centerX = this.game.world.centerX;
            lavaFlowLarge.bottom = this.game.height;
            this.add.existing(lavaFlowLarge);
            this.lavaFlows.unshift(lavaFlowLarge); // pushes the item into the start of the array
            /////////////////////////////////////////////////////////
            // EFFECTS LAYER
            /////////////////////////////////////////////////////////
            this.miniDinos.forEach((miniDino) => {
                miniDino.hide();
            }, this);
            this.puffMinMaxXPos = [
                this.game.width * 0.35,
                this.game.width * 0.65 // max centerX pos
            ];
            this.puffMinMaxYPos = [
                this.mainVolcano.top - this.mainVolcano.width * 0.1,
                this.mainVolcano.top + this.mainVolcano.width * 0.1 // max centerY pos
            ];
            this.puffLayer = new Phaser.Group(this.game);
            this.add.existing(this.puffLayer);
            // Tutorial arrows
            // Needs to appear underneath balloons
            this.tutorialArrowRight = new Phaser.Image(this.game, 0, 0, 'arrow-right');
            ScaleUtil_17.scaleToBounds(this.tutorialArrowRight, this.game.width * 0.5, this.game.height);
            this.tutorialArrowRight.left = this.game.width * 0.5;
            this.tutorialArrowRight.bottom = this.mainVolcano.top;
            this.tutorialArrowRight.alpha = 0;
            this.add.existing(this.tutorialArrowRight);
            this.tutorialArrowLeft = new Phaser.Image(this.game, 0, 0, 'arrow-right');
            ScaleUtil_17.scaleToBounds(this.tutorialArrowLeft, this.game.width * 0.5, this.game.height);
            this.tutorialArrowLeft.scale.x *= -1;
            this.tutorialArrowLeft.left = this.game.width * 0.5;
            this.tutorialArrowLeft.bottom = this.mainVolcano.top;
            this.tutorialArrowLeft.alpha = 0;
            this.add.existing(this.tutorialArrowLeft);
            /////////////////////////////////////////////////////////
            // BALLOONS LAYER
            /////////////////////////////////////////////////////////
            this.balloonLayer = new Phaser.Group(this.game);
            this.add.existing(this.balloonLayer);
            /////////////////////////////////////////////////////////
            // TRIXIE LAYER
            /////////////////////////////////////////////////////////
            this.trixieEmote = new TrixieEmote_1.TrixieEmote(this.game);
            this.trixieEmote.right = this.game.width * 1.05;
            this.trixieEmote.bottom = this.game.height * 1.05;
            this.add.existing(this.trixieEmote);
            // Score and timer, bar along top of screen
            const topBar = new Phaser.Graphics(this.game);
            topBar.beginFill(0x042a5b, 1);
            topBar.drawRect(0, 0, this.game.width, this.game.height * 0.08);
            topBar.endFill();
            topBar.inputEnabled = true;
            this.add.existing(topBar);
            const timerIcon = new Phaser.Image(this.game, 0, 0, 'timer-icon');
            ScaleUtil_17.scaleToBounds(timerIcon, this.game.width * 0.1, this.game.width * 0.1);
            timerIcon.left = this.game.width * 0.01;
            timerIcon.centerY = topBar.height * 0.5;
            this.add.existing(timerIcon);
            this.timeLeftInTurnCounter = new Phaser.Text(this.game, 0, 0, '0', {
                font: Game_24.Game.DEFAULT_FONT,
                fontSize: Game_24.Game.FONT_SIZE_LARGE,
                fill: 'white',
                stroke: Game_24.Game.TEXT_COLOR_BLUE,
                strokeThickness: Game_24.Game.TEXT_STROKE_THICKNESS_LARGE,
            });
            this.timeLeftInTurnCounter.left = timerIcon.right + this.game.width * 0.005;
            this.timeLeftInTurnCounter.centerY = timerIcon.centerY;
            this.add.existing(this.timeLeftInTurnCounter);
            const scoreIcon = new Phaser.Image(this.game, 0, 0, 'score-icon');
            ScaleUtil_17.scaleToBounds(scoreIcon, this.game.width * 0.13, this.game.width * 0.13);
            scoreIcon.right = this.game.width - this.game.width * 0.01;
            scoreIcon.centerY = topBar.height * 0.5;
            this.add.existing(scoreIcon);
            this.scoreCounter = new Phaser.Text(this.game, 0, 0, '0', {
                font: Game_24.Game.DEFAULT_FONT,
                fontSize: Game_24.Game.FONT_SIZE_LARGE,
                fill: 'white',
                stroke: Game_24.Game.TEXT_COLOR_BLUE,
                strokeThickness: Game_24.Game.TEXT_STROKE_THICKNESS_LARGE,
                boundsAlignH: 'right',
            });
            this.scoreCounter.setTextBounds(0, 0, scoreIcon.left - this.game.width * 0.005, 0);
            this.scoreCounter.x = 0;
            this.scoreCounter.centerY = scoreIcon.centerY;
            this.add.existing(this.scoreCounter);
            /////////////////////////////////////////////////////////
            // Adding infinite updateable shake effect to all bg items
            /////////////////////////////////////////////////////////
            const miniVolcanoBgTween = this.game.add.tween(miniVolcanoBg).to({ y: miniVolcanoBg.y + this.game.height * 0.003 }, 250, Phaser.Easing.Linear.None, true, null, Infinity, true);
            const midGrdLayerTween = this.game.add.tween(midGrdLayer).to({ y: midGrdLayer.y + this.game.height * 0.004 }, 200, Phaser.Easing.Linear.None, true, null, Infinity, true);
            const fgTween = this.game.add.tween(fg).to({ y: fg.y + this.game.height * 0.007 }, 150, Phaser.Easing.Linear.None, true, null, Infinity, true);
            /////////////////////////////////////////////////////////
            // Track all shakeable components and tweens
            /////////////////////////////////////////////////////////
            this.bgTweensToUpdate = [
                fgTween,
                midGrdLayerTween,
                miniVolcanoBgTween,
            ];
            this.componentsToShake = [
                lavaFlowSmall,
                lavaFlowMed,
                lavaFlowLarge,
                this.mainVolcano,
                this.trixieEmote
            ];
            // Instructions
            this.curtain = new Curtain_6.Curtain(this.game, true);
            this.add.existing(this.curtain);
            this.tutorialTopMessageBar = new MessageBar_10.MessageBar(this.game, LocalizationManager_20.LocalizationManager.Instance.text("volcano.instructions_intro_1"), 30);
            this.tutorialTopMessageBar.centerX = this.game.width * 0.5;
            this.tutorialTopMessageBar.bottom = 0;
            this.add.existing(this.tutorialTopMessageBar);
            this.tutorialBottomMessageBar = new MessageBar_10.MessageBar(this.game, LocalizationManager_20.LocalizationManager.Instance.text("volcano.instructions_intro_2"));
            this.tutorialBottomMessageBar.centerX = this.game.width * 0.5;
            this.tutorialBottomMessageBar.top = this.game.height;
            this.add.existing(this.tutorialBottomMessageBar);
            this.instructionImage = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.45, 'instruction-graphic-volcano');
            ScaleUtil_17.scaleToBounds(this.instructionImage, this.game.width * 0.8, this.game.height * 0.225);
            this.instructionImage.alpha = 0;
            this.instructionImage.anchor.set(0.5);
            this.add.existing(this.instructionImage);
            this.headerBar = new HeaderBar_5.HeaderBar(this.game, LocalizationManager_20.LocalizationManager.Instance.text("common.how_to_play"));
            this.headerBar.centerX = this.game.width * 0.5;
            this.headerBar.bottom = 0;
            this.add.existing(this.headerBar);
            // Pass and play
            this.passAndPlayAnimation = new PassAndPlayShowable_4.PassAndPlayShowable(this.game);
            this.passAndPlayAnimation.x = this.game.width * 0.5;
            this.passAndPlayAnimation.y = this.game.height * 0.5;
            this.add.existing(this.passAndPlayAnimation);
            this.readyButton = new TextArrowButton_7.TextArrowButton(this.game, LocalizationManager_20.LocalizationManager.Instance.text('common.ready').toUpperCase());
            this.readyButton.centerX = this.game.width * 0.5;
            this.readyButton.centerY = this.game.height * 0.785;
            this.readyButton.alpha = 0;
            this.readyButton.startPulse();
            this.add.existing(this.readyButton);
            this.currentGameState = GameState.LOADING;
            for (let i = 0; i < this.players.length; i++) {
                this.score[i] = 0;
            }
            this.dismissOverlay();
            if (ToyStoryConfig_14.ToyStoryConfig.Instance.skipGames) {
                for (let i = 0; i < this.players.length; i++) {
                    this.score[i] = 10;
                }
                this.endGame(this.score);
            }
            else {
                this.play();
            }
        }
        play() {
            return __awaiter(this, void 0, void 0, function* () {
                while (this.currentPlayerIndex < this.players.length) {
                    yield this.showPassAndPlay();
                    this.trackGameStartTime();
                    yield this.doTurn();
                }
                this.endGame(this.score);
            });
        }
        doTurn() {
            return __awaiter(this, void 0, void 0, function* () {
                this.currentGameState = GameState.PLAYING;
                this.timeLeftInTurn = this.config.maxTimePerPlayerTurnMS;
                // Reset balloon spawn timer from last turn
                this.balloonSpawnRateMs = this.difficultyConfig.spawnRate;
                // Tween the balloon spawn rate, so that spawning speeds up towards the end of the round
                this.game.add.tween(this).to({ balloonSpawnRateMs: this.balloonSpawnRateMs * 0.35 }, this.config.maxTimePerPlayerTurnMS, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.tutorialArrowLeft).to({ alpha: 1 }, 400, Phaser.Easing.Linear.None, true, 0, 5, true);
                this.game.add.tween(this.tutorialArrowRight).to({ alpha: 1 }, 400, Phaser.Easing.Linear.None, true, 0, 5, true);
                // Show eruption after player turn concludes
                yield PromiseUtil_38.PromiseUtil.time(this.game, this.config.maxTimePerPlayerTurnMS);
                // stop ballon dragging and spawning
                for (let child of this.balloonLayer.children) {
                    const balloon = child;
                    balloon.stopDrag();
                    balloon.input.disableDrag();
                    balloon.inputEnabled = false;
                }
                this.balloonsSpawning = false;
                this.currentGameState = GameState.TRANSITIONING;
                yield this.setVolatilityState(10);
                this.currentPlayerIndex++;
                this.scoreCounter.text = "0";
            });
        }
        update() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("update").call(this);
                // Update countdown display
                const timeLeft = Math.max((this.timeLeftInTurn / 1000), 0).toFixed(0);
                if (this.timeLeftInTurnCounter.text !== timeLeft) {
                    this.timeLeftInTurnCounter.text = timeLeft;
                    // Play countdown sound for last 4 secs
                    if (this.timeLeftInTurn / 1000 < TrixieState.VOLATILITY_COUNTDOWN) {
                        this.game.sound.play('countdown-beep');
                    }
                }
                if (this.balloonsSpawning) {
                    this.timeTilNextBalloonSpawn -= this.game.time.elapsedMS;
                    // If balloon timer < 0, spawn balloon
                    if (this.timeTilNextBalloonSpawn <= 0) {
                        // spawn balloon
                        this.spawnBalloon();
                        this.timeTilNextBalloonSpawn = this.balloonSpawnRateMs;
                    }
                }
                // If we're playing, decrement turn/balloon timers, spawn balloons
                // Check for balloon/volcano collision
                if (this.currentGameState === GameState.PLAYING) {
                    this.timeLeftInTurn -= this.game.time.elapsedMS;
                    this.volatility = ((this.config.maxTimePerPlayerTurnMS - this.timeLeftInTurn) / this.config.maxTimePerPlayerTurnMS) * 9;
                    if (Math.floor(this.volatility) !== this.displayedVolatility) {
                        this.setVolatilityState(Math.floor(this.volatility));
                    }
                    // check collision between balloons and the volcano insertion hitbox
                    this.physics.arcade.overlap(this.balloonLayer, this.volcanoInsertionHitBox, (volcanoBoundary, balloon) => __awaiter(this, void 0, void 0, function* () {
                        if (balloon.isBeingDragged) {
                            balloon.input.draggable = false;
                            balloon.inputEnabled = false;
                            this.balloonLayer.remove(balloon);
                            this.balloonInsertingLayer.add(balloon);
                            balloon.tweenIntoVolcano(this.mainVolcano.centerX);
                            this.score[this.currentPlayerIndex] += balloon.pointsAwarded;
                            this.scoreCounter.text = this.score[this.currentPlayerIndex].toString();
                            this.playSuccess();
                        }
                    }), null, this);
                }
            });
        }
        popAllBalloons() {
            return __awaiter(this, void 0, void 0, function* () {
                for (let child of this.balloonLayer.children) {
                    const balloon = child;
                    this.playBalloonPop(balloon);
                }
                this.balloonLayer.children = [];
            });
        }
        playBalloonPop(balloon) {
            return __awaiter(this, void 0, void 0, function* () {
                const animation = new ResultAnimations_1.BurstAnimation(this.game, balloon.balloonType);
                this.game.sound.play(`balloon-pop-${this.game.rnd.integerInRange(1, 3)}`);
                balloon.visible = false;
                yield animation.show(new Phaser.Point(balloon.centerX, balloon.centerY));
                balloon.destroy();
            });
        }
        showPassAndPlay() {
            return __awaiter(this, void 0, void 0, function* () {
                this.currentGameState = GameState.TRANSITIONING;
                this.trackGamePassAndPlay();
                if (!this.curtain.isShowing) {
                    yield this.curtain.show(true);
                }
                // Communicate the device handover
                // Set text on pass and play animation
                const passAndPlayText = LocalizationManager_20.LocalizationManager.Instance.textWithReplacements('target_toss.instructions_pass', { player_name: this.players[this.currentPlayerIndex][0].name }).toUpperCase();
                this.passAndPlayAnimation.text = passAndPlayText;
                // Show pass and play animation
                yield this.passAndPlayAnimation.show();
                this.readyButton.visible = true;
                this.game.add.tween(this.readyButton).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true);
                this.setVolatilityState(0);
                // Wait for input
                yield PromiseUtil_38.PromiseUtil.signal(this.readyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                // Hide pass and play, ready button
                this.game.add.tween(this.readyButton).to({ alpha: 0 }, 250, Phaser.Easing.Linear.None, true);
                yield this.passAndPlayAnimation.hide();
                // Start spawning balloons
                yield this.showThenHideInstructions();
                this.balloonsSpawning = true;
            });
        }
        showThenHideInstructions() {
            return __awaiter(this, void 0, void 0, function* () {
                this.trackGameInstructions();
                if (!this.curtain.isShowing) {
                    yield this.curtain.show(true);
                }
                const durationMs = 500;
                // Reposition and fade in all the instruction elements
                const yPadding = this.game.height * 0.015;
                yield Promise.all([
                    PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(this.headerBar)
                        .to({ top: 0 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialTopMessageBar)
                        .to({ bottom: this.instructionImage.top - yPadding }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialBottomMessageBar)
                        .to({ top: this.instructionImage.bottom + yPadding }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionImage)
                        .to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(this.readyButton)
                        .to({ alpha: 1 }, durationMs, Phaser.Easing.Linear.None, true)),
                ]);
                yield PromiseUtil_38.PromiseUtil.signal(this.readyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                yield Promise.all([
                    PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(this.headerBar)
                        .to({ bottom: 0 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialTopMessageBar)
                        .to({ bottom: 0 }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialBottomMessageBar)
                        .to({ top: this.game.height }, durationMs, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(this.instructionImage)
                        .to({ alpha: 0 }, durationMs * 0.5, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(this.readyButton)
                        .to({ alpha: 0 }, durationMs * 0.5, Phaser.Easing.Linear.None, true)),
                ]);
                this.readyButton.visible = false;
                this.curtain.hide();
            });
        }
        setVolatilityState(volatilityState) {
            return __awaiter(this, void 0, void 0, function* () {
                this.displayedVolatility = volatilityState;
                let shakeFactor = 1 + (volatilityState % 10) / 5;
                if (volatilityState === 0) {
                    shakeFactor = 0;
                }
                // update all shakes
                this.componentsToShake.forEach((component) => {
                    component.updateShake(shakeFactor);
                });
                this.tiledBg.changeState(BackgroundFrame.NEUTRAL);
                switch (volatilityState) {
                    case 10: { // ERUPTION
                        this.setShakingBg(0, shakeFactor);
                        this.game.sound.play('volcano-explode');
                        yield Promise.all([
                            this.trixieEmote.hide(),
                            this.setVisibleMiniDinos(0),
                            this.setShakingLavaFlows(3),
                            this.mainVolcano.playEruptAnimation(),
                            this.tiledBg.changeState(BackgroundFrame.ERUPTING, 500),
                            this.showPuffs(10),
                            PromiseUtil_38.PromiseUtil.time(this.game, 2000)
                        ]);
                        this.volatility = 0;
                        break;
                    }
                    case 9: {
                        this.trixieEmote.show(TrixieEmote_1.EmoteState.ABSOLUTELY_TERRIFIED);
                        this.mainVolcano.setVisibleLavaBalloons(3);
                        this.mainVolcano.startShake();
                        this.setShakingLavaFlows(3);
                        this.setVisibleMiniDinos(0);
                        this.showPuffs(3);
                        this.setShakingBg(3, shakeFactor);
                        break;
                    }
                    case 8: {
                        this.trixieEmote.show(TrixieEmote_1.EmoteState.WORRIED);
                        this.mainVolcano.setVisibleLavaBalloons(3);
                        this.mainVolcano.startShake();
                        this.setShakingLavaFlows(3);
                        this.setVisibleMiniDinos(0);
                        this.showPuffs(3);
                        this.setShakingBg(3, shakeFactor);
                        break;
                    }
                    case 7: {
                        this.trixieEmote.show(TrixieEmote_1.EmoteState.NEUTRAL);
                        this.mainVolcano.setVisibleLavaBalloons(3);
                        this.mainVolcano.startShake();
                        this.setShakingLavaFlows(3);
                        this.setVisibleMiniDinos(0);
                        this.setShakingBg(3, shakeFactor);
                        break;
                    }
                    case 6: {
                        this.trixieEmote.hide();
                        this.setShakingLavaFlows(3);
                        this.mainVolcano.startShake();
                        this.mainVolcano.setVisibleLavaBalloons(3);
                        this.showPuffs(1, this.game.width * 0.556, this.game.height * 0.55, 0); // spawns a puff with 0ms delay
                        this.setVisibleMiniDinos(0);
                        this.setShakingBg(3, shakeFactor);
                        break;
                    }
                    case 5: {
                        this.trixieEmote.hide();
                        this.setShakingLavaFlows(3);
                        this.mainVolcano.startShake();
                        this.mainVolcano.setVisibleLavaBalloons(2);
                        this.showPuffs(1, this.game.width * 0.5, this.game.height * 0.555, 0); // spawns a puff with 0ms delay
                        this.setVisibleMiniDinos(0);
                        this.setShakingBg(3, shakeFactor);
                        break;
                    }
                    case 4: {
                        this.trixieEmote.hide();
                        this.setShakingLavaFlows(3);
                        this.mainVolcano.startShake();
                        this.mainVolcano.setVisibleLavaBalloons(1);
                        this.showPuffs(1, this.game.width * 0.45, this.game.height * 0.55, 0); // spawns a puff with 0ms delay
                        this.setVisibleMiniDinos(1);
                        this.setShakingBg(3, shakeFactor);
                        break;
                    }
                    case 3: {
                        this.trixieEmote.hide();
                        this.mainVolcano.setVisibleLavaBalloons(0);
                        this.mainVolcano.startShake();
                        this.setShakingLavaFlows(3);
                        this.setVisibleMiniDinos(1);
                        this.setShakingBg(3, shakeFactor);
                        break;
                    }
                    case 2: {
                        this.trixieEmote.hide();
                        this.mainVolcano.setVisibleLavaBalloons(0);
                        this.mainVolcano.stopShake();
                        this.setShakingLavaFlows(2);
                        this.setVisibleMiniDinos(2);
                        this.setShakingBg(2, shakeFactor);
                        break;
                    }
                    case 1: {
                        this.trixieEmote.hide();
                        this.mainVolcano.setVisibleLavaBalloons(0);
                        this.mainVolcano.stopShake();
                        this.setShakingLavaFlows(1);
                        this.setVisibleMiniDinos(2);
                        this.setShakingBg(1, shakeFactor);
                        break;
                    }
                    // nothing is moving when VolatilityState is 0
                    case 0:
                    default: {
                        this.trixieEmote.hide();
                        this.mainVolcano.setVisibleLavaBalloons(0);
                        this.mainVolcano.stopShake();
                        this.setShakingLavaFlows(0);
                        this.setVisibleMiniDinos(3);
                        this.setShakingBg(0, shakeFactor);
                        break;
                    }
                }
            });
        }
        /**
         * Sets the total number of shaking bgs to @param count,
         * Starts shaking from the bottommost background(fg)
         * @param count = Total number of shaking backgrounds on screen
         */
        setShakingBg(count, timeScale) {
            for (let i = 0; i < Math.min(count, this.bgTweensToUpdate.length); i++) {
                this.updateTween(this.bgTweensToUpdate[i], timeScale);
            }
            for (let i = count; i < this.bgTweensToUpdate.length; i++) {
                this.updateTween(this.bgTweensToUpdate[i], 0);
            }
        }
        updateTween(tween, timeScale) {
            tween.timeScale = timeScale;
        }
        /**
         * Sets the total number of shaking lava flows to @param count,
         * Starts shaking from the bottommost (largest) lava flow
         * @param count = Total number of shaking lava flows on screen
         */
        setShakingLavaFlows(count) {
            for (let i = 0; i < Math.min(count, this.lavaFlows.length); i++) {
                this.lavaFlows[i].startShake();
            }
            for (let i = count; i < this.lavaFlows.length; i++) {
                this.lavaFlows[i].stopShake();
            }
        }
        /**
         * Randomly spawns/hides mini dinos based on @param count
         * @param count = total number of mini dinos showing on screen
         */
        setVisibleMiniDinos(count) {
            Phaser.ArrayUtils.shuffle(this.miniDinos);
            for (let i = 0; i < Math.min(count, this.miniDinos.length); i++) {
                const rndDelayMS = this.game.rnd.integerInRange(0, 2000);
                this.miniDinos[i].show(rndDelayMS);
            }
            for (let i = count; i < this.miniDinos.length; i++) {
                const rndDelayMS = this.game.rnd.integerInRange(0, 2000);
                this.miniDinos[i].hide(rndDelayMS);
            }
        }
        /**
         * Randomly creates and shows number of puffs equal to @param count
         * @param count = number of puffs to create and show
         */
        showPuffs(count, x, y, delayMS) {
            return __awaiter(this, void 0, void 0, function* () {
                for (let i = 0; i < count; i++) {
                    const puff = new Puff_1.Puff(this.game);
                    puff.x = this.game.rnd.integerInRange(this.puffMinMaxXPos[0], this.puffMinMaxXPos[1]);
                    puff.y = this.game.rnd.integerInRange(this.puffMinMaxYPos[0], this.puffMinMaxYPos[1]);
                    // overwrite the puff's x and/or y if they are passed in as a parameter
                    if (x) {
                        puff.centerX = x;
                    }
                    if (y) {
                        puff.centerY = y;
                    }
                    const puffDelayMS = (delayMS || delayMS === 0) ? delayMS : this.game.rnd.integerInRange(0, 800);
                    this.puffLayer.add(puff);
                    puff.emit(puffDelayMS);
                }
            });
        }
        spawnBalloon() {
            // Orange balloon is good, others bad
            let balloonType;
            let chance;
            if (this.numBalloonsSinceHazard > this.config.hazardSpawnChances.length) {
                chance = 1;
            }
            else {
                chance = this.config.hazardSpawnChances[this.numBalloonsSinceHazard];
            }
            if (chance < this.game.rnd.frac()) {
                if (this.difficultyConfig.hasRed && this.game.rnd.frac() <= this.config.redSpawnChance) {
                    balloonType = TrixieConfig_3.BalloonType.RED;
                }
                else {
                    balloonType = TrixieConfig_3.BalloonType.ORANGE;
                }
                this.numBalloonsSinceHazard++;
            }
            else {
                balloonType = TrixieConfig_3.BalloonType.HAZARD;
                this.numBalloonsSinceHazard = 0;
            }
            const balloon = this.makeBalloon(balloonType);
            if (balloonType === TrixieConfig_3.BalloonType.HAZARD) {
                balloon.events.onInputDown.addOnce(() => {
                    this.popAllBalloons();
                });
            }
            const rndXPos = this.game.rnd.integerInRange(0, this.game.width);
            balloon.x = rndXPos;
            balloon.top = this.game.height;
            // fix for body size and offset positioning found from:
            // http://www.html5gamedevs.com/topic/25405-center-arcade-circle-body-on-sprite-and-possible-bugs/
            const bodyRadius = balloon.width * 0.5;
            const offSet = (-bodyRadius) + (bodyRadius / balloon.scale.x);
            balloon.body.setCircle(bodyRadius, offSet, offSet);
            this.balloonLayer.add(balloon);
        }
        makeBalloon(balloonType) {
            const balloon = new Balloon_1.Balloon(this.game, 0, this.game.height, this.config.balloonAssetKeys[balloonType], // assetkey
            this.config.balloonPoints[balloonType], // points when inserted into volcano,
            this.config.balloonBounce, // bounce amount of balloons (between 0 to 1)
            this.difficultyConfig.balloonGravity[balloonType], // bounce amount of balloons (between 0 to 1)
            balloonType);
            ScaleUtil_17.scaleToBounds(balloon, this.game.width * this.config.balloonMaxWidthGameScreenPercent[balloonType], this.game.height * this.config.balloonMaxHeightGameScreenPercent[balloonType]);
            return balloon;
        }
        playSuccess() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play('balloon-correct');
                const animation = new ResultAnimations_1.SuccessAnimation(this.game);
                yield animation.show(new Phaser.Point(this.game.width * 0.5, this.game.height * 0.55));
            });
        }
        endGame(score) {
            return __awaiter(this, void 0, void 0, function* () {
                // Save off total score
                const teamScore = score.reduce((a, b) => a + b, 0);
                // Send score to achievement manager, to see if we get an achievement
                yield GameController_21.GameController.Instance.achievementManager.onTrixieEndGame(teamScore);
                TSMAnalyticsManager_8.TSMAnalyticsManager.Instance.onGameScore(this.stateName, teamScore);
                this.curtain.show();
                // Show top bar first
                this.tutorialTopMessageBar.text = LocalizationManager_20.LocalizationManager.Instance.text("volcano.results");
                yield PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(this.tutorialTopMessageBar).to({ top: this.game.height * 0.05 }, 300, Phaser.Easing.Sinusoidal.In, true));
                const scoreBarGroup = new Phaser.Group(this.game);
                let barPosition = this.game.height;
                // Show scores
                for (let i = 0; i < this.players.length; i++) {
                    let isHighScore = false;
                    // If this is the highest score, add a star to the score bar
                    if (score[i] === Math.max(...score)) {
                        isHighScore = true;
                    }
                    const scoreBar = new ScoreBar_1.ScoreBar(this.game, this.players[i][0].name, score[i], isHighScore);
                    scoreBar.top = barPosition;
                    scoreBarGroup.add(scoreBar);
                    barPosition = scoreBar.bottom;
                }
                // Tween results onto screen
                this.add.existing(scoreBarGroup);
                const screenCenter = this.tutorialTopMessageBar.bottom + (this.readyButton.top - this.tutorialTopMessageBar.bottom) * 0.5;
                yield PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(scoreBarGroup).to({ centerY: screenCenter }, 1000, Phaser.Easing.Sinusoidal.In, true));
                // Change the button text, show
                this.readyButton.text = LocalizationManager_20.LocalizationManager.Instance.text('common.finish').toUpperCase();
                this.readyButton.visible = true;
                yield PromiseUtil_38.PromiseUtil.tween(this.game, this.game.add.tween(this.readyButton).to({ alpha: 1 }, 250, Phaser.Easing.Linear.None, true));
                // Wait for button input
                yield PromiseUtil_38.PromiseUtil.signal(this.readyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                this.transitionTo(StateNames_17.StateNames.RESULTS);
                GameController_21.GameController.Instance.playerManager.resetPlayers();
                GameController_21.GameController.Instance.postScore(this.stateName, teamScore);
            });
        }
        shutdown() {
            this.currentGameState = null;
            this.players = null;
            this.scoreCounter = null;
            this.balloonLayer = null;
            this.balloonInsertingLayer = null;
            this.puffLayer = null;
            this.volcanoInsertionHitBox = null;
            this.tiledBg = null;
            this.trixieEmote = null;
            this.mainVolcano = null;
            this.miniDinos = null;
            this.lavaFlows = null;
            this.bgTweensToUpdate = null;
            this.componentsToShake = null;
            this.timeLeftInTurnCounter = null;
            this.passAndPlayAnimation = null;
            this.readyButton = null;
            this.curtain = null;
            this.tutorialTopMessageBar = null;
            this.tutorialBottomMessageBar = null;
            this.instructionImage = null;
            this.headerBar = null;
            this.tutorialArrowLeft = null;
            this.tutorialArrowRight = null;
            this.game.cache.removeImage('balloon-orange');
            this.game.cache.removeImage('balloon-orange-pop');
            this.game.cache.removeImage('balloon-red');
            this.game.cache.removeImage('balloon-red-pop');
            this.game.cache.removeImage('balloon-hazard');
            this.game.cache.removeImage('balloon-hazard-pop');
            // bg scenes
            for (let i = 1; i <= 5; i++) {
                this.game.cache.removeImage(`bg-scene-${i}`);
            }
            // fg layers 
            this.game.cache.removeImage('mini-volcanoes');
            this.game.cache.removeImage('ground-1');
            this.game.cache.removeImage('ground-2');
            this.game.cache.removeImage('tree-tilt-right');
            this.game.cache.removeImage('tree-tilt-left');
            this.game.cache.removeImage('lava-flow');
            this.game.cache.removeImage('mini-dino');
            // volcano
            this.game.cache.removeImage('volcano-base');
            // trixie emote states
            this.game.cache.removeImage('trixie-emotes');
            // effects
            this.game.cache.removeImage('lava-balloon');
            this.game.cache.removeImage('steam');
            this.game.cache.removeImage('puff');
            this.game.cache.removeImage('instruction-graphic-volcano');
            // UI
            this.game.cache.removeImage('arrow-right');
            this.game.cache.removeImage('score-icon');
            this.game.cache.removeImage('rank-star');
            // Success/failure UI
            this.game.cache.removeImage('success-ray');
            this.game.cache.removeImage('star-green');
            this.game.cache.removeImage('star-pink');
            this.game.cache.removeImage('star-blue');
            this.game.cache.removeImage('balloon-pop');
            // Success/failure UI
            this.game.load.image('balloon-pop', 'assets/minigame/trixie/effects/pop.png');
            // Audio
            this.game.cache.removeSound('balloon-correct');
            this.game.cache.removeSound('balloon-grab-1');
            this.game.cache.removeSound('balloon-grab-2');
            this.game.cache.removeSound('balloon-grab-3');
            this.game.cache.removeSound('balloon-grab-4');
            this.game.cache.removeSound('balloon-grab-5');
            this.game.cache.removeSound('balloon-pop-1');
            this.game.cache.removeSound('balloon-pop-2');
            this.game.cache.removeSound('balloon-pop-3');
            this.game.cache.removeSound('countdown-beep');
            this.game.cache.removeSound('volcano-explode');
            super.shutdown();
        }
    }
    TrixieState.VOLATILITY_COUNTDOWN = 5;
    exports.TrixieState = TrixieState;
});
define("component/hub/BasicHubNode", ["require", "exports", "component/hub/HubNode", "switchback-common/util/PromiseUtil"], function (require, exports, HubNode_2, PromiseUtil_39) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class BasicHubNode extends HubNode_2.HubNode {
        constructor(game, characterTag, nodeTag, stateName, boxTint) {
            super(game, stateName);
            this.base = new Phaser.Image(game, 0, 0, nodeTag, this.getFrameForLevel(this.visualLevel));
            this.base.anchor.set(0.5, 1);
            this.base.inputEnabled = true;
            this.base.input.pixelPerfectClick = true;
            this.add(this.base);
            this.characterImage = new Phaser.Sprite(this.game, 0, 0, characterTag);
            this.characterImage.anchor.set(0.5, 1);
            this.characterImage.scale.x = this.base.scale.x;
            this.characterImage.scale.y = this.base.scale.y;
            this.characterImage.inputEnabled = true;
            this.characterImage.input.pixelPerfectClick = true;
            this.add(this.characterImage);
            if (!this.isViewUnlocked) {
                this.box = new Phaser.Sprite(game, 0, 0, 'node-box', 0);
                this.box.tint = boxTint;
                this.box.animations.add('reveal', [0, 1, 2, 3, 4, 5, 6, 7], 12);
                this.box.anchor.set(0.5, 1);
                this.box.width = this.base.width;
                this.box.scale.y = this.box.scale.x;
                this.add(this.box);
                this.base.alpha = 0;
                this.characterImage.alpha = 0;
            }
        }
        onNodeClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                this.playGameSignal.dispatch(this.stateName, true);
            });
        }
        // TODO: Actual animation
        unlock() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_39.PromiseUtil.animation(this.box.animations.play('reveal'));
                this.box.destroy();
                this.base.alpha = 1;
                this.characterImage.alpha = 1;
                _super("unlock").call(this);
            });
        }
        /**
         * Perform upgrade animation
         * Level 0 is locked - you shouldn't be here
         * Level 1 is frame 0 - going to frame 1
         * Level 2 is frame 1 - going to frame 3
         * Level 3 is frame 2 - can't upgrade any further
         * TODO: Actual animation
         */
        upgradeOne() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                // I know this is a little redundant but I wanted to explicitly
                // show that it goes +1 level then -1 (for the zero-indexed frames)
                const levelToGoTo = Math.min(this.maxLevel, this.visualLevel + 1);
                const frameToGoTo = this.getFrameForLevel(levelToGoTo);
                this.base.frame = frameToGoTo;
                _super("upgradeOne").call(this);
            });
        }
        getFrameForLevel(level) {
            return Math.max(0, Math.min(this.maxLevel, level) - 1);
        }
    }
    exports.BasicHubNode = BasicHubNode;
});
define("component/hub/WoodyHubNode", ["require", "exports", "component/hub/BasicHubNode", "StateNames"], function (require, exports, BasicHubNode_1, StateNames_18) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class WoodyHubNode extends BasicHubNode_1.BasicHubNode {
        constructor(game) {
            super(game, "node-char-woody", "node-woody", StateNames_18.StateNames.GAME_WOODY_LASSO, 0xffdc4d);
        }
    }
    exports.WoodyHubNode = WoodyHubNode;
});
define("component/hub/HammHubNode", ["require", "exports", "component/hub/BasicHubNode", "StateNames"], function (require, exports, BasicHubNode_2, StateNames_19) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HammHubNode extends BasicHubNode_2.BasicHubNode {
        constructor(game) {
            super(game, "node-char-hamm", "node-hamm", StateNames_19.StateNames.GAME_HAMM, 0xff90a6);
        }
    }
    exports.HammHubNode = HammHubNode;
});
define("component/hub/FlyingTossersHubNode", ["require", "exports", "component/hub/BasicHubNode", "StateNames"], function (require, exports, BasicHubNode_3, StateNames_20) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class FlyingTossersHubNode extends BasicHubNode_3.BasicHubNode {
        constructor(game) {
            super(game, "node-char-buzz", "node-buzz", StateNames_20.StateNames.GAME_FLYING_TOSSERS, 0x5d49cf);
        }
    }
    exports.FlyingTossersHubNode = FlyingTossersHubNode;
});
define("component/hub/SargeHubNode", ["require", "exports", "component/hub/BasicHubNode", "StateNames"], function (require, exports, BasicHubNode_4, StateNames_21) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SargeHubNode extends BasicHubNode_4.BasicHubNode {
        constructor(game) {
            super(game, "node-char-sarge", "node-sarge", StateNames_21.StateNames.GAME_SARGE, 0x3b8948);
        }
    }
    exports.SargeHubNode = SargeHubNode;
});
define("component/hub/VolcanoHubNode", ["require", "exports", "component/hub/BasicHubNode", "StateNames"], function (require, exports, BasicHubNode_5, StateNames_22) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class VolcanoHubNode extends BasicHubNode_5.BasicHubNode {
        constructor(game) {
            super(game, null, "node-trixie", StateNames_22.StateNames.GAME_TRIXIE, 0x69bdff); // TODO: Give trixie a character
        }
    }
    exports.VolcanoHubNode = VolcanoHubNode;
});
define("component/hub/StaticHubNode", ["require", "exports", "component/hub/HubNode", "logic/GameController", "switchback-common/util/PromiseUtil"], function (require, exports, HubNode_3, GameController_22, PromiseUtil_40) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class StaticHubNode extends HubNode_3.HubNode {
        constructor(game, nodeTag, stateName, boxTint = 0x4A9112) {
            super(game, stateName, 1);
            this.base = new Phaser.Image(game, 0, 0, nodeTag, GameController_22.GameController.Instance.state.getGameVisualLevel(stateName));
            this.base.anchor.set(0.5, 1);
            this.base.inputEnabled = this.isViewUnlocked;
            if (this.isViewUnlocked) {
                this.base.input.pixelPerfectClick = true;
            }
            this.add(this.base);
            if (!this.isViewUnlocked) {
                this.box = new Phaser.Sprite(game, 0, 0, 'node-box', 0);
                this.box.tint = boxTint;
                this.box.animations.add('reveal', [0, 1, 2, 3, 4, 5, 6, 7], 12);
                this.box.anchor.set(0.5, 1);
                this.box.width = this.base.width * 0.8;
                this.box.scale.y = this.box.scale.x;
                this.add(this.box);
                this.base.alpha = 0;
            }
        }
        unlock() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_40.PromiseUtil.animation(this.box.animations.play('reveal'));
                this.box.destroy();
                this.base.alpha = 1;
                _super("unlock").call(this);
            });
        }
    }
    exports.StaticHubNode = StaticHubNode;
});
define("component/hub/DressUpHubNode", ["require", "exports", "component/hub/StaticHubNode", "StateNames"], function (require, exports, StaticHubNode_1, StateNames_23) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DressUpHubNode extends StaticHubNode_1.StaticHubNode {
        constructor(game) {
            super(game, "node-rex", StateNames_23.StateNames.GAME_DRESS_UP, 0x7dbc1f);
        }
    }
    exports.DressUpHubNode = DressUpHubNode;
});
define("component/hub/DrawingHubNode", ["require", "exports", "component/hub/StaticHubNode", "StateNames"], function (require, exports, StaticHubNode_2, StateNames_24) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DrawingHubNode extends StaticHubNode_2.StaticHubNode {
        constructor(game) {
            super(game, "node-jessie", StateNames_24.StateNames.GAME_DRAWING, 0xbe0013);
        }
    }
    exports.DrawingHubNode = DrawingHubNode;
});
define("component/hub/HubHeader", ["require", "exports", "Game", "switchback-common/util/TextUtil"], function (require, exports, Game_25, TextUtil_17) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class HubHeader extends Phaser.Group {
        constructor(game, textString) {
            super(game);
            const bgPanel = new Phaser.Image(this.game, 0, 0, 'hub-header-panel');
            bgPanel.width = this.game.width;
            bgPanel.scale.y = bgPanel.scale.x;
            this.add(bgPanel);
            const text = new Phaser.Text(this.game, bgPanel.width * 0.15, bgPanel.height * 0.375, textString, {
                font: Game_25.Game.DEFAULT_FONT,
                fill: Game_25.Game.TEXT_COLOR_WHITE,
                fontSize: Game_25.Game.FONT_SIZE_LARGE
            });
            text.setShadow(Game_25.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_25.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_25.Game.TEXT_SHADOW_HARD, 0, true, true);
            TextUtil_17.TextUtil.scaleDownSingleLineTextToWidth(text, bgPanel.width * 0.6);
            text.anchor.set(0, 0.5);
            this.add(text);
        }
    }
    exports.HubHeader = HubHeader;
});
define("component/StageProgressMessageBar", ["require", "exports", "Game", "component/MessageBar", "switchback-common/util/ScaleUtil"], function (require, exports, Game_26, MessageBar_11, ScaleUtil_18) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class StageProgressMessageBar extends Phaser.Group {
        constructor(game, currentStageNum) {
            super(game);
            const contentGroup = new Phaser.Group(this.game);
            const min = Math.max(0, currentStageNum - 1);
            const max = min + 2;
            let xPos = -this.game.width * 0.25;
            const xInc = this.game.width * 0.25;
            for (let i = min; i <= max; i++) {
                // Figure out how we need to display this stage
                let textFill;
                let imgKey = 'stageicon_0'; // Unachieved icon
                if (i === currentStageNum) {
                    textFill = Game_26.Game.TEXT_COLOR_YELLOW;
                }
                else if (i > currentStageNum) {
                    textFill = Game_26.Game.TEXT_COLOR_DISABLED;
                }
                else {
                    textFill = Game_26.Game.TEXT_COLOR_WHITE;
                    imgKey = `stageicon_${Math.min(i + 1, 3)}`; // Achieved icon
                }
                // Create and size the icon representing the stage
                const stageIcon = new Phaser.Image(this.game, xPos, 0, imgKey);
                stageIcon.anchor.set(0.5, 0.9);
                ScaleUtil_18.scaleToBounds(stageIcon, this.game.width * 0.2, this.game.height * 0.1);
                // Draw lines for the middle connectors, and fade out the future lines.
                if (i !== max) {
                    const line = new Phaser.Image(this.game, xPos, stageIcon.centerY, 'stageicon_connector_base');
                    line.anchor.set(0, 0.5);
                    line.width = xInc;
                    contentGroup.add(line);
                    if (i >= currentStageNum) {
                        line.alpha = 0.5;
                    }
                }
                contentGroup.add(stageIcon);
                // If this is the target stage, show a highlighted overlay img
                if (i === currentStageNum) {
                    const highlightIcon = new Phaser.Image(this.game, xPos, 0, 'stageicon_highlighted');
                    highlightIcon.anchor = stageIcon.anchor;
                    ScaleUtil_18.scaleToBounds(highlightIcon, this.game.width * 0.2, this.game.height * 0.1);
                    contentGroup.add(highlightIcon);
                }
                // Display the stage number (0 indexed) below the icon
                const numberText = new Phaser.Text(this.game, xPos, 0, (i + 1).toString(), {
                    font: Game_26.Game.DEFAULT_FONT,
                    fontSize: Game_26.Game.FONT_SIZE_EXTRA_LARGE,
                    fill: textFill,
                    stroke: Game_26.Game.TEXT_COLOR_BLUE,
                    strokeThickness: Game_26.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                    align: 'center'
                });
                numberText.anchor.set(0.5, 0.1);
                contentGroup.add(numberText);
                xPos += xInc;
            }
            const containingBar = new MessageBar_11.MessageBarWithContent(this.game, contentGroup);
            this.add(containingBar);
        }
    }
    exports.StageProgressMessageBar = StageProgressMessageBar;
});
define("state/HubState", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/localization/LocalizationManager", "switchback-common/util/ScaleUtil", "state/ToyStoryBaseState", "component/hub/WoodyHubNode", "component/hub/HammHubNode", "component/hub/FlyingTossersHubNode", "component/hub/SargeHubNode", "component/hub/VolcanoHubNode", "component/hub/DressUpHubNode", "component/hub/DrawingHubNode", "StateNames", "logic/GameController", "component/dialog/CharacterDialog", "Character", "state/CutSceneState", "component/dialog/ConfirmationDialog", "model/ToyStoryConfig", "component/hub/HubFooter", "component/hub/HubHeader", "component/dialog/WayfindingDialog", "switchback-common/enum/Park", "logic/TSMAnalyticsManager", "component/MessageBar", "component/button/TextArrowButton", "component/StageProgressMessageBar", "component/dialog/PosterReminderDialog"], function (require, exports, PromiseUtil_41, LocalizationManager_21, ScaleUtil_19, ToyStoryBaseState_4, WoodyHubNode_1, HammHubNode_1, FlyingTossersHubNode_1, SargeHubNode_1, VolcanoHubNode_1, DressUpHubNode_1, DrawingHubNode_1, StateNames_25, GameController_23, CharacterDialog_4, Character_6, CutSceneState_5, ConfirmationDialog_4, ToyStoryConfig_15, HubFooter_2, HubHeader_1, WayfindingDialog_2, Park_8, TSMAnalyticsManager_9, MessageBar_12, TextArrowButton_8, StageProgressMessageBar_1, PosterReminderDialog_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * TODO:
     * Move a bunch of the things on GameController to private static vars on here
     */
    class HubState extends ToyStoryBaseState_4.ToyStoryBaseState {
        init() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                this.stateName = StateNames_25.StateNames.HUB;
                _super("init").call(this);
                this.canClickInstallationButton = true;
                // If they've made it here, they must have come through 
                // the intro cutscene. Mark it as complete.
                GameController_23.GameController.Instance.state.ftueCompleted = true;
                this.nodes = new Map();
                this.nodeChildIndexes = new Map();
                this.canShowExitSequence = false;
                this.needToShowExitSequence = false;
            });
        }
        preload() {
            super.preload();
            this.game.load.image('close-game-btn', 'assets/common/exit-button.png');
            this.game.load.image('map-bg', 'assets/hub/map-cosmetics/floor.jpg');
            this.game.load.image('tiling-bar', 'assets/common/CTAbar_tilingbg.png');
            this.game.load.image('next-button', 'assets/common/next-button.png');
            this.game.load.image('arrow_large', 'assets/common/CTA_arrow_large.png');
            // NODES
            this.game.load.spritesheet('node-box', 'assets/hub/map-cosmetics/nodes/box-reveal.png', 391, 520);
            this.game.load.image('cta-spotlight', 'assets/hub/map-cosmetics/nodes/cta-spotlight.png');
            this.game.load.image('cta-arrow', 'assets/hub/map-cosmetics/nodes/cta-arrow.png');
            // Node characters
            this.game.load.image('node-char-hamm', 'assets/hub/map-cosmetics/nodes/hamm/character.png');
            this.game.load.image('node-char-sarge', 'assets/hub/map-cosmetics/nodes/sarge/character.png');
            this.game.load.image('node-char-buzz', 'assets/hub/map-cosmetics/nodes/buzz/character.png');
            this.game.load.image('node-char-woody', 'assets/hub/map-cosmetics/nodes/woody/character.png');
            // Node bases
            this.game.load.atlas('node-hamm', 'assets/hub/map-cosmetics/nodes/node-hamm.png', 'assets/hub/map-cosmetics/nodes/node-hamm.json');
            this.game.load.atlas('node-sarge', 'assets/hub/map-cosmetics/nodes/node-sarge.png', 'assets/hub/map-cosmetics/nodes/node-sarge.json');
            this.game.load.atlas('node-trixie', 'assets/hub/map-cosmetics/nodes/node-trixie.png', 'assets/hub/map-cosmetics/nodes/node-trixie.json');
            this.game.load.atlas('node-buzz', 'assets/hub/map-cosmetics/nodes/node-buzz.png', 'assets/hub/map-cosmetics/nodes/node-buzz.json');
            this.game.load.atlas('node-woody', 'assets/hub/map-cosmetics/nodes/node-woody.png', 'assets/hub/map-cosmetics/nodes/node-woody.json');
            this.game.load.image('node-jessie', 'assets/hub/map-cosmetics/nodes/node-jessie.png');
            this.game.load.image('node-rex', 'assets/hub/map-cosmetics/nodes/node-rex.png');
            // character dialog assets
            this.game.load.image('character-dialog-backplate', 'assets/dialog/character_dialog_backplate.png');
            this.game.load.image('character-dialog-woody', 'assets/dialog/character_dialog_woody.png');
            this.game.load.image('character-dialog-buzz', 'assets/dialog/character_dialog_buzz.png');
            this.game.load.image('character-dialog-hamm', 'assets/dialog/character_dialog_hamm.png');
            this.game.load.image('character-dialog-jessie', 'assets/dialog/character_dialog_jessie.png');
            this.game.load.image('character-dialog-rex', 'assets/dialog/character_dialog_rex.png');
            this.game.load.image('character-dialog-sarge', 'assets/dialog/character_dialog_sarge.png');
            this.game.load.image('character-dialog-trixie', 'assets/dialog/character_dialog_trixie.png');
            // load the minigames' intro dialog assets
            this.game.load.image('banner-footer', 'assets/common/midway-footer.png');
            this.game.load.image('banner-header', 'assets/common/midway-header.png');
            this.game.load.spritesheet('big-btn-lights', 'assets/dialog/bigbtn_lights.png', 635, 640);
            this.game.load.image('big-btn-left-clean', 'assets/dialog/BigBtn_left_clean.png');
            this.game.load.image('big-btn-right-clean', 'assets/dialog/BigBtn_right_clean.png');
            this.game.load.image('big-btn-left-neutral', 'assets/dialog/BigBtn_left_neutral.png');
            this.game.load.image('big-btn-right-neutral', 'assets/dialog/BigBtn_right_neutral.png');
            this.game.load.image('instruction-dialog-backplate', 'assets/dialog/instruction_dialog_backplate.png');
            this.game.load.image('instruction-dialog-backplate-noprompt', 'assets/dialog/CTA_dialog_full_noprompt.png');
            this.game.load.image('progress-bar', 'assets/dialog/progress_bar.png');
            this.game.load.image('progress-bar-fill', 'assets/dialog/progress_bar_fill.png');
            this.game.load.image('instructions-graphic-hold', 'assets/dialog/instruction_graphic_hold.png');
            // TEMP PORTRAITS
            this.game.load.image('portrait-hamm-idle', 'assets/character-portraits/hamm/idle.png');
            this.game.load.image('portrait-rex-idle', 'assets/character-portraits/rex/idle.png');
            this.game.load.image('portrait-sarge-idle', 'assets/character-portraits/sarge/idle.png');
            this.game.load.image('portrait-woody-idle', 'assets/character-portraits/woody/idle.png');
            this.game.load.image('portrait-buzz-idle', 'assets/character-portraits/buzz/idle.png');
            this.game.load.image('portrait-jessie-idle', 'assets/character-portraits/jessie/idle.png');
            this.game.load.image('portrait-trixie-idle', 'assets/character-portraits/trixie/idle.png');
            // Footer and Header assets
            this.game.load.image('hub-footer-panel', 'assets/hub/hub-footer-panel.png');
            this.game.load.image('hub-header-panel', 'assets/hub/hub-header-panel.png');
            this.game.load.image('poster-icon', 'assets/hub/poster-icon.png');
            this.game.load.image('jail-icon', 'assets/jail-installation/jail-icon.png');
            // Wayfinding Dialog
            this.load.image('button-big-backplate', 'assets/title/button-big-backplate.png');
            this.load.image('button-big-frame', 'assets/title/button-big-frame.png');
            this.load.image('button-green-next', 'assets/title/button-green-next.png');
            this.load.image('button-orange-replay', 'assets/title/button-orange-replay.png');
            this.load.image('ui-bottom-panel', 'assets/common/ui-bottom-panel.jpg');
            this.load.image('ui-top-panel', 'assets/common/ui-top-panel.png');
            this.load.image('poster-room-wayfind', 'assets/hub/poster-room-wayfind.jpg');
            this.load.image('jail-wayfind', 'assets/jail-installation/jail-overlay.png'); // TODO: replace with correct wayfinding photo when we have it
            this.load.image('stageicon_0', 'assets/hub/stageicon_0.png');
            this.load.image('stageicon_1', 'assets/hub/stageicon_1.png');
            this.load.image('stageicon_2', 'assets/hub/stageicon_2.png');
            this.load.image('stageicon_3', 'assets/hub/stageicon_3.png');
            this.load.image('stageicon_highlighted', 'assets/hub/stageicon_highlighted.png');
            this.load.image('stageicon_connector_base', 'assets/hub/stageicon_connector_base.png');
            this.load.audio('snd_unbox', ['assets/sound/ui/unbox.m4a', 'assets/sound/ui/unbox.ogg']);
            this.load.audio('snd_node_upgrade', ['assets/sound/ui/node_upgrade.m4a', 'assets/sound/ui/node_upgrade.ogg']);
            this.load.audio('snd_node_complete', ['assets/sound/ui/node_complete.m4a', 'assets/sound/ui/node_complete.ogg']);
            this.load.audio('snd_click_node', ['assets/sound/ui/click_node.m4a', 'assets/sound/ui/click_node.ogg']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                if (ToyStoryConfig_15.ToyStoryConfig.Instance.showDebugMenu) {
                    this.debugButtonLayer.add(this.createDebugButton('R', 4, () => __awaiter(this, void 0, void 0, function* () {
                        GameController_23.GameController.Instance.state.forcedProgressionRound += 1;
                        yield this.onForcedProgressionRoundIncrease();
                    })));
                    this.debugButtonLayer.add(this.createDebugButton('W', 7, () => __awaiter(this, void 0, void 0, function* () {
                        this.debugStartGame(0);
                    })));
                    this.debugButtonLayer.add(this.createDebugButton('H', 8, () => __awaiter(this, void 0, void 0, function* () {
                        this.debugStartGame(1);
                    })));
                    this.debugButtonLayer.add(this.createDebugButton('T', 9, () => __awaiter(this, void 0, void 0, function* () {
                        this.debugStartGame(2);
                    })));
                    this.debugButtonLayer.add(this.createDebugButton('B', 10, () => __awaiter(this, void 0, void 0, function* () {
                        this.debugStartGame(3);
                    })));
                    this.debugButtonLayer.add(this.createDebugButton('A', 11, () => __awaiter(this, void 0, void 0, function* () {
                        this.debugStartGame(4);
                    })));
                    this.debugButtonLayer.add(this.createDebugButton('D', 12, () => __awaiter(this, void 0, void 0, function* () {
                        this.debugStartGame(5);
                    })));
                    this.debugButtonLayer.add(this.createDebugButton('R', 13, () => __awaiter(this, void 0, void 0, function* () {
                        this.debugStartGame(6);
                    })));
                }
                this.bg = new Phaser.Image(this.game, 0, 0, 'map-bg');
                this.bg.width = this.game.width;
                this.bg.height = this.game.height;
                this.add.existing(this.bg);
                this.nodeLayer = new Phaser.Group(this.game);
                this.add.existing(this.nodeLayer);
                const woodyNode = new WoodyHubNode_1.WoodyHubNode(this.game);
                woodyNode.position.set(this.game.width * 0.5, this.game.height * 0.35);
                this.nodeLayer.add(woodyNode);
                this.nodes.set(woodyNode.stateName, woodyNode);
                const flyingTossersNode = new FlyingTossersHubNode_1.FlyingTossersHubNode(this.game);
                flyingTossersNode.position.set(this.game.width * 0.78, this.game.height * 0.5);
                this.nodeLayer.add(flyingTossersNode);
                this.nodes.set(flyingTossersNode.stateName, flyingTossersNode);
                const sargeNode = new SargeHubNode_1.SargeHubNode(this.game);
                sargeNode.position.set(this.game.width * 0.27, this.game.height * 0.5);
                this.nodeLayer.add(sargeNode);
                this.nodes.set(sargeNode.stateName, sargeNode);
                const hammNode = new HammHubNode_1.HammHubNode(this.game);
                hammNode.position.set(this.game.width * 0.28, this.game.height * 0.72);
                this.nodeLayer.add(hammNode);
                this.nodes.set(hammNode.stateName, hammNode);
                const volcanoNode = new VolcanoHubNode_1.VolcanoHubNode(this.game);
                volcanoNode.position.set(this.game.width * 0.74, this.game.height * 0.72);
                this.nodeLayer.add(volcanoNode);
                this.nodes.set(volcanoNode.stateName, volcanoNode);
                const dressUpNode = new DressUpHubNode_1.DressUpHubNode(this.game);
                dressUpNode.position.set(this.game.width * 0.67, this.game.height * 0.91);
                this.nodeLayer.add(dressUpNode);
                this.nodes.set(dressUpNode.stateName, dressUpNode);
                const coloringNode = new DrawingHubNode_1.DrawingHubNode(this.game);
                coloringNode.position.set(this.game.width * 0.33, this.game.height * 0.91);
                this.nodeLayer.add(coloringNode);
                this.nodes.set(coloringNode.stateName, coloringNode);
                // Scale all the nodes
                this.nodes.forEach((node) => {
                    node.width = this.game.width * 0.6;
                    node.scale.y = node.scale.x;
                    // And figure out the child index (ordering) of each 
                    // one for later - we'll need it if we highlight any
                    this.nodeChildIndexes.set(node.stateName, this.nodeLayer.getChildIndex(node));
                });
                ////////////////////////////////////
                // Screen Dimming Overlay
                ////////////////////////////////////
                this.screenDimOverlay = new Phaser.Graphics(this.game, 0, 0);
                this.screenDimOverlay.beginFill(0x000000, 0.8);
                this.screenDimOverlay.drawRect(0, 0, this.game.width, this.game.height);
                this.screenDimOverlay.alpha = 0;
                this.add.existing(this.screenDimOverlay);
                ////////////////////////////////////
                // Spotlight
                ////////////////////////////////////
                this.spotlight = new Phaser.Image(this.game, 0, 0, 'cta-spotlight');
                this.spotlight.alpha = 0;
                this.add.existing(this.spotlight);
                ////////////////////////////////////
                // Highlight Target Layer
                ////////////////////////////////////
                this.highlightTargetLayer = new Phaser.Group(this.game);
                this.add.existing(this.highlightTargetLayer);
                ////////////////////////////////////
                // Arrow
                ////////////////////////////////////
                this.arrow = new Phaser.Image(this.game, 0, 0, 'cta-arrow');
                this.arrow.anchor.set(0.5, 1);
                this.add.existing(this.arrow);
                ////////////////////////////////////
                // Click Blocker
                ////////////////////////////////////
                this.clickBlocker = new Phaser.Graphics(this.game, 0, 0);
                this.clickBlocker.beginFill(0x330000, 0);
                this.clickBlocker.drawRect(0, 0, this.game.width, this.game.height);
                this.clickBlocker.inputEnabled = true;
                this.clickBlocker.visible = true;
                this.add.existing(this.clickBlocker);
                ////////////////////////////////////
                // Header
                ////////////////////////////////////
                let headerStringKey = "objectives.unbox_all";
                if (GameController_23.GameController.Instance.getAreAllGamesUnlocked()) {
                    headerStringKey = "objectives.assemble_all";
                }
                if (GameController_23.GameController.Instance.getAreAllQuotasComplete()) {
                    headerStringKey = "objectives.finished";
                }
                this.header = new HubHeader_1.HubHeader(this.game, LocalizationManager_21.LocalizationManager.Instance.text(headerStringKey));
                this.header.bottom = 0;
                this.add.existing(this.header);
                ////////////////////////////////////
                // Footer
                // Not shown if they're in WDW &&
                // have completed the WDW installation
                ////////////////////////////////////
                // TODO: The installation at WDW has been disabled until it is installed
                let isFooterShowing = true;
                if (ToyStoryConfig_15.ToyStoryConfig.Instance.park === Park_8.Park.WDW && !ToyStoryConfig_15.ToyStoryConfig.Instance.useWDWInstallation) {
                    isFooterShowing = false;
                }
                this.characterDialogTopY = this.header.centerY + this.header.height;
                this.characterDialogBottomY = this.game.height * 0.9;
                if (isFooterShowing) {
                    let footerStringKey = 'poster.hub_description';
                    let footerImageTag = 'poster-icon';
                    if (ToyStoryConfig_15.ToyStoryConfig.Instance.park === Park_8.Park.WDW) {
                        footerStringKey = "jail.hub_description";
                        footerImageTag = "jail-icon";
                    }
                    this.footer = new HubFooter_2.HubFooter(this.game, footerImageTag, LocalizationManager_21.LocalizationManager.Instance.text(footerStringKey));
                    this.footer.top = this.game.height;
                    this.add.existing(this.footer);
                    if (ToyStoryConfig_15.ToyStoryConfig.Instance.park === Park_8.Park.DLR) {
                        this.characterDialogBottomY = this.footer.top - this.footer.height;
                        this.footer.onChildInputDown.add(this.onInstallationButtonClickedDLR, this);
                    }
                    else if (ToyStoryConfig_15.ToyStoryConfig.Instance.park === Park_8.Park.WDW) {
                        this.footer.onChildInputDown.add(this.onInstallationButtonClickedWDW, this);
                        // TODO: Re-add this later. We don't have anything for this to do yet, so hide it in WDW.
                        if (GameController_23.GameController.Instance.state.hasCompletedJailInstallation) {
                            this.footer.visible = false;
                        }
                    }
                }
                ////////////////////////////////////
                // Quit Confirmation Dialog
                ////////////////////////////////////
                const confirmDialog = new ConfirmationDialog_4.ConfirmationDialog(this.game, LocalizationManager_21.LocalizationManager.Instance.text("confirm_dialog.warning_leave"), LocalizationManager_21.LocalizationManager.Instance.text("confirm_dialog.exit_qn"), LocalizationManager_21.LocalizationManager.Instance.text("confirm_dialog.cancel"), LocalizationManager_21.LocalizationManager.Instance.text("confirm_dialog.exit"), false);
                confirmDialog.onButtonClickedSignal.add((confirmed) => __awaiter(this, void 0, void 0, function* () {
                    if (confirmed) {
                        this.game.sound.play('snd_ui_click');
                        confirmDialog.onButtonClickedSignal.dispose();
                        yield GameController_23.GameController.Instance.closeGame();
                    }
                    else {
                        this.game.sound.play('snd_back');
                        confirmDialog.hide();
                    }
                }), this);
                ////////////////////////////////////
                // Quit Button
                ///////////////////////////////////
                this.quitButton = new Phaser.Image(this.game, 0, 0, 'close-game-btn');
                ScaleUtil_19.scaleToBounds(this.quitButton, this.game.width * 0.15, this.game.height * 0.1);
                this.quitButton.top = this.game.height * 0.01;
                this.quitButton.right = this.game.width * 0.98;
                this.quitButton.visible = false;
                this.quitButton.inputEnabled = true;
                this.quitButton.events.onInputDown.add(() => {
                    this.game.sound.play('snd_back');
                    confirmDialog.parent.bringToTop(confirmDialog);
                    confirmDialog.show();
                }, this);
                this.add.existing(this.quitButton);
                this.add.existing(confirmDialog);
                ////////////////////////////////////
                // Installation Wayfinding Dialog
                ///////////////////////////////////
                let wayfindStringKey = 'poster.have_you_entered';
                let wayfindImageTag = 'poster-room-wayfind';
                let isInstallation = false;
                if (ToyStoryConfig_15.ToyStoryConfig.Instance.park === Park_8.Park.WDW) {
                    wayfindStringKey = "jail.can_you_see";
                    wayfindImageTag = "jail-wayfind";
                    isInstallation = true;
                }
                this.installationWayfindingDialog = new WayfindingDialog_2.WayfindingDialog(this.game, wayfindImageTag, LocalizationManager_21.LocalizationManager.Instance.text(wayfindStringKey), isInstallation);
                this.add.existing(this.installationWayfindingDialog);
                this.posterReminderDialog = new PosterReminderDialog_2.PosterReminderDialog(this.game);
                this.posterReminderDialog.centerX = this.game.width * 0.5;
                this.posterReminderDialog.centerY = this.game.height * 0.6;
                this.add.existing(this.posterReminderDialog);
                yield this.dismissOverlay();
                this.start();
            });
        }
        start() {
            return __awaiter(this, void 0, void 0, function* () {
                TSMAnalyticsManager_9.TSMAnalyticsManager.Instance.onGameHub();
                const nodeKeysArray = Array.from(this.nodes.keys());
                // Firstly, check to see if they've hit the exit beacon
                if (GameController_23.GameController.Instance.state.exitTriggered) {
                    this.canShowExitSequence = true;
                    const didExit = yield this.onExitTriggered();
                    // If they exited the game, we've probably changed state,
                    // but make sure this function doesn't continue anyway,
                    if (didExit) {
                        return;
                    }
                }
                // Loop through nodes and see if any need upgrading
                for (let i = 0; i < nodeKeysArray.length; i++) {
                    const node = this.nodes.get(nodeKeysArray[i]);
                    while (node.shouldUpgrade) {
                        yield this.upgradeNode(node);
                    }
                }
                if (GameController_23.GameController.Instance.didJustStartForcedProgressionRound) {
                    yield this.onForcedProgressionRoundIncrease();
                }
                // Should we remind them about the installation?
                if (this.footer) {
                    if (GameController_23.GameController.Instance.shouldShowInstallationReminder()) {
                        if (ToyStoryConfig_15.ToyStoryConfig.Instance.park === Park_8.Park.DLR &&
                            !GameController_23.GameController.Instance.state.hasCompletedPosterInstallation &&
                            !GameController_23.GameController.Instance.state.hasEnteredPosterRoom) {
                            yield this.onInstallationButtonClickedDLR();
                        }
                        else if (ToyStoryConfig_15.ToyStoryConfig.Instance.park === Park_8.Park.WDW &&
                            !GameController_23.GameController.Instance.state.hasCompletedPosterInstallation) {
                            yield this.onInstallationButtonClickedWDW();
                        }
                    }
                }
                else {
                    if (ToyStoryConfig_15.ToyStoryConfig.Instance.park === Park_8.Park.WDW && ToyStoryConfig_15.ToyStoryConfig.Instance.useWDWExitTimer && GameController_23.GameController.Instance.shouldShowExitReminder()) {
                        this.canShowExitSequence = true;
                        const didExit = yield this.onExitTriggered();
                        if (didExit) {
                            return;
                        }
                    }
                }
                // Bring in header and footer bars now
                yield this.showUI();
                // Loop through nodes and see which need unlocking / playing
                const gameToPlay = GameController_23.GameController.Instance.getGameToPlayNext();
                if (GameController_23.GameController.Instance.state.getGameTargetLevel(gameToPlay) === 0) {
                    this.unlockNode(this.nodes.get(gameToPlay));
                }
                else {
                    this.focusNode(this.nodes.get(gameToPlay));
                }
                this.canShowExitSequence = true;
                if (this.needToShowExitSequence) {
                    yield this.onExitTriggered();
                }
            });
        }
        /**
         * This function deals with the installation button in the corner -
         * for DLR only, as the logic is a little different.
         *
         * It performs a wayfinding check - unless they've already passed it.
         * It then saves the result of the check back to the save state, and
         * either shows the installation state (if they passed it) or hides the dialog
         * for them to try again later (if they didn't).
         */
        onInstallationButtonClickedDLR() {
            return __awaiter(this, void 0, void 0, function* () {
                // Prevent spam
                if (!this.canClickInstallationButton) {
                    return;
                }
                this.game.sound.play('snd_ui_click');
                this.canClickInstallationButton = false;
                // In DLR you can revisit this state, so it's possible to
                // have already passed the wayfinding check. Don't re-show it.
                let hasReachedInstallation = GameController_23.GameController.Instance.state.hasEnteredPosterRoom;
                if (!hasReachedInstallation) {
                    hasReachedInstallation = yield this.showInstallationWayfindDLR();
                }
                if (hasReachedInstallation) {
                    yield this.showOverlay();
                    CutSceneState_5.CutSceneState.showCutScene(this.game, "posters_intro", StateNames_25.StateNames.POSTER);
                }
                else {
                    yield this.installationWayfindingDialog.hide();
                    yield this.showPosterReminder();
                }
                this.canClickInstallationButton = true;
            });
        }
        showInstallationWayfindDLR() {
            return __awaiter(this, void 0, void 0, function* () {
                TSMAnalyticsManager_9.TSMAnalyticsManager.Instance.onPosterRoomWayfindState();
                this.installationWayfindingDialog.parent.bringToTop(this.installationWayfindingDialog);
                const result = yield this.installationWayfindingDialog.show();
                GameController_23.GameController.Instance.state.hasEnteredPosterRoom = result;
                TSMAnalyticsManager_9.TSMAnalyticsManager.Instance.onPosterRoomWayfind(result);
                return result;
            });
        }
        showPosterReminder() {
            return __awaiter(this, void 0, void 0, function* () {
                const wasClickBlockerEnabled = this.clickBlocker.inputEnabled;
                this.clickBlocker.inputEnabled = false;
                if (this.activeCharacterDialog) {
                    this.activeCharacterDialog.visible = false;
                }
                this.showUI(); // If you have just come back from a scene UI wont be there.
                yield this.posterReminderDialog.show();
                const footerParent = this.footer.parent;
                let footerIndex = 0;
                if (footerParent) {
                    footerIndex = footerParent.getChildIndex(this.footer);
                    footerParent.setChildIndex(this.footer, footerParent.children.length - 1);
                }
                yield PromiseUtil_41.PromiseUtil.signal(this.posterReminderDialog.DialogClosedSignal);
                if (footerParent) {
                    footerParent.setChildIndex(this.footer, footerIndex);
                }
                if (this.activeCharacterDialog) {
                    this.activeCharacterDialog.visible = true;
                }
                this.clickBlocker.inputEnabled = wasClickBlockerEnabled;
            });
        }
        /**
         * The WDW equivalent of the installation logic is a bit simpler than DLR.
         * It's a one-time thing - once the guests have completed the installation,
         * they won't see this again. So we only have to figure out if they're in
         * the correct location via a wayfinding check. We also won't bother to
         * save this back to the save state, as it's important we verify again
         * if they were to exit out during the installation flow.
         */
        onInstallationButtonClickedWDW() {
            return __awaiter(this, void 0, void 0, function* () {
                // Prevent spam
                if (!this.canClickInstallationButton) {
                    return;
                }
                this.game.sound.play('snd_ui_click');
                this.canClickInstallationButton = false;
                this.installationWayfindingDialog.parent.bringToTop(this.installationWayfindingDialog);
                let hasReachedInstallation = yield this.installationWayfindingDialog.show();
                if (hasReachedInstallation) {
                    yield this.showOverlay();
                    CutSceneState_5.CutSceneState.showCutScene(this.game, 'pete_caught', StateNames_25.StateNames.HUB);
                }
                else {
                    yield this.installationWayfindingDialog.hide();
                    yield this.showPosterReminder();
                }
                this.canClickInstallationButton = true;
            });
        }
        /**
         * Shows the dialogs communicating the forced progression stages
         */
        onForcedProgressionRoundIncrease() {
            return __awaiter(this, void 0, void 0, function* () {
                const stageNum = GameController_23.GameController.Instance.state.forcedProgressionRound;
                // Create the arrow that allows players to dismiss
                const nextArrow = new TextArrowButton_8.TextArrowButton(this.game, LocalizationManager_21.LocalizationManager.Instance.text("common.ok"));
                nextArrow.centerX = this.game.width * 0.5;
                nextArrow.centerY = this.game.height * 0.9;
                nextArrow.alpha = 0;
                nextArrow.startPulse();
                const topText = LocalizationManager_21.LocalizationManager.Instance.text(stageNum > 0 ? 'forced_progression.stage_title_n' : 'forced_progression.stage_title_1');
                const bottomText = LocalizationManager_21.LocalizationManager.Instance.textWithReplacements(stageNum > 0 ? 'forced_progression.stage_subtitle_n' : 'forced_progression.stage_subtitle_1', {
                    stage_num: stageNum + 1
                });
                // Create the message bars telling the player about the stages
                const topMessageBar = new MessageBar_12.MessageBarWithHeader(this.game, topText, bottomText);
                topMessageBar.centerX = this.game.width * 0.5;
                topMessageBar.bottom = 0;
                const bottomMessageBar = new StageProgressMessageBar_1.StageProgressMessageBar(this.game, stageNum);
                bottomMessageBar.centerX = this.game.width * 0.5;
                bottomMessageBar.top = this.game.height;
                this.add.existing(nextArrow);
                this.add.existing(topMessageBar);
                this.add.existing(bottomMessageBar);
                this.screenDimOverlay.alpha = 0;
                yield Promise.all([
                    PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(this.screenDimOverlay)
                        .to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(topMessageBar)
                        .to({ centerY: this.game.height * 0.25 }, 450, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(bottomMessageBar)
                        .to({ centerY: this.game.height * 0.65 }, 450, Phaser.Easing.Sinusoidal.InOut, true))
                ]);
                yield PromiseUtil_41.PromiseUtil.time(this.game, 2500);
                yield PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(nextArrow)
                    .to({ alpha: 1 }, 300, Phaser.Easing.Linear.None, true, 0));
                // Wait for them to tap the arrow (or anywhere on the screen)
                yield Promise.race([
                    PromiseUtil_41.PromiseUtil.signal(nextArrow.onChildInputDown),
                    PromiseUtil_41.PromiseUtil.signal(this.game.input.onTap)
                ]);
                this.game.sound.play('snd_ui_click');
                GameController_23.GameController.Instance.startForcedProgressionRound(stageNum);
                yield Promise.all([
                    PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(nextArrow)
                        .to({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(this.screenDimOverlay)
                        .to({ alpha: 0 }, 300, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(topMessageBar)
                        .to({ bottom: 0 }, 300, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(bottomMessageBar)
                        .to({ top: this.game.height }, 300, Phaser.Easing.Sinusoidal.InOut, true))
                ]);
                nextArrow.destroy();
            });
        }
        showUI() {
            return __awaiter(this, void 0, void 0, function* () {
                this.quitButton.visible = true;
                if (this.footer) {
                    yield Promise.all([
                        PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(this.header)
                            .to({ top: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true)),
                        PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(this.footer)
                            .to({ bottom: this.game.height }, 500, Phaser.Easing.Sinusoidal.InOut, true)),
                        PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(this.quitButton)
                            .from({ alpha: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true))
                    ]);
                }
                else {
                    yield Promise.all([
                        PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(this.header)
                            .to({ top: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true)),
                        PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(this.quitButton)
                            .from({ alpha: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true))
                    ]);
                }
            });
        }
        upgradeNode(node) {
            return __awaiter(this, void 0, void 0, function* () {
                this.highlightNode(node);
                this.activeCharacterDialog = yield this.showCharacterDialog(node.bottom, node.characterText, node.speakingCharacter);
                yield Promise.race([
                    PromiseUtil_41.PromiseUtil.signal(this.clickBlocker.events.onInputDown),
                    this.activeCharacterDialog.waitForTap()
                ]);
                // Fade out the arrow and kill its bouncing tween
                this.game.add.tween(this.arrow).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true).onComplete.addOnce(() => {
                    this.game.tweens.removeFrom(this.arrow);
                });
                if (node.visualLevel + 1 < node.maxLevel) {
                    this.game.sound.play('snd_node_upgrade');
                }
                else {
                    this.game.sound.play('snd_node_complete');
                }
                yield node.upgradeOne();
                yield this.dismissCharacterDialog();
                yield this.dismissHighlight(node);
            });
        }
        unlockNode(node) {
            return __awaiter(this, void 0, void 0, function* () {
                this.highlightNode(node);
                this.activeCharacterDialog = yield this.showCharacterDialog(node.bottom, node.characterText, node.speakingCharacter);
                yield Promise.race([
                    PromiseUtil_41.PromiseUtil.signal(this.clickBlocker.events.onInputDown),
                    this.activeCharacterDialog.waitForTap()
                ]);
                // Fade out the arrow and kill its bouncing tween
                this.game.add.tween(this.arrow).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true).onComplete.addOnce(() => {
                    this.game.tweens.removeFrom(this.arrow);
                });
                GameController_23.GameController.Instance.state.incrementGameTargetLevel(node.stateName);
                GameController_23.GameController.Instance.state.forcedProgressionStep++;
                this.game.sound.play('snd_unbox');
                yield node.unlock();
                yield this.dismissCharacterDialog();
                this.startGame(node.stateName);
            });
        }
        focusNode(node) {
            return __awaiter(this, void 0, void 0, function* () {
                this.highlightNode(node);
                yield PromiseUtil_41.PromiseUtil.signal(this.clickBlocker.events.onInputDown);
                GameController_23.GameController.Instance.state.forcedProgressionStep++;
                this.startGame(node.stateName);
            });
        }
        debugStartGame(gameIndex) {
            this.game.sound.play('snd_ui_click');
            GameController_23.GameController.Instance.state.forcedProgressionStep = (GameController_23.GameController.Instance.state.forcedProgressionRound * GameController_23.GameController.CORE_GAMES.length) + gameIndex;
            const node = this.nodes.get(GameController_23.GameController.Instance.getGameToPlayNext());
            this.startGame(node.stateName);
        }
        /**
         * Allows players to complete the hub then proceed to the outro if they accept a confirmation.
         * If players decline the option, this function will return false
         * If players accept the option to exit, this function will return true
         */
        onExitTriggered() {
            return __awaiter(this, void 0, void 0, function* () {
                // Don't do this if we receive an exit trigger
                // during the regular hub animations
                if (!this.canShowExitSequence) {
                    this.needToShowExitSequence = true;
                    return false;
                }
                TSMAnalyticsManager_9.TSMAnalyticsManager.Instance.onGameExit(TSMAnalyticsManager_9.TSMAnalyticsManager.EXITING_SHOW);
                // TODO: The installation at WDW has been disabled until it is installed. 
                // Uses beacon with confirmation dialog for now
                if (ToyStoryConfig_15.ToyStoryConfig.Instance.park === Park_8.Park.WDW && !ToyStoryConfig_15.ToyStoryConfig.Instance.useWDWInstallation) {
                    if (!(yield this.showExitConfirmation())) {
                        return false;
                    }
                }
                if (this.activeCharacterDialog) {
                    this.game.add.tween(this.header)
                        .to({ bottom: 0 }, 500, Phaser.Easing.Sinusoidal.In).start();
                    this.arrow.visible = false;
                    yield this.dismissCharacterDialog();
                    yield this.dismissHighlight(this.highlightedNode);
                }
                // If they decided to leave, upgrade all nodes
                this.clickBlocker.visible = true;
                // Say something
                const dialog = new CharacterDialog_4.CharacterDialog(this.game, Character_6.Character.WOODY);
                this.add.existing(dialog);
                yield dialog.show();
                yield dialog.writeMessage(LocalizationManager_21.LocalizationManager.Instance.text("outro.hub_1"));
                // allows for tapping the whole screen
                yield Promise.race([
                    dialog.waitForTap(),
                    PromiseUtil_41.PromiseUtil.signal(this.game.input.onTap)
                ]);
                yield dialog.hide();
                // Upgrade everything
                yield this.completeAllNodes();
                // Remove any progress stored against the current user
                GameController_23.GameController.Instance.state.clear();
                // Congratulate them
                yield dialog.show();
                yield dialog.writeMessage(LocalizationManager_21.LocalizationManager.Instance.text("outro.hub_2"));
                // allows for tapping the whole screen
                yield Promise.race([
                    dialog.waitForTap(),
                    PromiseUtil_41.PromiseUtil.signal(this.game.input.onTap)
                ]);
                yield dialog.hide();
                dialog.destroy();
                // Take them to the outro
                yield this.showOverlay();
                // TODO: The installation at WDW has been disabled until it is installed. 
                // Pete is caught during outro in the meantime
                if (ToyStoryConfig_15.ToyStoryConfig.Instance.park === Park_8.Park.WDW && !ToyStoryConfig_15.ToyStoryConfig.Instance.useWDWInstallation) {
                    CutSceneState_5.CutSceneState.showCutScene(this.game, "outro_with_pete", StateNames_25.StateNames.OUTRO);
                }
                else {
                    CutSceneState_5.CutSceneState.showCutScene(this.game, "outro", StateNames_25.StateNames.OUTRO);
                }
                return true;
            });
        }
        showExitConfirmation() {
            return __awaiter(this, void 0, void 0, function* () {
                // Throw up a confirmation asking if they want to exit the queue
                let confirmDialog = new ConfirmationDialog_4.ConfirmationDialog(this.game, LocalizationManager_21.LocalizationManager.Instance.text("outro_confirm_dialog.title"), LocalizationManager_21.LocalizationManager.Instance.text("outro_confirm_dialog.outro_qn"), LocalizationManager_21.LocalizationManager.Instance.text("outro_confirm_dialog.no"), LocalizationManager_21.LocalizationManager.Instance.text("outro_confirm_dialog.yes"), true);
                TSMAnalyticsManager_9.TSMAnalyticsManager.Instance.onOutroPrompt();
                yield confirmDialog.show();
                let confirmedExit = yield PromiseUtil_41.PromiseUtil.signal(confirmDialog.onButtonClickedSignal);
                TSMAnalyticsManager_9.TSMAnalyticsManager.Instance.onQueueEndConfirmation(confirmedExit);
                // Either way, dismiss then destroy the dialog
                yield confirmDialog.hide();
                confirmDialog.destroy();
                if (confirmedExit) {
                    // show another confirmation, confirming their confirmation
                    confirmDialog = new ConfirmationDialog_4.ConfirmationDialog(this.game, LocalizationManager_21.LocalizationManager.Instance.text("installation_confirm_dialog.title"), LocalizationManager_21.LocalizationManager.Instance.text("installation_confirm_dialog.warning"), LocalizationManager_21.LocalizationManager.Instance.text("installation_confirm_dialog.no"), LocalizationManager_21.LocalizationManager.Instance.text("installation_confirm_dialog.yes"), true);
                    TSMAnalyticsManager_9.TSMAnalyticsManager.Instance.onOutroPrompt();
                    yield confirmDialog.show();
                    confirmedExit = yield PromiseUtil_41.PromiseUtil.signal(confirmDialog.onButtonClickedSignal);
                    TSMAnalyticsManager_9.TSMAnalyticsManager.Instance.onQueueEndConfirmation(confirmedExit);
                    // Either way, dismiss then destroy the dialog
                    yield confirmDialog.hide();
                    confirmDialog.destroy();
                }
                // If they decided to stay, go back
                if (confirmedExit) {
                    TSMAnalyticsManager_9.TSMAnalyticsManager.Instance.onGameExit(TSMAnalyticsManager_9.TSMAnalyticsManager.EXITING_CONFIRM);
                }
                else {
                    TSMAnalyticsManager_9.TSMAnalyticsManager.Instance.onGameExit(TSMAnalyticsManager_9.TSMAnalyticsManager.EXITING_CANCEL);
                }
                return confirmedExit;
            });
        }
        completeAllNodes() {
            return __awaiter(this, void 0, void 0, function* () {
                let unfinishedNodes = this.getUnfinishedNodes();
                while (unfinishedNodes.length > 0) {
                    const completionPromises = [];
                    for (let i = 0; i < unfinishedNodes.length; i++) {
                        const node = unfinishedNodes[i];
                        GameController_23.GameController.Instance.state.incrementGameTargetLevel(node.stateName);
                        if (node.visualLevel === 0) {
                            completionPromises.push(PromiseUtil_41.PromiseUtil.time(this.game, i * 250).then(() => __awaiter(this, void 0, void 0, function* () {
                                this.game.sound.play('snd_unbox');
                                yield node.unlock();
                            })));
                        }
                        else {
                            completionPromises.push(PromiseUtil_41.PromiseUtil.time(this.game, i * 250).then(() => __awaiter(this, void 0, void 0, function* () {
                                if (node.visualLevel + 1 < node.maxLevel) {
                                    this.game.sound.play('snd_node_upgrade');
                                }
                                else {
                                    this.game.sound.play('snd_node_complete');
                                }
                                yield node.upgradeOne();
                            })));
                        }
                    }
                    yield Promise.all(completionPromises);
                    unfinishedNodes = this.getUnfinishedNodes();
                }
            });
        }
        getUnfinishedNodes() {
            return Array.from(this.nodes.values()).filter(node => node.visualLevel < node.maxLevel);
        }
        highlightNode(target) {
            return __awaiter(this, void 0, void 0, function* () {
                this.highlightedNode = target;
                // Move the target to the layer in front
                this.highlightTargetLayer.add(target);
                // Fade in the dark shade
                this.screenDimOverlay.alpha = 0;
                yield PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(this.screenDimOverlay)
                    .to({ alpha: 1 }, 500, Phaser.Easing.Sinusoidal.In).start());
                // Set up the spotlight
                this.spotlight.width = target.width;
                this.spotlight.scale.y = this.spotlight.scale.x;
                this.spotlight.centerX = target.centerX;
                this.spotlight.bottom = target.bottom;
                // Set up the arrow
                this.arrow.width = this.spotlight.width * 0.4;
                this.arrow.scale.y = this.arrow.scale.x;
                this.arrow.x = this.spotlight.centerX;
                this.arrow.alpha = 0;
                let yTweenDest;
                if (target.bottom < this.game.height * 0.5) {
                    // It's on the top half - point upwards, at its bottom
                    this.arrow.y = target.bottom;
                    this.arrow.angle = 180;
                    yTweenDest = this.arrow.y - this.spotlight.height * 0.15;
                }
                else {
                    // It's on the bottom half. Point downwards, at a bit above its bottom
                    this.arrow.y = target.bottom - (this.spotlight.height * 0.35);
                    this.arrow.angle = 0;
                    yTweenDest = this.arrow.y + this.spotlight.height * 0.15;
                }
                yield PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(this.spotlight)
                    .to({ alpha: 1 }, 200, Phaser.Easing.Sinusoidal.In).start());
                this.game.add.tween(this.arrow)
                    .to({ y: yTweenDest }, 1000, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                yield PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(this.arrow).to({ alpha: 1 }, 200).start());
            });
        }
        dismissHighlight(target) {
            return __awaiter(this, void 0, void 0, function* () {
                // Fade out the spotlight
                yield PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(this.spotlight)
                    .to({ alpha: 0 }, 200, Phaser.Easing.Sinusoidal.In).start());
                // Fade out the dark shade
                yield PromiseUtil_41.PromiseUtil.tween(this.game, this.game.add.tween(this.screenDimOverlay)
                    .to({ alpha: 0 }, 500, Phaser.Easing.Sinusoidal.In).start());
                // Return the node to where it belongs
                this.nodeLayer.addChildAt(target, this.nodeChildIndexes.get(target.stateName));
            });
        }
        showCharacterDialog(nodeBottom, whatToSay, whoToSpeak) {
            return __awaiter(this, void 0, void 0, function* () {
                this.activeCharacterDialog = new CharacterDialog_4.CharacterDialog(this.game, whoToSpeak);
                if (nodeBottom < this.game.height * 0.7) {
                    this.activeCharacterDialog.bottom = this.characterDialogBottomY;
                }
                else {
                    this.activeCharacterDialog.top = this.characterDialogTopY;
                }
                this.add.existing(this.activeCharacterDialog);
                this.activeCharacterDialog.parent.setChildIndex(this.activeCharacterDialog, this.activeCharacterDialog.parent.children.length - 2);
                yield this.activeCharacterDialog.show();
                yield this.activeCharacterDialog.writeMessage(whatToSay);
                return this.activeCharacterDialog;
            });
        }
        dismissCharacterDialog() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.activeCharacterDialog) {
                    yield this.activeCharacterDialog.hide();
                    this.activeCharacterDialog.destroy();
                    this.activeCharacterDialog = null;
                }
            });
        }
        startGame(stateName) {
            return __awaiter(this, void 0, void 0, function* () {
                this.canShowExitSequence = false;
                this.needToShowExitSequence = false;
                this.clickBlocker.visible = true;
                yield this.showOverlay();
                TSMAnalyticsManager_9.TSMAnalyticsManager.Instance.onGameStart(stateName);
                this.transitionToGameTitle(stateName);
            });
        }
        transitionToGameTitle(stateName) {
            return __awaiter(this, void 0, void 0, function* () {
                GameController_23.GameController.Instance.currentGameState = stateName;
                this.game.state.start(StateNames_25.StateNames.MINIGAME_TITLE);
            });
        }
        shutdown() {
            this.nodeLayer = null;
            this.nodes = null;
            this.bg = null;
            this.footer = null;
            this.header = null;
            this.installationWayfindingDialog = null;
            this.posterReminderDialog = null;
            // Node Highlighting
            this.highlightedNode = null;
            this.screenDimOverlay = null;
            this.spotlight = null;
            this.highlightTargetLayer = null;
            this.arrow = null;
            this.clickBlocker = null;
            this.quitButton = null;
            this.activeCharacterDialog = null;
            this.cache.removeSound('snd_unbox');
            this.cache.removeSound('snd_node_upgrade');
            this.cache.removeSound('snd_node_complete');
            this.cache.removeSound('snd_click_node');
            super.shutdown();
        }
    }
    exports.HubState = HubState;
});
define("minigame/poster/PosterDraggable", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PosterDraggable extends Phaser.Image {
        constructor(game, tag, id, destinationPos) {
            super(game, 0, 0, tag);
            this.inputEnabled = true;
            this.destinationPos = destinationPos;
            this.id = id;
        }
        set snapPoint(pos) {
            this._snapPoint = pos;
            this.position = this._snapPoint.clone();
        }
        get snapPoint() {
            return this._snapPoint;
        }
    }
    exports.PosterDraggable = PosterDraggable;
});
define("component/TextBanner", ["require", "exports", "Game", "switchback-common/util/TextUtil"], function (require, exports, Game_27, TextUtil_18) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class TextBanner extends Phaser.Group {
        constructor(game, textString) {
            super(game);
            const bg = new Phaser.Image(this.game, 0, 0, 'banner');
            bg.anchor.set(0.5);
            this.add(bg);
            const text = new Phaser.Text(this.game, bg.centerX, bg.top + bg.height * 0.43, textString, {
                font: Game_27.Game.DEFAULT_FONT,
                fill: Game_27.Game.TEXT_COLOR_YELLOW,
                fontSize: Game_27.Game.FONT_SIZE_LARGE,
                align: 'center'
            });
            text.anchor.set(0.5);
            TextUtil_18.TextUtil.scaleDownSingleLineTextToWidth(text, bg.width * 0.8);
            text.setShadow(Game_27.Game.TEXT_SHADOW_OFFSET_MED_X, Game_27.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_27.Game.TEXT_SHADOW_HARD, 0, true, true);
            this.add(text);
        }
    }
    exports.TextBanner = TextBanner;
});
define("minigame/poster/PosterData", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PosterTargetData {
    }
    exports.PosterTargetData = PosterTargetData;
    class PosterData {
    }
    exports.PosterData = PosterData;
});
define("minigame/poster/Poster", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "logic/TSMAnalyticsManager", "StateNames", "logic/GameController"], function (require, exports, ScaleUtil_20, PromiseUtil_42, TSMAnalyticsManager_10, StateNames_26, GameController_24) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class Poster extends Phaser.Group {
        constructor(game, posterData) {
            super(game);
            this.completeSignal = new Phaser.Signal();
            this.dragStartSignal = new Phaser.Signal();
            this.posterData = posterData;
            // The poster will try to be as wide as the game, but always 
            // leave at least 15% screen height for the items at the bottom
            this.posterBackground = new Phaser.Image(this.game, 0, 0, posterData.backgroundTag);
            ScaleUtil_20.scaleToBounds(this.posterBackground, this.game.width, this.game.height * 0.85);
            this.add(this.posterBackground);
        }
        set enabled(enabled) { }
        show(durationMs) {
            return __awaiter(this, void 0, void 0, function* () {
                this.trackStart();
                this.footerGroup.top = this.game.height;
                this.footerGroup.alpha = 1;
                yield PromiseUtil_42.PromiseUtil.tween(this.game, this.game.add.tween(this.footerGroup)
                    .to({ bottom: this.game.height }, durationMs, Phaser.Easing.Sinusoidal.Out, true));
            });
        }
        hide(durationMs) {
            return __awaiter(this, void 0, void 0, function* () {
                this.trackEnd();
                yield PromiseUtil_42.PromiseUtil.tween(this.game, this.game.add.tween(this.footerGroup)
                    .to({ y: this.footerGroup.y + this.footerPanel.height }, durationMs, Phaser.Easing.Sinusoidal.Out, true));
                this.footerGroup.alpha = 0;
                this.footerGroup.bottom = this.game.height; // probably to avoid calculation of size errors in PosterGameState.ts
            });
        }
        /// Tracking
        trackStart() {
            const microName = Poster.MICRO_NAMES_FOR_ID.get(this.posterData.id);
            if (!microName) {
                return;
            }
            TSMAnalyticsManager_10.TSMAnalyticsManager.Instance.onMicroGameStart(StateNames_26.StateNames.POSTER, microName);
            TSMAnalyticsManager_10.TSMAnalyticsManager.Instance.onMicroTime(StateNames_26.StateNames.POSTER, microName);
            TSMAnalyticsManager_10.TSMAnalyticsManager.Instance.onGameState(StateNames_26.StateNames.POSTER, null, microName);
        }
        trackEnd() {
            const microName = Poster.MICRO_NAMES_FOR_ID.get(this.posterData.id);
            if (!microName) {
                return;
            }
            const score = GameController_24.GameController.Instance.state.getMinigameSaveData(StateNames_26.StateNames.POSTER + "-" + this.posterData.id).scorePendingResultsScreen;
            TSMAnalyticsManager_10.TSMAnalyticsManager.Instance.onMicroTime(StateNames_26.StateNames.POSTER, microName, false, score);
        }
        destroy() {
            this.completeSignal.dispose();
            this.dragStartSignal.dispose();
            super.destroy();
        }
    }
    // notice the installation poster is not added
    Poster.MICRO_NAMES_FOR_ID = new Map([
        ["how_to_play", "HowToPlay"],
        ["travel_between_games", "Travel"],
        ["woody", "Woody"],
        ["buzz", "Buzz"],
        ["sarge", "Sarge"],
        ["hamm", "Hamm"]
    ]);
    exports.Poster = Poster;
});
define("minigame/poster/DragPosterGame", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/localization/LocalizationManager", "minigame/poster/PosterDraggable", "switchback-common/util/ScaleUtil", "logic/GameController", "component/TextBanner", "minigame/poster/Poster", "StateNames"], function (require, exports, PromiseUtil_43, LocalizationManager_22, PosterDraggable_1, ScaleUtil_21, GameController_25, TextBanner_1, Poster_1, StateNames_27) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DragPosterGame extends Poster_1.Poster {
        constructor(game, posterData) {
            super(game, posterData);
            this.solvedItems = [];
            this.draggableItems = [];
            this.numCompleted = 0;
            this.dropThreshold = this.game.width * 0.15;
            this.solvedGroup = new Phaser.Group(this.game);
            this.add(this.solvedGroup);
            this.footerGroup = new Phaser.Group(this.game);
            this.add(this.footerGroup);
            // The footer panel will always be as wide as the game, and as 
            // tall as the space left by the poster (at least 15% height)
            this.footerPanel = new Phaser.Image(this.game, 0, this.game.height, 'ui-bottom-panel');
            this.footerPanel.anchor.set(0, 1);
            this.footerPanel.width = this.game.width;
            this.footerPanel.height = this.game.height - this.posterBackground.height;
            this.footerGroup.add(this.footerPanel);
            this.footerGroup.top = this.posterBackground.bottom;
            this.footerGroup.centerX = this.posterBackground.centerX;
            this.footerGroup.alpha = 0;
            this.dragGroup = new Phaser.Group(this.game);
            this.footerGroup.add(this.dragGroup);
            posterData.targetItems.forEach((targetItem, index) => {
                const solvedItem = new Phaser.Image(this.game, targetItem.pos.x * this.posterBackground.width, targetItem.pos.y * this.posterBackground.height, targetItem.solvedTag);
                solvedItem.anchor.set(0.5);
                solvedItem.alpha = 0;
                solvedItem.scale.set(this.posterBackground.scale.x);
                this.solvedGroup.add(solvedItem);
                this.solvedItems.push(solvedItem);
                // Create the draggable items that belong on the poster
                const draggableItem = new PosterDraggable_1.PosterDraggable(this.game, targetItem.dragTag, index, targetItem.pos);
                this.draggableItems.push(draggableItem);
            });
            // Set up some positioning vars
            const numItems = posterData.targetItems.length;
            const xInc = (this.posterBackground.width / (numItems + 1));
            let xPos = xInc;
            const yPos = this.footerPanel.centerY;
            Phaser.ArrayUtils.shuffle(this.draggableItems);
            this.draggableItems.forEach((draggable) => {
                draggable.events.onDragStart.add(() => {
                    this.dragStartSignal.dispatch();
                }, this);
                draggable.events.onDragStop.add(this.onDraggableReleased, this);
                draggable.anchor.set(0.5);
                ScaleUtil_21.scaleToBounds(draggable, this.posterBackground.width / this.draggableItems.length, this.game.height * 0.15);
                draggable.snapPoint = new Phaser.Point(xPos, yPos);
                this.dragGroup.add(draggable);
                xPos += xInc;
            });
        }
        set enabled(enabled) {
            this.draggableItems.forEach((item) => {
                if (item.inputEnabled) {
                    item.input.draggable = enabled;
                }
            });
        }
        onDraggableReleased(draggable) {
            return __awaiter(this, void 0, void 0, function* () {
                let placedCorrectly = false;
                draggable.inputEnabled = false;
                if (draggable.id !== undefined) {
                    const distFromTarget = Math.sqrt(Math.pow((this.solvedItems[draggable.id].worldPosition.x - draggable.worldPosition.x), 2) +
                        Math.pow((this.solvedItems[draggable.id].worldPosition.y - draggable.worldPosition.y), 2));
                    if (distFromTarget < this.dropThreshold) {
                        placedCorrectly = true;
                    }
                }
                if (placedCorrectly) {
                    // Lock in
                    yield Promise.all([
                        PromiseUtil_43.PromiseUtil.tween(this.game, this.game.add.tween(draggable)
                            .to({ x: this.posterBackground.width * draggable.destinationPos.x,
                            y: this.posterBackground.height * draggable.destinationPos.y }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                        PromiseUtil_43.PromiseUtil.tween(this.game, this.game.add.tween(this.solvedItems[draggable.id])
                            .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true)),
                        PromiseUtil_43.PromiseUtil.tween(this.game, this.game.add.tween(draggable)
                            .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                    ]);
                    this.onItemCompleted();
                }
                else {
                    this.game.sound.play('invalid_location');
                    // Snap back
                    yield PromiseUtil_43.PromiseUtil.tween(this.game, this.game.add.tween(draggable)
                        .to({
                        x: draggable.snapPoint.x,
                        y: draggable.snapPoint.y
                    }, 350, Phaser.Easing.Elastic.Out, true));
                    // Let them try again
                    draggable.inputEnabled = true;
                }
            });
        }
        onItemCompleted() {
            this.numCompleted++;
            // we need to calculate the score at each step, because the user can go back at any time
            this.trackScore();
            if (this.numCompleted >= this.solvedItems.length) {
                this.onPosterCompleted();
            }
            else {
                this.game.sound.play('valid_location');
            }
        }
        onPosterCompleted() {
            return __awaiter(this, void 0, void 0, function* () {
                GameController_25.GameController.Instance.state.addCompletedPoster(this.posterData.id);
                const completedBanner = new TextBanner_1.TextBanner(this.game, LocalizationManager_22.LocalizationManager.Instance.text("poster.completed"));
                ScaleUtil_21.scaleToBounds(completedBanner, this.posterBackground.width * 0.8, this.posterBackground.height * 0.2);
                completedBanner.position.set(this.posterBackground.centerX, this.posterBackground.height * 0.8);
                this.add(completedBanner);
                const origBannerScale = completedBanner.scale.x;
                completedBanner.scale.set(5);
                this.game.sound.play('jigsaw_complete');
                yield PromiseUtil_43.PromiseUtil.tween(this.game, this.game.add.tween(completedBanner.scale)
                    .to({ x: origBannerScale, y: origBannerScale }, 350, Phaser.Easing.Back.Out, true));
                this.completeSignal.dispatch();
            });
        }
        //// Tracking
        trackScore() {
            const score = this.numCompleted;
            let saveData = GameController_25.GameController.Instance.state.getMinigameSaveData(StateNames_27.StateNames.POSTER + "-" + this.posterData.id);
            saveData.scorePendingResultsScreen = score;
            saveData.highscore = Math.max(saveData.highscore, score);
        }
    }
    exports.DragPosterGame = DragPosterGame;
});
define("minigame/poster/PosterPreview", ["require", "exports", "component/TextBanner", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager"], function (require, exports, TextBanner_2, ScaleUtil_22, LocalizationManager_23) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PosterPreview extends Phaser.Group {
        constructor(game, posterData) {
            super(game);
            this.currentlyHighlighted = false;
            this.posterData = posterData;
            this.img = new Phaser.Image(this.game, 0, 0, this.posterData.backgroundTag);
            this.backlight = new Phaser.Image(this.game, this.img.centerX, this.img.centerY, 'poster-backlight');
            this.backlight.width = this.img.width * 1.13;
            this.backlight.height = this.img.height * 1.09;
            this.backlight.anchor.set(0.5);
            this.backlight.alpha = 0;
            this.add(this.backlight);
            this.add(this.img);
            this.completedBanner = new TextBanner_2.TextBanner(this.game, LocalizationManager_23.LocalizationManager.Instance.text("poster.completed"));
            ScaleUtil_22.scaleToBounds(this.completedBanner, this.img.width * 0.8, this.img.height * 0.2);
            this.completedBanner.position.set(this.img.centerX, this.img.height * 0.8);
            this.completedBanner.visible = false;
            this.add(this.completedBanner);
        }
        /**
         * This getter helps us avoid issues with the backlight
         * throwing off the width of the preview
         */
        get posterWidth() {
            return this.img.width * this.scale.x;
        }
        set completed(completed) {
            this.img.loadTexture(completed ? this.posterData.completeTag : this.posterData.backgroundTag);
            this.completedBanner.visible = completed;
        }
        set highlighted(highlighted) {
            if (!this.currentlyHighlighted && highlighted) {
                this.game.add.tween(this.backlight).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
            }
            else if (this.currentlyHighlighted && !highlighted) {
                this.game.add.tween(this.backlight).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
            }
            this.currentlyHighlighted = highlighted;
        }
    }
    exports.PosterPreview = PosterPreview;
});
define("minigame/poster/PosterInstallationConfirmation", ["require", "exports", "component/button/BigButton", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "component/MessageBar"], function (require, exports, BigButton_3, LocalizationManager_24, PromiseUtil_44, ScaleUtil_23, MessageBar_13) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PosterInstallationConfirmation extends Phaser.Group {
        constructor(game) {
            super(game);
            this.responseSignal = new Phaser.Signal();
            this._isShowing = false;
            this.topMessageBar = new MessageBar_13.MessageBar(this.game, LocalizationManager_24.LocalizationManager.Instance.text("poster.installation_confirmation"), 32);
            this.topMessageBar.centerX = this.game.width * 0.5;
            this.topMessageBar.centerY = -this.topMessageBar.height;
            this.yesButton = new BigButton_3.BigButton(this.game, LocalizationManager_24.LocalizationManager.Instance.text("common.yes"), 'poster-button-yes');
            ScaleUtil_23.scaleToBounds(this.yesButton, this.game.width * 0.475, this.game.height * 0.18);
            this.yesButton.left = this.game.width * 0.51;
            this.yesButton.bottom = this.game.height + this.yesButton.height;
            this.noButton = new BigButton_3.BigButton(this.game, LocalizationManager_24.LocalizationManager.Instance.text("common.no"), 'poster-button-no');
            ScaleUtil_23.scaleToBounds(this.noButton, this.game.width * 0.475, this.game.height * 0.18);
            this.noButton.right = this.game.width * 0.49;
            this.noButton.bottom = this.game.height + this.noButton.height;
            this.add(this.topMessageBar);
            this.add(this.yesButton);
            this.add(this.noButton);
            this.yesButton.onChildInputDown.add(() => {
                this.game.sound.play('snd_ui_click');
                this.responseSignal.dispatch(true);
            }, this);
            this.noButton.onChildInputDown.add(() => {
                this.game.sound.play('snd_back');
                this.responseSignal.dispatch(false);
            }, this);
        }
        isShowing() {
            return this._isShowing;
        }
        show() {
            return __awaiter(this, void 0, void 0, function* () {
                this._isShowing = true;
                PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                    .to({ centerY: this.game.height * 0.5 }, 500, Phaser.Easing.Back.Out, true));
                yield PromiseUtil_44.PromiseUtil.time(this.game, 2500);
                yield Promise.all([
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.yesButton)
                        .to({ bottom: this.game.height * 0.99 }, 300, Phaser.Easing.Back.Out, true)),
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.noButton)
                        .to({ bottom: this.game.height * 0.99 }, 300, Phaser.Easing.Back.Out, true))
                ]);
                this.yesButton.inputEnableChildren = true;
                this.noButton.inputEnableChildren = true;
                const response = yield PromiseUtil_44.PromiseUtil.signal(this.responseSignal);
                yield PromiseUtil_44.PromiseUtil.time(this.game, 200);
                return response;
            });
        }
        hide() {
            return __awaiter(this, void 0, void 0, function* () {
                this._isShowing = false;
                this.yesButton.inputEnableChildren = false;
                this.noButton.inputEnableChildren = false;
                yield Promise.all([
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                        .to({ centerY: -this.topMessageBar.height }, 500, Phaser.Easing.Back.Out, true)),
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.yesButton)
                        .to({ bottom: this.game.height + this.yesButton.height }, 300, Phaser.Easing.Back.Out, true)),
                    PromiseUtil_44.PromiseUtil.tween(this.game, this.game.add.tween(this.noButton)
                        .to({ bottom: this.game.height + this.noButton.height }, 300, Phaser.Easing.Back.Out, true))
                ]);
            });
        }
    }
    exports.PosterInstallationConfirmation = PosterInstallationConfirmation;
});
define("minigame/poster/PosterGameState", ["require", "exports", "minigame/poster/DragPosterGame", "switchback-common/util/ScaleUtil", "StateNames", "component/MessageBar", "Game", "switchback-common/util/PromiseUtil", "component/button/BigButton", "switchback-common/localization/LocalizationManager", "minigame/poster/PosterPreview", "logic/GameController", "minigame/poster/Poster", "logic/TSMAnalyticsManager", "switchback-common/util/TextUtil", "minigame/poster/PosterInstallationConfirmation", "state/GameBaseState"], function (require, exports, DragPosterGame_1, ScaleUtil_24, StateNames_28, MessageBar_14, Game_28, PromiseUtil_45, BigButton_4, LocalizationManager_25, PosterPreview_1, GameController_26, Poster_2, TSMAnalyticsManager_11, TextUtil_19, PosterInstallationConfirmation_1, GameBaseState_8) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PosterGameState extends GameBaseState_8.GameBaseState {
        constructor() {
            super(...arguments);
            this.numPosters = 0;
        }
        init() {
            this.stateName = StateNames_28.StateNames.POSTER;
            super.init();
            this.game.input.maxPointers = 1;
            this.canStartPoster = false;
            this.posterPreviews = [];
            this.canMove = false;
        }
        preload() {
            super.preload();
            this.posterDataArray = this.game.cache.getJSON('poster-data');
            this.numPosters = this.posterDataArray.length;
            this.posterDataArray.forEach((data) => {
                this.load.image(data.backgroundTag, `assets/poster/${data.backgroundTag}.jpg`);
                this.load.image(data.completeTag, `assets/poster/${data.completeTag}.jpg`);
                data.targetItems.forEach((item) => {
                    this.load.image(item.dragTag, `assets/poster/${item.dragTag}.png`);
                    this.load.image(item.solvedTag, `assets/poster/${item.solvedTag}.png`);
                }, this);
            }, this);
            this.load.image('map-bg', 'assets/hub/map-cosmetics/floor.jpg');
            this.load.image('close-game-btn', 'assets/common/exit-button.png');
            this.load.image('tiling-bar', 'assets/common/CTAbar_tilingbg.png');
            this.load.image('button-big-backplate', 'assets/title/button-big-backplate.png');
            this.load.image('button-big-frame', 'assets/title/button-big-frame.png');
            this.load.image('button-green-next', 'assets/title/button-green-next.png');
            this.load.image('button-green-next-disabled', 'assets/title/button-green-next-disabled.png');
            this.load.image('button-orange-replay', 'assets/title/button-orange-replay.png');
            this.load.image('button-green-fire', 'assets/poster/installation/button-fire.png');
            this.load.image('ui-bottom-panel', 'assets/common/ui-bottom-panel.jpg');
            this.load.image('ui-top-panel', 'assets/common/ui-top-panel.png');
            this.load.image('poster-backlight', 'assets/poster/backlight.png');
            this.load.image('button-back', 'assets/title/button-back.png');
            this.load.image('next-arrow', 'assets/poster/next_arrow.png');
            this.load.image('banner', 'assets/common/banner.png');
            // installation
            this.load.image('poster-glow-mask', 'assets/poster/installation/poster-glow-mask.png');
            this.load.image('poster-button-yes', 'assets/poster/installation/poster-button-yes.png');
            this.load.image('poster-button-no', 'assets/poster/installation/poster-button-no.png');
            this.load.image('inst-cannon', 'assets/poster/installation/cannon.png');
            this.load.image('inst-dart', 'assets/poster/installation/dart.png');
            this.game.load.spritesheet('inst-loot', "assets/poster/installation/loot-sheet.png", 230, 230);
            this.game.load.spritesheet('balloon-orange-pop', "assets/minigame/trixie/balloons/balloon_orange_pop.png", 495, 539);
            this.load.audio('invalid_location', ['assets/sound/poster/invalid_location.mp4', 'assets/sound/poster/invalid_location.ogg']);
            this.load.audio('jigsaw_complete', ['assets/sound/poster/jigsaw_complete.mp4', 'assets/sound/poster/jigsaw_complete.ogg']);
            this.load.audio('valid_location', ['assets/sound/poster/valid_location.mp4', 'assets/sound/poster/valid_location.ogg']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                const bg = new Phaser.Image(this.game, 0, 0, 'map-bg');
                bg.width = this.game.width;
                bg.height = this.game.height;
                this.add.existing(bg);
                const overlay = new Phaser.Graphics(this.game);
                overlay.beginFill(0x000000, 0.75);
                overlay.drawRect(0, 0, this.game.width, this.game.height);
                overlay.endFill();
                this.add.existing(overlay);
                this.glowMask = new Phaser.Image(this.game, 0, 0, 'poster-glow-mask');
                this.glowMask.width = this.game.width;
                this.glowMask.height = this.game.height;
                this.add.existing(this.glowMask);
                this.glowMask.visible = false;
                ////////////////////////////////////
                // POSTER PREVIEWS
                ///////////////////////////////////
                const scrollingPosterGroup = new Phaser.Group(this.game);
                this.posterWidth = this.game.width * 0.75;
                this.posterXPad = this.game.width * 0;
                let xPos = this.posterXPad;
                for (let i = 0; i < this.numPosters; i++) {
                    const completed = GameController_26.GameController.Instance.state.getHasCompletedPoster(this.posterDataArray[i].id);
                    const posterPreview = new PosterPreview_1.PosterPreview(this.game, this.posterDataArray[i]);
                    posterPreview.completed = completed;
                    posterPreview.width = this.posterWidth;
                    posterPreview.scale.y = posterPreview.scale.x;
                    posterPreview.x = xPos;
                    xPos += (this.posterWidth + (this.posterXPad * 2));
                    scrollingPosterGroup.add(posterPreview);
                    this.posterPreviews.push(posterPreview);
                }
                ////////////////////////////////////
                // POSTER SCROLLING SPRITE
                ///////////////////////////////////
                this.posterScroller = new Phaser.Sprite(this.game, 0, 0);
                this.posterScroller.width = scrollingPosterGroup.width;
                this.posterScroller.height = scrollingPosterGroup.height;
                this.posterScroller.addChild(scrollingPosterGroup);
                scrollingPosterGroup.scale.x = 1 / this.posterScroller.scale.x;
                scrollingPosterGroup.scale.y = 1 / this.posterScroller.scale.y;
                // Center the first poster on the screen
                this.posterScroller.x = (this.game.width * 0.5) - ((this.posterPreviews[0].posterWidth * 0.5) + this.posterXPad);
                this.origPosterScrollerOffset = this.posterScroller.x;
                this.posterScroller.centerY = this.game.height * 0.5;
                this.posterScroller.inputEnabled = true;
                this.posterScroller.input.draggable = true;
                this.posterScroller.input.allowVerticalDrag = false;
                this.posterScroller.events.onDragUpdate.add(this.onDragUpdate, this);
                this.posterScroller.events.onDragStop.add(this.onDragStop, this);
                this.add.existing(this.posterScroller);
                ////////////////////////////////////
                // UI GROUPS
                ///////////////////////////////////
                this.activePosterGroup = new Phaser.Group(this.game);
                this.add.existing(this.activePosterGroup);
                this.topUIGroup = new Phaser.Group(this.game);
                this.add.existing(this.topUIGroup);
                this.bottomUIGroup = new Phaser.Group(this.game);
                this.add.existing(this.bottomUIGroup);
                ////////////////////////////////////
                // HEADER
                ///////////////////////////////////
                this.header = new Phaser.Image(this.game, 0, 0, 'ui-top-panel');
                this.header.width = this.game.width;
                this.header.height = this.game.height * 0.225;
                this.topUIGroup.add(this.header);
                this.headerText = new Phaser.Text(this.game, this.game.width * 0.5, this.header.height * 0.57, LocalizationManager_25.LocalizationManager.Instance.text("poster.poster_caption"), {
                    font: Game_28.Game.DEFAULT_FONT,
                    fill: Game_28.Game.TEXT_COLOR_WHITE,
                    fontSize: Game_28.Game.FONT_SIZE_MEDIUM,
                    wordWrap: true,
                    wordWrapWidth: this.header.width * 0.6,
                    align: 'center'
                });
                this.headerText.lineSpacing = this.headerText.fontSize * -0.20;
                this.headerText.anchor.set(0.5);
                this.headerText.setShadow(Game_28.Game.TEXT_SHADOW_OFFSET_SMALL_X, Game_28.Game.TEXT_SHADOW_OFFSET_SMALL_Y, Game_28.Game.TEXT_SHADOW_DIAG, 0, true, true);
                this.topUIGroup.add(this.headerText);
                TextUtil_19.TextUtil.scaleDownTextToHeight(this.headerText, this.header.height * 0.47);
                ////////////////////////////////////
                // QUIT BUTTON
                ///////////////////////////////////
                const quitButton = new Phaser.Image(this.game, 0, 0, 'button-back');
                ScaleUtil_24.scaleToBounds(quitButton, this.game.width * 0.15, this.game.height * 0.1);
                quitButton.anchor.set(0.5);
                quitButton.top = this.game.height * 0.01;
                quitButton.left = this.game.width * 0.02;
                quitButton.inputEnabled = true;
                quitButton.events.onInputDown.addOnce(this.onQuitButton, this);
                this.topUIGroup.add(quitButton);
                ////////////////////////////////////
                // FOOTER
                ///////////////////////////////////
                this.footer = new Phaser.Image(this.game, 0, this.game.height, 'ui-bottom-panel');
                this.footer.anchor.set(0, 1);
                this.footer.width = this.game.width;
                this.footer.height = this.game.height * 0.2;
                this.bottomUIGroup.add(this.footer);
                ////////////////////////////////////
                // PLAY BUTTON
                ///////////////////////////////////
                this.buttonPlay = new BigButton_4.BigButton(this.game, LocalizationManager_25.LocalizationManager.Instance.text("common.play"), 'button-green-next', 'button-green-next-disabled');
                this.buttonPlay.height = this.game.height * 0.23;
                this.buttonPlay.scale.x = this.buttonPlay.scale.y;
                this.buttonPlay.centerX = this.game.width * 0.5;
                this.buttonPlay.bottom = this.game.height * 0.99;
                this.bottomUIGroup.add(this.buttonPlay);
                ////////////////////////////////////
                // BACK BUTTON
                ///////////////////////////////////
                this.backButton = new Phaser.Image(this.game, 0, 0, 'button-back');
                ScaleUtil_24.scaleToBounds(this.backButton, this.game.width * 0.15, this.game.height * 0.1);
                this.backButton.anchor.set(0.5);
                this.backButton.top = this.game.height * 0.01;
                this.backButton.left = this.game.width * 0.02;
                this.backButton.inputEnabled = true;
                this.backButton.visible = false;
                this.add.existing(this.backButton);
                ////////////////////////////////////
                // ARROW BUTTONS
                ///////////////////////////////////
                this.leftArrow = new Phaser.Image(this.game, 0, 0, 'next-arrow');
                ScaleUtil_24.scaleToBounds(this.leftArrow, this.game.width * 0.2, this.game.height * 0.15);
                this.leftArrow.scale.x *= -1;
                this.leftArrow.anchor.set(0.5);
                this.leftArrow.y = this.posterScroller.centerY;
                this.leftArrow.x = this.origPosterScrollerOffset;
                this.leftArrow.events.onInputDown.add(this.prevPage, this);
                this.leftArrow.alpha = 0;
                this.add.existing(this.leftArrow);
                this.game.add.tween(this.leftArrow)
                    .to({ right: 0 }, 300, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                this.rightArrow = new Phaser.Image(this.game, 0, 0, 'next-arrow');
                this.rightArrow.scale.set(this.leftArrow.scale.y);
                this.rightArrow.anchor.set(0.5);
                this.rightArrow.y = this.posterScroller.centerY;
                this.rightArrow.x = this.game.width - this.leftArrow.x;
                this.rightArrow.events.onInputDown.add(this.nextPage, this);
                this.add.existing(this.rightArrow);
                this.game.add.tween(this.rightArrow)
                    .to({ right: this.game.width }, 300, Phaser.Easing.Sinusoidal.InOut, true, 0, Infinity, true);
                this.messageBar = new MessageBar_14.MessageBar(this.game, LocalizationManager_25.LocalizationManager.Instance.text("poster.instructions"), 30);
                this.messageBar.centerX = this.game.width * 0.5;
                this.messageBar.top = this.game.height;
                this.add.existing(this.messageBar);
                this.installationConfirmation = new PosterInstallationConfirmation_1.PosterInstallationConfirmation(this.game);
                this.add.existing(this.installationConfirmation);
                this.initialPlayButtonY = this.buttonPlay.y;
                yield this.dismissOverlay();
                this.start();
            });
        }
        start() {
            return __awaiter(this, void 0, void 0, function* () {
                this.canMove = true;
                this.canStartPoster = true;
                this.leftArrow.inputEnabled = true;
                this.rightArrow.inputEnabled = true;
                this.buttonPlay.onChildInputDown.add(this.onPosterSelected, this);
                // Analytics
                TSMAnalyticsManager_11.TSMAnalyticsManager.Instance.onGameTime(this.stateName);
                // ~
                // start on page one and animate to the previously displayed page
                const prevPosterShown = GameController_26.GameController.Instance.state.lastPosterShown;
                this.setCurrentPoster(0, true);
                yield this.animateToPoster(prevPosterShown);
            });
        }
        onQuitButton() {
            this.game.sound.play('snd_ui_click');
            // Analytics
            const score = GameController_26.GameController.Instance.state.getMinigameSaveData(this.stateName).totalScore;
            TSMAnalyticsManager_11.TSMAnalyticsManager.Instance.onGameTime(this.stateName, score, true);
            TSMAnalyticsManager_11.TSMAnalyticsManager.Instance.onPosterHubExit();
            // ~
            this.transitionTo(StateNames_28.StateNames.HUB);
        }
        onInstallationPosterSelected() {
            return __awaiter(this, void 0, void 0, function* () {
                TSMAnalyticsManager_11.TSMAnalyticsManager.Instance.onInstallationWayfindState();
                let response = false;
                // hide the play button and transition on the confirm buttons
                PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.buttonPlay)
                    .to({ y: this.game.height + this.buttonPlay.height }, 300, Phaser.Easing.Back.Out, true));
                response = yield this.installationConfirmation.show();
                TSMAnalyticsManager_11.TSMAnalyticsManager.Instance.onInstallationWayfind(response);
                yield this.installationConfirmation.hide();
                if (!response) {
                    // reset the state back to as it was
                    PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.buttonPlay)
                        .to({ y: this.initialPlayButtonY }, 300, Phaser.Easing.Back.Out, true));
                }
                return response;
            });
        }
        onPosterSelected() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[PosterGameState] onPosterSelected() ${this.currentPosterNum}`);
                if (!this.canStartPoster) {
                    return;
                }
                this.game.sound.play('snd_ui_click');
                // Lock the behaviour of the scrolling list
                this.canMove = false;
                this.canStartPoster = false;
                this.posterScroller.input.disableDrag();
                // If it's a regular poster, proceed to create a poster game for them.
                let shouldCreatePoster = true;
                // Is this the special installation poster that they have to confirm via a dialog?
                if (this.isCurrentPosterInstallation) {
                    shouldCreatePoster = yield this.onInstallationPosterSelected();
                    if (shouldCreatePoster) {
                        this.canMove = false;
                        TSMAnalyticsManager_11.TSMAnalyticsManager.Instance.onPosterInstallationTime(true, true);
                        TSMAnalyticsManager_11.TSMAnalyticsManager.Instance.onPosterInstallationTime(true, false);
                        yield this.showOverlay();
                        this.game.state.start(StateNames_28.StateNames.POSTER_INSTALLATION);
                        return;
                    }
                }
                if (shouldCreatePoster) {
                    this.currentPoster = this.createPoster(this.currentPosterNum);
                    // Bring it in, wait for them to finish playing it, then hide it.
                    yield this.animatePlayablePosterIn(this.currentPoster);
                    yield this.playPoster(this.currentPoster);
                    if (this.isCurrentPosterInstallation) {
                        // stop timing
                        TSMAnalyticsManager_11.TSMAnalyticsManager.Instance.onPosterInstallationTime(false, true);
                    }
                    // If they completed the poster, we should ensure our preview reflects that completion state.
                    if (GameController_26.GameController.Instance.state.getHasCompletedPoster(this.posterDataArray[this.currentPosterNum].id)) {
                        this.posterPreviews[this.currentPosterNum].completed = true;
                    }
                    yield this.animatePlayablePosterOut(this.currentPoster);
                    // Clean up
                    this.currentPoster.destroy();
                }
                // Resume normal behaviour of the scrolling list
                this.canStartPoster = true;
                this.canMove = true;
                this.posterScroller.input.enableDrag();
            });
        }
        createPoster(posterNum) {
            let poster = new DragPosterGame_1.DragPosterGame(this.game, this.posterDataArray[posterNum]);
            poster.width = this.posterPreviews[posterNum].posterWidth;
            poster.scale.y = poster.scale.x;
            poster.left = this.posterPreviews[posterNum].worldPosition.x;
            poster.top = this.posterPreviews[posterNum].worldPosition.y;
            poster.alpha = 0;
            this.activePosterGroup.add(poster);
            return poster;
        }
        playPoster(poster) {
            return __awaiter(this, void 0, void 0, function* () {
                poster.enabled = true;
                // Show the tutorial if this is a regular poster.
                if (poster.posterData.id !== PosterGameState.INSTALLATION_POSTER_ID) {
                    this.game.add.tween(this.messageBar)
                        .to({ bottom: poster.footerPanel.top }, 350, Phaser.Easing.Sinusoidal.InOut, true);
                    poster.dragStartSignal.addOnce(() => {
                        this.game.tweens.removeFrom(this.messageBar);
                        this.game.add.tween(this.messageBar)
                            .to({ top: this.game.height }, 350, Phaser.Easing.Sinusoidal.InOut, true);
                    }, this);
                }
                else {
                    this.backButton.visible = false; // this works. disable back button on installation poster
                }
                // Wait until they either finish the poster, or decide to stop
                yield Promise.race([
                    PromiseUtil_45.PromiseUtil.signal(this.backButton.events.onInputDown),
                    PromiseUtil_45.PromiseUtil.signal(poster.completeSignal),
                ]);
                this.game.add.tween(this.messageBar)
                    .to({ top: this.game.height }, 350, Phaser.Easing.Sinusoidal.InOut, true);
                poster.enabled = false;
                this.updateScore();
            });
        }
        /**
         * Expands @param poster to a playable size
         */
        animatePlayablePosterIn(poster) {
            return __awaiter(this, void 0, void 0, function* () {
                const durationMs = 750;
                this.hideUI(durationMs * 0.35);
                // Get rid of the arrows, and hide them when complete.
                this.game.add.tween(this.rightArrow)
                    .to({ alpha: 0 }, durationMs * 0.2, Phaser.Easing.Linear.None, true).onComplete.addOnce(() => {
                    this.rightArrow.visible = false;
                });
                this.game.add.tween(this.leftArrow)
                    .to({ alpha: 0 }, durationMs * 0.2, Phaser.Easing.Linear.None, true).onComplete.addOnce(() => {
                    this.leftArrow.visible = false;
                });
                yield PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(poster)
                    .to({ alpha: 1 }, durationMs * 0.2, Phaser.Easing.Linear.None, true));
                // Fade out the scrolling list, and block until it's disappeared.
                yield PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.posterScroller)
                    .to({ alpha: 0 }, durationMs * 0.3, Phaser.Easing.Linear.None, true));
                // With the scrolling list gone, we can deal with our single poster object.
                // Figure out what scale we need to be moving it to
                const origScale = poster.scale.x;
                poster.height = this.game.height;
                const scaleDest = poster.scale.y;
                this.currentPoster.scale.set(origScale);
                yield Promise.all([
                    PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.currentPoster.scale)
                        .to({ x: scaleDest, y: scaleDest }, durationMs * 0.5, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.currentPoster)
                        .to({ top: 0 }, durationMs * 0.5, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.currentPoster)
                        .to({ centerX: this.game.width * 0.5 }, durationMs * 0.5, Phaser.Easing.Sinusoidal.InOut, true))
                ]);
                poster.show(durationMs * 0.5);
                // The back button can now come in
                const backButtonOrigScale = this.backButton.scale.x;
                this.backButton.scale.set(0);
                this.backButton.visible = true;
                this.game.add.tween(this.backButton.scale)
                    .to({ x: backButtonOrigScale, y: backButtonOrigScale }, durationMs * 0.35, Phaser.Easing.Back.Out, true);
            });
        }
        /**
         * Returns @param poster to the size of the
         * preview images in the scrolling list
         */
        animatePlayablePosterOut(poster) {
            return __awaiter(this, void 0, void 0, function* () {
                const durationMs = 1000;
                // Get rid of the back button over the poster
                const backButtonOrigScale = this.backButton.scale.x;
                yield Promise.all([
                    poster.hide(durationMs * 0.2),
                    PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.backButton.scale)
                        .to({ x: 0, y: 0 }, 200, Phaser.Easing.Back.In, true)),
                ]);
                this.backButton.visible = false;
                this.backButton.scale.set(backButtonOrigScale);
                const scaleTween = this.game.add.tween(poster)
                    .to({ width: this.posterPreviews[this.currentPosterNum].posterWidth }, durationMs * 0.5, Phaser.Easing.Sinusoidal.InOut, true);
                scaleTween.onUpdateCallback(() => {
                    poster.scale.y = poster.scale.x;
                }, this);
                yield Promise.all([
                    PromiseUtil_45.PromiseUtil.tween(this.game, scaleTween),
                    PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(poster)
                        .to({ top: this.posterPreviews[this.currentPosterNum].worldPosition.y }, durationMs * 0.5, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(poster)
                        .to({ left: this.origPosterScrollerOffset + this.posterXPad }, durationMs * 0.5, Phaser.Easing.Sinusoidal.InOut, true))
                ]);
                this.showUI(durationMs * 0.5);
                // Show the scroller and the arrows again 
                this.leftArrow.visible = this.rightArrow.visible = true;
                yield Promise.all([
                    PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.posterScroller)
                        .to({ alpha: 1 }, durationMs * 0.35, Phaser.Easing.Linear.None, true)),
                    this.animateArrows(this.currentPosterNum)
                ]);
                yield PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(poster)
                    .to({ alpha: 0 }, durationMs * 0.2, Phaser.Easing.Linear.None, true));
            });
        }
        showUI(durationMs) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this.topUIGroup)
                    .to({ top: 0 }, durationMs, Phaser.Easing.Sinusoidal.Out, true);
                this.game.add.tween(this.bottomUIGroup)
                    .to({ bottom: this.game.height }, durationMs, Phaser.Easing.Sinusoidal.Out, true);
            });
        }
        hideUI(durationMs) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.add.tween(this.topUIGroup)
                    .to({ bottom: 0 }, durationMs, Phaser.Easing.Sinusoidal.In, true);
                this.game.add.tween(this.bottomUIGroup)
                    .to({ top: this.game.height }, durationMs, Phaser.Easing.Sinusoidal.In, true);
            });
        }
        onDragUpdate() {
            return __awaiter(this, void 0, void 0, function* () {
                // If the user is trying to scroll past the start/end of the content
                if (this.posterScroller.left > this.game.width * 0.5 || this.posterScroller.right < this.game.width * 0.5) {
                    this.snapToCurrentPoster();
                }
            });
        }
        onDragStop(sprite, pointer) {
            return __awaiter(this, void 0, void 0, function* () {
                const movement = pointer.positionDown.x - pointer.positionUp.x;
                const threshold = this.game.width * PosterGameState.X_DRAG_THRESHOLD_FOR_NEXT_PAGE;
                if (movement < -threshold) {
                    this.animateToPoster(this.currentPosterNum - 1);
                }
                else if (movement > threshold) {
                    this.animateToPoster(this.currentPosterNum + 1);
                }
                else {
                    this.animateToPoster(this.currentPosterNum);
                }
            });
        }
        prevPage() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.canMove) {
                    if (this.installationConfirmation.isShowing) {
                        this.installationConfirmation.hide();
                    }
                    this.game.sound.play('snd_ui_click');
                    this.game.add.tween(this.leftArrow.scale)
                        .to({ x: this.leftArrow.scale.x * 0.75, y: this.leftArrow.scale.y * 0.75 }, 100, Phaser.Easing.Linear.None, true, 0, 0, true);
                    yield this.animateToPoster(this.currentPosterNum - 1);
                }
            });
        }
        nextPage() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.canMove) {
                    this.game.sound.play('snd_ui_click');
                    this.game.add.tween(this.rightArrow.scale)
                        .to({ x: this.rightArrow.scale.x * 0.75, y: this.rightArrow.scale.y * 0.75 }, 100, Phaser.Easing.Linear.None, true, 0, 0, true);
                    yield this.animateToPoster(this.currentPosterNum + 1);
                }
            });
        }
        snapToCurrentPoster() {
            return __awaiter(this, void 0, void 0, function* () {
                const currentXOffsetOfGroup = -this.posterScroller.x;
                const totalPosterWidth = this.posterWidth + (this.posterXPad * 2);
                let posterNum = Math.round(currentXOffsetOfGroup / totalPosterWidth);
                yield this.animateToPoster(posterNum);
            });
        }
        /**
         * Animate to a specific @param posterNum. Before we do this
         * we should:
         * * Ensure the value is within an acceptable range
         * * Disallow dragging of the poster group
         * * Disallow starting a new poster
         *
         * Then we can calculate the correct position that the scrolling
         * group needs to be in in order to center our desired poster.
         * Once we have that, we can tween to it and re-enable dragging.
         */
        animateToPoster(posterNum) {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.canMove) {
                    return;
                }
                // disable arrow buttons while transition happens
                this.leftArrow.events.onInputDown.remove(this.prevPage, this);
                this.rightArrow.events.onInputDown.remove(this.nextPage, this);
                const wasOnPosterInstallation = this.isCurrentPosterInstallation;
                this.canMove = false;
                this.canStartPoster = false;
                this.posterScroller.input.disableDrag();
                posterNum = Math.min(Math.max(0, posterNum), this.numPosters - 1);
                const isNewPage = this.setCurrentPoster(posterNum, false);
                this.animateArrows(posterNum);
                if (this.isCurrentPosterInstallation &&
                    GameController_26.GameController.Instance.state.getHasCompletedPoster(this.posterDataArray[posterNum].id)) {
                    this.buttonPlay.disable();
                }
                else {
                    this.buttonPlay.enable();
                }
                // Move the scroller along
                const totalPosterWidth = this.posterWidth + (this.posterXPad * 2);
                const xDest = -(posterNum * totalPosterWidth) + this.origPosterScrollerOffset;
                if (this.isCurrentPosterInstallation) {
                    yield this.transitionInstallationPoster(xDest, "poster.finale", isNewPage);
                }
                else if (wasOnPosterInstallation) {
                    yield this.transitionInstallationPoster(xDest, "common.play", isNewPage);
                }
                else {
                    // scroll to new page
                    yield PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.posterScroller)
                        .to({ x: xDest }, 350, Phaser.Easing.Back.Out, true));
                }
                if (isNewPage) {
                    if (this.isCurrentPosterInstallation) {
                        this.headerText.text = LocalizationManager_25.LocalizationManager.Instance.text("poster.installation_caption");
                    }
                    else {
                        this.headerText.text = LocalizationManager_25.LocalizationManager.Instance.text("poster.poster_caption");
                    }
                    this.headerText.fontSize = Game_28.Game.FONT_SIZE_MEDIUM;
                    TextUtil_19.TextUtil.scaleDownTextToHeight(this.headerText, this.header.height * 0.47);
                }
                this.canStartPoster = true;
                this.posterScroller.input.enableDrag();
                this.canMove = true;
                // reenable the appropriate buttons
                if (posterNum < this.numPosters - 1) {
                    this.rightArrow.events.onInputDown.add(this.nextPage, this);
                }
                if (posterNum > 0) {
                    this.leftArrow.events.onInputDown.add(this.prevPage, this);
                }
            });
        }
        transitionInstallationPoster(xDest, buttonTextTag, isNewPage) {
            return __awaiter(this, void 0, void 0, function* () {
                if (isNewPage) {
                    // send button off screen
                    PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.buttonPlay)
                        .to({ y: this.game.height + this.buttonPlay.height }, 300, Phaser.Easing.Back.Out, true));
                    // stop the background pulsing
                    if (this.backgroundPulseTween) {
                        this.backgroundPulseTween.stop();
                        this.backgroundPulseTween = null;
                    }
                    if (!this.isCurrentPosterInstallation) {
                        // fade out the pulsing background
                        this.game.add.tween(this.glowMask)
                            .to({ alpha: 0 }, 400, Phaser.Easing.Cubic.InOut, true).onComplete.addOnce(() => {
                            this.glowMask.visible = false;
                        });
                    }
                }
                // animate to next page
                yield PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.posterScroller)
                    .to({ x: xDest }, 350, Phaser.Easing.Back.Out, true));
                if (isNewPage) {
                    // swap text on button and animate it back in
                    this.buttonPlay.text = LocalizationManager_25.LocalizationManager.Instance.text(buttonTextTag);
                    yield PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.buttonPlay)
                        .to({ y: this.initialPlayButtonY }, 300, Phaser.Easing.Back.Out, true));
                    if (this.isCurrentPosterInstallation) {
                        // fade in then pulse the white background
                        this.glowMask.visible = true;
                        this.glowMask.alpha = 0;
                        yield PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.glowMask)
                            .to({ alpha: 1.0 }, 500, Phaser.Easing.Cubic.InOut, true));
                        this.glowMask.alpha = 1;
                        this.backgroundPulseTween = this.game.add.tween(this.glowMask)
                            .to({ alpha: 0.2 }, 750, Phaser.Easing.Cubic.InOut, true, 0, 0, true).loop(true);
                    }
                }
            });
        }
        setCurrentPoster(posterNum, isForced) {
            posterNum = Math.min(Math.max(0, posterNum), this.numPosters - 1);
            // Set highlighted states of previews accordingly
            this.posterPreviews.forEach((preview, index) => {
                preview.highlighted = index === posterNum;
            }, this);
            // don't change poster if its the same one it is already set to so that the analaytics aren't spammed
            if (!isForced && posterNum === GameController_26.GameController.Instance.state.lastPosterShown) {
                return false;
            }
            GameController_26.GameController.Instance.state.lastPosterShown = this.currentPosterNum = posterNum;
            this.isCurrentPosterInstallation = this.posterDataArray[posterNum].id === PosterGameState.INSTALLATION_POSTER_ID;
            if (!isForced) {
                this.trackPosterShown();
            }
            return true;
        }
        /**
         * Performs fade  in or out tweens on the arrows
         * based on the current poster number
         */
        animateArrows(posterNum) {
            return __awaiter(this, void 0, void 0, function* () {
                const leftArrowAlphaDest = (posterNum <= 0) ?
                    0 : 1;
                const rightArrowAlphaDest = (posterNum >= (this.numPosters - 1)) ?
                    0 : 1;
                const fadePromises = [];
                if (leftArrowAlphaDest !== this.leftArrow.alpha) {
                    fadePromises.push(PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.leftArrow)
                        .to({ alpha: leftArrowAlphaDest }, 200, Phaser.Easing.Linear.None, true)));
                }
                if (rightArrowAlphaDest !== this.rightArrow.alpha) {
                    fadePromises.push(PromiseUtil_45.PromiseUtil.tween(this.game, this.game.add.tween(this.rightArrow)
                        .to({ alpha: rightArrowAlphaDest }, 200, Phaser.Easing.Linear.None, true)));
                }
                yield Promise.all(fadePromises);
            });
        }
        updateScore() {
            let score = this.getTrackedTotalScore();
            let saveData = GameController_26.GameController.Instance.state.getMinigameSaveData(StateNames_28.StateNames.POSTER);
            saveData.highscore = Math.max(saveData.highscore, score);
            saveData.totalScore = saveData.highscore;
            saveData.numPlays++;
            GameController_26.GameController.Instance.state.allPlays++;
        }
        getTrackedTotalScore() {
            let totalScore = 0;
            for (let poster of this.posterDataArray) {
                totalScore += GameController_26.GameController.Instance.state.getHasCompletedPoster(poster.id) ? 1 : 0;
            }
            return totalScore;
        }
        trackPosterShown() {
            const thisPoster = this.posterDataArray[this.currentPosterNum];
            let micro = Poster_2.Poster.MICRO_NAMES_FOR_ID.get(thisPoster.id);
            if (!micro) {
                // fix so we can track "looking" at the installation poster (should not be in MICRO_NAMES_FOR_ID or it will mess with other posters)
                if (thisPoster.id === "volcano") {
                    micro = "Volcano"; // cosmetic
                }
                else {
                    return;
                }
            }
            TSMAnalyticsManager_11.TSMAnalyticsManager.Instance.onPosterTitleState(micro, this.currentPosterNum + 1, this.posterDataArray.length);
        }
        shutdown() {
            if (this.backgroundPulseTween) {
                this.backgroundPulseTween.stop();
                this.backgroundPulseTween = null;
            }
            this.posterDataArray.forEach((data) => {
                this.game.cache.removeImage(data.backgroundTag);
                this.game.cache.removeImage(data.completeTag);
                data.targetItems.forEach((item) => {
                    this.game.cache.removeImage(item.dragTag);
                    this.game.cache.removeImage(item.solvedTag);
                }, this);
            }, this);
            this.glowMask = null;
            this.messageBar = null;
            this.activePosterGroup = null;
            this.topUIGroup = null;
            this.bottomUIGroup = null;
            this.header = null;
            this.headerText = null;
            this.footer = null;
            this.backButton = null;
            this.installationConfirmation = null;
            this.buttonPlay = null;
            this.leftArrow = null;
            this.rightArrow = null;
            this.currentPoster = null;
            this.posterScroller = null;
            this.posterPreviews = null;
            this.posterDataArray = null;
            this.cache.removeImage('map-bg');
            this.cache.removeImage('button-green-next-disabled');
            this.cache.removeImage('button-green-fire');
            this.cache.removeImage('poster-backlight');
            // installation
            this.cache.removeImage('poster-glow-mask');
            this.cache.removeImage('poster-button-yes');
            this.cache.removeImage('poster-button-no');
            this.cache.removeImage('inst-cannon');
            this.cache.removeImage('inst-dart');
            this.cache.removeImage('inst-loot');
            this.cache.removeImage('balloon-orange-pop');
            this.cache.removeSound('invalid_location');
            this.cache.removeSound('jigsaw_complete');
            this.cache.removeSound('valid_location');
            super.shutdown();
        }
    }
    PosterGameState.INSTALLATION_POSTER_ID = "volcano";
    PosterGameState.X_DRAG_THRESHOLD_FOR_NEXT_PAGE = 0.2;
    exports.PosterGameState = PosterGameState;
});
define("component/GoalBarBig", ["require", "exports", "Game", "switchback-common/util/PromiseUtil", "component/fx/Snatch"], function (require, exports, Game_29, PromiseUtil_46, Snatch_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class GoalBarBig extends Phaser.Group {
        constructor(game, total, current, icon) {
            super(game);
            this.maxScore = total;
            this.currentScore = Math.min(this.maxScore, current);
            const proportionComplete = this.currentScore / this.maxScore;
            ////////////////////////////
            // BAR
            ////////////////////////////
            this.frame = new Phaser.Image(this.game, 0, 0, 'goal-bar-frame');
            this.frame.anchor.set(0, 1);
            this.sign = new Phaser.Sprite(this.game, 0, 0, 'goal-bar-sign', 0);
            this.sign.width = this.frame.width;
            this.sign.scale.y = this.sign.scale.y;
            this.sign.top = this.frame.top;
            if (current >= total) {
                this.sign.frame = 1;
            }
            this.snatch = new Snatch_3.Snatch(this.game);
            this.barOverlay = new Phaser.Image(this.game, 0, 0, 'goal-bar-overlay');
            this.barOverlay.width = this.frame.width * 0.83;
            this.barOverlay.height = this.frame.height * 0.815;
            this.barOverlay.anchor.set(0, 1);
            this.barOverlay.alpha = 0.6;
            ////////////////////////////
            // TEXT LABELS
            ////////////////////////////
            const numLabels = 5;
            this.textLabelGroup = new Phaser.Group(this.game);
            for (let i = 0; i <= numLabels; i++) {
                const label = new Phaser.Text(this.game, this.barOverlay.centerX, this.barOverlay.bottom - ((this.barOverlay.height / numLabels) * i), ((this.maxScore / numLabels) * i).toFixed(0), {
                    font: Game_29.Game.DEFAULT_FONT,
                    fill: Game_29.Game.TEXT_COLOR_WHITE,
                    stroke: Game_29.Game.TEXT_COLOR_CHAR_STROKE,
                    strokeThickness: Game_29.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                    fontSize: 75,
                    align: 'center'
                });
                label.anchor.set(0.5);
                this.textLabelGroup.add(label);
            }
            ////////////////////////////
            // COLORED PROGRESS RECTS
            ////////////////////////////
            this.unachievedRect = new Phaser.Image(this.game, this.barOverlay.left, this.barOverlay.top, 'goal-bar-fill');
            this.unachievedRect.width = this.barOverlay.width;
            this.unachievedRect.height = this.barOverlay.height * (1 - proportionComplete);
            this.unachievedRect.anchor.set(0, 0);
            this.unachievedRect.tint = 0x553cc9;
            // currently achieved
            this.latestRect = new Phaser.Image(this.game, this.barOverlay.left, this.unachievedRect.bottom, 'goal-bar-fill');
            this.latestRect.anchor.set(0, 1);
            this.latestRect.width = this.barOverlay.width;
            this.latestRect.height = 0;
            this.latestRect.tint = 0xffffff;
            // previously achieved bar
            this.achievedRect = new Phaser.Image(this.game, this.barOverlay.left, this.barOverlay.bottom, 'goal-bar-fill');
            this.achievedRect.anchor.set(0, 1);
            this.achievedRect.width = this.barOverlay.width;
            this.achievedRect.height = this.barOverlay.height * proportionComplete;
            this.achievedRect.tint = 0xffc222;
            ////////////////////////////
            // SCORE MARKER GROUP
            ////////////////////////////
            this.markerGroup = new Phaser.Group(this.game);
            const marker = new Phaser.Image(this.game, 0, 0, 'score-marker');
            marker.anchor.set(0, 0.5);
            const iconImg = new Phaser.Image(this.game, marker.right, marker.centerY, icon);
            iconImg.anchor.set(0.2, 0.5);
            this.markerGroup.left = 0;
            this.markerGroup.centerY = this.latestRect.top;
            this.markerGroup.add(marker);
            this.markerGroup.add(iconImg);
            ////////////////////////////
            // ORDERING
            ////////////////////////////
            this.add(this.unachievedRect);
            this.add(this.latestRect);
            this.add(this.achievedRect);
            this.add(this.barOverlay);
            this.add(this.frame);
            this.add(this.sign);
            this.add(this.snatch);
            this.add(this.markerGroup);
            this.add(this.textLabelGroup);
        }
        addScore(scoreGained) {
            return __awaiter(this, void 0, void 0, function* () {
                const scoreToAdd = Math.min(scoreGained, this.maxScore - this.currentScore);
                this.currentScore += scoreToAdd;
                const proportionGained = scoreToAdd / this.maxScore;
                const distanceMoved = this.barOverlay.height * proportionGained;
                if (scoreGained < 0) {
                    this.latestRect.tint = 0xFF0000;
                }
                let sndQuotaFilling;
                if (scoreToAdd > 0) {
                    sndQuotaFilling = this.game.sound.play('quota_filling');
                }
                const maxDuration = 2000;
                const duration = maxDuration * proportionGained;
                // Animate 1st bar
                Promise.all([
                    PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.latestRect)
                        .to({ height: this.latestRect.height + distanceMoved }, duration / 2, Phaser.Easing.Sinusoidal.In, true)),
                    PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.unachievedRect)
                        .to({ height: this.unachievedRect.height - distanceMoved }, duration / 2, Phaser.Easing.Sinusoidal.In, true)),
                ]);
                // notice no await on the 1st bar, but wait on a timer, to give it 100 ms in advance and then overlap the animations
                yield PromiseUtil_46.PromiseUtil.time(this.game, duration * 0.1);
                // Animate 2nd bar
                yield Promise.all([
                    PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.achievedRect)
                        .to({ height: this.achievedRect.height + distanceMoved }, duration, Phaser.Easing.Sinusoidal.In, true)),
                    PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.markerGroup)
                        .to({ centerY: this.markerGroup.centerY - distanceMoved }, duration, Phaser.Easing.Sinusoidal.In, true))
                ]);
                if (sndQuotaFilling) {
                    sndQuotaFilling.stop();
                }
                this.markerGroup.scale.set(1, 1);
                // Scales the marker when it reaches the end, notice is not being awaited, so its synced with the overflow animation on filling
                PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.markerGroup.scale)
                    .to({ x: 1.15, y: 1.15 }, 160, Phaser.Easing.Quartic.InOut, true, 0, 0, true));
                // If it has reached the max score, animate the overflow and snatch
                if (this.currentScore >= this.maxScore) {
                    this.sign.frame = 1;
                    if (scoreToAdd > 0) {
                        this.game.sound.play('quota_complete');
                        // Animates the snatching and the overflow of the marker
                        yield Promise.all([
                            this.snatch.show(this.sign.centerX, this.sign.bottom, 1),
                            PromiseUtil_46.PromiseUtil.tween(this.game, this.game.add.tween(this.markerGroup)
                                .to({ centerY: this.markerGroup.centerY - this.game.height * 0.05 }, 250, Phaser.Easing.Linear.None, true, 0, 0, true)),
                        ]);
                    }
                }
            });
        }
    }
    exports.GoalBarBig = GoalBarBig;
});
define("component/GameTitle", ["require", "exports", "switchback-common/localization/LocalizationManager", "Game", "switchback-common/util/ScaleUtil"], function (require, exports, LocalizationManager_26, Game_30, ScaleUtil_25) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class GameTitle extends Phaser.Group {
        constructor(game, characterKey, titleKey) {
            super(game);
            const titleBackplate = new Phaser.Image(this.game, 0, 0, 'title-backplate');
            titleBackplate.anchor.set(0.5);
            titleBackplate.width = this.game.width * 0.55;
            titleBackplate.height = this.game.height * 0.2;
            const characterText = new Phaser.Text(this.game, 0, titleBackplate.top + titleBackplate.height * 0.175, LocalizationManager_26.LocalizationManager.Instance.text(characterKey), {
                font: Game_30.Game.LOBSTER_FONT,
                fill: "#FDF0BA",
                stroke: "#AF1A3C",
                strokeThickness: Game_30.Game.TEXT_STROKE_THICKNESS_LARGE,
                fontStyle: 'italic',
                fontSize: Game_30.Game.FONT_SIZE_MEDIUM
            });
            characterText.anchor.set(0.5);
            const titleText = new Phaser.Text(this.game, 0, titleBackplate.top + titleBackplate.height * 0.575, LocalizationManager_26.LocalizationManager.Instance.text(titleKey), {
                font: Game_30.Game.CARNIVAL_FONT,
                fill: "#FDF0BA",
                stroke: "#AF1A3C",
                strokeThickness: Game_30.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                fontSize: Game_30.Game.FONT_SIZE_LARGE,
                wordWrap: true,
                wordWrapWidth: titleBackplate.width * 0.8,
                align: 'center'
            });
            titleText.lineSpacing = titleText.fontSize * -0.3;
            titleText.anchor.set(0.5);
            titleText.setShadow(Game_30.Game.TEXT_SHADOW_OFFSET_MED_X, Game_30.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_30.Game.TEXT_SHADOW_SOFT, 0, true, true);
            ScaleUtil_25.scaleToBounds(titleText, titleBackplate.width * 0.75, titleBackplate.height * 0.75);
            this.add(titleBackplate);
            this.add(characterText);
            this.add(titleText);
        }
    }
    exports.GameTitle = GameTitle;
});
define("state/MiniGameInfoState", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/localization/LocalizationManager", "state/ToyStoryBaseState", "StateNames", "logic/GameController", "state/CutSceneState", "model/CharacterInfo", "component/button/BigButton", "component/GoalBarBig", "component/GameTitle"], function (require, exports, ScaleUtil_26, LocalizationManager_27, ToyStoryBaseState_5, StateNames_29, GameController_27, CutSceneState_6, CharacterInfo_3, BigButton_5, GoalBarBig_1, GameTitle_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MiniGameInfoState extends ToyStoryBaseState_5.ToyStoryBaseState {
        preload() {
            super.preload();
            // Screen Buttons
            this.load.image('button-back', 'assets/title/button-back.png');
            this.load.image('button-big-backplate', 'assets/title/button-big-backplate.png');
            this.load.image('button-big-frame', 'assets/title/button-big-frame.png');
            this.load.image('button-green-next', 'assets/title/button-green-next.png');
            // Screen Assets
            this.load.image('goal-bar-frame', 'assets/title/goal-bar-frame.png');
            this.load.image('goal-bar-fill', 'assets/title/goal-bar-fill.png');
            this.load.image('goal-bar-overlay', 'assets/title/goal-bar-overlay.png');
            this.game.load.spritesheet('goal-bar-sign', 'assets/title/goal-sign.png', 296, 230);
            // Snatch FX
            this.game.load.image('snatch-radialfx', 'assets/minigame/woody/snatch_radialfx.png');
            this.game.load.image('snatch-star-green', 'assets/minigame/woody/star_green.png');
            this.game.load.image('snatch-star-pink', 'assets/minigame/woody/star_pink.png');
            this.game.load.image('snatch-star-blue', 'assets/minigame/woody/star_blue.png');
            this.load.image('score-marker', 'assets/title/score-marker.png');
            this.load.image('title-backdrop', 'assets/title/title-backdrop.jpg');
            this.load.image('title-backplate', 'assets/title/title-backplate.png');
            this.load.image('ui-bottom-panel', 'assets/common/ui-bottom-panel.jpg');
            this.load.image('generic-header', 'assets/common/generic-header.png');
            // Game Score Indicators
            this.load.image('score-icon-woody', 'assets/title/score_icon_woody.png');
            this.load.image('score-icon-trixie', 'assets/title/score_icon_trixie.png');
            this.load.image('score-icon-sarge', 'assets/title/score_icon_sarge.png');
            this.load.image('score-icon-hamm', 'assets/title/score_icon_hamm.png');
            this.load.image('score-icon-buzz', 'assets/title/score_icon_buzz.png');
            this.load.image('score-icon-jessie', 'assets/title/score_icon_jessie.png');
            this.load.image('score-icon-rex', 'assets/title/score_icon_rex.png');
            // Character Portraits
            this.game.load.image('portrait-woody-idle', 'assets/character-portraits/woody/idle.png');
            this.game.load.image('portrait-rex-idle', 'assets/character-portraits/rex/idle.png');
            this.game.load.image('portrait-trixie-idle', 'assets/character-portraits/trixie/idle.png');
            this.game.load.image('portrait-buzz-idle', 'assets/character-portraits/buzz/idle.png');
            this.game.load.image('portrait-hamm-idle', 'assets/character-portraits/hamm/idle.png');
            this.game.load.image('portrait-jessie-idle', 'assets/character-portraits/jessie/idle.png');
            this.game.load.image('portrait-sarge-idle', 'assets/character-portraits/sarge/idle.png');
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.currentGameState = GameController_27.GameController.Instance.currentGameState;
                this.backdrop = new Phaser.Image(this.game, 0, 0, 'title-backdrop');
                this.backdrop.width = this.game.width;
                this.backdrop.height = this.game.height * 0.75;
                this.header = new Phaser.Image(this.game, 0, 0, 'generic-header');
                this.header.width = this.game.width;
                this.header.height = this.game.height * 0.2;
                this.footer = new Phaser.Image(this.game, 0, this.backdrop.bottom, 'ui-bottom-panel');
                this.footer.width = this.game.width;
                this.footer.height = this.game.height * 0.25;
                this.characterPortrait = new Phaser.Image(this.game, this.game.width * 0.75, this.footer.top, CharacterInfo_3.CharacterInfo.getInfo(GameController_27.GameController.Instance.gameInfo.get(GameController_27.GameController.Instance.currentGameState).titleCharacter).portraitKey);
                this.characterPortrait.anchor.set(0.5, 0.8);
                this.characterPortrait.height = this.game.height * 0.68;
                this.characterPortrait.scale.x = this.characterPortrait.scale.y;
                this.title = new GameTitle_1.GameTitle(this.game, GameController_27.GameController.Instance.gameInfo.get(GameController_27.GameController.Instance.currentGameState).titleCharacterKey, GameController_27.GameController.Instance.gameInfo.get(GameController_27.GameController.Instance.currentGameState).titleTextKey);
                this.title.centerX = this.game.width * 0.5;
                this.title.top = this.game.height * 0.015;
                this.goalBar = new GoalBarBig_1.GoalBarBig(this.game, GameController_27.GameController.Instance.getQuotaTargetForGame(GameController_27.GameController.Instance.currentGameState), GameController_27.GameController.Instance.getQuotaProgressForGame(GameController_27.GameController.Instance.currentGameState), GameController_27.GameController.Instance.gameInfo.get(GameController_27.GameController.Instance.currentGameState).scoreIconTag);
                ScaleUtil_26.scaleToBounds(this.goalBar, this.game.width * 0.5, this.game.height * 0.65);
                this.goalBar.left = 0;
                this.goalBar.y = this.footer.top;
                this.buttonGroup = new Phaser.Group(this.game);
                this.scoreGroup = new Phaser.Group(this.game);
                this.buttonNext = new BigButton_5.BigButton(this.game, LocalizationManager_27.LocalizationManager.Instance.text("common.play"), 'button-green-next');
                this.buttonNext.width = this.game.width * 0.45;
                this.buttonNext.scale.y = this.buttonNext.scale.x;
                this.buttonNext.centerX = this.game.width * 0.5;
                this.buttonNext.centerY = this.game.height * 0.875;
                this.buttonGroup.add(this.buttonNext);
                this.add.existing(this.backdrop);
                this.add.existing(this.header);
                this.add.existing(this.title);
                this.add.existing(this.characterPortrait);
                this.add.existing(this.scoreGroup);
                this.add.existing(this.footer);
                this.add.existing(this.goalBar);
                this.add.existing(this.buttonGroup);
            });
        }
        transitionToGame() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.showOverlay();
                const gameToPlay = GameController_27.GameController.Instance.currentGameState;
                GameController_27.GameController.Instance.state.trackMinigamePlay(gameToPlay);
                const cutsceneName = GameController_27.GameController.Instance.gameInfo.get(gameToPlay).introCutsceneName;
                if (cutsceneName !== "") {
                    CutSceneState_6.CutSceneState.showCutScene(this.game, cutsceneName, gameToPlay);
                }
                else {
                    this.game.state.start(gameToPlay);
                }
            });
        }
        returnToHub() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.showOverlay();
                this.game.state.start(StateNames_29.StateNames.HUB);
            });
        }
        shutdown() {
            this.buttonNext = null;
            this.backdrop = null;
            this.header = null;
            this.footer = null;
            this.title = null;
            this.characterPortrait = null;
            this.goalBar = null;
            this.buttonGroup = null;
            this.scoreGroup = null;
            super.shutdown();
        }
    }
    exports.MiniGameInfoState = MiniGameInfoState;
});
define("state/MiniGameTitleState", ["require", "exports", "StateNames", "state/MiniGameInfoState", "logic/TSMAnalyticsManager"], function (require, exports, StateNames_30, MiniGameInfoState_1, TSMAnalyticsManager_12) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MiniGameTitleState extends MiniGameInfoState_1.MiniGameInfoState {
        init() {
            this.stateName = StateNames_30.StateNames.MINIGAME_TITLE;
            super.init();
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                TSMAnalyticsManager_12.TSMAnalyticsManager.Instance.onGameTitle(this.currentGameState);
                // The big button plays the game
                this.buttonNext.centerX = this.game.width * 0.5;
                this.buttonNext.onChildInputDown.addOnce(this.transitionToGame, this);
                this.buttonNext.visible = true;
                yield this.dismissOverlay();
                this.buttonNext.startPulse();
            });
        }
        returnToHub() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                TSMAnalyticsManager_12.TSMAnalyticsManager.Instance.onGameBack(this.currentGameState);
                return _super("returnToHub").call(this);
            });
        }
        transitionToGame() {
            this.game.sound.play('snd_ui_click');
            TSMAnalyticsManager_12.TSMAnalyticsManager.Instance.onGameTitleAction(this.currentGameState);
            return super.transitionToGame();
        }
    }
    exports.MiniGameTitleState = MiniGameTitleState;
});
define("component/ScorePanel", ["require", "exports", "switchback-common/component/ThreePieceImage", "Game", "switchback-common/util/TextUtil"], function (require, exports, ThreePieceImage_2, Game_31, TextUtil_20) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var ScoreFormat;
    (function (ScoreFormat) {
        ScoreFormat[ScoreFormat["NUMBER"] = 0] = "NUMBER";
        ScoreFormat[ScoreFormat["TIME"] = 1] = "TIME";
    })(ScoreFormat = exports.ScoreFormat || (exports.ScoreFormat = {}));
    class ScorePanel extends Phaser.Group {
        constructor(game, score, icon, labelString, format) {
            super(game);
            this.scoreBackplate = new ThreePieceImage_2.ThreePieceImage(this.game, [
                'topscore-leftplate', 'topscore-midplate', 'topscore-rightplate'
            ], ThreePieceImage_2.JoinMethod.TILED_HORIZONTALLY);
            this.scoreBackplate.height = this.game.height * 0.2;
            this.scoreBackplate.scale.x = this.scoreBackplate.scale.y;
            this.scoreboardLabel = new Phaser.Text(this.game, 0, 0, '', {
                font: Game_31.Game.CARNIVAL_FONT,
                fontSize: Game_31.Game.FONT_SIZE_LARGE,
                fill: 'white',
                stroke: 'black',
                strokeThickness: Game_31.Game.TEXT_STROKE_THICKNESS_HUMONGOUS
            });
            this.scoreIcon = new Phaser.Image(this.game, 0, 0, icon);
            this.scoreIcon.height = this.game.height * 0.08;
            this.scoreIcon.scale.x = this.scoreIcon.scale.y;
            this.topScore = new Phaser.Group(this.game); // creates an empty group for the score numbers
            this.add(this.scoreBackplate);
            this.add(this.scoreboardLabel);
            this.add(this.scoreIcon);
            this.add(this.topScore);
            // formatting of top score
            let scoreString = Math.min(99999, score).toString();
            if (format === ScoreFormat.TIME) {
                const minutes = Math.min(59, Math.floor(score / 60)); // max time that can be shown (59:59)
                // is the topScore in minutes more than 60? if it is, set the seconds to be always 59
                const isLessThan60Minutes = ((score / 60) < 60);
                const seconds = isLessThan60Minutes ? score % 60 : 59;
                // should look something like this now "04:53"
                scoreString = TextUtil_20.TextUtil.leftPad(minutes.toString(), "0", 2) + ":" + TextUtil_20.TextUtil.leftPad(seconds.toString(), "0", 2);
            }
            /**
             * Adds score tiles from left to right starting from the tenThousandth place
             */
            for (let i = 0; i < scoreString.length; i++) {
                const tileHeight = this.game.height * 0.1;
                let scoreDigit;
                if (scoreString[i] === ":") {
                    scoreDigit = new Phaser.Image(this.game, 0, 0, 'sancreek-colon');
                }
                else {
                    scoreDigit = new Phaser.Image(this.game, 0, 0, 'sancreek-digits', +scoreString[i]);
                }
                scoreDigit.height = tileHeight;
                scoreDigit.scale.x = scoreDigit.scale.y;
                const scoreDigitBackplate = new Phaser.Image(this.game, 0, 0, 'score-card-white');
                scoreDigitBackplate.height = scoreDigit.height;
                scoreDigitBackplate.scale.x = scoreDigitBackplate.scale.y;
                scoreDigit.centerX = (scoreDigitBackplate.width * 0.5) + (i * scoreDigitBackplate.width);
                scoreDigitBackplate.centerX = scoreDigit.centerX;
                scoreDigitBackplate.centerY = scoreDigit.centerY + scoreDigit.height * 0.05;
                this.topScore.add(scoreDigitBackplate);
                this.topScore.add(scoreDigit);
            }
            // resize the score backplate, and
            // positions the top score text, score numbers and score icon relative to the backplate
            let minPaddingOnEitherSide = this.game.width * 0.015;
            let paddingBetweenIconAndScoreNumbers = this.game.width * 0.01;
            this.scoreBackplate.resize(this.topScore.width + this.scoreIcon.width + paddingBetweenIconAndScoreNumbers, minPaddingOnEitherSide);
            this.scoreboardLabel.text = labelString;
            this.scoreboardLabel.centerX = this.scoreBackplate.centerX;
            this.scoreboardLabel.top = this.scoreBackplate.height * 0.008;
            // this ensures the score numbers and score icon will always be positioned perfectly in the center of the backplate
            // this is required to compensate for the way the images are tiled that may cause the backplate to be bigger than the width specified
            const totalWidthOfScoreAndScoreIcon = this.topScore.width + paddingBetweenIconAndScoreNumbers + this.scoreIcon.width;
            const amountOfPaddingPerSide = (this.scoreBackplate.width - totalWidthOfScoreAndScoreIcon) * 0.5;
            this.topScore.right = this.scoreBackplate.right - amountOfPaddingPerSide;
            this.topScore.y = this.scoreBackplate.height * 0.4;
            this.scoreIcon.left = amountOfPaddingPerSide;
            this.scoreIcon.centerY = this.topScore.centerY;
        }
    }
    exports.ScorePanel = ScorePanel;
});
define("state/MiniGameResultState", ["require", "exports", "state/MiniGameInfoState", "component/ScorePanel", "logic/GameController", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "StateNames", "component/button/BigButton", "state/CutSceneState", "component/hub/HubNode", "logic/TSMAnalyticsManager"], function (require, exports, MiniGameInfoState_2, ScorePanel_1, GameController_28, LocalizationManager_28, PromiseUtil_47, StateNames_31, BigButton_6, CutSceneState_7, HubNode_4, TSMAnalyticsManager_13) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MiniGameResultState extends MiniGameInfoState_2.MiniGameInfoState {
        constructor() {
            super(...arguments);
            this.isFinalScreen = false; // Means this this is the final screen and score, otherwise it will transition to a new result where Pete steals some score
            this.isStealing = false;
        }
        init() {
            this.stateName = StateNames_31.StateNames.RESULTS;
            super.init();
        }
        preload() {
            super.preload();
            // Score Plates
            this.load.image('topscore-leftplate', 'assets/title/score_leftplate.png');
            this.load.image('topscore-midplate', 'assets/title/score_midplate.png');
            this.load.image('topscore-rightplate', 'assets/title/score_rightplate.png');
            this.load.image('button-green-next', 'assets/title/button-green-next.png');
            this.load.image('score-card-white', 'assets/title/score_card_white.png');
            // Buttons
            this.load.image('button-back', 'assets/title/button-back.png');
            this.load.image('button-big-backplate', 'assets/title/button-big-backplate.png');
            this.load.image('button-big-frame', 'assets/title/button-big-frame.png');
            this.load.image('button-orange-replay', 'assets/title/button-orange-replay.png');
            this.load.audio('quota_filling', ['assets/sound/ui/quota_filling.mp4', 'assets/sound/ui/quota_filling.ogg']);
            this.load.audio('quota_complete', ['assets/sound/sarge/tick_correct.mp4', 'assets/sound/sarge/tick_correct.ogg']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.gameName = GameController_28.GameController.Instance.currentGameState;
                const gameInfo = GameController_28.GameController.Instance.gameInfo.get(this.gameName);
                this.scorePanel = new ScorePanel_1.ScorePanel(this.game, GameController_28.GameController.Instance.currentGameScore, gameInfo.scoreIconTag, LocalizationManager_28.LocalizationManager.Instance.text(gameInfo.scoreboardLabelKey), ScorePanel_1.ScoreFormat.NUMBER);
                const scorePanelPadding = this.game.width * 0.025;
                this.scorePanel.right = this.game.width - scorePanelPadding;
                this.scorePanel.top = this.footer.top;
                // The replay button will replay the game
                this.buttonReplay = new BigButton_6.BigButton(this.game, LocalizationManager_28.LocalizationManager.Instance.text("common.replay"), 'button-orange-replay');
                this.buttonReplay.scale.set(this.buttonNext.scale.x * 0.9);
                this.buttonReplay.centerX = this.game.width * 0.25;
                this.buttonReplay.centerY = this.game.height * 0.875;
                this.buttonGroup.add(this.buttonReplay);
                // The big button returns players to the hub
                this.buttonNext.centerX = this.game.width * 0.75;
                this.buttonNext.text = LocalizationManager_28.LocalizationManager.Instance.text("common.home");
                this.currentScore = GameController_28.GameController.Instance.getQuotaProgressForGame(this.gameName);
                this.scoreGained = GameController_28.GameController.Instance.getQuotaToRedeemForGameResults(this.gameName);
                const cutsceneName = GameController_28.GameController.Instance.gameInfo.get(this.gameName).resultsCutsceneName;
                this.isStealing = this.scoreGained <= 0 && cutsceneName !== "";
                const targetScore = GameController_28.GameController.Instance.getQuotaTargetForGame(this.gameName);
                const didJustFillQuota = (this.currentScore < targetScore) && ((this.currentScore + this.scoreGained) >= targetScore);
                // Don't show the replay button if they're in forced progression, or if they just filled their quota
                if (didJustFillQuota || GameController_28.GameController.Instance.state.forcedProgressionRound === 0) {
                    this.buttonReplay.visible = false;
                    this.buttonNext.centerX = this.game.width * 0.5;
                }
                if (didJustFillQuota) {
                    GameController_28.GameController.Instance.achievementManager.onQuotaReached(this.gameName);
                }
                if (GameController_28.GameController.Instance.state.forcedProgressionRound === 0 && this.scoreGained >= 0) { // TODO last parameter can be !isStealing (should i change it?)
                    // Upgrade the hub node to midway state
                    GameController_28.GameController.Instance.state.incrementGameTargetLevel(this.gameName);
                }
                else if (didJustFillQuota) {
                    // Upgrade the hub node to final state
                    GameController_28.GameController.Instance.state.setGameTargetLevel(this.gameName, HubNode_4.HubNode.MAX_NODE_LEVEL);
                }
                this.scoreGroup.add(this.scorePanel);
                this.buttonGroup.alpha = 0;
                yield this.dismissOverlay();
                // Pop the score panel up
                if (!this.isStealing) {
                    yield PromiseUtil_47.PromiseUtil.tween(this.game, this.game.add.tween(this.scorePanel).to({ bottom: this.footer.top - scorePanelPadding }, 450, Phaser.Easing.Back.Out, true));
                }
                // Redeem their game score and animate the bar
                GameController_28.GameController.Instance.redeemPendingScoreForGameResults(this.gameName);
                this.trackResult();
                yield this.goalBar.addScore(this.scoreGained);
                // Show the button group and add listeners
                yield PromiseUtil_47.PromiseUtil.tween(this.game, this.game.add.tween(this.buttonGroup).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true));
                this.buttonReplay.onChildInputDown.addOnce(this.onBtnReplay, this);
                // Don't steal anything if we just hit the quota,
                // or have already filled the quota,
                // or just came back from a cutscene,
                // or are on our very first minigame,
                // or have completed the jail installation
                const totalPlays = GameController_28.GameController.Instance.state.totalNumPlays;
                this.isFinalScreen = didJustFillQuota
                    || this.currentScore > targetScore
                    || this.isStealing
                    || totalPlays <= 1
                    || GameController_28.GameController.Instance.state.hasCompletedJailInstallation;
                this.buttonNext.onChildInputDown.addOnce(this.onBtnNext, this);
                this.buttonNext.startPulse();
            });
        }
        trackResult() {
            if (!this.isStealing) {
                TSMAnalyticsManager_13.TSMAnalyticsManager.Instance.onGameTime(this.gameName, this.scoreGained, true);
                TSMAnalyticsManager_13.TSMAnalyticsManager.Instance.onGameResult(this.gameName, this.scoreGained);
            }
        }
        onBtnReplay() {
            this.game.sound.play('snd_ui_click');
            const score = GameController_28.GameController.Instance.currentGameScore;
            const totalScore = this.currentScore + this.scoreGained;
            TSMAnalyticsManager_13.TSMAnalyticsManager.Instance.onGameTitleAction(this.gameName, TSMAnalyticsManager_13.TSMAnalyticsManager.GAMETITLE_REPLAY, score, totalScore);
            this.transitionToGame();
        }
        onBtnNext() {
            this.game.sound.play('snd_ui_click');
            const cutsceneName = GameController_28.GameController.Instance.gameInfo.get(this.gameName).resultsCutsceneName;
            if (this.isFinalScreen || cutsceneName === "") {
                // I rather use minigameinfo, but this works and is better than polluting the codebase
                const score = GameController_28.GameController.Instance.currentGameScore;
                let totalScore = this.currentScore;
                // this would imply that the game has no stealing. On those cases, the score gets reported differently
                if (!this.isStealing) {
                    totalScore += score;
                }
                TSMAnalyticsManager_13.TSMAnalyticsManager.Instance.onGameTitleAction(this.gameName, TSMAnalyticsManager_13.TSMAnalyticsManager.GAMETITLE_CONTINUE, score, totalScore);
                GameController_28.GameController.Instance.currentGameState = null;
                this.returnToHub();
            }
            else {
                this.transitionToCutscene(this.scoreGained);
            }
        }
        transitionToCutscene(score) {
            return __awaiter(this, void 0, void 0, function* () {
                // Steal minigame score
                // Steal 10% of score
                let scoreToSteal = score * 0.1;
                GameController_28.GameController.Instance.state.stealMinigameScore(this.gameName, scoreToSteal);
                // Transition to Stinky Pete cutscene
                // Set ResultState as post-cutscene state
                const cutsceneName = GameController_28.GameController.Instance.gameInfo.get(this.gameName).resultsCutsceneName;
                yield this.showOverlay();
                CutSceneState_7.CutSceneState.showCutScene(this.game, cutsceneName, StateNames_31.StateNames.RESULTS, score);
            });
        }
        shutdown() {
            this.scorePanel = null;
            this.buttonReplay = null;
            this.cache.removeSound('quota_filling');
            this.cache.removeSound('quota_complete');
            super.shutdown();
        }
    }
    exports.MiniGameResultState = MiniGameResultState;
});
define("state/IntroState", ["require", "exports", "switchback-common/localization/LocalizationManager", "state/ToyStoryBaseState", "Game", "component/ArrowPrompt", "switchback-common/util/PromiseUtil", "StateNames", "switchback-common/util/TextUtil", "switchback-common/util/ScaleUtil", "switchback-common/component/text/TypewriterText"], function (require, exports, LocalizationManager_29, ToyStoryBaseState_6, Game_32, ArrowPrompt_5, PromiseUtil_48, StateNames_32, TextUtil_21, ScaleUtil_27, TypewriterText_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class IntroState extends ToyStoryBaseState_6.ToyStoryBaseState {
        preload() {
            super.preload();
            this.game.load.image('cut-scene-lights-back', 'assets/cutscene/lights_back.png');
            this.game.load.image('cut-scene-lights-front', 'assets/cutscene/lights_front.png');
            this.game.load.image('cut-scene-lights-strip', 'assets/cutscene/lights_strip.png');
            this.game.load.image('generic-header', 'assets/common/generic-header.png');
            this.game.load.image('small-box', 'assets/common/small-box.png');
            this.game.load.image('logo', 'assets/common/logo.png');
            this.game.load.image('set-complete', 'assets/intro/set_complete.png');
            this.game.load.image('group-play', 'assets/dialog/instruction_graphic_rex.png');
            this.game.load.image('split-dialog-short', 'assets/dialog/split-dialog-short.png');
            this.game.load.image('map-bg', 'assets/hub/map-cosmetics/floor.jpg');
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                const bg = new Phaser.Image(this.game, 0, 0, 'map-bg');
                bg.width = this.game.width;
                bg.height = this.game.height;
                this.add.existing(bg);
                this.setCompleteImg = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.5, 'set-complete');
                this.setCompleteImg.anchor.set(0.5);
                this.add.existing(this.setCompleteImg);
                this.groupPlayImg = new Phaser.Image(this.game, this.game.width * 0.5, this.game.height * 0.35, 'group-play');
                this.groupPlayImg.anchor.set(0.5);
                ScaleUtil_27.scaleToBounds(this.groupPlayImg, this.game.width, this.game.height * 0.6);
                this.groupPlayImg.alpha = 0;
                this.add.existing(this.groupPlayImg);
                const headerBg = new Phaser.Image(this.game, 0, 0, 'generic-header');
                headerBg.width = this.game.width;
                headerBg.scale.y = headerBg.scale.x;
                this.add.existing(headerBg);
                const smallBox = new Phaser.Image(this.game, headerBg.centerX, 0, 'small-box');
                smallBox.anchor.set(0.5, 0);
                ScaleUtil_27.scaleToBounds(smallBox, this.game.width * 0.6, headerBg.height * 1.1);
                this.add.existing(smallBox);
                const logo = new Phaser.Image(this.game, smallBox.centerX, smallBox.centerY, 'logo');
                logo.anchor.set(0.5);
                ScaleUtil_27.scaleToBounds(logo, smallBox.width * 0.8, smallBox.height * 0.8);
                this.add.existing(logo);
                this.speechBox = new Phaser.Group(this.game);
                this.add.existing(this.speechBox);
                const textBoxMiddle = new Phaser.Image(this.game, 0, 0, 'split-dialog-short');
                textBoxMiddle.width = this.game.width;
                textBoxMiddle.scale.y = textBoxMiddle.scale.x;
                this.speechBox.add(textBoxMiddle);
                this.text = new TypewriterText_5.TypewriterText(this.game, this.game.width * 0.5, textBoxMiddle.height * 0.49, '', {
                    font: Game_32.Game.DEFAULT_FONT,
                    fontSize: 80,
                    fill: 'white',
                    wordWrap: true,
                    wordWrapWidth: this.game.width * 0.94,
                    align: 'center'
                });
                this.text.anchor.set(0.5);
                TextUtil_21.TextUtil.scaleDownTextToHeight(this.text, textBoxMiddle.height * 0.5);
                this.text.setShadow(Game_32.Game.TEXT_SHADOW_OFFSET_MED_X, Game_32.Game.TEXT_SHADOW_OFFSET_MED_Y, Game_32.Game.TEXT_SHADOW_DIAG, 0, true, true);
                this.speechBox.add(this.text);
                this.characterName = new Phaser.Text(this.game, 0, 0, '', {
                    font: Game_32.Game.CARNIVAL_FONT,
                    fontSize: Game_32.Game.FONT_SIZE_MEDIUM,
                    fill: Game_32.Game.TEXT_COLOR_CHAR_FILL,
                    stroke: Game_32.Game.TEXT_COLOR_CHAR_STROKE,
                    strokeThickness: this.speechBox.width * 0.01
                });
                this.speechBox.add(this.characterName);
                this.speechBox.bottom = this.game.height;
                const lightsBack = new Phaser.Image(this.game, 0, 0, 'cut-scene-lights-back');
                lightsBack.width = this.game.width;
                lightsBack.scale.y = lightsBack.scale.x;
                lightsBack.bottom = this.game.height;
                this.add.existing(lightsBack);
                this.lights = new Phaser.TileSprite(this.game, 0, 0, this.game.width, this.game.height * 0.04, 'cut-scene-lights-strip');
                this.lights.tileScale.x = this.lights.width / this.lights.texture.width;
                this.lights.bottom = lightsBack.bottom;
                this.lights.autoScroll(this.game.width * 0.5, 0);
                this.add.existing(this.lights);
                const lightsFront = new Phaser.Image(this.game, 0, 0, 'cut-scene-lights-front');
                lightsFront.width = lightsBack.width;
                lightsFront.scale.y = lightsBack.scale.y;
                lightsFront.bottom = lightsBack.bottom;
                this.add.existing(lightsFront);
                this.prompt = new ArrowPrompt_5.ArrowPrompt(this.game, 0, 0);
                this.prompt.height = lightsBack.height * 0.7;
                this.prompt.scale.x = this.prompt.scale.y;
                this.prompt.right = this.game.width - (this.game.width * 0.005);
                this.prompt.bottom = this.game.height - (this.game.width * 0.01);
                this.add.existing(this.prompt);
                ScaleUtil_27.scaleToBounds(this.setCompleteImg, this.game.width, this.game.height - headerBg.height - textBoxMiddle.height);
                ScaleUtil_27.scaleToBounds(this.groupPlayImg, this.game.width, this.game.height - headerBg.height - textBoxMiddle.height);
                this.setCompleteImg.centerY = headerBg.height + (this.speechBox.top - headerBg.bottom) * 0.5;
                this.groupPlayImg.centerY = headerBg.height + (this.speechBox.top - headerBg.bottom) * 0.5;
                yield this.dismissOverlay();
                yield this.writeText("intro.description_1");
                yield PromiseUtil_48.PromiseUtil.signal(this.game.input.onTap);
                yield this.writeText("intro.description_2");
                yield PromiseUtil_48.PromiseUtil.signal(this.game.input.onTap);
                yield PromiseUtil_48.PromiseUtil.tween(this.game, this.game.add.tween(this.setCompleteImg).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true));
                yield Promise.all([
                    this.writeText("intro.description_3"),
                    PromiseUtil_48.PromiseUtil.tween(this.game, this.game.add.tween(this.groupPlayImg).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true)),
                ]);
                yield PromiseUtil_48.PromiseUtil.signal(this.game.input.onTap);
                yield this.writeText("intro.description_4");
                yield PromiseUtil_48.PromiseUtil.signal(this.game.input.onTap);
                yield this.showOverlay();
                this.game.state.start(StateNames_32.StateNames.HUB);
            });
        }
        writeText(textKey) {
            return __awaiter(this, void 0, void 0, function* () {
                this.text.text = LocalizationManager_29.LocalizationManager.Instance.text(textKey);
                yield this.text.typewriter(15, 'typewriter_loop');
            });
        }
        shutdown() {
            this.speechBox = null;
            this.text = null;
            this.characterName = null;
            this.prompt = null;
            this.lights = null;
            this.setCompleteImg = null;
            this.groupPlayImg = null;
            this.game.cache.removeImage('cut-scene-lights-back');
            this.game.cache.removeImage('cut-scene-lights-front');
            this.game.cache.removeImage('cut-scene-lights-strip');
            this.game.cache.removeImage('generic-header');
            this.game.cache.removeImage('small-box');
            this.game.cache.removeImage('logo');
            this.game.cache.removeImage('set-complete');
            this.game.cache.removeImage('group-play');
            this.game.cache.removeImage('split-dialog-short');
            this.game.cache.removeImage('map-bg');
            super.shutdown();
        }
    }
    exports.IntroState = IntroState;
});
define("switchback-common/play/PlayCameraUtil", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_49) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlayCameraUtil {
        /**
         * The transparent class references the project style.css file, it MUST be set in there
         * for the default value to work. If not, the camera stream can be blacked out.
         * .transparent {
         *     background: rgba(0, 0, 0, 0);
         * }
         * This allows the bg and transparency to be specified in css and not in the code base.
         * The bg color can also be changed throughout the play session and closing of the camera
         * doesn't need to care about that information.
         */
        static startCamera(cameraType, transparentClass = "transparent") {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    yield PlayAPI.startCamera(cameraType).then((apiResult) => {
                        document.body.classList.add(transparentClass);
                        resolve(true);
                        console.log("[PlayCameraUtil] StartCamera accepted: " + JSON.stringify(apiResult));
                    }, (apiResult) => {
                        resolve(false);
                        console.log("[PlayCameraUtil] StartCamera rejected: " + JSON.stringify(apiResult));
                    }).catch((apiResult) => {
                        resolve(false);
                        console.log("[PlayCameraUtil] StartCamera exception: " + JSON.stringify(apiResult));
                    });
                }));
            });
        }
        static captureImage(game, timeout = 5000, imageName) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[PlayCameraUtil] captureImage()");
                yield Promise.race([
                    PlayAPI.captureImage(),
                    PromiseUtil_49.PromiseUtil.time(game, timeout),
                ]).then((result) => {
                    if (!result) {
                        return;
                    }
                    if (result.imageData) {
                        return PlayCameraUtil.loadCaptureImage(game, result.imageData, imageName);
                    }
                    else {
                        console.log("[PlayCameraUtil.captureCamera] Unable to get imageData from result");
                    }
                });
            });
        }
        static loadCaptureImage(game, imageData, imageName) {
            return __awaiter(this, void 0, void 0, function* () {
                return new Promise((resolve, reject) => {
                    const loader = new Phaser.Loader(game);
                    loader.image(imageName, imageData);
                    loader.onLoadComplete.addOnce(resolve, this);
                    loader.start();
                });
            });
        }
        static closeCamera(transparentClass = "transparent") {
            return __awaiter(this, void 0, void 0, function* () {
                document.body.classList.remove(transparentClass);
                return PlayAPI.endCamera();
            });
        }
        static checkCameraAccess(promptForCameraPermissionCallback) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[PlayCameraUtil] cameraAccessStep()`);
                let cameraStatus = CameraAccessStatus.NOT_DETERMINED;
                yield PlayAPI.cameraAccessStatus().then((result) => {
                    console.log(`[PlayCameraUtil] cameraAccessStep() result 1: ${JSON.stringify(result)}`);
                    cameraStatus = result.status;
                }).catch((error) => {
                    // We should assume there's an error with the camera and skip anything camera related.
                    return CameraAccessStatus.DENIED;
                });
                if (promptForCameraPermissionCallback) {
                    yield promptForCameraPermissionCallback(cameraStatus);
                }
                // Recheck camera permissions to see if they've accepted them now
                yield PlayAPI.cameraAccessStatus().then((result) => {
                    console.log(`[PlayCameraUtil] cameraAccessStep() result 2: ${JSON.stringify(result)}`);
                    cameraStatus = result.status;
                }).catch((error) => {
                    // We should assume there's an error with the camera and skip anything camera related.
                    cameraStatus = CameraAccessStatus.DENIED;
                });
                return cameraStatus;
            });
        }
    }
    exports.PlayCameraUtil = PlayCameraUtil;
});
define("switchback-common/util/DeviceUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Class to support device-specific functions.
     * @author nick@cerebralfix.com
     */
    class DeviceUtil {
        /*
        * This function applies the MaliGPU fix if this device is running a Mali-400 GPU.
        * This is required to apply a phaser render setting that fixes a
        * canvas strobing issue present in these GPUs.
        */
        static applyMaliGPUFix(game) {
            if (DeviceUtil.isMaliGPU()) {
                // Fix for strobing canvas when running Phaser on Mali-400 GPUs.
                game.preserveDrawingBuffer = true;
            }
        }
        /*
        * This function detects whether the device is running a Mali-400 GPU.
        * This is required to apply a phaser render setting that fixes a
        * canvas strobing issue present in these GPUs.
        */
        static isMaliGPU() {
            const canv = document.createElement("canvas");
            canv.setAttribute("id", "MaliTestCanvas");
            canv.setAttribute("width", "1");
            canv.setAttribute("height", "1");
            document.body.appendChild(canv);
            const canvas = document.getElementById("MaliTestCanvas");
            const gl = canvas.getContext("webgl", { stencil: true });
            canvas.parentNode.removeChild(canvas);
            if (!gl) {
                return false;
            }
            const dbgRenderInfo = gl.getExtension("WEBGL_debug_renderer_info");
            let renderer;
            if (dbgRenderInfo !== null) {
                renderer = gl.getParameter(dbgRenderInfo.UNMASKED_RENDERER_WEBGL);
            }
            else {
                return false;
            }
            return renderer.search("Mali-400") !== -1;
        }
        /**
         * Checks the userAgent to try to figure out
         * which platform we're running on.
         */
        static isIos() {
            return /iPad|iPhone|iPod/.test(navigator.userAgent);
        }
    }
    exports.DeviceUtil = DeviceUtil;
});
define("switchback-common/component/camera/PlayCamera", ["require", "exports", "switchback-common/play/PlayCameraUtil", "switchback-common/util/DeviceUtil", "switchback-common/util/PromiseUtil"], function (require, exports, PlayCameraUtil_1, DeviceUtil_1, PromiseUtil_50) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlayCamera extends Phaser.Group {
        constructor(game, overlayAsset, parent = null) {
            super(game, parent, null, false);
            this._capturedImage = new Phaser.Image(this.game, 0, 0, "");
            this._capturedImage.visible = false;
            this.add(this._capturedImage);
            if (overlayAsset) {
                this._overlay = new Phaser.Image(this.game, 0, 0, overlayAsset);
                this._overlay.width = this.game.width;
                this._overlay.height = this.game.height;
                this.add(this._overlay);
            }
        }
        openCamera() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!PlayCamera.cameraPermissionsChecked) {
                    yield PlayCameraUtil_1.PlayCameraUtil.checkCameraAccess(this.checkCameraCallBack.bind(this));
                    PlayCamera.cameraPermissionsChecked = true;
                }
                // We can't trust the CameraAccessStatus, so we'll attempt to start the camera regardless
                const isSuccess = PlayCameraUtil_1.PlayCameraUtil.startCamera(CameraType.BACK_CAMERA);
                if (isSuccess) {
                    // This is necessary so the ui actually process the changes
                    // otherwise the player can tap to take the image too quickly causing a black photo
                    yield PromiseUtil_50.PromiseUtil.time(this.game, 100);
                }
                return isSuccess;
            });
        }
        /**
         * PlayAPI.endCamera() was not returning on Android devices as at 03/09/2018
         * so could not be awaited on as was the original scanner flow. Safe bet is to
         * ignore the result and handle any situation where the camera may still be open.
         */
        closeCamera(hideCapturedImage = true) {
            this._capturedImage.visible = !hideCapturedImage;
            PlayCameraUtil_1.PlayCameraUtil.closeCamera();
            PlayAPI.endCamera();
        }
        capture(timeout = 5000) {
            return __awaiter(this, void 0, void 0, function* () {
                yield PlayCameraUtil_1.PlayCameraUtil.captureImage(this.game, timeout, PlayCamera.CAPTURE_TEXTURE);
                this.loadTexture();
                // Give some time for the frame to render before performing any other actions.
                // Calling endCamera too soon for example can cause a black flash between camera feed and photo
                yield PromiseUtil_50.PromiseUtil.time(this.game, 100);
            });
        }
        hideCaptureImage() {
            this._capturedImage.visible = false;
        }
        destroy() {
            this.game.cache.removeImage(PlayCamera.CAPTURE_TEXTURE);
            super.destroy();
        }
        onShowCameraPermissionsWarning() {
            return __awaiter(this, void 0, void 0, function* () {
            });
        }
        onShowCameraEnablePermissionsDialog(isIos) {
            return __awaiter(this, void 0, void 0, function* () {
            });
        }
        loadTexture() {
            if (this.game.cache.checkImageKey(PlayCamera.CAPTURE_TEXTURE)) {
                this._capturedImage.loadTexture(PlayCamera.CAPTURE_TEXTURE);
                // Adjust it because the data comes back rotated...
                this._capturedImage.height = this.game.width;
                this._capturedImage.scale.x = this._capturedImage.scale.y;
                this._capturedImage.angle = 90;
                this._capturedImage.anchor.set(0.5);
                this._capturedImage.position.set(0, 0);
                this._capturedImage.centerX = this.game.width * 0.5;
                this._capturedImage.centerY = this.game.height * 0.5;
                this._capturedImage.visible = true;
            }
        }
        checkCameraCallBack(status) {
            return __awaiter(this, void 0, void 0, function* () {
                // Show the appropriate dialog
                if (status === CameraAccessStatus.AUTHORIZED) {
                    // We don't need to do anything. They've accepted the permissions already.
                    return;
                }
                else if (status === CameraAccessStatus.NOT_DETERMINED) {
                    // This device hasn't been asked for camera perms before.
                    // We should show a dialog to let them know they should accept them.
                    yield this.onShowCameraPermissionsWarning();
                }
                else if (status === CameraAccessStatus.DENIED) {
                    // Fall back to assuming they've declined.
                    // This dialog has the most info about enabling permissions.
                    yield this.onShowCameraEnablePermissionsDialog(DeviceUtil_1.DeviceUtil.isIos());
                }
                else if (status === CameraAccessStatus.RESTRICTED) {
                    // If the camera is Restricted, the user can't change it. We must handle it as if there were no camera.
                    // We won't show anything, and B-Show needs to be shown
                }
            });
        }
    }
    PlayCamera.CAPTURE_TEXTURE = "camera-image-capture";
    PlayCamera.cameraPermissionsChecked = false;
    exports.PlayCamera = PlayCamera;
});
define("component/camera/ToyStoryCamera", ["require", "exports", "switchback-common/component/camera/PlayCamera", "switchback-common/localization/LocalizationManager", "switchback-common/util/PromiseUtil", "component/dialog/BasicCTADialog"], function (require, exports, PlayCamera_1, LocalizationManager_30, PromiseUtil_51, BasicCTADialog_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ToyStoryCamera extends PlayCamera_1.PlayCamera {
        onShowCameraPermissionsWarning() {
            return __awaiter(this, void 0, void 0, function* () {
                const cameraPermissionDialog = new BasicCTADialog_4.BasicCTADialog(this.game);
                cameraPermissionDialog.setTextContent(LocalizationManager_30.LocalizationManager.Instance.text("camera_permission_dialog.title").toUpperCase(), LocalizationManager_30.LocalizationManager.Instance.text("camera_permission_dialog.text"));
                yield this.showDialog(cameraPermissionDialog);
            });
        }
        onShowCameraEnablePermissionsDialog(isIos) {
            return __awaiter(this, void 0, void 0, function* () {
                const text = isIos ? "camera_permission_dialog.denied_ios" : "camera.denied_android";
                const cameraPermissionDialog = new BasicCTADialog_4.BasicCTADialog(this.game);
                cameraPermissionDialog.setTextContent(LocalizationManager_30.LocalizationManager.Instance.text("camera_permission_dialog.title").toUpperCase(), LocalizationManager_30.LocalizationManager.Instance.text(text));
                yield this.showDialog(cameraPermissionDialog);
            });
        }
        showDialog(cameraAccessDialog) {
            return __awaiter(this, void 0, void 0, function* () {
                // Wait for them to acknowledge the dialog
                // N.B. This is added to the stage - it has to sit above the overlay. It's manually destroyed later.
                if (cameraAccessDialog) {
                    this.game.stage.addChild(cameraAccessDialog);
                    cameraAccessDialog.centerY = this.game.height * 0.6;
                    yield cameraAccessDialog.show();
                    yield PromiseUtil_51.PromiseUtil.signal(this.game.input.onTap);
                    yield cameraAccessDialog.hide();
                    cameraAccessDialog.destroy();
                }
            });
        }
    }
    exports.ToyStoryCamera = ToyStoryCamera;
});
define("minigame/poster/PosterInstallationState", ["require", "exports", "state/ToyStoryBaseState", "StateNames", "switchback-common/util/PromiseUtil", "switchback-common/util/ScaleUtil", "logic/GameController", "switchback-common/localization/LocalizationManager", "logic/TSMAnalyticsManager", "component/button/BigButton", "component/MessageBar", "component/fx/Snatch", "minigame/poster/PosterGameState", "state/CutSceneState", "switchback-common/util/TextUtil", "switchback-common/play/ShowEffectController", "component/camera/ToyStoryCamera"], function (require, exports, ToyStoryBaseState_7, StateNames_33, PromiseUtil_52, ScaleUtil_28, GameController_29, LocalizationManager_31, TSMAnalyticsManager_14, BigButton_7, MessageBar_15, Snatch_4, PosterGameState_1, CutSceneState_8, TextUtil_22, ShowEffectController_2, ToyStoryCamera_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     * @author jeronimo@cerebralfix.com
     */
    class PosterInstallationState extends ToyStoryBaseState_7.ToyStoryBaseState {
        preload() {
            super.preload();
            this.load.image('image-capture', 'assets/common/image-capture-test.png');
            this.load.image('outline', 'assets/poster/outline.png');
            this.load.image('tiling-bar', 'assets/common/CTAbar_tilingbg.png');
            this.load.image('button-green-fire', 'assets/poster/installation/button-fire.png');
            this.load.image('button-big-backplate', 'assets/title/button-big-backplate.png');
            this.load.image('button-big-frame', 'assets/title/button-big-frame.png');
            this.load.image('ui-bottom-panel', 'assets/common/ui-bottom-panel.jpg');
            this.load.image('ui-top-panel', 'assets/common/ui-top-panel.png');
            this.load.image('map-bg', 'assets/hub/map-cosmetics/floor.jpg');
            this.load.image('poster-bg', 'assets/poster/poster-7.jpg');
            this.load.image('poster-find', 'assets/hub/poster-icon.png');
            this.load.image('inst-cannon', 'assets/poster/installation/cannon.png');
            this.load.image('inst-dart', 'assets/poster/installation/dart.png');
            this.load.image('snatch-radialfx', 'assets/minigame/woody/snatch_radialfx.png');
            this.load.image('snatch-star-green', 'assets/minigame/woody/star_green.png');
            this.load.image('snatch-star-pink', 'assets/minigame/woody/star_pink.png');
            this.load.image('snatch-star-blue', 'assets/minigame/woody/star_blue.png');
            this.load.spritesheet('inst-loot', "assets/poster/installation/loot-sheet.png", 230, 230);
            this.load.spritesheet('balloon-orange-pop', "assets/minigame/trixie/balloons/balloon_orange_pop.png", 495, 539);
            this.load.audio('dart_fire', ['assets/sound/poster/dart_fire.mp4', 'assets/sound/poster/dart_fire.ogg']);
        }
        init() {
            this.stateName = StateNames_33.StateNames.POSTER_INSTALLATION;
            super.init();
            this.targetYPosition = this.game.height * 0.35;
            // From this point on we don't want any new settings saved.
            // The save state will be cleared shortly, but if it doesn't make it that far we'll restore to this point
            GameController_29.GameController.Instance.state.disableSave();
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.bg = new Phaser.Image(this.game, 0, 0, 'map-bg');
                this.bg.width = this.game.width;
                this.bg.height = this.game.height;
                this.cameraSprite = new ToyStoryCamera_1.ToyStoryCamera(this.game);
                ////////////////////////////////////
                // B-SHOW PARTICLE EMITTER
                ////////////////////////////////////
                this.itemsEmitter = this.game.add.emitter(this.game.width * 0.5, this.game.height * 0.02, 50);
                this.itemsEmitter.makeParticles('inst-loot', [0, 1, 2, 3, 4, 5]);
                this.itemsEmitter.maxParticleScale = this.game.width / (2.5 * 230);
                this.itemsEmitter.minParticleScale = this.itemsEmitter.maxParticleScale * 0.5;
                let ySpeed = this.game.height * 0.4;
                this.itemsEmitter.setYSpeed(ySpeed * 0.6, ySpeed);
                this.itemsEmitter.gravity = 0;
                this.itemsEmitter.width = this.game.world.width;
                this.itemsEmitter.minRotation = -20;
                this.itemsEmitter.maxRotation = 20;
                ////////////////////////////////////
                // HEADER
                ////////////////////////////////////
                this.headerInstructions = new MessageBar_15.MessageBar(this.game, LocalizationManager_31.LocalizationManager.Instance.text('poster_inst.header_instructions'), 32);
                this.headerInstructions.bottom = 0;
                ////////////////////////////////////
                // FOOTER
                ////////////////////////////////////
                this.footerGroup = new Phaser.Group(this.game);
                const footerPanel = new Phaser.Image(this.game, 0, 0, 'ui-bottom-panel');
                footerPanel.width = this.game.width;
                footerPanel.height = this.game.height * 0.175;
                footerPanel.anchor.set(0, 1);
                this.footerGroup.add(footerPanel);
                this.fireButton = new BigButton_7.BigButton(this.game, LocalizationManager_31.LocalizationManager.Instance.text("poster_inst.btn_fire").toUpperCase(), 'button-green-fire');
                ScaleUtil_28.scaleToBounds(this.fireButton, footerPanel.width * 0.35, footerPanel.height * 0.85);
                this.fireButton.centerX = footerPanel.width * 0.5;
                this.fireButton.bottom = footerPanel.bottom;
                this.cannon = new Phaser.Image(this.game, 0, 0, "inst-cannon");
                this.cannon.anchor.set(0.5, 1);
                this.cannon.width = this.game.width * 0.5;
                this.cannon.scale.y = this.cannon.scale.x;
                this.cannon.bottom = footerPanel.bottom;
                this.cannon.centerX = this.game.width * 0.5;
                this.footerGroup.add(footerPanel);
                this.footerGroup.add(this.cannon);
                this.footerGroup.add(this.fireButton);
                this.footerGroup.top = this.game.height;
                ////////////////////////////////////
                // POSTER BACKGROUND
                ////////////////////////////////////
                this.posterBg = new Phaser.Image(this.game, 0, 0, 'poster-bg');
                ScaleUtil_28.scaleToBounds(this.posterBg, this.game.width, this.game.height);
                this.posterBg.centerX = this.game.width * 0.5;
                this.posterBg.centerY = this.game.height * 0.5;
                this.posterBg.alpha = 0;
                ////////////////////////////////////
                // CAPTURED IMAGE SNAP
                ////////////////////////////////////
                this.capturedImage = new Phaser.Image(this.game, 0, 0, '');
                this.capturedImage.visible = false;
                ////////////////////////////////////
                // BLACKOUT OVERLAY
                ////////////////////////////////////
                this.overlay = new Phaser.Graphics(this.game);
                this.overlay.beginFill(0x000000, 1);
                this.overlay.drawRect(0, 0, this.game.width, this.game.height);
                this.overlay.endFill();
                this.overlay.alpha = 0;
                this.overlay.inputEnabled = true;
                ////////////////////////////////////
                // AIMING ARROW
                ////////////////////////////////////
                this.outline = new Phaser.Image(this.game, 0, 0, 'outline');
                ScaleUtil_28.scaleToBounds(this.outline, this.game.width * 0.7, this.game.height * 0.65);
                this.outline.centerX = this.game.width * 0.5;
                this.outline.centerY = this.game.height * 0.45;
                this.outline.alpha = 0;
                this.dartLayer = new Phaser.Group(this.game);
                this.snatch = new Snatch_4.Snatch(this.game);
                this.add.existing(this.bg);
                this.add.existing(this.cameraSprite);
                this.add.existing(this.capturedImage);
                this.add.existing(this.posterBg);
                this.add.existing(this.itemsEmitter);
                this.add.existing(this.dartLayer);
                this.add.existing(this.overlay);
                this.add.existing(this.outline);
                this.add.existing(this.headerInstructions);
                this.add.existing(this.footerGroup);
                this.add.existing(this.snatch);
                yield this.dismissOverlay();
                this.playPoster();
            });
        }
        playPoster() {
            return __awaiter(this, void 0, void 0, function* () {
                TSMAnalyticsManager_14.TSMAnalyticsManager.Instance.onInstallationStarts();
                // What's the status of the camera permissions on this device?
                this.cameraSprite.hideCaptureImage();
                let didCameraEnableSucceed = yield this.cameraSprite.openCamera();
                TSMAnalyticsManager_14.TSMAnalyticsManager.Instance.onPosterWatch();
                const isSuccess = yield this.doShowControlEffect(didCameraEnableSucceed);
                if (!isSuccess) {
                    this.game.state.start(StateNames_33.StateNames.POSTER);
                    return;
                }
                // Either way, do B-Show flow (even if A-Show trigger succeeded!)
                this.doBShow();
                this.finish();
            });
        }
        doCameraFlow() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] doCameraFlow()`);
                this.game.add.tween(this.bg).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.headerInstructions).to({ top: 0 }, 350, Phaser.Easing.Sinusoidal.Out, true);
                this.game.add.tween(this.footerGroup).to({ y: this.game.height }, 350, Phaser.Easing.Sinusoidal.Out, true);
                this.game.add.tween(this.outline).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
                yield this.waitForClickFireButton();
            });
        }
        doFakeCameraFlow() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] doFakeCameraFlow()`);
                this.game.add.tween(this.posterBg).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.headerInstructions).to({ top: 0 }, 350, Phaser.Easing.Sinusoidal.Out, true);
                this.game.add.tween(this.footerGroup).to({ y: this.game.height }, 350, Phaser.Easing.Sinusoidal.Out, true);
                yield this.waitForClickFireButton();
            });
        }
        // Snapshot the screen
        snapShotScreen() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] snapShotScreen()`);
                yield this.cameraSprite.capture();
                this.cameraSprite.closeCamera(false);
            });
        }
        waitForClickFireButton() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] waitForClickFireButton()`);
                this.fireButton.startPulse();
                // Wait for player to click the fire button
                yield PromiseUtil_52.PromiseUtil.signal(this.fireButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                this.fireButton.stopPulse();
                this.game.add.tween(this.outline).to({ alpha: 0 }, 150, Phaser.Easing.Linear.None, true);
            });
        }
        doShowControlEffect(cameraEnabled) {
            return __awaiter(this, void 0, void 0, function* () {
                // Trigger the effect
                if (cameraEnabled) {
                    yield this.doCameraFlow();
                }
                else {
                    yield this.doFakeCameraFlow();
                }
                // The dart firing animation should be started at the same time the physical is triggered (approx)
                let showEffectResult;
                yield Promise.all([
                    GameController_29.GameController.Instance.showEffectController.triggerDefaultEffect(5).then((result) => {
                        showEffectResult = result;
                    }),
                    this.doFireDart(cameraEnabled).then(this.showHold.bind(this))
                ]);
                TSMAnalyticsManager_14.TSMAnalyticsManager.Instance.onPosterTrigger(showEffectResult);
                if (showEffectResult === ShowEffectController_2.ShowEffectResult.SUCCESS) {
                    // If it worked, do A-Show flow.
                    this.doAShow();
                }
                else if (showEffectResult === ShowEffectController_2.ShowEffectResult.FAILURE) {
                    // immediately go to B-Show
                }
                else if (showEffectResult === ShowEffectController_2.ShowEffectResult.IN_USE) {
                    if (PosterInstallationState.inUseAttemptCount++ < PosterInstallationState.MAX_IN_USE_ATTEMPTS) {
                        yield this.showTryAgain();
                        return false;
                    }
                }
                return true;
            });
        }
        doFireDart(cameraEnabled) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] doFireDart()`);
                if (cameraEnabled) {
                    yield this.snapShotScreen();
                }
                yield this.fireDart(new Phaser.Point(this.game.width * 0.3, this.targetYPosition));
                yield this.fireDart(new Phaser.Point(this.game.width * 0.7, this.targetYPosition));
                yield this.fireDart(new Phaser.Point(this.game.width * 0.3, this.targetYPosition));
                this.game.add.tween(this.footerGroup).to({ top: this.game.height }, 350, Phaser.Easing.Sinusoidal.In, true);
            });
        }
        fireDart(destinationPos) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] fireDart(${destinationPos.x, destinationPos.y})`);
                // Await rotate cannon to face point
                const destRotation = Phaser.Math.angleBetweenPoints(new Phaser.Point(this.cannon.worldPosition.x, this.cannon.worldPosition.y), destinationPos) + (Math.PI * 0.5);
                yield PromiseUtil_52.PromiseUtil.tween(this.game, this.game.add.tween(this.cannon).to({ rotation: destRotation }, 250, Phaser.Easing.Sinusoidal.InOut, true));
                const dart = new Phaser.Image(this.game, 0, 0, "inst-dart");
                dart.anchor.set(0.5, 0);
                dart.width = this.cannon.width * 0.175;
                dart.scale.y = dart.scale.x;
                dart.centerX = this.cannon.x;
                dart.top = this.cannon.worldPosition.y;
                dart.visible = true;
                dart.rotation = destRotation;
                this.dartLayer.add(dart);
                this.game.sound.play('dart_fire');
                yield Promise.all([
                    PromiseUtil_52.PromiseUtil.tween(this.game, this.game.add.tween(dart)
                        .to({ x: destinationPos.x, y: destinationPos.y }, 450, Phaser.Easing.Sinusoidal.Out, true)),
                    PromiseUtil_52.PromiseUtil.tween(this.game, this.game.add.tween(dart.scale)
                        .to({ y: dart.scale.y * 0.5, x: dart.scale.x * 0.5 }, 450, Phaser.Easing.Cubic.In, true)),
                    PromiseUtil_52.PromiseUtil.tween(this.game, this.game.add.tween(dart)
                        .to({ alpha: 0 }, 100, Phaser.Easing.Sinusoidal.Out, true, 350)),
                    PromiseUtil_52.PromiseUtil.time(this.game, 350).then(() => {
                        this.snatch.show(destinationPos.x, destinationPos.y);
                    })
                ]);
                dart.visible = false;
            });
        }
        showHold() {
            console.log(`[${this.stateName}] showHold()`);
            // Bring down the header and change its text
            this.headerInstructions.fontSize = TextUtil_22.TextUtil.getFontSize(this.game.width, 40);
            this.headerInstructions.text = LocalizationManager_31.LocalizationManager.Instance.text('poster_inst.look_up');
        }
        showTryAgain() {
            return __awaiter(this, void 0, void 0, function* () {
                this.headerInstructions.text = LocalizationManager_31.LocalizationManager.Instance.text('poster_inst.in_use1');
                this.fireButton.text = LocalizationManager_31.LocalizationManager.Instance.text("poster_inst.try_again");
                yield PromiseUtil_52.PromiseUtil.time(this.game, 3000);
                this.headerInstructions.text = LocalizationManager_31.LocalizationManager.Instance.text('poster_inst.in_use2');
                this.game.add.tween(this.footerGroup).to({ y: this.game.height }, 350, Phaser.Easing.Sinusoidal.Out, true);
                TSMAnalyticsManager_14.TSMAnalyticsManager.Instance.onPosterInUse();
                yield PromiseUtil_52.PromiseUtil.signal(this.fireButton.onChildInputDown);
            });
        }
        /**
         * Note: This function deliberately left empty.
         *
         * This installation has nothing for the guest to watch, so we do not need
         * to hold up the flow while they're distracted. There is essentially no
         * A-Show handling for this installation.
         */
        doAShow() {
            console.log(`[${this.stateName}] doAShow()`);
        }
        /**
         * Start raining objects from top of screen, then wait a short while.
         *
         * Note: this function should be called regardless of whether the
         * installation trigger succeeded or not.
         */
        doBShow() {
            console.log(`[${this.stateName}] doBShow()`);
            TSMAnalyticsManager_14.TSMAnalyticsManager.Instance.onPosterAnimation();
            // Move the header
            this.headerInstructions.fontSize = TextUtil_22.TextUtil.getFontSize(this.game.width, 25);
            this.game.add.tween(this.headerInstructions).to({ top: 0 }, 500, Phaser.Easing.Sinusoidal.InOut, true);
            this.headerInstructions.text = LocalizationManager_31.LocalizationManager.Instance.text('poster_inst.header_toys_found');
            // Fade out the overlay
            this.game.add.tween(this.overlay).to({ alpha: 0 }, 450, Phaser.Easing.Linear.None, true).onComplete.addOnce(() => {
                this.overlay.visible = false;
            });
            // Start B-Show FX
            this.itemsEmitter.start(false, 5000, 250);
        }
        finish() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[${this.stateName}] finish()`);
                yield PromiseUtil_52.PromiseUtil.time(this.game, 2000);
                yield PromiseUtil_52.PromiseUtil.signal(this.game.input.onTap);
                TSMAnalyticsManager_14.TSMAnalyticsManager.Instance.onPosterInstallationTime(false, false);
                TSMAnalyticsManager_14.TSMAnalyticsManager.Instance.onPosterInstallationTime(false, true);
                GameController_29.GameController.Instance.state.addCompletedPoster(PosterGameState_1.PosterGameState.INSTALLATION_POSTER_ID);
                yield this.showOverlay();
                CutSceneState_8.CutSceneState.showCutScene(this.game, 'pete_caught', StateNames_33.StateNames.HUB);
            });
        }
        shutdown() {
            // Ensure this style gets reset when leaving this state, whatever happens.
            document.body.style.backgroundColor = "rgba(0,0,0,1)";
            this.bg = null;
            this.posterBg = null;
            this.dartLayer = null;
            this.footerGroup = null;
            this.cannon = null;
            this.headerInstructions = null;
            this.itemsEmitter = null;
            this.overlay = null;
            this.cameraSprite = null;
            this.fireButton = null;
            this.outline = null;
            this.targetYPosition = null;
            this.snatch = null;
            this.capturedImage = null;
            this.cache.removeImage('image-capture');
            this.cache.removeImage('outline');
            this.cache.removeImage('button-green-fire');
            this.cache.removeImage('map-bg');
            this.cache.removeImage('poster-bg');
            this.cache.removeImage('poster-find');
            this.cache.removeImage('inst-cannon');
            this.cache.removeImage('inst-dart');
            this.cache.removeImage('snatch-radialfx');
            this.cache.removeImage('snatch-star-green');
            this.cache.removeImage('snatch-star-pink');
            this.cache.removeImage('snatch-star-blue');
            this.cache.removeImage('inst-loot');
            this.cache.removeImage('balloon-orange-pop');
            super.shutdown();
        }
    }
    PosterInstallationState.MAX_IN_USE_ATTEMPTS = 3;
    PosterInstallationState.inUseAttemptCount = 0;
    exports.PosterInstallationState = PosterInstallationState;
});
define("minigame/drawing/DrawingConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DrawingConfig {
        parse(data) {
            this.gameplay_duration_per_player = data.gameplay_duration_per_player || 20;
        }
    }
    exports.DrawingConfig = DrawingConfig;
});
define("minigame/drawing/DrawingState", ["require", "exports", "StateNames", "component/button/BigButton", "switchback-common/localization/LocalizationManager", "state/GameBaseState", "logic/GameController", "model/ToyStoryConfig", "component/TimerBar", "switchback-common/util/PromiseUtil", "component/MessageBar", "Game", "component/Curtain", "component/PassAndPlayShowable", "component/button/TextArrowButton", "minigame/drawing/DrawingConfig", "switchback-common/util/TextUtil", "logic/TSMAnalyticsManager"], function (require, exports, StateNames_34, BigButton_8, LocalizationManager_32, GameBaseState_9, GameController_30, ToyStoryConfig_16, TimerBar_2, PromiseUtil_53, MessageBar_16, Game_33, Curtain_7, PassAndPlayShowable_5, TextArrowButton_9, DrawingConfig_1, TextUtil_23, TSMAnalyticsManager_15) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DrawingState extends GameBaseState_9.GameBaseState {
        init() {
            this.stateName = StateNames_34.StateNames.GAME_DRAWING;
            super.init();
            this.config = this.loadConfig(DrawingConfig_1.DrawingConfig);
            this.score = 0;
            this.currentPlayer = GameController_30.GameController.Instance.playerManager.currentPlayer;
            this.gameDuration = GameController_30.GameController.Instance.players.length * this.config.gameplay_duration_per_player;
        }
        preload() {
            super.preload();
            this.game.load.image('tiling-bar', 'assets/common/CTAbar_tilingbg.png');
            this.game.load.image('cta-curtain', 'assets/common/cta-curtain.png');
            this.game.load.image('next-button', 'assets/common/next-button.png');
            this.game.load.image('right-figure', 'assets/common/right_figure.png');
            this.game.load.image('left-figure', 'assets/common/left_figure.png');
            this.game.load.image('phone', 'assets/common/phone.png');
            this.game.load.spritesheet('timerbar-icon', 'assets/common/timerbar-icon.png', 104, 114);
            this.game.load.image('timerbar-timer', 'assets/common/timerbar-timer.png');
            this.game.load.image('timerbar-frame-tile', 'assets/common/timerbar-frame-tile.png');
            this.game.load.image('drawing-background', 'assets/minigame/drawing/Drawing_BG.jpg');
            this.game.load.image('drawing-instruction-back', 'assets/minigame/drawing/DrawingInstructionPanel.png');
            this.game.load.image('drawing-draw-area', 'assets/minigame/drawing/DrawingArea.png');
            this.game.load.image('drawing-draw-area-back', 'assets/minigame/drawing/DrawingAreaBack.jpg');
            this.game.load.image('drawing-stamp', 'assets/minigame/drawing/stamp.png');
            this.game.load.image('button-big-backplate', 'assets/title/button-big-backplate.png');
            this.game.load.image('button-big-frame', 'assets/title/button-big-frame.png');
            this.game.load.image('button-clear', 'assets/minigame/drawing/Bigbtn_clear_btn.png');
            this.game.load.image('button-green-next', 'assets/title/button-green-next.png');
            this.game.load.audio('drawing_loop', ['assets/sound/jessie/drawing_loop.m4a', 'assets/sound/jessie/drawing_loop.ogg']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                window.t = this;
                this.canDraw = false;
                const aspectRatio = this.game.width / this.game.height;
                let aspectRatioMultiplier;
                if (aspectRatio > (9.0 / 16.0)) {
                    aspectRatioMultiplier = (9.0 / 16.0) / aspectRatio;
                }
                else {
                    aspectRatioMultiplier = 1;
                }
                const vGap = this.game.height * 0.01 * aspectRatioMultiplier;
                // ui elements
                const bg = new Phaser.Image(this.game, 0, 0, 'drawing-background');
                bg.width = this.game.width;
                bg.height = this.game.height;
                this.add.existing(bg);
                this.timerBar = new TimerBar_2.TimerBar(this.game, this.gameDuration * 1000);
                this.timerBar.top = this.game.height * 0.01;
                this.add.existing(this.timerBar);
                this.topMessageBar = new MessageBar_16.MessageBar(this.game, "", 32);
                this.topMessageBar.centerX = this.game.width * 0.5;
                this.topMessageBar.top = this.timerBar.bottom;
                this.add.existing(this.topMessageBar);
                this.buttonClear = new BigButton_8.BigButton(this.game, LocalizationManager_32.LocalizationManager.Instance.text("drawing.clear"), 'button-clear');
                this.buttonClear.width = this.game.width * 0.4 * aspectRatioMultiplier;
                this.buttonClear.scale.y = this.buttonClear.scale.x;
                this.buttonClear.centerX = this.game.width * 0.25;
                this.buttonClear.bottom = this.game.height - vGap;
                this.add.existing(this.buttonClear);
                this.buttonNext = new BigButton_8.BigButton(this.game, LocalizationManager_32.LocalizationManager.Instance.text("drawing.done"), 'button-green-next');
                this.buttonNext.width = this.game.width * 0.4 * aspectRatioMultiplier;
                this.buttonNext.scale.y = this.buttonNext.scale.x;
                this.buttonNext.centerX = this.game.width * 0.75;
                this.buttonNext.bottom = this.game.height - vGap;
                this.add.existing(this.buttonNext);
                const instructionTextBack = new Phaser.Image(this.game, 0, 0, 'drawing-instruction-back');
                instructionTextBack.width = this.game.width * 0.85 * aspectRatioMultiplier;
                instructionTextBack.scale.y = instructionTextBack.scale.x;
                instructionTextBack.centerX = this.game.width * 0.5;
                instructionTextBack.bottom = this.buttonClear.top - vGap;
                instructionTextBack.inputEnabled = false;
                this.add.existing(instructionTextBack);
                const instruction2Text = new Phaser.Text(this.game, this.game.width * 0.5, 0, LocalizationManager_32.LocalizationManager.Instance.text("drawing.instruction2"), {
                    font: Game_33.Game.DEFAULT_FONT,
                    fontSize: TextUtil_23.TextUtil.getFontSize(this.game.width, 32),
                    fill: Game_33.Game.TEXT_COLOR_WHITE,
                    stroke: Game_33.Game.TEXT_COLOR_BLUE,
                    strokeThickness: Game_33.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                    wordWrap: true,
                    wordWrapWidth: instructionTextBack.width * 0.98,
                    align: 'center'
                });
                instruction2Text.anchor.set(0.5, 0);
                instruction2Text.lineSpacing = instruction2Text.fontSize * -0.15;
                instruction2Text.inputEnabled = false;
                this.add.existing(instruction2Text);
                TextUtil_23.TextUtil.scaleDownTextToHeight(instruction2Text, instructionTextBack.height);
                instruction2Text.centerY = instructionTextBack.centerY;
                const drawingAreaBorder = new Phaser.Image(this.game, 0, 0, 'drawing-draw-area');
                drawingAreaBorder.width = this.game.width * 0.96;
                drawingAreaBorder.height = instructionTextBack.top - this.topMessageBar.bottom - (vGap * 2);
                drawingAreaBorder.centerX = this.game.width * 0.5;
                drawingAreaBorder.bottom = instructionTextBack.top - vGap;
                drawingAreaBorder.inputEnabled = false;
                const drawingAreaBackground = new Phaser.Image(this.game, 0, 0, 'drawing-draw-area-back');
                drawingAreaBackground.scale.x = drawingAreaBorder.scale.x;
                drawingAreaBackground.scale.y = drawingAreaBorder.scale.y;
                drawingAreaBackground.centerX = this.game.width * 0.5;
                drawingAreaBackground.centerY = drawingAreaBorder.centerY;
                this.add.existing(drawingAreaBackground);
                // drawing area
                this.drawingAreaX = drawingAreaBackground.left;
                this.drawingAreaY = drawingAreaBackground.top;
                // make sure these two numbers are rounded as otherwise it will not work correctly on mobile
                this.drawingAreaWidth = Math.ceil(drawingAreaBackground.width);
                this.drawingAreaHeight = Math.ceil(drawingAreaBackground.height);
                this.drawingArea = this.game.add.bitmapData(this.drawingAreaWidth, this.drawingAreaHeight);
                this.drawingArea.smoothed = false;
                this.drawingAreaContainer = new Phaser.Image(this.game, this.drawingAreaX, this.drawingAreaY, this.drawingArea);
                this.add.existing(this.drawingAreaContainer);
                this.drawingAreaContainer.inputEnabled = true;
                this.drawingAreaContainer.input.enableDrag();
                this.drawingAreaContainer.input.setDragLock(false, false);
                this.drawingAreaContainer.events.onDragUpdate.add(this.paint, this);
                this.drawingAreaContainer.events.onInputUp.add(this.onPointerUp, this);
                window.i = this.drawingAreaContainer;
                this.add.existing(drawingAreaBorder);
                this.stamp = this.game.make.sprite(0, 0, 'drawing-stamp');
                this.stamp.width = this.game.width * 0.02;
                this.stamp.scale.y = this.stamp.scale.x;
                this.stamp.anchor.set(0.5);
                // Instructions and Pass & Play
                this.curtain = new Curtain_7.Curtain(this.game, true);
                this.add.existing(this.curtain);
                this.passAndPlayAnimation = new PassAndPlayShowable_5.PassAndPlayShowable(this.game);
                this.passAndPlayAnimation.x = this.game.width * 0.5;
                this.passAndPlayAnimation.y = this.game.height * 0.5;
                this.add.existing(this.passAndPlayAnimation);
                this.readyButton = new TextArrowButton_9.TextArrowButton(this.game, LocalizationManager_32.LocalizationManager.Instance.text('common.ready').toUpperCase());
                this.readyButton.centerX = this.game.width * 0.5;
                this.readyButton.centerY = this.game.height * 0.785;
                this.readyButton.alpha = 0;
                this.readyButton.startPulse();
                this.add.existing(this.readyButton);
                this.sndDrawLoop = this.game.add.sound('drawing_loop', 0.0, true);
                yield this.dismissOverlay();
                if (ToyStoryConfig_16.ToyStoryConfig.Instance.skipGames) {
                    this.endGame(6);
                }
                else {
                    this.play();
                }
            });
        }
        update() {
            super.update();
            if (this.canDraw) {
                this.timeTaken += this.game.time.elapsedMS;
            }
        }
        play() {
            return __awaiter(this, void 0, void 0, function* () {
                this.trackGameState();
                this.trackGameStartTime();
                this.passToNextPlayer();
                yield PromiseUtil_53.PromiseUtil.signal(this.timerBar.timeUpSignal);
                this.onDrawingComplete();
            });
        }
        startDrawing() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("startDrawing");
                this.onClearClicked();
                this.topMessageBar.text = LocalizationManager_32.LocalizationManager.Instance.textWithReplacements("drawing.instruction1", {
                    player_name: this.currentPlayer.name
                }).toUpperCase();
                yield this.curtain.hide();
                this.readyButton.visible = false; // this will hitblock assets behind it
                this.buttonClear.onChildInputDown.add(this.onClearClicked, this);
                this.buttonNext.onChildInputDown.addOnce(this.onNextClicked, this);
                this.canDraw = true;
                this.timeTaken = 0;
                this.timerBar.start();
            });
        }
        passToNextPlayer() {
            return __awaiter(this, void 0, void 0, function* () {
                this.trackGamePassAndPlay();
                this.currentPlayer = GameController_30.GameController.Instance.playerManager.changePlayer();
                // Set pass and play text
                const passAndPlayText = LocalizationManager_32.LocalizationManager.Instance.textWithReplacements('drawing.instructions_pass', { player_name: this.currentPlayer.name }).toUpperCase();
                this.passAndPlayAnimation.text = passAndPlayText;
                // Show curtains, animation, button
                yield this.curtain.show(true);
                yield this.passAndPlayAnimation.show();
                yield this.game.add.tween(this.readyButton).to({ alpha: 1 }, 200, Phaser.Easing.Sinusoidal.In, true);
                // Wait for button input
                this.readyButton.visible = true;
                yield PromiseUtil_53.PromiseUtil.signal(this.readyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                // Hide button, animation, curtains
                yield this.game.add.tween(this.readyButton).to({ alpha: 0 }, 200, Phaser.Easing.Sinusoidal.Out, true);
                yield this.passAndPlayAnimation.hide();
                yield this.startDrawing();
            });
        }
        endGame(score) {
            return __awaiter(this, void 0, void 0, function* () {
                console.log(`[DrawingState] endGame()`);
                this.stopDrawing();
                GameController_30.GameController.Instance.postScore(this.stateName, score);
                yield this.curtain.show(false);
                this.transitionTo(StateNames_34.StateNames.RESULTS);
            });
        }
        onDrawingComplete() {
            return __awaiter(this, void 0, void 0, function* () {
                this.drawingAreaContainer.events.onDragUpdate.remove(this.paint, this);
                this.drawingAreaContainer.events.onInputUp.remove(this.onPointerUp, this);
                yield this.endGame(this.score);
            });
        }
        onNextClicked() {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.sound.play('snd_ui_click');
                GameController_30.GameController.Instance.achievementManager.onDrawingEndTurn(this.timeTaken);
                this.stopDrawing();
                this.score++;
                yield this.passToNextPlayer();
            });
        }
        stopDrawing() {
            this.timeTaken = 0;
            this.buttonClear.onChildInputDown.remove(this.onClearClicked, this);
            this.buttonNext.onChildInputDown.remove(this.onNextClicked, this);
            this.canDraw = false;
            this.timerBar.stop();
            this.sndDrawLoop.stop();
        }
        paint(obj, pointer, x, y) {
            if (!(pointer.isDown || this.canDraw)) {
                this.sndDrawLoop.stop();
                return;
            }
            let pos = pointer.position.clone();
            pos.x = Math.max(Math.min(pos.x, this.drawingAreaX + this.drawingAreaWidth), this.drawingAreaX);
            pos.y = Math.max(Math.min(pos.y, this.drawingAreaY + this.drawingAreaHeight), this.drawingAreaY);
            pos.x -= this.drawingAreaX;
            pos.y -= this.drawingAreaY;
            if (this.prevPoint) {
                this.drawingArea.line(this.prevPoint.x, this.prevPoint.y, pos.x, pos.y, 'black', this.game.width * 0.02);
            }
            else {
                this.drawingArea.draw(this.stamp, pos.x, pos.y);
                this.sndDrawLoop.play();
                this.sndDrawLoop.volume = 0;
                this.game.add.tween(this.sndDrawLoop).to({ volume: 0.75 }, 500, Phaser.Easing.Linear.None, true);
            }
            this.prevPoint = pos;
        }
        onPointerUp() {
            this.prevPoint = null;
            this.sndDrawLoop.stop();
        }
        onClearClicked() {
            TSMAnalyticsManager_15.TSMAnalyticsManager.Instance.onGameButtonClick(this.stateName, "Clear");
            this.game.sound.play('snd_ui_click');
            this.drawingArea.clear().update();
            this.sndDrawLoop.stop();
        }
        shutdown() {
            if (this.buttonClear) {
                this.buttonClear.onChildInputDown.remove(this.onClearClicked, this);
                this.buttonClear = null;
            }
            if (this.buttonNext) {
                this.buttonNext.onChildInputDown.remove(this.onNextClicked, this);
                this.buttonNext = null;
            }
            if (this.drawingAreaContainer) {
                this.drawingAreaContainer.events.onDragUpdate.remove(this.paint, this);
                this.drawingAreaContainer.events.onInputUp.remove(this.onPointerUp, this);
            }
            if (this.sndDrawLoop) {
                this.sndDrawLoop.destroy();
                this.sndDrawLoop = null;
            }
            if (this.drawingArea) {
                this.drawingArea.destroy();
                this.drawingArea = null;
            }
            this.currentPlayer = null;
            this.timerBar = null;
            this.topMessageBar = null;
            this.curtain = null;
            this.passAndPlayAnimation = null;
            this.readyButton = null;
            this.stamp = null;
            this.prevPoint = null;
            this.game.cache.removeImage('timerbar-icon');
            this.game.cache.removeImage('timerbar-timer');
            this.game.cache.removeImage('timerbar-frame-tile');
            this.game.cache.removeImage('drawing-background');
            this.game.cache.removeImage('drawing-instruction-back');
            this.game.cache.removeImage('drawing-draw-area');
            this.game.cache.removeImage('drawing-draw-area-back');
            this.game.cache.removeImage('drawing-stamp');
            this.game.cache.removeImage('button-big-backplate');
            this.game.cache.removeImage('button-big-frame');
            this.game.cache.removeImage('button-clear');
            this.game.cache.removeImage('button-green-next');
            this.game.cache.removeSound('drawing_loop');
            super.shutdown();
        }
    }
    exports.DrawingState = DrawingState;
});
define("minigame/sarge/ActionIcon", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil"], function (require, exports, ScaleUtil_29, PromiseUtil_54) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ActionIcon extends Phaser.Group {
        constructor(game) {
            super(game);
            this.shake = true; // flag for the shaking icon when the plates are dropped (player misses the action)
            this.correctActionWheel = new Phaser.Image(this.game, 0, 0, 'correct-icon-wheel');
            this.correctActionWheel.anchor.setTo(0.5);
            this.missedActionWheel = new Phaser.Image(this.game, 0, 0, 'incorrect-icon-wheel');
            this.missedActionWheel.anchor.setTo(0.5);
            this.correctActionBackground = new Phaser.Image(this.game, 0, 0, 'correct-icon-background');
            this.correctActionBackground.anchor.setTo(0.5);
            ScaleUtil_29.scaleToBounds(this.correctActionWheel, this.correctActionBackground.width * 0.8, this.correctActionBackground.height * 0.8);
            this.missedActionBackground = new Phaser.Image(this.game, 0, 0, 'incorrect-icon-background');
            this.missedActionBackground.anchor.setTo(0.5);
            ScaleUtil_29.scaleToBounds(this.missedActionWheel, this.missedActionBackground.width * 0.8, this.missedActionBackground.height * 0.8);
            // create the correct tick image
            this.correctActionTick = new Phaser.Image(this.game, 0, 0, 'correct-icon');
            this.correctActionTick.anchor.setTo(0.5);
            ScaleUtil_29.scaleToBounds(this.correctActionTick, this.game.width * 0.1, this.game.height * 0.1);
            // add the three assets to the group
            this.add(this.correctActionWheel);
            this.add(this.correctActionBackground);
            this.add(this.correctActionTick);
            this.add(this.missedActionWheel);
            this.add(this.missedActionBackground);
            // scale the correct action icon group and add it to the game
            ScaleUtil_29.scaleToBounds(this, this.game.width * 0.1, this.game.height * 0.1);
            this.incorrectActionIcon = new Phaser.Image(this.game, 0, 0, 'incorrect-icon');
            this.incorrectActionIcon.anchor.setTo(0.5);
            ScaleUtil_29.scaleToBounds(this.incorrectActionIcon, this.game.width * 0.1, this.game.height * 0.1);
            this.add(this.incorrectActionIcon);
            this.visible = false;
        }
        displayResultIcon(isCorrect, durationMs = 1000) {
            return __awaiter(this, void 0, void 0, function* () {
                const ANIMATION_TIME_MS = durationMs * 0.08;
                const WAIT_BEFORE_TICK_MS = durationMs * 0.2;
                const PULSE_ANIMATION_MS = durationMs * 0.04;
                const WHEEL_ANGLE_ROTATION_AMOUNT = 90;
                // reset the correctActionIcon and the tick assets
                ScaleUtil_29.scaleToBounds(this, this.game.width * 0.1, this.game.height * 0.1);
                ScaleUtil_29.scaleToBounds(this.correctActionTick, this.game.width * 0.1, this.game.height * 0.1);
                this.visible = true;
                this.correctActionTick.visible = false;
                this.incorrectActionIcon.visible = false;
                this.missedActionBackground.visible = false;
                this.missedActionWheel.visible = false;
                this.correctActionBackground.visible = false;
                this.correctActionWheel.visible = false;
                // if the action is correct, display the correct icon, else if they missed it, show the broken plates icon
                if (isCorrect) {
                    this.correctActionBackground.visible = true;
                    this.correctActionWheel.visible = true;
                }
                else {
                    this.missedActionBackground.visible = true;
                    this.missedActionWheel.visible = true;
                }
                if (isCorrect) {
                    this.game.sound.play('tick-correct');
                }
                else {
                    this.game.sound.play('tick-incorrect');
                }
                yield PromiseUtil_54.PromiseUtil.tween(this.game, this.game.add.tween(this.scale).to({ x: 0.7, y: 0.7 }, ANIMATION_TIME_MS, Phaser.Easing.Sinusoidal.InOut, true)); // scale the group up so it looks like it pops up
                if (!isCorrect) {
                    this.game.add.tween(this.missedActionWheel).to({ angle: this.missedActionWheel.angle + WHEEL_ANGLE_ROTATION_AMOUNT }, durationMs, Phaser.Easing.Linear.None, true);
                    this.shake = true;
                    this.shakeIcon();
                }
                else {
                    this.game.add.tween(this.correctActionWheel).to({ angle: this.correctActionWheel.angle + WHEEL_ANGLE_ROTATION_AMOUNT }, durationMs, Phaser.Easing.Linear.None, true);
                }
                yield PromiseUtil_54.PromiseUtil.time(this.game, WAIT_BEFORE_TICK_MS); // wait a little before displaying the tick icon
                if (!isCorrect) {
                    this.shake = false; // set shake to false so the shaking stops just before the tick appears
                }
                // if the action is correct, show the correct tick, else show the tick with the outline (where they should have gone)
                if (isCorrect) {
                    this.correctActionTick.visible = true;
                    this.correctActionTick.centerY = this.height * 0.3;
                    this.game.add.tween(this.correctActionTick.scale).to({ x: 0.7, y: 0.7 }, ANIMATION_TIME_MS, Phaser.Easing.Linear.None, true); // scale the tick up so it pops up
                }
                else {
                    this.incorrectActionIcon.visible = true;
                    this.incorrectActionIcon.centerY = this.height * 0.3;
                    this.game.add.tween(this.incorrectActionIcon.scale).to({ x: 0.7, y: 0.7 }, ANIMATION_TIME_MS, Phaser.Easing.Linear.None, true); // scale the tick up so it pops up
                }
                // add another pulse to the whole group
                yield PromiseUtil_54.PromiseUtil.tween(this.game, this.game.add.tween(this.scale).to({ x: 0.8, y: 0.8 }, PULSE_ANIMATION_MS, Phaser.Easing.Linear.None, true));
                this.game.add.tween(this.scale).to({ x: 0.7, y: 0.7 }, ANIMATION_TIME_MS, Phaser.Easing.Back.In, true);
                yield PromiseUtil_54.PromiseUtil.time(this.game, durationMs - ((WAIT_BEFORE_TICK_MS + ANIMATION_TIME_MS + PULSE_ANIMATION_MS) * 1.5));
                yield PromiseUtil_54.PromiseUtil.tween(this.game, this.game.add.tween(this.scale).to({ x: 0, y: 0 }, ANIMATION_TIME_MS, Phaser.Easing.Back.In, true)); // scale the group back down so it disappears
                // set the visibility of the assets to false
                if (isCorrect) {
                    this.correctActionTick.visible = false;
                    this.correctActionBackground.visible = false;
                    this.correctActionWheel.visible = false;
                }
                else {
                    this.incorrectActionIcon.visible = false;
                    this.missedActionBackground.visible = false;
                    this.missedActionWheel.visible = false;
                }
                this.visible = false;
            });
        }
        shakeIcon() {
            return __awaiter(this, void 0, void 0, function* () {
                const WOBBLE_OFFSET_X = 5;
                const WOBBLE_OFFSET_Y = 5;
                const SHAKE_DURATION = 10;
                const x = this.x;
                const y = this.y;
                while (this.shake) {
                    this.visible = true;
                    const randX = this.game.rnd.integerInRange(x + WOBBLE_OFFSET_X, x - WOBBLE_OFFSET_X);
                    const randY = this.game.rnd.integerInRange(y + WOBBLE_OFFSET_Y, y - WOBBLE_OFFSET_Y);
                    yield PromiseUtil_54.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ x: randX, y: randY }, SHAKE_DURATION, Phaser.Easing.Sinusoidal.InOut, true)); // shake the icon
                }
            });
        }
    }
    exports.ActionIcon = ActionIcon;
});
define("minigame/sarge/DropZoneTimer", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DropZoneTimer extends Phaser.Group {
        constructor(game) {
            super(game);
            this.dropCompleted = new Phaser.Signal();
            const timerBackground = new Phaser.Image(this.game, 0, 0, 'timer-bar-back');
            timerBackground.anchor.set(0.5, 0);
            timerBackground.centerX = 0;
            timerBackground.width = this.game.width * 0.2;
            timerBackground.scale.y = timerBackground.scale.x;
            this.addChild(timerBackground);
            this.timerForeground = new Phaser.Image(this.game, 0, 0, 'timer-bar-front');
            this.timerForeground.anchor.set(0.5, 0);
            this.timerForeground.centerX = 0;
            this.timerForeground.width = this.game.width * 0.2;
            this.timerForeground.scale.y = timerBackground.scale.x;
            this.addChild(this.timerForeground);
            this.visible = false;
        }
        beginTimer(dropDurationMs) {
            this.visible = true;
            this.timerForeground.scale.x = this.timerForeground.scale.y;
            this.dropTween = this.game.add.tween(this.timerForeground.scale)
                .to({ x: 0 }, dropDurationMs, Phaser.Easing.Linear.None, true);
            this.dropTween.onComplete.addOnce(this.onDropComplete, this);
        }
        interuptTimer() {
            if (this.visible) {
                this.endTimer();
                // kill the tween so it doesn't also trigger
                if (this.dropTween) {
                    this.dropTween.stop();
                    this.dropTween.onComplete.removeAll();
                    this.timerForeground.scale.x = 0;
                }
                // dispatch that this is now complete
                this.onDropComplete();
            }
        }
        endTimer() {
            this.visible = false;
        }
        onDropComplete() {
            this.dropCompleted.dispatch(this);
        }
        destroy() {
            this.dropCompleted.dispose();
            super.destroy();
        }
    }
    exports.DropZoneTimer = DropZoneTimer;
});
define("minigame/sarge/DropZone", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_55) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DropZone extends Phaser.Group {
        constructor(game, location, actionIcon, sargeLocationCall) {
            super(game);
            this.dropCompleteSignal = new Phaser.Signal();
            this.active = false;
            this.hasCrateWaiting = false;
            this.location = location;
            this.actionIcon = actionIcon;
            this.sargeLocationCall = sargeLocationCall;
            const zoneWidth = this.game.width * 0.25;
            const zoneHeight = this.game.height * 0.15;
            this.dropShadow = new Phaser.Image(this.game, 0, zoneHeight * 0.15, 'sarge-supply-crate-shadow');
            this.dropShadow.anchor.set(0.5, 0.2);
            this.dropShadow.width = zoneWidth * 2;
            this.dropShadow.scale.y = this.dropShadow.scale.x;
            this.dropShadow.alpha = 0;
            this.add(this.dropShadow);
            this.origDropShadowScale = this.dropShadow.scale.x;
            this.supplyCrate = new Phaser.Image(this.game, 0, -this.game.height, 'sarge-supply-crate');
            this.supplyCrate.width = zoneWidth * 0.55;
            this.supplyCrate.scale.y = this.supplyCrate.scale.x;
            this.supplyCrate.anchor.set(0.5);
            this.add(this.supplyCrate);
        }
        setDropZoneTime(timer) {
            this.countdownTimer = timer;
        }
        reset() {
            this.dropShadow.alpha = 0;
            this.dropShadow.scale.set(this.origDropShadowScale);
            this.supplyCrate.visible = true;
            this.supplyCrate.y = -this.game.height;
        }
        initiateDrop(durationSeconds = 5) {
            return __awaiter(this, void 0, void 0, function* () {
                this.active = true;
                this.reset();
                const dropDurationMs = durationSeconds * 1000;
                this.game.add.tween(this.dropShadow).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true);
                this.game.add.tween(this.dropShadow.scale).from({ x: 0, y: 0 }, dropDurationMs, Phaser.Easing.Linear.None, true);
                this.countdownTimer.dropCompleted.addOnce(this.onDropComplete, this);
                this.countdownTimer.beginTimer(dropDurationMs);
                yield PromiseUtil_55.PromiseUtil.signal(this.dropCompleteSignal);
            });
        }
        sargeEnterZone() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this.countdownTimer.visible) {
                    this.countdownTimer.interuptTimer();
                    if (this.active) {
                        yield PromiseUtil_55.PromiseUtil.signal(this.dropCompleteSignal);
                    }
                }
            });
        }
        onDropComplete() {
            return __awaiter(this, void 0, void 0, function* () {
                PromiseUtil_55.PromiseUtil.time(this.game, 250).then(() => {
                    this.game.sound.play('crate-fall-correct');
                });
                this.countdownTimer.endTimer();
                yield PromiseUtil_55.PromiseUtil.tween(this.game, this.game.add.tween(this.supplyCrate).to({ y: 0 }, 200, Phaser.Easing.Sinusoidal.In, true));
                this.game.add.tween(this.dropShadow).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true);
                // Was sarge on this dropzone when the crate landed?
                const succeeded = (this.sargeLocationCall() === this.location);
                const animDurationMs = 1250;
                PromiseUtil_55.PromiseUtil.time(this.game, animDurationMs * 0.25).then(() => {
                    this.supplyCrate.visible = false;
                });
                this.dropCompleteSignal.dispatch(succeeded);
                yield this.actionIcon.displayResultIcon(succeeded, animDurationMs);
                this.active = false;
            });
        }
        destroy() {
            this.dropCompleteSignal.dispose();
            super.destroy();
        }
    }
    exports.DropZone = DropZone;
});
define("minigame/sarge/DirectionButton", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "minigame/flying-tosser/ButtonPressTutorialPrompt", "minigame/sarge/SargeGameState"], function (require, exports, ScaleUtil_30, PromiseUtil_56, ButtonPressTutorialPrompt_2, SargeGameState_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DirectionButton extends Phaser.Group {
        constructor(game) {
            super(game);
            this.clickedSignal = new Phaser.Signal();
            this.button = new Phaser.Button(this.game, 0, 0, 'radio-btn-arrow');
            this.button.inputEnabled = true;
            this.button.onInputDown.add(this.dispatchDirection, this);
            this.button.anchor.set(0.5);
            this.add(this.button);
        }
        set direction(direction) {
            this._direction = direction;
            switch (direction) {
                case SargeGameState_1.DropZoneLocation.UP: {
                    this.button.angle = 0;
                    break;
                }
                case SargeGameState_1.DropZoneLocation.DOWN: {
                    this.button.angle = 180;
                    break;
                }
                case SargeGameState_1.DropZoneLocation.LEFT: {
                    this.button.angle = -90;
                    break;
                }
                case SargeGameState_1.DropZoneLocation.RIGHT: {
                    this.button.angle = 90;
                    break;
                }
            }
        }
        get direction() {
            return this._direction;
        }
        dispatchDirection() {
            this.game.sound.play('snd_ui_click');
            this.clickedSignal.dispatch(this._direction);
        }
    }
    exports.DirectionButton = DirectionButton;
    class DirectionButtonGroup extends Phaser.Group {
        constructor(game, dropZone) {
            super(game);
            this.inputEnableChildren = true;
            this.tutorialButtonTweens = [];
            this.walkieTalkieImage = new Phaser.Image(this.game, 0, 0, 'radio-base');
            ScaleUtil_30.scaleToBounds(this.walkieTalkieImage, this.game.width * 0.35, this.game.height * 0.4);
            this.add(this.walkieTalkieImage);
            this.buttonCenter = new Phaser.Point(this.walkieTalkieImage.width * 0.4, this.walkieTalkieImage.height * 0.558);
            this.redButtonGroup = new Phaser.Group(this.game);
            this.redButton = new Phaser.Image(this.game, 0, 0, 'radio-btn');
            this.redButton.inputEnabled = true;
            this.redButton.anchor.set(0.5);
            this.redButtonGroup.add(this.redButton);
            this.directionButton = new DirectionButton(this.game);
            this.directionButton.direction = dropZone;
            this.directionButton.centerY = this.redButton.top + this.redButton.height * 0.4246;
            this.redButtonGroup.add(this.directionButton);
            this.buttonGlow = new Phaser.Image(this.game, 0, 0, 'radio-btn-glow');
            this.buttonGlow.scale = this.redButton.scale;
            this.buttonGlow.alpha = 0;
            this.buttonGlow.centerX = this.redButton.centerX;
            this.buttonGlow.centerY = this.redButton.centerY;
            this.redButtonGroup.add(this.buttonGlow);
            this.buttonGroupOffset = (this.buttonGlow.width - this.walkieTalkieImage.width) * 0.5;
            this.redButtonGroup.scale.x = this.walkieTalkieImage.scale.x;
            this.redButtonGroup.scale.y = this.walkieTalkieImage.scale.y;
            this.redButtonGroup.centerX = this.buttonCenter.x;
            this.redButtonGroup.centerY = this.buttonCenter.y;
            this.add(this.redButtonGroup);
            this.redButtonGroup.onChildInputDown.add(this.directionButton.dispatchDirection, this.directionButton);
            this.antennaFlash = new Phaser.Image(this.game, 0, 0, 'radio-btn-spark');
            this.antennaFlash.anchor.set(0.5, 0);
            this.antennaFlash.scale.set(this.walkieTalkieImage.scale.x);
            this.antennaFlash.left = 0;
            this.antennaFlash.y = this.walkieTalkieImage.top - (this.walkieTalkieImage.height * 0.07);
            this.antennaFlash.alpha = 0;
            this.add(this.antennaFlash);
            this.pressPrompt = new ButtonPressTutorialPrompt_2.ButtonPressTutorialPrompt(this.game);
            this.pressPrompt.bottom = this.walkieTalkieImage.height * 0.3;
            this.pressPrompt.left = this.left + this.buttonGroupOffset;
            this.pressPrompt.alpha = 0;
            this.add(this.pressPrompt);
        }
        onClick() {
            return __awaiter(this, void 0, void 0, function* () {
                // Flash light on antenna
                this.game.add.tween(this.antennaFlash).to({ alpha: 1 }, 300, Phaser.Easing.Linear.None, true, 0, 1, true).onComplete.add(() => {
                    this.antennaFlash.alpha = 0;
                });
                // Apply tint to red button
                this.redButtonGroup.setAllChildren('tint', 0x5c3f2b);
                // Scale red button down
                this.game.add.tween(this.redButtonGroup.scale).to({ x: this.redButtonGroup.scale.x * 0.8, y: this.redButtonGroup.scale.y * 0.8 }, 150, Phaser.Easing.Sinusoidal.In, true, 0, 0, true)
                    .onComplete.add(() => {
                    // Clear tint
                    this.redButtonGroup.setAllChildren('tint', 0xFFFFFF);
                });
                // Move walkie talkie up and down. The direction depends on whether
                // the button is on the top or bottom of the screen
                let scaleTween;
                if (this.scale.y < 0) {
                    scaleTween = this.game.add.tween(this).to({ top: this.top - this.game.width * 0.05 }, 150, Phaser.Easing.Sinusoidal.In, true, 0, 0, true);
                }
                else {
                    scaleTween = this.game.add.tween(this).to({ bottom: this.bottom + this.game.width * 0.05 }, 150, Phaser.Easing.Sinusoidal.In, true, 0, 0, true);
                }
                yield PromiseUtil_56.PromiseUtil.tween(this.game, scaleTween);
            });
        }
        set location(loc) {
            if (loc.x === 1) {
                this.scale.x *= -1;
                // Revert scaling of elements we don't want flipped
                this.directionButton.scale.x *= -1;
                this.pressPrompt.scale.x *= -1;
            }
            if (loc.y === 0) {
                this.scale.y *= -1;
                // Revert scaling of elements we don't want flipped
                this.directionButton.scale.y *= -1;
                this.pressPrompt.scale.y *= -1;
                this.pressPrompt.angle = 180;
            }
        }
        showButtonPrompt() {
            this.pressPrompt.alpha = 1;
            this.pressPrompt.show();
            this.pressPrompt.showHoldAnimation("sarge.press");
            // Add glow to red button group
            this.tutorialButtonTweens.push(this.game.add.tween(this.buttonGlow).to({ alpha: 1 }, 300, Phaser.Easing.Linear.None, true, 0, Infinity, true));
            this.tutorialButtonTweens.push(this.game.add.tween(this.redButtonGroup).to({ bottom: this.redButtonGroup.bottom - this.redButtonGroup.height * 0.04 }, 400, Phaser.Easing.Elastic.InOut, true, 0, Infinity, true));
        }
        hideButtonPrompt() {
            this.pressPrompt.hide();
            for (let tween of this.tutorialButtonTweens) {
                this.game.tweens.remove(tween);
            }
            this.tutorialButtonTweens = [];
            this.buttonGlow.alpha = 0;
        }
    }
    exports.DirectionButtonGroup = DirectionButtonGroup;
});
define("minigame/sarge/SargeDifficultyConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SargeDifficultyConfig {
        parse(data) {
            this.timeBetweenDropsMsMin = data.timeBetweenDropsMsMin || 2500;
            this.timeBetweenDropsMsMax = data.timeBetweenDropsMsMax || 4000;
            this.timeBetweenDropsMsDelta = data.timeBetweenDropsMsDelta || 500;
            this.durationOfDropsSMin = data.durationOfDropsSMin || 3;
            this.durationOfDropsSMax = data.durationOfDropsSMax || 5;
            this.areButtonsRandomized = data.areButtonsRandomized || false;
            this.maxActiveDrops = data.maxActiveDrops || 2;
        }
    }
    exports.SargeDifficultyConfig = SargeDifficultyConfig;
});
define("minigame/sarge/SargeConfig", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SargeConfig {
        parse(data) {
            this.tutorialDropDurationS = data.tutorialDropDurationS || 2.5;
            this.gameDuration = data.gameDuration || 25000;
        }
    }
    exports.SargeConfig = SargeConfig;
});
define("minigame/sarge/SargeSprite", ["require", "exports", "switchback-common/util/ScaleUtil"], function (require, exports, ScaleUtil_31) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SargeSprite extends Phaser.Group {
        constructor(game) {
            super(game);
            this.sarge = new Phaser.Sprite(game, 0, 0, "army-man");
            ScaleUtil_31.scaleToBounds(this.sarge, this.game.width * 0.2, this.game.height * 0.2);
            this.sarge.anchor.set(0.5);
            this.sarge.animations.add('walk', null, SargeSprite.BASE_FRAME_RATE, true);
            this.add(this.sarge);
        }
        beginWalking() {
            this.sarge.animations.play('walk');
        }
        endWalking() {
            this.sarge.animations.stop("walk", true);
        }
    }
    SargeSprite.BASE_FRAME_RATE = 24;
    exports.SargeSprite = SargeSprite;
});
define("minigame/sarge/SargeGameTimerPanel", ["require", "exports", "Game", "switchback-common/util/TextUtil"], function (require, exports, Game_34, TextUtil_24) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SargeGameTimerPanel extends Phaser.Group {
        constructor(game) {
            super(game);
            const timerBackground = new Phaser.Image(this.game, 0, 0, 'sarge-timerbox');
            timerBackground.width = this.game.width * 0.3125;
            timerBackground.scale.y = timerBackground.scale.x;
            this.addChild(timerBackground);
            const timerIcon = new Phaser.Image(this.game, 0, 0, 'timerbar-icon');
            timerIcon.animations.add("tick", [0, 1], 2, true);
            timerIcon.play("tick");
            timerIcon.width = this.game.width * 0.12;
            timerIcon.scale.y = timerIcon.scale.x;
            timerIcon.left = timerBackground.left;
            timerIcon.centerY = timerBackground.centerY;
            this.addChild(timerIcon);
            this.timerText = new Phaser.Text(this.game, 0, 0, '30s', {
                fontSize: Game_34.Game.FONT_SIZE_LARGE,
                font: Game_34.Game.DEFAULT_FONT,
                fill: Game_34.Game.TEXT_COLOR_WHITE,
                stroke: "#394db5",
                strokeThickness: Game_34.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                align: 'left'
            });
            TextUtil_24.TextUtil.scaleDownSingleLineTextToWidth(this.timerText, timerBackground.width - timerIcon.width);
            this.timerText.left = timerIcon.right;
            this.timerText.centerY = timerBackground.centerY;
            this.addChild(this.timerText);
        }
        setTweenDestination(onScreenX, offScreenX) {
            this.onScreenX = onScreenX;
            this.offScreenX = offScreenX;
            this.x = this.offScreenX; // start off screen
        }
        showTimer() {
            this.x = this.offScreenX;
            this.game.add.tween(this).to({ x: this.onScreenX }, 500, Phaser.Easing.Linear.None, true);
        }
        hideTimer() {
            this.x = this.onScreenX;
            this.game.add.tween(this).to({ x: this.offScreenX }, 500, Phaser.Easing.Linear.None, true);
        }
        updateTime(timeRemainingMs) {
            this.timerText.text = Math.ceil(timeRemainingMs / 1000).toString() + "s";
        }
    }
    exports.SargeGameTimerPanel = SargeGameTimerPanel;
});
define("minigame/sarge/SargeGameState", ["require", "exports", "switchback-common/util/ScaleUtil", "switchback-common/util/PromiseUtil", "switchback-common/localization/LocalizationManager", "minigame/sarge/DropZone", "logic/GameController", "Game", "minigame/sarge/ActionIcon", "StateNames", "minigame/sarge/DirectionButton", "component/Curtain", "component/MessageBar", "component/button/TextArrowButton", "model/ToyStoryConfig", "minigame/sarge/SargeDifficultyConfig", "minigame/sarge/SargeConfig", "minigame/sarge/DropZoneTimer", "minigame/sarge/SargeSprite", "state/GameBaseState", "minigame/sarge/SargeGameTimerPanel"], function (require, exports, ScaleUtil_32, PromiseUtil_57, LocalizationManager_33, DropZone_1, GameController_31, Game_35, ActionIcon_1, StateNames_35, DirectionButton_1, Curtain_8, MessageBar_17, TextArrowButton_10, ToyStoryConfig_17, SargeDifficultyConfig_1, SargeConfig_1, DropZoneTimer_1, SargeSprite_1, GameBaseState_10, SargeGameTimerPanel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class DropZoneLocation {
    }
    DropZoneLocation.UP = "UP";
    DropZoneLocation.DOWN = "DOWN";
    DropZoneLocation.LEFT = "LEFT";
    DropZoneLocation.RIGHT = "RIGHT";
    DropZoneLocation.MIDDLE = "MIDDLE";
    exports.DropZoneLocation = DropZoneLocation;
    class SargeGameState extends GameBaseState_10.GameBaseState {
        constructor() {
            super(...arguments);
            this.gameStarted = false;
            this.graphicPos = 0.45;
            this.topMessageBarPos = 0.15;
            this.bottomMessageBarPos = 0.75;
            this.topButtonPos = 0.075;
            this.bottomButtonPos = 0.825;
        }
        init() {
            this.stateName = StateNames_35.StateNames.GAME_SARGE;
            super.init();
            this.config = this.loadConfig(SargeConfig_1.SargeConfig);
            this.difficultyConfig = this.loadDifficultyConfig(SargeDifficultyConfig_1.SargeDifficultyConfig);
            this.directions = [
                DropZoneLocation.UP,
                DropZoneLocation.DOWN,
                DropZoneLocation.RIGHT,
                DropZoneLocation.LEFT
            ];
            this.totalScore = 0;
            this.walkieTalkieButtons = [];
            this.hasPlayedTutorial = GameController_31.GameController.Instance.state.getMinigameSaveData(StateNames_35.StateNames.GAME_SARGE).numPlays > 0;
            this.activeGamePromiseResolution = null;
            this.currentSargeLocation = DropZoneLocation.MIDDLE;
            let rounds = GameController_31.GameController.Instance.players.length;
            if (rounds === 2) {
                // If we only have two players, we don't want them to
                // play more than one round
                rounds = 1;
            }
            this.players = GameController_31.GameController.Instance.playerManager.getPlayerSession(2, rounds, true);
            this.currentRoundIndex = 0;
            this.dropZones = new Map();
        }
        preload() {
            super.preload();
            // load background assets
            this.game.load.image('sarge-background', 'assets/minigame/sarge/Army_Main_background.jpg');
            // load button assets
            this.game.load.image('radio-base', 'assets/minigame/sarge/radio_button_base.png');
            this.game.load.image('radio-btn', 'assets/minigame/sarge/radio_btn_blank.png');
            this.game.load.image('radio-btn-arrow', 'assets/minigame/sarge/radio_btn_arrow.png');
            this.game.load.image('radio-btn-glow', 'assets/minigame/sarge/radio_btn_glow.png');
            this.game.load.image('radio-btn-spark', 'assets/minigame/sarge/radio_btn_spark.png');
            this.game.load.image('timer-icon', 'assets/common/timer-icon.png');
            // load tutorial assets
            this.game.load.image('tutorial-sarge', 'assets/minigame/sarge/tutorial_sarge.png');
            this.game.load.image('tutorial-walkie-talkie', 'assets/minigame/sarge/tutorial_walkie_talkies.png');
            // load the army man
            this.game.load.atlas('army-man', 'assets/minigame/sarge/ArmyMan_walk.png', 'assets/minigame/sarge/ArmyMan_walk.json');
            // load the "correct" and "incorrect" images for when the player guesses correctly/incorrectly
            this.game.load.image('correct-icon', 'assets/minigame/sarge/ArmyPopup_success_ico.png');
            this.game.load.image('incorrect-icon', 'assets/minigame/sarge/ArmyPopup_failure_ico.png');
            this.game.load.image('correct-icon-wheel', 'assets/minigame/sarge/ArmyPopup_wheel_success.png');
            this.game.load.image('incorrect-icon-wheel', 'assets/minigame/sarge/ArmyPopup_wheel_failure.png');
            this.game.load.image('correct-icon-background', 'assets/minigame/sarge/ArmyPopup_successframe_default.png');
            this.game.load.image('incorrect-icon-background', 'assets/minigame/sarge/ArmyPopup_failureframe.png');
            this.game.load.image('sarge-supply-crate', 'assets/minigame/sarge/SupplyCrate_default.png');
            this.game.load.image('sarge-supply-crate-shadow', 'assets/minigame/sarge/SupplyCrate_shadow.png');
            this.game.load.image('tiling-bar', 'assets/common/CTAbar_tilingbg.png');
            this.game.load.image('cta-curtain', 'assets/common/cta-curtain.png');
            this.game.load.image('next-button', 'assets/common/next-button.png');
            this.game.load.image('crate-icon', 'assets/minigame/sarge/crate_icon.png');
            this.game.load.image('timer-bar-back', 'assets/minigame/sarge/Sarge_dropbar_back.png');
            this.game.load.image('timer-bar-front', 'assets/minigame/sarge/Sarge_dropbar_bar.png');
            this.game.load.spritesheet('timerbar-icon', 'assets/common/timerbar-icon.png', 104, 114);
            this.game.load.image("sarge-timerbox", 'assets/minigame/sarge/sarge_timerbox.png');
            this.game.load.spritesheet("hold-prompt", "assets/minigame/planetary-putt/hold-prompt.png", 150, 304);
            this.game.load.spritesheet("release-prompt", "assets/minigame/planetary-putt/release-prompt.png", 151, 304);
            this.game.load.image("prompt-window", "assets/minigame/planetary-putt/prompt-window.png");
            // load the sounds
            this.game.load.audio('sarge-move', ['assets/sound/sarge/sarge_move.m4a', 'assets/sound/sarge/sarge_move.ogg']);
            this.game.load.audio('soldiers-move', ['assets/sound/sarge/soldiers_move.m4a', 'assets/sound/sarge/soldiers_move.ogg']);
            this.game.load.audio('tick-correct', ['assets/sound/sarge/tick_correct.m4a', 'assets/sound/sarge/tick_correct.ogg']);
            this.game.load.audio('tick-incorrect', ['assets/sound/sarge/tick_incorrect.m4a', 'assets/sound/sarge/tick_incorrect.ogg']);
            this.game.load.audio('crate-fall-correct', ['assets/sound/sarge/crate_fall_correct.m4a', 'assets/sound/sarge/crate_fall_correct.ogg']);
            this.game.load.audio('crate-fall-incorrect', ['assets/sound/sarge/crate_fall_incorrect.m4a', 'assets/sound/sarge/crate_fall_incorrect.ogg']);
        }
        create() {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                _super("create").call(this);
                this.trackGameState();
                const background = new Phaser.Image(this.game, 0, 0, 'sarge-background');
                background.width = this.game.width;
                background.height = this.game.height;
                this.add.existing(background);
                this.zoneLayer = new Phaser.Group(this.game);
                this.add.existing(this.zoneLayer);
                this.sarge = new SargeSprite_1.SargeSprite(this.game);
                this.sarge.position = this.getLocationForZone(DropZoneLocation.MIDDLE);
                this.add.existing(this.sarge);
                this.actionIconLayer = new Phaser.Group(this.game);
                this.add.existing(this.actionIconLayer);
                const dropZonesToMake = [DropZoneLocation.UP, DropZoneLocation.DOWN, DropZoneLocation.LEFT, DropZoneLocation.RIGHT];
                dropZonesToMake.forEach((zoneLocation) => {
                    // Make the action icon for the zone
                    const actionIcon = new ActionIcon_1.ActionIcon(this.game);
                    actionIcon.position = this.getLocationForZone(zoneLocation);
                    // Make the dropzone
                    const dropZone = new DropZone_1.DropZone(this.game, zoneLocation, actionIcon, this.getCurrentSargeLocation.bind(this));
                    dropZone.position = this.getLocationForZone(zoneLocation);
                    this.dropZones.set(zoneLocation, dropZone);
                    dropZone.dropCompleteSignal.add(this.onDropComplete, this);
                    // create the timer for the zone
                    const dropZoneTimer = new DropZoneTimer_1.DropZoneTimer(this.game);
                    dropZoneTimer.centerX = dropZone.centerX;
                    dropZoneTimer.bottom = dropZone.bottom - this.game.height * 0.024;
                    dropZone.setDropZoneTime(dropZoneTimer);
                    this.zoneLayer.add(dropZone);
                    this.actionIconLayer.add(actionIcon);
                    this.actionIconLayer.add(dropZoneTimer);
                });
                // Add score icons and text to each end of the screen
                const crateIconTop = new Phaser.Image(this.game, this.game.width * 0.525, 0, 'crate-icon');
                crateIconTop.width = this.game.height * 0.08;
                crateIconTop.scale.y = crateIconTop.scale.x;
                crateIconTop.anchor.set(1, 0.5);
                crateIconTop.top = 0;
                crateIconTop.angle = 180;
                this.add.existing(crateIconTop);
                const crateIconBottom = new Phaser.Image(this.game, this.game.width * 0.475, 0, 'crate-icon');
                crateIconBottom.width = this.game.height * 0.08;
                crateIconBottom.scale.y = crateIconBottom.scale.x;
                crateIconBottom.bottom = this.game.height;
                crateIconBottom.anchor.set(1, 0);
                this.add.existing(crateIconBottom);
                this.scoreTextTop = new Phaser.Text(this.game, this.game.width * 0.5, 0, 'x0', {
                    fontSize: Game_35.Game.FONT_SIZE_EXTRA_LARGE,
                    font: Game_35.Game.DEFAULT_FONT,
                    fill: Game_35.Game.TEXT_COLOR_WHITE,
                    stroke: "#394db5",
                    strokeThickness: Game_35.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                    align: 'center'
                });
                this.scoreTextTop.anchor.y = 0.5;
                this.scoreTextTop.left = crateIconTop.right;
                this.scoreTextTop.angle = 180;
                this.scoreTextTop.y = crateIconTop.centerY;
                this.add.existing(this.scoreTextTop);
                this.scoreTextBottom = new Phaser.Text(this.game, this.game.width * 0.5, this.game.height, 'x0', {
                    fontSize: Game_35.Game.FONT_SIZE_EXTRA_LARGE,
                    font: Game_35.Game.DEFAULT_FONT,
                    fill: Game_35.Game.TEXT_COLOR_WHITE,
                    stroke: "#394db5",
                    strokeThickness: Game_35.Game.TEXT_STROKE_THICKNESS_HUMONGOUS,
                    align: 'center'
                });
                this.scoreTextBottom.anchor.set(0.5);
                this.scoreTextBottom.left = crateIconBottom.right;
                this.scoreTextBottom.centerY = crateIconBottom.centerY;
                this.add.existing(this.scoreTextBottom);
                this.timerTop = new SargeGameTimerPanel_1.SargeGameTimerPanel(this.game);
                this.timerTop.angle = 180;
                this.timerTop.centerY = this.game.height * 0.345;
                this.add.existing(this.timerTop);
                this.timerTop.setTweenDestination(this.timerTop.width, 0);
                this.timerBottom = new SargeGameTimerPanel_1.SargeGameTimerPanel(this.game);
                this.timerBottom.centerY = this.game.height * 0.68;
                this.add.existing(this.timerBottom);
                this.timerBottom.setTweenDestination(this.game.width - this.timerBottom.width, this.game.width);
                // Generate walkie talkie buttons
                SargeGameState.BUTTON_LOCATIONS.forEach((loc, index) => {
                    const walkieTalkie = new DirectionButton_1.DirectionButtonGroup(this.game, this.directions[index]);
                    walkieTalkie.location = loc;
                    if (loc.x === 0) {
                        walkieTalkie.left = 0 - walkieTalkie.buttonGroupOffset;
                    }
                    else {
                        walkieTalkie.right = this.game.width + walkieTalkie.buttonGroupOffset;
                    }
                    if (loc.y === 0) {
                        walkieTalkie.top = 0;
                    }
                    else {
                        walkieTalkie.bottom = this.game.height;
                    }
                    this.walkieTalkieButtons.push(walkieTalkie);
                    walkieTalkie.directionButton.clickedSignal.add(() => {
                        if (this.gameStarted && this.canMove && (this.currentSargeLocation !== walkieTalkie.directionButton.direction)) {
                            const dest = this.getDestinationByDirection(walkieTalkie.directionButton.direction);
                            if (dest) {
                                this.moveToDestination(dest);
                                walkieTalkie.onClick();
                            }
                        }
                    }, this);
                    this.add.existing(walkieTalkie);
                });
                this.curtain = new Curtain_8.Curtain(this.game, true);
                this.add.existing(this.curtain);
                this.topReadyButton = new TextArrowButton_10.TextArrowButton(this.game, LocalizationManager_33.LocalizationManager.Instance.text("common.ready"));
                this.topReadyButton.angle = -180;
                this.topReadyButton.centerX = this.game.width * 0.5;
                this.topReadyButton.centerY = this.game.height * this.topButtonPos;
                this.topReadyButton.alpha = 0;
                ScaleUtil_32.scaleToBounds(this.topReadyButton, this.game.width * 0.5, this.game.height * 0.15);
                this.topReadyButton.startPulse();
                this.bottomReadyButton = new TextArrowButton_10.TextArrowButton(this.game, LocalizationManager_33.LocalizationManager.Instance.text("common.ready"));
                this.bottomReadyButton.centerX = this.game.width * 0.5;
                this.bottomReadyButton.centerY = this.game.height * this.bottomButtonPos;
                this.bottomReadyButton.alpha = 0;
                ScaleUtil_32.scaleToBounds(this.bottomReadyButton, this.game.width * 0.5, this.game.height * 0.15);
                this.bottomReadyButton.startPulse();
                this.walkieTalkieInfographic = new Phaser.Image(this.game, 0, 0, 'tutorial-walkie-talkie');
                ScaleUtil_32.scaleToBounds(this.walkieTalkieInfographic, this.game.width * 0.8, this.game.height * 0.325);
                this.walkieTalkieInfographic.anchor.set(0.5);
                this.walkieTalkieInfographic.centerX = this.game.width * 0.5;
                this.walkieTalkieInfographic.centerY = this.game.height * this.graphicPos;
                this.walkieTalkieInfographic.visible = false;
                this.add.existing(this.walkieTalkieInfographic);
                this.topMessageBar = new MessageBar_17.MessageBar(this.game, '', 25);
                this.topMessageBar.angle = -180;
                this.topMessageBar.centerX = this.game.width * 0.5;
                this.topMessageBar.bottom = 0;
                this.add.existing(this.topMessageBar);
                this.bottomMessageBar = new MessageBar_17.MessageBar(this.game, '', 25);
                this.bottomMessageBar.centerX = this.game.width * 0.5;
                this.bottomMessageBar.top = this.game.height;
                this.add.existing(this.bottomMessageBar);
                this.armyManInfographic = new Phaser.Image(this.game, 0, 0, "tutorial-sarge");
                this.armyManInfographic.scale.x = this.walkieTalkieInfographic.scale.x * -1;
                this.armyManInfographic.scale.y = this.walkieTalkieInfographic.scale.y;
                this.armyManInfographic.anchor.set(0.5);
                this.armyManInfographic.centerX = this.game.width * 0.5;
                this.armyManInfographic.centerY = this.game.height * 0.5;
                this.armyManInfographic.visible = false;
                this.add.existing(this.armyManInfographic);
                // Add the ready buttons at the end so they're on top
                this.add.existing(this.bottomReadyButton);
                this.add.existing(this.topReadyButton);
                this.animateInfographic();
                yield this.dismissOverlay();
                if (ToyStoryConfig_17.ToyStoryConfig.Instance.skipGames) {
                    this.endGame(10);
                }
                else {
                    this.play();
                }
            });
        }
        playTutorial() {
            return __awaiter(this, void 0, void 0, function* () {
                // Go up, down, left, right-right
                // Drop crates constantly til caught
                yield this.performTutorialStep(this.dropZones.get(DropZoneLocation.UP), [DropZoneLocation.UP]);
                yield this.performTutorialStep(this.dropZones.get(DropZoneLocation.LEFT), [DropZoneLocation.DOWN, DropZoneLocation.LEFT]);
                yield this.performTutorialStep(this.dropZones.get(DropZoneLocation.RIGHT), [DropZoneLocation.RIGHT, DropZoneLocation.RIGHT]);
                this.hasPlayedTutorial = true;
            });
        }
        performTutorialStep(dropZone, buttonLocs) {
            return __awaiter(this, void 0, void 0, function* () {
                // Sequentially show the button prompts required to get to
                // this step's crate drop location
                this.showTutorialButtonPrompts(buttonLocs);
                let isDropping = true;
                const dropCompleteListener = (success) => {
                    // If Sarge successfully caught the dropping crate
                    if (success) {
                        // Stop dropping the crates on that drop zone
                        isDropping = false;
                        dropZone.dropCompleteSignal.remove(dropCompleteListener);
                    }
                };
                dropZone.dropCompleteSignal.add(dropCompleteListener, this);
                // While we're waiting for a successful drop
                // keep displaying the drop animation with a short countdown
                while (isDropping) {
                    yield dropZone.initiateDrop(this.config.tutorialDropDurationS);
                }
            });
        }
        showTutorialButtonPrompts(buttonLocs) {
            return __awaiter(this, void 0, void 0, function* () {
                let button = this.walkieTalkieButtons.find(obj => obj.directionButton.direction === buttonLocs[0]);
                // Show the animating "PRESS" button prompt
                button.showButtonPrompt();
                // Currently all buttons are disabled, as we haven't "started" the game
                // Temporarily add a single listener to the button we want to press
                yield PromiseUtil_57.PromiseUtil.signal(button.directionButton.clickedSignal);
                button.onClick();
                const dest = this.getDestinationByDirection(button.directionButton.direction);
                button.hideButtonPrompt();
                yield this.moveToDestination(dest);
                if (buttonLocs.length > 1) {
                    yield this.showTutorialButtonPrompts(buttonLocs.splice(1));
                }
            });
        }
        play() {
            return __awaiter(this, void 0, void 0, function* () {
                // Game start
                while (this.currentRoundIndex < this.players.length) {
                    if (!this.curtain.isShowing) {
                        yield this.curtain.show(true);
                    }
                    this.flipButtons();
                    yield Promise.all([
                        this.doTopInterstitial(),
                        this.doBottomInterstitial()
                    ]);
                    // If this doesn't affect to playTutorial, it might as well be inside doTurn
                    // but will need to make sure it starts with turnScore set to 0
                    // its important that it is after the interstitial because targets keeps droping in the bg if they were about to drop
                    this.resetForTurn();
                    yield Promise.all([
                        this.dismissBothInterstitials(),
                        this.hideInfographic()
                    ]);
                    yield this.curtain.hide();
                    if (!this.hasPlayedTutorial) {
                        yield this.playTutorial();
                    }
                    this.trackGameStartTime();
                    yield this.doTurn();
                    // Turn over
                    this.currentRoundIndex++;
                }
                this.endGame(this.totalScore);
            });
        }
        endGame(score) {
            const _super = name => super[name];
            return __awaiter(this, void 0, void 0, function* () {
                // Post the score
                GameController_31.GameController.Instance.postScore(this.stateName, score);
                yield GameController_31.GameController.Instance.achievementManager.onSargeEndGame(score);
                yield this.curtain.show();
                // Game over
                _super("transitionTo").call(this, StateNames_35.StateNames.RESULTS);
            });
        }
        doTopInterstitial() {
            return __awaiter(this, void 0, void 0, function* () {
                this.trackGamePassAndPlay();
                this.topMessageBar.fontSize = 90;
                this.topMessageBar.text = LocalizationManager_33.LocalizationManager.Instance.textWithReplacements("sarge.instructions_pass", {
                    player_name: this.players[this.currentRoundIndex][1].name
                }).toUpperCase();
                this.topReadyButton.text = LocalizationManager_33.LocalizationManager.Instance.text("common.ok");
                yield PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                    .to({ top: this.game.height * this.topMessageBarPos }, 350, Phaser.Easing.Sinusoidal.InOut, true));
                yield PromiseUtil_57.PromiseUtil.time(this.game, 1500);
                yield PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.topReadyButton)
                    .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true));
                yield PromiseUtil_57.PromiseUtil.signal(this.topReadyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                yield Promise.all([
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                        .to({ bottom: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.topReadyButton)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true))
                ]);
                /// Instructions
                this.trackGameInstructions();
                this.topMessageBar.fontSize = 60;
                this.topMessageBar.text = LocalizationManager_33.LocalizationManager.Instance.text("sarge.instructions_how_to_play").toUpperCase();
                this.topReadyButton.text = LocalizationManager_33.LocalizationManager.Instance.text("common.ready").toUpperCase();
                yield Promise.all([
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                        .to({ top: this.game.height * this.topMessageBarPos }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.topReadyButton)
                        .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true, 2000)),
                    this.showInfographic()
                ]);
                yield PromiseUtil_57.PromiseUtil.signal(this.topReadyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                this.topMessageBar.fontSize = 90;
                this.topMessageBar.text = LocalizationManager_33.LocalizationManager.Instance.textWithReplacements("sarge.instructions_waiting_for_player", {
                    player_name: this.players[this.currentRoundIndex][0].name
                }).toUpperCase();
                this.game.add.tween(this.topReadyButton)
                    .to({ alpha: 0 }, 350, Phaser.Easing.Sinusoidal.InOut, true);
            });
        }
        doBottomInterstitial() {
            return __awaiter(this, void 0, void 0, function* () {
                this.bottomMessageBar.fontSize = 90;
                this.bottomMessageBar.text = LocalizationManager_33.LocalizationManager.Instance.textWithReplacements("sarge.instructions_pass", {
                    player_name: this.players[this.currentRoundIndex][0].name
                }).toUpperCase();
                this.bottomReadyButton.text = LocalizationManager_33.LocalizationManager.Instance.text("common.ok");
                yield PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                    .to({ bottom: this.game.height * this.bottomMessageBarPos }, 350, Phaser.Easing.Sinusoidal.InOut, true));
                yield PromiseUtil_57.PromiseUtil.time(this.game, 1500);
                yield PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomReadyButton)
                    .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true));
                yield PromiseUtil_57.PromiseUtil.signal(this.bottomReadyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                yield Promise.all([
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                        .to({ top: this.game.height }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomReadyButton)
                        .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true))
                ]);
                this.bottomMessageBar.fontSize = 60;
                this.bottomMessageBar.text = LocalizationManager_33.LocalizationManager.Instance.text("sarge.instructions_how_to_play").toUpperCase();
                this.bottomReadyButton.text = LocalizationManager_33.LocalizationManager.Instance.text("common.ready").toUpperCase();
                yield Promise.all([
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                        .to({ bottom: this.game.height * this.bottomMessageBarPos }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomReadyButton)
                        .to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true, 2000)),
                    this.showInfographic()
                ]);
                yield PromiseUtil_57.PromiseUtil.signal(this.bottomReadyButton.onChildInputDown);
                this.game.sound.play('snd_ui_click');
                this.bottomMessageBar.fontSize = 90;
                this.bottomMessageBar.text = LocalizationManager_33.LocalizationManager.Instance.textWithReplacements("sarge.instructions_waiting_for_player", {
                    player_name: this.players[this.currentRoundIndex][1].name
                }).toUpperCase();
                this.game.add.tween(this.bottomReadyButton)
                    .to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true);
            });
        }
        dismissBothInterstitials() {
            return __awaiter(this, void 0, void 0, function* () {
                yield Promise.all([
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                        .to({ top: this.game.height }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                        .to({ bottom: 0 }, 350, Phaser.Easing.Linear.None, true))
                ]);
            });
        }
        flipButtons() {
            if (this.difficultyConfig.areButtonsRandomized) {
                // assign new directions for each player
                this.directions = [];
                let vDirections = [DropZoneLocation.UP, DropZoneLocation.DOWN];
                let hDirections = [DropZoneLocation.RIGHT, DropZoneLocation.LEFT];
                // player 1 directions (always 1 vertical and 1 horizontal in any permutation)
                if (Math.random() < 0.5) {
                    this.directions.push(this.removeRandomDirection(vDirections));
                    this.directions.push(this.removeRandomDirection(hDirections));
                }
                else {
                    this.directions.push(this.removeRandomDirection(hDirections));
                    this.directions.push(this.removeRandomDirection(vDirections));
                }
                // player 2 directions (randomize the remaining directions)
                if (Math.random() < 0.5) {
                    this.directions.push(vDirections[0]);
                    this.directions.push(hDirections[0]);
                }
                else {
                    this.directions.push(hDirections[0]);
                    this.directions.push(vDirections[0]);
                }
                this.walkieTalkieButtons.forEach((butt, index) => {
                    butt.directionButton.direction = this.directions[index];
                });
            }
        }
        removeRandomDirection(directions) {
            let index = Math.round(Math.random());
            let direction = directions[index];
            directions.splice(index, 1);
            return direction;
        }
        resetForTurn() {
            this.timeRemainingMs = this.config.gameDuration;
            this.currentTimeBetweenDropsMs = this.difficultyConfig.timeBetweenDropsMsMax;
            this.timerTop.updateTime(this.timeRemainingMs);
            this.timerBottom.updateTime(this.timeRemainingMs);
            this.setTurnScore(0);
        }
        setTurnScore(score) {
            this.turnScore = score;
            if (this.scoreTextTop && this.scoreTextBottom) {
                this.scoreTextTop.text = this.scoreTextBottom.text = 'x' + this.turnScore;
            }
        }
        getCurrentSargeLocation() {
            return this.currentSargeLocation;
        }
        getDestinationByDirection(direction) {
            // If we're down, we can only move up
            if (this.currentSargeLocation === DropZoneLocation.DOWN) {
                if (direction === DropZoneLocation.UP) {
                    return DropZoneLocation.MIDDLE;
                }
            }
            else if (this.currentSargeLocation === DropZoneLocation.UP) {
                // If we're up, we can only move down...and so on
                if (direction === DropZoneLocation.DOWN) {
                    return DropZoneLocation.MIDDLE;
                }
            }
            else if (this.currentSargeLocation === DropZoneLocation.LEFT) {
                if (direction === DropZoneLocation.RIGHT) {
                    return DropZoneLocation.MIDDLE;
                }
            }
            else if (this.currentSargeLocation === DropZoneLocation.RIGHT) {
                if (direction === DropZoneLocation.LEFT) {
                    return DropZoneLocation.MIDDLE;
                }
            }
            else if (this.currentSargeLocation === DropZoneLocation.MIDDLE) {
                // If we're in the middle, we can move anywhere!
                return direction;
            }
            return null;
        }
        moveToDestination(destination) {
            return __awaiter(this, void 0, void 0, function* () {
                this.currentTargetDestination = destination;
                const locationPoint = this.getLocationForZone(destination);
                this.canMove = false;
                this.currentSargeLocation = null;
                this.game.sound.play('sarge-move');
                // calculate movement speed based on how far he is travelling
                let distanceTravelled = Math.abs(locationPoint.y - this.sarge.position.y);
                if (distanceTravelled < 1) {
                    distanceTravelled = Math.abs(locationPoint.x - this.sarge.position.x);
                }
                // sarge needs to take 350ms to move the full horizontal distance for the animation to look right
                // scale other movement distances by this baseline
                const leftPos = this.getLocationForZone(DropZoneLocation.LEFT);
                const middlePos = this.getLocationForZone(DropZoneLocation.MIDDLE);
                const walkSpeed = (distanceTravelled / (middlePos.x - leftPos.x)) * 350;
                this.sarge.beginWalking();
                yield PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.sarge)
                    .to({ x: locationPoint.x, y: locationPoint.y }, walkSpeed, Phaser.Easing.Linear.None, true));
                this.sarge.endWalking();
                this.currentSargeLocation = destination;
                const currentDropZone = this.dropZones.get(this.currentSargeLocation);
                if (currentDropZone) {
                    yield currentDropZone.sargeEnterZone();
                }
                this.canMove = true;
                this.currentTargetDestination = null;
            });
        }
        onDropComplete(succeeded) {
            if (succeeded) {
                this.turnScore++;
                this.totalScore++;
            }
            this.currentTimeBetweenDropsMs -= this.difficultyConfig.timeBetweenDropsMsDelta;
            this.currentTimeBetweenDropsMs = Math.max(this.currentTimeBetweenDropsMs, this.difficultyConfig.timeBetweenDropsMsMin);
            this.setTurnScore(Math.max(this.turnScore, 0));
        }
        doTurn() {
            return __awaiter(this, void 0, void 0, function* () {
                this.gameStarted = true;
                this.canMove = true;
                this.timerTop.showTimer();
                this.timerBottom.showTimer();
                // Turn start
                while (this.timeRemainingMs > 0) {
                    yield new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                        this.activeGamePromiseResolution = resolve;
                        // Pick a drop duration
                        const durationOfDropS = this.game.rnd.integerInRange(this.difficultyConfig.durationOfDropsSMin, this.difficultyConfig.durationOfDropsSMax);
                        // Try to find an available zone
                        const candidateDropZone = this.getRandomUnoccupiedDropZone();
                        if (candidateDropZone) {
                            // and perform the drop
                            candidateDropZone.initiateDrop(durationOfDropS);
                        }
                        // Wait a timeout until the next drop
                        yield PromiseUtil_57.PromiseUtil.time(this.game, this.currentTimeBetweenDropsMs);
                        if (this.activeGamePromiseResolution) {
                            this.activeGamePromiseResolution();
                        }
                    }));
                    this.activeGamePromiseResolution = null;
                }
                this.gameStarted = false;
                this.canMove = false;
                this.timerTop.hideTimer();
                this.timerBottom.hideTimer();
                if (this.currentSargeLocation !== DropZoneLocation.MIDDLE) {
                    this.moveToDestination(DropZoneLocation.MIDDLE);
                }
                yield this.showTurnScore();
            });
        }
        showTurnScore() {
            return __awaiter(this, void 0, void 0, function* () {
                this.topMessageBar.text = LocalizationManager_33.LocalizationManager.Instance.textWithCount("sarge.score", this.turnScore).toUpperCase();
                this.bottomMessageBar.text = LocalizationManager_33.LocalizationManager.Instance.textWithCount("sarge.score", this.turnScore).toUpperCase();
                yield Promise.all([
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.topMessageBar)
                        .to({ centerY: this.game.height * 0.25 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.bottomMessageBar)
                        .to({ centerY: this.game.height * 0.75 }, 350, Phaser.Easing.Sinusoidal.InOut, true)),
                    PromiseUtil_57.PromiseUtil.time(this.game, 3000),
                    this.curtain.show(true)
                ]);
            });
        }
        showInfographic() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.armyManInfographic.visible || !this.walkieTalkieInfographic.visible) {
                    this.armyManInfographic.alpha = 0;
                    this.armyManInfographic.visible = true;
                    this.walkieTalkieInfographic.alpha = 0;
                    this.walkieTalkieInfographic.visible = true;
                    yield Promise.all([
                        PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.armyManInfographic).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true)),
                        PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.walkieTalkieInfographic).to({ alpha: 1 }, 350, Phaser.Easing.Linear.None, true))
                    ]);
                }
            });
        }
        hideInfographic() {
            return __awaiter(this, void 0, void 0, function* () {
                yield Promise.all([
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.armyManInfographic).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true)),
                    PromiseUtil_57.PromiseUtil.tween(this.game, this.game.add.tween(this.walkieTalkieInfographic).to({ alpha: 0 }, 350, Phaser.Easing.Linear.None, true))
                ]);
                this.armyManInfographic.visible = false;
                this.walkieTalkieInfographic.visible = false;
            });
        }
        animateInfographic() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!this.game) {
                    return;
                }
                this.armyManInfographic.scale.x *= -1;
                this.walkieTalkieInfographic.scale.y *= -1;
                this.walkieTalkieInfographic.scale.x *= -1;
                yield PromiseUtil_57.PromiseUtil.time(this.game, 500);
                this.animateInfographic();
            });
        }
        update() {
            super.update();
            if (this.gameStarted) {
                this.timeRemainingMs -= this.game.time.elapsedMS;
                this.timerTop.updateTime(this.timeRemainingMs);
                this.timerBottom.updateTime(this.timeRemainingMs);
                if (this.timeRemainingMs < 0) {
                    if (this.activeGamePromiseResolution) {
                        this.activeGamePromiseResolution();
                        this.activeGamePromiseResolution = null;
                    }
                }
            }
        }
        getRandomUnoccupiedDropZone() {
            const dropZones = Array.from(this.dropZones.values());
            // cap the number of active zones
            let activeZoneCount = 0;
            for (let i = 0; i < dropZones.length; i++) {
                if (dropZones[i].active && ++activeZoneCount >= this.difficultyConfig.maxActiveDrops) {
                    return;
                }
            }
            Phaser.ArrayUtils.shuffle(dropZones);
            for (let i = 0; i < dropZones.length; i++) {
                // don't spawn on location Sarge is on or heading to
                if (!dropZones[i].active && this.currentSargeLocation !== dropZones[i].location &&
                    this.currentTargetDestination !== dropZones[i].location) {
                    return dropZones[i];
                }
            }
        }
        getLocationForZone(zone) {
            switch (zone) {
                case DropZoneLocation.UP: {
                    return new Phaser.Point(this.game.width * 0.5, this.game.height * 0.2);
                }
                case DropZoneLocation.DOWN: {
                    return new Phaser.Point(this.game.width * 0.5, this.game.height * 0.8);
                }
                case DropZoneLocation.RIGHT: {
                    return new Phaser.Point(this.game.width * 0.85, this.game.height * 0.45);
                }
                case DropZoneLocation.LEFT: {
                    return new Phaser.Point(this.game.width * 0.15, this.game.height * 0.45);
                }
                case DropZoneLocation.MIDDLE:
                default: {
                    return new Phaser.Point(this.game.width * 0.5, this.game.height * 0.45);
                }
            }
        }
        shutdown() {
            this.zoneLayer = null;
            this.sarge = null;
            this.actionIconLayer = null;
            this.dropZones = null;
            this.scoreTextTop = null;
            this.scoreTextBottom = null;
            this.timerTop = null;
            this.timerBottom = null;
            this.players = null;
            this.activeGamePromiseResolution = null;
            this.currentSargeLocation = null;
            this.currentTargetDestination = null;
            this.curtain = null;
            this.topMessageBar = null;
            this.bottomMessageBar = null;
            this.topReadyButton = null;
            this.bottomReadyButton = null;
            this.walkieTalkieInfographic = null;
            this.armyManInfographic = null;
            this.walkieTalkieButtons = null;
            // unload background assets
            this.game.cache.removeImage('sarge-background');
            // unload button assets
            this.game.cache.removeImage('radio-base');
            this.game.cache.removeImage('radio-btn');
            this.game.cache.removeImage('radio-btn-arrow');
            this.game.cache.removeImage('radio-btn-glow');
            this.game.cache.removeImage('radio-btn-spark');
            // unload tutorial assets
            this.game.cache.removeImage('tutorial-sarge');
            this.game.cache.removeImage('tutorial-walkie-talkie');
            // unload the army man
            this.game.cache.removeImage('army-man');
            // unload the "correct" and "incorrect" images for when the player guesses correctly/incorrectly
            this.game.cache.removeImage('correct-icon');
            this.game.cache.removeImage('incorrect-icon');
            this.game.cache.removeImage('correct-icon-wheel');
            this.game.cache.removeImage('incorrect-icon-wheel');
            this.game.cache.removeImage('correct-icon-background');
            this.game.cache.removeImage('incorrect-icon-background');
            this.game.cache.removeImage('sarge-supply-crate');
            this.game.cache.removeImage('sarge-supply-crate-shadow');
            this.game.cache.removeImage('timer-bar-back');
            this.game.cache.removeImage('timer-bar-front');
            this.game.cache.removeImage("timerbar-icon");
            this.game.cache.removeImage("sarge-timerbox");
            this.game.cache.removeImage("hold-prompt");
            this.game.cache.removeImage("release-prompt");
            this.game.cache.removeImage("prompt-window");
            // unload the sounds
            this.game.cache.removeSound('sarge-move');
            this.game.cache.removeSound('soldiers-move');
            this.game.cache.removeSound('tick-correct');
            this.game.cache.removeSound('tick-incorrect');
            this.game.cache.removeSound('crate-fall-correct');
            this.game.cache.removeSound('crate-fall-incorrect');
            super.shutdown();
        }
    }
    SargeGameState.BUTTON_LOCATIONS = [
        new Phaser.Point(0, 0),
        new Phaser.Point(1, 0),
        new Phaser.Point(1, 1),
        new Phaser.Point(0, 1)
    ];
    exports.SargeGameState = SargeGameState;
});
define("Game", ["require", "exports", "state/LoadingState", "state/OutroState", "minigame/flying-tosser/FlyingTossersState", "minigame/hamm/HammGameState", "minigame/woody/balance/BalanceState", "minigame/woody/lasso/LassoState", "minigame/woody/throw/ThrowState", "minigame/dress-up/DressUpState", "state/CutSceneState", "StateNames", "minigame/Trixie/TrixieState", "state/HubState", "minigame/poster/PosterGameState", "state/MiniGameTitleState", "state/MiniGameResultState", "state/IntroState", "minigame/poster/PosterInstallationState", "minigame/drawing/DrawingState", "minigame/sarge/SargeGameState", "switchback-common/state/BaseState", "switchback-common/util/DeviceUtil"], function (require, exports, LoadingState_1, OutroState_1, FlyingTossersState_3, HammGameState_1, BalanceState_2, LassoState_1, ThrowState_2, DressUpState_1, CutSceneState_9, StateNames_36, TrixieState_1, HubState_1, PosterGameState_2, MiniGameTitleState_1, MiniGameResultState_1, IntroState_1, PosterInstallationState_1, DrawingState_1, SargeGameState_2, BaseState_3, DeviceUtil_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class Game extends Phaser.Game {
        constructor() {
            super('100', '100', Phaser.CANVAS, null, null, true);
            console.log(`[Game] constructor()`);
            if (DeviceUtil_2.DeviceUtil.isMaliGPU()) {
                // Fix for strobing canvas when running Phaser on Mali-400 GPUs.
                this.preserveDrawingBuffer = true;
            }
            Game.TEXT_SHADOW_OFFSET_SMALL_X = this.width * 0.004;
            Game.TEXT_SHADOW_OFFSET_SMALL_Y = this.width * 0.004;
            Game.TEXT_SHADOW_OFFSET_MED_X = this.width * 0.008;
            Game.TEXT_SHADOW_OFFSET_MED_Y = this.width * 0.008;
            Game.TEXT_STROKE_THICKNESS_MED = this.width * 0.004;
            Game.TEXT_STROKE_THICKNESS_LARGE = this.width * 0.008;
            Game.TEXT_STROKE_THICKNESS_EXTRA_LARGE = this.width * 0.013;
            Game.TEXT_STROKE_THICKNESS_HUMONGOUS = this.width * 0.018;
            // Set up a reference to our loadCommonAssets function, which will be used by all classes that extend BaseState
            BaseState_3.BaseState.loadCommonAssets = this.loadCommonAssets.bind(this);
            this.state.add(StateNames_36.StateNames.LOAD, LoadingState_1.LoadingState, false);
            this.state.add(StateNames_36.StateNames.INTRO, IntroState_1.IntroState, false);
            this.state.add(StateNames_36.StateNames.HUB, HubState_1.HubState, false);
            this.state.add(StateNames_36.StateNames.RESULTS, MiniGameResultState_1.MiniGameResultState, false);
            this.state.add(StateNames_36.StateNames.MINIGAME_TITLE, MiniGameTitleState_1.MiniGameTitleState, false);
            this.state.add(StateNames_36.StateNames.GAME_FLYING_TOSSERS, FlyingTossersState_3.FlyingTossersState, false);
            this.state.add(StateNames_36.StateNames.GAME_TRIXIE, TrixieState_1.TrixieState, false);
            this.state.add(StateNames_36.StateNames.GAME_HAMM, HammGameState_1.HammGameState, false);
            this.state.add(StateNames_36.StateNames.GAME_SARGE, SargeGameState_2.SargeGameState, false);
            this.state.add(StateNames_36.StateNames.GAME_WOODY_BALANCE, BalanceState_2.BalanceState, false);
            this.state.add(StateNames_36.StateNames.GAME_WOODY_LASSO, LassoState_1.LassoState, false);
            this.state.add(StateNames_36.StateNames.GAME_WOODY_THROW, ThrowState_2.ThrowState, false);
            this.state.add(StateNames_36.StateNames.GAME_DRESS_UP, DressUpState_1.DressUpState, false);
            this.state.add(StateNames_36.StateNames.GAME_DRAWING, DrawingState_1.DrawingState, false);
            this.state.add(StateNames_36.StateNames.OUTRO, OutroState_1.OutroState, false);
            this.state.add(StateNames_36.StateNames.CUT_SCENE, CutSceneState_9.CutSceneState, false);
            this.state.add(StateNames_36.StateNames.POSTER, PosterGameState_2.PosterGameState, false);
            this.state.add(StateNames_36.StateNames.POSTER_INSTALLATION, PosterInstallationState_1.PosterInstallationState, false);
            this.state.start(StateNames_36.StateNames.LOAD);
        }
        /*
        * This function is called in the preload of every state that extends BaseState. It will load
        * all our common assets into memory (unless they've already been loaded)
        */
        loadCommonAssets() { }
    }
    Game.DEFAULT_FONT = "PaytoneOne";
    Game.CARNIVAL_FONT = "Sancreek-Regular";
    Game.LOBSTER_FONT = "Lobster-Regular";
    Game.ARCADE_FONT = "Arcade";
    Game.FONT_SIZE_EXTRA_LARGE = 150;
    Game.FONT_SIZE_LARGE = 100;
    Game.FONT_SIZE_MEDIUM = 50;
    Game.FONT_SIZE_SMALL = 30;
    Game.TEXT_COLOR_WHITE = "#FFFFFF";
    Game.TEXT_COLOR_DISABLED = "#cb4b58";
    Game.TEXT_COLOR_YELLOW = "#F8B000";
    Game.TEXT_COLOR_RED = "#f73b39";
    Game.TEXT_COLOR_PINK = "#AE1A3C";
    Game.TEXT_COLOR_CREAM = "#FCF0BA";
    Game.TEXT_COLOR_BLUE = '#394db5';
    Game.TEXT_COLOR_CHAR_FILL = '#e7e0ce';
    Game.TEXT_COLOR_CHAR_STROKE = '#811e23';
    Game.TEXT_COLOR_STROKE_DISABLED = '#1A0626';
    Game.TEXT_SHADOW_SOFT = "rgba(0, 0, 0, 0.2)";
    Game.TEXT_SHADOW_HARD = "rgba(0, 0, 0, 0.5)";
    Game.TEXT_SHADOW_DIAG = 'rgba(17, 91, 166, 1)';
    exports.Game = Game;
});
/// <reference path="../node_modules/phaser/typescript/phaser.comments.d.ts" />
/// <reference path="./switchback-common/localization/typing/polyglot.d.ts" />
/// <reference path="../src/switchback-common/extension/skew.d.ts" />
define("index", ["require", "exports", "Game"], function (require, exports, Game_36) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    new Game_36.Game();
});
define("component/ProgressBar", ["require", "exports", "switchback-common/util/ScaleUtil"], function (require, exports, ScaleUtil_33) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ProgressBar extends Phaser.Group {
        constructor(game, progress = 0) {
            super(game);
            this._progress = progress;
            const border = new Phaser.Image(game, 0, 0, 'progress-bar');
            ScaleUtil_33.scaleToBounds(border, this.game.width * 0.3, this.game.height * 0.1);
            this.fullBarWidth = border.width * 0.93;
            this.bar = new Phaser.Image(game, 0, 0, 'progress-bar-fill');
            this.bar.anchor.set(0, 0.5);
            this.bar.height = border.height * 0.75;
            this.bar.width = this.fullBarWidth * this._progress;
            this.bar.centerX = border.centerX - (this.fullBarWidth * 0.5);
            this.bar.centerY = border.centerY;
            this.add(this.bar);
            this.add(border);
        }
        setProgress(progress) {
            return __awaiter(this, void 0, void 0, function* () {
                const newProgress = Phaser.Math.clamp(progress, 0, 1);
                this.bar.width = this.fullBarWidth * this._progress;
                this._progress = newProgress;
            });
        }
    }
    exports.ProgressBar = ProgressBar;
});
define("component/OrientationHoldBar", ["require", "exports", "component/ProgressBar"], function (require, exports, ProgressBar_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class OrientationHoldBar extends ProgressBar_1.ProgressBar {
        constructor(game) {
            super(game);
            this.alignmentComplete = new Phaser.Signal();
            this.shouldProgress = false;
            this.isActive = false;
            this.progress = 0;
            this.motionCallback = (result) => {
                this.onMotionChanged(result);
            };
            PlayAPI.subscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.motionCallback);
        }
        update() {
            super.update();
            if (!this.isActive) {
                return;
            }
            if (this.shouldProgress) {
                if (this.progress <= 1) {
                    this.progress += this.game.time.elapsedMS / OrientationHoldBar.PROGRESS_BAR_INCREASE_TIME_MS;
                    this.setProgress(this.progress);
                    if (this.progress > 1) {
                        this.shouldProgress = false;
                        this.alignmentComplete.dispatch();
                    }
                }
            }
            else {
                if (this.progress > 0) {
                    this.progress -= this.game.time.elapsedMS / OrientationHoldBar.PROGRESS_BAR_DECREASE_TIME_MS;
                    this.setProgress(this.progress);
                }
            }
        }
        start() {
            this.isActive = true;
            this.shouldProgress = true;
        }
        stop() {
            this.isActive = false;
            this.shouldProgress = false;
            this.progress = 0;
            this.setProgress(0);
        }
        onMotionChanged(result) {
            let roll = result.attitude.roll;
            // Amplify the value
            roll *= OrientationHoldBar.DEVICE_MOTION_COEFFICIENT;
            // Normalise for forehead orientation
            roll -= 100;
            // Ignore values smaller than GYRO_MINIMUM_THRESHOLD
            if (Math.abs(roll) < OrientationHoldBar.GYRO_MINIMUM_THRESHOLD) {
                roll = 0;
            }
            const isVerticalEnough = roll === 0;
            this.shouldProgress = isVerticalEnough;
        }
        destroy() {
            PlayAPI.unsubscribeToEvent(PlayAPI.gameEvents.DEVICE_ON_MOTION_CHANGED, this.motionCallback);
            this.motionCallback = null;
            super.destroy();
        }
    }
    OrientationHoldBar.PROGRESS_BAR_INCREASE_TIME_MS = 4000;
    OrientationHoldBar.PROGRESS_BAR_DECREASE_TIME_MS = 10000;
    OrientationHoldBar.GYRO_MINIMUM_THRESHOLD = 15; // Degrees either side of vertical
    OrientationHoldBar.DEVICE_MOTION_COEFFICIENT = 65; // gets us close to a 360d range
    exports.OrientationHoldBar = OrientationHoldBar;
});
define("switchback-common/component/RadialProgressBar", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_58) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RadialProgressBar extends Phaser.Graphics {
        constructor(game, x, y, color = 0xFFFFFF, lineThicknessPx = 32, radiusPx = 100, startAngleOffset = 0) {
            super(game, x, y);
            this.angleMin = 0;
            this.angleMax = 0;
            this.color = color;
            this.lineThicknessPx = lineThicknessPx;
            this.radiusPx = radiusPx;
            this.angleMin = this.angleMax = startAngleOffset;
        }
        loop(timeToLoopInMs) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.tweens.removeFrom(this);
                yield PromiseUtil_58.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ angleMax: this.angleMin + 360 }, timeToLoopInMs, Phaser.Easing.Linear.None, true, 0));
            });
        }
        update() {
            super.update();
            this.clear();
            this.lineStyle(this.lineThicknessPx, this.color);
            this.arc(0, 0, this.radiusPx, Phaser.Math.degToRad(this.angleMin), Phaser.Math.degToRad(this.angleMax), false);
            this.endFill();
        }
    }
    exports.RadialProgressBar = RadialProgressBar;
});
define("switchback-common/component/RadialCountdownBar", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/component/RadialProgressBar"], function (require, exports, PromiseUtil_59, RadialProgressBar_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RadialCountdownBar extends RadialProgressBar_1.RadialProgressBar {
        constructor(game, x, y, color = 0xFFFFFF, lineThicknessPx = 32, radiusPx = 100) {
            super(game, x, y, color, lineThicknessPx, radiusPx);
            // Start full
            this.angleMax = 360;
        }
        loop(timeToLoopInMs) {
            return __awaiter(this, void 0, void 0, function* () {
                this.game.tweens.removeFrom(this);
                yield PromiseUtil_59.PromiseUtil.tween(this.game, this.game.add.tween(this).to({ angleMax: 0 }, timeToLoopInMs, Phaser.Easing.Linear.None, true, 0));
            });
        }
    }
    exports.RadialCountdownBar = RadialCountdownBar;
});
define("switchback-common/component/button/Button", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     *
     * This class adds an optional text label to the standard Phaser.Button.
     * The Text and TextStyle can be set on construction, or later.
     */
    class Button extends Phaser.Button {
        constructor(game, x, y, key, callback, callbackContext, text, style) {
            super(game, x, y, key, callback, callbackContext, "over", "out", "down", "out");
            this.textStyle = {};
            this.textStyle = style || {};
            if (text !== null && text !== "") {
                this.text = text;
            }
        }
        get text() {
            return this.textField.text;
        }
        set text(text) {
            if (!this.textField) {
                this.textField = new Phaser.Text(this.game, (this.width * 0.5), (this.height * 0.5), text, this.textStyle);
                this.textField.anchor.set(0.5);
                this.addChild(this.textField);
            }
            else {
                this.textField.text = text;
            }
        }
        get style() {
            return this.textStyle;
        }
        set style(style) {
            this.textStyle = style;
            if (this.textField) {
                this.textField.setStyle(this.textStyle, true);
            }
        }
        // This ensures that the text remains centered within the button when the button's anchor changes
        setAnchor(x, y) {
            // This allows developers to only pass in one argument if they want them both to be the same (as Phaser does)
            y = y || x;
            // Set the button's anchor as normal
            this.anchor.set(x, y);
            // Reposition the textField to remain centered in the button
            if (this.textField) {
                this.textField.position.set((this.width * 0.5) - this.offsetX, (this.height * 0.5) - this.offsetY);
            }
        }
    }
    exports.Button = Button;
});
define("switchback-common/component/button/DisablingButton", ["require", "exports", "switchback-common/component/button/Button"], function (require, exports, Button_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     *
     * A disableable Phaser.Button with default button frame names
     */
    class DisablingButton extends Button_1.Button {
        constructor(game, x, y, key, callback, callbackContext, overFrame, outFrame, downFrame, upFrame, disabledFrame, disabled) {
            super(game, x, y, key, callback, callbackContext);
            this.dBool = false;
            this.originalOverFrame = overFrame || "over";
            this.originalOutFrame = outFrame || "out";
            this.originalDownFrame = downFrame || "down";
            this.originalUpFrame = upFrame || "out";
            this.disabledFrame = disabledFrame || "disabled";
            this.dBool = disabled;
        }
        get disabled() {
            return this.dBool;
        }
        set disabled(val) {
            this.dBool = val;
            this.updateState();
        }
        changeFrames(overFrame, outFrame, downFrame, upFrame, disabledFrame) {
            this.originalOverFrame = overFrame;
            this.originalOutFrame = outFrame;
            this.originalDownFrame = downFrame;
            this.originalUpFrame = upFrame;
            this.disabledFrame = disabledFrame;
            this.updateState();
        }
        updateState() {
            // Hack to work around issue where phaser does not correctly update the input handlers in constructors
            this.game.time.events.add(1, () => {
                this.inputEnabled = !this.dBool;
            });
            if (this.dBool) {
                super.setFrames(this.disabledFrame, this.disabledFrame, this.disabledFrame, this.disabledFrame);
            }
            else {
                super.setFrames(this.originalOverFrame, this.originalOutFrame, this.originalDownFrame, this.originalUpFrame);
            }
        }
    }
    exports.DisablingButton = DisablingButton;
});
define("switchback-common/component/button/GenericButton", ["require", "exports", "switchback-common/util/PromiseUtil", "switchback-common/util/TextUtil"], function (require, exports, PromiseUtil_60, TextUtil_25) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class GenericButton extends Phaser.Group {
        constructor(game, config) {
            super(game);
            this.onClickedSignal = new Phaser.Signal();
            this.enabled = true;
            this._cooldownMS = 0;
            this._buttonEffectInProgress = false;
            if (!config.backgroundKey) {
                console.error("[Generic Button] 'backgroundKey' will soon be a required parameter.");
            }
            this._shutdownSignal = config.shutdownSignal;
            this._cooldownMS = config.waitTime ? config.waitTime : 0;
            this._upTint = config.upTint ? config.upTint : 0xffffff;
            this._downTint = config.downTint ? config.downTint : 0xcccccc;
            this.pivot.set(0.5, 0.5);
            this.image = new Phaser.Image(this.game, 0, 0, config.backgroundKey || "common-button");
            this.image.inputEnabled = true;
            this.add(this.image);
            const buttonText = new Phaser.Text(this.game, 0, 0, config.text, config.fontStyle || {
                fill: 0xff0000,
                fontSize: 72,
                wordWrap: true,
                wordWrapWidth: this.image.width * 0.8,
            });
            buttonText.anchor.set(0.5);
            buttonText.scale.y = buttonText.scale.x;
            buttonText.position.set(this.image.width / 2, this.image.height / 2);
            buttonText.centerY -= this.height * 0.033;
            this.text = buttonText;
            this.add(buttonText);
            if (config.onClickSoundKey) {
                this._clickSound = this.game.add.sound(config.onClickSoundKey, 1, false);
            }
            this.image.events.onInputUp.add(this.onClick, this);
            this.recenterText();
        }
        static preload(loader, url) {
            console.warn("[Generic Button] static preload is now deprecated and will be removed in future");
            loader.image("common-button", url);
        }
        static unload(cache) {
            console.warn("[Generic Button] static unload is now deprecated and will be removed in future");
            cache.removeImage("common-button");
        }
        destroy() {
            this.image.events.onInputUp.removeAll();
            this.onClickedSignal.dispose();
            super.destroy();
            this._shutdownSignal = null;
        }
        delayedFadeIn(delay) {
            return __awaiter(this, void 0, void 0, function* () {
                this.alpha = 0;
                this.enabled = false; // makes sure that once the button is visible it still cant be clicked
                this.visible = true;
                yield PromiseUtil_60.PromiseUtil.tween2(this.game.add.tween(this).to({ alpha: 1 }, 250, Phaser.Easing.Cubic.InOut, true, delay), this._shutdownSignal).catch(() => undefined);
                this.enabled = true;
            });
        }
        recenterText() {
            this.text.centerX = this.image.centerX;
            this.text.centerY = this.image.centerY - this.image.height * 0.033;
            TextUtil_25.TextUtil.scaleDownTextToHeight(this.text, this.image.height * 0.9);
        }
        fadeOut() {
            return __awaiter(this, void 0, void 0, function* () {
                this.alpha = 1;
                let isShutdown = false;
                yield PromiseUtil_60.PromiseUtil.tween2(this.game.add.tween(this).to({ alpha: 0 }, 200, Phaser.Easing.Cubic.InOut, true), this._shutdownSignal).catch(() => { isShutdown = true; });
                if (isShutdown) {
                    return;
                }
                this.visible = false;
                this.enabled = false;
            });
        }
        onClick() {
            return __awaiter(this, void 0, void 0, function* () {
                if (this._buttonEffectInProgress || !this.enabled || this.onClickedSignal.getNumListeners() <= 0) {
                    return;
                }
                if (this._clickSound) {
                    this._clickSound.play();
                }
                if (this.text) {
                    this.text.tint = this._downTint;
                }
                this.image.tint = this._downTint;
                this._buttonEffectInProgress = true;
                // made new function in case need for tweens
                this.onClickedSignal.dispatch();
                let isShutdown = false;
                yield PromiseUtil_60.PromiseUtil.time(this.game, this._cooldownMS, this._shutdownSignal).catch(() => { isShutdown = true; });
                if (isShutdown) {
                    return;
                }
                this._buttonEffectInProgress = false;
                this.image.tint = this._upTint;
                if (this.text) {
                    this.text.tint = this._upTint;
                }
            });
        }
    }
    exports.GenericButton = GenericButton;
});
define("switchback-common/component/button/ToggleButton", ["require", "exports", "switchback-common/component/button/DisablingButton"], function (require, exports, DisablingButton_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     *
     * A Phaser.Button with 2 toggle states. Can be disabled.
     */
    class ToggleButton extends DisablingButton_1.DisablingButton {
        constructor(game, x, y, key, callback, callbackContext, upFrame, downFrame, overFrame, disabledFrame, toggledUpFrame, toggledDownFrame, toggledOverFrame, toggledDisabledFrame) {
            super(game, x, y, key, callback, callbackContext, overFrame, upFrame, downFrame, upFrame);
            this.overFrame = overFrame || "over";
            this.upFrame = upFrame || "out";
            this.downFrame = downFrame || "down";
            this.normalDisabledFrame = disabledFrame || "disabled";
            this.toggledUpFrame = toggledUpFrame || "toggleout";
            this.toggledDownFrame = toggledDownFrame || "toggledown";
            this.toggledOverFrame = toggledOverFrame || "toggleover";
            this.toggledDisabledFrame = toggledDisabledFrame || "toggledisabled";
            this.t = false;
            this.onInputDown.add(() => {
                this.toggled = !this.t;
            });
        }
        get toggled() {
            return this.t;
        }
        set toggled(value) {
            this.t = value;
            if (this.t) {
                this.changeFrames(this.toggledOverFrame, this.toggledUpFrame, this.toggledDownFrame, this.toggledUpFrame, this.toggledDisabledFrame);
            }
            else {
                this.changeFrames(this.overFrame, this.upFrame, this.downFrame, this.upFrame, this.normalDisabledFrame);
            }
        }
        set disabled(val) {
            this.toggled = this.t;
            super.disabled = val;
        }
    }
    exports.ToggleButton = ToggleButton;
});
define("switchback-common/component/keyboard/SpecialChars", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var SpecialChar;
    (function (SpecialChar) {
        SpecialChar["DELETE"] = "";
        SpecialChar["SUBMIT"] = "OK";
        SpecialChar["CANCEL"] = "CANCEL";
    })(SpecialChar = exports.SpecialChar || (exports.SpecialChar = {}));
});
define("switchback-common/component/keyboard/KeyCode", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("switchback-common/component/keyboard/Key", ["require", "exports", "switchback-common/util/TextUtil", "switchback-common/component/keyboard/SpecialChars"], function (require, exports, TextUtil_26, SpecialChars_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class Key extends Phaser.Group {
        constructor(game, keyCode, keyTexture, width, height, fontStyle) {
            super(game);
            this.pressedSignal = new Phaser.Signal();
            const keyBg = new Phaser.Button(this.game, 0, 0, keyTexture, () => { this.pressedSignal.dispatch(keyCode); }, this, 0, 0, 1, 0);
            keyBg.width = width * Key.keyWidthMultiplier(keyCode);
            keyBg.height = height;
            this.add(keyBg);
            this.keyChar = new Phaser.Text(this.game, keyBg.centerX, keyBg.centerY, keyCode, fontStyle);
            this.keyChar.anchor.set(0.5, 0.45);
            TextUtil_26.TextUtil.scaleDownSingleLineTextToWidth(this.keyChar, keyBg.width * 0.95);
            this.add(this.keyChar);
        }
        static keyWidthMultiplier(char) {
            switch (char) {
                case SpecialChars_1.SpecialChar.DELETE:
                    return 1.56;
                case SpecialChars_1.SpecialChar.CANCEL:
                    return 1.83;
                case SpecialChars_1.SpecialChar.SUBMIT:
                    return 3.125;
                default:
                    return 1;
            }
        }
        get keyFontSize() {
            return this.keyChar.fontSize;
        }
        set keyFontSize(size) {
            this.keyChar.fontSize = size;
        }
    }
    exports.Key = Key;
});
define("switchback-common/component/keyboard/KeyLayoutData", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class KeyLayoutData {
    }
    exports.KeyLayoutData = KeyLayoutData;
});
define("switchback-common/component/keyboard/KeyLayouts", ["require", "exports", "switchback-common/component/keyboard/SpecialChars"], function (require, exports, SpecialChars_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class KeyLayouts {
    }
    KeyLayouts.LETTERS = [
        ["Q", "W", "E", "R", "T", "Y", "U", "I", "O", "P", SpecialChars_2.SpecialChar.DELETE],
        ["A", "S", "D", "F", "G", "H", "J", "K", "L", SpecialChars_2.SpecialChar.CANCEL],
        ["Z", "X", "C", "V", "B", "N", "M", SpecialChars_2.SpecialChar.SUBMIT]
    ];
    KeyLayouts.NUMBERS = [
        ["7", "8", "9"],
        ["4", "5", "6"],
        ["1", "2", "3"],
        [SpecialChars_2.SpecialChar.DELETE, "0", SpecialChars_2.SpecialChar.SUBMIT]
    ];
    exports.KeyLayouts = KeyLayouts;
});
define("switchback-common/component/keyboard/KeyboardOptions", ["require", "exports", "switchback-common/component/keyboard/KeyLayouts"], function (require, exports, KeyLayouts_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class KeyboardOptions {
        constructor(params) {
            params = params || {};
            this.bgTexture = params.bgTexture || "kb-bg";
            this.cancelTexture = params.cancelTexture || "kb-cancel";
            this.charLimit = params.charLimit || 13;
            this.deleteTexture = params.deleteTexture || "kb-delete";
            this.keys = params.keys || KeyLayouts_1.KeyLayouts.LETTERS;
            this.keysTexture = params.keysTexture || "kb-regular";
            this.labelText = params.labelText || "";
            this.output = (params.output === undefined) ? true : params.output;
            this.outputBgTexture = params.outputBgTexture || "kb-output";
            this.submitTexture = params.submitTexture || "kb-submit";
            this.xPaddingProportion = (params.xPaddingProportion === undefined) ? 0.15 : params.xPaddingProportion;
            this.keysStyle = params.keyStyle || { fill: "#FFF", fontSize: 44 };
            this.labelStyle = params.labelStyle || { fill: "#FFF", fontSize: 44 };
            this.outputStyle = params.outputStyle || { fill: "#FFF", fontSize: 44 };
        }
    }
    exports.KeyboardOptions = KeyboardOptions;
});
define("switchback-common/component/keyboard/KeyboardController", ["require", "exports", "switchback-common/component/Signal", "switchback-common/component/keyboard/SpecialChars"], function (require, exports, Signal_2, SpecialChars_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class KeyboardController {
        constructor(options) {
            this._currentString = "";
            this.stringChangedSignal = new Signal_2.Signal();
            this.submitSignal = new Signal_2.Signal();
            this.cancelSignal = new Signal_2.Signal();
            this._options = options;
        }
        get options() {
            return this._options;
        }
        getTextureForKey(keyCode) {
            switch (keyCode) {
                case SpecialChars_3.SpecialChar.SUBMIT: {
                    return this.options.submitTexture;
                }
                case SpecialChars_3.SpecialChar.DELETE: {
                    return this.options.deleteTexture;
                }
                case SpecialChars_3.SpecialChar.CANCEL: {
                    return this.options.cancelTexture;
                }
                default: {
                    return this.options.keysTexture;
                }
            }
        }
        keyPressed(keyCode) {
            switch (keyCode) {
                case SpecialChars_3.SpecialChar.DELETE: {
                    if (this._currentString.length > 0) {
                        this._currentString = this._currentString.substr(0, this._currentString.length - 1);
                        this.stringChangedSignal.dispatch(this._currentString);
                    }
                    break;
                }
                case SpecialChars_3.SpecialChar.SUBMIT: {
                    this.submitSignal.dispatch(this._currentString);
                    break;
                }
                case SpecialChars_3.SpecialChar.CANCEL: {
                    this.cancelSignal.dispatch();
                    break;
                }
                default: {
                    if (this._currentString.length < this._options.charLimit) {
                        this._currentString += keyCode;
                        this.stringChangedSignal.dispatch(this._currentString);
                    }
                }
            }
        }
        get currentString() {
            return this._currentString;
        }
        clearString() {
            this._currentString = "";
        }
        destroy() {
            this.stringChangedSignal.dispose();
            this.submitSignal.dispose();
            this.cancelSignal.dispose();
        }
    }
    exports.KeyboardController = KeyboardController;
});
define("switchback-common/component/keyboard/Keyboard", ["require", "exports", "switchback-common/component/keyboard/KeyboardController", "switchback-common/component/keyboard/Key", "switchback-common/util/ScaleUtil"], function (require, exports, KeyboardController_1, Key_1, ScaleUtil_34) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author joel@cerebralfix.com
     */
    class Keyboard extends Phaser.Group {
        get onSubmit() { return this.controller.submitSignal; }
        get onCancel() { return this.controller.cancelSignal; }
        get onChange() { return this.controller.stringChangedSignal; }
        constructor(game, bounds, options) {
            super(game);
            this.keysStyle = options.keysStyle;
            this.bounds = bounds;
            this.controller = new KeyboardController_1.KeyboardController(options);
            const label = new Phaser.Text(this.game, this.bounds.centerX, this.bounds.height * -0.3, options.labelText, options.labelStyle);
            label.anchor.set(0.5);
            this.add(label);
            this.keys = new Phaser.Group(this.game);
            this.add(this.keys);
            let keyBounds = new Phaser.Rectangle(0, this.bounds.height * 0.1, this.bounds.width, this.bounds.height);
            if (options.output) {
                const outputBG = new Phaser.Image(game, 0, 0, options.outputBgTexture);
                ScaleUtil_34.scaleToSingleBound(outputBG, bounds.width * 0.67);
                outputBG.centerX = this.bounds.width * 0.5;
                outputBG.centerY = this.bounds.height * -0.075;
                this.add(outputBG);
                this.outputString = new Phaser.Text(this.game, outputBG.centerX, outputBG.top + outputBG.height * 0.55, "", options.outputStyle);
                this.outputString.anchor.set(0.5);
                this.add(this.outputString);
                this.controller.stringChangedSignal.add(this.onStringChanged, this);
                keyBounds = new Phaser.Rectangle(0, this.bounds.height * 0.2, this.bounds.width, this.bounds.height * 0.75);
            }
            this.showKeys(this.controller.options.keys, keyBounds);
            this.bg = new Phaser.Image(game, 0, 0, options.bgTexture);
            this.bg.width = bounds.width;
            this.bg.height = bounds.height * 1.2;
            if (options.output) {
                this.bg.height = bounds.height * 1.5;
            }
            this.addAt(this.bg, 0);
        }
        destroy() {
            this.controller.destroy();
            super.destroy();
        }
        get currentString() {
            return this.controller.currentString;
        }
        clearString() {
            this.controller.clearString();
            this.outputString.text = this.controller.currentString;
        }
        showKeys(keys, keyBounds) {
            const layout = this.calculateKeyLayout(keys, keyBounds);
            let xPos;
            let yPos = keyBounds.y;
            keys.forEach((row) => {
                let rowWidthMulti = 0;
                for (let i = 0; i < row.length; i++) {
                    rowWidthMulti += Key_1.Key.keyWidthMultiplier(row[i]);
                }
                const totalKeyWidth = rowWidthMulti * layout.width;
                const totalSpaceWidth = (row.length - 1) * layout.xSpacing;
                const xOffset = (keyBounds.width - totalKeyWidth - totalSpaceWidth) * 0.5;
                xPos = xOffset + keyBounds.x + (layout.xSpacing * 0.5);
                row.forEach((key) => {
                    const keySprite = new Key_1.Key(this.game, key, this.controller.getTextureForKey(key), layout.width, layout.height, this.keysStyle);
                    keySprite.position.set(xPos, yPos);
                    this.keys.add(keySprite);
                    keySprite.pressedSignal.add((k) => {
                        this.controller.keyPressed(k);
                    });
                    xPos += layout.width + layout.xSpacing;
                }, this);
                yPos += layout.height + layout.ySpacing;
            }, this);
            /**
             * Now that we've added all the keys, we need to scale their
             * text down uniformly so that they fit on the key.
             */
            let minSize = Infinity;
            this.keys.children.forEach((child) => {
                if (child.keyFontSize) {
                    minSize = Math.min(child.keyFontSize, minSize);
                }
            });
            this.keys.children.forEach((child) => {
                child.keyFontSize = minSize;
            });
        }
        calculateKeyLayout(keys, keyBounds) {
            let longestRow = 0;
            keys.forEach((row) => {
                longestRow = Math.max(row.length, longestRow);
            });
            longestRow++; // Give us some extra room for special characters
            const width = (keyBounds.width * (1 - this.controller.options.xPaddingProportion)) / longestRow;
            const xSpacing = (keyBounds.width * this.controller.options.xPaddingProportion) / longestRow;
            const height = (keyBounds.height * 0.9) / keys.length;
            const ySpacing = (keyBounds.height * 0.15) / keys.length;
            return {
                width: width,
                height: height,
                xSpacing: xSpacing,
                ySpacing: ySpacing,
            };
        }
        onStringChanged(newString) {
            this.outputString.text = newString;
        }
    }
    exports.Keyboard = Keyboard;
});
define("switchback-common/component/scrolling/IScrollableGroupSettings", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
});
define("switchback-common/component/scrolling/ScrollableGroup", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ScrollableGroup extends Phaser.Group {
        constructor(game, viewableWidth, viewableHeight, settings = {}, parent = null) {
            super(game, parent, null, false);
            this.scrollerTapSignal = new Phaser.Signal();
            this._settings = {
                horizontalScroll: false,
                paddingBottom: 0,
                paddingRight: 0,
                timeConstantScroll: 325,
                verticalScroll: true,
            };
            this._hasAddedListeners = false;
            this.settings = settings;
            this._previousPosition = new Phaser.Point();
            this._velocity = new Phaser.Point();
            this._autoScrollTarget = new Phaser.Point();
            this._autoScrollAmplitude = new Phaser.Point();
            this._contents = new Phaser.Group(this.game, null, null, false);
            this._displayMask = new Phaser.Graphics(this.game);
            this._displayMask.beginFill(0x000000);
            this._displayMask.drawRect(0, 0, viewableWidth, viewableHeight);
            this._displayMask.alpha = 0;
            this._displayMask.endFill();
            this._inputId = -1;
            this.add(this._displayMask);
            this.add(this._contents);
            // Hijack the group's add function to ensure all game objects are
            // added to the content layer that gets scrolled
            this.add = this.addToContentLayer.bind(this);
        }
        static addTapInputSignal(game, obj, clickSignal) {
            const tapDistance = game.width * 0.05;
            let startClickPosition = null;
            let endClickPosition = null;
            obj.inputEnabled = true;
            obj.events.onInputDown.add(() => {
                startClickPosition = new Phaser.Point(game.input.x, game.input.y);
            }, this);
            obj.events.onInputUp.add(() => {
                endClickPosition = new Phaser.Point(game.input.x, game.input.y);
                const distance = Phaser.Point.distance(startClickPosition, endClickPosition, true);
                if (distance < tapDistance) {
                    clickSignal.dispatch();
                }
            }, this);
        }
        set settings(settings) {
            for (const property in settings) {
                if (this._settings.hasOwnProperty(property)) {
                    this._settings[property] = settings[property];
                }
            }
        }
        onContentsUpdated() {
            this._contentLocalBounds = this._contents.getLocalBounds();
            this.updateListeners();
        }
        start() {
            ScrollableGroup.addTapInputSignal(this.game, this._displayMask, this.scrollerTapSignal);
            this._contentLocalBounds = this._contents.getLocalBounds();
            this.updateListeners();
            this._contents.position.set(0, 0);
        }
        updateListeners() {
            const isHorizontalScrollable = this._settings.horizontalScroll &&
                this._contentLocalBounds.x + this._contentLocalBounds.width > this._displayMask.width;
            const isVerticalScrollable = this._settings.verticalScroll &&
                this._contentLocalBounds.y + this._contentLocalBounds.height > this._displayMask.height;
            // If it should be scrollable (in either direction), but has no listeners - add 'em all
            if (isHorizontalScrollable || isVerticalScrollable && !this._hasAddedListeners) {
                this.addListeners();
            }
            else if (!isHorizontalScrollable && !isVerticalScrollable && this._hasAddedListeners) {
                // Otherwise if it shouldn't be scrollable (in both directions) but has listeners - remove them!
                this.removeListeners();
            }
        }
        // TODO: Find a way to smooth this. Tween doesn't work 
        // when content is undersized due to bounds check
        moveToBottom() {
            this._contents.y = -(this._contents.height - this._displayMask.height);
        }
        scrollToY(y) {
            this._contents.y = -y;
        }
        stop() {
            this.removeListeners();
            this.reset();
        }
        update() {
            super.update();
            if (this._inputId >= 0) {
                return;
            }
            const elapsed = Date.now() - this._timestamp;
            if (this._autoScrollAmplitude.x !== 0) {
                const delta = -this._autoScrollAmplitude.x * Math.exp(-elapsed / this._settings.timeConstantScroll);
                if (delta > 0.5 || delta < -0.5) {
                    this._contents.x = this._autoScrollTarget.x + delta;
                }
                else {
                    this._autoScrollAmplitude.x = 0;
                }
            }
            if (this._autoScrollAmplitude.y !== 0) {
                const delta = -this._autoScrollAmplitude.y * Math.exp(-elapsed / this._settings.timeConstantScroll);
                if (delta > 0.5 || delta < -0.5) {
                    this._contents.y = this._autoScrollTarget.y + delta;
                }
                else {
                    this._autoScrollAmplitude.y = 0;
                }
            }
            this.limitMovement();
        }
        destroy() {
            this.removeListeners();
            this.scrollerTapSignal.dispose();
            super.destroy();
        }
        reset() {
            this._inputId = -1;
            this._timestamp = Date.now();
            this._velocity.set(0, 0);
            this._autoScrollTarget.set(0, 0);
            this._autoScrollAmplitude.set(0, 0);
        }
        addListeners() {
            if (!this._hasAddedListeners) {
                this.game.input.onDown.add(this.beginMove, this);
                this.game.input.addMoveCallback(this.moveCanvas, this);
                this.game.input.onUp.add(this.endMove, this);
                this._displayMask.inputEnabled = true;
                this._hasAddedListeners = true;
            }
        }
        removeListeners() {
            if (this._hasAddedListeners) {
                this.game.input.onDown.remove(this.beginMove, this);
                this.game.input.deleteMoveCallback(this.moveCanvas, this);
                this.game.input.onUp.remove(this.endMove, this);
                this._displayMask.inputEnabled = false;
                this._hasAddedListeners = false;
            }
        }
        beginMove(pointer) {
            if (this._inputId < 0 && this._displayMask.getBounds().contains(pointer.x, pointer.y)) {
                this.reset();
                this._inputId = pointer.id;
                this._previousPosition.set(pointer.x, pointer.y);
            }
        }
        endMove(pointer) {
            if (this._inputId === pointer.id) {
                if (this.game.input.activePointer.withinGame) {
                    const elapsed = Date.now() - this._timestamp;
                    if (elapsed <= 500) {
                        if (this._velocity.x > 20 || this._velocity.x < -20) {
                            this._autoScrollAmplitude.x = this._velocity.x;
                            this._autoScrollTarget.x = Math.round(this._contents.x + this._autoScrollAmplitude.x);
                        }
                        if (this._velocity.y > 20 || this._velocity.y < -20) {
                            this._autoScrollAmplitude.y = this._velocity.y;
                            this._autoScrollTarget.y = Math.round(this._contents.y + this._autoScrollAmplitude.y);
                        }
                    }
                }
                this._inputId = -1;
            }
        }
        moveCanvas(pointer, x, y) {
            if (this._inputId !== pointer.id) {
                return;
            }
            const now = Date.now();
            const elapsed = now - this._timestamp;
            this._timestamp = now;
            // If horizontal scrolling is enabled, and the content actually warrants scrolling...
            if (this._settings.horizontalScroll) {
                const delta = x - this._previousPosition.x;
                this._previousPosition.x = x;
                this._contents.position.x += delta;
                this._velocity.x = 100 * (delta / elapsed);
            }
            // If vertical scrolling is enabled, and the content actually warrants scrolling...
            if (this._settings.verticalScroll) {
                const delta = y - this._previousPosition.y;
                this._previousPosition.y = y;
                this._contents.position.y += delta;
                this._velocity.y = 100 * (delta / elapsed);
            }
            this.limitMovement();
        }
        limitMovement() {
            if (this._settings.horizontalScroll) {
                this.limitHorizontal();
            }
            if (this._settings.verticalScroll) {
                this.limitVertical();
            }
        }
        limitHorizontal() {
            if (this._contents.position.x >= 0) {
                this._contents.position.x = 0;
            }
            else {
                const contentWidth = this._contentLocalBounds.width + this._contentLocalBounds.x;
                const limit = this._displayMask.width - contentWidth - this._settings.paddingRight;
                if (this._contents.position.x <= limit) {
                    this._contents.position.x = limit;
                }
            }
        }
        limitVertical() {
            if (this._contents.position.y >= 0) {
                this._contents.position.y = 0;
            }
            else {
                const contentHeight = this._contentLocalBounds.height + this._contentLocalBounds.y;
                const limit = this._displayMask.height - contentHeight - this._settings.paddingBottom;
                if (this._contents.position.y <= limit) {
                    this._contents.position.y = limit;
                }
            }
        }
        /*
         * This function is a replacement for this.add.existing and routes the
         * object added to the gameGroup instead of the top of the state.
         */
        addToContentLayer(object) {
            this._contents.add(object);
        }
    }
    exports.ScrollableGroup = ScrollableGroup;
});
define("switchback-common/component/scrolling/ScrollableMaskGroup", ["require", "exports", "switchback-common/component/scrolling/ScrollableGroup"], function (require, exports, ScrollableGroup_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * A scrollable group whose contents outside the display area are masked out.
     */
    class ScrollableMaskGroup extends ScrollableGroup_1.ScrollableGroup {
        constructor(game, viewableWidth, viewableHeight, settings = {}, parent = null) {
            super(game, viewableWidth, viewableHeight, settings, parent);
            const mask = this.game.add.graphics(0, 0);
            mask.beginFill(0x000000, 0);
            mask.drawRect(0, 0, this._displayMask.width, this._displayMask.height);
            mask.endFill();
            this._contents.mask = mask;
        }
        /**
         * As the mask is added globally, we have to ensure we're masking from the correct location.
         * This must be called after this object moves.
         */
        updateMaskPosition() {
            this._contents.mask.x = this.worldPosition.x;
            this._contents.mask.y = this.worldPosition.y;
        }
        // Fix for masks not working on WebGL
        // https://github.com/photonstorm/phaser-ce/issues/334
        beginMove(pointer) {
            super.beginMove(pointer);
            this._contents.mask.dirty = true;
        }
        // Fix for masks not working on WebGL
        // https://github.com/photonstorm/phaser-ce/issues/334
        onContentsUpdated() {
            super.onContentsUpdated();
            this._contents.mask.dirty = true;
        }
    }
    exports.ScrollableMaskGroup = ScrollableMaskGroup;
});
/**
 * I can't pretend I understand this...
 * Found @ http://tiny.cc/uppary
 *
 * Modified:
 * 1. to fix retrieving the image from Phaser cache
 * 2. added frame default as 0
 *
 * Note: Don't configure a NineSlice to be smaller than its borders
 * e.g. if it has 100px borders make sure it's at least 200x200
 */
define("switchback-common/extension/NineSlice", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class NineSlice {
        static create(aGame, aWidth, aHeight, aKey, aFrame = 0, aTop = 0, aLeft = 0, aBottom = 0, aRight = 0, aRepeats = false) {
            // image
            NineSlice.image = new Phaser.Image(aGame, 0, 0, aKey);
            // get frame
            if (typeof aFrame === "string") {
                NineSlice.frame = aGame.cache.getFrameByName(aKey, aFrame);
            }
            else {
                NineSlice.frame = aGame.cache.getFrameByIndex(aKey, aFrame);
            }
            // calculate dims
            NineSlice.calculateNineImage(aWidth, aHeight, aTop, aLeft, aBottom, aRight, aRepeats);
            // create nine image bitmap data
            NineSlice.nineImage = new Phaser.BitmapData(aGame, "NineImage" + (NineSlice.textureKey++), NineSlice.width, NineSlice.height);
            // render into bitmap data
            NineSlice.renderNineImage();
            return NineSlice.nineImage;
        }
        // -------------------------------------------------------------------------
        static calculateNineImage(aWidth, aHeight, aTop, aLeft, aBottom, aRight, aRepeats) {
            const frame = NineSlice.frame;
            NineSlice.centralWidth = frame.width - aLeft - aRight;
            NineSlice.centralHeight = frame.height - aTop - aBottom;
            if (aRepeats) {
                NineSlice.horizontalRepeats = aWidth;
                NineSlice.verticalRepeats = aHeight;
                NineSlice.width = aLeft + aRight + NineSlice.centralWidth * aWidth;
                NineSlice.height = aTop + aBottom + NineSlice.centralHeight * aHeight;
                NineSlice.lastWidth = 0;
                NineSlice.lastHeight = 0;
            }
            else {
                const w = aWidth - aLeft - aRight;
                NineSlice.horizontalRepeats = Math.floor(w / NineSlice.centralWidth);
                NineSlice.lastWidth = w % NineSlice.centralWidth;
                const h = aHeight - aTop - aBottom;
                NineSlice.verticalRepeats = Math.floor(h / NineSlice.centralHeight);
                NineSlice.lastHeight = h % NineSlice.centralHeight;
                NineSlice.width = aWidth;
                NineSlice.height = aHeight;
            }
            NineSlice.leftWidth = aLeft;
            NineSlice.rightWidth = aRight;
            NineSlice.topHeight = aTop;
            NineSlice.bottomHeight = aBottom;
        }
        static renderNineImage() {
            let sourceY = NineSlice.frame.y;
            let destY = 0;
            // top row
            if (NineSlice.topHeight > 0) {
                NineSlice.renderNineImageRow(NineSlice.image, sourceY, destY, NineSlice.topHeight);
                sourceY += NineSlice.topHeight;
                destY += NineSlice.topHeight;
            }
            // centrals
            for (let i = 0; i < NineSlice.verticalRepeats; i++) {
                NineSlice.renderNineImageRow(NineSlice.image, sourceY, destY, NineSlice.centralHeight);
                destY += NineSlice.centralHeight;
            }
            // last height
            if (NineSlice.lastHeight > 0) {
                NineSlice.renderNineImageRow(NineSlice.image, sourceY, destY, NineSlice.lastHeight);
                destY += NineSlice.lastHeight;
            }
            sourceY += NineSlice.centralHeight;
            // bottom
            if (NineSlice.bottomHeight > 0) {
                NineSlice.renderNineImageRow(NineSlice.image, sourceY, destY, NineSlice.bottomHeight);
            }
        }
        static renderNineImageRow(aImage, aSourceY, aDestY, aHeight) {
            let sourceX = NineSlice.frame.x;
            let destX = 0;
            // left
            if (NineSlice.leftWidth > 0) {
                NineSlice.nineImage.copy(aImage, sourceX, aSourceY, NineSlice.leftWidth, aHeight, destX, aDestY);
                destX += NineSlice.leftWidth;
                sourceX += NineSlice.leftWidth;
            }
            // centrals
            for (let i = 0; i < NineSlice.horizontalRepeats; i++) {
                NineSlice.nineImage.copy(aImage, sourceX, aSourceY, NineSlice.centralWidth, aHeight, destX, aDestY);
                destX += NineSlice.centralWidth;
            }
            // last width
            if (NineSlice.lastWidth > 0) {
                NineSlice.nineImage.copy(aImage, sourceX, aSourceY, NineSlice.lastWidth, aHeight, destX, aDestY);
                destX += NineSlice.lastWidth;
            }
            sourceX += NineSlice.centralWidth;
            // right
            if (NineSlice.rightWidth > 0) {
                NineSlice.nineImage.copy(aImage, sourceX, aSourceY, NineSlice.rightWidth, aHeight, destX, aDestY);
            }
        }
    }
    NineSlice.textureKey = 0;
    exports.NineSlice = NineSlice;
});
const pDisplayObjCont = PIXI.DisplayObjectContainer;
const pDisplayObj = PIXI.DisplayObject;
pDisplayObjCont.prototype.displayObjectUpdateTransform = pDisplayObj.prototype.updateTransform = function (parent) {
    let a;
    let b;
    let c;
    let d;
    let p;
    let pivot;
    let position;
    let pt;
    let ref;
    let rotation;
    let scale;
    let skew;
    let skewX;
    let skewY;
    let tx;
    let ty;
    let wt;
    if (!parent && !this.parent && !this.game) {
        return this;
    }
    p = this.parent;
    if (parent) {
        p = parent;
    }
    else if (!this.parent) {
        p = this.game.world;
    }
    pt = p.worldTransform;
    wt = this.worldTransform;
    ref = this, pivot = ref.pivot, position = ref.position, rotation = ref.rotation, scale = ref.scale, skew = ref.skew;
    a = b = c = d = tx = ty = null;
    if ((rotation % PIXI.PI_2) || skew) {
        skewX = (skew && skew.x) || 0;
        skewY = (skew && skew.y) || 0;
        a = scale.x * Math.cos(rotation + skewY);
        b = scale.x * Math.sin(rotation + skewY);
        c = scale.y * Math.sin(-rotation - skewX);
        d = scale.y * Math.cos(rotation + skewX);
        tx = position.x;
        ty = position.y;
        if (pivot.x || pivot.y) {
            tx -= pivot.x * a + pivot.y * c;
            ty -= pivot.x * b + pivot.y * d;
        }
        wt.a = a * pt.a + b * pt.c;
        wt.b = a * pt.b + b * pt.d;
        wt.c = c * pt.a + d * pt.c;
        wt.d = c * pt.b + d * pt.d;
        wt.tx = tx * pt.a + ty * pt.c + pt.tx;
        wt.ty = tx * pt.b + ty * pt.d + pt.ty;
    }
    else {
        a = scale.x;
        d = scale.y;
        tx = position.x - (pivot.x * a);
        ty = position.y - (pivot.y * d);
        wt.a = a * pt.a;
        wt.b = a * pt.b;
        wt.c = d * pt.c;
        wt.d = d * pt.d;
        wt.tx = tx * pt.a + ty * pt.c + pt.tx;
        wt.ty = tx * pt.b + ty * pt.d + pt.ty;
    }
    this.worldAlpha = this.alpha * p.worldAlpha;
    this.worldPosition.x = wt.tx;
    this.worldPosition.y = wt.ty;
    this.worldScale.x = scale.x * Math.sqrt(wt.a * wt.a + wt.c * wt.c);
    this.worldScale.y = scale.y * Math.sqrt(wt.b * wt.b + wt.d * wt.d);
    this.worldRotation = Math.atan2(-wt.c, wt.d);
    this._currentBounds = null;
    if (this.transformCallback) {
        this.transformCallback.call(this.transformCallbackContext, wt, pt);
    }
    return this;
};
define("switchback-common/play/PlayController", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     */
    class PlayController {
        constructor() {
            window.webkit = {};
            window.webkit.messageHandlers = {};
            window.webkit.messageHandlers.NativeInterop = {};
            window.webkit.messageHandlers.NativeInterop.postMessage = (obj) => { return; };
        }
        start() {
            console.log("[PlayController] start()");
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.GAME_INIT_EVENT));
        }
        eject() {
            console.log("[PlayController] eject()");
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.GAME_EJECT));
        }
        movePlayer(location) {
            console.log(`[PlayController] movePlayer(${location})`);
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.DEVICE_ON_LOCATION_CHANGED, {
                detail: location,
            }));
        }
    }
    exports.PlayController = PlayController;
});
define("switchback-common/play/DummyApiRunner", ["require", "exports", "switchback-common/play/PlayController", "switchback-common/util/PromiseUtil"], function (require, exports, PlayController_1, PromiseUtil_61) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     */
    class DummyApiRunner {
        constructor() {
            this.controller = new PlayController_1.PlayController();
        }
        start() {
            return __awaiter(this, void 0, void 0, function* () {
                yield PromiseUtil_61.PromiseUtil.realTime(3000);
                this.controller.start();
                this.controller.movePlayer({
                    attractionName: "Little Foot's Wild Ride",
                    landName: "Land Before Time",
                    latitude: 66.33,
                    locationID: "beacon_" + Math.floor(Math.random() * 20),
                    longitude: 100.22,
                    parkName: "Animal Kingdom",
                });
                yield PromiseUtil_61.PromiseUtil.realTime(1500);
                this.controller.eject();
            });
        }
    }
    exports.DummyApiRunner = DummyApiRunner;
});
define("switchback-common/play/MiniGamePlaySession", ["require", "exports", "switchback-common/util/CollectionUtil"], function (require, exports, CollectionUtil_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MiniGamePlaySession {
        constructor(playerList) {
            this._playGroups = [];
            this._playerList = playerList;
        }
        startMiniGame(playerCount) {
            switch (playerCount) {
                case MiniGamePlaySession.PlayerCount.ALL:
                    this._playGroups = MiniGamePlaySession.generateAllPlayerGroup(this._playerList);
                    break;
                case MiniGamePlaySession.PlayerCount.SINGLE:
                    this._playGroups = MiniGamePlaySession.generateSinglePlayerGroups(this._playerList, this.previousMiniGameLastPlayers);
                    break;
                case MiniGamePlaySession.PlayerCount.TWO:
                    this._playGroups = MiniGamePlaySession.generateTwoPlayerGroups(this._playerList, this.previousMiniGameLastPlayers);
                    break;
            }
            this._currentPlayGroupIndex = 0;
        }
        playGroupFinished() {
            this._currentPlayGroupIndex++;
        }
        get roundIndex() {
            return this._currentPlayGroupIndex;
        }
        get isMiniGameSessionFinished() {
            return this._currentPlayGroupIndex >= this._playGroups.length;
        }
        get currentPlayGroup() {
            return this._playGroups[this._currentPlayGroupIndex];
        }
        get previousMiniGameLastPlayers() {
            return this._playGroups.length === 0 ? [] : this._playGroups[this._playGroups.length - 1].players;
        }
        static generateAllPlayerGroup(playerList) {
            const playGroups = [];
            playGroups.push({ players: playerList });
            return playGroups;
        }
        /**
         * Returns an array of mini game play groups
         * - Each group will only contain a single player.
         * The first group will not include any player from the list of players who should not play first
         * - Exception to this rule is if we only have one player
         * @param playerList
         * @param shouldNotPlayFirstList
         */
        static generateSinglePlayerGroups(playerList, shouldNotPlayFirstList) {
            const playGroups = [];
            const playerPool = [...playerList];
            const playersAvailableToBeFirst = CollectionUtil_4.exclusiveLeftJoinArrays(playerPool, shouldNotPlayFirstList);
            const randomFirstPlayer = CollectionUtil_4.getRandomItem(playersAvailableToBeFirst);
            if (randomFirstPlayer) {
                playGroups.push({ players: [randomFirstPlayer] });
                playerPool.splice(playerPool.indexOf(randomFirstPlayer), 1);
            }
            CollectionUtil_4.shuffle(playerPool);
            playerPool.forEach((player) => {
                playGroups.push({ players: [player] });
            });
            return playGroups;
        }
        /**
         * Returns an array of mini game play groups
         * - Each group will contain two players
         * The first group will have at least one player who is not in the list of players who should not play first
         * - Exception to this rule if we only have 2 or 3 players
         * NOTE: If we have less than two players then an empty mini game session is returned
         * @param playerList
         * @param shouldNotPlayFirstList
         */
        static generateTwoPlayerGroups(playerList, shouldNotPlayFirstList) {
            if (playerList.length < 2) {
                console.error(`[MiniGamePlaySession] Unable to generate play groups for two players when we don't have that many players`);
                return [];
            }
            const playerPool = [...playerList];
            const playersAvailableToBeFirst = CollectionUtil_4.exclusiveLeftJoinArrays(playerPool, shouldNotPlayFirstList);
            const randomFirstPlayer = CollectionUtil_4.getRandomItem(playersAvailableToBeFirst);
            CollectionUtil_4.shuffle(playerPool);
            if (randomFirstPlayer) {
                // Remove the randomly selected player from the player pool,
                // Then insert them second in the array so that they get the first two plays
                playerPool.splice(playerPool.indexOf(randomFirstPlayer), 1);
                playerPool.splice(1, 0, randomFirstPlayer);
            }
            if (playerPool.length === 2) {
                return [{ players: playerPool }];
            }
            const playSession = [];
            for (let i = 0; i < playerPool.length; i++) {
                const playerOne = playerPool[i];
                const playerTwo = (i + 1 === playerPool.length) ? playerPool[0] : playerPool[i + 1];
                const group = [playerOne, playerTwo];
                if (i % 2 === 1) {
                    group.reverse();
                }
                playSession.push({ players: group });
            }
            return playSession;
        }
        /**
         * ONLY TO BE CALLED FROM TEST FUNCTIONS.
         * Allows the current play group index to be set
         * @param index
         */
        TEST_setCurrentPlayGroupIndex(index) {
            this._currentPlayGroupIndex = index;
        }
        /**
         * ONLY TO BE CALLED FROM TEST FUNCTIONS.
         * Allows the last know players to be set
         * @param lastKnowPlayers
         */
        TEST_setLastKnownPlayers(lastKnowPlayers) {
            this._playGroups = [{ players: lastKnowPlayers }];
        }
    }
    exports.MiniGamePlaySession = MiniGamePlaySession;
    (function (MiniGamePlaySession) {
        let PlayerCount;
        (function (PlayerCount) {
            PlayerCount[PlayerCount["ALL"] = 0] = "ALL";
            PlayerCount[PlayerCount["SINGLE"] = 1] = "SINGLE";
            PlayerCount[PlayerCount["TWO"] = 2] = "TWO";
        })(PlayerCount = MiniGamePlaySession.PlayerCount || (MiniGamePlaySession.PlayerCount = {}));
    })(MiniGamePlaySession = exports.MiniGamePlaySession || (exports.MiniGamePlaySession = {}));
});
define("switchback-common/play/PlayerManager", ["require", "exports", "switchback-common/util/CollectionUtil"], function (require, exports, CollectionUtil_5) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PlayerManager {
        openPlayerManagement() {
            return __awaiter(this, void 0, void 0, function* () {
                let players;
                try {
                    const result = yield PlayAPI.openPlayerManagement();
                    // if result contains no players, populate the players array with dummy players
                    if (result && result.players && result.players.length > 0) {
                        players = result.players.map((player) => {
                            return Object.assign({}, player, { instructionsViewed: new Set() });
                        });
                    }
                    else {
                        players = [
                            { name: "Player 1", type: "", identifier: "" },
                            { name: "Player 2", type: "", identifier: "" },
                        ];
                    }
                }
                catch (error) {
                    console.log("openPlayerManagement Error " + JSON.stringify(error));
                    // Initialize with fake data just so the app doesn't break
                    players = [
                        { name: "Player 1", type: "", identifier: "" },
                        { name: "Player 2", type: "", identifier: "" },
                    ];
                }
                this._players = players;
                return players;
            });
        }
        get players() {
            return this._players;
        }
        set players(p) {
            this._players = p;
        }
        /**
         * Player grouping
         * Returns a session, which is a list of groups. A group is a list of players that plays in a round.
         * @param GroupSize number = 1 : how many players per group. =1 individual game, =2 pair game, =more for bigger groups
         * If groupSize is bigger than the number of available players this will be set to this.players.length
         * @param rounds number = null : hom many rounds are needed, =null as many as possible without repeating,
         * if there aren't enough players it will start repeating
         * @param fixedPairs boolean = false : If true, for pair games, avoids making the players switch sides
         * after sequential rounds.
         * @param isShuffle boolean = false : If true, the player pool array is shuffled
         * @return list of groups. group=list of players : [ [A, B], [B, C], [C, A]]
         */
        // currently not being used in TSM 
        getPlayerSession(groupSize = 1, rounds = null, fixedPairs = false, isShuffle = false) {
            const result = [];
            if (rounds == null) {
                rounds = this.players.length;
            }
            groupSize = Math.min(groupSize, this.players.length);
            if (isShuffle) {
                CollectionUtil_5.shuffleNonRepeatingArray(this.players);
            }
            // set the pool of players
            const pool = [];
            for (const player of this.players) {
                pool.push(player);
            }
            for (let i = 0; i < rounds; i++) {
                const group = pool.slice(0, groupSize);
                // In case the game is in pairs and we want static positions, revert every other group
                if (fixedPairs && (groupSize === 2) && (i % 2 === 0)) {
                    group.reverse();
                }
                pool.push(pool.shift());
                result.push(group);
            }
            return result;
        }
        set prevPlayerNames(prevPlayers) {
            // stores the previous player names for comparison
            if (prevPlayers) {
                this.previousSessionPlayerNames = prevPlayers.slice();
            }
        }
        get prevPlayerNames() {
            return this.previousSessionPlayerNames;
        }
    }
    exports.PlayerManager = PlayerManager;
});
/**
 * Wrapper for Play Disney Parks AR Kit
 * @author joel@cerebralfix.com
 */
define("switchback-common/play/augmented-reality/ThreeARScene", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class ThreeARScene {
        constructor(canvas) {
            this._sessionStarted = false;
            this.markerDetectedSignal = new Phaser.Signal();
            console.log("[ThreeARScene] new()");
            this._canvasTarget = canvas;
        }
        get renderer() {
            return this._renderer;
        }
        get scene() {
            return this._scene;
        }
        get canvas() {
            return this._canvasTarget;
        }
        get camera() {
            return this._camera;
        }
        registerImage(name, pathInBundle, realWorldWidth) {
            if (!this._sessionStarted) {
                console.error("[ThreeARScene] registerImage() cannot register images before AR session is started");
                return;
            }
            PlayAPI.AugmentedReality.registerImage(name, pathInBundle, realWorldWidth).then(function () {
                console.log(`[ThreeARScene] registerImage(): successfully registered ${name}`);
            }).catch((e) => {
                console.warn(`[ThreeARScene] registerImage(): failed to register ${name}`);
                console.error(JSON.stringify(e));
            });
        }
        sessionStart() {
            return __awaiter(this, void 0, void 0, function* () {
                console.log("[ThreeARScene] startSession()");
                PlayAPI.AugmentedReality.sessionStart(this._canvasTarget).then(() => {
                    console.log(`[ThreeARScene] startSession(): success`);
                    this._sessionStarted = true;
                }).catch((e) => {
                    console.warn(`[ThreeARScene] startSession(): error`);
                    console.error(JSON.stringify(e));
                    this._sessionStarted = false;
                });
                // TODO: Figure out why sessionStart isn't returning, then remove this hack
                this._sessionStarted = true;
                if (!this._sessionStarted) {
                    return false;
                }
                else {
                    document.body.classList.add("transparent");
                }
                // Subscribe to PlayAPI image recognize events
                PlayAPI.subscribeToEvent(PlayAPI.AugmentedReality.events.AR_SESSION_DID_RECOGNIZE_IMAGE, (event) => {
                    console.log("[ThreeARScene] PlayAPI.AugmentedReality.events.AR_SESSION_DID_RECOGNIZE_IMAGE");
                    // Let any listeners know the details of the marker we found.
                    this.markerDetectedSignal.dispatch(event);
                });
                this._renderer = PlayAPI.AugmentedReality.renderer;
                this._scene = PlayAPI.AugmentedReality.scene;
                this._camera = PlayAPI.AugmentedReality.camera;
                return this._sessionStarted;
            });
        }
        endSession() {
            console.log("[ThreeARScene] endSession()");
            let response;
            PlayAPI.AugmentedReality.sessionEnd().then(() => {
                console.log(`[ThreeARScene] endSession(): success`);
                response = true;
            }).catch((e) => {
                console.warn(`[ThreeARScene] endSession(): error`);
                console.error(JSON.stringify(e));
                response = false;
            });
            this._sessionStarted = this._sessionStarted && response;
            document.body.classList.remove("transparent");
            return response;
        }
        destroy() {
            this._renderer.clear();
            this.markerDetectedSignal.dispose();
        }
    }
    exports.ThreeARScene = ThreeARScene;
});
define("switchback-common/play/debug/PlayDebugPanel", ["require", "exports", "switchback-common/play/api/MockPlayAPI"], function (require, exports, MockPlayAPI_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * PlayDebugPanel can trigger events that the PlayAPI will respond to.
     * @author joel@cerebralfix.com
     */
    class PlayDebugPanel extends Phaser.Group {
        constructor(game) {
            super(game);
            const BUTTON_PADDING = 5;
            const BUTTON_WIDTH = 200;
            const BUTTON_HEIGHT = 100;
            // EJECT beacon button
            const beaconEject = new Phaser.Button(this.game, BUTTON_PADDING, BUTTON_PADDING, "button-basic", this.onBeaconEjectButtonClicked, this, "over", "out", "down", "out");
            beaconEject.width = BUTTON_WIDTH;
            beaconEject.height = BUTTON_HEIGHT;
            beaconEject.alpha = 0.5;
            // EJECT beacon button text
            const beaconEjectText = new Phaser.Text(game, beaconEject.centerX, beaconEject.centerY, "EJECT\nBEACON");
            beaconEjectText.anchor.set(0.5);
            // START beacon button
            const beaconStart = new Phaser.Button(this.game, beaconEject.x + beaconEject.width + BUTTON_PADDING, beaconEject.y, "button-basic", this.onBeaconStartButtonClicked, this, "over", "out", "down", "out");
            beaconStart.width = BUTTON_WIDTH;
            beaconStart.height = BUTTON_HEIGHT;
            beaconStart.alpha = 0.5;
            // START beacon button text
            const beaconStartText = new Phaser.Text(game, beaconStart.centerX, beaconStart.centerY, "START\nBEACON");
            beaconStartText.anchor.set(0.5);
            // LOCATION change button
            const locationChange = new Phaser.Button(this.game, beaconStart.x + beaconStart.width + BUTTON_PADDING, beaconStart.y, "button-basic", this.onLocationChangeButtonClicked, this, "over", "out", "down", "out");
            locationChange.width = BUTTON_WIDTH;
            locationChange.height = BUTTON_HEIGHT;
            locationChange.alpha = 0.5;
            // LOCATION change button text
            const locationChangeText = new Phaser.Text(game, locationChange.centerX, locationChange.centerY, "RANDOM\nLOCATION");
            locationChangeText.anchor.set(0.5);
            // Background panel graphic
            const background = new Phaser.Image(game, 0, 0, "button-basic");
            background.alpha = 0.2;
            this.add(background);
            this.add(beaconEject);
            this.add(beaconStart);
            this.add(locationChange);
            this.add(beaconEjectText);
            this.add(beaconStartText);
            this.add(locationChangeText);
            // Size the background to sit behind everything that has been added
            background.width = this.width + BUTTON_PADDING;
            background.height = this.height + BUTTON_PADDING;
        }
        onLocationChangeButtonClicked() {
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.DEVICE_ON_LOCATION_CHANGED, {
                detail: MockPlayAPI_2.MockPlayAPI.getFakeLocation(),
            }));
        }
        onBeaconEjectButtonClicked() {
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.GAME_EJECT));
        }
        onBeaconStartButtonClicked() {
            window.dispatchEvent(new CustomEvent(PlayAPI.gameEvents.GAME_INIT_EVENT));
        }
    }
    exports.PlayDebugPanel = PlayDebugPanel;
});
define("switchback-common/play/debug/PlayDebug", ["require", "exports", "switchback-common/play/debug/PlayDebugPanel"], function (require, exports, PlayDebugPanel_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * PlayDebug creates a PlayDebugPanel that can be used to trigger events via the PlayAPI.
     * @author joel@cerebralfix.com
     */
    class PlayDebug {
        constructor(game) {
            // Toggle button
            const toggleButton = new Phaser.Button(game, 0, 0, "button-basic", this.onDebugButtonClicked, this, "over", "out", "down", "out");
            toggleButton.bottom = game.height;
            toggleButton.alpha = 0.3;
            // Toggle button text
            const toggleButtonText = new Phaser.Text(game, toggleButton.centerX, toggleButton.centerY, "DEBUG");
            toggleButtonText.anchor.set(0.5);
            // Debug panel
            this.debugPanel = new PlayDebugPanel_1.PlayDebugPanel(game);
            this.debugPanel.centerX = game.world.centerX;
            this.debugPanel.bottom = game.height;
            this.debugPanel.visible = false;
            game.add.existing(toggleButton);
            game.add.existing(toggleButtonText);
        }
        onDebugButtonClicked() {
            this.debugPanel.visible = !this.debugPanel.visible;
        }
    }
    exports.PlayDebug = PlayDebug;
});
define("switchback-common/test/keyboardController.spec", ["require", "exports", "switchback-common/component/keyboard/KeyboardController", "switchback-common/component/keyboard/KeyboardOptions", "switchback-common/component/keyboard/SpecialChars"], function (require, exports, KeyboardController_2, KeyboardOptions_1, SpecialChars_4) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe("keyboardController", () => {
        let keyboardController;
        beforeEach(() => {
            keyboardController = new KeyboardController_2.KeyboardController(new KeyboardOptions_1.KeyboardOptions());
        });
        it("can add letters to a string", () => {
            "TEST".split("").forEach((char) => {
                keyboardController.keyPressed(char);
            });
            expect(keyboardController.currentString).toBe("TEST");
        });
        it("can submit a string", (done) => {
            keyboardController.submitSignal.addOnce((response) => {
                expect(response).toBe("TEST");
                done();
            });
            "TEST".split("").forEach((char) => {
                keyboardController.keyPressed(char);
            });
            keyboardController.keyPressed(SpecialChars_4.SpecialChar.SUBMIT);
        });
        it("can delete letters from a string", () => {
            "TEST".split("").forEach((char) => {
                keyboardController.keyPressed(char);
            });
            keyboardController.keyPressed(SpecialChars_4.SpecialChar.DELETE);
            expect(keyboardController.currentString).toBe("TES");
        });
    });
});
define("switchback-common/test/playerManager.spec", ["require", "exports", "switchback-common/play/PlayerManager"], function (require, exports, PlayerManager_2) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe("PlayerManager", () => {
        let playerManager;
        beforeEach(() => {
            playerManager = new PlayerManager_2.PlayerManager();
        });
        describe("Getters and Setters", () => {
            describe("handles null parameters", () => {
                [null, undefined].forEach((value) => {
                    it(`prevPlayerNames handles ${value}`, () => {
                        playerManager.prevPlayerNames = value;
                        expect(playerManager.prevPlayerNames).toBeUndefined();
                    });
                });
                [null, undefined].forEach((value) => {
                    it(`players handles ${value}`, () => {
                        playerManager.prevPlayerNames = value;
                        expect(playerManager.prevPlayerNames).toBeUndefined();
                    });
                });
            });
            it("overwrites existing previous player names", () => {
                playerManager.prevPlayerNames = ["n1", "n2"];
                playerManager.prevPlayerNames = ["n3", "n4"];
                expect(playerManager.prevPlayerNames).toEqual(["n3", "n4"]);
            });
        });
        describe("Player Management", () => {
            it("has players", () => {
                playerManager.players = [
                    { name: "Player 1", type: "", identifier: "" },
                    { name: "Player 2", type: "", identifier: "" },
                ];
                expect(playerManager.players).toBeTruthy();
            });
        });
        describe("Player Sessions", () => {
            beforeEach(() => {
                playerManager.players = [
                    { name: "Player 1", type: "", identifier: "" },
                    { name: "Player 2", type: "", identifier: "" },
                ];
            });
            [2, 3, 4, 5].forEach((numberOfRounds) => {
                it("generates the correct number of rounds", () => {
                    const session = playerManager.getPlayerSession(2, numberOfRounds);
                    expect(session.length).toBe(numberOfRounds);
                });
            });
            [1, 2].forEach((numberOfRounds) => {
                it("generates rounds of the correct size", () => {
                    const session = playerManager.getPlayerSession(numberOfRounds);
                    session.forEach((round) => {
                        expect(round.length).toBe(numberOfRounds);
                    });
                });
            });
            it("has different players in each round", () => {
                const session = playerManager.getPlayerSession(2, 5, true);
                session.forEach((round) => {
                    expect(round[0]).not.toEqual(round[1]);
                });
            });
            it("prevents players from swapping sides", () => {
                const session = playerManager.getPlayerSession(2, 5, true);
                expect(session[0][1]).toEqual(session[1][1]);
            });
        });
    });
});
define("switchback-common/test/signal.spec", ["require", "exports", "switchback-common/component/Signal"], function (require, exports, Signal_3) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe("Signal", () => {
        it("a listener is called each time a signal is dispatched", () => {
            const signal = new Signal_3.Signal();
            let callCount = 0;
            signal.add(() => {
                callCount++;
            });
            signal.dispatch();
            signal.dispatch();
            expect(callCount).toBe(2);
        });
        it("addOnce causes a listener to be called only once", () => {
            const signal = new Signal_3.Signal();
            let callCount = 0;
            signal.addOnce(() => {
                callCount++;
            });
            signal.dispatch();
            signal.dispatch();
            expect(callCount).toBe(1);
        });
        it("dispatch calls every attached listener", () => {
            const signal = new Signal_3.Signal();
            let callCount = 0;
            signal.addOnce(() => {
                callCount++;
            });
            signal.addOnce(() => {
                callCount++;
            });
            signal.dispatch();
            expect(callCount).toBe(2);
        });
        it("disposing a signal stops a listener being called", () => {
            const signal = new Signal_3.Signal();
            let callCount = 0;
            signal.add(() => {
                callCount++;
            });
            signal.dispatch();
            signal.dispose();
            signal.dispatch();
            expect(callCount).toBe(1);
        });
        it("listeners are called in the order they are added", () => {
            const signal = new Signal_3.Signal();
            const results = [];
            signal.addOnce(() => {
                results.push(1);
            });
            signal.addOnce(() => {
                results.push(2);
            });
            signal.dispatch();
            expect(results[0]).toBe(1);
            expect(results[1]).toBe(2);
        });
    });
});
define("switchback-common/test/unit/CollectionUtil.spec", ["require", "exports", "switchback-common/util/CollectionUtil"], function (require, exports, CollectionUtil_6) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe(`CollectionUtil`, () => {
        describe(`Exclusive Left Join`, () => {
            it(`Excluded array contains 1 item found in the array`, () => {
                const result = CollectionUtil_6.exclusiveLeftJoinArrays(["a", "b", "c"], ["b"]);
                expect(result).toEqual(["a", "c"]);
            });
            it(`Excluded array contains 1 item not found in the arry`, () => {
                const result = CollectionUtil_6.exclusiveLeftJoinArrays(["a", "b", "c"], ["d"]);
                expect(result).toEqual(["a", "b", "c"]);
            });
            it(`Excluded array contains 2 items found in the array`, () => {
                const result = CollectionUtil_6.exclusiveLeftJoinArrays(["a", "b", "c"], ["b", "c"]);
                expect(result).toEqual(["a"]);
            });
            it(`Excluded array contains 2 items with 1 item found in the array`, () => {
                const result = CollectionUtil_6.exclusiveLeftJoinArrays(["a", "b", "c"], ["b", "d"]);
                expect(result).toEqual(["a", "c"]);
            });
            it(`Excluded array contains 2 items with 2 items found in the array with a different order`, () => {
                const result = CollectionUtil_6.exclusiveLeftJoinArrays(["a", "b", "c"], ["c", "a"]);
                expect(result).toEqual(["b"]);
            });
            it(`Excluded array contains 2 items with 1 item found in the array with a different order`, () => {
                const result = CollectionUtil_6.exclusiveLeftJoinArrays(["a", "b", "c"], ["d", "a"]);
                expect(result).toEqual(["b", "c"]);
            });
            it(`Excluded array is empty`, () => {
                const result = CollectionUtil_6.exclusiveLeftJoinArrays(["a", "b", "c"], []);
                expect(result).toEqual(["a", "b", "c"]);
            });
        });
        describe(`Inner Join`, () => {
            it(`Second array contains 1 item found in the array`, () => {
                const result = CollectionUtil_6.innerJoinArrays(["a", "b", "c"], ["b"]);
                expect(result).toEqual(["b"]);
            });
            it(`Second array contains 1 item not found in the arry`, () => {
                const result = CollectionUtil_6.innerJoinArrays(["a", "b", "c"], ["d"]);
                expect(result).toEqual([]);
            });
            it(`Second array contains 2 items found in the array`, () => {
                const result = CollectionUtil_6.innerJoinArrays(["a", "b", "c"], ["b", "c"]);
                expect(result).toEqual(["b", "c"]);
            });
            it(`Second array contains 2 items with 1 item found in the array`, () => {
                const result = CollectionUtil_6.innerJoinArrays(["a", "b", "c"], ["b", "d"]);
                expect(result).toEqual(["b"]);
            });
            it(`Second array contains 2 items with 2 items found in the array with a different order`, () => {
                const result = CollectionUtil_6.innerJoinArrays(["a", "b", "c"], ["c", "a"]);
                expect(result).toEqual(["a", "c"]);
            });
            it(`Second array contains 2 items with 1 item found in the array with a different order`, () => {
                const result = CollectionUtil_6.innerJoinArrays(["a", "b", "c"], ["d", "a"]);
                expect(result).toEqual(["a"]);
            });
            it(`Second array is empty`, () => {
                const result = CollectionUtil_6.innerJoinArrays(["a", "b", "c"], []);
                expect(result).toEqual([]);
            });
        });
    });
});
define("switchback-common/test/unit/MiniGamePlaySession.spec", ["require", "exports", "switchback-common/play/MiniGamePlaySession"], function (require, exports, MiniGamePlaySession_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    describe(`MiniGamePlaySession`, () => {
        const player0 = { identifier: "0", name: "player 0", type: "" };
        const player1 = { identifier: "1", name: "player 1", type: "" };
        const player2 = { identifier: "2", name: "player 2", type: "" };
        const player3 = { identifier: "3", name: "player 3", type: "" };
        const player4 = { identifier: "4", name: "player 4", type: "" };
        describe(`Player group generation`, () => {
            describe(`Single Player Mini Games`, () => {
                it(`The correct number of play session for a single player are created for 5 players`, () => {
                    const players = [player0, player1, player2, player3, player4];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateSinglePlayerGroups(players, []);
                    expect(result.length).toBe(5);
                });
                it(`First player is not one of the last known players`, () => {
                    const players = [player0, player1, player2, player3, player4];
                    const lastKnownPlayers = [player0, player1, player2, player3];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateSinglePlayerGroups(players, lastKnownPlayers);
                    expect(result[0].players).toEqual([player4]);
                });
            });
            describe(`Two Player Mini Games`, () => {
                it(`The correct number of play sessions for two people are created for 5 players`, () => {
                    const players = [player0, player1, player2, player3, player4];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateTwoPlayerGroups(players, []);
                    expect(result.length).toBe(5);
                });
                it(`Player sequence is correct for 5 players. One player plays twice in a row and in the same position`, () => {
                    const players = [player0, player1, player2, player3, player4];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateTwoPlayerGroups(players, []);
                    expect(result[0].players[1]).toEqual(result[1].players[1]);
                    expect(result[1].players[0]).toEqual(result[2].players[0]);
                    expect(result[2].players[1]).toEqual(result[3].players[1]);
                    expect(result[3].players[0]).toEqual(result[4].players[0]);
                    expect(result[0].players[0]).toEqual(result[4].players[1]);
                });
                it(`Unable to generate player groups for two player game with only one player`, () => {
                    const players = [player0];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateTwoPlayerGroups(players, []);
                    expect(result.length).toBe(0);
                });
                it(`Only generates one play group when there is only two players`, () => {
                    const players = [player0, player1];
                    const result = MiniGamePlaySession_1.MiniGamePlaySession.generateTwoPlayerGroups(players, []);
                    expect(result.length).toBe(1);
                });
            });
        });
        describe(`Player Group Cycling`, () => {
            let playSession;
            beforeEach(() => {
                playSession = new MiniGamePlaySession_1.MiniGamePlaySession([player0, player1, player2, player3]);
            });
            it(`Starting a single player mini game returns a single player group`, () => {
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.SINGLE);
                expect(playSession.currentPlayGroup.players.length).toBe(1);
            });
            it(`Starting a two player mini game returns a two player group`, () => {
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.TWO);
                expect(playSession.currentPlayGroup.players.length).toBe(2);
            });
            it(`The last group to finish should trigger the mini game session to be finished`, () => {
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.SINGLE);
                playSession.TEST_setCurrentPlayGroupIndex(3);
                playSession.playGroupFinished();
                expect(playSession.isMiniGameSessionFinished).toBe(true);
            });
            it(`Starting a single player mini game does not start with a player who played last last mini game`, () => {
                playSession.TEST_setLastKnownPlayers([player0, player2, player3]);
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.SINGLE);
                expect(playSession.currentPlayGroup.players).toEqual([player1]);
            });
            it(`Starting a two player mini game does not start with a player who played last last mini game`, () => {
                playSession.TEST_setLastKnownPlayers([player0, player2, player3]);
                playSession.startMiniGame(MiniGamePlaySession_1.MiniGamePlaySession.PlayerCount.TWO);
                expect(playSession.currentPlayGroup.players[1]).toEqual(player1);
            });
        });
    });
});
define("switchback-common/util/Animation", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function popSpin(x, y, width, imageKey, trailKey, game, parameters) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = parameters || {};
            const parent = params.parent || game.add.group();
            const tweenMS = params.tweenMS || 600;
            const spinMS = params.spinMS || 750;
            const bezierCurve = [
                [[0, 0], [-0.287, -0.413], [-0.705, -0.508], [-1, -0.539]],
                [[0, 0], [-0.609, 0.025], [-0.882, 0.280], [-1, 0.593]],
                [[0, 0], [0.287, -0.413], [0.705, -0.508], [1, -0.539]],
            ];
            const startPositions = [
                [x - width * 0.2, y - width * 0.1],
                [x - width * 0.25, y + width * 0.05],
                [x + width * 0.2, y],
            ];
            for (let i = 0; i < bezierCurve.length; i++) {
                const pos = startPositions[i];
                const bezier = bezierCurve[i];
                const star = new Phaser.Image(game, pos[0], pos[1], imageKey);
                const curveScale = width * 0.4;
                star.anchor.set(0.5);
                star.pivot.set(0.5, 0.5);
                star.position.set(pos[0], pos[1]);
                const starRatio = star.width / star.height;
                star.width = params.iconWidth || star.width;
                star.height = star.width * starRatio;
                star.visible = true;
                game.add.tween(star).to({ angle: 360 }, spinMS, Phaser.Easing.Linear.None, true, 0 - 1).start();
                const tween = game.add.tween(star).to({
                    x: [
                        pos[0] + curveScale * bezier[0][0],
                        pos[0] + curveScale * bezier[1][0],
                        pos[0] + curveScale * bezier[2][0],
                        pos[0] + curveScale * bezier[3][0],
                    ],
                    y: [
                        pos[1] + curveScale * bezier[0][1],
                        pos[1] + curveScale * bezier[1][1],
                        pos[1] + curveScale * bezier[2][1],
                        pos[1] + curveScale * bezier[3][1],
                    ],
                }, tweenMS, Phaser.Easing.Linear.None, false);
                const emitter = game.add.emitter(star.x, star.y, 50);
                emitter.maxParticleScale = 0.5;
                parent.addChild(emitter);
                parent.addChild(star);
                emitter.makeParticles(trailKey);
                emitter.setRotation(0, 0);
                emitter.setXSpeed(0, 0);
                emitter.setYSpeed(0, 0);
                emitter.alpha = 0.8;
                emitter.start(false, 5000, 10);
                emitter.emitX = star.position.x;
                emitter.emitY = star.position.y;
                emitter.gravity = 0;
                const emitTween = game.add.tween(emitter).to({
                    emitX: [
                        pos[0] + curveScale * bezier[0][0],
                        pos[0] + curveScale * bezier[1][0],
                        pos[0] + curveScale * bezier[2][0],
                        pos[0] + curveScale * bezier[3][0],
                    ],
                    emitY: [
                        pos[1] + curveScale * bezier[0][1],
                        pos[1] + curveScale * bezier[1][1],
                        pos[1] + curveScale * bezier[2][1],
                        pos[1] + curveScale * bezier[3][1],
                    ],
                }, tweenMS, Phaser.Easing.Linear.None, false);
                tween.interpolation(Phaser.Math.bezierInterpolation);
                emitTween.interpolation(Phaser.Math.bezierInterpolation);
                tween.onComplete.add(() => {
                    const fade = game.add.tween(star).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true, 0, 0);
                    fade.onComplete.add(() => {
                        star.destroy();
                    });
                    fade.start();
                }, this);
                emitTween.onComplete.addOnce(() => {
                    const fade = game.add.tween(emitter).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true, 0, 0);
                    fade.onComplete.add(() => {
                        // if (emitter) {
                        // 	emitter.destroy();
                        // }
                    });
                    emitter.start();
                }, this);
                tween.start();
                emitTween.start();
            }
        });
    }
    exports.popSpin = popSpin;
    function wiggle(game, tweenObj, angle = 0, offsetAngle = 15, duration = 500, autoStart = true) {
        const tween = game.add.tween(tweenObj);
        // this first tween is to get the object in the opposite position
        // returning chained tweens is a bad idea
        // can't be easily looped and signaled
        // so we only repeat the second tween and return that one
        tween.to({ angle: angle + offsetAngle }, duration, Phaser.Easing.Cubic.apply, autoStart, 0, 0, false);
        const tween2 = game.add.tween(tweenObj);
        tween2.to({ angle: angle - offsetAngle }, duration, Phaser.Easing.Cubic.apply, false, 0, 0, true);
        tween.chain(tween2);
        return tween2;
    }
    exports.wiggle = wiggle;
    function attentionWiggle(game, tweenObj, angle = 0, duration = 500) {
        const startAngle = tweenObj.angle;
        const tween = game.add.tween(tweenObj);
        // this first tween is to get the object in the opposite position
        // returning chained tweens is a bad idea
        // can't be easily looped and signaled
        // so we only repeat the second tween and return that one
        tween.to({ angle: startAngle - angle }, duration * 0.25, Phaser.Easing.Cubic.apply, true, 0, 0, false);
        const tween2 = game.add.tween(tweenObj);
        tween2.to({ angle: startAngle + angle }, duration * 0.5, Phaser.Easing.Cubic.apply, false, 0, 0, false);
        const tween3 = game.add.tween(tweenObj);
        tween3.to({ angle: startAngle }, duration * 0.25, Phaser.Easing.Cubic.apply, false, 0, 0, false);
        tween.chain(tween2);
        tween2.chain(tween3);
        return tween;
    }
    exports.attentionWiggle = attentionWiggle;
    function tintTween(game, tweenObj, tint, duration = 500, ease = Phaser.Easing.Linear.None, autoStart = false, delay = 0, repeat = 0, yoyo = false) {
        const colorBlend = { step: 0 };
        const startColor = tweenObj.tint;
        const colorTween = game.add.tween(colorBlend).to({ step: 100 }, duration, ease, autoStart, delay, repeat, yoyo);
        colorTween.onUpdateCallback(function () {
            tweenObj.tint = Phaser.Color.interpolateColor(startColor, tint, 100, colorBlend.step);
        });
        return colorTween.start();
    }
    exports.tintTween = tintTween;
    function fadeIn(game, object, duration, easing = Phaser.Easing.Linear.None) {
        object.alpha = 0;
        object.visible = true;
        return game.add.tween(object).to({
            alpha: 1,
        }, duration, easing, false);
    }
    exports.fadeIn = fadeIn;
    function fadeOut(game, object, duration, easing = Phaser.Easing.Linear.None) {
        object.alpha = 1;
        return game.add.tween(object).to({
            alpha: 0,
        }, duration, easing, false);
    }
    exports.fadeOut = fadeOut;
    function bounce(game, object, repeat = 0, bounceHeight) {
        bounceHeight = bounceHeight || this.game.height * 0.1;
        const destination = object.position.y - bounceHeight;
        const origin = object.position.y;
        const tween = game.add.tween(object).to({
            y: destination,
        }, 200, Phaser.Easing.Linear.None, false, 0).to({
            y: origin,
        }, 200, Phaser.Easing.Linear.None, false, 0);
        tween.repeatAll(repeat);
        return tween;
    }
    exports.bounce = bounce;
    function slideTo(game, object, targetPosition, duration = 200, easing = Phaser.Easing.Linear.None) {
        const tween = game.add.tween(object).to({
            x: targetPosition.x,
            y: targetPosition.y,
        }, duration, easing, false, 0, 0);
        return tween;
    }
    exports.slideTo = slideTo;
    function pulse(game, object, targetScale, duration = 200, easing = Phaser.Easing.Linear.None) {
        const originalWidth = object.width;
        const originalHeight = object.height;
        const tweenIn = game.add.tween(object).to({
            width: originalWidth * targetScale,
            height: originalHeight * targetScale,
        }, duration, easing, false, 0, 0);
        const tweenOut = game.add.tween(object).to({
            width: originalWidth,
            height: originalHeight,
        }, duration, easing, false, 0, 0);
        tweenOut.onComplete.add(() => {
            tweenIn.start();
        });
        tweenIn.chain(tweenOut);
        return tweenIn;
    }
    exports.pulse = pulse;
    function shake(game, object, amount, severity) {
        const times = amount || game.rnd.integerInRange(5, 7);
        const maxOffset = game.width * (severity || 0.02);
        const orgPos = object.position.clone();
        let firstTween;
        let prevTween;
        for (let i = 0; i < times; ++i) {
            const tween = game.add.tween(object).to({
                x: orgPos.x + game.rnd.integerInRange(-maxOffset, maxOffset),
                y: orgPos.y + game.rnd.integerInRange(-maxOffset, maxOffset),
            }, 20, Phaser.Easing.Linear.None, false);
            if (prevTween) {
                prevTween.chain(tween);
            }
            else {
                firstTween = tween;
            }
            prevTween = tween;
        }
        const tween = game.add.tween(object).to({
            x: orgPos.x,
            y: orgPos.y,
        }, 20, Phaser.Easing.Linear.None, false);
        prevTween.chain(tween);
        return firstTween;
    }
    exports.shake = shake;
    function blink(game, object, duration, times, autostart = true) {
        const tween = game.add.tween(object);
        tween.to({}, duration, Phaser.Easing.Linear.None, autostart, 0, times);
        tween.onRepeat.add(() => {
            object.visible = !object.visible;
        });
        return tween;
    }
    exports.blink = blink;
    function pulseFade(game, object, finalScale, duration, times = 0, autoStart = true) {
        const tweens = [
            // scale
            game.add.tween(object.scale).to({ x: finalScale, y: finalScale }, duration, Phaser.Easing.Linear.None, autoStart, 0, times),
            // tween alphas
            game.add.tween(object).to({ alpha: 0 }, duration, Phaser.Easing.Quartic.In, autoStart, 0, times)
        ];
        return tweens;
    }
    exports.pulseFade = pulseFade;
});
define("switchback-common/util/CameraUtil", ["require", "exports", "switchback-common/util/PromiseUtil"], function (require, exports, PromiseUtil_62) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function cameraShake(game, intensity = 0.005, durationMS = 200) {
        return __awaiter(this, void 0, void 0, function* () {
            game.camera.shake(intensity, durationMS);
            yield PromiseUtil_62.PromiseUtil.time(game, durationMS);
        });
    }
    exports.cameraShake = cameraShake;
    /**
     * Scales up/down the given group so that it appears as though the camera is zooming in/out respectively.
     * Set duration > 0 to make the zoom a tween.
     *
     * @param camGroup: Phaser.Group - Should contain everything in the scene you wish to zoom
     * @param zoomLevel: number - 1.0 is standard, higher (eg. 1.5 or 2.0) will zoom in, use < 1.0 with caution.
     * @param focusPnt: Phaser.Point (Scale 0-1) - Centre of the screen once the zoom is complete, (0.5, 0.5) would be game world centre
     * @param duration: number - Default is instant, higher numbers will tween the zoom for the given time.
     */
    function cameraZoom(game, camGroup, zoomLevel, focusPnt, duration = -1, easing = Phaser.Easing.Linear.None) {
        return __awaiter(this, void 0, void 0, function* () {
            const movementProps = {};
            const camW = game.width * zoomLevel;
            const camH = game.height * zoomLevel;
            movementProps.x = game.width * 0.5 - camW * focusPnt.x;
            movementProps.y = game.height * 0.5 - camH * focusPnt.y;
            const scaleProps = {
                x: zoomLevel,
                y: zoomLevel,
            };
            if (duration > 0) {
                yield Promise.all([
                    PromiseUtil_62.PromiseUtil.tween2(game.add.tween(camGroup).to(movementProps, duration, easing, true)),
                    PromiseUtil_62.PromiseUtil.tween2(game.add.tween(camGroup.scale).to(scaleProps, duration, easing, true)),
                ]);
            }
            else {
                camGroup.scale.set(scaleProps.x, scaleProps.y);
                camGroup.position.set(movementProps.x, movementProps.y);
            }
        });
    }
    exports.cameraZoom = cameraZoom;
});
define("switchback-common/util/LiquidScaler", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     */
    class LiquidScaler {
        static init(game, minWidth, minHeight, maxScale) {
            if (this.instance) {
                console.error("LiquidScaler already initialized.");
                return;
            }
            this.instance = new LiquidScaler(game, minWidth, minHeight, maxScale);
        }
        static getScaledSize(widthPercent, heightPercent, maintainAspectRatio = true, useMaxScale = true) {
            return this.Instance.getScaledSize(widthPercent, heightPercent, maintainAspectRatio, useMaxScale);
        }
        static get Instance() {
            if (!this.instance) {
                console.error("LiquidScaler not initialized.");
            }
            return this.instance;
        }
        constructor(game, minWidth, minHeight, maxScale) {
            this.game = game;
            this.minWidth = minWidth;
            this.minHeight = minHeight;
            this.maxScale = maxScale;
        }
        getScaledSize(widthPercent, heightPercent, maintainAspectRatio = true, useMaxScale = true) {
            let width = widthPercent * this.minWidth;
            let height = heightPercent * this.minHeight;
            let verticalScale = this.game.height / this.minHeight;
            let horizontalScale = this.game.width / this.minWidth;
            if (maintainAspectRatio) {
                const isLong = horizontalScale > verticalScale;
                if (isLong) {
                    if (useMaxScale) {
                        verticalScale = Math.min(verticalScale, this.maxScale);
                    }
                    width *= verticalScale;
                    height *= verticalScale;
                }
                else {
                    if (useMaxScale) {
                        horizontalScale = Math.min(horizontalScale, this.maxScale);
                    }
                    width *= horizontalScale;
                    height *= horizontalScale;
                }
            }
            else {
                width = widthPercent * this.game.width;
                height = widthPercent * this.game.height;
            }
            return new Phaser.Rectangle(0, 0, width, height);
        }
    }
    exports.LiquidScaler = LiquidScaler;
});
define("switchback-common/util/MathUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Returns a random number between min (inclusive) and max (exclusive)
     */
    function getRandomArbitrary(min, max) {
        return Math.random() * (max - min) + min;
    }
    exports.getRandomArbitrary = getRandomArbitrary;
    /**
     * Returns a random integer between min (inclusive) and max (inclusive)
     * Using Math.round() will give you a non-uniform distribution!
     */
    function getRandomInt(min, max) {
        return Math.floor(Math.random() * (max - min + 1)) + min;
    }
    exports.getRandomInt = getRandomInt;
    /**
     * Generate a nomally distibuted number based on the central limit theorem between 0 and 1
     * https://stackoverflow.com/questions/25582882/javascript-math-random-normal-distribution-gaussian-bell-curve/39187274#39187274
     */
    function normalRandom(numSamples = 7) {
        let rand = 0;
        for (let i = 0; i < numSamples; i++) {
            rand += Math.random();
        }
        return rand / numSamples;
    }
    exports.normalRandom = normalRandom;
    /*
        Utility function to get the shortest difference between two angles
        Used for tweens, it works, given the fact that phaser wraps the angle to the
        correct space whenever we assign it (which allows us to overflow the range by doing current+dif)
        And also because the tweens don't seem to rely on re-reading the value of the variable (which is awesome)
    */
    function getShorterAngleDif(currentAngle, newAngle) {
        if (currentAngle < 0) {
            currentAngle += 360;
        }
        if (newAngle < 0) {
            newAngle += 360;
        }
        let dif = newAngle - currentAngle;
        if (dif < -180) {
            dif += 360;
        }
        else if (dif > 180) {
            dif -= 360;
        }
        return dif;
    }
    exports.getShorterAngleDif = getShorterAngleDif;
    function base64ToByteArray(base64) {
        const binary_string = window.atob(base64);
        const len = binary_string.length;
        const bytes = new Uint8Array(len);
        for (var i = 0; i < len; i++) {
            bytes[i] = binary_string.charCodeAt(i);
        }
        return bytes;
    }
    exports.base64ToByteArray = base64ToByteArray;
    function lerp2D(pointA, pointB, amount) {
        const dX = pointA.x + (pointB.x - pointA.x) * amount;
        const dY = pointA.y + (pointB.y - pointA.y) * amount;
        return { x: dX, y: dY };
    }
    exports.lerp2D = lerp2D;
    class MathUtil {
        /**
         * Tests to see if @param n is between angles @param a and @param b.
         * Code as per https://www.xarg.org/2010/06/is-an-angle-between-two-other-angles/
         * TODO: Write tests
         */
        static isAngleBetween(n, a, b) {
            n = (360 + (n % 360)) % 360;
            a = (3600000 + a) % 360;
            b = (3600000 + b) % 360;
            if (a < b) {
                return a <= n && n <= b;
            }
            return a <= n || n <= b;
        }
        /**
         * Returns a point located at @param distance along a @param line.
         * Formula as per https://math.stackexchange.com/a/409721
         */
        static getPointAlongLine(line, distance) {
            const x1 = line.start.x;
            const y1 = line.start.y;
            const x2 = line.end.x;
            const y2 = line.end.x;
            const d = distance;
            const d1 = line.length;
            return new Phaser.Point((x1 + (d / d1) * (x2 - x1)), y1 + (d / d1) * (y2 - y1));
        }
    }
    exports.MathUtil = MathUtil;
});
define("switchback-common/util/MemoryUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class MemoryUtil {
        /*
            Checks that all the child objects contained as attributes have been destroyed properly.
            Does not descend into the tree (childs of childs).
            Works for Phaser objects and Signals.
            To be used for debugging, and profiling only.
            How to use:
                When trying to profile certain class, call this function between destroy and nulling. ie:
    
                ```
                super.destroy();
                MemoryUtil.checkAllDestroyed(this);
    
                this.prop1 = null;
                this.prop2 = null;
                ```
    
                If certain attribute has not been destroyed properly it will be reported on the console.
        */
        static checkAllDestroyed(obj) {
            if (obj.stateName) {
                console.warn("Checking state " + obj.stateName);
            }
            const toIgnore = ["game", "make", "add", "camera", "cache", "input", "sound",
                "scale", "stage", "state", "time", "tweens", "world", "particles", "physics", "load"];
            let ret = true;
            for (const name in obj) {
                if (!obj.hasOwnProperty(name)) {
                    continue;
                }
                if (toIgnore.indexOf(name) > -1) {
                    continue;
                }
                const child = obj[name];
                if (!child) {
                    continue;
                }
                // if the object contains one of this properties and is not null, then it hasn't been destroyed properly
                if (child.parent || child.game || child.exists) {
                    ret = false;
                    console.warn("Object not properly destroyed. '" + name + "'", child);
                }
                // signals
                if (child._bindings || child._prevParams) {
                    ret = false;
                    console.warn("Signal not properly disposed. '" + name + "'", child);
                }
                // sounds
                if (child._buffer || child.externalNode) {
                    ret = false;
                    console.warn("Sound not properly disposed. '" + name + "'", child);
                }
            }
            return ret;
        }
    }
    exports.MemoryUtil = MemoryUtil;
});
define("switchback-common/util/PhysicsUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class PhysicsUtil {
        /**
         * Custom physics data doesn't cope with being scaled.
         *
         * This function found at https://stackoverflow.com/a/43302120/1343176 fixes this
         * by scaling all vertices on the shape and rewriting them back to the cache at runtime.
         * @param originalPhysicsKey the key in cache of your physics data
         * @param newPhysicsKey the key you want the new physics data to have
         * @param shapeKey the specific shape in the original data to scale
         * @param scale the scale value
         */
        static resizePolygonData(game, originalPhysicsKey, newPhysicsKey, shapeKey, scale) {
            const newData = [];
            const data = game.cache.getPhysicsData(originalPhysicsKey, shapeKey);
            for (let i = 0; i < data.length; i++) {
                const vertices = [];
                for (let j = 0; j < data[i].shape.length; j += 2) {
                    vertices[j] = data[i].shape[j] * scale;
                    vertices[j + 1] = data[i].shape[j + 1] * scale;
                }
                newData.push({ shape: vertices });
            }
            const item = {};
            item[shapeKey] = newData;
            game.load.physics(newPhysicsKey, "", item);
        }
    }
    exports.PhysicsUtil = PhysicsUtil;
});
/**
 * A simple timer that avoids many of the problems with the timer from Phaser
 * Even though it looks unnervingly simple, that's actually the design of it.
 * The code is solid, and works logically as it would be expected to,
 * and it has as minimal overhead as possible. So it's efficient, lightweight
 * and easy to understand, debug and maintain.
 * It doesn't depend on Phaser nor es6 or anything fancy, that'll make it reusable,
 * solid and predictable as it possible.
 *
 * @author Joel@CerebralFix.com
 * @author Jeronimo@CerebralFix.com
 */
define("switchback-common/util/SimpleTimer", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SimpleTimer {
        constructor(delay = 0, callback = null) {
            this.onTimer = null;
            this.delay = 0;
            this.elapsed = 0;
            this.stopped = true;
            this.delay = delay;
            this.onTimer = callback;
        }
        stop() {
            this.stopped = true;
        }
        start() {
            this.elapsed = 0;
            this.stopped = false;
        }
        update(dt) {
            if (this.stopped) {
                return;
            }
            this.elapsed += dt;
            // Doing it this way ensures that it will be triggered only once per frame,
            // but overall the accuracy of the timer is maximized even during frame drops
            // which is the whole reason of using dt
            if (this.elapsed > this.delay) {
                this.elapsed -= this.delay;
                if (this.onTimer) {
                    this.onTimer();
                }
            }
        }
    }
    exports.SimpleTimer = SimpleTimer;
});
define("switchback-common/util/SpriteUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class SpriteUtil {
        static greyscale(game, frame) {
            const bmd = game.make.bitmapData();
            bmd.load(frame);
            bmd.processPixelRGB(SpriteUtil.forEachPixel, this);
            return bmd;
        }
        static forEachPixel(pixel) {
            const gray = (pixel.r + pixel.g + pixel.b) / 3;
            pixel.r = gray;
            pixel.g = gray;
            pixel.b = gray;
            return pixel;
        }
    }
    exports.SpriteUtil = SpriteUtil;
});
define("switchback-common/util/TimeUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * @author nick@cerebralfix.com
     */
    class TimeUtil {
        /**
         * Outputs time as a string in minutes:seconds format.
         * Seconds are padded, minutes are not
         *
         * @param timeMS In milliseconds
         */
        static toDigitalTime(timeMS) {
            timeMS = Math.round(timeMS * 0.001);
            const minutes = Math.floor(timeMS / 60);
            const seconds = timeMS % 60;
            return minutes + ":" + (seconds < 10 ? "0" + seconds : seconds).toString();
        }
    }
    exports.TimeUtil = TimeUtil;
});
define("switchback-common/util/WebUtil", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Helper functions for web-specific tasks
     * @author joel@cerebralfix.com
     * @author nick@cerebralfix.com
     */
    class WebUtil {
        /**
         * Grabs the URL parameters and returns them as a dictionary.
         */
        static getQueryParams() {
            if (WebUtil.queryDict) {
                return WebUtil.queryDict;
            }
            const params = {};
            location.search.substr(1).split("&").forEach((item) => {
                const s = item.split("=");
                const key = s[0];
                const value = s[1] && decodeURIComponent(s[1]);
                if (!(key in params)) {
                    params[key] = new Array();
                }
                params[key].push(value);
            });
            WebUtil.queryDict = params;
            return WebUtil.queryDict;
        }
        // Fast syntax lookup for a specific query parametre
        static getQueryFor(key, index = 0) {
            const dict = WebUtil.getQueryParams();
            if (dict[key]) {
                return dict[key][index];
            }
            return null;
        }
    }
    exports.WebUtil = WebUtil;
});
